@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./cocci.mli"
 open Common
 
 (* full_engine takes (coccifile, isofile) and cfiles in parameters and
@@ -36,10 +39,17 @@ open Common
  * using -dir to not redo twice the same work. So take care!
  *)
 type cocci_info
-val pre_engine : (filename * filename) -> cocci_info
+type constant_info =
+    string list option (*grep tokens*) *
+      string list option (*glimpse tokens*) *
+      (Str.regexp * Str.regexp list) option (*coccigrep tokens*) *
+      Get_constants2.combine option
+val pre_engine : (filename * filename) -> cocci_info * constant_info
+val worth_trying : filename list -> constant_info -> bool
 val full_engine :
   cocci_info -> filename list -> (filename * filename option) list
 val post_engine : cocci_info -> unit
+val has_finalize : cocci_info -> bool
 
 (* because of the #include "toto.c" and also because we may associate the
  * same C file to multiple drivers because they share code, we can
@@ -62,7 +72,10 @@ val sp_of_file :
 	 (*fresh used after list seeds*)
 	 Ast_cocci.meta_name list list list) *
       Ast_cocci.meta_name list list list *
-      string list option (*grep tokens*) *
-      string list option (*glimpse tokens*)
+      (string list option (*grep tokens*) *
+	 string list option (*glimpse tokens*) *
+	 (Str.regexp * Str.regexp list) option (*coccigrep tokens*) *
+	 Get_constants2.combine option) *
+      bool (* format information needed for strings? *)
 
 val normalize_path : string -> string
