@@ -67,7 +67,7 @@ val save_tmp_files : bool ref
 type filename = string
 type dirname = string
 
-(* Trick in case you dont want to do an 'open Common' while still wanting
+(* Trick in case you don't want to do an 'open Common' while still wanting
  * more pervasive types than the one in Pervasives. Just do the selective
  * open Common.BasicType.
  *)
@@ -137,7 +137,7 @@ val pr2_xxxxxxxxxxxxxxxxx : unit -> unit
 
 (* use Dumper.dump *)
 val pr2_gen: 'a -> unit
-val dump: 'a -> string
+(* val dump: 'a -> string *)
 
 (* see flag: val disable_pr2_once : bool ref *)
 val _already_printed : (string, bool) Hashtbl.t
@@ -258,6 +258,8 @@ type score_result = Ok | Pb of string
 type score =      (string (* usually a filename *), score_result) Hashtbl.t
 type score_list = (string (* usually a filename *) * score_result) list
 val empty_score : unit -> score
+val load_score : string -> unit -> score
+val save_score : score -> string -> unit
 val regression_testing :
   score -> filename (* old score file on disk (usually in /tmp) *) -> unit
 val regression_testing_vs: score -> score -> score
@@ -433,12 +435,20 @@ val cache_computation :
  *)
 val cache_computation_robust :
   filename ->
-  string (* extension for marshalled object *) ->
+  string (* extension for marshaled object *) ->
   (filename list * 'x) ->
   string (* extension for marshalled dependencies *) ->
   (unit -> 'a) ->
   'a
 
+val cache_computation_robust_in_dir :
+  string option (* destination directory *) -> filename ->
+  string (* extension for marshalled object *) ->
+  (filename list * 'x) ->
+  string (* extension for marshaled dependencies *) ->
+  (unit -> 'a) ->
+  'a
+
 
 
 val once : ('a -> unit) -> ('a -> unit)
@@ -465,7 +475,7 @@ val release_file_lock : filename -> unit
 (* Error managment *)
 (*****************************************************************************)
 exception Todo
-exception Impossible
+exception Impossible of int
 exception Here
 exception ReturnExn
 
@@ -592,7 +602,7 @@ val (=*=): 'a -> 'a -> bool
  * val (=): unit -> unit -> bool
  *
  * But it will not forbid you to use caml functions like List.find, List.mem
- * which internaly use this convenient but evolution-unfriendly (=)
+ * which internally use this convenient but evolution-unfriendly (=)
 *)
 
 
@@ -1066,6 +1076,9 @@ val process_output_to_list : string -> string list
 val cmd_to_list :            string -> string list (* alias *)
 val cmd_to_list_and_status : string -> string list * Unix.process_status
 
+val file_to_stdout : string -> unit
+val file_to_stderr : string -> unit
+
 val command2 : string -> unit
 val _batch_mode: bool ref
 val command2_y_or_no : string -> bool
@@ -1137,6 +1150,7 @@ val timeout_function : int -> (unit -> 'a) -> 'a
 
 val timeout_function_opt : int option -> (unit -> 'a) -> 'a
 
+val remove_file : string -> unit
 
 (* creation of /tmp files, a la gcc
  * ex: new_temp_file "cocci" ".c" will give "/tmp/cocci-3252-434465.c"
@@ -1181,6 +1195,8 @@ val acc_map : ('a -> 'b) -> 'a list -> 'b list
 
 
 val zip : 'a list -> 'b list -> ('a * 'b) list
+val combine4 : 'a list -> 'b list -> 'c list -> 'd list ->
+                  ('a * 'b * 'c * 'd) list
 val zip_safe : 'a list -> 'b list -> ('a * 'b) list
 val unzip : ('a * 'b) list -> 'a list * 'b list
 
@@ -1535,16 +1551,30 @@ module IntMap :
     type +'a t
     val empty : 'a t
     val is_empty : 'a t -> bool
+    val mem : key -> 'a t -> bool
     val add : key -> 'a -> 'a t -> 'a t
+    val singleton : key -> 'a -> 'a t
     val find : key -> 'a t -> 'a
     val remove : key -> 'a t -> 'a t
-    val mem : key -> 'a t -> bool
-    val iter : (key -> 'a -> unit) -> 'a t -> unit
-    val map : ('a -> 'b) -> 'a t -> 'b t
-    val mapi : (key -> 'a -> 'b) -> 'a t -> 'b t
-    val fold : (key -> 'a -> 'b -> 'b) -> 'a t -> 'b -> 'b
+    val merge : (key -> 'a option -> 'b option -> 'c option) ->
+      'a t -> 'b t -> 'c t
     val compare : ('a -> 'a -> int) -> 'a t -> 'a t -> int
     val equal : ('a -> 'a -> bool) -> 'a t -> 'a t -> bool
+    val iter : (key -> 'a -> unit) -> 'a t -> unit
+    val fold : (key -> 'a -> 'b -> 'b) -> 'a t -> 'b -> 'b
+    val for_all : (key -> 'a -> bool) -> 'a t -> bool
+    val exists : (key -> 'a -> bool) -> 'a t -> bool
+    val filter : (key -> 'a -> bool) -> 'a t -> 'a t
+    val partition: (key -> 'a -> bool) -> 'a t -> 'a t * 'a t
+    val cardinal : 'a t -> int
+    val bindings : 'a t -> (key * 'a) list
+    val min_binding : 'a t -> key * 'a
+    val max_binding : 'a t -> key * 'a
+    val choose : 'a t -> key * 'a
+    val split : key -> 'a t -> 'a t * 'a option * 'a t
+    val find : key -> 'a t -> 'a
+    val map : ('a -> 'b) -> 'a t -> 'b t
+    val mapi : (key -> 'a -> 'b) -> 'a t -> 'b t
   end
 val intmap_to_list : 'a IntMap.t -> (IntMap.key * 'a) list
 val intmap_string_of_t : 'a -> 'b -> string
@@ -1555,16 +1585,30 @@ module IntIntMap :
     type +'a t
     val empty : 'a t
     val is_empty : 'a t -> bool
+    val mem : key -> 'a t -> bool
     val add : key -> 'a -> 'a t -> 'a t
+    val singleton : key -> 'a -> 'a t
     val find : key -> 'a t -> 'a
     val remove : key -> 'a t -> 'a t
-    val mem : key -> 'a t -> bool
-    val iter : (key -> 'a -> unit) -> 'a t -> unit
-    val map : ('a -> 'b) -> 'a t -> 'b t
-    val mapi : (key -> 'a -> 'b) -> 'a t -> 'b t
-    val fold : (key -> 'a -> 'b -> 'b) -> 'a t -> 'b -> 'b
+    val merge : (key -> 'a option -> 'b option -> 'c option) ->
+      'a t -> 'b t -> 'c t
     val compare : ('a -> 'a -> int) -> 'a t -> 'a t -> int
     val equal : ('a -> 'a -> bool) -> 'a t -> 'a t -> bool
+    val iter : (key -> 'a -> unit) -> 'a t -> unit
+    val fold : (key -> 'a -> 'b -> 'b) -> 'a t -> 'b -> 'b
+    val for_all : (key -> 'a -> bool) -> 'a t -> bool
+    val exists : (key -> 'a -> bool) -> 'a t -> bool
+    val filter : (key -> 'a -> bool) -> 'a t -> 'a t
+    val partition: (key -> 'a -> bool) -> 'a t -> 'a t * 'a t
+    val cardinal : 'a t -> int
+    val bindings : 'a t -> (key * 'a) list
+    val min_binding : 'a t -> key * 'a
+    val max_binding : 'a t -> key * 'a
+    val choose : 'a t -> key * 'a
+    val split : key -> 'a t -> 'a t * 'a option * 'a t
+    val find : key -> 'a t -> 'a
+    val map : ('a -> 'b) -> 'a t -> 'b t
+    val mapi : (key -> 'a -> 'b) -> 'a t -> 'b t
   end
 val intintmap_to_list : 'a IntIntMap.t -> (IntIntMap.key * 'a) list
 val intintmap_string_of_t : 'a -> 'b -> string
@@ -1668,7 +1712,7 @@ val tree_iter : ('a -> unit) -> 'a tree -> unit
 
 
 (*****************************************************************************)
-(* N-ary tree with updatable childrens *)
+(* N-ary tree with updatable children *)
 (*****************************************************************************)
 
 (* no empty tree, must have one root at least *)
@@ -1927,7 +1971,7 @@ val error_message_short : filename -> (string * int) -> string
 val error_messagebis : filename -> (string * int) -> int -> string
 
 (*****************************************************************************)
-(* Scope managment (cocci) *)
+(* Scope management (cocci) *)
 (*****************************************************************************)
 
 (* for example of use, see the code used in coccinelle *)
