@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./ctl_engine.ml"
 (*external c_counter : unit -> int = "c_counter"*)
 let timeout = 800
 (* Optimize triples_conj by first extracting the intersection of the two sets,
@@ -199,9 +202,7 @@ let maybe f g opt =
 
 let some_map f opts = map (maybe (fun x -> Some (f x)) None) opts
 
-let some_tolist_alt opts = concatmap (maybe (fun x -> [x]) []) opts
-
-let rec some_tolist opts =
+let rec some_tolist (opts : 'a option list) : 'a list =
   match opts with
     | []             -> []
     | (Some x)::rest -> x::(some_tolist rest)
@@ -231,20 +232,13 @@ let rec nubBy eq ls =
   | (x::xs) -> x::(nubBy eq xs)
 ;;
 
-let rec nub ls =
-  match ls with
-    [] -> []
-  | (x::xs) when (List.mem x xs) -> nub xs
-  | (x::xs) -> x::(nub xs)
-;;
-
 let state_compare (s1,_,_) (s2,_,_) = compare s1 s2
 
 let setifyBy eq xs = nubBy eq xs;;
 
-let setify xs = nub xs;;
+let setify xs = Common.nub xs;;
 
-let inner_setify xs = List.sort compare (nub xs);;
+let inner_setify xs = List.sort compare (Common.nub xs);;
 
 let unionBy compare eq xs = function
     [] -> xs
@@ -264,9 +258,9 @@ let subseteqBy eq xs ys = List.for_all (fun x -> memBy eq x ys) xs;;
 let subseteq xs ys = List.for_all (fun x -> List.mem x ys) xs;;
 let supseteq xs ys = subseteq ys xs
 
-let setequalBy eq xs ys = (subseteqBy eq xs ys) & (subseteqBy eq ys xs);;
+let setequalBy eq xs ys = (subseteqBy eq xs ys) && (subseteqBy eq ys xs);;
 
-let setequal xs ys = (subseteq xs ys) & (subseteq ys xs);;
+let setequal xs ys = (subseteq xs ys) && (subseteq ys xs);;
 
 (* Fix point calculation *)
 let rec fix eq f x =
@@ -277,7 +271,7 @@ let rec fix eq f x =
 let setfix f x = (fix subseteq f x) (*if new is a subset of old, stop*)
 let setgfix f x = (fix supseteq f x) (*if new is a supset of old, stop*)
 
-let get_states l = nub (List.map (function (s,_,_) -> s) l)
+let get_states l = Common.nub (List.map (function (s,_,_) -> s) l)
 
 (* ********************************************************************** *)
 (* Module: CTL_ENGINE                                                     *)
@@ -398,7 +392,7 @@ let print_graph grp required_states res str = function
 		  ) in
 	      (if not (List.mem file !graph_stack) then
 		graph_stack := file :: !graph_stack);
-	    let filename = Filename.temp_file (file^":") ".dot" in	
+	    let filename = Filename.temp_file (file^":") ".dot" in
 	    Hashtbl.add graph_hash file filename;
 	    G.print_graph grp
 	      (if !Flag_ctl.gt_without_label then None else (Some label))
@@ -973,7 +967,7 @@ let triples_witness x unchecked not_keep trips =
 		failwith"unexpected negative binding with positive witnesses")*)
 	  | _ ->
 	      let new_triple =
-		if unchecked or not_keep
+		if unchecked || not_keep
 		then (s,newth,wit)
 		else
 		  if anynegwit wit && allnegwit wit
@@ -1002,12 +996,11 @@ let rec pre_exist dir (grp,_,_) y reqst =
   let check s =
     match reqst with None -> true | Some reqst -> List.mem s reqst in
   let exp (s,th,wit) =
-    concatmap
-      (fun s' -> if check s' then [(s',th,wit)] else [])
-      (match dir with
-	A.FORWARD -> G.predecessors grp s
-      | A.BACKWARD -> G.successors grp s) in
-  setify (concatmap exp y)
+    let ss' = match dir with
+                A.FORWARD  -> G.predecessors grp s
+              | A.BACKWARD -> G.successors grp s
+      in concatmap (fun s' -> if check s' then [(s',th,wit)] else []) ss'
+    in setify (concatmap exp y)
 ;;
 
 exception Empty
@@ -1293,7 +1286,7 @@ let satAW dir ((grp,_,states) as m) s1 s2 reqst =
 	 out of the loop. s1 is like a guard. To see the problem, consider
 	 an example where both s1 and s2 match some code after the loop.
 	 we only want the witness from s2. *)
-      setgfix f (triples_union (nub(drop_wits s1)) s2)
+      setgfix f (triples_union (Common.nub(drop_wits s1)) s2)
 ;;
 
 let satAF dir m s reqst =
@@ -1509,7 +1502,7 @@ let drop_required v required =
 
 (* no idea how to write this function ... *)
 let memo_label =
-  (Hashtbl.create(50) : (P.t, (G.node * substitution) list) Hashtbl.t)
+  (Hashtbl.create(101) : (P.t, (G.node * substitution) list) Hashtbl.t)
 
 let satLabel label required p =
     let triples =
@@ -1525,19 +1518,20 @@ let satLabel label required p =
 	    (List.map (function (st,th,_) -> (st,th)) triples);
 	  triples
     else setify(label p) in
-    normalize
-      (if !pREQUIRED_ENV_OPT
-      then
-	foldl
-	  (function rest ->
-	    function ((s,th,_) as t) ->
-	      if List.for_all
-		  (List.exists (function th' -> not(conj_subst th th' = None)))
-		  required
-	      then t::rest
-	      else rest)
-	  [] triples
-      else triples)
+    (* normalize first; conj_subst relies on sorting *)
+    let ntriples = normalize triples in
+    if !pREQUIRED_ENV_OPT
+    then
+      foldl
+	(function rest ->
+	  function ((s,th,_) as t) ->
+	    if List.for_all
+		(List.exists (function th' -> not(conj_subst th th' = None)))
+		required
+	    then t::rest
+	    else rest)
+	[] ntriples
+    else ntriples
 
 let get_required_states l =
   if !pREQUIRED_STATES_OPT && not !Flag_ctl.partial_match
@@ -1628,7 +1622,6 @@ type ('code,'value) cell = Frozen of 'code | Thawed of 'value
 let rec satloop unchecked required required_states
     ((grp,label,states) as m) phi env =
   let rec loop unchecked required required_states phi =
-    (*Common.profile_code "satloop" (fun _ -> *)
     let res =
       match phi with
       A.False              -> []
@@ -1647,14 +1640,19 @@ let rec satloop unchecked required required_states
 	triples_union
 	  (loop unchecked required required_states phi1)
 	  (loop unchecked required required_states phi2)
-    | A.SeqOr(phi1,phi2)      ->
+    | A.SeqOr(phi1,phi2)   ->
 	let res1 = loop unchecked required required_states phi1 in
 	let res2 = loop unchecked required required_states phi2 in
 	let res1neg = unwitify res1 in
-	triples_union res1
-	  (triples_conj
-	     (triples_complement (mkstates states required_states) res1neg)
-	     res2)
+	let pm = !Flag_ctl.partial_match in
+	(match (pm,res1,res2) with
+	  (false,res1,[]) -> res1
+	| (false,[],res2) -> res2
+	| _ ->
+	    triples_union res1
+	      (triples_conj
+		 (triples_complement (mkstates states required_states) res1neg)
+		 res2))
     | A.And(strict,phi1,phi2)     ->
 	(* phi1 is considered to be more likely to be [], because of the
 	   definition of asttoctl.  Could use heuristics such as the size of
@@ -1742,21 +1740,21 @@ let rec satloop unchecked required required_states
 		      phi1res phi2res))
     | A.InnerAnd(phi) ->
 	inner_and(loop unchecked required required_states phi)
-    | A.EX(dir,phi)      ->
+    | A.EX(dir,phi)   ->
 	let new_required_states =
 	  get_children_required_states dir m required_states in
 	satEX dir m (loop unchecked required new_required_states phi)
 	  required_states
-    | A.AX(dir,strict,phi)      ->
+    | A.AX(dir,strict,phi) ->
 	let new_required_states =
 	  get_children_required_states dir m required_states in
 	let res = loop unchecked required new_required_states phi in
 	strict_A1 strict satAX satEX dir m res required_states
-    | A.EF(dir,phi)            ->
+    | A.EF(dir,phi) ->
 	let new_required_states = get_reachable dir m required_states in
 	satEF dir m (loop unchecked required new_required_states phi)
 	  new_required_states
-    | A.AF(dir,strict,phi)            ->
+    | A.AF(dir,strict,phi) ->
 	if !Flag_ctl.loop_in_src_code
 	then
 	  loop unchecked required required_states
@@ -1765,15 +1763,15 @@ let rec satloop unchecked required required_states
 	  let new_required_states = get_reachable dir m required_states in
 	  let res = loop unchecked required new_required_states phi in
 	  strict_A1 strict satAF satEF dir m res new_required_states
-    | A.EG(dir,phi)            ->
+    | A.EG(dir,phi) ->
 	let new_required_states = get_reachable dir m required_states in
 	satEG dir m (loop unchecked required new_required_states phi)
 	  new_required_states
-    | A.AG(dir,strict,phi)            ->
+    | A.AG(dir,strict,phi) ->
 	let new_required_states = get_reachable dir m required_states in
 	let res = loop unchecked required new_required_states phi in
 	strict_A1 strict satAG satEF dir m res new_required_states
-    | A.EU(dir,phi1,phi2)      ->
+    | A.EU(dir,phi1,phi2) ->
 	let new_required_states = get_reachable dir m required_states in
 	(match loop unchecked required new_required_states phi2 with
 	  [] when !pLazyOpt -> []
@@ -1821,17 +1819,18 @@ let rec satloop unchecked required required_states
 		)
     | A.Implies(phi1,phi2) ->
 	loop unchecked required required_states (A.Or(A.Not phi1,phi2))
-    | A.Exists (keep,v,phi)     ->
+    | A.Exists (keep,v,phi) ->
 	let new_required = drop_required v required in
 	triples_witness v unchecked (not keep)
 	  (loop unchecked new_required required_states phi)
-    | A.Let(v,phi1,phi2)   ->
+
+    | A.Let(v,phi1,phi2) ->
 	(* should only be used when the properties unchecked, required,
 	   and required_states are known to be the same or at least
 	   compatible between all the uses.  this is not checked. *)
 	let res = loop unchecked required required_states phi1 in
 	satloop unchecked required required_states m phi2 ((v,res) :: env)
-    | A.LetR(dir,v,phi1,phi2)   ->
+    | A.LetR(dir,v,phi1,phi2) ->
 	(* should only be used when the properties unchecked, required,
 	   and required_states are known to be the same or at least
 	   compatible between all the uses.  this is not checked. *)
@@ -1839,7 +1838,7 @@ let rec satloop unchecked required required_states
 	let new_required_states = get_reachable dir m required_states in
 	let res = loop unchecked required new_required_states phi1 in
 	satloop unchecked required required_states m phi2 ((v,res) :: env)
-    | A.Ref(v)             ->
+    | A.Ref(v) ->
 	let res = List.assoc v env in
 	if unchecked
 	then List.map (function (s,th,_) -> (s,th,[])) res
@@ -1855,6 +1854,9 @@ let rec satloop unchecked required required_states
 
 
 (* SAT with tracking *)
+let output str =
+  Printf.printf "%s\n" str
+
 let rec sat_verbose_loop unchecked required required_states annot maxlvl lvl
     ((_,label,states) as m) phi env =
   let anno res children = (annot lvl phi res children,res) in
@@ -1869,24 +1871,24 @@ let rec sat_verbose_loop unchecked required required_states annot maxlvl lvl
       A.False              -> anno [] []
     | A.True               -> anno (triples_top states) []
     | A.Pred(p)            ->
-	Printf.printf "label\n"; flush stdout;
+	output "label";
 	anno (satLabel label required p) []
     | A.Uncheck(phi1) ->
 	let unchecked = if !pUNCHECK_OPT then true else false in
 	let (child1,res1) = satv unchecked required required_states phi1 env in
-	Printf.printf "uncheck\n"; flush stdout;
+	output "uncheck";
 	anno res1 [child1]
     | A.Not(phi1)          ->
 	let (child,res) =
 	  satv unchecked required required_states phi1 env in
-	Printf.printf "not\n"; flush stdout;
+	output "not";
 	anno (triples_complement (mkstates states required_states) res) [child]
     | A.Or(phi1,phi2)      ->
 	let (child1,res1) =
 	  satv unchecked required required_states phi1 env in
 	let (child2,res2) =
 	  satv unchecked required required_states phi2 env in
-	Printf.printf "or\n"; flush stdout;
+	output "or";
 	anno (triples_union res1 res2) [child1; child2]
     | A.SeqOr(phi1,phi2)      ->
 	let (child1,res1) =
@@ -1895,27 +1897,32 @@ let rec sat_verbose_loop unchecked required required_states annot maxlvl lvl
 	  satv unchecked required required_states phi2 env in
 	let res1neg =
 	  List.map (function (s,th,_) -> (s,th,[])) res1 in
-	Printf.printf "seqor\n"; flush stdout;
-	anno (triples_union res1
-		(triples_conj
-		   (triples_complement (mkstates states required_states)
-		      res1neg)
-		   res2))
-	  [child1; child2]
+	output "seqor";
+	let pm = !Flag_ctl.partial_match in
+	(match (pm,res1,res2) with
+	  (false,res1,[]) -> anno res1 [child1; child2]
+	| (false,[],res2) -> anno res2 [child1; child2]
+	| _ ->
+	    anno (triples_union res1
+		    (triples_conj
+		       (triples_complement (mkstates states required_states)
+			  res1neg)
+		       res2))
+	      [child1; child2])
     | A.And(strict,phi1,phi2)     ->
 	let pm = !Flag_ctl.partial_match in
 	(match (pm,satv unchecked required required_states phi1 env) with
 	  (false,(child1,[])) ->
-	    Printf.printf "and\n"; flush stdout; anno [] [child1]
+	    output "and"; anno [] [child1]
 	| (_,(child1,res1)) ->
 	    let new_required = extend_required res1 required in
 	    let new_required_states = get_required_states res1 in
 	    (match (pm,satv unchecked new_required new_required_states phi2
 		      env) with
 	      (false,(child2,[])) ->
-		Printf.printf "and\n"; flush stdout; anno [] [child1;child2]
+		output "and"; anno [] [child1;child2]
 	    | (_,(child2,res2)) ->
-		Printf.printf "and\n"; flush stdout;
+		output "and";
 		let res =
 		  strict_triples_conj strict
 		    (mkstates states required_states)
@@ -1925,7 +1932,7 @@ let rec sat_verbose_loop unchecked required required_states annot maxlvl lvl
 	let pm = !Flag_ctl.partial_match in
 	(match (pm,satv unchecked required required_states phi1 env) with
 	  (false,(child1,[])) ->
-	    Printf.printf "and\n"; flush stdout; anno [] [child1]
+	    output "and"; anno [] [child1]
 	| (_,(child1,res1)) ->
 	    let new_required = extend_required res1 required in
 	    let new_required_states = get_required_states res1 in
@@ -1934,7 +1941,7 @@ let rec sat_verbose_loop unchecked required required_states annot maxlvl lvl
 	    (match (pm,satv unchecked new_required new_required_states phi2
 		env) with
 	      (false,(child2,[])) ->
-		Printf.printf "andany\n"; flush stdout;
+		output "andany";
 		anno res1 [child1;child2]
 	    | (_,(child2,res2)) ->
 		(match res1 with
@@ -1952,7 +1959,7 @@ let rec sat_verbose_loop unchecked required required_states annot maxlvl lvl
 		| [(state,_,_)] ->
 		    let res2 =
 		      List.map (function (s,e,w) -> [(state,e,w)]) res2 in
-		    Printf.printf "andany\n"; flush stdout;
+		    output "andany";
 		    let res =
 		      let s = mkstates states required_states in
 		      List.fold_left
@@ -1967,7 +1974,7 @@ let rec sat_verbose_loop unchecked required required_states annot maxlvl lvl
 	let pm = !Flag_ctl.partial_match in
 	(match (pm,satv unchecked required required_states phi1 env) with
 	  (false,(child1,[])) ->
-	    Printf.printf "and\n"; flush stdout; anno [] [child1]
+	    output "and"; anno [] [child1]
 	| (_,(child1,res1)) ->
 	    let new_required = extend_required res1 required in
 	    let new_required_states = get_required_states res1 in
@@ -1976,7 +1983,7 @@ let rec sat_verbose_loop unchecked required required_states annot maxlvl lvl
 	    (match (pm,satv unchecked new_required new_required_states phi2
 		env) with
 	      (false,(child2,[])) ->
-		Printf.printf "andany\n"; flush stdout;
+		output "andany";
 		anno res1 [child1;child2]
 	    | (_,(child2,res2)) ->
 		let res =
@@ -1992,28 +1999,28 @@ let rec sat_verbose_loop unchecked required required_states annot maxlvl lvl
 		anno res [child1; child2]))
     | A.InnerAnd(phi1) ->
 	let (child1,res1) = satv unchecked required required_states phi1 env in
-	Printf.printf "uncheck\n"; flush stdout;
+	output "uncheck";
 	anno (inner_and res1) [child1]
     | A.EX(dir,phi1)       ->
 	let new_required_states =
 	  get_children_required_states dir m required_states in
 	let (child,res) =
 	  satv unchecked required new_required_states phi1 env in
-	Printf.printf "EX\n"; flush stdout;
+	output "EX";
 	anno (satEX dir m res required_states) [child]
     | A.AX(dir,strict,phi1)       ->
 	let new_required_states =
 	  get_children_required_states dir m required_states in
 	let (child,res) =
 	  satv unchecked required new_required_states phi1 env in
-	Printf.printf "AX\n"; flush stdout;
+	output "AX";
 	let res = strict_A1 strict satAX satEX dir m res required_states in
 	anno res [child]
     | A.EF(dir,phi1)       ->
 	let new_required_states = get_reachable dir m required_states in
 	let (child,res) =
 	  satv unchecked required new_required_states phi1 env in
-	Printf.printf "EF\n"; flush stdout;
+	output "EF";
 	anno (satEF dir m res new_required_states) [child]
     | A.AF(dir,strict,phi1) ->
 	if !Flag_ctl.loop_in_src_code
@@ -2025,7 +2032,7 @@ let rec sat_verbose_loop unchecked required required_states annot maxlvl lvl
 	  (let new_required_states = get_reachable dir m required_states in
 	  let (child,res) =
 	    satv unchecked required new_required_states phi1 env in
-	  Printf.printf "AF\n"; flush stdout;
+	  output "AF";
 	  let res =
 	    strict_A1 strict satAF satEF dir m res new_required_states in
 	  anno res [child])
@@ -2033,13 +2040,13 @@ let rec sat_verbose_loop unchecked required required_states annot maxlvl lvl
 	let new_required_states = get_reachable dir m required_states in
 	let (child,res) =
 	  satv unchecked required new_required_states phi1 env in
-	Printf.printf "EG\n"; flush stdout;
+	output "EG";
 	anno (satEG dir m res new_required_states) [child]
     | A.AG(dir,strict,phi1)       ->
 	let new_required_states = get_reachable dir m required_states in
 	let (child,res) =
 	  satv unchecked required new_required_states phi1 env in
-	Printf.printf "AG\n"; flush stdout;
+	output "AG";
 	let res = strict_A1 strict satAG satEF dir m res new_required_states in
 	anno res [child]
 
@@ -2047,13 +2054,13 @@ let rec sat_verbose_loop unchecked required required_states annot maxlvl lvl
 	let new_required_states = get_reachable dir m required_states in
 	(match satv unchecked required new_required_states phi2 env with
 	  (child2,[]) ->
-	    Printf.printf "EU\n"; flush stdout;
+	    output "EU";
 	    anno [] [child2]
 	| (child2,res2) ->
 	    let new_required = extend_required res2 required in
 	    let (child1,res1) =
 	      satv unchecked new_required new_required_states phi1 env in
-	    Printf.printf "EU\n"; flush stdout;
+	    output "EU";
 	    anno (satEU dir m res1 res2 new_required_states (fun y str -> ()))
 	      [child1; child2])
     | A.AW(dir,strict,phi1,phi2)      ->
@@ -2061,12 +2068,12 @@ let rec sat_verbose_loop unchecked required required_states annot maxlvl lvl
 	  let new_required_states = get_reachable dir m required_states in
 	  (match satv unchecked required new_required_states phi2 env with
 	    (child2,[]) ->
-	      Printf.printf "AW %b\n" unchecked; flush stdout; anno [] [child2]
+	      output (Printf.sprintf "AW %b" unchecked); anno [] [child2]
 	  | (child2,res2) ->
 	      let new_required = extend_required res2 required in
 	      let (child1,res1) =
 		satv unchecked new_required new_required_states phi1 env in
-	      Printf.printf "AW %b\n" unchecked; flush stdout;
+	      output (Printf.sprintf "AW %b" unchecked);
 	      let res =
 		strict_A2 strict satAW satEF dir m res1 res2
 		  new_required_states in
@@ -2075,12 +2082,12 @@ let rec sat_verbose_loop unchecked required required_states annot maxlvl lvl
 	let new_required_states = get_reachable dir m required_states in
 	(match satv unchecked required new_required_states phi2 env with
 	  (child2,[]) ->
-	    Printf.printf "AU\n"; flush stdout; anno [] [child2]
+	    output "AU"; anno [] [child2]
 	| (child2,s2) ->
 	    let new_required = extend_required s2 required in
 	    let (child1,s1) =
 	      satv unchecked new_required new_required_states phi1 env in
-	    Printf.printf "AU\n"; flush stdout;
+	    output "AU";
 	    let res =
 	      strict_A2au strict satAU satEF dir m s1 s2 new_required_states
 		(fun y str -> ()) in
@@ -2093,7 +2100,7 @@ let rec sat_verbose_loop unchecked required required_states annot maxlvl lvl
 		   A[E[phi1 U phi2] & phi1 W phi2]
 		   the and is nonstrict *)
 		(* tmp_res is bigger than s2, so perhaps closer to s1 *)
-	      Printf.printf "AW\n"; flush stdout;
+	      output "AW";
 	      let s1 =
 		triples_conj
 		  (satEU dir m s1 tmp_res new_required_states
@@ -2111,7 +2118,7 @@ let rec sat_verbose_loop unchecked required required_states annot maxlvl lvl
 	let new_required = drop_required v required in
 	let (child,res) =
 	  satv unchecked new_required required_states phi1 env in
-	Printf.printf "exists\n"; flush stdout;
+	output "exists";
 	anno (triples_witness v unchecked (not keep) res) [child]
     | A.Let(v,phi1,phi2)   ->
 	let (child1,res1) =
@@ -2127,7 +2134,7 @@ let rec sat_verbose_loop unchecked required required_states annot maxlvl lvl
 	  satv unchecked required required_states phi2 ((v,res1) :: env) in
 	anno res2 [child1;child2]
     | A.Ref(v)             ->
-	Printf.printf "Ref\n"; flush stdout;
+	output "Ref";
 	let res = List.assoc v env in
 	let res =
 	  if unchecked
@@ -2172,9 +2179,9 @@ let simpleanno l phi res =
   match phi with
     | A.False              -> pp "False"
     | A.True               -> pp "True"
-    | A.Pred(p)            -> pp ("Pred" ^ (Common.dump p))
+    | A.Pred(p)            -> pp ("Pred" ^ (Dumper.dump p))
     | A.Not(phi)           -> pp "Not"
-    | A.Exists(_,v,phi)    -> pp ("Exists " ^ (Common.dump(v)))
+    | A.Exists(_,v,phi)    -> pp ("Exists " ^ (Dumper.dump(v)))
     | A.And(_,phi1,phi2)   -> pp "And"
     | A.AndAny(dir,_,phi1,phi2) -> pp "AndAny"
     | A.HackForStmt(dir,_,phi1,phi2) -> pp "HackForStmt"
@@ -2401,30 +2408,22 @@ let print_bench _ =
 (* ---------------------------------------------------------------------- *)
 (* preprocessing: ignore irrelevant functions *)
 
-let preprocess (cfg,_,_) label = function
+let preprocess (cfg,label,preproc,_) = function
     [] -> true (* no information, try everything *)
   | l ->
       let sz = G.size cfg in
       let verbose_output pred = function
-	  [] ->
+	  false ->
 	    Printf.printf "did not find:\n";
 	    P.print_predicate pred; Format.print_newline()
-	| _ ->
+	| true ->
 	    Printf.printf "found:\n";
 	    P.print_predicate pred; Format.print_newline();
 	    Printf.printf "but it was not enough\n" in
       let get_any verbose x =
-	let res =
-	  try Hashtbl.find memo_label x
-	  with
-	    Not_found ->
-	      (let triples = label x in
-	      let filtered =
-		List.map (function (st,th,_) -> (st,th)) triples in
-	      Hashtbl.add memo_label x filtered;
-	      filtered) in
-	if verbose then verbose_output x res;
-	not([] = res) in
+	let res = preproc x in
+	(if verbose then verbose_output x res);
+	res in
       let get_all l =
 	(* don't bother testing when there are more patterns than nodes *)
 	if List.length l > sz-2
@@ -2461,9 +2460,9 @@ let sat m phi reqopt =
     | Some x -> step_count := x);
     Hashtbl.clear reachable_table;
     Hashtbl.clear memo_label;
-    let (x,label,states) = m in
-    if (!Flag_ctl.bench > 0) or (preprocess m label reqopt)
-    then
+    let (x,label,preproc,states) = m in
+    if (!Flag_ctl.bench > 0) || preprocess m reqopt
+    then    
       ((* to drop when Yoann initialized this flag *)
       if List.exists (G.extract_is_loop x) states
       then Flag_ctl.loop_in_src_code := true;
@@ -2494,7 +2493,6 @@ let sat m phi reqopt =
       then Common.pr2 "missing something required";
        [])
   with Steps -> []
-;;
 
 (* ********************************************************************** *)
 (* End of Module: CTL_ENGINE                                              *)
