@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./ctl_engine.mli"
 open Ast_ctl
 
 module type SUBST =
@@ -80,11 +83,12 @@ module CTL_ENGINE :
 	type ('pred,'anno) triples =
 	    (G.node * substitution * ('pred,'anno) witness list) list
 
-        val sat :
-          G.cfg * (P.t -> (P.t,'anno) triples) * G.node list ->
-            (P.t, SUB.mvar, 'c) Ast_ctl.generic_ctl ->
-	      (P.t list list (* optional and required things *)) ->
-		(P.t,'anno) triples
+	val sat :
+	    G.cfg * (P.t -> (P.t,'anno) triples) * (P.t -> bool) *
+	    G.node list ->
+              (P.t, SUB.mvar, 'c) Ast_ctl.generic_ctl ->
+		(P.t list list (* optional and required things *)) ->
+		  (P.t,'anno) triples
 
 	val print_bench : unit -> unit
       end
