@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./asttomember.ml"
 (* on the first pass, onlyModif is true, so we don't see all matched nodes,
 only modified ones *)
 
@@ -41,23 +44,30 @@ let contains_modif used_after x =
   if List.exists (function x -> List.mem x used_after) (Ast.get_fvs x)
   then true
   else
-    let bind x y = x or y in
+    let bind x y = x || y in
     let option_default = false in
     let do_nothing r k e = k e in
+    let annotated_decl_bef decl =
+      match Ast.unwrap decl with
+	Ast.DElem(bef,_,_) -> bef
+      | _ -> failwith "not possible" in
     let rule_elem r k re =
       let res = k re in
       match Ast.unwrap re with
-	Ast.FunHeader(bef,_,fninfo,name,lp,params,rp) ->
-	  bind (mcode r ((),(),bef,Ast.NoMetaPos)) res
-      | Ast.Decl(bef,_,decl) ->
-	  bind (mcode r ((),(),bef,Ast.NoMetaPos)) res
+	Ast.FunHeader(bef,_,fninfo,name,lp,params,va,rp) ->
+	  bind (mcode r ((),(),bef,[])) res
+      | Ast.Decl decl ->
+	  bind (mcode r ((),(),annotated_decl_bef decl,[])) res
+      | Ast.ForHeader(fr,lp,Ast.ForDecl decl,e2,sem2,e3,rp) ->
+	  bind (mcode r ((),(),annotated_decl_bef decl,[])) res
       | _ -> res in
     let recursor =
       V.combiner bind option_default
 	mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
-	mcode
+	mcode mcode mcode
 	do_nothing do_nothing do_nothing do_nothing do_nothing
 	do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
+	do_nothing do_nothing do_nothing do_nothing do_nothing
 	do_nothing rule_elem do_nothing do_nothing do_nothing do_nothing in
     recursor.V.combiner_rule_elem x
 
@@ -65,7 +75,7 @@ let contains_modif used_after x =
 let contains_constant x =
   match Ast.get_inherited x with
     [] ->
-      let bind x y = x or y in
+      let bind x y = x || y in
       let option_default = false in
       let do_nothing r k e = k e in
       let mcode _ _ = false in
@@ -80,9 +90,10 @@ let contains_constant x =
       let recursor =
 	V.combiner bind option_default
 	  mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
-	  mcode
+	  mcode mcode mcode
 	  do_nothing do_nothing do_nothing do_nothing do_nothing
 	  ident expr do_nothing do_nothing do_nothing do_nothing
+	  do_nothing do_nothing do_nothing do_nothing do_nothing
 	  do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing in
       recursor.V.combiner_rule_elem x
   | _ -> true
@@ -98,35 +109,57 @@ let print_info = function
 	    (List.length disj);
 	  List.iter
 	    (function (_,thing) ->
+	      Printf.printf "%s\n"
+		(Dumper.dump thing.Ast.positive_inherited_positions);
 	      Printf.printf "%s\n"
 		(Pretty_print_cocci.rule_elem_to_string thing))
-	    disj;)
+	    disj)
 	l
 
 (* --------------------------------------------------------------------- *)
 
-(* drop all distinguishing information from a term *)
-let strip =
-  let do_nothing r k e = Ast.make_term (Ast.unwrap (k e)) in
+(* drop all distinguishing information from a term except inherited
+   variables, which are used to improve efficiency of matching process *)
+let strip x =
+  let do_nothing r k e =
+    let inh = Ast.get_inherited e in
+    let inh_pos = Ast.get_inherited_pos e in
+    Ast.make_inherited_term (Ast.unwrap (k e)) inh inh_pos in
   let do_absolutely_nothing r k e = k e in
   let mcode m = Ast.make_mcode(Ast.unwrap_mcode m) in
+  let decl r k d =
+    let res = do_nothing r k d in
+    if Ast.get_safe_decl d
+    then {res with Ast.safe_for_multi_decls = true}
+    else res in
+  let annotated_decl no_mcode decl =
+    match Ast.unwrap decl with
+      Ast.DElem(bef,allminus,d) ->
+	Ast.rewrap decl (Ast.DElem(no_mcode,allminus,d))
+    | _ -> failwith "not possible" in
   let rule_elem r k re =
     let res = do_nothing r k re in
     let no_mcode = Ast.CONTEXT(Ast.NoPos,Ast.NOTHING) in
     match Ast.unwrap res with
-      Ast.FunHeader(bef,b,fninfo,name,lp,params,rp) ->
+      Ast.FunHeader(bef,b,fninfo,name,lp,params,va,rp) ->
 	Ast.rewrap res
-	  (Ast.FunHeader(no_mcode,b,fninfo,name,lp,params,rp))
-    | Ast.Decl(bef,b,decl) -> Ast.rewrap res (Ast.Decl(no_mcode,b,decl))
+	  (Ast.FunHeader(no_mcode,b,fninfo,name,lp,params,va,rp))
+    | Ast.Decl decl -> Ast.rewrap res (Ast.Decl(annotated_decl no_mcode decl))
+    | Ast.ForHeader(fr,lp,Ast.ForDecl(decl),e2,sem2,e3,rp) ->
+	Ast.rewrap res
+	  (Ast.ForHeader(fr,lp,Ast.ForDecl(annotated_decl no_mcode decl),
+			 e2,sem2,e3,rp))
     | _ -> res in
   let recursor =
     V.rebuilder
-      mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
+      mcode mcode mcode mcode mcode mcode mcode mcode mcode
+      mcode mcode mcode mcode mcode
       do_nothing do_nothing do_nothing do_nothing do_nothing
       do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
-      do_nothing rule_elem do_nothing do_nothing
+      do_nothing do_nothing do_nothing do_nothing
+      decl do_absolutely_nothing rule_elem do_nothing do_nothing
       do_nothing do_absolutely_nothing in
-  recursor.V.rebuilder_rule_elem
+  recursor.V.rebuilder_rule_elem x
 
 (* --------------------------------------------------------------------- *)
 
@@ -156,9 +189,23 @@ let conj_wrapped x l = conj [List.map (function x -> (1,strip x)) x] l
 (* --------------------------------------------------------------------- *)
 (* the main translation loop *)
 
-let rule_elem re =
+let rec rule_elem re =
   match Ast.unwrap re with
-    Ast.DisjRuleElem(res) -> [[(List.length res,strip re)]]
+    Ast.DisjRuleElem(res) ->
+      (* why was the following done?  ors have to be kept together for
+	efficiency, so they are considered at once and not individually
+        anded with everything else *)
+      let re =
+	let all_inhs = List.map Ast.get_inherited res in
+	let inhs =
+	  List.fold_left
+	    (function prev ->
+	      function inh ->
+		Common.inter_set inh prev)
+	    (List.hd all_inhs) (List.tl all_inhs) in
+	let inhs_poss = Ast.get_inherited_pos re in (* already intersection *)
+	Ast.make_inherited_term (Ast.unwrap re) inhs inhs_poss in
+      [[(List.length res,strip re)]]
   | _ -> [[(1,strip re)]]
 
 let conj_one testfn x l =
@@ -187,7 +234,7 @@ and statement testfn mcode tail stmt : 'a list list =
       |	_ -> if testfn ast then rule_elem ast else [])
   | Ast.Seq(lbrace,body,rbrace) ->
       let body_info = statement_list testfn mcode tail body in
-      if testfn lbrace or testfn rbrace
+      if testfn lbrace || testfn rbrace
       then conj_wrapped [lbrace;rbrace] body_info
       else body_info
 
@@ -195,7 +242,7 @@ and statement testfn mcode tail stmt : 'a list list =
   | Ast.While(header,branch,(_,_,_,aft))
   | Ast.For(header,branch,(_,_,_,aft))
   | Ast.Iterator(header,branch,(_,_,_,aft)) ->
-      if testfn header or mcode () ((),(),aft,Ast.NoMetaPos)
+      if testfn header || mcode () ((),(),aft,[])
       then conj (rule_elem header) (statement testfn mcode tail branch)
       else statement testfn mcode tail branch
 
@@ -204,7 +251,7 @@ and statement testfn mcode tail stmt : 'a list list =
 	conj
 	  (statement_list testfn mcode false decls)
 	  (case_lines testfn mcode tail cases) in
-      if testfn header or testfn lb or testfn rb
+      if testfn header || testfn lb || testfn rb
       then conj (rule_elem header) body_info
       else body_info
 
@@ -213,7 +260,7 @@ and statement testfn mcode tail stmt : 'a list list =
 	conj
 	  (statement testfn mcode tail branch1)
 	  (statement testfn mcode tail branch2) in
-      if testfn ifheader or mcode () ((),(),aft,Ast.NoMetaPos)
+      if testfn ifheader || mcode () ((),(),aft,[])
       then conj (rule_elem ifheader) branches
       else branches
 
@@ -232,15 +279,21 @@ and statement testfn mcode tail stmt : 'a list list =
 
   | Ast.Dots(_,whencodes,_,_) -> []
 
-  | Ast.FunDecl(header,lbrace,body,rbrace) ->
+  | Ast.FunDecl(header,lbrace,body,rbrace,(_,_,_,aft)) ->
       let body_info = statement_list testfn mcode true body in
-      if testfn header or testfn lbrace or testfn rbrace
+      if testfn header || testfn lbrace || testfn rbrace ||
+	mcode () ((),(),aft,[])
       then conj (rule_elem header) body_info
       else body_info
 
   | Ast.Define(header,body) ->
       conj_one testfn header (statement_list testfn mcode tail body)
 
+  | Ast.AsStmt(stm,asstm) ->
+      conj
+	(statement testfn mcode tail stm)
+	(statement testfn mcode tail asstm)
+
   | Ast.OptStm(stm) -> []
 
   | Ast.UniqueStm(stm) -> statement testfn mcode tail stm
@@ -270,7 +323,7 @@ and case_line testfn mcode tail case =
 let top_level testfn mcode t : 'a list list =
   match Ast.unwrap t with
     Ast.FILEINFO(old_file,new_file) -> failwith "not supported fileinfo"
-  | Ast.DECL(stmt) -> statement testfn mcode false stmt
+  | Ast.NONDECL(stmt) -> statement testfn mcode false stmt
   | Ast.CODE(stmt_dots) -> statement_list testfn mcode false stmt_dots
   | Ast.ERRORWORDS(exps) -> failwith "not supported errorwords"
 
@@ -297,6 +350,14 @@ let asttomemberz (_,_,l) used_after =
         let info =
           List.sort (function (n1,_) -> function (n2,_) -> compare n1 n2)
             info in
+	let info =
+	  let (with_pos,without_pos) =
+	    (* put cases with inherited positions first *)
+	    List.partition
+	      (function (_,thing) ->
+		not (thing.Ast.positive_inherited_positions = []))
+	      info in
+	  with_pos @ without_pos in
         List.map (function (_,x) -> (Lib_engine.Match(x),CTL.Control)) info)
       l in
   List.map2
@@ -309,14 +370,14 @@ let asttomemberz (_,_,l) used_after =
       | _ -> process_one min)
     (List.map (top_level contains_constant no_mcode) l)
     (List.combine
-        (List.map2
-           (function x -> function ua -> function _ ->
-             top_level (contains_modif ua) mcode x)
-           l used_after)
-        (List.map
-           (function x -> function _ ->
-             top_level (function _ -> true) no_mcode x)
-           l))
+       (List.map2
+          (function x -> function ua -> function _ ->
+            top_level (contains_modif ua) mcode x)
+          l used_after)
+       (List.map
+          (function x -> function _ ->
+            top_level (function _ -> true) no_mcode x)
+          l))
 
 let asttomember r used_after =
   match r with
