@@ -1,5 +1,9 @@
 (*
- * Copyright 2005-2010, Ecole des Mines de Nantes, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
+ * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
+ * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
  * This file is part of Coccinelle.
  *
@@ -20,5 +24,7 @@
  *)
 
 
+# 0 "./asttomember.mli"
 val asttomember : Ast_cocci.rule -> Ast_cocci.meta_name list list ->
-  (Lib_engine.predicate * Ast_cocci.meta_name Ast_ctl.modif) list list list
+  (Lib_engine.predicate * Ast_cocci.meta_name Ast_ctl.modif)
+    list list list
