@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./check_exhaustive_pattern.ml"
 
 (* Just to warn me when there is some news in the types in
  * ast_cocci.ml or even ast_c.ml, so that I can then adjust my code in
@@ -45,8 +48,8 @@ let dumb_astcocci_rule_elem = function
  | A.TopExp expr -> ()
  | A.Ty ty -> ()
  | A.TopInit init -> ()
- | A.FunHeader (bef,allminus, fninfo, ida, _, paramsa, _) -> ()
- | A.Decl (bef,allminus,decla) -> ()
+ | A.FunHeader (bef,allminus, fninfo, ida, _, paramsa, _, _) -> ()
+ | A.Decl decl -> ()
  | A.SeqStart _ -> ()
  | A.SeqEnd _ -> ()
  | A.ExprStatement (ea, _) -> ()
@@ -55,7 +58,7 @@ let dumb_astcocci_rule_elem = function
  | A.WhileHeader (_, _, ea, _) -> ()
  | A.DoHeader _ -> ()
  | A.WhileTail (_,_,ea,_,_) -> ()
- | A.ForHeader (_, _, ea1opt, _, ea2opt, _, ea3opt, _) -> ()
+ | A.ForHeader (_, _, _, ea2opt, _, ea3opt, _) -> ()
  | A.IteratorHeader (ia1, ia2, ea, ia3) -> ()
  | A.SwitchHeader _ -> ()
  | A.Break _ -> ()
@@ -64,8 +67,10 @@ let dumb_astcocci_rule_elem = function
  | A.Goto(_,_,_) -> ()
  | A.Return _ -> ()
  | A.ReturnExpr (_, ea, _) -> ()
+ | A.Exec(_,_,_,_) -> ()
  | A.DefineHeader _ -> ()
  | A.Undef _ -> ()
+ | A.Pragma _ -> ()
  | A.Include _ -> ()
  | A.Default _ -> ()
  | A.Case _ -> ()
@@ -74,32 +79,37 @@ let dumb_astcocci_rule_elem = function
 let dumb_astcocci_decl = function
  | A.UnInit (stg, typa, sa, _)     -> ()
  | A.Init (stg, typa, sa, _, expa, _) -> ()
+ | A.FunProto _ -> ()
  | A.TyDecl (typa, _)     -> ()
  | A.MacroDecl(fn, _, eas, _, _) -> ()
- | A.Ddots(dots,whencode) -> ()
+ | A.MacroDeclInit(fn, _, eas, _, _, _, _) -> ()
  | A.MetaDecl _ -> ()
  | A.MetaField _ -> ()
  | A.MetaFieldList _ -> ()
+ | A.AsDecl _ -> ()
  | A.Typedef(d,ty1,ty2,pv) -> ()
  | A.DisjDecl xs -> ()
  | A.OptDecl _ | A.UniqueDecl _ -> ()
 
-let dumb_astcocci_initialiser = function
+let dumb_astcocci_initialiser = function (* seems same as the above *)
     A.Init(stg,ty,id,eq,ini,sem) -> ()
   | A.UnInit(stg,ty,id,sem) -> ()
+  | A.FunProto _ -> ()
   | A.MacroDecl(fn, _, eas, _, _) -> ()
+  | A.MacroDeclInit(fn, _, eas, _, _, _, _) -> ()
   | A.TyDecl(ty,sem) -> ()
   | A.Typedef(d,ty1,ty2,pv) -> ()
   | A.DisjDecl(decls) -> ()
-  | A.Ddots(dots,whencode) -> ()
   | A.MetaDecl(name,_,_) -> ()
   | A.MetaField(name,_,_) -> ()
   | A.MetaFieldList(name,_,_,_) -> ()
+  | A.AsDecl(_,_) -> ()
   | A.OptDecl(decl) -> ()
   | A.UniqueDecl(decl) -> ()
 
 let dumb_astcocci_expr = function
  | A.MetaExpr (ida,_,_, opttypa, _, _) -> ()
+ | A.AsExpr (_,_) -> ()
  | A.Edots (_,_) -> ()
  | A.MetaErr _ -> ()
  | A.Ident ida -> ()
@@ -107,8 +117,11 @@ let dumb_astcocci_expr = function
  | A.Constant (A.Char sa,_,_,_) -> ()
  | A.Constant (A.Int sa,_,_,_) -> ()
  | A.Constant (A.Float sa,_,_,_) -> ()
+ | A.Constant (A.DecimalConst _,_,_,_) -> ()
+ | A.StringConstant (lq,frags,rq) -> ()
  | A.FunCall (ea1, _, eas, _) -> ()
  | A.Assignment (ea1, opa, ea2, _) -> ()
+ | A.Sequence (ea1, opa, ea2) -> ()
  | A.CondExpr (ea1,_,ea2opt,_,ea3) -> ()
  | A.Postfix (ea, opa) -> ()
  | A.Infix (ea, opa) -> ()
@@ -122,6 +135,7 @@ let dumb_astcocci_expr = function
  | A.SizeOfExpr (_, ea) -> ()
  | A.SizeOfType (_, _, typa, _) -> ()
  | A.TypeExp (typa) -> ()
+ | A.Constructor (_, typa, _, ia) -> ()
  | A.Paren (_, ea, _) -> ()
  | A.NestExpr _ -> ()
  | A.MetaExprList _ -> ()
@@ -133,7 +147,8 @@ let dumb_astcocci_expr = function
  | A.OptExp _ -> ()
 
 let dumb_astcocci_fulltype = function
-    A.Type(cv,ty) -> ()
+    A.Type(_,cv,ty) -> ()
+  | A.AsType(_,_) -> ()
   | A.DisjType(types) -> ()
   | A.OptType(ty) -> ()
   | A.UniqueType(ty) -> ()
@@ -144,8 +159,8 @@ let dumb_astcocci_type = function
  | A.SignedT (signa,tya) -> ()
  | A.Pointer (typa, _) -> ()
  | A.FunctionPointer(ty,lp1,star,rp1,lp2,params,rp2) -> ()
- | A.FunctionType _ -> ()
  | A.Array (typa, _, eaopt, _) -> ()
+ | A.Decimal(_, _, _, _, _, _) -> ()
  | A.EnumName(en, ena) -> ()
  | A.EnumDef(ty, lb, ids, rb) -> ()
  | A.StructUnionName(sa, sua) -> ()
