@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./check_reachability.ml"
 (* ---------------------------------------------------------------- *)
 (* code to check for ambiguities *)
 
@@ -164,9 +167,10 @@ let test_formula state formula cfg =
 	    (List.map
 	       (fun (nodei, node) ->
 		 match Control_flow_c.unwrap node with
-		   Control_flow_c.AfterNode -> [(nodei,[],[])]
+		   Control_flow_c.AfterNode _ -> [(nodei,[],[])]
 		 | _ -> [])
 	       cfg#nodes#tolist) in
+    let preproc _ = true in
     let verbose = !Flag_ctl.verbose_ctl_engine in
     let pm = !Flag_ctl.partial_match in
 (*     let gt = !Flag_ctl.graphical_trace in *)
@@ -175,7 +179,7 @@ let test_formula state formula cfg =
     Flag_ctl.checking_reachability := true;
 (*     Flag_ctl.graphical_trace := ""; *)
     let res =
-      ENGINE.sat (cfg,label,List.map fst cfg#nodes#tolist)
+      ENGINE.sat (cfg,label,preproc,List.map fst cfg#nodes#tolist)
 	(CTL.And(CTL.NONSTRICT,CTL.Pred(Node(state)),formula))
 	[[Node(state)]] in
     Flag_ctl.verbose_ctl_engine := verbose;
@@ -201,7 +205,7 @@ type ('a,'b,'c,'d,'e) triples =
       ('b, ('c, 'd) Wrapper_ctl.wrapped_binding) CTL.generic_subst list, 'e)
      CTL.generic_witnesstree list) list
 
-let check_reachability triples cfg =
+let check_reachability rulename triples cfg =
   Hashtbl.clear modified;
   List.iter build_modified triples;
   let formulas = create_formulas() in
@@ -211,11 +215,15 @@ let check_reachability triples cfg =
       then
 	if test_formula node ef_formula cfg
 	then
-	  Printf.printf "warning: node %d may be inconsistently modified\n"
-	    node
+	  let n = cfg#nodes#find node in
+	  Printf.printf
+	    "warning: %s, node %d: %s in %s may be inconsistently modified\n"
+	    rulename node (snd n) !Flag.current_element
 	else ()
       else
+	let n = cfg#nodes#find node in
 	failwith
 	  (Printf.sprintf
-	     "node %d reachable by inconsistent control-flow paths" node))
+	     "%s: node %d: %s in %s reachable by inconsistent control-flow paths"
+	     rulename node (snd n) !Flag.current_element))
     formulas
