@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./cocci_vs_c.mli"
 (*****************************************************************************)
 (* Cocci vs C *)
 (*****************************************************************************)
@@ -109,7 +112,10 @@ module type PARAM =
 
     val distrf_e :
       (Ast_cocci.meta_name Ast_cocci.mcode, Ast_c.expression) matcher
-
+    val distrf_assignOp :
+      (Ast_cocci.meta_name Ast_cocci.mcode, Ast_c.assignOp) matcher
+    val distrf_binaryOp :
+      (Ast_cocci.meta_name Ast_cocci.mcode, Ast_c.binaryOp) matcher
     val distrf_args :
       (Ast_cocci.meta_name Ast_cocci.mcode,
       (Ast_c.argument, Ast_c.il) Common.either list)
@@ -138,11 +144,19 @@ module type PARAM =
     val distrf_node :
       (Ast_cocci.meta_name Ast_cocci.mcode, Control_flow_c.node) matcher
 
+    val distrf_fragments :
+      (Ast_cocci.meta_name Ast_cocci.mcode,
+       (Ast_c.string_fragment, Ast_c.il) Common.either list) matcher
+    val distrf_format :
+      (Ast_cocci.meta_name Ast_cocci.mcode, Ast_c.string_format) matcher
+
     val distrf_define_params :
       (Ast_cocci.meta_name Ast_cocci.mcode,
       (string Ast_c.wrap, Ast_c.il) Common.either list)
       matcher
 
+    val distrf_pragmainfo :
+      (Ast_cocci.meta_name Ast_cocci.mcode, Ast_c.pragmainfo) matcher
     val distrf_enum_fields :
       (Ast_cocci.meta_name Ast_cocci.mcode,
        (Ast_c.oneEnumType, Ast_c.il) Common.either list) matcher
@@ -156,6 +170,18 @@ module type PARAM =
       (Ast_c.constant, string) Common.either Ast_c.wrap)
       matcher
 
+    val distrf_ident_list :
+      (Ast_cocci.meta_name Ast_cocci.mcode,
+       (Ast_c.name, Ast_c.il) Common.either list) matcher
+
+    val distrf_exec_code_list :
+      (Ast_cocci.meta_name Ast_cocci.mcode,
+       (Ast_c.exec_code, Ast_c.il) Common.either list) matcher
+
+    val distrf_attrs :
+      (Ast_cocci.meta_name Ast_cocci.mcode,
+       (Ast_c.attribute, Ast_c.il) Common.either list) matcher
+
     (* -------------------------------------------------------------------- *)
     (* Modifying nested expression and nested types, with Exp and Ty *)
     (* -------------------------------------------------------------------- *)
@@ -165,6 +191,7 @@ module type PARAM =
       (Ast_cocci.expression, Control_flow_c.node) matcher
 
     val cocciExpExp :
+      Ast_cocci.mcodekind ->
       (Ast_cocci.expression, Ast_c.expression) matcher ->
       (Ast_cocci.expression, Ast_c.expression) matcher
 
@@ -201,7 +228,10 @@ module type PARAM =
     val optional_storage_flag : (bool -> tin -> 'x tout) -> (tin -> 'x tout)
     val optional_qualifier_flag : (bool -> tin -> 'x tout) -> (tin -> 'x tout)
     val value_format_flag: (bool -> tin -> 'x tout) -> (tin -> 'x tout)
-
+    val optional_declarer_semicolon_flag :
+	(bool -> tin -> 'x tout) -> (tin -> 'x tout)
+    val optional_attributes_flag :
+	(bool -> tin -> 'x tout) -> (tin -> 'x tout)
 
   end
 
