@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./ctlcocci_integration.ml"
 open Common
 
 open Ograph_extended
@@ -80,103 +83,220 @@ let show_isos rule_elem =
 (*****************************************************************************)
 let (-->) x v = Ast_ctl.Subst (x,v);;
 
+type vp = SUCCESS_POS | FAIL_POS | NOPOS
+
+let valid_positions binding = function
+  Lib_engine.Match re ->
+    let vars = re.Ast_cocci.positive_inherited_positions in
+    (match vars with
+      [] -> NOPOS
+    | _ ->
+	let res =
+	  List.for_all
+	    (function v ->
+	      try
+		let b = List.assoc v binding in
+		match b with
+		  Ast_c.MetaPosValList l ->
+		    List.exists
+		      (function (_,elem,_,_) -> !Flag.current_element = elem)
+		      l
+		| _ ->
+		    failwith "position variable should have a position binding"
+	      with Not_found -> false)
+	    vars in
+	if res then SUCCESS_POS else FAIL_POS)
+  | _ -> NOPOS
+
 (* Take list of predicate and for each predicate returns where in the
- * control flow it matches, and the set of subsitutions for this match.
+ * control flow it matches, and the set of substitutions for this match.
  *)
-let (labels_for_ctl: string list (* dropped isos *) ->
- (nodei * F.node) list -> Lib_engine.metavars_binding ->
- Lib_engine.label_ctlcocci) =
-  fun dropped_isos nodes binding ->
 
-   (fun p ->
-     show_or_not_predicate p;
+let loop_nodes p check nodes =
+  List.fold_left
+    (fun prev (nodei,node) ->
+      if check node
+      then (nodei, (p,[])) :: prev
+      else prev)
+    [] nodes
 
-     let nodes' = nodes +> List.map (fun (nodei, node) ->
-      (* todo? put part of this code in pattern ? *)
-      (match p, F.unwrap node with
-      | Lib_engine.Paren s,  (F.SeqStart (_, bracelevel, _)) ->
-	  let make_var x = ("",i_to_s x) in
-          [(nodei, (p,[(s --> (Lib_engine.ParenVal (make_var bracelevel)))]))]
-      | Lib_engine.Paren s,  (F.SeqEnd (bracelevel, _)) ->
-	  let make_var x = ("",i_to_s x) in
-          [(nodei, (p,[(s --> (Lib_engine.ParenVal (make_var bracelevel)))]))]
-      | Lib_engine.Paren _, _ -> []
-      | Lib_engine.Label s, _ ->
-          let labels = F.extract_labels node in
-          [(nodei,
-	    (p,[(s --> (Lib_engine.LabelVal (Lib_engine.Absolute labels)))]))]
-      | Lib_engine.BCLabel s, _ ->
-	  (match F.extract_bclabels node with
-	    [] -> [] (* null for all nodes that are not break or continue *)
-	  | labels ->
-              [(nodei,
-		(p,[(s -->
-		  (Lib_engine.LabelVal (Lib_engine.Absolute labels)))]))])
-      | Lib_engine.PrefixLabel s, _ ->
-          let labels = F.extract_labels node in
-          [(nodei,
-	    (p,[(s --> (Lib_engine.LabelVal (Lib_engine.Prefix labels)))]))]
-
-      | Lib_engine.Match (re), _unwrapnode ->
-          let substs =
-            Pattern_c.match_re_node dropped_isos re node binding
-            +> List.map (fun (re', subst) ->
-              Lib_engine.Match (re'), subst
-            )
-          in
-          substs +> List.map (fun (p', subst) ->
-            (nodei,
-              (p',
+let labels_for_ctl (dropped_isos : string list)
+                    (nodes : (nodei * F.node) list)
+                    (binding : Lib_engine.metavars_binding)
+                    p =
+  show_or_not_predicate p;
+  let nodes' =
+    nodes +>
+    match p with
+    | Lib_engine.Match (re) ->
+	List.fold_left
+	  (fun prev (nodei,node) ->
+	    Pattern_c.match_re_node dropped_isos re node binding +>
+            List.fold_left (fun prev (re', subst) ->
+	      let p' = Lib_engine.Match (re') in
+              (nodei,
+               (p',
                 subst +> List.map (fun (s, meta) ->
-                  s --> Lib_engine.NormalMetaVal meta
-                ))))
+                  s --> Lib_engine.NormalMetaVal meta)))
+	      :: prev)
+	      prev)
+	  []
 
-      | Lib_engine.InLoop,      F.InLoopNode ->  [nodei, (p,[])]
-      | Lib_engine.TrueBranch , F.TrueNode ->  [nodei, (p,[])]
-      | Lib_engine.FalseBranch, F.FalseNode -> [nodei, (p,[])]
-      | Lib_engine.After,       F.AfterNode -> [nodei, (p,[])]
-      | Lib_engine.FallThrough, F.FallThroughNode -> [nodei,(p,[])]
-      | Lib_engine.LoopFallThrough, F.LoopFallThroughNode -> [nodei,(p,[])]
-      | Lib_engine.FunHeader,   F.FunHeader _ -> [nodei, (p,[])]
-      | Lib_engine.Top,         F.TopNode ->   [nodei, (p,[])]
-      | Lib_engine.Exit,        F.Exit ->      [nodei, (p,[])]
-      | Lib_engine.ErrorExit,   F.ErrorExit -> [nodei, (p,[])]
-      |	Lib_engine.Goto,        F.Goto(_,_,_) -> [nodei, (p,[])]
+    | Lib_engine.Paren s ->
+	List.fold_left
+	  (fun prev (nodei,node) ->
+	    match F.unwrap node with
+	      F.SeqStart (_, bracelevel, _) ->
+		let make_var x = ("",i_to_s x) in
+		let vl = Lib_engine.ParenVal (make_var bracelevel) in
+		(nodei, (p,[(s --> vl)])) :: prev
+	    | F.SeqEnd (bracelevel, _) ->
+		let make_var x = ("",i_to_s x) in
+		let vl = Lib_engine.ParenVal (make_var bracelevel) in
+		(nodei, (p,[(s --> vl)])) :: prev
+	    | _ -> prev)
+	  []
 
-      | Lib_engine.InLoop , _ -> []
-      | Lib_engine.TrueBranch , _ -> []
-      | Lib_engine.FalseBranch, _ -> []
-      | Lib_engine.After, _ -> []
-      | Lib_engine.FallThrough, _ -> []
-      | Lib_engine.LoopFallThrough, _ -> []
-      | Lib_engine.FunHeader, _  -> []
-      | Lib_engine.Top, _  -> []
-      | Lib_engine.Exit, _  -> []
-      | Lib_engine.ErrorExit, _  -> []
-      | Lib_engine.Goto, _  -> []
+    | Lib_engine.Label s ->
+	List.map
+	  (function (nodei,node) ->
+	    let labels = F.extract_labels node in
+	    let vl = Lib_engine.LabelVal (Lib_engine.Absolute labels) in
+	    (nodei, (p,[(s --> vl)])))
 
-      |	Lib_engine.BindGood s, _ -> [(nodei, (p,[(s --> Lib_engine.GoodVal)]))]
-      |	Lib_engine.BindBad s, _ ->  [(nodei, (p,[(s --> Lib_engine.BadVal)]))]
-      |	Lib_engine.FakeBrace, _ ->
-	  if F.extract_is_fake node then [nodei, (p,[])] else []
+    | Lib_engine.BCLabel s ->
+	List.fold_left
+	  (fun prev (nodei,node) ->
+	    match F.extract_bclabels node with
+	      [] -> (* null for all nodes that are not break or continue *)
+		prev
+	    | labels ->
+		let vl = Lib_engine.LabelVal (Lib_engine.Absolute labels) in
+		(nodei, (p,[(s --> vl)]))::prev)
+	  []
 
-      | Lib_engine.Return, node ->
-          (match node with
-            (* todo? should match the Exit code ?
-             * todo: one day try also to match the special function
-             * such as panic();
-             *)
-          | F.Return _ ->  [nodei, (p,[])]
-          | F.ReturnExpr _ -> [nodei, (p,[])]
-          | _ -> []
-          )
-      )
-     ) +> List.concat
-     in
+    | Lib_engine.PrefixLabel s ->
+	List.map
+	  (function (nodei,node) ->
+	    let labels = F.extract_labels node in
+	    let vl = Lib_engine.LabelVal (Lib_engine.Prefix labels) in
+	    (nodei, (p,[(s --> vl)])))
+
+    | Lib_engine.InLoop ->
+	loop_nodes p
+	  (function node ->
+	    match F.unwrap node with F.InLoopNode -> true | _ -> false)
+
+    | Lib_engine.TrueBranch ->
+	loop_nodes p
+	  (function node ->
+	    match F.unwrap node with F.TrueNode _ -> true | _ -> false)
+
+    | Lib_engine.EscTrueBranch ->
+	loop_nodes p
+	  (function node ->
+	    match F.unwrap node with
+	      F.TrueNode esc when !esc -> true
+	    | _ -> false)
+
+    | Lib_engine.FalseBranch ->
+	loop_nodes p
+	  (function node ->
+	    match F.unwrap node with F.FalseNode -> true | _ -> false)
+
+    | Lib_engine.After ->
+	loop_nodes p
+	  (function node ->
+	    match F.unwrap node with F.AfterNode _ -> true | _ -> false)
+
+    | Lib_engine.FallThrough ->
+	loop_nodes p
+	  (function node ->
+	    match F.unwrap node with F.FallThroughNode -> true | _ -> false)
+
+    | Lib_engine.LoopFallThrough ->
+	loop_nodes p
+	  (function node ->
+	    match F.unwrap node with
+	      F.LoopFallThroughNode -> true
+	    | _ -> false)
+
+    | Lib_engine.FunHeader ->
+	loop_nodes p
+	  (function node ->
+	    match F.unwrap node with F.FunHeader _ -> true | _ -> false)
+
+    | Lib_engine.Top ->
+	loop_nodes p
+	  (function node ->
+	    match F.unwrap node with F.TopNode -> true | _ -> false)
+
+    | Lib_engine.Exit ->
+	loop_nodes p
+	  (function node ->
+	    match F.unwrap node with F.Exit -> true | _ -> false)
+
+    | Lib_engine.ErrorExit ->
+	loop_nodes p
+	  (function node ->
+	    match F.unwrap node with F.ErrorExit -> true | _ -> false)
+
+    | Lib_engine.Goto ->
+	loop_nodes p
+	  (function node ->
+	    match F.unwrap node with F.Goto _ -> true | _ -> false)
+
+    | Lib_engine.Return ->
+	loop_nodes p
+	  (function node ->
+	      (* todo? should match the Exit code ?
+		 * todo: one day try also to match the special function
+		 * such as panic();
+              *)
+	    match F.unwrap node with
+            | F.Return _ -> true
+            | F.ReturnExpr _ -> true
+            | _ -> false)
+
+    | Lib_engine.FakeBrace ->
+	loop_nodes p F.extract_is_fake
+
+    | Lib_engine.BindGood s ->
+	List.map
+	  (function (nodei,_) -> (nodei, (p,[(s --> Lib_engine.GoodVal)])))
+
+    | Lib_engine.BindBad s ->
+	List.map
+	  (function (nodei,_) -> (nodei, (p,[(s --> Lib_engine.BadVal)])))
+
+    | Lib_engine.UnsafeBrace ->
+	  (* cases where it it not safe to put something on the outer side
+	     of braces *)
+	List.fold_left
+	  (fun prev (nodei,node) ->
+	    match F.unwrap node with
+		F.FunHeader _ | F.DoHeader _ | F.TrueNode _ | F.Else _
+	      | F.InLoopNode (* while, for *) | F.SwitchHeader _ ->
+		  (nodei, (p,[])) :: prev
+	      | _ -> prev)
+	  [] in
 
      show_or_not_nodes nodes';
      nodes'
-   )
+
+let quick_labels_for_ctl dropped_isos nodes binding p =
+  show_or_not_predicate p;
+  match valid_positions binding p with
+    SUCCESS_POS -> true
+  | FAIL_POS -> false
+  | NOPOS ->
+      (match p with
+	Lib_engine.Match (re) ->
+	  List.exists
+	    (function (_,node) ->
+	      not (Pattern_c.match_re_node dropped_isos re node binding = []))
+	    nodes
+      |	_ -> true)
 
 (*****************************************************************************)
 (* Some fix flow, for CTL, for unparse *)
@@ -188,7 +308,7 @@ let (control_flow_for_ctl: F.cflow -> ('a, 'b) ograph_mutable) =
 
 
 (* Just make the final node of the control flow loop over itself.
- * It seems that one hypothesis of the SAT algorithm is that each node as at
+ * It seems that one hypothesis of the SAT algorithm is that each node has at
  * least a successor.
  *
  * update: do same for errorexit node.
@@ -210,7 +330,7 @@ let (control_flow_for_ctl: F.cflow -> ('a, 'b) ograph_mutable) =
  * update: also make loop the deadcode nodes, the one that have
  * no predecessor.
  *)
-let (fix_flow_ctl2: F.cflow -> F.cflow) = fun flow ->
+let fix_flow_ctl2 (flow : F.cflow) : F.cflow =
   let g = ref flow in
 
   let topi = F.first_node !g in
@@ -279,8 +399,10 @@ let fix_flow_ctl a =
 let model_for_ctl dropped_isos cflow binding =
  let newflow = cflow (* old: fix_flow_ctl (control_flow_for_ctl cflow) *) in
  let labels = labels_for_ctl dropped_isos (newflow#nodes#tolist) binding  in
+ let quicklabels =
+   quick_labels_for_ctl dropped_isos (newflow#nodes#tolist) binding  in
  let states = List.map fst  newflow#nodes#tolist  in
- newflow, labels, states
+ newflow, labels, quicklabels, states
 
 
 (*****************************************************************************)
@@ -334,7 +456,7 @@ module ENV =
       match (v,v') with
 	(Lib_engine.NormalMetaVal(Ast_c.MetaPosVal(min1,max1)),
 	 Lib_engine.NormalMetaVal(Ast_c.MetaPosVal(min2,max2))) ->
-	   ((min1 <= min2) && (max1 >= max2)) or
+	   ((min1 <= min2) && (max1 >= max2)) ||
 	   ((min2 <= min1) && (max2 >= max1))
       |	(Lib_engine.NormalMetaVal(Ast_c.MetaTypeVal a),
 	 Lib_engine.NormalMetaVal(Ast_c.MetaTypeVal b)) ->
@@ -433,7 +555,7 @@ let (satbis_to_trans_info:
 	  ::prev
 	     (* see BindGood in asttotctl2 *)
       | Lib_engine.BindGood (_) -> prev
-      | _ -> raise Impossible
+      | _ -> raise (Impossible 50)
     ) []
 
 (*****************************************************************************)
@@ -443,12 +565,12 @@ let rec coalesce_positions = function
   | (x,Ast_c.MetaPosValList l)::rest ->
       let (same,others) = List.partition (function (x1,_) -> x =*= x1) rest in
       let ls =
-	List.concat
-	  (List.map
-	     (function
-		 (_,Ast_c.MetaPosValList l) -> l
-	       | _ -> failwith "unexpected non-position")
-	     same) in
+	List.fold_left
+	  (function prev ->
+	    function
+		(_,Ast_c.MetaPosValList l) -> l@prev
+	      | _ -> failwith "unexpected non-position")
+	  [] same in
       let new_ls = List.sort compare (l@ls) in
       (x,Ast_c.MetaPosValList new_ls) :: coalesce_positions others
   | x::rest -> x :: coalesce_positions rest
@@ -466,43 +588,38 @@ let strip env =
 	    Ast_c.MetaStmtVal(Lib_parsing_c.al_inh_statement a)
 	| Ast_c.MetaInitVal a ->
 	    Ast_c.MetaInitVal(Lib_parsing_c.al_inh_init a)
+	| Ast_c.MetaInitListVal a ->
+	    Ast_c.MetaInitListVal(Lib_parsing_c.al_inh_inits a)
 	| x -> (*don't contain binding info*) x in
       (v,vl))
     env
 
-let rec nub ls =
-  match ls with
-    [] -> []
-  | (x::xs) when (List.mem x xs) -> nub xs
-  | (x::xs) -> x::(nub xs)
-
 (*****************************************************************************)
 (* Call ctl engine *)
 (*****************************************************************************)
 let (mysat2:
   Lib_engine.model ->
   (Lib_engine.ctlcocci * (pred list list)) ->
-  (Lib_engine.mvar list*Lib_engine.metavars_binding) ->
+  (string (*rulename*) * Lib_engine.mvar list*Lib_engine.metavars_binding) ->
   (Lib_engine.numbered_transformation_info * bool *
      Lib_engine.metavars_binding * Lib_engine.metavars_binding list)) =
-  fun (flow, label, states) ctl (used_after, binding) ->
+  fun ((flow, label, preproc, states) as m)
+      ctl (rulename, used_after, binding) ->
     let binding2 = metavars_binding_to_binding2 binding in
     let (triples,(trans_info2, returned_any_states, used_after_envs)) =
-      WRAPPED_ENGINE.satbis (flow, label, states) ctl
-	(used_after, binding2)
-    in
+      WRAPPED_ENGINE.satbis m ctl (used_after, binding2) in
     if not (!Flag_parsing_cocci.sgrep_mode || !Flag.sgrep_mode2 ||
             !Flag_matcher.allow_inconsistent_paths)
-    then Check_reachability.check_reachability triples flow;
+    then Check_reachability.check_reachability rulename triples flow;
     let (trans_info2,used_after_fresh_envs) =
       Postprocess_transinfo.process used_after binding2 trans_info2 in
     let used_after_envs =
-      Common.uniq(List.map2 (@) used_after_fresh_envs used_after_envs) in
+      Common.nub (List.map2 (@) used_after_fresh_envs used_after_envs) in
     let trans_info = satbis_to_trans_info trans_info2 in
     let newbindings = List.map metavars_binding2_to_binding used_after_envs in
     let newbindings = List.map coalesce_positions newbindings in
     let newbindings = List.map strip newbindings in
-    let newbindings = nub newbindings in
+    let newbindings = Common.nub newbindings in
     (trans_info, returned_any_states, binding, newbindings)
 
 let mysat a b c =
