@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./ctltotex.ml"
 module CTL = Ast_ctl
 
 let prelude =
@@ -263,12 +266,14 @@ let meta2c (_,s) = s
 let pred2c = function
     Lib_engine.InLoop -> ("\\msf{InLoop}",6)
   | Lib_engine.TrueBranch -> ("\\msf{TrueBranch}",10)
+  | Lib_engine.EscTrueBranch -> ("\\msf{EscTrueBranch}",13)
   | Lib_engine.FalseBranch -> ("\\msf{FalseBranch}",11)
   | Lib_engine.After -> ("\\msf{After}",5)
   | Lib_engine.FallThrough -> ("\\msf{FallThrough}",11)
   | Lib_engine.LoopFallThrough -> ("\\msf{LoopFallThrough}",15)
   | Lib_engine.Return -> ("\\msf{Return}",6)
   | Lib_engine.FunHeader -> ("\\msf{FunHeader}",9)
+  | Lib_engine.UnsafeBrace -> ("\\msf{UnsafeBrace}",11)
   | Lib_engine.Top -> ("\\msf{Top}",3)
   | Lib_engine.Exit -> ("\\msf{Exit}",4)
   | Lib_engine.ErrorExit -> ("\\msf{ErrorExit}",9)
