@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,10 +24,9 @@
  *)
 
 
+# 0 "./ctltotex.mli"
 val totex :
   string ->
     Ast_cocci.rule list ->
-      ((Lib_engine.predicate * Ast_cocci.meta_name Ast_ctl.modif,
-	Ast_cocci.meta_name,Wrapper_ctl.info)
-	 Ast_ctl.generic_ctl * 'a) list list ->
-	   unit
+      (Lib_engine.ctlcocci * 'a) list list ->
+	unit
