@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./lib_engine.ml"
 open Ograph_extended
 
 (*****************************************************************************)
@@ -30,11 +33,11 @@ open Ograph_extended
 type mvar = Ast_cocci.meta_name
 
 type predicate =
-    InLoop | TrueBranch | FalseBranch
-  | After (* pointer to the code after an if or while *)
+    InLoop | TrueBranch | EscTrueBranch | FalseBranch
+  | After (* pointer to the code after a block, if, or while *)
   | FallThrough | LoopFallThrough
   | Return (* any exit from the current function *)
-  | FunHeader | Top | Exit | ErrorExit | Goto
+  | FunHeader | UnsafeBrace | Top | Exit | ErrorExit | Goto
   | Paren of Ast_cocci.meta_name
   | Match of Ast_cocci.rule_elem
   | Label of Ast_cocci.meta_name
@@ -76,7 +79,10 @@ type label_ctlcocci =
  (predicate * (mvar, metavar_binding_kind2) Ast_ctl.generic_substitution))
  list
 
-type model = Control_flow_c.cflow * label_ctlcocci * nodei list
+type quicklabel_ctlcocci = predicate -> bool
+
+type model = Control_flow_c.cflow * label_ctlcocci * quicklabel_ctlcocci *
+      nodei list
 
 type transformation_info =
  (nodei * metavars_binding * Ast_cocci.rule_elem) list
