@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./pattern_c.ml"
 open Common
 
 module Flag_engine = Flag_matcher
@@ -34,6 +37,8 @@ type xinfo = {
   optional_storage_iso : bool;
   optional_qualifier_iso : bool;
   value_format_iso : bool;
+  optional_declarer_semicolon_iso : bool;
+  optional_attributes_iso : bool;
 }
 
 module XMATCH = struct
@@ -85,7 +90,7 @@ module XMATCH = struct
     binding: Lib_engine.metavars_binding;
     binding0: Lib_engine.metavars_binding; (* inherited bindings *)
   }
-  (* 'x is a ('a * 'b) but in fact dont care about 'b, we just tag the SP *)
+  (* 'x is a ('a * 'b) but in fact don't care about 'b, we just tag the SP *)
   (* opti? use set instead of list *)
   type 'x tout = ('x * Lib_engine.metavars_binding) list
 
@@ -115,12 +120,14 @@ module XMATCH = struct
 *)
     let res1 = m1 tin in
     let res2 = m2 tin in
-    let list_bindings_already = List.map snd res1 in
-    res1 ++
-      (res2 +> List.filter (fun (x, binding) ->
-        not
-          (list_bindings_already +> List.exists (fun already ->
-            Lib_engine.equal_binding binding already))
+    if res2 = [] (*try to avoid a trivial @*)
+    then res1
+    else
+      res1 ++
+	(res2 +> List.filter (fun (x, binding) ->
+          not
+            (res1 +> List.exists (fun (_,already) ->
+              Lib_engine.equal_binding binding already))
       ))
 
 
@@ -185,7 +192,7 @@ module XMATCH = struct
 
   (* same as cocciExp, but for expressions in an expression, not expressions
      in a node *)
-  let cocciExpExp = fun expf expa expb -> fun tin ->
+  let cocciExpExp = fun _ expf expa expb -> fun tin ->
 
     let globals = ref [] in
     let bigf = {
@@ -264,29 +271,48 @@ module XMATCH = struct
   let tag_mck_pos_mcode (x,info,mck,pos) posmck stuff = fun tin ->
     [((x, info, tag_mck_pos mck posmck, pos),stuff), tin.binding]
 
+  let is_abstract ii =
+    match Ast_c.pinfo_of_info ii with
+      Ast_c.AbstractLineTok pi -> true
+    | _ -> false
 
   let distrf (ii_of_x_f) =
     fun mcode x -> fun tin ->
-    let (max, min) = Lib_parsing_c.max_min_by_pos (ii_of_x_f x)
-    in
-    let posmck = Ast_cocci.FixPos (min, max) (* subtil: and not max, min !!*)
-    in
-    tag_mck_pos_mcode mcode posmck x tin
+      let iis = ii_of_x_f x in
+      let all_abstract =
+	(* this occurs when matching a metavar type against a type *)
+	List.for_all is_abstract iis in
+      if all_abstract
+      then tag_mck_pos_mcode mcode Ast_cocci.NoPos x tin (* do nothing *)
+      else
+	let (max, min) = Lib_parsing_c.max_min_by_pos iis in
+	let posmck =
+	  Ast_cocci.FixPos (min, max) (* subtil: and not max, min !!*)
+	in
+	tag_mck_pos_mcode mcode posmck x tin
 
-  let distrf_e      = distrf (Lib_parsing_c.ii_of_expr)
-  let distrf_args   = distrf (Lib_parsing_c.ii_of_args)
-  let distrf_type   = distrf (Lib_parsing_c.ii_of_type)
-  let distrf_param  = distrf (Lib_parsing_c.ii_of_param)
-  let distrf_params = distrf (Lib_parsing_c.ii_of_params)
-  let distrf_ini    = distrf (Lib_parsing_c.ii_of_ini)
-  let distrf_inis   = distrf (Lib_parsing_c.ii_of_inis)
-  let distrf_decl   = distrf (Lib_parsing_c.ii_of_decl)
-  let distrf_field  = distrf (Lib_parsing_c.ii_of_field)
-  let distrf_node   = distrf (Lib_parsing_c.ii_of_node)
-  let distrf_enum_fields = distrf (Lib_parsing_c.ii_of_enum_fields)
-  let distrf_struct_fields = distrf (Lib_parsing_c.ii_of_struct_fields)
-  let distrf_cst    = distrf (Lib_parsing_c.ii_of_cst)
-  let distrf_define_params = distrf (Lib_parsing_c.ii_of_define_params)
+  let distrf_e              = distrf Lib_parsing_c.ii_of_expr
+  let distrf_assignOp       = distrf Lib_parsing_c.ii_of_assignOp
+  let distrf_binaryOp       = distrf Lib_parsing_c.ii_of_binaryOp
+  let distrf_args           = distrf Lib_parsing_c.ii_of_args
+  let distrf_type           = distrf Lib_parsing_c.ii_of_type
+  let distrf_param          = distrf Lib_parsing_c.ii_of_param
+  let distrf_params         = distrf Lib_parsing_c.ii_of_params
+  let distrf_ini            = distrf Lib_parsing_c.ii_of_ini
+  let distrf_inis           = distrf Lib_parsing_c.ii_of_inis
+  let distrf_decl           = distrf Lib_parsing_c.ii_of_decl
+  let distrf_field          = distrf Lib_parsing_c.ii_of_field
+  let distrf_node           = distrf Lib_parsing_c.ii_of_node
+  let distrf_fragments      = distrf Lib_parsing_c.ii_of_fragments
+  let distrf_format         = distrf Lib_parsing_c.ii_of_format
+  let distrf_enum_fields    = distrf Lib_parsing_c.ii_of_enum_fields
+  let distrf_struct_fields  = distrf Lib_parsing_c.ii_of_struct_fields
+  let distrf_cst            = distrf Lib_parsing_c.ii_of_cst
+  let distrf_define_params  = distrf Lib_parsing_c.ii_of_define_params
+  let distrf_pragmainfo     = distrf Lib_parsing_c.ii_of_pragmainfo
+  let distrf_ident_list     = distrf Lib_parsing_c.ii_of_ident_list
+  let distrf_exec_code_list = distrf Lib_parsing_c.ii_of_exec_code_list
+  let distrf_attrs          = distrf Lib_parsing_c.ii_of_attrs
 
 
   (* ------------------------------------------------------------------------*)
@@ -332,7 +358,7 @@ module XMATCH = struct
   (* pre: if have declared a new metavar that hide another one, then
    * must be passed with a binding that deleted this metavar
    *
-   * Here we dont use the keep argument of julia. cf f(X,X), J'ai
+   * Here we don't use the keep argument of julia. cf f(X,X), J'ai
    * besoin de garder le X en interne, meme si julia s'en fout elle du
    * X et qu'elle a mis X a DontSaved.
    *)
@@ -372,6 +398,10 @@ module XMATCH = struct
 		    | Some _ -> failwith "Not possible"
 		    | None -> success(Ast_c.MetaIdVal(a,[]))) in
 	      loop c
+          | Ast_c.MetaAssignOpVal op      ->
+	      success(Ast_c.MetaAssignOpVal op)
+          | Ast_c.MetaBinaryOpVal op      ->
+	      success(Ast_c.MetaBinaryOpVal op)
           | Ast_c.MetaFuncVal a      ->
 	      success(Ast_c.MetaFuncVal a)
           | Ast_c.MetaLocalFuncVal a ->
@@ -448,6 +478,13 @@ module XMATCH = struct
 		   then Lib_parsing_c.al_init a
 		   else Lib_parsing_c.semi_al_init a))
 		
+          | Ast_c.MetaInitListVal a ->
+	      success
+		(Ast_c.MetaInitListVal
+		   (if strip
+		   then Lib_parsing_c.al_inits a
+		   else Lib_parsing_c.semi_al_inits a))
+		
           | Ast_c.MetaListlenVal a -> success(Ast_c.MetaListlenVal a)
 		
           | Ast_c.MetaParamVal a ->
@@ -463,37 +500,56 @@ module XMATCH = struct
 		   then Lib_parsing_c.al_params a
 		   else Lib_parsing_c.semi_al_params a))
 		
+          | Ast_c.MetaFragListVal a ->
+	      success
+		(Ast_c.MetaFragListVal
+		   (if strip
+		   then Lib_parsing_c.al_string_fragments a
+		   else Lib_parsing_c.semi_al_string_fragments a))
+          | Ast_c.MetaFmtVal a ->
+	      success
+		(Ast_c.MetaFmtVal
+		   (if strip
+		   then Lib_parsing_c.al_string_format a
+		   else Lib_parsing_c.semi_al_string_format a))
+		
           | Ast_c.MetaPosVal (pos1,pos2) ->
 	      success(Ast_c.MetaPosVal (pos1,pos2))
           | Ast_c.MetaPosValList l -> success (Ast_c.MetaPosValList l))
 
+  let pos_variables tin ia get_pvalu finish =
+    match Ast_cocci.get_pos_var ia with
+      [] -> finish tin
+    | positions ->
+	let pvalu = Ast_c.MetaPosValList(get_pvalu()) in
+	let rec loop tin = function
+	    [] -> finish tin
+	  | Ast_cocci.MetaPos(name,constraints,per,keep,inherited) :: rest ->
+	      check_pos_constraints constraints pvalu
+		(function () ->
+	    (* constraints are satisfied, now see if we are compatible
+	       with existing bindings *)
+		  function new_tin ->
+		    let x = Ast_cocci.unwrap_mcode name in
+		    let new_binding =
+		      check_add_metavars_binding false keep inherited
+			(x, pvalu) tin in
+		    (match  new_binding with
+		      Some binding -> loop {tin with binding = binding} rest
+		    | None -> fail tin))
+		tin in
+	loop tin positions
+
   let envf keep inherited = fun (k, valu, get_max_min) f tin ->
     let x = Ast_cocci.unwrap_mcode k in
     match check_add_metavars_binding true keep inherited (x, valu) tin with
     | Some binding ->
 	let new_tin = {tin with binding = binding} in
-	(match Ast_cocci.get_pos_var k with
-	  Ast_cocci.MetaPos(name,constraints,per,keep,inherited) ->
-	    let pvalu =
-	      let (file,current_element,min,max) = get_max_min() in
-	      Ast_c.MetaPosValList[(file,current_element,min,max)] in
-	    (* check constraints.  success means that there is a match with
-	       one of the constraints, which will ultimately result in
-	       failure. *)
-	    check_pos_constraints constraints pvalu
-	      (function () ->
-		(* constraints are satisfied, now see if we are compatible
-		   with existing bindings *)
-		function new_tin ->
-		  let x = Ast_cocci.unwrap_mcode name in
-		  (match
-		    check_add_metavars_binding false keep inherited (x, pvalu)
-		      new_tin with
-		  | Some binding ->
-		      f () {new_tin with binding = binding}
-		  | None -> fail tin))
-	      new_tin
-	| Ast_cocci.NoMetaPos -> f () new_tin)
+	pos_variables new_tin k
+	  (function _ ->
+	    let (file,current_element,min,max) = get_max_min() in
+	    [(file,current_element,min,max)])
+	  (f ())
     | None -> fail tin
 
   (* ------------------------------------------------------------------------*)
@@ -523,31 +579,29 @@ module XMATCH = struct
   let value_format_flag f = fun tin ->
     f (tin.extra.value_format_iso) tin
 
+  let optional_declarer_semicolon_flag f = fun tin ->
+    f (tin.extra.optional_declarer_semicolon_iso) tin
+
+  let optional_attributes_flag f = fun tin ->
+    f (tin.extra.optional_attributes_iso) tin
+
   (* ------------------------------------------------------------------------*)
   (* Tokens *)
   (* ------------------------------------------------------------------------*)
   let tokenf ia ib = fun tin ->
-    let pos = Ast_c.info_to_fixpos ib in
-    let posmck = Ast_cocci.FixPos (pos, pos) in
-    let finish tin = tag_mck_pos_mcode ia posmck ib tin in
-    match Ast_cocci.get_pos_var ia with
-      Ast_cocci.MetaPos(name,constraints,per,keep,inherited) ->
-	let mpos = Lib_parsing_c.lin_col_by_pos [ib] in
-	let pvalu = Ast_c.MetaPosValList [mpos] in
-	check_pos_constraints constraints pvalu
-	  (function () ->
-	    (* constraints are satisfied, now see if we are compatible
-	       with existing bindings *)
-	    function new_tin ->
-	      let x = Ast_cocci.unwrap_mcode name in
-	      (match
-		check_add_metavars_binding false keep inherited (x, pvalu) tin
-	      with
-		Some binding -> finish {tin with binding = binding}
-	      | None -> fail tin))
-	  tin
-    | _ -> finish tin
-
+    if is_abstract ib
+    then
+      (* for meta var type against type case *)
+      let posmck = Ast_cocci.NoPos in
+      tag_mck_pos_mcode ia posmck ib tin
+    else
+      let pos = Ast_c.info_to_fixpos ib in
+      let posmck = Ast_cocci.FixPos (pos, pos) in
+      let finish tin = tag_mck_pos_mcode ia posmck ib tin in
+      pos_variables tin ia
+	(function _ -> [Lib_parsing_c.lin_col_by_pos [ib]])
+	finish
+      
   let tokenf_mck mck ib = fun tin ->
     let pos = Ast_c.info_to_fixpos ib in
     let posmck = Ast_cocci.FixPos (pos, pos) in
@@ -568,6 +622,10 @@ let match_re_node2 dropped_isos a b binding0 =
       optional_storage_iso   = not(List.mem "optional_storage"   dropped_isos);
       optional_qualifier_iso = not(List.mem "optional_qualifier" dropped_isos);
       value_format_iso       = not(List.mem "value_format"       dropped_isos);
+      optional_declarer_semicolon_iso =
+        not(List.mem "optional_declarer_semicolon"   dropped_isos);
+      optional_attributes_iso =
+        not(List.mem "optional_attributes" dropped_isos);
     };
     XMATCH.binding = [];
     XMATCH.binding0 = binding0;
@@ -579,5 +637,6 @@ let match_re_node2 dropped_isos a b binding0 =
 
 
 let match_re_node a b c d =
-  Common.profile_code "Pattern3.match_re_node"
-    (fun () -> match_re_node2 a b c d)
+  (*Common.profile_code "Pattern3.match_re_node"
+    (fun () -> *) match_re_node2 a b c d
+   (* ) *)
