@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./postprocess_transinfo.ml"
 (* two goals: first drop from the environments things that are not used,
    and second prompt for the names of fresh variables that are used *)
 
@@ -98,7 +101,9 @@ let process_tree inherited_env l =
 			       str
 			   | _ -> failwith "bad id value")
 			 with
-			   Not_found -> failwith "fresh: no binding for meta")
+			   Not_found ->
+			     failwith
+			       ("fresh: no binding for meta "^(Dumper.dump id)))
 		   seed in
 	    string2val(String.concat "" strings)))
       all_fresh in
@@ -142,10 +147,22 @@ let collect_used_after used_after envs l inherited_env =
 
 (* ----------------------------------------------------------------------- *)
 (* distinguish between distinct witness trees, each gets an index n *)
+(* index should be global, so that it can extend over environments *)
+
+let index = ref (-1)
+
+let fold_left_with_index f acc =
+  let rec fold_lwi_aux acc = function
+    | [] -> acc
+    | x::xs ->
+	let n = !index in
+	index := !index + 1;
+	fold_lwi_aux (f acc x n) xs
+  in fold_lwi_aux acc
 
 let numberify trees =
   let trees =
-    Common.fold_left_with_index
+    fold_left_with_index
       (function acc -> function xs -> function n ->
 	(List.map (function x -> (n,x)) xs) @ acc)
       [] trees in
