@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./postprocess_transinfo.mli"
 val process :
     Ast_cocci.meta_name list (* used after *) ->
       (Ast_cocci.meta_name * Lib_engine.metavar_binding_kind2) list
@@ -29,7 +32,7 @@ val process :
     (Ograph_extended.nodei *
        (Ast_cocci.meta_name * Lib_engine.metavar_binding_kind2) list *
        Lib_engine.predicate) list list ->
-	 (int list *
+	 (int list (*index*) *
 	    (Ograph_extended.nodei *
 	       (Ast_cocci.meta_name * Lib_engine.metavar_binding_kind2)
 	       list *
