@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./pretty_print_engine.ml"
 open Common.Infix
 
 open Lib_engine
@@ -36,9 +39,16 @@ let rec pp_binding_kind = function
   | Ast_c.MetaFuncVal      s -> pp ("func " ^ s)
   | Ast_c.MetaLocalFuncVal s -> pp ("localfunc " ^ s)
   | Ast_c.MetaExprVal      (expr,_) -> Pretty_print_c.pp_expression_simple expr
+  | Ast_c.MetaAssignOpVal op        ->
+      pp "meta assign op ";
+      Pretty_print_c.pp_assignOp_simple op
+  | Ast_c.MetaBinaryOpVal op        ->
+      pp "meta binary op ";
+      Pretty_print_c.pp_binaryOp_simple op
   | Ast_c.MetaExprListVal  expr_list -> pp "<<exprlist>>"
   | Ast_c.MetaInitVal      ini ->
       Pretty_print_c.pp_init_simple ini
+  | Ast_c.MetaInitListVal      ini -> pp "<<initlist>>"
   | Ast_c.MetaTypeVal      typ ->
       Pretty_print_c.pp_type_simple typ
   | Ast_c.MetaDeclVal      decl ->
@@ -49,6 +59,9 @@ let rec pp_binding_kind = function
       List.iter Pretty_print_c.pp_field_simple decls
   | Ast_c.MetaStmtVal      statement ->
       Pretty_print_c.pp_statement_simple statement
+  | Ast_c.MetaFmtVal fmt -> Pretty_print_c.pp_string_format_simple fmt
+  | Ast_c.MetaFragListVal frags ->
+      frags +> (List.iter Pretty_print_c.pp_string_fragment_simple)
   | Ast_c.MetaParamVal     params -> pp "<<param>>"
   | Ast_c.MetaParamListVal params -> pp "<<paramlist>>"
   | Ast_c.MetaListlenVal n -> pp (string_of_int n)
@@ -101,11 +114,13 @@ let pp_binding_kind2 = function
 let rec pp_predicate = function
   | InLoop -> pp "InLoop"
   | TrueBranch -> pp "TrueBranch"
+  | EscTrueBranch -> pp "EscTrueBranch"
   | FalseBranch -> pp "FalseBranch"
   | After -> pp "After"
   | FallThrough -> pp "FallThrough"
   | LoopFallThrough -> pp "LoopFallThrough"
   | Return -> pp "Return"
+  | UnsafeBrace -> pp "UnsafeBrace"
   | FunHeader -> pp "FunHeader"
   | Top -> pp "Top"
   | ErrorExit -> pp "ErrorExit"
