@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./transformation_c.ml"
 open Common
 
 module F = Control_flow_c
@@ -35,6 +38,8 @@ type xinfo = {
   optional_storage_iso : bool;
   optional_qualifier_iso : bool;
   value_format_iso : bool;
+  optional_declarer_semicolon_iso : bool;
+  optional_attributes_iso : bool;
   current_rule_name : string; (* used for errors *)
   index : int list (* witness tree indices *)
 }
@@ -103,8 +108,43 @@ module XTRANS = struct
   let value_format_flag f = fun tin ->
     f (tin.extra.value_format_iso) tin
 
+  let optional_declarer_semicolon_flag f = fun tin ->
+    f (tin.extra.optional_declarer_semicolon_iso) tin
+
+  let optional_attributes_flag f = fun tin ->
+    f (tin.extra.optional_attributes_iso) tin
+
   let mode = Cocci_vs_c.TransformMode
 
+  (* ------------------------------------------------------------------------*)
+  (* Env *)
+  (* ------------------------------------------------------------------------*)
+
+  (* When env is used in + code, have to strip it more to avoid circular
+  references due to local variable information *)
+
+  let clean_env env =
+    List.map
+      (function (v,vl) ->
+	match vl with
+	| Ast_c.MetaExprVal(e,ml) ->
+	    (v,Ast_c.MetaExprVal(Lib_parsing_c.real_al_expr e,ml))
+	| Ast_c.MetaExprListVal(es) ->
+	    (v,Ast_c.MetaExprListVal(Lib_parsing_c.real_al_arguments es))
+	| Ast_c.MetaTypeVal(ty) ->
+	    (v,Ast_c.MetaTypeVal(Lib_parsing_c.real_al_type ty))
+	| Ast_c.MetaInitVal(i) ->
+	    (v,Ast_c.MetaInitVal(Lib_parsing_c.real_al_init i))
+	| Ast_c.MetaInitListVal(is) ->
+	    (v,Ast_c.MetaInitListVal(Lib_parsing_c.real_al_inits is))
+	| Ast_c.MetaDeclVal(d) ->
+	    (v,Ast_c.MetaDeclVal(Lib_parsing_c.real_al_decl d))
+	| Ast_c.MetaStmtVal(s) ->
+	    (v,Ast_c.MetaStmtVal(Lib_parsing_c.real_al_statement s))
+	| _ -> (v,vl))
+      env
+
+
   (* ------------------------------------------------------------------------*)
   (* Exp  *)
   (* ------------------------------------------------------------------------*)
@@ -123,7 +163,10 @@ module XTRANS = struct
 
   (* same as cocciExp, but for expressions in an expression, not expressions
      in a node *)
-  let cocciExpExp = fun expf expa expb -> fun tin ->
+  let cocciExpExp = fun mc expf expa expb -> fun tin ->
+    match mc with
+      Ast_cocci.MINUS _ -> Some (expa,expb) (* do nothing *)
+    | _ ->
 
     let bigf = {
       Visitor_c.default_visitor_c_s with
@@ -166,7 +209,7 @@ module XTRANS = struct
   (* ------------------------------------------------------------------------*)
    let check_pos info mck pos =
      match mck with
-     | Ast_cocci.PLUS _ -> raise Impossible
+     | Ast_cocci.PLUS _ -> raise (Impossible 51)
      | Ast_cocci.CONTEXT (Ast_cocci.FixPos (i1,i2),_)
      | Ast_cocci.MINUS   (Ast_cocci.FixPos (i1,i2),_,_,_) ->
          pos <= i2 && pos >= i1
@@ -183,6 +226,62 @@ module XTRANS = struct
 	 | None ->
 	     failwith "weird: dont have position info for the mcodekind"
 
+    (* these remove constraints, at least those that contain pcre regexps,
+       which cannot be compared (problem in the unparser) *)
+  let strip_anything anything =
+    let donothing r k e = k e in
+    let mcode mc = mc in
+    let ident r k e =
+      let e = k e in
+      match Ast_cocci.unwrap e with
+	Ast_cocci.MetaId(name,constraints,u,i) ->
+          Ast_cocci.rewrap e
+	    (Ast_cocci.MetaId(name,Ast_cocci.IdNoConstraint,u,i))
+      |  Ast_cocci.MetaFunc(name,constraints,u,i) ->
+          Ast_cocci.rewrap e
+	    (Ast_cocci.MetaFunc(name,Ast_cocci.IdNoConstraint,u,i))
+      |  Ast_cocci.MetaLocalFunc(name,constraints,u,i) ->
+          Ast_cocci.rewrap e
+	    (Ast_cocci.MetaLocalFunc(name,Ast_cocci.IdNoConstraint,u,i))
+      |  _ -> e in
+    let expression r k e =
+      let e = k e in
+      match Ast_cocci.unwrap e with
+	Ast_cocci.MetaErr(name,constraints,u,i) ->
+	  Ast_cocci.rewrap e
+	    (Ast_cocci.MetaErr(name,Ast_cocci.NoConstraint,u,i))
+      | Ast_cocci.MetaExpr(name,constraints,u,ty,form,i) ->
+          Ast_cocci.rewrap e
+	    (Ast_cocci.MetaExpr(name,Ast_cocci.NoConstraint,u,ty,form,i))
+      | _ -> e in
+    let fn = Visitor_ast.rebuilder
+	mcode mcode mcode mcode mcode mcode mcode mcode mcode
+	mcode mcode mcode mcode mcode
+	donothing donothing donothing donothing donothing
+	ident expression donothing donothing donothing donothing
+	donothing donothing donothing donothing donothing
+	donothing donothing donothing donothing donothing donothing in
+
+  fn.Visitor_ast.rebuilder_anything anything
+
+  let strip_minus_code = function
+      Ast_cocci.REPLACEMENT(l,c) ->
+	Ast_cocci.REPLACEMENT(List.map (List.map strip_anything) l,c)
+    | Ast_cocci.NOREPLACEMENT -> Ast_cocci.NOREPLACEMENT
+  let strip_context_code = function
+      Ast_cocci.BEFORE(l,c) ->
+	Ast_cocci.BEFORE(List.map (List.map strip_anything) l,c)
+    | Ast_cocci.AFTER(l,c) ->
+	Ast_cocci.AFTER(List.map (List.map strip_anything) l,c)
+    | Ast_cocci.BEFOREAFTER(l1,l2,c) ->
+	Ast_cocci.BEFOREAFTER(List.map (List.map strip_anything) l1,
+			      List.map (List.map strip_anything) l2,c)
+    | Ast_cocci.NOTHING -> Ast_cocci.NOTHING
+  let strip_mck_code = function
+      Ast_cocci.MINUS(p,l,a,repl) ->
+	Ast_cocci.MINUS(p,l,a,strip_minus_code repl)
+    | Ast_cocci.CONTEXT(p,ba) -> Ast_cocci.CONTEXT(p,strip_context_code ba)
+    | Ast_cocci.PLUS(c) -> Ast_cocci.PLUS(c)
 
   let tag_with_mck mck ib = fun tin ->
 
@@ -198,7 +297,7 @@ module XTRANS = struct
         mck
     in
     (match mck, Ast_c.pinfo_of_info ib with
-    | _,                  Ast_c.AbstractLineTok _ -> raise Impossible
+    | _,                  Ast_c.AbstractLineTok _ -> raise (Impossible 52)
     | Ast_cocci.MINUS(_), Ast_c.ExpandedTok _ ->
         failwith
 	  (Printf.sprintf
@@ -208,11 +307,15 @@ module XTRANS = struct
     | _ -> ()
     );
 
-    let many_count = function
+    let many_context_count = function
 	Ast_cocci.BEFORE(_,Ast_cocci.MANY) | Ast_cocci.AFTER(_,Ast_cocci.MANY)
       |	 Ast_cocci.BEFOREAFTER(_,_,Ast_cocci.MANY) -> true
       |	_ -> false in
 
+    let many_minus_count = function
+	Ast_cocci.REPLACEMENT(_,Ast_cocci.MANY) -> true
+      |	_ -> false in
+
     (match (oldmcode,mck) with
     | (Ast_cocci.CONTEXT(_,Ast_cocci.NOTHING),      _) ->
 	(* nothing there, so take the new stuff *)
@@ -220,34 +323,98 @@ module XTRANS = struct
 	    Ast_cocci.MINUS (pos,_,adj,any_xxs) ->
 	      Ast_cocci.MINUS (pos,inst,adj,any_xxs)
 	  | mck -> mck in
-        cocciinforef := Some (update_inst tin.extra.index mck, [tin.binding])
+	let mck = strip_mck_code (update_inst tin.extra.index mck) in
+	(* clean_env actually only needed if there is an addition
+	   not sure the extra efficiency would be worth duplicating the code *)
+        cocciinforef :=
+	  Some (mck, [clean_env tin.binding])
     | (_,   Ast_cocci.CONTEXT(_,Ast_cocci.NOTHING)) ->
 	(* can this case occur? stay with the old stuff *)
 	()
-    | (Ast_cocci.MINUS(old_pos,old_inst,old_adj,[]),
-       Ast_cocci.MINUS(new_pos,new_inst,new_adj,[]))
-	when old_pos = new_pos &&
-	  (List.mem tin.binding oldenvs or !Flag.sgrep_mode2)
+    | (Ast_cocci.MINUS(old_pos,old_inst,old_adj,Ast_cocci.NOREPLACEMENT),
+       Ast_cocci.MINUS(new_pos,new_inst,new_adj,Ast_cocci.NOREPLACEMENT))
+	(* Now put no constraints on double remove.  This is needed for
+	   tests/multidecl2: - T i; with int *i, i[12], k; (for symbol i)
+	   The positions of the two matches are different because the types
+	   end in different places (on * and then on ]) *)
+      (*when old_pos = new_pos*)
+	    (* not sure why the following condition is useful.
+	       should be ok to double remove even if the environments are
+	       different *)
+          (* &&
+	  (List.mem tin.binding oldenvs or !Flag.sgrep_mode2) *)
 	    (* no way to combine adjacency information, just drop one *)
       ->
         cocciinforef := Some
 	  (Ast_cocci.MINUS
-	     (old_pos,Common.union_set old_inst new_inst,old_adj,[]),
+	     (old_pos,Common.union_set old_inst new_inst,old_adj,
+	      Ast_cocci.NOREPLACEMENT),
 	   [tin.binding]);
         (if !Flag_matcher.show_misc
-        then pr2 "already tagged but only removed, so safe")
+        then pr2_once "already tagged but only removed, so safe")
+
+    (* ++ cases *)
+    | (Ast_cocci.MINUS(old_pos,old_inst,old_adj,old_modif),
+       Ast_cocci.MINUS(new_pos,new_inst,new_adj,new_modif))
+	when old_pos = new_pos &&
+	  old_modif = strip_minus_code new_modif &&
+	  many_minus_count old_modif ->
+
+          cocciinforef :=
+	    Some(Ast_cocci.MINUS(old_pos,Common.union_set old_inst new_inst,
+				 old_adj,old_modif),
+		 (clean_env tin.binding)::oldenvs)
 
     | (Ast_cocci.CONTEXT(old_pos,old_modif),
        Ast_cocci.CONTEXT(new_pos,new_modif))
 	when old_pos = new_pos &&
-	  old_modif = new_modif && many_count old_modif ->
+	  old_modif = strip_context_code new_modif &&
+	  many_context_count old_modif ->
 	    (* iteration only allowed on context; no way to replace something
 	       more than once; now no need for iterable; just check a flag *)
 
           cocciinforef :=
-	    Some(Ast_cocci.CONTEXT(old_pos,old_modif),tin.binding::oldenvs)
+	    Some(Ast_cocci.CONTEXT(old_pos,old_modif),
+		 (clean_env tin.binding)::oldenvs)
+
+    (* non ++ but same modif for two reasons *)
+    | (Ast_cocci.MINUS(old_pos,old_inst,old_adj,old_modif),
+       Ast_cocci.MINUS(new_pos,new_inst,new_adj,new_modif))
+	when old_pos = new_pos &&
+	  old_modif = strip_minus_code new_modif &&
+	  List.mem (clean_env tin.binding) oldenvs ->
+
+          cocciinforef :=
+	    Some(Ast_cocci.MINUS(old_pos,Common.union_set old_inst new_inst,
+				 old_adj,old_modif),
+		 oldenvs)
+
+    | (Ast_cocci.CONTEXT(old_pos,old_modif),
+       Ast_cocci.CONTEXT(new_pos,new_modif))
+	when old_pos = new_pos &&
+	  old_modif = strip_context_code new_modif &&
+	  List.mem (clean_env tin.binding) oldenvs ->
+	    (* iteration only allowed on context; no way to replace something
+	       more than once; now no need for iterable; just check a flag *)
+
+          cocciinforef := Some(Ast_cocci.CONTEXT(old_pos,old_modif),oldenvs)
 
     | _ ->
+
+(*
+	(match (oldmcode,mck) with
+	| (Ast_cocci.CONTEXT(old_pos,old_modif),
+	   Ast_cocci.CONTEXT(new_pos,new_modif)) ->
+	     Printf.printf "failed because of %b %b %b\n"
+	       (old_pos = new_pos)
+	       (old_modif = strip_context_code new_modif)
+	       (many_context_count old_modif);
+	     Printf.printf "failed because of %s %s\n"
+	       (Dumper.dump old_modif)
+	       (Dumper.dump (strip_context_code new_modif))
+	| _ -> Printf.printf "other failure\n");
+*)
+
           (* coccionly:
           if !Flag.sgrep_mode2
           then ib (* safe *)
@@ -255,7 +422,7 @@ module XTRANS = struct
           *)
              begin
             (* coccionly:
-               pad: if dont want cocci write:
+               pad: if don't want cocci write:
                 failwith
 	        (match Ast_c.pinfo_of_info ib with
 		  Ast_c.FakeTok _ -> "already tagged fake token"
@@ -335,9 +502,11 @@ module XTRANS = struct
           (fun ib ->
 	    tag_with_mck (Ast_cocci.MINUS (pos,inst,adj,any_xxs)) ib tin),
           (fun ib ->
-	    tag_with_mck (Ast_cocci.MINUS (pos,inst,adj,[])) ib tin),
+	    tag_with_mck
+	      (Ast_cocci.MINUS (pos,inst,adj,Ast_cocci.NOREPLACEMENT)) ib tin),
           (fun ib ->
-	    tag_with_mck (Ast_cocci.MINUS (pos,inst,adj,[])) ib tin),
+	    tag_with_mck
+	      (Ast_cocci.MINUS (pos,inst,adj,Ast_cocci.NOREPLACEMENT)) ib tin),
           (fun ib ->
 	    tag_with_mck (Ast_cocci.MINUS (pos,inst,adj,any_xxs)) ib tin)
         ) expr
@@ -377,7 +546,7 @@ module XTRANS = struct
             ) expr
 
         )
-    | Ast_cocci.PLUS _ -> raise Impossible
+    | Ast_cocci.PLUS _ -> raise (Impossible 53)
 
 
   (* use new strategy, collect ii, sort, recollect and tag *)
@@ -400,6 +569,12 @@ module XTRANS = struct
   let distribute_mck_expr (maxpos, minpos) = fun (lop,mop,rop,bop) -> fun x ->
     Visitor_c.vk_expr_s (mk_bigf (maxpos, minpos) (lop,mop,rop,bop)) x
 
+  let distribute_mck_assignOp (maxpos, minpos) = fun (lop,mop,rop,bop) -> fun x ->
+    Visitor_c.vk_assignOp_s (mk_bigf (maxpos, minpos) (lop,mop,rop,bop)) x
+
+  let distribute_mck_binaryOp (maxpos, minpos) = fun (lop,mop,rop,bop) -> fun x ->
+    Visitor_c.vk_binaryOp_s (mk_bigf (maxpos, minpos) (lop,mop,rop,bop)) x
+
   let distribute_mck_args (maxpos, minpos) = fun (lop,mop,rop,bop) -> fun x ->
     Visitor_c.vk_args_splitted_s (mk_bigf (maxpos, minpos) (lop,mop,rop,bop)) x
 
@@ -425,6 +600,16 @@ module XTRANS = struct
     Visitor_c.vk_params_splitted_s (mk_bigf (maxpos, minpos) (lop,mop,rop,bop))
       x
 
+  let distribute_mck_fragments (maxpos, minpos) =
+    fun (lop,mop,rop,bop) ->fun x ->
+      Visitor_c.vk_string_fragments_splitted_s
+	(mk_bigf (maxpos, minpos) (lop,mop,rop,bop))
+	x
+
+  let distribute_mck_format (maxpos, minpos) = fun (lop,mop,rop,bop) ->fun x ->
+    Visitor_c.vk_string_format_s (mk_bigf (maxpos, minpos) (lop,mop,rop,bop))
+      x
+
   let distribute_mck_node (maxpos, minpos) = fun (lop,mop,rop,bop) ->fun x ->
     Visitor_c.vk_node_s (mk_bigf (maxpos, minpos) (lop,mop,rop,bop))
       x
@@ -445,16 +630,36 @@ module XTRANS = struct
       Visitor_c.vk_cst_s (mk_bigf (maxpos, minpos) (lop,mop,rop,bop))
         x
 
-
   let distribute_mck_define_params (maxpos, minpos) = fun (lop,mop,rop,bop) ->
    fun x ->
     Visitor_c.vk_define_params_splitted_s
       (mk_bigf (maxpos, minpos) (lop,mop,rop,bop))
       x
 
+  let distribute_mck_pragmainfo (maxpos, minpos) = fun (lop,mop,rop,bop) ->
+   fun x ->
+    Visitor_c.vk_pragmainfo_s (mk_bigf (maxpos, minpos) (lop,mop,rop,bop)) x
+
+  let distribute_mck_ident_list (maxpos, minpos) = fun (lop,mop,rop,bop) ->
+   fun x ->
+    Visitor_c.vk_ident_list_splitted_s
+       (mk_bigf (maxpos, minpos) (lop,mop,rop,bop)) x
+
+  let distribute_mck_exec_code_list (maxpos, minpos) =
+    fun (lop,mop,rop,bop) -> fun x ->
+      Visitor_c.vk_exec_code_list_splitted_s
+	(mk_bigf (maxpos, minpos) (lop,mop,rop,bop))
+	x
+
+  let distribute_mck_attrs (maxpos, minpos) =
+    fun (lop,mop,rop,bop) -> fun x ->
+      Visitor_c.vk_attrs_splitted_s
+	(mk_bigf (maxpos, minpos) (lop,mop,rop,bop))
+	x
+
    let get_pos mck =
      match mck with
-     | Ast_cocci.PLUS _ -> raise Impossible
+     | Ast_cocci.PLUS _ -> raise (Impossible 54)
      | Ast_cocci.CONTEXT (Ast_cocci.FixPos (i1,i2),_)
      | Ast_cocci.MINUS   (Ast_cocci.FixPos (i1,i2),_,_,_) ->
          Ast_cocci.FixPos (i1,i2)
@@ -480,7 +685,7 @@ module XTRANS = struct
       | Ast_cocci.DontCarePos -> true
       | Ast_cocci.FixPos (i1, i2) ->
           i1 =*= min && i2 =*= max
-      | _ -> raise Impossible
+      | _ -> raise (Impossible 55)
       )
 
     then
@@ -492,6 +697,8 @@ module XTRANS = struct
 
 
   let distrf_e    = distrf (Lib_parsing_c.ii_of_expr,  distribute_mck_expr)
+  let distrf_assignOp = distrf (Lib_parsing_c.ii_of_assignOp, distribute_mck_assignOp)
+  let distrf_binaryOp = distrf (Lib_parsing_c.ii_of_binaryOp, distribute_mck_binaryOp)
   let distrf_args = distrf (Lib_parsing_c.ii_of_args,  distribute_mck_args)
   let distrf_type = distrf (Lib_parsing_c.ii_of_type,  distribute_mck_type)
   let distrf_param  = distrf (Lib_parsing_c.ii_of_param, distribute_mck_param)
@@ -501,6 +708,9 @@ module XTRANS = struct
   let distrf_decl = distrf (Lib_parsing_c.ii_of_decl,distribute_mck_decl)
   let distrf_field = distrf (Lib_parsing_c.ii_of_field,distribute_mck_field)
   let distrf_node = distrf (Lib_parsing_c.ii_of_node,distribute_mck_node)
+  let distrf_fragments =
+    distrf (Lib_parsing_c.ii_of_fragments,distribute_mck_fragments)
+  let distrf_format = distrf (Lib_parsing_c.ii_of_format,distribute_mck_format)
   let distrf_enum_fields =
     distrf (Lib_parsing_c.ii_of_enum_fields, distribute_mck_enum_fields)
   let distrf_struct_fields =
@@ -509,6 +719,14 @@ module XTRANS = struct
     distrf (Lib_parsing_c.ii_of_cst, distribute_mck_cst)
   let distrf_define_params =
     distrf (Lib_parsing_c.ii_of_define_params,distribute_mck_define_params)
+  let distrf_pragmainfo =
+    distrf (Lib_parsing_c.ii_of_pragmainfo,distribute_mck_pragmainfo)
+  let distrf_ident_list =
+    distrf (Lib_parsing_c.ii_of_ident_list,distribute_mck_ident_list)
+  let distrf_exec_code_list =
+    distrf (Lib_parsing_c.ii_of_exec_code_list,distribute_mck_exec_code_list)
+  let distrf_attrs =
+    distrf (Lib_parsing_c.ii_of_attrs,distribute_mck_attrs)
 
 
   (* ------------------------------------------------------------------------*)
@@ -575,7 +793,7 @@ module TRANS  = Cocci_vs_c.COCCI_VS_C (XTRANS)
 
 let transform_re_node a b tin =
   match TRANS.rule_elem_node a b tin with
-  | None -> raise Impossible
+  | None -> raise (Impossible 56)
   | Some (_sp, b') -> b'
 
 let (transform2: string (* rule name *) -> string list (* dropped_isos *) ->
@@ -586,6 +804,10 @@ let (transform2: string (* rule name *) -> string list (* dropped_isos *) ->
      optional_storage_iso   = not(List.mem "optional_storage" dropped_isos);
      optional_qualifier_iso = not(List.mem "optional_qualifier" dropped_isos);
      value_format_iso = not(List.mem "value_format" dropped_isos);
+     optional_declarer_semicolon_iso =
+       not(List.mem "optional_declarer_semicolon"   dropped_isos);
+     optional_attributes_iso =
+       not(List.mem "optional_attributes" dropped_isos);
      current_rule_name = rule_name;
      index = [];
    } in
@@ -597,7 +819,7 @@ let (transform2: string (* rule name *) -> string list (* dropped_isos *) ->
       let node  = acc#nodes#assoc nodei in
 
       if !Flag.show_transinfo
-      then pr2 "transform one node";
+      then pr2 (Printf.sprintf "transform one node: %d" nodei);
 
       let tin = {
         XTRANS.extra = {extra with index = index};
@@ -613,12 +835,12 @@ let (transform2: string (* rule name *) -> string list (* dropped_isos *) ->
       | F.Enter | F.Exit | F.ErrorExit
       | F.EndStatement _ | F.CaseNode _
       | F.Fake
-      | F.TrueNode | F.FalseNode | F.AfterNode | F.FallThroughNode
+      | F.TrueNode _ | F.FalseNode | F.AfterNode _ | F.FallThroughNode
           -> ()
       | _ -> () (* assert (not (node =*= node')); *)
       );
 
-      (* useless, we dont go back from flow to ast now *)
+      (* useless, we don't go back from flow to ast now *)
       (* let node' = lastfix_comma_struct node' in *)
 
       acc#replace_node (nodei, node');
