@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,12 +24,14 @@
  *)
 
 
+# 0 "./flag_cocci.ml"
 (* the inputs *)
 let show_c = ref false
 let show_cocci = ref false
 
 (* the output *)
 let show_diff = ref true
+let force_diff = ref false (*show diff even if there are only space changes*)
 
 (* the derived inputs *)
 let show_flow = ref false
@@ -49,7 +53,7 @@ let windows = ref false
 
 let popl = ref false
 
-let ifdef_to_if = ref true(*false*)
+let include_headers_for_types = ref false
 
 type include_options =
     I_UNSPECIFIED | I_NO_INCLUDES | I_NORMAL_INCLUDES
@@ -63,6 +67,14 @@ let include_path = ref ([] : string list)
 
 let relax_include_path = ref false
 
+let extra_includes = ref ([] : string list)
+
 let timeout = ref (None : int option)
 
-let worth_trying_opt = ref true
+let selected_only = ref false (* just print files that would be treated *)
+
+let use_saved_typedefs = ref true (* hack! *)
+
+(* caching of header file information *)
+let cache_threshold = 500
+let elem_threshold = 10
