@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./flag.ml"
 let sgrep_mode2 = ref false
 
 let show_misc = ref true
@@ -32,15 +35,17 @@ let show_trying = ref false
 
 let track_iso_usage = ref false
 
-type scanner = IdUtils | Glimpse | Grep | Google of string | NoScanner
-let scanner = ref Grep
+let worth_trying_opt = ref true
+
+type scanner = IdUtils | Glimpse | CocciGrep | NoScanner
+let scanner = ref NoScanner
 
 let pyoutput = ref "coccilib.output.Console"
 
-let ocamlc = ref "ocamlc"
-let ocamlopt = ref "ocamlopt"
-let ocamldep = ref "ocamldep"
-let ocamlfind = ref "ocamlfind"
+let ocamlc = ref Commands.ocamlc_cmd
+let ocamlopt = ref Commands.ocamlopt_cmd
+let ocamldep = ref Commands.ocamldep_cmd
+let ocamlfind = ref Commands.ocamlfind_cmd
 
 (*"Some" value is the path with respect to which the patch should be created*)
 let patch = ref (None : string option)
@@ -62,3 +67,9 @@ let set_defined_virtual_rules s =
   | _ -> defined_virtual_rules := s :: !defined_virtual_rules
 
 let c_plus_plus = ref false
+let ibm = ref false
+
+(* was in main *)
+let include_headers = ref false
+
+exception UnreadableFile of string
