@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,8 +24,10 @@
  *)
 
 
+# 0 "./iteration.mli"
 type init_info = (string (* language *) * string (* rule name *)) *
-      string list (* defined virtual rules *)
+      (string list (* defined virtual rules *) *
+	 (string * string) list (* defined virtual env *))
 
 val initialization_stack : init_info list ref
 
@@ -40,8 +44,10 @@ type pending_info = string list (* files to treat *) *
       (string * string) list (* virtual identifiers *)
 
 val add_pending_instance :
-    (* input is like pending_info, but with an extra option on files *)
-    (string list option * string list * (string * string) list) -> unit
+    (* input is like pending_info, but with an extra option on files and bool
+    for environment extension *)
+    (string list option * string list * (string * string) list * bool) ->
+    unit
 
 val get_pending_instance : unit -> pending_info option
 
