@@ -26,7 +26,8 @@ let ast_binding vl = function
 	Ast_c.MetaIdVal(id,_) | Ast_c.MetaFuncVal id
       | Ast_c.MetaLocalFuncVal id ->
 	  Coccilib.Str id
-
+      | Ast_c.MetaAssignOpVal op -> Coccilib.AssignOp op
+      | Ast_c.MetaBinaryOpVal op -> Coccilib.BinaryOp op
       | Ast_c.MetaExprVal(expr,_) -> Coccilib.Expr expr
       | Ast_c.MetaExprListVal arglist -> Coccilib.ExprList arglist
       | Ast_c.MetaParamVal param -> Coccilib.Param param
@@ -34,10 +35,13 @@ let ast_binding vl = function
 
       | Ast_c.MetaTypeVal ty -> Coccilib.Type ty
       | Ast_c.MetaInitVal init -> Coccilib.Init init
+      | Ast_c.MetaInitListVal init -> Coccilib.InitList init
       | Ast_c.MetaDeclVal decl -> Coccilib.Decl decl
       | Ast_c.MetaFieldVal field -> Coccilib.Field field
       | Ast_c.MetaFieldListVal field -> Coccilib.FieldList field
       | Ast_c.MetaStmtVal stm -> Coccilib.Stmt stm
+      | Ast_c.MetaFragListVal frags -> Coccilib.FragList frags
+      | Ast_c.MetaFmtVal fmt -> Coccilib.Fmt fmt
 
       | Ast_c.MetaPosVal _ | Ast_c.MetaPosValList _ | Ast_c.MetaListlenVal _ ->
 	  failwith "not associated with a declared metavariable"]
@@ -65,6 +69,7 @@ let run mv ve script_vars name code =
 
   (* call the function *)
   Coccilib.include_match true;
+  Coccilib.exited := false;
   let fn =
     try Hashtbl.find Coccilib.fcts name
     with Not_found -> failwith (Printf.sprintf "%s not found" name) in
