@@ -43,7 +43,7 @@ open Common
  *
  * convention: I often use 'ii' for the name of a list of info.
  *
- * Sometimes we want to add someting at the beginning or at the end
+ * Sometimes we want to add something at the beginning or at the end
  * of a construct. For 'function' and 'decl' we want to add something
  * to their left and for 'if' 'while' et 'for' and so on at their right.
  * We want some kinds of "virtual placeholders" that represent the start or
@@ -90,6 +90,10 @@ type parse_info =
   | AbstractLineTok of Common.parse_info (* local to the abstracted thing *)
  (* with sexp *)
 
+(* for tokens that are shared in the C ast, and thus require care
+when transforming *)
+type danger = DangerStart | DangerEnd | Danger | NoDanger
+
 type info = {
   pinfo : parse_info;
 
@@ -100,6 +104,11 @@ type info = {
   (* set in comment_annotater_c.ml *)
   comments_tag: comments_around ref;
 
+  (* annotations on the token (mutable) *)
+  mutable annots_tag: Token_annot.annots;
+
+  danger: danger ref;
+
   (* todo? token_info : sometimes useful to know what token it was *)
   }
 and il = info list
@@ -125,12 +134,12 @@ and 'a wrap3 = 'a * il (* * evotype*)
  * as concatenated strings can be used not only for identifiers and for
  * declarators, but also for fields, for labels, etc.
  *
- * Note: because now the info is embeded in the name, the info for
+ * Note: because now the info is embedded in the name, the info for
  * expression like Ident, or types like Typename, are not anymore
  * stored in the expression or type. Hence if you assume this,
  * which was true before, you are now wrong. So never write code like
  * let (unwrape,_), ii = e  and use 'ii' believing it contains
- * the local ii to e. If you want to do that, use the appropiate
+ * the local ii to e. If you want to do that, use the appropriate
  * wrapper get_local_ii_of_expr_inlining_ii_of_name.
  *)
 and name =
@@ -154,9 +163,9 @@ and name =
  * from scratch, because many stuff are just sugar.
  *
  * invariant: Array and FunctionType have also typeQualifier but they
- * dont have sense. I put this to factorise some code. If you look in
+ * don't have sense. I put this to factorise some code. If you look in
  * the grammar, you see that we can never specify const for the array
- * himself (but we can do it for pointer) or function, we always
+ * itself (but we can do it for pointer) or function, we always
  * have in the action rule of the grammar a { (nQ, FunctionType ...) }.
  *
  *
@@ -172,11 +181,12 @@ and fullType = typeQualifier * typeC
  and typeC = typeCbis wrap (* todo reput wrap3 *)
 
   and typeCbis =
-    NoType (* for c++ only *)
+    NoType (* for c++ only, and for K&R C *)
   | BaseType        of baseType
 
   | Pointer         of fullType
   | Array           of constExpression option * fullType
+  | Decimal         of constExpression * constExpression option
   | FunctionType    of functionType
 
   | Enum            of string option * enumType
@@ -247,7 +257,7 @@ and fullType = typeQualifier * typeC
 
         (* before unparser, I didn't have a FieldDeclList but just a Field. *)
          and field_declaration  =
-           | FieldDeclList of fieldkind wrap2 list (* , *) wrap  (* ; *)
+           | FieldDeclList of fieldkind wrap2 list (* , *) wrap (*; fakestart*)
 
           (* At first I thought that a bitfield could be only Signed/Unsigned.
            * But it seems that gcc allow char i:4. C rule must say that you
@@ -291,7 +301,8 @@ and attribute = attributebis wrap
 and expression = (expressionbis * exp_info ref (* semantic: *)) wrap3
  and exp_info = exp_type option * test
   and exp_type = fullType (* Type_c.completed_and_simplified *) * local
-    and local = LocalVar of parse_info | NotLocalVar (* cocci: *)
+    and local = LocalVar of parse_info | StaticLocalVar of parse_info
+              | NotLocalVar (* cocci: *)
   and test = Test | NotTest (* cocci: *)
 
  and expressionbis =
@@ -302,6 +313,7 @@ and expression = (expressionbis * exp_info ref (* semantic: *)) wrap3
   | Ident          of name (* todo? more semantic info such as LocalFunc *)
 
   | Constant       of constant
+  | StringConstant of string_fragment list * string (*src string*) * isWchar
   | FunCall        of expression * argument wrap2 (* , *) list
   (* gccext: x ? /* empty */ : y <=> x ? x : y;  hence the 'option' below *)
   | CondExpr       of expression * expression option * expression
@@ -330,13 +342,13 @@ and expression = (expressionbis * exp_info ref (* semantic: *)) wrap3
 
   (* gccext: *)
   | StatementExpr of compound wrap (* ( )     new scope *)
-  | Constructor  of fullType * initialiser wrap2 (* , *) list
+  | Constructor  of fullType * initialiser
 
   (* for unparser: *)
   | ParenExpr of expression
 
   (* for C++: *)
-  | New of argument
+  | New of (argument wrap2 (* , *) list) option * argument
   | Delete of expression
 
   (* cppext: IfdefExpr TODO *)
@@ -365,21 +377,24 @@ and expression = (expressionbis * exp_info ref (* semantic: *)) wrap3
     | Char   of (string * isWchar) (* normally it is equivalent to Int *)
     | Int    of (string * intType)
     | Float  of (string * floatType)
+    | DecimalConst of (string * string * string)
 
     and isWchar = IsWchar | IsChar
 
 
   and unaryOp  = GetRef | DeRef | UnPlus |  UnMinus | Tilde | Not
                  | GetRefLabel (* gccext: GetRefLabel, via &&label notation *)
-  and assignOp = SimpleAssign | OpAssign of arithOp
+  and assignOpbis = SimpleAssign | OpAssign of arithOp
+  and assignOp = assignOpbis wrap
   and fixOp    = Dec | Inc
 
-  and binaryOp = Arith of arithOp | Logical of logicalOp
+  and binaryOpbis = Arith of arithOp | Logical of logicalOp
+  and binaryOp = binaryOpbis wrap
 
        and arithOp   =
          | Plus | Minus | Mul | Div | Mod
          | DecLeft | DecRight
-         | And | Or | Xor
+         | And | Or | Xor | Max | Min
 
        and logicalOp =
          | Inf | Sup | InfEq | SupEq
@@ -388,10 +403,21 @@ and expression = (expressionbis * exp_info ref (* semantic: *)) wrap3
 
  and constExpression = expression (* => int *)
 
+and string_fragment = string_fragment_bis wrap
+
+and string_fragment_bis =
+  ConstantFragment of string
+| FormatFragment of string_format (* format *)
+
+and string_format = string_format_bis wrap
+
+and string_format_bis =
+  ConstantFormat of string
+
 (* ------------------------------------------------------------------------- *)
 (* C statement *)
 (* ------------------------------------------------------------------------- *)
-(* note: that assignement is not a statement but an expression;
+(* note: that assignment is not a statement but an expression;
  * wonderful C langage.
  *
  * note: I use 'and' for type definition cos gccext allow statement as
@@ -417,13 +443,12 @@ and statement = statementbis wrap3
 
   (* cppext: *)
   | MacroStmt
-
-
+  | Exec of exec_code list
 
   and labeled = Label   of name * statement
               | Case    of expression * statement
               | CaseRange of expression * expression * statement (* gccext: *)
-	      |	Default of statement
+              |	Default of statement
 
   (* cppext:
    * old: compound = (declaration list * statement list)
@@ -465,18 +490,24 @@ and statement = statementbis wrap3
 
   and exprStatement = expression option
 
+  and declOrExpr = ForDecl of declaration | ForExp of expression option wrap
+
  (* for Switch, need check that all elements in the compound start
   * with a case:, otherwise unreachable code.
   *)
   and selection     =
    | If     of expression * statement * statement
    | Switch of expression * statement
+   (* #ifdef A if e S1 else #endif S2 *)
+   | Ifdef_Ite of expression * statement * statement
+   (* #ifdef A if e S1 else #else S2 #endif S3 *)
+   | Ifdef_Ite2 of expression * statement * statement * statement
 
 
   and iteration     =
     | While   of expression * statement
     | DoWhile of statement * expression
-    | For     of exprStatement wrap * exprStatement wrap * exprStatement wrap *
+    | For     of declOrExpr * exprStatement wrap * exprStatement wrap *
                  statement
     (* cppext: *)
     | MacroIteration of string * argument wrap2 list * statement
@@ -493,6 +524,11 @@ and statement = statementbis wrap3
       and colon_option = colon_option_bis wrap
           and colon_option_bis = ColonMisc | ColonExpr of expression
 
+  and exec_code_bis =
+    ExecEval of expression
+  | ExecToken
+
+  and exec_code = exec_code_bis wrap
 
 (* ------------------------------------------------------------------------- *)
 (* Declaration *)
@@ -514,7 +550,10 @@ and statement = statementbis wrap3
 and declaration =
   | DeclList of onedecl wrap2 (* , *) list wrap (* ; fakestart sto *)
   (* cppext: *)
-  | MacroDecl of (string * argument wrap2 list) wrap (* fakestart *)
+    (* bool is true if there is a ; at the end *)
+  | MacroDecl of (string * argument wrap2 list * bool) wrap (* fakestart *)
+  | MacroDeclInit of
+      (string * argument wrap2 list * initialiser) wrap (* fakestart *)
 
      and onedecl =
        { v_namei: (name * v_init) option;
@@ -538,7 +577,7 @@ and declaration =
 
      (* fullType is the type used if the type should be converted to
 	an assignment.  It can be adjusted in the type annotation
-	phase when typedef information is availalble *)
+	phase when typedef information is available *)
      and initialiser = initialiserbis wrap
        and initialiserbis =
           | InitExpr of expression
@@ -585,7 +624,8 @@ and definition = definitionbis wrap (* ( ) { } fakestart sto *)
 and cpp_directive =
   | Define of define
   | Include of includ
-  | PragmaAndCo of il
+  | Pragma of string wrap * pragmainfo
+  | OtherDirective of il
 (*| Ifdef ? no, ifdefs are handled differently, cf ifdef_directive below *)
 
 and define = string wrap (* #define s eol *) * (define_kind * define_val)
@@ -604,15 +644,13 @@ and define = string wrap (* #define s eol *) * (define_kind * define_val)
      | DefineFunction of definition
      | DefineInit of initialiser (* in practice only { } with possible ',' *)
 
-     (* TODO DefineMulti of define_val list *)
+     | DefineMulti of statement list
 
      | DefineText of string wrap
      | DefineEmpty
 
      | DefineTodo
 
-
-
 and includ =
   { i_include: inc_file wrap; (* #include s *)
     (* cocci: computed in ? *)
@@ -622,11 +660,11 @@ and includ =
     (* cf cpp_ast_c.ml. set to None at parsing time. *)
     i_content: (Common.filename (* full path *) * program) option;
   }
- and inc_file =
+and inc_file =
   | Local    of inc_elem list
   | NonLocal of inc_elem list
   | Weird of string (* ex: #include SYSTEM_H *)
-  and inc_elem = string
+and inc_elem = string
 
  (* cocci: to tag the first of #include <xx/> and last of #include <yy/>
   *
@@ -637,21 +675,48 @@ and includ =
   *
   * This is set after parsing, in cocci.ml, in update_rel_pos.
   *)
- and include_rel_pos = {
+and include_rel_pos = {
    first_of : string list list;
    last_of :  string list list;
  }
 
-
+and pragmainfo =
+    PragmaTuple of argument wrap2 (* , *) list wrap
+  | PragmaIdList of name wrap2 list (* no commas, wrap2 is always empty *)
 
 (* todo? to specialize if someone need more info *)
 and ifdef_directive = (* or and 'a ifdefed = 'a list wrap *)
   | IfdefDirective of (ifdefkind * matching_tag) wrap
   and ifdefkind =
-    | Ifdef (* todo? of string ? of formula_cpp ? *)
-    | IfdefElseif (* same *)
-    | IfdefElse (* same *)
+    | Ifdef of ifdef_guard
+    | IfdefElseif of ifdef_guard
+    | IfdefElse
     | IfdefEndif
+    (** Guards for #if conditionals
+     *
+     * Having #if guards in the AST is useful for cpp-aware analyses, or to
+     * later add support for matching against #ifS.
+     *
+     * General #if guards are stored as a string in a [Gif_str] constructor.
+     * A traversal of the syntax tree with a parsing function would transform
+     * these into the parsed [Gif] form.
+     *
+     * NOTE that there is no actually guaranteee that a traversal will
+     * eliminate all [Gif_str] constructors, since the parsing function may
+     * fail.
+     *
+     * See [Parsing #if guards] to know why this design choice.
+     *
+     * @author Iago Abal
+     *)
+  and ifdef_guard = Gifdef  of macro_symbol (* #ifdef *)
+                  | Gifndef of macro_symbol (* #ifndef *)
+                  | Gif_str of string       (* #if <string to be parsed> *)
+                  | Gif     of expression   (* #if *)
+                  | Gnone   (* ignored #if condition: TIfdefBool,
+                             * TIfdefMisc, and TIfdefVersion
+                             *)
+  and macro_symbol = string
   (* set in Parsing_hacks.set_ifdef_parenthize_info. It internally use
    * a global so it means if you parse the same file twice you may get
    * different id. I try now to avoid this pb by resetting it each
@@ -661,7 +726,33 @@ and ifdef_directive = (* or and 'a ifdefed = 'a list wrap *)
     IfdefTag of (int (* tag *) * int (* total with this tag *))
 
 
-
+(* Note [Parsing #if guards]
+ *
+ * What I wanted to do:
+ * The lexer should tokenize the #if guard, we use the TDefEOL-trick to
+ * mark the end of the guard, and finally we add a rule to the ocamlyacc
+ * parser.
+ *
+ * Problem:
+ * The [lookahead] pass in [Parsing_hacks] assumes that an #if header is
+ * a single token. The above solution breaks that assumption. So, when an
+ * #if appears in some weird position, [lookahead] will comment out the #if
+ * token, but not any subsequent token of the guard. It seems doable to
+ * modify [lookahead] to handle this new situation, but this seems a complex
+ * and delicate function to touch, and (who knows) we may break something
+ * else. Also note that [TCommentCpp] only takes one [info], so we would
+ * need to combine all the [info]s of an #if into one.
+ *
+ * What I finally did:
+ * The safest way I found is to save the #if guard in the [TIfdef] token as
+ * a (yet-to-be-parsed) string. This is enough to reason about, or match
+ * against, simple and most-common #ifdef and #ifndef conditionals. (Rough
+ * estimate: 80% of #if conditionals in Linux are #if[n]def.) But the
+ * AST for #if guards can still be easily obtained by traversing the syntax
+ * tree with a parsing function.
+ *
+ * @author Iago Abal
+ *)
 
 
 (* ------------------------------------------------------------------------- *)
@@ -683,6 +774,9 @@ and toplevel =
 
   | FinalDef of info (* EOF *)
 
+  (* c++ *)
+  | Namespace of toplevel list * il
+
 (* ------------------------------------------------------------------------- *)
 and program = toplevel list
 
@@ -709,11 +803,15 @@ and metavars_binding = (Ast_cocci.meta_name, metavar_binding_kind) assoc
 
   | MetaTypeVal      of fullType
   | MetaInitVal      of initialiser
+  | MetaInitListVal  of initialiser wrap2 list
   | MetaDeclVal      of declaration
   | MetaFieldVal     of field
   | MetaFieldListVal of field list
   | MetaStmtVal      of statement
-
+  | MetaFmtVal       of string_format
+  | MetaFragListVal  of string_fragment list
+  | MetaAssignOpVal  of assignOp
+  | MetaBinaryOpVal  of binaryOp
   (* Could also be in Lib_engine.metavars_binding2 with the ParenVal,
    * because don't need to have the value for a position in the env of
    * a '+'. But ParenVal or LabelVal are used only by CTL, they are not
@@ -813,14 +911,16 @@ let noInIfdef () =
 (* When want add some info in ast that does not correspond to
  * an existing C element.
  * old: or when don't want 'synchronize' on it in unparse_c.ml
- * (now have other mark for tha matter).
+ * (now have other mark for the matter).
  *)
 let no_virt_pos = ({str="";charpos=0;line=0;column=0;file=""},-1)
 
 let fakeInfo pi  =
   { pinfo = FakeTok ("",no_virt_pos);
     cocci_tag = ref emptyAnnot;
+    annots_tag = Token_annot.empty;
     comments_tag = ref emptyComments;
+    danger = ref NoDanger;
   }
 
 let noii = []
@@ -982,6 +1082,11 @@ let compare_pos ii1 ii2 =
 let equal_posl (l1,c1) (l2,c2) =
   (l1 =|= l2) && (c1 =|= c2)
 
+let compare_posl (l1,c1) (l2,c2) =
+  match l2 - l1 with
+    0 -> c2 - c1
+  | r -> r
+
 let info_to_fixpos ii =
   match pinfo_of_info ii with
     OriginTok pi -> Ast_cocci.Real pi.Common.charpos
@@ -1020,13 +1125,16 @@ let al_info tokenindex x =
 	 file = "";
 	 str = str_of_info x});
     cocci_tag = ref emptyAnnot;
+    annots_tag = Token_annot.empty;
     comments_tag = ref emptyComments;
+    danger = ref NoDanger;
   }
 
 let semi_al_info x =
   { x with
     cocci_tag = ref emptyAnnot;
     comments_tag = ref emptyComments;
+    danger = ref NoDanger;
   }
 
 let magic_real_number = -10
@@ -1040,7 +1148,9 @@ let real_al_info x =
 	 file = "";
 	 str = str_of_info x});
     cocci_tag = ref emptyAnnot;
+    annots_tag = Token_annot.empty;
     comments_tag = ref emptyComments;
+    danger = ref NoDanger;
   }
 
 let al_comments x =
@@ -1049,6 +1159,7 @@ let al_comments x =
   let al_com (x,i) =
     (x,{i with Common.charpos = magic_real_number;
 	 Common.line = magic_real_number;
+	 Common.file = "";
 	 Common.column = magic_real_number}) in
   {mbefore = []; (* duplicates mafter of the previous token *)
    mafter = List.map al_com (keep_cpp x.mafter);
@@ -1065,13 +1176,17 @@ let al_info_cpp tokenindex x =
 	 file = "";
 	 str = str_of_info x});
     cocci_tag = ref emptyAnnot;
+    annots_tag = Token_annot.empty;
     comments_tag = ref (al_comments !(x.comments_tag));
+    danger = ref NoDanger;
   }
 
 let semi_al_info_cpp x =
   { x with
     cocci_tag = ref emptyAnnot;
+    annots_tag = Token_annot.empty;
     comments_tag = ref (al_comments !(x.comments_tag));
+    danger = ref NoDanger;
   }
 
 let real_al_info_cpp x =
@@ -1083,7 +1198,9 @@ let real_al_info_cpp x =
 	 file = "";
 	 str = str_of_info x});
     cocci_tag = ref emptyAnnot;
+    annots_tag = Token_annot.empty;
     comments_tag =  ref (al_comments !(x.comments_tag));
+    danger = ref NoDanger;
   }
 
 
@@ -1096,7 +1213,7 @@ let real_al_info_cpp x =
  * a list where the comma are on their own. f(1,2,2) was
  * [(1,[]); (2,[,]); (2,[,])] and become [1;',';2;',';2].
  *
- * Used in cocci_vs_c.ml, to have a more direct correspondance between
+ * Used in cocci_vs_c.ml, to have a more direct correspondence between
  * the ast_cocci of julia and ast_c.
  *)
 let rec (split_comma: 'a wrap2 list -> ('a, il) either list) =
@@ -1116,10 +1233,15 @@ let rec (unsplit_comma: ('a, il) either list -> 'a wrap2 list) =
       let empty_ii = [] in
       (e, empty_ii)::unsplit_comma xs
   | Right ii::_ ->
-      raise Impossible
-
+      raise (Impossible 59)
 
+let (split_nocomma: 'a list -> ('a, il) either list) = function l ->
+  List.map (function x -> Left x) l
 
+let (unsplit_nocomma: ('a, il) either list -> 'a list) = function l ->
+  l +>
+  List.map
+    (function Left x -> x | Right x -> failwith "not possible")
 
 (*****************************************************************************)
 (* Helpers, could also be put in lib_parsing_c.ml instead *)
@@ -1172,7 +1294,7 @@ let get_s_and_ii_of_name name =
       s, [iis]
   | CppConcatenatedName xs ->
       (match xs with
-      | [] -> raise Impossible
+      | [] -> raise (Impossible 60)
       | ((s,iis),noiiop)::xs ->
           s, iis
       )
@@ -1213,8 +1335,8 @@ let info_of_type ft =
   (* bugfix: because of string->name, the ii can be deeper *)
   let ii = get_local_ii_of_tybis_inlining_ii_of_name ty in
   match ii with
-  | ii::_ -> ii.pinfo
-  | [] -> failwith "type has no text; need to think again"
+  | ii::_ -> Some ii.pinfo
+  | [] -> None
 
 (* only Label and Goto have name *)
 let get_local_ii_of_st_inlining_ii_of_name st =
@@ -1231,3 +1353,19 @@ let get_local_ii_of_st_inlining_ii_of_name st =
 let name_of_parameter param =
   param.p_namei +> Common.map_option (str_of_name)
 
+
+(* ------------------------------------------------------------------------- *)
+(* Annotations on tokens *)
+(* ------------------------------------------------------------------------- *)
+
+(* to put a given annotation on a token *)
+let put_annot_info info key value =
+  info.annots_tag <- Token_annot.put_annot key value info.annots_tag
+
+(* to check if an annotation has such a token *)
+let get_annot_info info key =
+  Token_annot.get_annot info.annots_tag key
+
+let get_comments_before info = (!(info.comments_tag)).mbefore
+let get_comments_after info = (!(info.comments_tag)).mafter
+
