@@ -71,16 +71,27 @@ let normal_form_program xs =
           k (InitList xs, [i1;i2])
       | _ -> k ini
     );
+
     Visitor_c.kexpr_s = (fun (k,bigf) e ->
       match e with
       (* todo: should also do something for multistrings *)
-      | (Constant (String (s,kind)), typ), [ii]
+      |	 (Constant (String (s,kind)), typ), [ii]
           when Common.string_match_substring cvs_keyword_regexp s ->
           let newstr = cvs_compute_newstr s in
           (Constant (String (newstr,kind)), typ), [rewrap_str newstr ii]
-      | _ -> k e
-
+      |	 _ -> k e
     );
+
+    Visitor_c.kfragment_s = (fun (k,bigf) e ->
+      match e with
+      (* todo: should also do something for multistrings *)
+      | (ConstantFragment s), [ii]
+          when Common.string_match_substring cvs_keyword_regexp s ->
+          let newstr = cvs_compute_newstr s in
+	  (ConstantFragment newstr), [rewrap_str newstr ii]
+      | _ -> k e
+    );
+
     Visitor_c.ktoplevel_s = (fun (k,bigf) p ->
       match p with
       | CppTop (Define _) ->
@@ -151,78 +162,10 @@ let normal_form_token adjust_cvs x =
  *
  * todo: do iso between if() S and  if() { S }
  *)
-let compare_ast filename1 filename2  =
-
-  let xs =
-    match !Flag_parsing_c.diff_lines with
-      None ->
-	Common.cmd_to_list ("diff -u -b -B "^filename1^ " "  ^ filename2)
-    | Some n ->
-	Common.cmd_to_list ("diff -U "^n^" -b -B "^filename1^" "^filename2) in
-
-  (* get rid of the --- and +++ lines *)
-  let xs =
-    if null xs
-    then xs
-    else Common.drop 2 xs
-  in
-
-
-  let process_filename filename =
-    let (c, _stat) = Parse_c.parse_c_and_cpp filename in
-    let c = List.map fst c in
-    c +> Lib_parsing_c.al_program +> normal_form_program
-  in
-
-  let c1 = process_filename filename1 in
-  let c2 = process_filename filename2 in
-
-  let error = ref 0 in
-  let pb_notparsed = ref 0 in
-
-  let res =
-    if List.length c1 <> List.length c2
-    then Pb "not same number of entities (func, decl, ...)"
-    else
-      begin
-        zip c1 c2 +> List.iter (function
-        | Declaration a, Declaration b -> if not (a =*= b) then incr error
-        | Definition a, Definition b ->   if not (a =*= b) then incr error
-        | EmptyDef a, EmptyDef b ->       if not (a =*= b) then incr error
-        | MacroTop (a1,b1,c1), MacroTop (a2,b2,c2) ->
-            if not ((a1,b1,c1) =*= (a2,b2,c2)) then incr error
-        | CppTop (Include {i_include = a}), CppTop (Include {i_include = b}) ->
-            if not (a =*= b) then incr error
-        | CppTop Define _, CppTop Define _ ->
-            raise Todo
-            (* if not (a =*= b) then incr error *)
-        | NotParsedCorrectly a, NotParsedCorrectly b ->
-            if not (a =*= b) then incr pb_notparsed
-        | NotParsedCorrectly a, _ ->
-            (* Pb only in generated file *)
-            incr error;
-
-        | _, NotParsedCorrectly b ->
-            incr pb_notparsed
-        | FinalDef a, FinalDef b -> if not (a =*= b) then incr error
-
-        | IfdefTop a, IfdefTop b -> if not (a =*= b) then incr error
-
-        | (FinalDef _|EmptyDef _|
-           MacroTop (_, _, _)|IfdefTop _|
-           CppTop _|Definition _|Declaration _), _ -> incr error
-
-        );
-        (match () with
-        | _ when !pb_notparsed > 0 && !error =|= 0 ->
-            PbOnlyInNotParsedCorrectly ""
-        | _ when !error > 0 -> Pb ""
-        | _ -> Correct
-        )
-      end
-  in
-  res, xs
-
+(* I have removed this function as it was not used anywhere.
+ *     let compare_ast filename1 filename2  = ...
+ * /Iago
+ *)
 
 
 (*****************************************************************************)
@@ -234,7 +177,7 @@ let compare_ast filename1 filename2  =
  * Here I compare token, and so have still the TCommentCpp and TCommentMisc
  * so at least detect such differences.
  *
- * Morover compare_ast is not very precise in his report when it
+ * Moreover compare_ast is not very precise in his report when it
  * detects a difference. So token_diff is better.
  *
  * I do token_diff but I use programCelement2, so that
@@ -248,22 +191,34 @@ let compare_ast filename1 filename2  =
  *)
 
 
-(* Pass only "true" comments, dont pass TCommentMisc and TCommentCpp *)
-let is_normal_space_or_comment = function
-  | Parser_c.TComment _
+(* Pass only "true" comments, don't pass TCommentMisc and TCommentCpp *)
+let is_normal_space_or_comment to_expected = function
   | Parser_c.TCommentSpace _
   | Parser_c.TCommentNewline _
 
 (*  | Parser_c.TComma _ *) (* UGLY, because of gcc_opt_comma isomorphism  *)
       -> true
+  | Parser_c.TComment _ -> to_expected (* only ignore in compare to expected *)
   | _ -> false
 
+let get_diff filename1 filename2 bs =
+  let com =
+    match !Flag_parsing_c.diff_lines with
+      None -> Printf.sprintf "diff -u %s %s %s" bs filename1 filename2
+    | Some n ->
+	Printf.sprintf "diff -U %s %s %s %s" n bs filename1 filename2 in
+  let xs = Common.cmd_to_list com in
 
-(* convetion: compare_token generated_file  expected_res
+  (* get rid of the --- and +++ lines *)
+  if null xs
+  then xs
+  else Common.drop 2 xs
+
+(* convention: compare_token generated_file  expected_res
  * because when there is a notparsablezone in generated_file, I
  * don't issue a PbOnlyInNotParsedCorrectly
  *)
-let do_compare_token adjust_cvs filename1 filename2 =
+let do_compare_token adjust_cvs to_expected filename1 filename2 =
 
   let rec loop xs ys =
     match xs, ys with
@@ -298,8 +253,10 @@ let do_compare_token adjust_cvs filename1 filename2 =
   in
   let final_loop xs ys =
     loop
-      (xs +> List.filter (fun x -> not (is_normal_space_or_comment x)))
-      (ys +> List.filter (fun x -> not (is_normal_space_or_comment x)))
+      (xs +>
+       List.filter (fun x -> not (is_normal_space_or_comment to_expected x)))
+      (ys +>
+       List.filter (fun x -> not (is_normal_space_or_comment to_expected x)))
   in
 
   (*
@@ -308,8 +265,8 @@ let do_compare_token adjust_cvs filename1 filename2 =
     loop toks1 toks2 in
   *)
 
-  let (c1, _stat) = Parse_c.parse_c_and_cpp filename1 in
-  let (c2, _stat) = Parse_c.parse_c_and_cpp filename2 in
+  let (c1, _stat) = Parse_c.parse_c_and_cpp false filename1 in
+  let (c2, _stat) = Parse_c.parse_c_and_cpp false filename2 in
 
   let res =
     if List.length c1 <> List.length c2
@@ -336,19 +293,7 @@ let do_compare_token adjust_cvs filename1 filename2 =
           (Correct)
   in
 
-  let xs =
-    match !Flag_parsing_c.diff_lines with
-      None ->
-	Common.cmd_to_list ("diff -u -b -B "^filename1^ " "  ^ filename2)
-    | Some n ->
-	Common.cmd_to_list ("diff -U "^n^" -b -B "^filename1^" "^filename2) in
-
-  (* get rid of the --- and +++ lines *)
-  let xs =
-    if null xs
-    then xs
-    else Common.drop 2 xs
-  in
+  let xs = get_diff filename1 filename2 "-b -B" in
 
   if null xs && (res <> Correct)
   then failwith
@@ -356,17 +301,21 @@ let do_compare_token adjust_cvs filename1 filename2 =
 
   res, xs
 
-let compare_token = do_compare_token true
+let compare_token = do_compare_token true true
 
 
 (*****************************************************************************)
 
 (* compare to a res file *)
-let compare_default = do_compare_token true
+let compare_default = do_compare_token true true
 
 (* compare to the source of the transformation *)
-let compare_to_original = do_compare_token false
+let compare_to_original = do_compare_token false false
 
+let exact_compare file1 file2 =
+  match get_diff file1 file2 "" with
+    [] -> (Correct, [])
+  | res -> (Pb "files differ", res)
 
 let compare_result_to_string (correct, diffxs) =
   match correct with
