@@ -74,7 +74,7 @@ open Ast_c
  * choice: Julia proposed that the flow is in fact just
  * a view through the Ast, which means just Ocaml ref, so that when we
  * modify some nodes, in fact it modifies the ast. But I prefer do it
- * the functionnal way.
+ * the functional way.
  *
  * The node2 type should be as close as possible to Ast_cocci.rule_elem to
  * facilitate the job of cocci_vs_c.
@@ -155,7 +155,7 @@ type node = node1 * string
   | DoHeader of fullstatement * info
   | DoWhileTail of expression wrap
   | ForHeader of fullstatement *
-                 (exprStatement wrap * exprStatement wrap * exprStatement wrap)
+                 (declOrExpr * exprStatement wrap * exprStatement wrap)
                  wrap
   | SwitchHeader of fullstatement * expression wrap
   | MacroIterHeader of fullstatement * (string * argument wrap2 list) wrap
@@ -176,7 +176,7 @@ type node = node1 * string
    * of an if (maybe a '}', maybe a ';')
    *
    * The problem is that this stuff should be in transformation.ml,
-   * but need information available in flow_to_ast, but we dont want
+   * but need information available in flow_to_ast, but we don't want
    * to polluate both files.
    *
    * So the choices are
@@ -208,6 +208,9 @@ type node = node1 * string
   | IfdefEndif of ifdef_directive
 
 
+  (* ------------------------ *)
+  | IfdefIteHeader of il
+
   (* ------------------------ *)
   | DefineHeader of string wrap * define_kind
 
@@ -218,6 +221,8 @@ type node = node1 * string
 
   | Include of includ
 
+  | PragmaHeader of string wrap * pragmainfo
+
   (* obsolete? *)
   | MacroTop of string * argument wrap2 list * il
 
@@ -226,7 +231,7 @@ type node = node1 * string
   | Default of fullstatement * unit wrap
 
   | Continue of fullstatement * unit wrap
-  | Break    of fullstatement * unit wrap
+  | Break    of fullstatement * unit wrap * bool (* true if from switch *)
 
   (* no counter part in cocci *)
   | CaseRange of fullstatement * (expression * expression) wrap
@@ -237,6 +242,8 @@ type node = node1 * string
   | Asm of fullstatement * asmbody wrap
   | MacroStmt of fullstatement * unit wrap
 
+  | Exec of fullstatement * exec_code list wrap
+
   (* ------------------------ *)
   (* some control nodes *)
   | Enter
@@ -258,16 +265,24 @@ type node = node1 * string
 
   (* ------------------------ *)
   (* for ctl:  *)
-  | TrueNode
+  | TrueNode of bool ref
   | FalseNode
   | InLoopNode (* almost equivalent to TrueNode but just for loops *)
 
-  | AfterNode
+  | AfterNode of after_type
   | FallThroughNode
   | LoopFallThroughNode
 
   | ErrorExit
 
+and after_type =
+  | RetAfterNode (* after for a block ending in return *)
+  | GotoAfterNode (* after for a block ending in goto *)
+  | BreakAfterNode (* after for a block ending in break *)
+  | ContAfterNode (* after for a block ending in continue *)
+  | SWBreakAfterNode (* after for a block ending in break from switch *)
+  | NormalAfterNode
+
 type edge = Direct (* Normal | Shadow *)
 
 type cflow = (node, edge) Ograph_extended.ograph_mutable
@@ -342,13 +357,15 @@ let extract_fullstatement node =
 
   | Include _
   | DefineHeader _ | DefineType _ | DefineExpr  _ | DefineDoWhileZeroHeader _
-  | DefineTodo
-  | MacroTop _
+  | DefineTodo | PragmaHeader _ | MacroTop _
       -> None
 
   | IfdefHeader _ | IfdefElse _ | IfdefEndif _
       -> None
 
+  | IfdefIteHeader _
+      -> None
+
   | SeqStart (st,_,_)
   | ExprStatement (st, _)
   | IfHeader  (st, _)
@@ -365,8 +382,9 @@ let extract_fullstatement node =
   | Default   (st, _)
   | Goto (st, _, _)
   | Continue (st, _)
-  | Break    (st, _)
+  | Break    (st, _, _)
   | Asm (st,_)
+  | Exec (st,_)
       -> Some st
 
   | TopNode|EndNode
@@ -379,10 +397,10 @@ let extract_fullstatement node =
   | Exit
   | Fake
   | CaseNode _
-  | TrueNode
+  | TrueNode _
   | FalseNode
   | InLoopNode
-  | AfterNode
+  | AfterNode _
   | FallThroughNode
   | LoopFallThroughNode
   | ErrorExit
