@@ -73,11 +73,15 @@ let al_field     x = Visitor_c.vk_struct_field_s (strip_info_visitor()) x
 let al_statement x = Visitor_c.vk_statement_s (strip_info_visitor()) x
 let al_type      x = Visitor_c.vk_type_s      (strip_info_visitor()) x
 let al_init      x = Visitor_c.vk_ini_s       (strip_info_visitor()) x
+let al_inits     x = Visitor_c.vk_inis_s      (strip_info_visitor()) x
 let al_param     x = Visitor_c.vk_param_s     (strip_info_visitor()) x
 let al_params    x = Visitor_c.vk_params_s    (strip_info_visitor()) x
 let al_arguments x = Visitor_c.vk_arguments_s (strip_info_visitor()) x
 let al_fields    x = Visitor_c.vk_struct_fields_s (strip_info_visitor()) x
 let al_name      x = Visitor_c.vk_name_s      (strip_info_visitor()) x
+let al_string_format x = Visitor_c.vk_string_format_s (strip_info_visitor()) x
+let al_string_fragments x =
+  Visitor_c.vk_string_fragments_s (strip_info_visitor()) x
 
 let al_node      x = Visitor_c.vk_node_s      (strip_info_visitor()) x
 
@@ -88,13 +92,15 @@ let al_ii    x = Visitor_c.vk_ii_s (strip_info_visitor()) x
 
 
 let strip_inh_info_visitor _ =  (* for inherited metavariables *)
-  let drop_test_lv ty =
+  let drop_test_lv ty bigf =
     let (ty,_) = !ty in
     let ty =
       match ty with
 	None -> None
-      |	Some (ty,_) -> Some (ty,Ast_c.NotLocalVar) in
-    ref (ty,Ast_c.NotTest) in
+      |	Some (ty,_) ->
+	  let ty = Visitor_c.vk_type_s bigf ty in
+	  Some (ty,Ast_c.NotLocalVar) in
+    ref ((ty,Ast_c.NotTest) : Ast_c.exp_info) in
 
   { Visitor_c.default_visitor_c_s with
     Visitor_c.kinfo_s =
@@ -103,9 +109,9 @@ let strip_inh_info_visitor _ =  (* for inherited metavariables *)
     (function (k,_) ->
     function i -> ctr := !ctr + 1; Ast_c.al_info_cpp !ctr i));
 
-    Visitor_c.kexpr_s = (fun (k,_) e ->
+    Visitor_c.kexpr_s = (fun (k,bigf) e ->
       let (e', ty), ii' = k e in
-      (e', drop_test_lv ty), ii' (* keep type - jll *)
+      (e', drop_test_lv ty bigf), ii' (* keep type, but process it - jll *)
     );
 
 (*
@@ -129,7 +135,12 @@ let al_inh_field_list x =
 let al_inh_statement x = Visitor_c.vk_statement_s (strip_inh_info_visitor()) x
 let al_inh_type      x = Visitor_c.vk_type_s      (strip_inh_info_visitor()) x
 let al_inh_init      x = Visitor_c.vk_ini_s       (strip_inh_info_visitor()) x
+let al_inh_inits     x = Visitor_c.vk_inis_s      (strip_inh_info_visitor()) x
 let al_inh_arguments x = Visitor_c.vk_arguments_s (strip_inh_info_visitor()) x
+let al_inh_string_format x =
+  Visitor_c.vk_string_format_s (strip_inh_info_visitor()) x
+let al_inh_string_fragments x =
+  Visitor_c.vk_string_fragments_s (strip_inh_info_visitor()) x
 
 
 
@@ -155,9 +166,14 @@ let semi_al_fields = Visitor_c.vk_struct_fields_s semi_strip_info_visitor
 let semi_al_statement = Visitor_c.vk_statement_s semi_strip_info_visitor
 let semi_al_type      = Visitor_c.vk_type_s      semi_strip_info_visitor
 let semi_al_init      = Visitor_c.vk_ini_s       semi_strip_info_visitor
+let semi_al_inits     = Visitor_c.vk_inis_s      semi_strip_info_visitor
 let semi_al_param     = Visitor_c.vk_param_s     semi_strip_info_visitor
 let semi_al_params    = Visitor_c.vk_params_s    semi_strip_info_visitor
 let semi_al_arguments = Visitor_c.vk_arguments_s semi_strip_info_visitor
+let semi_al_string_format =
+  Visitor_c.vk_string_format_s semi_strip_info_visitor
+let semi_al_string_fragments =
+  Visitor_c.vk_string_fragments_s semi_strip_info_visitor
 
 let semi_al_program =
   List.map (Visitor_c.vk_toplevel_s semi_strip_info_visitor)
@@ -192,8 +208,12 @@ let real_strip_info_visitor _ =
   }
 
 let real_al_expr      x = Visitor_c.vk_expr_s   (real_strip_info_visitor()) x
+let real_al_arguments x = Visitor_c.vk_arguments_s (real_strip_info_visitor()) x
 let real_al_node      x = Visitor_c.vk_node_s   (real_strip_info_visitor()) x
 let real_al_type      x = Visitor_c.vk_type_s   (real_strip_info_visitor()) x
+let real_al_decl      x = Visitor_c.vk_decl_s   (real_strip_info_visitor()) x
+let real_al_init      x = Visitor_c.vk_ini_s    (real_strip_info_visitor()) x
+let real_al_inits     x = Visitor_c.vk_inis_s   (real_strip_info_visitor()) x
 let real_al_statement x = Visitor_c.vk_statement_s (real_strip_info_visitor()) x
 let real_al_def       x = Visitor_c.vk_toplevel_s (real_strip_info_visitor()) x
 
@@ -213,11 +233,15 @@ let extract_info_visitor recursor x =
     !globals
   end
 
+let ii_of_def = extract_info_visitor Visitor_c.vk_def
 let ii_of_decl = extract_info_visitor Visitor_c.vk_decl
 let ii_of_field = extract_info_visitor Visitor_c.vk_struct_field
 let ii_of_node = extract_info_visitor Visitor_c.vk_node
 let ii_of_expr = extract_info_visitor Visitor_c.vk_expr
+let ii_of_assignOp = extract_info_visitor Visitor_c.vk_assignOp
+let ii_of_binaryOp = extract_info_visitor Visitor_c.vk_binaryOp
 let ii_of_stmt = extract_info_visitor Visitor_c.vk_statement
+let ii_of_stmtseq = extract_info_visitor Visitor_c.vk_statement_sequencable
 let ii_of_args = extract_info_visitor Visitor_c.vk_args_splitted
 let ii_of_type = extract_info_visitor Visitor_c.vk_type
 let ii_of_ini  = extract_info_visitor Visitor_c.vk_ini
@@ -227,10 +251,19 @@ let ii_of_params = extract_info_visitor Visitor_c.vk_params_splitted
 let ii_of_enum_fields = extract_info_visitor Visitor_c.vk_enum_fields_splitted
 let ii_of_struct_fields = extract_info_visitor Visitor_c.vk_struct_fields
 (*let ii_of_struct_field = extract_info_visitor Visitor_c.vk_struct_field*)
-let ii_of_struct_fieldkinds = extract_info_visitor Visitor_c.vk_struct_fieldkinds
+let ii_of_struct_fieldkinds =
+  extract_info_visitor Visitor_c.vk_struct_fieldkinds
 let ii_of_cst = extract_info_visitor Visitor_c.vk_cst
+let ii_of_fragments =
+  extract_info_visitor Visitor_c.vk_string_fragments_splitted
+let ii_of_format = extract_info_visitor Visitor_c.vk_string_format
 let ii_of_define_params =
   extract_info_visitor Visitor_c.vk_define_params_splitted
+let ii_of_pragmainfo = extract_info_visitor Visitor_c.vk_pragmainfo
+let ii_of_ident_list = extract_info_visitor Visitor_c.vk_ident_list_splitted
+let ii_of_exec_code_list =
+  extract_info_visitor Visitor_c.vk_exec_code_list_splitted
+let ii_of_attrs = extract_info_visitor Visitor_c.vk_attrs_splitted
 let ii_of_toplevel = extract_info_visitor Visitor_c.vk_toplevel
 
 (*****************************************************************************)
@@ -255,7 +288,8 @@ let info_to_fixpos ii =
       Ast_cocci.Virt (pi.Common.charpos,offset)
   | Ast_c.FakeTok (_,(pi,offset)) ->
       Ast_cocci.Virt (pi.Common.charpos,offset)
-  | Ast_c.AbstractLineTok pi -> failwith "unexpected abstract"
+  | Ast_c.AbstractLineTok pi ->
+      failwith ("unexpected abstract: "^(Dumper.dump pi))
 
 let max_min_by_pos xs =
   let (i1, i2) = max_min_ii_by_pos xs in
