@@ -37,17 +37,24 @@ type 'a printer = 'a -> unit
 
 type pretty_printers = {
   expression      : Ast_c.expression printer;
+  assignOp        : Ast_c.assignOp printer;
+  binaryOp        : Ast_c.binaryOp printer;
   arg_list        : (Ast_c.argument Ast_c.wrap2 list) printer;
+  arg             : Ast_c.argument printer;
   statement       : Ast_c.statement printer;
   decl            : Ast_c.declaration printer;
   field           : Ast_c.field printer;
   field_list      : Ast_c.field list printer;
   init            : Ast_c.initialiser printer;
+  init_list       : (Ast_c.initialiser wrap2 list) printer;
   param           : Ast_c.parameterType printer;
   paramlist       : (Ast_c.parameterType Ast_c.wrap2 list) printer;
   ty              : Ast_c.fullType printer;
   type_with_ident : type_with_ident;
   toplevel        : Ast_c.toplevel printer;
+  fragment        : Ast_c.string_fragment printer;
+  fragment_list   : (Ast_c.string_fragment list) printer;
+  format          : Ast_c.string_format printer;
   flow            : Control_flow_c.node printer
 }
 
@@ -70,7 +77,11 @@ let mk_pretty_printers
  =
   let start_block () = pr_nl(); pr_indent() in
   let end_block   () = pr_unindent(); pr_nl() in
-
+(*
+  let pr_nl_slash _ = (* multiline macro *)
+    let slash = (Ast_c.fakeInfo() +> Ast_c.rewrap_str " \\") in
+    pr_elem slash; pr_nl() in
+*)
   let indent_if_needed st f =
     match Ast_c.unwrap_st st with
       Compound _ -> pr_space(); f()
@@ -79,14 +90,26 @@ let mk_pretty_printers
 	start_block(); f(); pr_unindent() in
 
 
+  let pp_list printer l =
+    l +> List.iter (fun (e, opt) ->
+      assert (List.length opt <= 1); (* opt must be a comma? *)
+      opt +> List.iter (function x -> pr_elem x; pr_space());
+      printer e) in
 
+  let pp_list2 printer l = (* no comma case *)
+    l +> List.iter printer in
 
   let rec pp_expression = fun ((exp, typ), ii) ->
     (match exp, ii with
     | Ident (ident),         []     -> pp_name ident
     (* only a MultiString can have multiple ii *)
-    | Constant (MultiString _), is     -> is +> List.iter pr_elem
+    | Constant (MultiString _), is     ->
+	is +> Common.print_between pr_space pr_elem
     | Constant (c),         [i]     -> pr_elem i
+    | StringConstant(s,os,w),  [i1;i2] ->
+	pr_elem i1;
+	s +> (List.iter pp_string_fragment);
+	pr_elem i2
     | FunCall  (e, es),     [i1;i2] ->
         pp_expression e; pr_elem i1;
 	pp_arg_list es;
@@ -98,14 +121,14 @@ let mk_pretty_printers
         pr_space(); pp_expression e3
     | Sequence (e1, e2),          [i]  ->
         pp_expression e1; pr_elem i; pr_space(); pp_expression e2
-    | Assignment (e1, op, e2),    [i]  ->
-        pp_expression e1; pr_space(); pr_elem i; pr_space(); pp_expression e2
+    | Assignment (e1, op, e2),    []  ->
+        pp_expression e1; pr_space(); pr_assignOp op; pr_space(); pp_expression e2
 
     | Postfix  (e, op),    [i] -> pp_expression e; pr_elem i;
     | Infix    (e, op),    [i] -> pr_elem i; pp_expression e;
     | Unary    (e, op),    [i] -> pr_elem i; pp_expression e
-    | Binary   (e1, op, e2),    [i] ->
-        pp_expression e1; pr_space(); pr_elem i; pr_space(); pp_expression e2
+    | Binary   (e1, op, e2),    [] ->
+        pp_expression e1; pr_space(); pr_binaryOp op; pr_space(); pp_expression e2
 
     | ArrayAccess    (e1, e2),   [i1;i2] ->
         pp_expression e1; pr_elem i1; pp_expression e2; pr_elem i2
@@ -131,32 +154,26 @@ let mk_pretty_printers
         statxs +> List.iter pp_statement_seq;
         pr_elem ii2;
         pr_elem i2;
-    | Constructor (t, xs), lp::rp::i1::i2::iicommaopt ->
+    | Constructor (t, init), [lp;rp] ->
         pr_elem lp;
         pp_type t;
         pr_elem rp;
-        pr_elem i1;
-        xs +> List.iter (fun (x, ii) ->
-          assert (List.length ii <= 1);
-          ii +> List.iter (function x -> pr_elem x; pr_space());
-          pp_init x
-        );
-        iicommaopt +> List.iter pr_elem;
-        pr_elem i2;
+	pp_init init
 
     | ParenExpr (e), [i1;i2] -> pr_elem i1; pp_expression e; pr_elem i2;
 
-    | New   (t),     [i1] -> pr_elem i1; pp_argument t
+    | New   (None, t),     [i1] -> pr_elem i1; pp_argument t
+    | New   (Some ts, t),     [i1; i2; i3] ->
+	pr_elem i1; pr_elem i2; pp_arg_list ts; pr_elem i3; pp_argument t
     | Delete(t),     [i1] -> pr_elem i1; pp_expression t
 
-    | (Ident (_) | Constant _ | FunCall (_,_) | CondExpr (_,_,_)
-    | Sequence (_,_)
-    | Assignment (_,_,_)
+    | (Ident (_) | Constant _ | StringConstant _ | FunCall (_,_)
+    | CondExpr (_,_,_) | Sequence (_,_) | Assignment (_,_,_)
     | Postfix (_,_) | Infix (_,_) | Unary (_,_) | Binary (_,_,_)
     | ArrayAccess (_,_) | RecordAccess (_,_) | RecordPtAccess (_,_)
     | SizeOfExpr (_) | SizeOfType (_) | Cast (_,_)
     | StatementExpr (_) | Constructor _
-    | ParenExpr (_) | New (_) | Delete (_)),_ -> raise Impossible
+    | ParenExpr (_) | New (_) | Delete (_)),_ -> raise (Impossible 95)
     );
 
     if !Flag_parsing_c.pretty_print_type_info
@@ -173,11 +190,15 @@ let mk_pretty_printers
       pr_elem (Ast_c.fakeInfo() +> Ast_c.rewrap_str "*/");
     end
 
-  and pp_arg_list es =
-    es +> List.iter (fun (e, opt) ->
-      assert (List.length opt <= 1); (* opt must be a comma? *)
-      opt +> List.iter (function x -> pr_elem x; pr_space());
-      pp_argument e)
+  and pr_assignOp (_,ii) =
+    let i = Common.tuple_of_list1 ii in
+    pr_elem i
+
+  and pr_binaryOp (_,ii) =
+    let i = Common.tuple_of_list1 ii in
+    pr_elem i
+
+  and pp_arg_list es = pp_list pp_argument es
 
   and pp_argument argument =
     let rec pp_action (ActMisc ii) = ii +> List.iter pr_elem in
@@ -210,6 +231,24 @@ let mk_pretty_printers
         );
         pr_elem icp
 
+and pp_string_fragment (e,ii) =
+  match (e,ii) with
+    ConstantFragment(str), ii ->
+      let (i) = Common.tuple_of_list1 ii in
+      pr_elem i
+  | FormatFragment(fmt), ii ->
+      let (i) = Common.tuple_of_list1 ii in
+      pr_elem i;
+      pp_string_format fmt
+
+and pp_string_fragment_list sfl = pp_list2 pp_string_fragment sfl
+
+and pp_string_format (e,ii) =
+  match (e,ii) with
+    ConstantFormat(str), ii ->
+      let (i) = Common.tuple_of_list1 ii in
+      pr_elem i
+
 (* ---------------------- *)
   and pp_statement = fun st ->
     match Ast_c.get_st_and_ii st with
@@ -240,16 +279,25 @@ let mk_pretty_printers
            ';' hence the [] for ii *)
     | ExprStatement (Some e), [] -> pp_expression e;
     | Selection  (If (e, st1, st2)), i1::i2::i3::is ->
-        pr_elem i1; pr_space(); pr_elem i2; pp_expression e; pr_elem i3;
-	indent_if_needed st1 (function _ -> pp_statement st1);
-        (match (Ast_c.get_st_and_ii st2, is) with
-        | ((ExprStatement None, []), [])  -> ()
-        | ((ExprStatement None, []), [iifakend])  -> pr_elem iifakend
-        | _st2, [i4;iifakend] -> pr_elem i4;
-	    indent_if_needed st2 (function _ -> pp_statement st2);
-	    pr_elem iifakend
-        | x -> raise Impossible
-        )
+        pp_ifthen e st1 i1 i2 i3;
+        pp_else st2 is
+    | Selection  (Ifdef_Ite (e, st1, st2)), i1::i2::i3::i4::is ->
+        pr_elem i1;
+        pp_ifthen e st1 i2 i3 i4;
+        pp_else st2 is
+    | Selection (Ifdef_Ite2 (e, st1, st2, st3)), i1::i2::i3::i4::is ->
+        pr_elem i1;
+        pp_ifthen e st1 i2 i3 i4;
+        (* else #else S #endif *)
+	(match is with
+          [i4;i5;i6;iifakend] ->
+            pr_elem i4; (* else *)
+            pr_elem i5; (* #else *)
+            indent_if_needed st2 (function _ -> pp_statement st2);
+            pr_elem i6; (* #endif *)
+            indent_if_needed st3 (function _ -> pp_statement st3);
+            pr_elem iifakend
+	| _ -> raise (Impossible 90))
     | Selection  (Switch (e, st)), [i1;i2;i3;iifakend] ->
         pr_elem i1; pr_space(); pr_elem i2; pp_expression e; pr_elem i3;
 	indent_if_needed st (function _-> pp_statement st); pr_elem iifakend
@@ -264,12 +312,15 @@ let mk_pretty_printers
         pr_elem iifakend
 
 
-    | Iteration  (For ((e1opt,il1),(e2opt,il2),(e3opt, il3),st)),
+    | Iteration  (For (first,(e2opt,il2),(e3opt, il3),st)),
         [i1;i2;i3;iifakend] ->
 
           pr_elem i1; pr_space();
           pr_elem i2;
-          pp_statement (Ast_c.mk_st (ExprStatement e1opt) il1);
+	  (match first with
+	    ForExp (e1opt,il1) ->
+              pp_statement (Ast_c.mk_st (ExprStatement e1opt) il1)
+	  | ForDecl decl -> pp_decl decl);
           pp_statement (Ast_c.mk_st (ExprStatement e2opt) il2);
           assert (null il3);
           pp_statement (Ast_c.mk_st (ExprStatement e3opt) il3);
@@ -311,7 +362,7 @@ let mk_pretty_printers
             pr_elem iasm; pr_elem ivolatile; pr_elem iopar;
             pp_asmbody asmbody;
             pr_elem icpar; pr_elem iptvirg
-        | _ -> raise Impossible
+        | _ -> raise (Impossible 97)
         )
 
     | NestedFunc def, ii ->
@@ -320,17 +371,22 @@ let mk_pretty_printers
     | MacroStmt, ii ->
         ii +> List.iter pr_elem ;
 
+    | Exec(code), [exec;lang;sem] ->
+	pr_elem exec; pr_space(); pr_elem lang; pr_space();
+	pp_list2 pp_exec_code code; pr_elem sem
+
     | (Labeled (Case  (_,_))
     | Labeled (CaseRange  (_,_,_)) | Labeled (Default _)
     | Compound _ | ExprStatement _
     | Selection  (If (_, _, _)) | Selection  (Switch (_, _))
+    | Selection (Ifdef_Ite _) | Selection (Ifdef_Ite2 _)
     | Iteration  (While (_, _)) | Iteration  (DoWhile (_, _))
-    | Iteration  (For ((_,_), (_,_), (_, _), _))
+    | Iteration  (For (_, (_,_), (_, _), _))
     | Iteration  (MacroIteration (_,_,_))
     | Jump ((Continue|Break|Return)) | Jump (ReturnExpr _)
     | Jump (GotoComputed _)
-    | Decl _
-	), _ -> raise Impossible
+    | Decl _ | Exec _
+	), _ -> raise (Impossible 98)
 
   and pp_statement_seq = function
     | StmtElem st -> pp_statement st
@@ -338,13 +394,33 @@ let mk_pretty_printers
     | CppDirectiveStmt cpp -> pp_directive cpp
     | IfdefStmt2 (ifdef, xxs) -> pp_ifdef_tree_sequence ifdef xxs
 
+  and pp_ifthen e st1 i1 i2 i3 =
+        (* if (<e>) *)
+        pr_elem i1; pr_space(); pr_elem i2; pp_expression e; pr_elem i3;
+        indent_if_needed st1 (function _ -> pp_statement st1);
+  and pp_else st2 is =
+        match (Ast_c.get_st_and_ii st2, is) with
+          | ((ExprStatement None, []), [])  -> ()
+          | ((ExprStatement None, []), [iifakend])  -> pr_elem iifakend
+          | _st2, [i4;iifakend] ->
+              pr_elem i4;
+              indent_if_needed st2 (function _ -> pp_statement st2);
+              pr_elem iifakend
+          | _st2, [i4;i5;iifakend] -> (* else #endif S *)
+              pr_elem i4;
+              pr_elem i5;
+              indent_if_needed st2 (function _ -> pp_statement st2);
+              pr_elem iifakend
+          | x -> raise (Impossible 96)
+
+
 (* ifdef XXX elsif YYY elsif ZZZ endif *)
   and pp_ifdef_tree_sequence ifdef xxs =
     match ifdef with
     | if1::ifxs ->
-	pp_ifdef if1;
-	pp_ifdef_tree_sequence_aux  ifxs xxs
-    | _ -> raise Impossible
+        pp_ifdef if1;
+        pp_ifdef_tree_sequence_aux  ifxs xxs
+    | _ -> raise (Impossible 99)
 
 (* XXX elsif YYY elsif ZZZ endif *)
   and pp_ifdef_tree_sequence_aux ifdefs xxs =
@@ -384,9 +460,14 @@ let mk_pretty_printers
 		pr_elem iopar;
 		pp_expression e;
 		pr_elem icpar
-	    | _ -> raise Impossible))
+	    | _ -> raise (Impossible 100)))
         ))
 
+  and pp_exec_code = function
+    ExecEval name, [colon] -> pr_elem colon; pp_expression name
+  | ExecToken, [tok] -> pr_elem tok
+  | _ -> raise (Impossible 101)
+
 
 (* ---------------------- *)
 
@@ -457,7 +538,7 @@ let mk_pretty_printers
               pr_elem i1; pr_elem i2; pr_elem i3;
           | None, [i1;i2;i3] ->
               pr_elem i1; pr_elem i2;
-          | x -> raise Impossible
+          | x -> raise (Impossible 101)
 	  );
 
           fields +> List.iter pp_field;
@@ -465,7 +546,7 @@ let mk_pretty_printers
           (match sopt,iis with
           | Some s , [i1;i2;i3;i4] -> pr_elem i4
           | None, [i1;i2;i3] ->       pr_elem i3;
-          | x -> raise Impossible
+          | x -> raise (Impossible 102)
 	  );
 
 
@@ -478,7 +559,7 @@ let mk_pretty_printers
               pr_elem i1; pr_elem i2; pr_elem i3;
           | (None, ([i1;i2;i3]|[i1;i2;i3;_])) ->
               pr_elem i1; pr_elem i2
-          | x -> raise Impossible
+          | x -> raise (Impossible 103)
 	  );
 
           enumt +> List.iter (fun ((name, eopt), iicomma) ->
@@ -499,7 +580,7 @@ let mk_pretty_printers
               pr_elem i4; pr_elem i3 (* trailing comma *)
 
 
-          | x -> raise Impossible
+          | x -> raise (Impossible 104)
 	  );
 
 
@@ -514,6 +595,13 @@ let mk_pretty_printers
           assert (List.length iis =|= 2);
           print_sto_qu_ty (sto, qu, iis);
 
+      | (Decimal(l,p), [dec;lp;cm;rp]) ->
+	  (* hope that sto before qu is right... cf print_sto_qu_ty *)
+	  let stoqulp = get_sto sto ++ (snd qu) ++ [dec] in
+	  Common.print_between pr_space pr_elem stoqulp;
+	  pr_elem lp; pp_expression l; pr_elem cm;
+	  do_option pp_expression p; pr_elem rp
+
       | (TypeName (name,typ), noii) ->
           assert (null noii);
           let (_s, iis) = get_s_and_info_of_name name in
@@ -535,7 +623,7 @@ let mk_pretty_printers
               pr_elem itypeof; pr_elem iopar;
               pp_expression e;
               pr_elem icpar;
-          | _ -> raise Impossible
+          | _ -> raise (Impossible 105)
           )
 
       | (TypeOfType (t), iis) ->
@@ -545,18 +633,20 @@ let mk_pretty_printers
               pr_elem itypeof; pr_elem iopar;
               pp_type t;
               pr_elem icpar;
-          | _ -> raise Impossible
+          | _ -> raise (Impossible 106)
 	  )
 
-      | (Pointer _ | (*ParenType _ |*) Array _ | FunctionType _
+      | (Pointer _ | (*ParenType _ |*) Array _ | FunctionType _ | Decimal _
             (* | StructUnion _ | Enum _ | BaseType _ *)
             (* | StructUnionName _ | EnumName _ | TypeName _  *)
             (* | TypeOfExpr _ | TypeOfType _ *)
-         ), _ -> raise Impossible
+         ), _ -> raise (Impossible 107)
 
   and pp_field_list fields = fields +>  Common.print_between pr_nl pp_field
   and pp_field = function
-      DeclarationField(FieldDeclList(onefield_multivars,iiptvirg))->
+      DeclarationField
+	(FieldDeclList(onefield_multivars,[iiptvirg;ifakestart])) ->
+	pr_elem ifakestart;
         (match onefield_multivars with
           x::xs ->
 	    (* handling the first var. Special case, with the
@@ -564,7 +654,7 @@ let mk_pretty_printers
 
 	    (match x with
 	      (Simple (nameopt, typ)), iivirg ->
-              (* first var cant have a preceding ',' *)
+              (* first var cannot have a preceding ',' *)
 		assert (List.length iivirg =|= 0);
 		let identinfo =
                   match nameopt with
@@ -572,9 +662,9 @@ let mk_pretty_printers
                   | Some name -> Some (get_s_and_info_of_name name)
                 in
 		pp_type_with_ident identinfo None typ Ast_c.noattr;
-		
+
 	    | (BitField (nameopt, typ, iidot, expr)), iivirg ->
-                      (* first var cant have a preceding ',' *)
+                      (* first var cannot have a preceding ',' *)
 		assert (List.length iivirg =|= 0);
 		(match nameopt with
 		| None ->
@@ -617,11 +707,12 @@ let mk_pretty_printers
 		      pp_expression expr
 			)); (* iter other vars *)
 
-	| [] -> raise Impossible
+	| [] -> raise (Impossible 108)
 	      ); (* onefield_multivars *)
-	assert (List.length iiptvirg =|= 1);
-	iiptvirg +> List.iter pr_elem;
+	pr_elem iiptvirg
 
+    | DeclarationField(FieldDeclList(onefield_multivars,_)) ->
+	failwith "wrong number of tokens"
 
     | MacroDeclField ((s, es), ii)  ->
         let (iis, lp, rp, iiend, ifakestart) =
@@ -649,7 +740,7 @@ let mk_pretty_printers
     | CppDirectiveStruct cpp -> pp_directive cpp
     | IfdefStruct ifdef -> pp_ifdef ifdef
 
-(* used because of DeclList, in    int i,*j[23];  we dont print anymore the
+(* used because of DeclList, in    int i,*j[23];  we don't print anymore the
    int before *j *)
   and (pp_type_with_ident_rest: (string * info) option ->
     fullType -> attribute list -> unit) =
@@ -664,12 +755,13 @@ let mk_pretty_printers
 
       match ty, iity with
       (* the work is to do in base_type !! *)
-      | (NoType _, iis)                         -> failwith "printing notype"
+      | (NoType, iis)                           -> ()
       | (BaseType _, iis)                       -> print_ident ident
       | (Enum  (sopt, enumt), iis)              -> print_ident ident
       | (StructUnion (_, sopt, fields),iis)     -> print_ident ident
       | (StructUnionName (s, structunion), iis) -> print_ident ident
       | (EnumName  s, iis)                      -> print_ident ident
+      | (Decimal _, iis)                        -> print_ident ident
       | (TypeName (_name,_typ), iis)            -> print_ident ident
       | (TypeOfExpr (e), iis)                   -> print_ident ident
       | (TypeOfType (e), iis)                   -> print_ident ident
@@ -752,7 +844,7 @@ let mk_pretty_printers
 
 
       | (FunctionType _ | Array _ | ParenType _ | Pointer _), _ ->
-	  raise Impossible
+	  raise (Impossible 109)
 
 
   and (pp_type_left: fullType -> unit) =
@@ -775,12 +867,13 @@ let mk_pretty_printers
       | (StructUnion (_, sopt, fields),iis)  -> ()
       | (StructUnionName (s, structunion), iis) -> ()
       | (EnumName  s, iis) -> ()
+      | (Decimal(l,p), iis) -> ()
       | (TypeName (_name,_typ), iis) -> ()
 
       | TypeOfType _, _ -> ()
       | TypeOfExpr _, _ -> ()
 
-      | (FunctionType _ | Array _ | Pointer _), _ -> raise Impossible
+      | (FunctionType _ | Array _ | Pointer _), _ -> raise (Impossible 110)
 
 
   and pp_param param =
@@ -832,12 +925,13 @@ let mk_pretty_printers
     | (StructUnion (_, sopt, fields),iis)-> ()
     | (StructUnionName (s, structunion), iis) -> ()
     | (EnumName  s, iis) -> ()
+    | (Decimal(l,p), iis) -> ()
     | (TypeName (name,_typ), iis) -> ()
 
     | TypeOfType _, _ -> ()
     | TypeOfExpr _, _ -> ()
 
-    | (FunctionType _ | Array _ | Pointer _), _ -> raise Impossible
+    | (FunctionType _ | Array _ | Pointer _), _ -> raise (Impossible 111)
 
   and pp_type t =
     pp_type_with_ident None None t Ast_c.noattr
@@ -868,7 +962,7 @@ let mk_pretty_printers
 	    | Ast_c.ValInit(iini,init) -> pr_elem iini; pp_init init
 	    | Ast_c.ConstrInit((init,[lp;rp])) ->
 		pr_elem lp; pp_arg_list init; pr_elem rp
-	    | Ast_c.ConstrInit _ -> raise Impossible)
+	    | Ast_c.ConstrInit _ -> raise (Impossible 112))
 	| None -> pp_type returnType
 	);
 
@@ -890,15 +984,15 @@ let mk_pretty_printers
 	    | Ast_c.ValInit(iini,init) -> pr_elem iini; pp_init init
 	    | Ast_c.ConstrInit((init,[lp;rp])) ->
 		pr_elem lp; pp_arg_list init; pr_elem rp
-	    | Ast_c.ConstrInit _ -> raise Impossible);
+	    | Ast_c.ConstrInit _ -> raise (Impossible 113));
 
 
-	| x -> raise Impossible
+	| x -> raise (Impossible 114)
 	);
 
 	pr_elem iivirg;
 
-    | MacroDecl ((s, es), iis::lp::rp::iiend::ifakestart::iisto) ->
+    | MacroDecl ((s, es, true), iis::lp::rp::iiend::ifakestart::iisto) ->
 	pr_elem ifakestart;
 	iisto +> List.iter pr_elem; (* static and const *)
 	pr_elem iis;
@@ -912,8 +1006,38 @@ let mk_pretty_printers
 	pr_elem rp;
 	pr_elem iiend;
 
-    | (DeclList (_, _) | (MacroDecl _)) -> raise Impossible
+    | MacroDecl ((s, es, false), iis::lp::rp::ifakestart::iisto) ->
+	pr_elem ifakestart;
+	iisto +> List.iter pr_elem; (* static and const *)
+	pr_elem iis;
+	pr_elem lp;
+	es +> List.iter (fun (e, opt) ->
+          assert (List.length opt <= 1);
+          opt +> List.iter pr_elem;
+          pp_argument e;
+	);
 
+	pr_elem rp;
+
+    | MacroDeclInit
+	((s, es, ini), iis::lp::rp::eq::iiend::ifakestart::iisto) ->
+	pr_elem ifakestart;
+	iisto +> List.iter pr_elem; (* static and const *)
+	pr_elem iis;
+	pr_elem lp;
+	es +> List.iter (fun (e, opt) ->
+          assert (List.length opt <= 1);
+          opt +> List.iter pr_elem;
+          pp_argument e;
+	);
+
+	pr_elem rp;
+	pr_elem eq;
+	pp_init ini;
+	pr_elem iiend;
+
+    | (DeclList (_, _) | (MacroDecl _) | (MacroDeclInit _)) ->
+	raise (Impossible 115)
 
 (* ---------------------- *)
 and pp_init (init, iinit) =
@@ -944,9 +1068,9 @@ and pp_init (init, iinit) =
 
       | (InitIndexOld _ | InitFieldOld _ | InitDesignators _
       | InitList _ | InitExpr _
-	  ), _ -> raise Impossible
-
+	  ), _ -> raise (Impossible 116)
 
+  and pp_init_list ini = pp_list pp_init ini
 
   and pp_designator = function
     | DesignatorField (s), [i1; i2] ->
@@ -959,7 +1083,7 @@ and pp_init (init, iinit) =
 	pp_expression e2; pr_elem iccro;
 
     | (DesignatorField _ | DesignatorIndex _ | DesignatorRange _
-	), _ -> raise Impossible
+	), _ -> raise (Impossible 117)
 
 
 (* ---------------------- *)
@@ -1036,13 +1160,9 @@ and pp_init (init, iinit) =
         pr_elem i1;
         statxs +> List.iter pp_statement_seq;
         pr_elem i2;
-    | _ -> raise Impossible
+    | _ -> raise (Impossible 118)
 
-  and pp_param_list paramst =
-    paramst +> List.iter (fun (param,iicomma) ->
-      assert ((List.length iicomma) <= 1);
-      iicomma +> List.iter (function x -> pr_elem x; pr_space());
-      pp_param param)
+  and pp_param_list paramst = pp_list pp_param paramst
 
 (* ---------------------- *)
 
@@ -1072,7 +1192,7 @@ and pp_init (init, iinit) =
                   pr_elem iwhile; pr_elem iopar;
                   pp_expression e;
                   pr_elem icpar
-              | _ -> raise Impossible
+              | _ -> raise (Impossible 119)
 	      )
           | DefineFunction def -> pp_def def
 
@@ -1080,7 +1200,8 @@ and pp_init (init, iinit) =
           | DefineText (s, ii) -> List.iter pr_elem ii
           | DefineEmpty -> ()
           | DefineInit ini -> pp_init ini
-
+	  | DefineMulti ss ->
+	      ss +> List.iter pp_statement
           | DefineTodo -> pr2 "DefineTodo"
 	in
 	(match defkind with
@@ -1098,13 +1219,30 @@ and pp_init (init, iinit) =
 	define_val defval;
 	pr_elem ieol
 
-    | PragmaAndCo (ii) ->
-	List.iter pr_elem ii in
+    | Pragma ((s,ii), pragmainfo) ->
+	let (ipragma,iident,ieol) = Common.tuple_of_list3 ii in
+	pr_elem ipragma;
+	pr_elem iident;
+	pp_pragmainfo pragmainfo;
+	pr_elem ieol
 
+    | OtherDirective (ii) ->
+	List.iter pr_elem ii
 
+  and pp_pragmainfo = function
+      PragmaTuple(args,ii) ->
+	let (ilp,irp) = Common.tuple_of_list2 ii in
+	pr_elem ilp;
+	pp_arg_list args;
+        pr_elem irp
+    | PragmaIdList(ids) ->
+	let rec loop = function
+	    [] -> ()
+	  | [id,_] -> pp_name id
+	  | (id,_)::rest -> pp_name id; pr_space() in
+	loop ids in
 
-
-  let pp_toplevel = function
+  let rec pp_toplevel = function
     | Declaration decl -> pp_decl decl
     | Definition def -> pp_def def
 
@@ -1131,7 +1269,11 @@ and pp_init (init, iinit) =
 
     | IfdefTop ifdefdir -> pp_ifdef ifdefdir
 
-    | (MacroTop _) -> raise Impossible in
+    | Namespace (tls, [i1; i2; i3; i4]) ->
+	pr_elem i1; pr_elem i2; pr_elem i3;
+	List.iter pp_toplevel tls;
+	pr_elem i4;
+    | (MacroTop _) | (Namespace _) -> raise (Impossible 120) in
 
 
 
@@ -1176,7 +1318,7 @@ and pp_init (init, iinit) =
 	pr2 "XXX";
 
 
-    | F.ForHeader (_st, (((e1opt,i1), (e2opt,i2), (e3opt,i3)), ii)) ->
+    | F.ForHeader (_st, ((first, (e2opt,i2), (e3opt,i3)), ii)) ->
         (*
            iif i1; iif i2; iif i3;
            iif ii;
@@ -1231,6 +1373,11 @@ and pp_init (init, iinit) =
         (* iif ii *)
 	pr2 "XXX"
 
+    | F.PragmaHeader((s,ii), pragmainfo) ->
+	let (ipragma,iident,ieol) = Common.tuple_of_list3 ii in
+	pr_elem ipragma;
+	pr_elem iident;
+	pp_pragmainfo pragmainfo
 
     | F.Include {i_include = (s, ii);} ->
         (* iif ii; *)
@@ -1243,7 +1390,7 @@ and pp_init (init, iinit) =
 	pr2 "XXX"
 
 
-    | F.Break    (st,((),ii)) ->
+    | F.Break    (st,((),ii),fromswitch) ->
         (* iif ii *)
 	pr2 "XXX"
     | F.Continue (st,((),ii)) ->
@@ -1287,6 +1434,7 @@ and pp_init (init, iinit) =
         *)
 	pr2 "XXX"
 
+    | F.Exec(st,(code,ii)) -> pr2 "XXX"
 
     | F.IfdefHeader (info) ->
 	pp_ifdef info
@@ -1295,29 +1443,39 @@ and pp_init (init, iinit) =
     | F.IfdefEndif (info) ->
 	pp_ifdef info
 
+    | F.IfdefIteHeader _ii ->
+        pr2 "XXX"
+
     | F.DefineTodo ->
 	pr2 "XXX"
 
 
     | (F.TopNode|F.EndNode|
       F.ErrorExit|F.Exit|F.Enter|F.LoopFallThroughNode|F.FallThroughNode|
-      F.AfterNode|F.FalseNode|F.TrueNode|F.InLoopNode|
+      F.AfterNode _|F.FalseNode|F.TrueNode _|F.InLoopNode|
       F.Fake) ->
         pr2 "YYY" in
 
 
   { expression = pp_expression;
+    assignOp   = pr_assignOp;
+    binaryOp   = pr_binaryOp;
     arg_list   = pp_arg_list;
+    arg        = pp_argument;
     statement  = pp_statement;
     decl       = pp_decl;
     field      = pp_field;
     field_list = pp_field_list;
     init       = pp_init;
+    init_list  = pp_init_list;
     param      = pp_param;
     paramlist  = pp_param_list;
     ty         = pp_type;
     type_with_ident = pp_type_with_ident;
     toplevel   = pp_toplevel;
+    fragment   = pp_string_fragment;
+    fragment_list = pp_string_fragment_list;
+    format     = pp_string_format;
     flow       = pp_flow;
   }
 
@@ -1326,18 +1484,29 @@ and pp_init (init, iinit) =
 (* Here we do not use (mcode, env). It is a simple C pretty printer. *)
 let pr_elem info =
   let s = Ast_c.str_of_info info in
-  if !Flag_parsing_c.pretty_print_comment_info then begin
-    let before = !(info.comments_tag).mbefore in
-    if not (null before) then begin
-      pp "-->";
-      before +> List.iter (fun (comment_like, pinfo) ->
-        let s = pinfo.Common.str in
-        pp s
-      );
-      pp "<--";
-    end;
-  end;
-  pp s
+  if !Flag_parsing_c.pretty_print_comment_info
+  then
+    (match get_comments_before info with
+      [] -> ()
+    | before ->
+	pp "-->";
+	before +> List.iter (fun (comment_like, pinfo) ->
+          let s = pinfo.Common.str in
+          pp s
+	    );
+	pp "<--");
+  pp s;
+  if !Flag_parsing_c.pretty_print_comment_info
+  then
+    (match get_comments_after info with
+      [] -> ()
+    | before ->
+	pp "==>";
+	before +> List.iter (fun (comment_like, pinfo) ->
+          let s = pinfo.Common.str in
+          pp s
+	    );
+	pp "<==")
 
 let pr_space _ = Format.print_space()
 
@@ -1352,12 +1521,16 @@ let ppc =
     ~pr_elem ~pr_space ~pr_nl ~pr_outdent ~pr_indent ~pr_unindent
 
 let pp_expression_simple = ppc.expression
+let pp_assignOp_simple   = ppc.assignOp
+let pp_binaryOp_simple   = ppc.binaryOp
 let pp_decl_simple       = ppc.decl
 let pp_field_simple      = ppc.field
 let pp_statement_simple  = ppc.statement
 let pp_type_simple       = ppc.ty
 let pp_init_simple       = ppc.init
 let pp_toplevel_simple   = ppc.toplevel
+let pp_string_fragment_simple = ppc.fragment
+let pp_string_format_simple = ppc.format
 let pp_flow_simple       = ppc.flow
 
 
@@ -1369,9 +1542,18 @@ let pp_elem_sp ~pr_elem ~pr_space =
 let pp_expression_gen ~pr_elem ~pr_space =
   (pp_elem_sp pr_elem pr_space).expression
 
+let pp_assignOp_gen ~pr_elem ~pr_space =
+  (pp_elem_sp pr_elem pr_space).assignOp
+
+let pp_binaryOp_gen ~pr_elem ~pr_space =
+  (pp_elem_sp pr_elem pr_space).binaryOp
+
 let pp_arg_list_gen ~pr_elem ~pr_space =
   (pp_elem_sp pr_elem pr_space).arg_list
 
+let pp_arg_gen ~pr_elem ~pr_space =
+  (pp_elem_sp pr_elem pr_space).arg
+
 let pp_statement_gen ~pr_elem ~pr_space =
   (pp_elem_sp pr_elem pr_space).statement
 
@@ -1387,6 +1569,9 @@ let pp_field_list_gen ~pr_elem ~pr_space =
 let pp_init_gen ~pr_elem ~pr_space =
   (pp_elem_sp pr_elem pr_space).init
 
+let pp_init_list_gen ~pr_elem ~pr_space =
+  (pp_elem_sp pr_elem pr_space).init_list
+
 let pp_param_gen ~pr_elem ~pr_space =
   (pp_elem_sp pr_elem pr_space).param
 
@@ -1399,6 +1584,12 @@ let pp_type_gen ~pr_elem ~pr_space =
 let pp_type_with_ident_gen pr_elem pr_space =
   (pp_elem_sp pr_elem pr_space).type_with_ident
 
+let pp_string_fragment_list_gen ~pr_elem ~pr_space =
+  (pp_elem_sp pr_elem pr_space).fragment_list
+
+let pp_string_format_gen ~pr_elem ~pr_space =
+  (pp_elem_sp pr_elem pr_space).format
+
 let pp_program_gen ~pr_elem ~pr_space =
   (pp_elem_sp pr_elem pr_space).toplevel
 
@@ -1408,6 +1599,13 @@ let string_of_expression e =
     pp_expression_simple e
   )
 
+let string_of_ifdef_guard = function
+  | Gifdef s  -> "defined(" ^ s ^ ")"
+  | Gifndef s -> "!defined(" ^ s ^ ")"
+  | Gif_str s -> s
+  | Gif e     -> string_of_expression e
+  | Gnone     -> "0"
+
 let string_of_toplevel top =
   Common.format_to_string (fun () ->
     pp_toplevel_simple top
