@@ -76,14 +76,21 @@ let is_not_comment x =
 
 let is_cpp_instruction = function
   | TInclude _
-  | TDefine _
-  | TIfdef _   | TIfdefelse _ | TIfdefelif _ | TEndif _
+  | TDefine _  | TPragma _
+  | TIfdef _ | TIfdefelse _ | TIfdefelif _ | TEndif _
   | TIfdefBool _ | TIfdefMisc _ | TIfdefVersion _
   | TUndef _
   | TCppDirectiveOther _
       -> true
   | _ -> false
 
+let is_cpp_else = function
+  | TIfdefelse _ -> true
+  | _            -> false
+
+let is_cpp_endif = function
+  | TEndif _ -> true
+  | _        -> false
 
 let is_gcc_token = function
   | Tasm _
@@ -93,7 +100,9 @@ let is_gcc_token = function
       -> true
   | _ -> false
 
-
+let is_escaped_newline = function
+  | TCppEscapedNewline _ -> true
+  | _                    -> false
 
 
 (* ---------------------------------------------------------------------- *)
@@ -130,6 +139,15 @@ let is_eom = function
   | TDefEOL _ -> true
   | _ -> false
 
+let is_else = function
+  | Telse _ -> true
+  | _       -> false
+
+let is_if_or_else = function
+  | Tif _ | Telse _
+      -> true
+  | _ -> false
+
 let is_statement = function
   | Tfor _ | Tdo _ | Tif _ | Twhile _ | Treturn _
   | Tbreak _ | Telse _ | Tswitch _ | Tcase _ | Tcontinue _
@@ -162,7 +180,7 @@ let is_start_of_something = function
   | Tauto _ | Tregister _ | Textern _ | Tstatic _
   | Tconst _ | Tvolatile _
   | Ttypedef _
-  | Tstruct _ | Tunion _ | Tenum _
+  | Tstruct _ | Tunion _ | Tenum _ | Tdecimal _
     -> true
   | _ -> false
 
@@ -172,7 +190,7 @@ let is_binary_operator = function
   | TOrLog _ | TAndLog _ |  TOr _ |  TXor _ |  TAnd _
   | TEqEq _ |  TNotEq _  | TInf _ |  TSup _ |  TInfEq _ |  TSupEq _
   | TShl _ | TShr _
-  | TPlus _ |  TMinus _ |  TMul _ |  TDiv _ |  TMod _
+  | TPlus _ |  TMinus _ |  TMul _ |  TDiv _ |  TMod _ | TMax _ | TMin _
         -> true
   | _ -> false
 
@@ -189,6 +207,7 @@ let is_stuff_taking_parenthized = function
 (* used in the algorithms for "10 most problematic errors" *)
 let is_ident_like = function
   | TIdent _
+  | TKRParam _
   | TypedefIdent _
   | TIdentDefine  _
   | TDefParamVariadic _
@@ -206,22 +225,84 @@ let is_ident_like = function
 
   | _ -> false
 
+(*****************************************************************************)
+(* Matching functions for 'Nasty Undisciplined Cpp' *)
+(*****************************************************************************)
+
+(* matches 'if ... else ...'
+ * where there is no other if the '...'.
+ *)
+let match_simple_if_else
+    : token list -> (token * token list * token * token list) option
+  = function
+  | (Tif _ as tok_if)::rest_if ->
+    begin try
+      let (body,tok_else,rest_else) = split_when is_else rest_if in
+      if List.exists is_if_or_else body
+         || List.exists is_if_or_else rest_else
+         then None (* no nested if/else wanted *)
+         else Some (tok_if,body,tok_else,rest_else)
+    with Not_found -> None end
+  | _ -> None
+
+(* matches '#ifdef ... #endif ...' *)
+let match_cpp_simple_ifdef_endif_aux
+    : token list -> (token * token list * token * token list) option
+  = function
+  | (TIfdef _ as tok_ifdef)::rest_ifdef ->
+    begin try
+      let (body,tok_endif,rest_endif) = split_when is_cpp_endif rest_ifdef in
+      Some (tok_ifdef,body,tok_endif,rest_endif)
+    with Not_found -> None end
+  | _ -> None
+
+(* matches '#ifdef ...1 #endif ...2'
+ * where there is no other #ifdef within '...1'
+ *)
+let match_cpp_simple_ifdef_endif (xs :token list)
+      : (token * token list * token * token list) option
+  = match match_cpp_simple_ifdef_endif_aux xs with
+  | Some (tok_ifdef,body,tok_endif,rest_endif)
+      when not(List.exists is_cpp_instruction body)
+        (* no nested CPP wanted *)
+      -> Some (tok_ifdef,body,tok_endif,rest_endif)
+  | _ -> None
+
+let match_cpp_simple_ifdef_else_endif (xs :token list)
+      : (token * token list * token * token list * token * token list) option
+  = match match_cpp_simple_ifdef_endif_aux xs with
+  | Some (tok_ifdef,body,tok_endif,rest_endif) ->
+    begin try
+      let (body_if,tok_else,body_else) = split_when is_cpp_else body in
+      if List.exists is_cpp_instruction body_if
+         || List.exists is_cpp_instruction body_else
+         then None (* no nested CPP wanted *)
+         else Some (tok_ifdef,body_if,tok_else,body_else,tok_endif,rest_endif)
+    with Not_found -> None end
+  | _ -> None
 
 (*****************************************************************************)
 (* Visitors *)
 (*****************************************************************************)
 
 (* Because ocamlyacc force us to do it that way. The ocamlyacc token
- * cant be a pair of a sum type, it must be directly a sum type.
+ * cannot be a pair of a sum type, it must be directly a sum type.
  *)
 let info_of_tok = function
   | TString ((string, isWchar), i) -> i
   | TChar  ((string, isWchar), i) -> i
   | TFloat ((string, floatType), i) -> i
+  | TDecimal ((string, n, p), i) -> i
 
-  | TAssign  (assignOp, i) -> i
+  | TQuote (_,i) -> i
+  | TPct i -> i
+  | TFormat(str,i) -> i
+  | TSubString(str,i) -> i
+
+  | TAssign  (assignOp, ii) -> Common.tuple_of_list1 ii
 
   | TIdent  (s, i) -> i
+  | TKRParam  (s, i) -> i
   | Tconstructorname  (s, i) -> i
   | TypedefIdent  (s, i) -> i
 
@@ -231,6 +312,7 @@ let info_of_tok = function
   | TInclude (includes, filename, inifdef, i1) ->     i1
 
   | TUndef (ii) -> ii
+  | TPragma (ii) -> ii
   | TCppDirectiveOther (ii) -> ii
 
   | TIncludeStart (i1, inifdef) ->     i1
@@ -270,14 +352,18 @@ let info_of_tok = function
   | TCommentSkipTagStart (i) -> i
   | TCommentSkipTagEnd (i) -> i
 
-  | TIfdef               (_, i) -> i
+  | TIfdef               (_, _, i) -> i
   | TIfdefelse           (_, i) -> i
-  | TIfdefelif           (_, i) -> i
+  | TIfdefelif           (_, _, i) -> i
   | TEndif               (_, i) -> i
   | TIfdefBool           (b, _, i) -> i
   | TIfdefMisc           (b, _, i) -> i
   | TIfdefVersion           (b, _, i) -> i
 
+  | TUifdef              (i) -> i
+  | TUelseif             (i) -> i
+  | TUendif              (i) -> i
+
   | TOPar                (i) -> i
   | TCPar                (i) -> i
   | TOBrace              (i) -> i
@@ -313,8 +399,9 @@ let info_of_tok = function
   | TMinus               (i) -> i
   | TMul                 (i) -> i
   | TDiv                 (i) -> i
+  | TMin                 (i) -> i
+  | TMax                 (i) -> i
   | TMod                 (i) -> i
-
   | Tchar                (i) -> i
   | Tshort               (i) -> i
   | Tint                 (i) -> i
@@ -338,6 +425,8 @@ let info_of_tok = function
 
   | Tstruct              (i) -> i
   | Tenum                (i) -> i
+  | Tdecimal             (i) -> i
+  | Texec                (i) -> i
   | Ttypedef             (i) -> i
   | Tunion               (i) -> i
   | Tbreak               (i) -> i
@@ -363,6 +452,7 @@ let info_of_tok = function
   | TOParCplusplusInit   (i) -> i
 
   | EOF                  (i) -> i
+  | Tnamespace           (i) -> i
 
 
 
@@ -372,9 +462,16 @@ let visitor_info_of_tok f = function
   | TString ((s, isWchar), i)  -> TString ((s, isWchar), f i)
   | TChar  ((s, isWchar), i)   -> TChar  ((s, isWchar), f i)
   | TFloat ((s, floatType), i) -> TFloat ((s, floatType), f i)
-  | TAssign  (assignOp, i)     -> TAssign  (assignOp, f i)
+  | TDecimal ((s, n, p), i)    -> TDecimal ((s, n, p), f i)
+  | TAssign  (assignOp, ii)     -> TAssign  (assignOp, List.map f ii)
+
+  | TQuote ((str,isW),i) -> TQuote ((str,isW),f i)
+  | TPct i -> TPct (f i)
+  | TFormat(str,i) -> TFormat(str,f i)
+  | TSubString(str,i) -> TSubString(str,f i)
 
   | TIdent  (s, i)         -> TIdent  (s, f i)
+  | TKRParam(s, i)         -> TKRParam(s, f i)
   | Tconstructorname(s, i) -> Tconstructorname  (s, f i)
   | TypedefIdent  (s, i)   -> TypedefIdent  (s, f i)
   | TInt  (s, i)           -> TInt  (s, f i)
@@ -382,6 +479,7 @@ let visitor_info_of_tok f = function
   | TDefine (i1) -> TDefine(f i1)
 
   | TUndef (i1) -> TUndef(f i1)
+  | TPragma (i1) -> TPragma(f i1)
   | TCppDirectiveOther (i1) -> TCppDirectiveOther(f i1)
 
   | TInclude (includes, filename, inifdef, i1) ->
@@ -428,14 +526,18 @@ let visitor_info_of_tok f = function
   | TCommentSkipTagStart         (i) -> TCommentSkipTagStart         (f i)
   | TCommentSkipTagEnd         (i) -> TCommentSkipTagEnd         (f i)
 
-  | TIfdef               (t, i) -> TIfdef               (t, f i)
+  | TIfdef               (c, t, i) -> TIfdef            (c, t, f i)
   | TIfdefelse           (t, i) -> TIfdefelse           (t, f i)
-  | TIfdefelif           (t, i) -> TIfdefelif           (t, f i)
+  | TIfdefelif           (c, t, i) -> TIfdefelif        (c, t, f i)
   | TEndif               (t, i) -> TEndif               (t, f i)
   | TIfdefBool           (b, t, i) -> TIfdefBool        (b, t, f i)
   | TIfdefMisc           (b, t, i) -> TIfdefMisc        (b, t, f i)
   | TIfdefVersion        (b, t, i) -> TIfdefVersion     (b, t, f i)
 
+  | TUifdef              (i) -> TUifdef              (f i)
+  | TUelseif             (i) -> TUelseif             (f i)
+  | TUendif              (i) -> TUendif              (f i)
+
   | TOPar                (i) -> TOPar                (f i)
   | TCPar                (i) -> TCPar                (f i)
   | TOBrace              (i) -> TOBrace              (f i)
@@ -471,6 +573,8 @@ let visitor_info_of_tok f = function
   | TMinus               (i) -> TMinus               (f i)
   | TMul                 (i) -> TMul                 (f i)
   | TDiv                 (i) -> TDiv                 (f i)
+  | TMin                 (i) -> TMin                 (f i)
+  | TMax                 (i) -> TMax                 (f i)
   | TMod                 (i) -> TMod                 (f i)
   | Tchar                (i) -> Tchar                (f i)
   | Tshort               (i) -> Tshort               (f i)
@@ -495,6 +599,8 @@ let visitor_info_of_tok f = function
 
   | Tstruct              (i) -> Tstruct              (f i)
   | Tenum                (i) -> Tenum                (f i)
+  | Tdecimal             (i) -> Tdecimal             (f i)
+  | Texec                (i) -> Texec                (f i)
   | Ttypedef             (i) -> Ttypedef             (f i)
   | Tunion               (i) -> Tunion               (f i)
   | Tbreak               (i) -> Tbreak               (f i)
@@ -519,6 +625,7 @@ let visitor_info_of_tok f = function
   | Tdelete              (i) -> Tdelete              (f i)
   | TOParCplusplusInit   (i) -> TOParCplusplusInit   (f i)
   | EOF                  (i) -> EOF                  (f i)
+  | Tnamespace           (i) -> Tnamespace           (f i)
 
 
 (*****************************************************************************)
@@ -536,6 +643,14 @@ let str_of_tok x =  Ast_c.str_of_info (info_of_tok x)
 let file_of_tok x = Ast_c.file_of_info (info_of_tok x)
 let pinfo_of_tok x = Ast_c.pinfo_of_info (info_of_tok x)
 
+(* for a comment, the end line is not the same as line_of_tok *)
+let end_line_of_tok = function
+    (TComment _) as t ->
+      let newlines =
+	List.length (Str.split_delim (Str.regexp "\n") (str_of_tok t)) - 1 in
+      line_of_tok t + newlines
+  | t -> line_of_tok t
+
 let is_origin x =
   match pinfo_of_tok x with Ast_c.OriginTok _ -> true | _ -> false
 let is_expanded x =
