@@ -71,7 +71,7 @@ module Lib = Lib_parsing_c
  * annotate_type.
  *
  * todo: how deal with typedef isomorphisms ? How store them in Ast_c ?
- * store all posible variations in ast_c ? a list of type instead of just
+ * store all possible variations in ast_c ? a list of type instead of just
  * the type ?
  *
  * todo: how to handle multiple possible definitions for entities like
@@ -133,7 +133,7 @@ let pr2, pr2_once = Common.mk_pr2_wrappers Flag_parsing_c.verbose_type
  * so don't need make difference between namespaces here.
  *
  * But, why not make simply a (string, kindstring) assoc ?
- * Because we dont want that a variable shadow a struct definition, because
+ * Because we don't want that a variable shadow a struct definition, because
  * they are still in 2 different namespace. But could for typedef,
  * because VarOrFunc and Typedef are in the same namespace.
  * But could do a record as in c_info.ml
@@ -164,59 +164,73 @@ type namedef =
   (* cppext: *)
   | Macro        of string * (define_kind * define_val)
 
+module StringMap = Map.Make (String)
+let singleton k v = StringMap.add k v StringMap.empty
 
-(* Because have nested scope, have nested list, hence the list list.
- *
- * opti? use a hash to accelerate ? hmm but may have some problems
- * with hash to handle recursive lookup. For instance for the typedef
- * example where have mutually recursive definition of the type,
- * we must take care to not loop by starting the second search
- * from the previous environment. With the list scheme in
- * lookup_env below it's quite easy to do. With hash it may be
- * more complicated.
-*)
-type environment = namedef list list
+(* Maps are used instead of lists in order to guarantee O(log(n))
+   complexity when doing a lookup. The case of typedefs is a bit
+   different from the others because we may want to do a second
+   search, using the environment as it was when the typedef first
+   searched was declared. *)
 
+type typedefs = { defs : (fullType * typedefs * int) StringMap.t }
+
+type nameenv = {
+    level : int;
+    var_or_func : Ast_c.exp_type StringMap.t;
+    enum_constant : string option StringMap.t;
+    typedef : typedefs;
+    struct_union_name_def : ((structUnion * structType) wrap) StringMap.t;
+    macro : (define_kind * define_val) StringMap.t
+  }
+
+type environment = nameenv list
 
 (* ------------------------------------------------------------ *)
 (* can be modified by the init_env function below, by
  * the file environment_unix.h
  *)
-let initial_env = ref [
-  [VarOrFunc("NULL",
-            (Lib.al_type (Parse_c.type_of_string "void *"),
-	    Ast_c.NotLocalVar));
 
+let empty_frame =
+  { level = 0;
+    var_or_func = StringMap.empty;
+    enum_constant = StringMap.empty;
+    typedef = { defs = StringMap.empty };
+    struct_union_name_def = StringMap.empty;
+    macro = StringMap.empty; }
+
+let initial_env = ref [
+  { empty_frame with
+    var_or_func =
+    singleton
+      "NULL"
+      (Lib.al_type (Parse_c.type_of_string "void *"),Ast_c.NotLocalVar) }
   (*
    VarOrFunc("malloc",
-            (Lib.al_type(Parse_c.type_of_string "void* (*)(int size)"),
+            (Lib.al_type(Parse_c.type_of_string "void* ( * )(int size)"),
 	    Ast_c.NotLocalVar));
    VarOrFunc("free",
-            (Lib.al_type(Parse_c.type_of_string "void (*)(void *ptr)"),
+            (Lib.al_type(Parse_c.type_of_string "void ( * )(void *ptr)"),
 	    Ast_c.NotLocalVar));
   *)
-  ]
 ]
 
+let _scoped_env = ref !initial_env
+let build_env prev level =
+  let rec ret = function
+    | [] -> assert false
+    | hd :: tl ->
+	if hd.level = level then
+	  { hd with typedef = prev }
+	else
+	  ret tl in
+  [ret !_scoped_env]
 
 let typedef_debug = ref false
 
 
 (* ------------------------------------------------------------ *)
 (* generic, lookup and also return remaining env for further lookup *)
-let rec lookup_env2 f env =
-  match env with
-  | [] -> raise Not_found
-  | []::zs -> lookup_env2 f zs
-  | (x::xs)::zs ->
-      (match f x with
-      | None -> lookup_env2 f (xs::zs)
-      | Some y -> y, xs::zs
-      )
-let lookup_env a b =
-  Common.profile_code "TAC.lookup_env" (fun () -> lookup_env2  a b)
-
-
 
 let member_env lookupf env =
   try
@@ -224,52 +238,61 @@ let member_env lookupf env =
     true
   with Not_found -> false
 
-
-
-
 (* ------------------------------------------------------------ *)
 
-
 let lookup_var s env =
-  let f = function
-    | VarOrFunc (s2, typ) -> if s2 =$= s then Some typ else None
-    | _ -> None
-  in
-  lookup_env f env
+  match env with
+  | [] -> raise Not_found
+  | env :: _ -> StringMap.find s env.var_or_func
 
-let lookup_typedef s env =
+let member_env_lookup_var s env =
+  match env with
+  | [] -> false
+  | env :: _ -> StringMap.mem s env.var_or_func
+
+let lookup_typedef (s : string) (env : environment) =
   if !typedef_debug then pr2 ("looking for: " ^ s);
-  let f = function
-    | TypeDef (s2, typ) -> if s2 =$= s then Some typ else None
-    | _ -> None
-  in
-  lookup_env f env
+  match env with
+  | [] -> raise Not_found
+  | env :: tl ->
+      let typ, prev, level = StringMap.find s env.typedef.defs in
+      let res : fullType * environment = typ, build_env prev level in
+      res
+
+let member_env_lookup_typedef (s : string) (env : environment) =
+  match env with
+  | [] -> false
+  | env :: _ -> StringMap.mem s env.typedef.defs
 
 let lookup_structunion (_su, s) env =
-  let f = function
-    | StructUnionNameDef (s2, typ) -> if s2 =$= s then Some typ else None
-    | _ -> None
-  in
-  lookup_env f env
+  match env with
+  | [] -> raise Not_found
+  | env :: _ -> StringMap.find s env.struct_union_name_def
+
+let member_env_lookup_structunion (_su, s) env =
+  match env with
+  | [] -> false
+  | env :: _ -> StringMap.mem s env.struct_union_name_def
 
 let lookup_macro s env =
-  let f = function
-    | Macro (s2, typ) -> if s2 =$= s then Some typ else None
-    | _ -> None
-  in
-  lookup_env f env
+  match env with
+  | [] -> raise Not_found
+  | env :: _ -> StringMap.find s env.macro
+
+let member_env_lookup_macro s env =
+  match env with
+  | [] -> false
+  | env :: _ -> StringMap.mem s env.macro
 
 let lookup_enum s env =
-  let f = function
-    | EnumConstant (s2, typ) -> if s2 =$= s then Some typ else None
-    | _ -> None
-  in
-  lookup_env f env
-
-
-let lookup_typedef a b =
-  Common.profile_code "TAC.lookup_typedef" (fun () -> lookup_typedef  a b)
+  match env with
+  | [] -> raise Not_found
+  | env :: _ -> StringMap.find s env.enum_constant
 
+let member_env_lookup_enum s env =
+  match env with
+  | [] -> false
+  | env :: _ -> StringMap.mem s env.enum_constant
 
 
 (*****************************************************************************)
@@ -334,12 +357,14 @@ let rec find_final_type ty env =
 
 (* ------------------------------------------------------------ *)
 let rec type_unfold_one_step ty env =
+  let rec loop seen ty env =
 
   match Ast_c.unwrap_typeC ty with
   | NoType        -> ty
   | BaseType x    -> ty
   | Pointer t     -> ty
   | Array (e, t)  -> ty
+  | Decimal (len,prec_opt) -> ty
 
   | StructUnion (sopt, su, fields) -> ty
 
@@ -350,7 +375,7 @@ let rec type_unfold_one_step ty env =
 
   | StructUnionName (su, s) ->
       (try
-          let (((su,fields),ii), env') = lookup_structunion (su, s) env in
+          let ((su,fields),ii) = lookup_structunion (su, s) env in
           Ast_c.mk_ty (StructUnion (su, Some s, fields)) ii
           (* old: +> Ast_c.rewrap_typeC ty
            * but must wrap with good ii, otherwise pretty_print_c
@@ -365,7 +390,9 @@ let rec type_unfold_one_step ty env =
       (try
           if !typedef_debug then pr2 "type_unfold_one_step: lookup_typedef";
           let (t', env') = lookup_typedef s env in
-          type_unfold_one_step t' env'
+	  if List.mem s seen (* avoid pb with recursive typedefs *)
+	  then type_unfold_one_step t' env'
+          else loop (s::seen) t' env
        with Not_found ->
           ty
       )
@@ -374,7 +401,8 @@ let rec type_unfold_one_step ty env =
   | TypeOfExpr e ->
       pr2_once ("Type_annoter: not handling typeof");
       ty
-  | TypeOfType t -> type_unfold_one_step t env
+  | TypeOfType t -> type_unfold_one_step t env in
+  loop [] ty env
 
 
 
@@ -391,60 +419,70 @@ let rec type_unfold_one_step ty env =
  * for most tasks.
  *)
 let rec typedef_fix ty env =
-  match Ast_c.unwrap_typeC ty with
-  | NoType  ->
-      ty
-  | BaseType x  ->
-      ty
-  | Pointer t ->
-      Pointer (typedef_fix t env)  +> Ast_c.rewrap_typeC ty
-  | Array (e, t) ->
-      Array (e, typedef_fix t env) +> Ast_c.rewrap_typeC ty
-  | StructUnion (su, sopt, fields) ->
+  let rec loop seen ty env =
+    match Ast_c.unwrap_typeC ty with
+    | NoType  ->
+	ty
+    | BaseType x  ->
+	ty
+    | Pointer t ->
+	Pointer (typedef_fix t env)  +> Ast_c.rewrap_typeC ty
+    | Array (e, t) ->
+	Array (e, typedef_fix t env) +> Ast_c.rewrap_typeC ty
+    | StructUnion (su, sopt, fields) ->
       (* normalize, fold.
-       * todo? but what if correspond to a nested struct def ?
-       *)
-      Type_c.structdef_to_struct_name ty
-  | FunctionType ft ->
-      (FunctionType ft) (* todo ? *) +> Ast_c.rewrap_typeC ty
-  | Enum  (s, enumt) ->
-      (Enum  (s, enumt)) (* todo? *) +> Ast_c.rewrap_typeC ty
-  | EnumName s ->
-      (EnumName s) (* todo? *) +> Ast_c.rewrap_typeC ty
-
+	 * todo? but what if correspond to a nested struct def ?
+      *)
+	Type_c.structdef_to_struct_name ty
+    | FunctionType ft ->
+	(FunctionType ft) (* todo ? *) +> Ast_c.rewrap_typeC ty
+    | Enum  (s, enumt) ->
+	(Enum  (s, enumt)) (* todo? *) +> Ast_c.rewrap_typeC ty
+    | EnumName s ->
+	(EnumName s) (* todo? *) +> Ast_c.rewrap_typeC ty
+    | Decimal(l,p) ->
+	(Decimal(l,p)) (* todo? *) +> Ast_c.rewrap_typeC ty
+	  
   (* we prefer StructUnionName to StructUnion when it comes to typed metavar *)
-  | StructUnionName (su, s) -> ty
-
+    | StructUnionName (su, s) ->
+	ty
+	  
   (* keep the typename but complete with more information *)
-  | TypeName (name, typ) ->
-      let s = Ast_c.str_of_name name in
-      (match typ with
-      | Some _ ->
-          pr2 ("typedef value already there:" ^ s);
-          ty
-      | None ->
-        (try
-          if !typedef_debug then pr2 "typedef_fix: lookup_typedef";
-          let (t', env') = lookup_typedef s env in
-
+    | TypeName (name, typ) ->
+	let s = Ast_c.str_of_name name in
+	(match typ with
+	| Some _ ->
+            pr2 ("typedef value already there:" ^ s);
+            ty
+	| None ->
+            (try
+              if !typedef_debug then pr2 "typedef_fix: lookup_typedef";
+              let (t', env') = lookup_typedef s env in
+	      
           (* bugfix: termination bug if use env instead of env' below, because
-           * can have some weird mutually recursive typedef which
-           * each new type alias search for its mutual def.
-           *)
-          TypeName (name, Some (typedef_fix t' env')) +> Ast_c.rewrap_typeC ty
-        with Not_found ->
-          ty
-      ))
+             * can have some weird mutually recursive typedef which
+             * each new type alias search for its mutual def.
+	     * seen is an attempt to do better.
+          *)
+	      let fixed =
+		if List.mem s seen
+		then loop (s::seen) t' env
+		else typedef_fix t' env' in
+	      TypeName (name, Some fixed) +>
+	      Ast_c.rewrap_typeC ty
+            with Not_found ->
+              ty))
 
   (* remove paren for better matching with typed metavar. kind of iso again *)
-  | ParenType t ->
-      typedef_fix t env
-  | TypeOfExpr e ->
-      pr2_once ("Type_annoter: not handling typeof");
-      ty
+    | ParenType t ->
+	typedef_fix t env
+    | TypeOfExpr e ->
+	pr2_once ("Type_annoter: not handling typeof");
+	ty
 
-  | TypeOfType t ->
-      typedef_fix t env
+    | TypeOfType t ->
+	typedef_fix t env in
+  loop [] ty env
 
 
 (*****************************************************************************)
@@ -495,14 +533,18 @@ let rec is_simple_expr expr =
 (* (Semi) Globals, Julia's style *)
 (*****************************************************************************)
 
-(* opti: cache ? use hash ? *)
-let _scoped_env = ref !initial_env
 
 (* memoise unnanoted var, to avoid too much warning messages *)
-let _notyped_var = ref (Hashtbl.create 100)
+let _notyped_var = ref (Hashtbl.create 101)
 
-let new_scope() = _scoped_env := []::!_scoped_env
-let del_scope() = _scoped_env := List.tl !_scoped_env
+let new_scope() =
+  match !_scoped_env with
+  | hd :: _ ->
+      _scoped_env := { hd with level = succ hd.level; } :: !_scoped_env
+  | [] ->
+      _scoped_env := [ empty_frame ]
+let del_scope() =
+  _scoped_env := List.tl !_scoped_env
 
 let do_in_new_scope f =
   begin
@@ -512,10 +554,27 @@ let do_in_new_scope f =
     res
   end
 
+(* this is not functional at all, so why not use a hash table? *)
 let add_in_scope namedef =
   let (current, older) = Common.uncons !_scoped_env in
-  _scoped_env := (namedef::current)::older
-
+  let current =
+    match namedef with
+      | VarOrFunc (s, typ) ->
+	  {current with
+	   var_or_func = StringMap.add s typ current.var_or_func}
+      | TypeDef   (s, typ) ->
+	  let v = typ, current.typedef, current.level in
+	  let new_typedef : typedefs = { defs = StringMap.add s v current.typedef.defs } in
+	  {current with typedef = new_typedef}
+      | StructUnionNameDef (s, (su, typ)) ->
+	  {current with
+	   struct_union_name_def = StringMap.add s (su, typ) current.struct_union_name_def}
+      | Macro (s, body) ->
+	  {current with macro = StringMap.add s body current.macro}
+      | EnumConstant (s, body) ->
+	  {current with
+	   enum_constant = StringMap.add s body current.enum_constant} in
+  _scoped_env := current::older
 
 (* ------------------------------------------------------------ *)
 
@@ -550,21 +609,26 @@ let add_binding2 namedef warning =
     | _ -> ()
     );
 
-    let (memberf, s) =
+    let (member, s) =
+      let env = [current_scope] in
       (match namedef with
       | VarOrFunc (s, typ) ->
-          member_env (lookup_var s), s
+          (* XXX do not define member_env_lookup_var,
+	     call "ignore (lookup_var ...)" and return
+	     - false if a Not_found exception is raised;
+	     - true otherwise *)
+          member_env_lookup_var s env, s
       | TypeDef   (s, typ) ->
-          member_env (lookup_typedef s), s
+          member_env_lookup_typedef s env, s
       | StructUnionNameDef (s, (su, typ)) ->
-          member_env (lookup_structunion (su, s)), s
+          member_env_lookup_structunion (su, s) env, s
       | Macro (s, body) ->
-          member_env (lookup_macro s), s
+          member_env_lookup_macro s env, s
       | EnumConstant (s, body) ->
-          member_env (lookup_enum s), s
+          member_env_lookup_enum s env, s
       ) in
 
-    if  memberf [current_scope] && warning
+    if member && warning
     then pr2 ("Type_annoter: warning, " ^ s ^
                  " is already in current binding" ^ "\n" ^
                  " so there is a weird shadowing");
@@ -585,11 +649,9 @@ let lookup_opt_env lookupf s =
     lookupf s !_scoped_env
   )
 
-let unwrap_unfold_env2 typ =
+let unwrap_unfold_env typ =
   Ast_c.unwrap_typeC
     (type_unfold_one_step typ !_scoped_env)
-let unwrap_unfold_env typ =
-  Common.profile_code "TAC.unwrap_unfold_env" (fun () -> unwrap_unfold_env2 typ)
 
 let typedef_fix a b =
   Common.profile_code "TAC.typedef_fix" (fun () -> typedef_fix a b)
@@ -614,6 +676,7 @@ let annotater_expr_visitor_subpart = (fun (k,bigf) expr ->
 
     (* -------------------------------------------------- *)
     (* todo: should analyse the 's' for int to know if unsigned or not *)
+    | StringConstant (s,os,kind) -> make_info_def (type_of_s "char []")
     | Constant (String (s,kind)) -> make_info_def (type_of_s "char []")
     | Constant MultiString _  -> make_info_def (type_of_s "char []")
     | Constant (Char   (s,kind)) -> make_info_def (type_of_s "char")
@@ -635,6 +698,19 @@ let annotater_expr_visitor_subpart = (fun (k,bigf) expr ->
         let fake = Ast_c.rewrap_str "float" fake in
         let iinull = [fake] in
         make_info_def (Ast_c.mk_ty (BaseType (FloatType kind)) iinull)
+    | Constant (DecimalConst(s,n,p)) ->
+        let fake = Ast_c.fakeInfo (Common.fake_parse_info) in
+        let fake1 = Ast_c.rewrap_str "decimal" fake in
+        let fake2 = Ast_c.rewrap_str "(" fake in
+        let fake3 = Ast_c.rewrap_str "," fake in
+        let fake4 = Ast_c.rewrap_str ")" fake in
+        let iinull = [fake1;fake2;fake3;fake4] in
+        let faken = Ast_c.rewrap_str n fake in
+        let fakep = Ast_c.rewrap_str p fake in
+	let sign = Ast_c.Si(Ast_c.Signed,CInt) in
+	let n = mk_e(Ast_c.Constant(Ast_c.Int (n, sign))) [faken] in
+	let p = mk_e(Ast_c.Constant(Ast_c.Int (p, sign))) [fakep] in
+	make_info_def (Ast_c.mk_ty (Decimal(n,Some p)) iinull)
 
 
     (* -------------------------------------------------- *)
@@ -657,7 +733,7 @@ let annotater_expr_visitor_subpart = (fun (k,bigf) expr ->
         );
         let s = Ast_c.str_of_name ident in
         (match lookup_opt_env lookup_var s with
-        | Some ((typ,local),_nextenv) ->
+        | Some (typ,local) ->
 
             (* set type for ident *)
             let tyinfo = make_info_fix (typ, local) in
@@ -679,7 +755,7 @@ let annotater_expr_visitor_subpart = (fun (k,bigf) expr ->
         | None  ->
 
             (match lookup_opt_env lookup_macro s with
-            | Some ((defkind, defval), _nextenv) ->
+            | Some (defkind, defval) ->
                 (match defkind, defval with
                 | DefineFunc _, DefineExpr e ->
                     let rettype = Ast_c.get_onlytype_expr e in
@@ -740,11 +816,10 @@ let annotater_expr_visitor_subpart = (fun (k,bigf) expr ->
     | Ident (ident) ->
         let s = Ast_c.str_of_name ident in
         (match lookup_opt_env lookup_var s with
-        | Some ((typ,local),_nextenv) ->
-            make_info_fix (typ,local)
+        | Some (typ,local) -> make_info_fix (typ,local)
         | None  ->
             (match lookup_opt_env lookup_macro s with
-            | Some ((defkind, defval), _nextenv) ->
+            | Some (defkind, defval) ->
                 (match defkind, defval with
                 | DefineVar, DefineExpr e ->
                     Ast_c.get_type_expr e
@@ -761,7 +836,7 @@ let annotater_expr_visitor_subpart = (fun (k,bigf) expr ->
                 )
             | None ->
                 (match lookup_opt_env lookup_enum s with
-                | Some (_, _nextenv) ->
+                | Some _ ->
                     make_info_def (type_of_s "int")
                 | None ->
                     if not (s =~ "[A-Z_]+") (* if macro then no warning *)
@@ -811,12 +886,10 @@ let annotater_expr_visitor_subpart = (fun (k,bigf) expr ->
           make_info_def_fix ft
         )
 
-
     (* -------------------------------------------------- *)
     (* fields *)
     | RecordAccess  (e, namefld)
     | RecordPtAccess (e, namefld) as x ->
-
         let fld = Ast_c.str_of_name namefld in
 
         k expr; (* recurse to set the types-ref of sub expressions *)
@@ -831,7 +904,7 @@ let annotater_expr_visitor_subpart = (fun (k,bigf) expr ->
                 | Pointer (t) -> Some t
                 | _ -> None
                 )
-            | _ -> raise Impossible
+            | _ -> raise (Impossible 159)
 
           in
           (match topt with
@@ -853,7 +926,8 @@ let annotater_expr_visitor_subpart = (fun (k,bigf) expr ->
                         pr2 "TAC:MultiFound";
                         Type_c.noTypeHere
                   )
-              | _ -> Type_c.noTypeHere
+              | _ ->
+		  Type_c.noTypeHere
           )
         )
 
@@ -878,12 +952,12 @@ let annotater_expr_visitor_subpart = (fun (k,bigf) expr ->
         k expr;
         Ast_c.get_type_expr e2
 
-    | Binary (e1, Logical _, e2) ->
+    | Binary (e1, ((Logical _),_), e2) ->
         k expr;
         make_info_def (type_of_s "int")
 
     (* todo: lub *)
-    | Binary (e1, Arith op, e2) ->
+    | Binary (e1, (Arith op, _), e2) ->
         k expr;
         Type_c.lub op (Type_c.get_opt_type e1) (Type_c.get_opt_type e2)
 
@@ -940,7 +1014,7 @@ let annotater_expr_visitor_subpart = (fun (k,bigf) expr ->
             | _ -> k expr; Type_c.noTypeHere
           *)
 
-    | New ty ->
+    | New (_, ty) ->
 	k expr;
 	pr2_once "Type annotater:not handling New";
 	Type_c.noTypeHere (* TODO *)
@@ -984,7 +1058,7 @@ let annotater_expr_visitor_subpart = (fun (k,bigf) expr ->
  * Because There would be some copy paste with annotate_program, it is
  * better to factorize code hence the just_add_in_env parameter below.
  *
- * todo? alternative optimisation for the include problem:
+ * todo? alternative optimization for the include problem:
  *  - processing all headers files one time and construct big env
  *  - use hashtbl for env (but apparently not biggest problem)
  *)
@@ -1045,7 +1119,13 @@ let rec visit_toplevel ~just_add_in_env ~depth elem =
 
           add_binding (Macro (s, (defkind, defval) )) true;
 
-      | PragmaAndCo _ -> ()
+      |	Pragma((s,ii), pragmainfo) ->
+	  (match pragmainfo with
+	    PragmaTuple(args,ii) ->
+              args +> List.iter (fun (e,ii) -> Visitor_c.vk_argument bigf e)
+	  | PragmaIdList _ -> ())
+
+      | OtherDirective _ -> ()
     );
 
     (* ------------------------------------------------------------ *)
@@ -1072,13 +1152,36 @@ let rec visit_toplevel ~just_add_in_env ~depth elem =
 
 
 	    let local =
-	      match local with
-	      | Ast_c.NotLocalDecl -> Ast_c.NotLocalVar
-	      |	Ast_c.LocalDecl -> Ast_c.LocalVar (Ast_c.info_of_type t)
+	      match (sto,local) with
+	      | (_,Ast_c.NotLocalDecl) -> Ast_c.NotLocalVar
+	      |	((Ast_c.Sto Ast_c.Static, _), Ast_c.LocalDecl) ->
+		  (match Ast_c.info_of_type t with
+		    (* if there is no info about the type it must not be
+		       present, so we don't know what the variable is *)
+		    None -> Ast_c.NotLocalVar
+		  | Some ii -> Ast_c.StaticLocalVar ii)
+	      |	(_,Ast_c.LocalDecl) ->
+		  (match Ast_c.info_of_type t with
+		    (* if there is no info about the type it must not be
+		       present, so we don't know what the variable is *)
+		    None -> Ast_c.NotLocalVar
+		  | Some ii -> Ast_c.LocalVar ii)
             in
             var +> Common.do_option (fun (name, iniopt) ->
               let s = Ast_c.str_of_name name in
 
+	      let t =
+		match Ast_c.unwrap_typeC t with
+		| Ast_c.Decimal (len,None) ->
+		    let newp =
+		      Ast_c.rewrap_expr len
+			 (Ast_c.Constant
+			    (Ast_c.Int
+			       ("0",Ast_c.Si(Ast_c.Signed,Ast_c.CInt)))) in
+		    Ast_c.rewrap_typeC t (Ast_c.Decimal (len,Some newp))
+		| _ -> t in
+		
+
               match sto with
               | StoTypedef, _inline ->
                   add_binding (TypeDef (s,Lib.al_type t)) true;
@@ -1100,7 +1203,7 @@ let rec visit_toplevel ~just_add_in_env ~depth elem =
                   end
             );
           );
-      | MacroDecl _ ->
+      | MacroDecl _ | MacroDeclInit _ ->
           if need_annotate_body
           then k d
       );
@@ -1169,7 +1272,7 @@ let rec visit_toplevel ~just_add_in_env ~depth elem =
                *)
             | iifunc1::iifunc2::ibrace1::ibrace2::ifakestart::isto ->
                 iifunc1, iifunc2
-            | _ -> raise Impossible
+            | _ -> raise (Impossible 160)
           in
           let funcs = Ast_c.str_of_name name in
 
@@ -1187,7 +1290,14 @@ let rec visit_toplevel ~just_add_in_env ~depth elem =
                     match nameopt with
                     | Some name ->
                         let s = Ast_c.str_of_name name in
-		        let local = Ast_c.LocalVar (Ast_c.info_of_type t) in
+		        let local =
+			  (match Ast_c.info_of_type t with
+			    (* if there is no info about the type it must
+			       not be present, so we don't know what the
+			       variable is *)
+			    None -> Ast_c.NotLocalVar
+			  | Some ii -> Ast_c.LocalVar ii)
+			in
 		        add_binding (VarOrFunc (s,(Lib.al_type t,local))) true
                     | None ->
                     pr2 "no type, certainly because Void type ?"
@@ -1225,6 +1335,7 @@ let rec visit_toplevel ~just_add_in_env ~depth elem =
       | EmptyDef _
       | NotParsedCorrectly _
       | FinalDef _
+      | Namespace _
           ->
           k elem
     );
@@ -1275,8 +1386,8 @@ let annotate_test_expressions prog =
     let (ty,_) = !info in
     info := (ty,Test);
     match e_term with
-      Binary(e1,Logical(AndLog),e2)
-    | Binary(e1,Logical(OrLog),e2) -> propagate_test e1; propagate_test e2
+      Binary(e1,(Logical AndLog,_),e2)
+    | Binary(e1,(Logical OrLog,_),e2) -> propagate_test e1; propagate_test e2
     | Unary(e1,Not) -> propagate_test e1
     | ParenExpr(e) -> propagate_test e
     | FunCall(e,args) -> (* not very nice, but so painful otherwise *)
@@ -1293,8 +1404,8 @@ let annotate_test_expressions prog =
     Visitor_c.kexpr = (fun (k,bigf) expr ->
       (match unwrap_expr expr with
 	CondExpr(e,_,_) -> propagate_test e
-      |	Binary(e1,Logical(AndLog),e2)
-      | Binary(e1,Logical(OrLog),e2) -> propagate_test e1; propagate_test e2
+      |	Binary(e1,(Logical AndLog,_),e2)
+      | Binary(e1,(Logical OrLog,_),e2) -> propagate_test e1; propagate_test e2
       | Unary(e1,Not) -> propagate_test e1
       | _ -> ()
       );
@@ -1336,19 +1447,3 @@ let annotate_program env prog =
 let annotate_type_and_localvar env prog =
   Common.profile_code "TAC.annotate_type"
     (fun () -> annotate_program2 env prog)
-
-
-(*****************************************************************************)
-(* changing default typing environment, do concatenation *)
-let init_env filename =
-  pr2 ("init_env: " ^ filename);
-  let (ast2, _stat) = Parse_c.parse_c_and_cpp filename in
-  let ast = Parse_c.program_of_program2 ast2 in
-
-  let res = annotate_type_and_localvar !initial_env ast in
-  match List.rev res with
-  | [] -> pr2 "empty environment"
-  | (_top,(env1,env2))::xs ->
-      initial_env := !initial_env ++ env2;
-      ()
-
