@@ -93,7 +93,11 @@ let get_function_name rule env =
       [] any_list_list in
   let mcode r mc =
     match Ast.get_mcodekind mc with
-      Ast.MINUS(_,_,_,any_list_list) -> do_any_list_list r any_list_list
+      Ast.MINUS(_,_,_,any_list_list) ->
+	(match any_list_list with
+	  Ast.NOREPLACEMENT -> []
+	| Ast.REPLACEMENT(any_list_list,_) ->
+	    do_any_list_list r any_list_list)
     | Ast.CONTEXT(_,any_befaft) ->
 	(match any_befaft with
 	  Ast.BEFORE(any_list_list,_) | Ast.AFTER(any_list_list,_) ->
@@ -127,9 +131,11 @@ let get_function_name rule env =
     | _ -> []) in
   let names =
     (V.combiner bind option_default
-      mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
+      mcode mcode mcode mcode mcode mcode mcode mcode mcode
+      mcode mcode mcode mcode mcode
       donothing donothing donothing donothing donothing
       donothing expression donothing donothing donothing donothing donothing
+      donothing donothing donothing donothing donothing
       donothing donothing donothing donothing donothing).V.combiner_top_level
       rule in
   match names with
@@ -226,6 +232,7 @@ let print_extra_typedefs pr env =
     (function (_,vl) ->
       match vl with
 	Ast_c.MetaIdVal(_) | Ast_c.MetaFuncVal(_)
+      | Ast_c.MetaAssignOpVal(_) | Ast_c.MetaBinaryOpVal(_)
       | Ast_c.MetaLocalFuncVal(_) -> ()
       | Ast_c.MetaExprVal(exp,_) -> Visitor_c.vk_expr bigf exp
       | Ast_c.MetaExprListVal(args) -> Visitor_c.vk_argument_list bigf args
@@ -234,9 +241,14 @@ let print_extra_typedefs pr env =
 
       | Ast_c.MetaTypeVal(ty) -> Visitor_c.vk_type bigf ty
       | Ast_c.MetaInitVal(ty) -> Visitor_c.vk_ini bigf ty
+      | Ast_c.MetaInitListVal(ty) -> Visitor_c.vk_ini_list bigf ty
       | Ast_c.MetaDeclVal(decl) -> Visitor_c.vk_decl bigf decl
       | Ast_c.MetaFieldVal(field) -> Visitor_c.vk_struct_field bigf field
-      | Ast_c.MetaFieldListVal(fields) -> Visitor_c.vk_struct_fields bigf fields
+      | Ast_c.MetaFieldListVal(fields) ->
+	  Visitor_c.vk_struct_fields bigf fields
+      | Ast_c.MetaFmtVal(fmt) -> Visitor_c.vk_string_format bigf fmt
+      | Ast_c.MetaFragListVal(frags) ->
+	  Visitor_c.vk_string_fragments bigf frags
       | Ast_c.MetaStmtVal(stm) -> Visitor_c.vk_statement bigf stm
       | Ast_c.MetaPosVal _ | Ast_c.MetaPosValList _
       | Ast_c.MetaListlenVal _ -> ())
@@ -266,6 +278,7 @@ let rename argids env =
       (x,
        match vl with
 	 Ast_c.MetaIdVal(_) | Ast_c.MetaFuncVal(_)
+       | Ast_c.MetaAssignOpVal(_) | Ast_c.MetaBinaryOpVal(_)
        | Ast_c.MetaLocalFuncVal(_) -> vl
        | Ast_c.MetaExprVal(exp,c) ->
 	   Ast_c.MetaExprVal(Visitor_c.vk_expr_s bigf exp,c)
@@ -280,12 +293,18 @@ let rename argids env =
 	   Ast_c.MetaTypeVal(Visitor_c.vk_type_s bigf ty)
        | Ast_c.MetaInitVal(ini) ->
 	   Ast_c.MetaInitVal(Visitor_c.vk_ini_s bigf ini)
+       | Ast_c.MetaInitListVal(ini) ->
+	   Ast_c.MetaInitListVal(Visitor_c.vk_inis_s bigf ini)
        | Ast_c.MetaDeclVal(stm) ->
 	   Ast_c.MetaDeclVal(Visitor_c.vk_decl_s bigf stm)
        | Ast_c.MetaFieldVal(stm) ->
 	   Ast_c.MetaFieldVal(Visitor_c.vk_struct_field_s bigf stm)
        | Ast_c.MetaFieldListVal(stm) ->
 	   Ast_c.MetaFieldListVal(Visitor_c.vk_struct_fields_s bigf stm)
+       | Ast_c.MetaFmtVal(fmt) ->
+	   Ast_c.MetaFmtVal(Visitor_c.vk_string_format_s bigf fmt)
+       | Ast_c.MetaFragListVal(frags) ->
+	   Ast_c.MetaFragListVal(Visitor_c.vk_string_fragments_s bigf frags)
        | Ast_c.MetaStmtVal(stm) ->
 	   Ast_c.MetaStmtVal(Visitor_c.vk_statement_s bigf stm)
        | Ast_c.MetaPosVal _ | Ast_c.MetaPosValList _
@@ -341,11 +360,18 @@ let pp_meta_decl pr env decl =
       no_arity ar; pr "type "; pp_name name; pr ";\n"
   | Ast.MetaInitDecl(ar, name) ->
       no_arity ar; pr "initialiser "; pp_name name; pr ";\n"
+  | Ast.MetaInitListDecl(ar, name, len) ->
+      no_arity ar; pr "initialiser list "; pp_name name; pp_len pr len;
+      pr ";\n"
   | Ast.MetaListlenDecl(name) -> ()
   | Ast.MetaParamDecl(ar, name) ->
       no_arity ar; pr "parameter "; pp_name name; pr ";\n"
   | Ast.MetaParamListDecl(ar, name, len) ->
       no_arity ar; pr "parameter list "; pp_name name; pp_len pr len; pr ";\n"
+  | Ast.MetaBinaryOperatorDecl(ar, name) ->
+      no_arity ar; pr "binary operator "; pp_name name; pr ";\n"
+  | Ast.MetaAssignmentOperatorDecl(ar, name) ->
+      no_arity ar; pr "assignment operator "; pp_name name; pr ";\n"
   | Ast.MetaConstDecl(ar, name, types) ->
       no_arity ar; pr "constant "; print_types pr env types;
       pp_name name; pr ";\n"
@@ -361,6 +387,9 @@ let pp_meta_decl pr env decl =
   | Ast.MetaLocalIdExpDecl(ar, name, types) ->
       no_arity ar; pr "local idexpression ";
       print_types pr env types; pp_name name; pr ";\n"
+  | Ast.MetaGlobalIdExpDecl(ar, name, types) ->
+      no_arity ar; pr "global idexpression ";
+      print_types pr env types; pp_name name; pr ";\n"
   | Ast.MetaExpListDecl(ar, name, len) ->
       no_arity ar; pr "parameter list "; pp_name name; pp_len pr len; pr ";\n"
   | Ast.MetaDeclDecl(ar, name) ->
@@ -379,6 +408,12 @@ let pp_meta_decl pr env decl =
       no_arity ar; pr "local function "; pp_name name; pr ";\n"
   | Ast.MetaPosDecl(ar, name) ->
       no_arity ar; pr "position "; pp_name name; pr ";\n"
+  | Ast.MetaFmtDecl(ar, name) ->
+      no_arity ar; pr "format "; pp_name name; pr ";\n"
+  | Ast.MetaFragListDecl(ar, name, len) ->
+      no_arity ar; pr "format list "; pp_name name; pp_len pr len; pr ";\n"
+  | Ast.MetaAnalysisDecl(code, name) ->
+      pr "analysis"; pr code; pr " "; pp_name name; pr ";\n"
   | Ast.MetaDeclarerDecl(ar, name) ->
       no_arity ar; pr "declarer "; pp_name name; pr ";\n"
   | Ast.MetaIteratorDecl(ar, name) ->
@@ -439,7 +474,7 @@ let pp_rule local_metas ast env srcfile =
       Ast.CocciRule(_,_,[body],_,_) -> (* could extend to use attributes *)
 	(body,
 	 match Ast.unwrap body with
-	   Ast.DECL(s) -> [[Ast.StatementTag s]]
+	   Ast.NONDECL(s) -> [[Ast.StatementTag s]]
 	 | Ast.CODE(ss) -> [[Ast.StmtDotsTag ss]]
 	 | _ -> error body "bad rule body")
     | _ -> failwith "bad rule" in
@@ -493,12 +528,13 @@ let pp_rule local_metas ast env srcfile =
 	Ast_c.AbstractLineTok _ -> pr (Ast_c.str_of_info info)
       | Ast_c.FakeTok (s,_) -> pr s
       |	_ ->
-	  Printf.printf "line: %s\n" (Common.dump info);
+	  Printf.printf "line: %s\n" (Dumper.dump info);
 	  error rule "not an abstract line" in
     let pr_space _ = pr " " in
     Unparse_cocci.pp_list_list_any
       ([env], (fun s _ _ _ _ -> pr s), pr_c, pr_space, pr_space, pr,
-       (fun _ _ -> ()), (function _ -> ()), (function _ -> ()))
+       (fun _ _ -> ()), (function _ -> ()), (function _ -> ()),
+       (function _ -> ()))
       true printable Unparse_cocci.InPlace;
     print_end pr;
     pr "\n")
