@@ -22,7 +22,7 @@ module F = Control_flow_c
 (* Prelude *)
 (*****************************************************************************)
 
-(* todo? dont go in Include. Have a visitor flag ? disable_go_include ?
+(* todo? don't go in Include. Have a visitor flag ? disable_go_include ?
  * disable_go_type_annotation ?
  *)
 
@@ -124,7 +124,7 @@ let pr2, pr2_once = Common.mk_pr2_wrappers Flag_parsing_c.verbose_visit
  *      end in analysis#expr
  *
  * The problem is that you don't have control about what is generated
- * and in our case we sometimes dont want to visit too much. For instance
+ * and in our case we sometimes don't want to visit too much. For instance
  * our visitor don't recurse on the type annotation of expressions
  * Ok, this could be worked around, but the pb remains, you
  * don't have control and at some point you may want. In the same
@@ -189,12 +189,12 @@ let (iter_expr:((expression -> unit) -> expression -> unit) -> expression -> uni
     | FunCall  (e, es)         ->  f k e; List.iter (f k) es
     | CondExpr (e1, e2, e3)    -> f k e1; f k e2; f k e3
     | Sequence (e1, e2)        -> f k e1; f k e2;
-    | Assignment (e1, op, e2)  -> f k e1; f k e2;
+    | Assignment (e1, op, e2)  -> f k e1; vk_asignOp k op; f k e2;
 
     | Postfix  (e, op) -> f k e
     | Infix    (e, op) -> f k e
     | Unary    (e, op) -> f k e
-    | Binary   (e1, op, e2) -> f k e1; f k  e2;
+    | Binary   (e1, op, e2) -> f k e1; vk_binaryOp k op; f k  e2;
 
     | ArrayAccess    (e1, e2) -> f k e1; f k e2;
     | RecordAccess   (e, s) -> f k e
@@ -234,6 +234,8 @@ let test =
 type visitor_c =
  {
    kexpr:      (expression  -> unit) * visitor_c -> expression  -> unit;
+   kassignOp:  (assignOp    -> unit) * visitor_c -> assignOp    -> unit;
+   kbinaryOp:  (binaryOp    -> unit) * visitor_c -> binaryOp    -> unit;
    kstatement: (statement   -> unit) * visitor_c -> statement   -> unit;
    ktype:      (fullType    -> unit) * visitor_c -> fullType    -> unit;
 
@@ -247,6 +249,7 @@ type visitor_c =
    kfield: (field -> unit) * visitor_c -> field -> unit;
 
    kcppdirective: (cpp_directive -> unit) * visitor_c -> cpp_directive -> unit;
+   kifdefdirective : (ifdef_directive -> unit) * visitor_c -> ifdef_directive -> unit;
    kdefineval : (define_val -> unit) * visitor_c -> define_val -> unit;
    kstatementseq: (statement_sequencable   -> unit) * visitor_c -> statement_sequencable   -> unit;
 
@@ -255,27 +258,34 @@ type visitor_c =
    knode: (F.node -> unit) * visitor_c -> F.node -> unit;
    (* AST *)
    ktoplevel: (toplevel -> unit) * visitor_c -> toplevel -> unit;
+   kfragment: (string_fragment -> unit) * visitor_c -> string_fragment -> unit;
+   kformat: (string_format -> unit) * visitor_c -> string_format -> unit;
 
    kinfo: (info -> unit) * visitor_c -> info -> unit;
  }
 
 let default_visitor_c =
-  { kexpr         = (fun (k,_) e  -> k e);
-    kstatement    = (fun (k,_) st -> k st);
-    ktype         = (fun (k,_) t  -> k t);
-    kdecl         = (fun (k,_) d  -> k d);
-    konedecl      = (fun (k,_) d  -> k d);
-    kparam        = (fun (k,_) d  -> k d);
-    kdef          = (fun (k,_) d  -> k d);
-    kini          = (fun (k,_) ie  -> k ie);
-    kname         = (fun (k,_) x -> k x);
-    kinfo         = (fun (k,_) ii  -> k ii);
-    knode         = (fun (k,_) n  -> k n);
-    ktoplevel     = (fun (k,_) p  -> k p);
-    kcppdirective = (fun (k,_) p  -> k p);
-    kdefineval    = (fun (k,_) p  -> k p);
-    kstatementseq = (fun (k,_) p  -> k p);
-    kfield        = (fun (k,_) p  -> k p);
+  { kexpr           = (fun (k,_) e  -> k e);
+    kassignOp       = (fun (k,_) op -> k op);
+    kbinaryOp       = (fun (k,_) op -> k op);
+    kstatement      = (fun (k,_) st -> k st);
+    ktype           = (fun (k,_) t  -> k t);
+    kdecl           = (fun (k,_) d  -> k d);
+    konedecl        = (fun (k,_) d  -> k d);
+    kparam          = (fun (k,_) d  -> k d);
+    kdef            = (fun (k,_) d  -> k d);
+    kini            = (fun (k,_) ie  -> k ie);
+    kname           = (fun (k,_) x -> k x);
+    kfragment       = (fun (k,_) f  -> k f);
+    kformat         = (fun (k,_) f  -> k f);
+    kinfo           = (fun (k,_) ii  -> k ii);
+    knode           = (fun (k,_) n  -> k n);
+    ktoplevel       = (fun (k,_) p  -> k p);
+    kcppdirective   = (fun (k,_) p  -> k p);
+    kifdefdirective = (fun (k,_) p  -> k p);
+    kdefineval      = (fun (k,_) p  -> k p);
+    kstatementseq   = (fun (k,_) p  -> k p);
+    kfield          = (fun (k,_) p  -> k p);
   }
 
 
@@ -286,24 +296,25 @@ let rec vk_expr = fun bigf expr ->
   let iif ii = vk_ii bigf ii in
 
   let rec exprf e = bigf.kexpr (k,bigf) e
-  (* !!! dont go in _typ !!! *)
+  (* !!! don't go in _typ !!! *)
   and k ((e,_typ), ii) =
     iif ii;
     match e with
     | Ident (name) -> vk_name bigf name
     | Constant (c) -> ()
+    | StringConstant(s,os,w) -> vk_string_fragments bigf s
     | FunCall  (e, es)         ->
         exprf e;
         vk_argument_list bigf es;
     | CondExpr (e1, e2, e3)    ->
         exprf e1; do_option (exprf) e2; exprf e3
     | Sequence (e1, e2)        -> exprf e1; exprf e2;
-    | Assignment (e1, op, e2)  -> exprf e1; exprf e2;
+    | Assignment (e1, op, e2)  -> exprf e1; vk_assignOp bigf op; exprf e2;
 
     | Postfix  (e, op) -> exprf e
     | Infix    (e, op) -> exprf e
     | Unary    (e, op) -> exprf e
-    | Binary   (e1, op, e2) -> exprf e1; exprf  e2;
+    | Binary   (e1, op, e2) -> exprf e1; vk_binaryOp bigf op; exprf  e2;
 
     | ArrayAccess    (e1, e2) -> exprf e1; exprf e2;
     | RecordAccess   (e, name) -> exprf e; vk_name bigf name
@@ -321,21 +332,27 @@ let rec vk_expr = fun bigf expr ->
         iif is;
         statxs +> List.iter (vk_statement_sequencable bigf);
 
-    | Constructor (t, initxs) ->
-        vk_type bigf t;
-        initxs +> List.iter (fun (ini, ii) ->
-          vk_ini bigf ini;
-          vk_ii bigf ii;
-        )
+    | Constructor (t, init) ->
+        vk_type bigf t; vk_ini bigf init
 
     | ParenExpr (e) -> exprf e
 
-    | New  t   -> vk_argument bigf t
+    | New  (None, t)   -> vk_argument bigf t
+    | New  (Some ts, t)   ->
+        vk_argument_list bigf ts;
+	vk_argument bigf t
     | Delete e -> vk_expr bigf e
 
 
   in exprf expr
 
+and vk_assignOp = fun bigf (_,ii) ->
+  let iif ii = vk_ii bigf ii in
+  iif ii
+
+and vk_binaryOp = fun bigf (_,ii) ->
+  let iif ii = vk_ii bigf ii in
+  iif ii
 
 (* ------------------------------------------------------------------------ *)
 and vk_name = fun bigf ident ->
@@ -385,13 +402,24 @@ and vk_statement = fun bigf (st: Ast_c.statement) ->
 
     | Selection  (If (e, st1, st2)) ->
         vk_expr bigf e; statf st1; statf st2;
+    | Selection  (Ifdef_Ite (e, st1, st2)) ->
+        vk_expr bigf e;
+        statf st1;
+        statf st2;
+    | Selection  (Ifdef_Ite2 (e, st1, st2, st3)) ->
+        vk_expr bigf e;
+        statf st1;
+        statf st2;
+        statf st3;
     | Selection  (Switch (e, st)) ->
         vk_expr bigf e; statf st;
     | Iteration  (While (e, st)) ->
         vk_expr bigf e; statf st;
     | Iteration  (DoWhile (st, e)) -> statf st; vk_expr bigf e;
-    | Iteration  (For ((e1opt,i1), (e2opt,i2), (e3opt,i3), st)) ->
-        statf (mk_st (ExprStatement (e1opt)) i1);
+    | Iteration  (For (first, (e2opt,i2), (e3opt,i3), st)) ->
+	(match first with
+	  ForExp (e1opt,i1) -> statf (mk_st (ExprStatement (e1opt)) i1)
+	| ForDecl decl -> vk_decl bigf decl);
         statf (mk_st (ExprStatement (e2opt)) i2);
         statf (mk_st (ExprStatement (e3opt)) i3);
         statf st;
@@ -410,6 +438,8 @@ and vk_statement = fun bigf (st: Ast_c.statement) ->
     | NestedFunc def -> vk_def bigf def
     | MacroStmt -> ()
 
+    | Exec (code) -> List.iter (vk_exec_code bigf) code
+
   in statf st
 
 and vk_statement_sequencable = fun bigf stseq ->
@@ -449,6 +479,9 @@ and vk_type = fun bigf t ->
     | Array (eopt, t) ->
         do_option (vk_expr bigf) eopt;
         typef t
+    | Decimal(length,precision_opt) ->
+        vk_expr bigf length;
+        do_option (vk_expr bigf) precision_opt
     | FunctionType (returnt, paramst) ->
         typef returnt;
         (match paramst with
@@ -466,7 +499,7 @@ and vk_type = fun bigf t ->
     | StructUnionName (s, structunion) -> ()
     | EnumName  s -> ()
 
-    (* dont go in _typ *)
+    (* don't go in _typ *)
     | TypeName (name,_typ) ->
         vk_name bigf name
 
@@ -484,6 +517,12 @@ and vk_attribute = fun bigf attr ->
       iif ii
 
 
+and vk_exec_code = fun bigf e ->
+  let iif ii = vk_ii bigf ii in
+  match e with
+    ExecEval name, ii -> iif ii; vk_expr bigf name
+  | ExecToken, ii -> iif ii
+
 (* ------------------------------------------------------------------------ *)
 
 and vk_decl = fun bigf d ->
@@ -496,11 +535,15 @@ and vk_decl = fun bigf d ->
 	iif ii;
 	xs +> List.iter (fun (x,ii) ->
         iif ii;
-        vk_onedecl bigf x;
+        vk_onedecl bigf x
       );
-    | MacroDecl ((s, args),ii) ->
+    | MacroDecl ((s, args, ptvg),ii) ->
+        iif ii;
+        vk_argument_list bigf args
+    | MacroDeclInit ((s, args, ini),ii) ->
         iif ii;
         vk_argument_list bigf args;
+	vk_ini bigf ini
   in f (k, bigf) d
 
 and vk_decl_list = fun bigf ts ->
@@ -518,7 +561,7 @@ and vk_onedecl = fun bigf onedecl ->
       v_attr = attrs})  ->
 
     vk_type bigf t;
-    (* dont go in tbis *)
+    (* don't go in tbis *)
     attrs +> List.iter (vk_attribute bigf);
     var +> Common.do_option (fun (name, iniopt) ->
       vk_name bigf name;
@@ -553,6 +596,12 @@ and vk_ini = fun bigf ini ->
 
   in inif ini
 
+and vk_ini_list = fun bigf ts ->
+  let iif ii = vk_ii bigf ii in
+  ts +> List.iter (fun (ini,iicomma) ->
+    vk_ini bigf ini;
+    iif iicomma;
+  )
 
 and vk_designator = fun bigf design ->
   let iif ii = vk_ii bigf ii in
@@ -678,16 +727,31 @@ and vk_toplevel = fun bigf p ->
 
     | NotParsedCorrectly ii -> iif ii
     | FinalDef info -> vk_info bigf info
+
+    | Namespace (tls, ii) -> List.iter (vk_toplevel bigf) tls
   in f (k, bigf) p
 
 and vk_program = fun bigf xs ->
   xs +> List.iter (vk_toplevel bigf)
 
 and vk_ifdef_directive bigf directive =
-  let iif ii =  vk_ii bigf ii in
-  match directive with
-  | IfdefDirective (ifkind, ii) -> iif ii
+  let f = bigf.kifdefdirective in
+  let k d =
+    let iif ii =  vk_ii bigf ii in
+    match d with
+    | IfdefDirective ((ifkind,_tag), ii) ->
+        vk_ifdefkind bigf ifkind;
+        iif ii
+  in f (k, bigf) directive
 
+and vk_ifdefkind bigf = function
+  | Ifdef       ifguard
+  | IfdefElseif ifguard -> vk_ifdef_guard bigf ifguard
+  | x                   -> ()
+
+and vk_ifdef_guard bigf = function
+  | Gif e -> vk_expr bigf e
+  | x     -> ()
 
 and vk_cpp_directive bigf directive =
   let iif ii =  vk_ii bigf ii in
@@ -711,7 +775,10 @@ and vk_cpp_directive bigf directive =
         iif ii;
         vk_define_kind bigf defkind;
         vk_define_val bigf defval
-    | PragmaAndCo (ii) ->
+    | Pragma ((s,ii), pragmainfo) ->
+        iif ii;
+        vk_pragmainfo bigf pragmainfo
+    | OtherDirective (ii) ->
         iif ii
   in f (k, bigf) directive
 
@@ -744,14 +811,44 @@ and vk_define_val bigf defval =
   | DefineText (s, ii) -> vk_ii bigf ii
   | DefineEmpty -> ()
   | DefineInit ini -> vk_ini bigf ini
-
+    (* christia: added multi *)
+  | DefineMulti stmts ->
+      List.fold_left (fun () d -> vk_statement bigf d) () stmts
   | DefineTodo ->
       pr2_once "DefineTodo";
       ()
   in f (k, bigf) defval
 
+and vk_pragmainfo bigf pragmainfo =
+  match pragmainfo with
+    PragmaTuple(args,ii) ->
+      vk_ii bigf ii;
+      vk_argument_list bigf args
+  | PragmaIdList ids ->
+      ids +> List.iter (function (id, _) -> vk_name bigf id)
 
+and vk_string_fragment = fun bigf x ->
+  let rec fragf x = bigf.kfragment (k, bigf) x
+  and k st =
+    let (unwrap_x, ii) = x in
+    vk_ii bigf ii;
+    match unwrap_x with
+      ConstantFragment s -> ()
+    | FormatFragment(fmt) -> vk_string_format bigf fmt in
+  fragf x
 
+and vk_string_fragments = fun bigf ts ->
+  ts +> List.iter (vk_string_fragment bigf)
+
+and vk_string_format = fun bigf x ->
+  let rec fmtf x = bigf.kformat (k, bigf) x
+  and k st =
+    let (unwrap_x, ii) = x in
+    vk_ii bigf ii;
+    match unwrap_x with (* probably not very useful... *)
+      ConstantFormat s -> ()
+  in
+  fmtf x
 
 (* ------------------------------------------------------------------------ *)
 (* Now keep fullstatement inside the control flow node,
@@ -791,12 +888,18 @@ and vk_node = fun bigf node ->
         iif ii;
         vk_expr bigf e
 
-    | F.ForHeader (_st, (((e1opt,i1), (e2opt,i2), (e3opt,i3)), ii)) ->
+    | F.ForHeader (_st, ((ForExp (e1opt,i1), (e2opt,i2), (e3opt,i3)), ii)) ->
         iif i1; iif i2; iif i3;
         iif ii;
         e1opt +> do_option (vk_expr bigf);
         e2opt +> do_option (vk_expr bigf);
         e3opt +> do_option (vk_expr bigf);
+    | F.ForHeader (_st, ((ForDecl decl, (e2opt,i2), (e3opt,i3)), ii)) ->
+        iif i2; iif i3;
+        iif ii;
+        decl  +> (vk_decl bigf);
+        e2opt +> do_option (vk_expr bigf);
+        e3opt +> do_option (vk_expr bigf);
     | F.MacroIterHeader (_s, ((s,es), ii)) ->
         iif ii;
         vk_argument_list bigf es;
@@ -821,6 +924,10 @@ and vk_node = fun bigf node ->
         pr2_once "DefineTodo";
         ()
 
+    | F.PragmaHeader((s,ii), pragmainfo) ->
+        iif ii;
+        vk_pragmainfo bigf pragmainfo
+
     | F.Include {i_include = (s, ii);} -> iif ii;
 
     | F.MacroTop (s, args, ii) ->
@@ -831,7 +938,9 @@ and vk_node = fun bigf node ->
     | F.IfdefElse    (info) ->  vk_ifdef_directive bigf info
     | F.IfdefEndif    (info) ->  vk_ifdef_directive bigf info
 
-    | F.Break    (st,((),ii)) -> iif ii
+    | F.IfdefIteHeader ii -> iif ii
+
+    | F.Break    (st,((),ii),_) -> iif ii
     | F.Continue (st,((),ii)) -> iif ii
     | F.Default  (st,((),ii)) -> iif ii
     | F.Return   (st,((),ii)) -> iif ii
@@ -851,10 +960,14 @@ and vk_node = fun bigf node ->
         iif ii;
         vk_asmbody bigf asmbody
 
+    | F.Exec (st,(code,ii)) ->
+	iif ii;
+	List.iter (vk_exec_code bigf) code
+
     | (
         F.TopNode|F.EndNode|
         F.ErrorExit|F.Exit|F.Enter|F.LoopFallThroughNode|F.FallThroughNode|
-        F.AfterNode|F.FalseNode|F.TrueNode|F.InLoopNode|
+        F.AfterNode _|F.FalseNode|F.TrueNode _|F.InLoopNode|
         F.Fake
       ) -> ()
 
@@ -943,6 +1056,10 @@ let vk_define_params_splitted = vk_splitted (fun bigf (_,ii) -> vk_ii bigf ii)
 let vk_params_splitted = vk_splitted vk_param
 let vk_enum_fields_splitted = vk_splitted vk_oneEnum
 let vk_inis_splitted = vk_splitted vk_ini
+let vk_ident_list_splitted = vk_splitted vk_name
+let vk_string_fragments_splitted = vk_splitted vk_string_fragment
+let vk_exec_code_list_splitted = vk_splitted vk_exec_code
+let vk_attrs_splitted = vk_splitted vk_attribute
 
 (* ------------------------------------------------------------------------ *)
 let vk_cst = fun bigf (cst, ii) ->
@@ -971,6 +1088,8 @@ type 'a inout = 'a -> 'a
  *)
 type visitor_c_s = {
   kexpr_s:      (expression inout * visitor_c_s) -> expression inout;
+  kassignOp_s:  (assignOp inout * visitor_c_s) -> assignOp inout;
+  kbinaryOp_s:  (binaryOp inout * visitor_c_s) -> binaryOp inout;
   kstatement_s: (statement  inout * visitor_c_s) -> statement  inout;
   ktype_s:      (fullType   inout * visitor_c_s) -> fullType   inout;
 
@@ -981,6 +1100,7 @@ type visitor_c_s = {
   kini_s:  (initialiser  inout * visitor_c_s) -> initialiser inout;
 
   kcppdirective_s: (cpp_directive inout * visitor_c_s) -> cpp_directive inout;
+  kifdefdirective_s : (ifdef_directive inout * visitor_c_s) -> ifdef_directive inout;
   kdefineval_s: (define_val inout * visitor_c_s) -> define_val inout;
   kstatementseq_s: (statement_sequencable inout * visitor_c_s) -> statement_sequencable inout;
   kstatementseq_list_s: (statement_sequencable list inout * visitor_c_s) -> statement_sequencable list inout;
@@ -989,11 +1109,15 @@ type visitor_c_s = {
 
 
   ktoplevel_s: (toplevel inout * visitor_c_s) -> toplevel inout;
+  kfragment_s: (string_fragment inout * visitor_c_s) -> string_fragment inout;
+  kformat_s: (string_format inout * visitor_c_s) -> string_format inout;
   kinfo_s: (info inout * visitor_c_s) -> info inout;
  }
 
 let default_visitor_c_s =
-  { kexpr_s =      (fun (k,_) e  -> k e);
+  { kexpr_s      = (fun (k,_) e  -> k e);
+    kassignOp_s  = (fun (k,_) op -> k op);
+    kbinaryOp_s  = (fun (k,_) op -> k op);
     kstatement_s = (fun (k,_) st -> k st);
     ktype_s      = (fun (k,_) t  -> k t);
     kdecl_s      = (fun (k,_) d  -> k d);
@@ -1002,11 +1126,14 @@ let default_visitor_c_s =
     kini_s       = (fun (k,_) d  -> k d);
     ktoplevel_s  = (fun (k,_) p  -> k p);
     knode_s      = (fun (k,_) n  -> k n);
+    kfragment_s  = (fun (k,_) f  -> k f);
+    kformat_s    = (fun (k,_) f  -> k f);
     kinfo_s      = (fun (k,_) i  -> k i);
     kdefineval_s = (fun (k,_) x  -> k x);
     kstatementseq_s = (fun (k,_) x  -> k x);
     kstatementseq_list_s = (fun (k,_) x  -> k x);
     kcppdirective_s = (fun (k,_) x  -> k x);
+    kifdefdirective_s = (fun (k,_) x -> k x);
   }
 
 let rec vk_expr_s = fun bigf expr ->
@@ -1022,6 +1149,8 @@ let rec vk_expr_s = fun bigf expr ->
       match unwrap_e with
       | Ident (name) -> Ident (vk_name_s bigf name)
       | Constant (c) -> Constant (c)
+      |	StringConstant(s,os,w) ->
+	  StringConstant(s +> (List.map (vk_string_fragment_s bigf)),os,w)
       | FunCall  (e, es)         ->
           FunCall (exprf e,
                   es +> List.map (fun (e,ii) ->
@@ -1030,43 +1159,51 @@ let rec vk_expr_s = fun bigf expr ->
 
       | CondExpr (e1, e2, e3)   -> CondExpr (exprf e1, fmap exprf e2, exprf e3)
       | Sequence (e1, e2)        -> Sequence (exprf e1, exprf e2)
-      | Assignment (e1, op, e2)  -> Assignment (exprf e1, op, exprf e2)
-
+      | Assignment (e1, op, e2)  ->
+        let e1 = exprf e1 in 
+        let op = vk_assignOp_s bigf op in
+        let e2 = exprf e2 in
+        Assignment (e1, op, e2)
       | Postfix  (e, op) -> Postfix (exprf e, op)
       | Infix    (e, op) -> Infix   (exprf e, op)
       | Unary    (e, op) -> Unary   (exprf e, op)
-      | Binary   (e1, op, e2) -> Binary (exprf e1, op, exprf e2)
+      | Binary   (e1, op, e2) ->
+        let e1 = exprf e1 in
+        let op = vk_binaryOp_s bigf op in
+        let e2 = exprf e2 in
+        Binary (e1, op, e2)
 
-      | ArrayAccess    (e1, e2) -> ArrayAccess (exprf e1, exprf e2)
-      | RecordAccess   (e, name) ->
-          RecordAccess     (exprf e, vk_name_s bigf name)
-      | RecordPtAccess (e, name) ->
-          RecordPtAccess   (exprf e, vk_name_s bigf name)
+      | ArrayAccess    (e1, e2)  -> ArrayAccess (exprf e1, exprf e2)
+      | RecordAccess   (e, name) -> RecordAccess (exprf e, vk_name_s bigf name)
+      | RecordPtAccess (e, name) -> RecordPtAccess (exprf e, vk_name_s bigf name)
 
-      | SizeOfExpr  (e) -> SizeOfExpr   (exprf e)
+      | SizeOfExpr  (e) -> SizeOfExpr (exprf e)
       | SizeOfType  (t) -> SizeOfType (vk_type_s bigf t)
-      | Cast    (t, e) ->  Cast   (vk_type_s bigf t, exprf e)
+      | Cast    (t, e)  -> Cast (vk_type_s bigf t, exprf e)
 
       | StatementExpr (statxs, is) ->
           StatementExpr (
             vk_statement_sequencable_list_s bigf statxs,
             iif is)
-      | Constructor (t, initxs) ->
-          Constructor
-            (vk_type_s bigf t,
-            (initxs +> List.map (fun (ini, ii) ->
-              vk_ini_s bigf ini, vk_ii_s bigf ii)
-            ))
+      | Constructor (t, init) -> Constructor (vk_type_s bigf t, vk_ini_s bigf init)
 
-      | ParenExpr (e) -> ParenExpr (exprf e)
+      | ParenExpr (e)    -> ParenExpr (exprf e)
 
-      | New  t        -> New (vk_argument_s bigf t)
+      | New (None, t)    -> New (None, vk_argument_s bigf t)
+      | New (Some ts, t) ->
+	  New (Some (ts +> List.map (fun (e,ii) ->
+	    vk_argument_s bigf e, iif ii)), vk_argument_s bigf t)
       | Delete e      -> Delete (vk_expr_s bigf e)
 
     in
     (e', typ'), (iif ii)
   in exprf expr
 
+and vk_assignOp_s bigf (op,ii) =
+  let iif ii = vk_ii_s bigf ii in (op, iif ii)
+
+and vk_binaryOp_s bigf (op,ii) =
+  let iif ii = vk_ii_s bigf ii in (op, iif ii)
 
 and vk_argument_s bigf argument =
   let iif ii = vk_ii_s bigf ii in
@@ -1126,27 +1263,42 @@ and vk_statement_s = fun bigf st ->
       | ExprStatement (Some e) -> ExprStatement (Some ((vk_expr_s bigf) e))
       | Selection (If (e, st1, st2)) ->
           Selection  (If ((vk_expr_s bigf) e, statf st1, statf st2))
+      | Selection (Ifdef_Ite (e, st1, st2)) ->
+          Selection  (Ifdef_Ite (vk_expr_s bigf e,statf st1,statf st2))
+      | Selection (Ifdef_Ite2 (e, st1, st2, st3)) ->
+          Selection  (Ifdef_Ite2 (vk_expr_s bigf e,statf st1
+                                                  ,statf st2
+                                                  ,statf st3))
       | Selection (Switch (e, st))   ->
           Selection  (Switch ((vk_expr_s bigf) e, statf st))
       | Iteration (While (e, st))    ->
           Iteration  (While ((vk_expr_s bigf) e, statf st))
       | Iteration (DoWhile (st, e))  ->
           Iteration  (DoWhile (statf st, (vk_expr_s bigf) e))
-      | Iteration (For ((e1opt,i1), (e2opt,i2), (e3opt,i3), st)) ->
-          let e1opt' = statf (mk_st (ExprStatement (e1opt)) i1) in
+      | Iteration (For (first, (e2opt,i2), (e3opt,i3), st)) ->
+	  let first =
+	    match first with
+	      ForExp (e1opt,i1) ->
+		let e1opt' = statf (mk_st (ExprStatement (e1opt)) i1) in
+		let e1' = Ast_c.unwrap_st e1opt' in
+		let i1' = Ast_c.get_ii_st_take_care e1opt' in
+		(match e1' with
+		  ExprStatement x1 -> ForExp (x1,i1')
+		| _ ->
+		    failwith
+		      "cant be here if iterator keep ExprStatement as is")
+	    | ForDecl decl -> ForDecl (vk_decl_s bigf decl) in
           let e2opt' = statf (mk_st (ExprStatement (e2opt)) i2) in
           let e3opt' = statf (mk_st (ExprStatement (e3opt)) i3) in
 
-          let e1' = Ast_c.unwrap_st e1opt' in
           let e2' = Ast_c.unwrap_st e2opt' in
           let e3' = Ast_c.unwrap_st e3opt' in
-          let i1' = Ast_c.get_ii_st_take_care e1opt' in
           let i2' = Ast_c.get_ii_st_take_care e2opt' in
           let i3' = Ast_c.get_ii_st_take_care e3opt' in
 
-          (match (e1', e2', e3') with
-          | ((ExprStatement x1), (ExprStatement x2), ((ExprStatement x3))) ->
-              Iteration (For ((x1,i1'), (x2,i2'), (x3,i3'), statf st))
+          (match (e2', e3') with
+          | ((ExprStatement x2), ((ExprStatement x3))) ->
+              Iteration (For (first, (x2,i2'), (x3,i3'), statf st))
 
           | x -> failwith "cant be here if iterator keep ExprStatement as is"
          )
@@ -1171,6 +1323,7 @@ and vk_statement_s = fun bigf st ->
       | Asm asmbody -> Asm (vk_asmbody_s bigf asmbody)
       | NestedFunc def -> NestedFunc (vk_def_s bigf def)
       | MacroStmt -> MacroStmt
+      |	Exec(code) -> Exec(List.map (vk_exec_code_s bigf) code)
     in
     st', vk_ii_s bigf ii
   in statf st
@@ -1240,6 +1393,8 @@ and vk_type_s = fun bigf t ->
       | BaseType x -> BaseType x
       | Pointer t  -> Pointer (typef t)
       | Array (eopt, t) -> Array (fmap (vk_expr_s bigf) eopt, typef t)
+      | Decimal (len,prec_opt) ->
+	  Decimal (vk_expr_s bigf len, fmap (vk_expr_s bigf) prec_opt)
       | FunctionType (returnt, paramst) ->
           FunctionType
             (typef returnt,
@@ -1276,6 +1431,12 @@ and vk_attribute_s = fun bigf attr ->
   | Attribute s, ii ->
       Attribute s, iif ii
 
+and vk_exec_code_s = fun bigf e ->
+  let iif ii = vk_ii_s bigf ii in
+  match e with
+    ExecEval name, ii -> ExecEval (vk_expr_s bigf name), iif ii
+  | ExecToken, ii -> ExecToken, iif ii
+
 
 
 and vk_decl_s = fun bigf d ->
@@ -1285,11 +1446,17 @@ and vk_decl_s = fun bigf d ->
     match decl with
     | DeclList (xs, ii) ->
         DeclList (List.map aux xs,   iif ii)
-    | MacroDecl ((s, args),ii) ->
+    | MacroDecl ((s, args, ptvg),ii) ->
         MacroDecl
           ((s,
-           args +> List.map (fun (e,ii) -> vk_argument_s bigf e, iif ii)
-           ),
+           args +> List.map (fun (e,ii) -> vk_argument_s bigf e, iif ii),
+           ptvg),
+          iif ii)
+    | MacroDeclInit ((s, args, ini),ii) ->
+        MacroDeclInit
+          ((s,
+           args +> List.map (fun (e,ii) -> vk_argument_s bigf e, iif ii),
+	   vk_ini_s bigf ini),
           iif ii)
 
 
@@ -1472,10 +1639,11 @@ and vk_toplevel_s = fun bigf p ->
 
     | NotParsedCorrectly ii -> NotParsedCorrectly (iif ii)
     | FinalDef info -> FinalDef (vk_info_s bigf info)
+    | Namespace (tls, ii) -> Namespace (List.map (vk_toplevel_s bigf) tls, ii)
   in f (k, bigf) p
 
-and vk_program_s = fun bigf xs ->
-  xs +> List.map (vk_toplevel_s bigf)
+and vk_program_s : visitor_c_s -> toplevel list -> toplevel list =
+      fun bigf -> List.map (vk_toplevel_s bigf)
 
 
 and vk_cpp_directive_s = fun bigf top ->
@@ -1499,16 +1667,31 @@ and vk_cpp_directive_s = fun bigf top ->
     | Define ((s,ii), (defkind, defval)) ->
         Define ((s, iif ii),
                (vk_define_kind_s bigf defkind, vk_define_val_s bigf defval))
-    | PragmaAndCo (ii) -> PragmaAndCo (iif ii)
+    | Pragma((s,ii), pragmainfo) ->
+	Pragma((s,iif ii), vk_pragmainfo_s bigf pragmainfo)
+    | OtherDirective (ii) -> OtherDirective (iif ii)
 
   in f (k, bigf) top
 
-and vk_ifdef_directive_s = fun bigf ifdef ->
-  let iif ii = vk_ii_s bigf ii in
-  match ifdef with
-  | IfdefDirective (ifkind, ii) -> IfdefDirective (ifkind, iif ii)
+and vk_ifdef_directive_s bigf ifdef =
+  let f = bigf.kifdefdirective_s in
+  let k d =
+    let iif ii = vk_ii_s bigf ii in
+    match d with
+    | IfdefDirective ((ifkind,tag), ii) ->
+        let ifkind' = vk_ifdefkind_s bigf ifkind in
+        IfdefDirective ((ifkind',tag), iif ii)
+  in f (k, bigf) ifdef
 
+and vk_ifdefkind_s bigf = function
+  | Ifdef       ifguard -> Ifdef       (vk_ifdef_guard_s bigf ifguard)
+  | IfdefElseif ifguard -> IfdefElseif (vk_ifdef_guard_s bigf ifguard)
+  | IfdefElse           -> IfdefElse
+  | IfdefEndif          -> IfdefEndif
 
+and vk_ifdef_guard_s bigf = function
+  | Gif e -> Gif (vk_expr_s bigf e)
+  | x     -> x
 
 and vk_define_kind_s  = fun bigf defkind ->
   match defkind with
@@ -1539,6 +1722,9 @@ and vk_define_val_s = fun bigf x ->
     | DefineText (s, ii) -> DefineText (s, iif ii)
     | DefineEmpty -> DefineEmpty
     | DefineInit ini -> DefineInit (vk_ini_s bigf ini)
+    (* christia: added multi *)
+    | DefineMulti ds ->
+	DefineMulti (List.map (vk_statement_s bigf) ds)
 
     | DefineTodo ->
         pr2_once "DefineTodo";
@@ -1546,6 +1732,48 @@ and vk_define_val_s = fun bigf x ->
   in
   f (k, bigf) x
 
+and vk_pragmainfo_s bigf pragmainfo =
+  match pragmainfo with
+    PragmaTuple(args,ii) ->
+      PragmaTuple(
+      args +> List.map (fun (e,ii) -> vk_argument_s bigf e, vk_ii_s bigf ii),
+      vk_ii_s bigf ii)
+  | PragmaIdList ids ->
+      PragmaIdList
+	(ids +>
+	 List.map
+	   (function
+	       id, [] -> vk_name_s bigf id, []
+	     | _ -> failwith "bad ident_list"))
+
+
+and vk_string_fragment_s = fun bigf x ->
+  let rec fragf x = bigf.kfragment_s (k, bigf) x
+  and k st =
+    let (unwrap_x, ii) = x in
+    let x' =
+      match unwrap_x with
+	ConstantFragment s -> ConstantFragment s
+      | FormatFragment(fmt) ->
+	  FormatFragment(vk_string_format_s bigf fmt) in
+    x', vk_ii_s bigf ii
+  in
+  fragf x
+
+and vk_string_fragments_s = fun bigf frags ->
+  frags +> List.map (vk_string_fragment_s bigf)
+
+and vk_string_format_s = fun bigf x ->
+  let rec fmtf x = bigf.kformat_s (k, bigf) x
+  and k st =
+    let (unwrap_x, ii) = x in
+    let x' =
+      match unwrap_x with
+	ConstantFormat s -> ConstantFormat s
+    in
+    x', vk_ii_s bigf ii
+  in
+  fmtf x
 
 and vk_info_s = fun bigf info ->
   let rec infof ii = bigf.kinfo_s (k, bigf) ii
@@ -1582,9 +1810,15 @@ and vk_node_s = fun bigf node ->
     | F.DoWhileTail (e,ii)  ->
         F.DoWhileTail (vk_expr_s bigf e, iif ii)
 
-    | F.ForHeader (st, (((e1opt,i1), (e2opt,i2), (e3opt,i3)), ii)) ->
+    | F.ForHeader (st, ((first, (e2opt,i2), (e3opt,i3)), ii)) ->
+	let first =
+	  match first with
+	    ForExp (e1opt,i1) ->
+	      ForExp (e1opt +> Common.map_option (vk_expr_s bigf), iif i1)
+	  | ForDecl decl -> ForDecl (vk_decl_s bigf decl) in
+
         F.ForHeader (st,
-                    (((e1opt +> Common.map_option (vk_expr_s bigf), iif i1),
+                    ((first,
                      (e2opt +> Common.map_option (vk_expr_s bigf), iif i2),
                      (e3opt +> Common.map_option (vk_expr_s bigf), iif i3)),
                     iif ii))
@@ -1614,6 +1848,9 @@ and vk_node_s = fun bigf node ->
         F.DefineDoWhileZeroHeader ((),iif ii)
     | F.DefineTodo -> F.DefineTodo
 
+    | F.PragmaHeader ((s,ii),pragmainfo) ->
+        F.PragmaHeader((s,iif ii), vk_pragmainfo_s bigf pragmainfo)
+
     | F.Include {i_include = (s, ii);
                  i_rel_pos = h_rel_pos;
                  i_is_in_ifdef = b;
@@ -1636,8 +1873,11 @@ and vk_node_s = fun bigf node ->
 
     | F.MacroStmt (st, ((),ii)) -> F.MacroStmt (st, ((),iif ii))
     | F.Asm (st, (body,ii)) -> F.Asm (st, (vk_asmbody_s bigf body,iif ii))
+    | F.Exec(st, (code,ii)) ->
+	F.Exec(st,((List.map (vk_exec_code_s bigf) code),ii))
 
-    | F.Break    (st,((),ii)) -> F.Break    (st,((),iif ii))
+    | F.Break    (st,((),ii),fromswitch) ->
+	F.Break    (st,((),iif ii),fromswitch)
     | F.Continue (st,((),ii)) -> F.Continue (st,((),iif ii))
     | F.Default  (st,((),ii)) -> F.Default  (st,((),iif ii))
     | F.Return   (st,((),ii)) -> F.Return   (st,((),iif ii))
@@ -1655,11 +1895,13 @@ and vk_node_s = fun bigf node ->
     | F.IfdefElse (info) -> F.IfdefElse (vk_ifdef_directive_s bigf info)
     | F.IfdefEndif (info) -> F.IfdefEndif (vk_ifdef_directive_s bigf info)
 
+    | F.IfdefIteHeader ii -> F.IfdefIteHeader (iif ii)
+
     | (
         (
           F.TopNode|F.EndNode|
           F.ErrorExit|F.Exit|F.Enter|F.LoopFallThroughNode|F.FallThroughNode|
-          F.AfterNode|F.FalseNode|F.TrueNode|F.InLoopNode|
+          F.AfterNode _|F.FalseNode|F.TrueNode _|F.InLoopNode|
           F.Fake
         ) as x) -> x
 
@@ -1681,6 +1923,10 @@ let vk_arguments_s = fun bigf args ->
   let iif ii = vk_ii_s bigf ii in
   args +> List.map (fun (e, ii) -> vk_argument_s bigf e, iif ii)
 
+let vk_inis_s = fun bigf inis ->
+  let iif ii = vk_ii_s bigf ii in
+  inis +> List.map (fun (e, ii) -> vk_ini_s bigf e, iif ii)
+
 let vk_params_s = fun bigf args ->
   let iif ii = vk_ii_s bigf ii in
   args +> List.map (fun (p,ii) -> vk_param_s bigf p, iif ii)
@@ -1707,3 +1953,7 @@ let vk_define_params_splitted_s =
   vk_splitted_s (fun bigf (s,ii) -> (s,vk_ii_s bigf ii))
 let vk_enum_fields_splitted_s = vk_splitted_s vk_oneEnum_s
 let vk_inis_splitted_s = vk_splitted_s vk_ini_s
+let vk_ident_list_splitted_s = vk_splitted_s vk_name_s
+let vk_string_fragments_splitted_s = vk_splitted_s vk_string_fragment_s
+let vk_exec_code_list_splitted_s = vk_splitted_s vk_exec_code_s
+let vk_attrs_splitted_s = vk_splitted_s vk_attribute_s
