@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./adjust_pragmas.ml"
 (* Find a directive or comment at the end of a statement.  Things with aft
 given None, because they can accomodate their own directives or comments *)
 
@@ -45,13 +48,17 @@ let right_mcode (a,b,info,mcodekind,d,e) =
     ([],_) | (_,Ast0.PLUS _) -> None
   | (l,_) -> Some(l,(a,b,{info with Ast0.strings_after = []},mcodekind,d,e))
 
-let update_before pragmas (info,x) =
+let update_before pragmas (info,x,adj) =
   ({info with Ast0.strings_before = pragmas @ info.Ast0.strings_before},
-   Ast0.PLUS Ast.ONE) (* not sure what the arg should be... one seems safe *)
+   Ast0.PLUS Ast.ONE,adj)(*not sure what the arg should be... one seems safe*)
+
+let update_before2 pragmas (info,x) =
+  ({info with Ast0.strings_before = pragmas @ info.Ast0.strings_before},
+   Ast0.PLUS Ast.ONE)(*not sure what the arg should be... one seems safe*)
 
 let update_after pragmas (info,x) =
   ({info with Ast0.strings_after = info.Ast0.strings_after @ pragmas},
-   Ast0.PLUS Ast.ONE) (* not sure what the arg should be... one seems safe *)
+   Ast0.PLUS Ast.ONE) (*not sure what the arg should be... one seems safe*)
 
 let rec right_decl d =
   match Ast0.unwrap d with
@@ -64,21 +71,22 @@ let rec right_decl d =
   | Ast0.MetaFieldList(name,lenname,pure) ->
       call_right right_mcode name d
 	(function name -> Ast0.MetaFieldList(name,lenname,pure))
-  | Ast0.Init(Some stg,ty,id,eq,ini,sem) ->
+  | Ast0.AsDecl(decl,asdecl) -> failwith "not possible"
+  | Ast0.Init(stg,ty,id,eq,ini,sem) ->
       call_right right_mcode sem d
-	(function sem -> Ast0.Init(Some stg,ty,id,eq,ini,sem))
-  | Ast0.Init(None,ty,id,eq,ini,sem) ->
+	(function sem -> Ast0.Init(stg,ty,id,eq,ini,sem))
+  | Ast0.UnInit(stg,ty,id,sem) ->
       call_right right_mcode sem d
-	(function sem -> Ast0.Init(None,ty,id,eq,ini,sem))
-  | Ast0.UnInit(Some stg,ty,id,sem) ->
+	(function sem -> Ast0.UnInit(stg,ty,id,sem))
+  | Ast0.FunProto(fninfo,id,lp1,params,va,rp1,sem) ->
       call_right right_mcode sem d
-	(function sem -> Ast0.UnInit(Some stg,ty,id,sem))
-  | Ast0.UnInit(None,ty,id,sem) ->
-      call_right right_mcode sem d
-	(function sem -> Ast0.UnInit(None,ty,id,sem))
+	(function ty -> Ast0.FunProto(fninfo,id,lp1,params,va,rp1,sem))
   | Ast0.MacroDecl(name,lp,args,rp,sem) ->
       call_right right_mcode sem d
 	(function sem -> Ast0.MacroDecl(name,lp,args,rp,sem))
+  | Ast0.MacroDeclInit(name,lp,args,rp,eq,ini,sem) ->
+      call_right right_mcode sem d
+	(function sem -> Ast0.MacroDeclInit(name,lp,args,rp,eq,ini,sem))
   | Ast0.TyDecl(ty,sem) ->
       call_right right_mcode sem d
 	(function sem -> Ast0.TyDecl(ty,sem))
@@ -94,7 +102,7 @@ let rec right_decl d =
 
 let rec right_statement s =
   match Ast0.unwrap s with
-    Ast0.FunDecl(bef,fi,name,lp,params,rp,lbrace,body,rbrace) -> None
+    Ast0.FunDecl(bef,fi,name,lp,params,va,rp,lbrace,body,rbrace,aft) -> None
   | Ast0.Decl(bef,decl) ->
       call_right right_decl decl s
 	(function decl -> Ast0.Decl(bef,decl))
@@ -110,7 +118,7 @@ let rec right_statement s =
   | Ast0.Do(d,body,whl,lp,exp,rp,sem) ->
       call_right right_mcode sem s
 	(function sem -> Ast0.Do(d,body,whl,lp,exp,rp,sem))
-  | Ast0.For(fr,lp,e1,sem1,e2,sem2,e3,rp,body,aft) -> None
+  | Ast0.For(fr,lp,first,e2,sem2,e3,rp,body,aft) -> None
   | Ast0.Iterator(nm,lp,args,rp,body,aft) -> None
   | Ast0.Switch(switch,lp,exp,rp,lb,decls,cases,rb) ->
       call_right right_mcode rb s
@@ -133,12 +141,16 @@ let rec right_statement s =
   | Ast0.ReturnExpr(ret,exp,sem) ->
       call_right right_mcode sem s
 	(function sem -> Ast0.ReturnExpr(ret,exp,sem))
+  | Ast0.Exec(exec,lang,exp,sem) ->
+      call_right right_mcode sem s
+	(function sem -> Ast0.Exec(exec,lang,exp,sem))
   | Ast0.MetaStmt(name,pure) ->
       call_right right_mcode name s
 	(function name -> Ast0.MetaStmt(name,pure))
   | Ast0.MetaStmtList(name,pure) ->
       call_right right_mcode name s
 	(function name -> Ast0.MetaStmtList(name,pure))
+  | Ast0.AsStmt(stm,asstm) -> failwith "not possible"
   | Ast0.Disj(starter,statement_dots_list,mids,ender) -> None
   | Ast0.Nest(starter,stmt_dots,ender,whn,multi) -> None
   (* the following are None, because they can't be adjacent to an aft node *)
@@ -159,6 +171,9 @@ let rec right_statement s =
   | Ast0.Define(def,id,params,body) ->
       call_right right_statement_dots body s
 	(function body -> Ast0.Define(def,id,params,body))
+  | Ast0.Pragma(prg,id,body) ->
+      (* seems safe; let the code go with the enclosing statement, if any *)
+      None
   | Ast0.OptStm(re) ->
       call_right right_statement re s (function re -> Ast0.OptStm(re))
   | Ast0.UniqueStm(re) ->
@@ -188,14 +203,11 @@ let rec left_ty t =
   | Ast0.FunctionPointer(ty,lp1,star,rp1,lp2,params,rp2) ->
       call_right left_ty ty t
 	(function ty -> Ast0.FunctionPointer(ty,lp1,star,rp1,lp2,params,rp2))
-  | Ast0.FunctionType(Some ty,lp1,params,rp1) ->
-      call_right left_ty ty t
-	(function ty -> Ast0.FunctionType(Some ty,lp1,params,rp1))
-  | Ast0.FunctionType(None,lp1,params,rp1) ->
-      call_right left_mcode lp1 t
-	(function lp1 -> Ast0.FunctionType(None,lp1,params,rp1))
   | Ast0.Array(ty,lb,size,rb) ->
       call_right left_ty ty t (function ty -> Ast0.Array(ty,lb,size,rb))
+  | Ast0.Decimal(dec,lp,length,comma,precision_opt,rp) ->
+      call_right left_mcode dec t
+	(function dec -> Ast0.Decimal(dec,lp,length,comma,precision_opt,rp))
   | Ast0.EnumName(kind,name) ->
       call_right left_mcode kind t (function kind -> Ast0.EnumName(kind,name))
   | Ast0.EnumDef(ty,lb,ids,rb) ->
@@ -211,6 +223,7 @@ let rec left_ty t =
       call_right left_mcode name t (function name -> Ast0.TypeName(name))
   | Ast0.MetaType(name,x) ->
       call_right left_mcode name t (function name -> Ast0.MetaType(name,x))
+  | Ast0.AsType(ty,asty) -> failwith "not possible"
   | Ast0.DisjType(starter,types,mids,ender) -> None
   | Ast0.OptType(ty) ->
       call_right left_ty ty t (function ty -> Ast0.OptType(ty))
@@ -221,8 +234,8 @@ let rec left_ident i =
   match Ast0.unwrap i with
     Ast0.Id(name) ->
       call_right left_mcode name i (function name -> Ast0.Id(name))
-  | Ast0.MetaId(name,a,b) ->
-      call_right left_mcode name i (function name -> Ast0.MetaId(name,a,b))
+  | Ast0.MetaId(name,a,b,c) ->
+      call_right left_mcode name i (function name -> Ast0.MetaId(name,a,b,c))
   | Ast0.MetaFunc(name,a,b) ->
       call_right left_mcode name i (function name -> Ast0.MetaFunc(name,a,b))
   | Ast0.MetaLocalFunc(name,a,b) ->
@@ -233,6 +246,7 @@ let rec left_ident i =
       call_right left_ident id i (function id -> Ast0.OptIdent(id))
   | Ast0.UniqueIdent(id) ->
       call_right left_ident id i (function id -> Ast0.UniqueIdent(id))
+  | Ast0.AsIdent(id,asid) -> failwith "not possible"
 
 let left_fundecl name fninfo =
   let fncall_right processor data cont =
@@ -264,6 +278,7 @@ let rec left_decl decl =
   | Ast0.MetaFieldList(name,lenname,pure) ->
       call_right right_mcode name decl
 	(function name -> Ast0.MetaFieldList(name,lenname,pure))
+  | Ast0.AsDecl(decl,asdecl) -> failwith "not possible"
   | Ast0.Init(Some stg,ty,id,eq,ini,sem) ->
       call_right left_mcode stg decl
 	(function stg -> Ast0.Init(Some stg,ty,id,eq,ini,sem))
@@ -276,9 +291,33 @@ let rec left_decl decl =
   | Ast0.UnInit(None,ty,id,sem) ->
       call_right left_ty ty decl
 	(function ty -> Ast0.UnInit(None,ty,id,sem))
+  | Ast0.FunProto(fi,name,lp1,params,va,rp1,sem) ->
+      (match fi with
+	[] ->
+          call_right left_ident name decl
+	    (function name -> Ast0.FunProto(fi,name,lp1,params,va,rp1,sem))
+      | (Ast0.FStorage sto)::x ->
+	  call_right left_mcode sto decl
+	    (function sto ->
+	      Ast0.FunProto((Ast0.FStorage sto)::x,name,lp1,params,va,rp1,sem))
+      |	(Ast0.FType ty)::x ->
+	  call_right left_ty ty decl
+	    (function ty ->
+	      Ast0.FunProto((Ast0.FType ty)::x,name,lp1,params,va,rp1,sem))
+      | (Ast0.FInline inl)::x ->
+	  call_right left_mcode inl decl
+	    (function inl ->
+	      Ast0.FunProto((Ast0.FInline inl)::x,name,lp1,params,va,rp1,sem))
+      | (Ast0.FAttr attr)::x ->
+	  call_right left_mcode attr decl
+	    (function attr ->
+	      Ast0.FunProto((Ast0.FAttr attr)::x,name,lp1,params,va,rp1,sem)))
   | Ast0.MacroDecl(name,lp,args,rp,sem) ->
       call_right left_ident name decl
 	(function name -> Ast0.MacroDecl(name,lp,args,rp,sem))
+  | Ast0.MacroDeclInit(name,lp,args,rp,eq,ini,sem) ->
+      call_right left_ident name decl
+	(function name -> Ast0.MacroDeclInit(name,lp,args,rp,eq,ini,sem))
   | Ast0.TyDecl(ty,sem) ->
       call_right left_ty ty decl (function ty -> Ast0.TyDecl(ty,sem))
   | Ast0.Typedef(stg,ty,id,sem) ->
@@ -296,18 +335,23 @@ let process =
     let s = k s in
     Ast0.rewrap s
       (match Ast0.unwrap s with
-	Ast0.FunDecl(bef,fi,name,lp,params,rp,lbrace,body,rbrace) ->
+	Ast0.FunDecl(bef,fi,name,lp,params,va,rp,lbrace,body,rbrace,aft) ->
+	  let (rbrace,aft) =
+	    match right_mcode rbrace with
+	      None -> (rbrace,aft)
+	    | Some (pragmas,rbrace) -> (rbrace,update_after pragmas aft) in
 	  (match left_fundecl name fi with
-	    None -> Ast0.unwrap s
-	  | Some (pragmas,fi,name) ->
-	      Ast0.FunDecl
-		(update_after pragmas bef,
-		 fi,name,lp,params,rp,lbrace,body,rbrace))
+	      None ->
+		Ast0.FunDecl(bef,fi,name,lp,params,va,rp,lbrace,body,rbrace,aft)
+	    | Some (pragmas,fi,name) ->
+		Ast0.FunDecl
+		  (update_before2 pragmas bef,
+		   fi,name,lp,params,va,rp,lbrace,body,rbrace,aft))
       | Ast0.Decl(bef,decl) ->
 	  (match left_decl decl with
 	    None -> Ast0.unwrap s
 	  | Some (pragmas,decl) ->
-	      Ast0.Decl(update_after pragmas bef,decl))
+	      Ast0.Decl(update_before2 pragmas bef,decl))
       | Ast0.IfThen(iff,lp,exp,rp,branch1,aft) ->
 	  (match right_statement branch1 with
 	    None -> Ast0.unwrap s
@@ -326,12 +370,12 @@ let process =
 	    None -> Ast0.unwrap s
 	  | Some (pragmas,body) ->
 	      Ast0.While(whl,lp,exp,rp,body,update_before pragmas aft))
-      | Ast0.For(fr,lp,e1,sem1,e2,sem2,e3,rp,body,aft) ->
+      | Ast0.For(fr,lp,first,e2,sem2,e3,rp,body,aft) ->
 	  (match right_statement body with
 	    None -> Ast0.unwrap s
 	  | Some (pragmas,body) ->
 	      Ast0.For
-		(fr,lp,e1,sem1,e2,sem2,e3,rp,body,
+		(fr,lp,first,e2,sem2,e3,rp,body,
 		 update_before pragmas aft))
       | Ast0.Iterator(nm,lp,args,rp,body,aft) ->
 	  (match right_statement body with
