@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./arity.ml"
 (* Arities matter for the minus slice, but not for the plus slice. *)
 
 (* ? only allowed on rule_elems, and on subterms if the context is ? also. *)
@@ -84,6 +87,25 @@ let allopt l fn =
 
 let mcode2line (_,_,info,_,_,_) = info.Ast0.pos_info.Ast0.line_start
 let mcode2arity (_,arity,_,_,_,_) = arity
+let mcodeassignOp2line op = match Ast0.unwrap op with
+    Ast0.SimpleAssign op -> mcode2line op
+  | Ast0.OpAssign op -> mcode2line op
+  | Ast0.MetaAssign(mv,_,_) -> mcode2line mv
+
+let mcodeassignOp2arity op = match Ast0.unwrap op with
+    Ast0.SimpleAssign op -> mcode2arity op
+  | Ast0.OpAssign op -> mcode2arity op
+  | Ast0.MetaAssign(mv,_,_) -> mcode2arity mv
+
+let mcodebinaryOp2line op = match Ast0.unwrap op with
+    Ast0.Arith op -> mcode2line op
+  | Ast0.Logical op -> mcode2line op
+  | Ast0.MetaBinary(mv,_,_) -> mcode2line mv
+
+let mcodebinaryOp2arity op = match Ast0.unwrap op with
+    Ast0.Arith op -> mcode2arity op
+  | Ast0.Logical op -> mcode2arity op
+  | Ast0.MetaBinary(mv,_,_) -> mcode2arity mv
 
 let mcode x = x (* nothing to do ... *)
 
@@ -163,12 +185,12 @@ let rec ident opt_allowed tgt i =
 	  [mcode2arity name] in
       let name = mcode name in
       make_id i tgt arity (Ast0.Id(name))
-  | Ast0.MetaId(name,constraints,pure) ->
+  | Ast0.MetaId(name,constraints,seed,pure) ->
       let arity =
 	all_same opt_allowed tgt (mcode2line name)
 	  [mcode2arity name] in
       let name = mcode name in
-      make_id i tgt arity (Ast0.MetaId(name,constraints,pure))
+      make_id i tgt arity (Ast0.MetaId(name,constraints,seed,pure))
   | Ast0.MetaFunc(name,constraints,pure) ->
       let arity =
 	all_same opt_allowed tgt (mcode2line name)
@@ -189,12 +211,12 @@ let rec ident opt_allowed tgt i =
 	  then fail i "opt only allowed in the last disjunct"
       |	_ -> ());
       Ast0.rewrap i (Ast0.DisjId(starter,id_list,mids,ender))
-  | Ast0.OptIdent(_) | Ast0.UniqueIdent(_) ->
+  | Ast0.OptIdent(_) | Ast0.UniqueIdent(_) | Ast0.AsIdent _ ->
       failwith "unexpected code"
-	
+
 (* --------------------------------------------------------------------- *)
 (* Expression *)
-	
+
 let make_exp =
   make_opt_unique
     (function x -> Ast0.OptExp x)
@@ -216,6 +238,13 @@ let rec top_expression opt_allowed tgt expr =
       let arity = exp_same (mcode2line const) [mcode2arity const] in
       let const = mcode const in
       make_exp expr tgt arity (Ast0.Constant(const))
+  | Ast0.StringConstant(lq,str,rq) ->
+      (* all components on the same line, so this is probably pointless... *)
+      let arity = exp_same (mcode2line lq) [mcode2arity lq;mcode2arity rq] in
+      let lq = mcode lq in
+      let str = dots (string_fragment arity) str in
+      let rq = mcode rq in
+      make_exp expr tgt arity (Ast0.StringConstant(lq,str,rq))
   | Ast0.FunCall(fn,lp,args,rp) ->
       let arity = exp_same (mcode2line lp) [mcode2arity lp;mcode2arity rp] in
       let fn = expression arity fn in
@@ -224,11 +253,17 @@ let rec top_expression opt_allowed tgt expr =
       let rp = mcode rp in
       make_exp expr tgt arity (Ast0.FunCall(fn,lp,args,rp))
   | Ast0.Assignment(left,op,right,simple) ->
-      let arity = exp_same (mcode2line op) [mcode2arity op] in
+      let arity = exp_same (mcodeassignOp2line op) [mcodeassignOp2arity op] in
       let left = expression arity left in
       let op = mcode op in
       let right = expression arity right in
       make_exp expr tgt arity (Ast0.Assignment(left,op,right,simple))
+  | Ast0.Sequence(left,op,right) ->
+      let arity = exp_same (mcode2line op) [mcode2arity op] in
+      let left = expression arity left in
+      let op = mcode op in
+      let right = expression arity right in
+      make_exp expr tgt arity (Ast0.Sequence(left,op,right))
   | Ast0.CondExpr(exp1,why,exp2,colon,exp3) ->
       let arity =
 	exp_same (mcode2line why) [mcode2arity why; mcode2arity colon] in
@@ -254,7 +289,7 @@ let rec top_expression opt_allowed tgt expr =
       let op = mcode op in
       make_exp expr tgt arity (Ast0.Unary(exp,op))
   | Ast0.Binary(left,op,right) ->
-      let arity = exp_same (mcode2line op) [mcode2arity op] in
+      let arity = exp_same (mcodebinaryOp2line op) [mcodebinaryOp2arity op] in
       let left = expression arity left in
       let op = mcode op in
       let right = expression arity right in
@@ -330,33 +365,87 @@ let rec top_expression opt_allowed tgt expr =
 	  then fail expr "opt only allowed in the last disjunct"
       |	_ -> ());
       Ast0.rewrap expr (Ast0.DisjExpr(starter,exps,mids,ender))
-  | Ast0.NestExpr(starter,exp_dots,ender,whencode,multi) ->
+  |  Ast0.NestExpr(starter,exp_dots,ender,whencode,multi) ->
       let res =
-	Ast0.NestExpr(starter,
-		      dots (top_expression true Ast0.NONE) exp_dots,
-		      ender,whencode,multi) in
+        Ast0.NestExpr(starter,
+                      dots (top_expression true Ast0.NONE) exp_dots,
+                      ender,whencode,multi) in
       Ast0.rewrap expr res
-  | Ast0.Edots(dots,whencode) ->
+  |  Ast0.Edots(dots,whencode) ->
       let arity = exp_same (mcode2line dots) [mcode2arity dots] in
       let dots = mcode dots in
-      let whencode = get_option (expression Ast0.NONE) whencode in
+      let whencode =
+        get_option (fun (a,e,b) -> (a,e,expression Ast0.NONE b)) whencode in
       make_exp expr tgt arity (Ast0.Edots(dots,whencode))
   | Ast0.Ecircles(dots,whencode) ->
       let arity = exp_same (mcode2line dots) [mcode2arity dots] in
       let dots = mcode dots in
-      let whencode = get_option (expression Ast0.NONE) whencode in
+      let whencode =
+        get_option (fun (a,e,b) -> (a,e,expression Ast0.NONE b)) whencode in
       make_exp expr tgt arity (Ast0.Ecircles(dots,whencode))
   | Ast0.Estars(dots,whencode) ->
       let arity = exp_same (mcode2line dots) [mcode2arity dots] in
       let dots = mcode dots in
-      let whencode = get_option (expression Ast0.NONE) whencode in
+      let whencode =
+        get_option (fun (a,e,b) -> (a,e,expression Ast0.NONE b)) whencode in
       make_exp expr tgt arity (Ast0.Estars(dots,whencode))
+  | Ast0.Constructor(lp,ty,rp,init) ->
+      let arity = exp_same (mcode2line lp) [mcode2arity lp;mcode2arity rp] in
+      let lp = mcode lp in
+      let ty = typeC arity ty in
+      let rp = mcode rp in
+      let init = initialiser arity init in
+      make_exp expr tgt arity (Ast0.Constructor(lp,ty,rp,init))
   (* why does optexp exist???? *)
-  | Ast0.OptExp(_) | Ast0.UniqueExp(_) ->
+  | Ast0.OptExp(_) | Ast0.UniqueExp(_) | Ast0.AsExpr _ ->
       failwith "unexpected code"
 
 and expression tgt exp = top_expression false tgt exp
 
+and make_fragment =
+  make_opt_unique
+    (function x -> failwith "opt not allowed for string fragment")
+    (function x -> failwith "unique not allowed for string fragment")
+
+and string_fragment tgt e =
+  match Ast0.unwrap e with
+    Ast0.ConstantFragment(str) ->
+      let arity = all_same false tgt (mcode2line str) [mcode2arity str] in
+      let str = mcode str in
+      make_fragment e tgt arity (Ast0.ConstantFragment(str))
+  | Ast0.FormatFragment(pct,fmt) ->
+      let arity = all_same false tgt (mcode2line pct) [mcode2arity pct] in
+      let pct = mcode pct in
+      let fmt = string_format arity fmt in
+      make_fragment e tgt arity (Ast0.FormatFragment(pct,fmt))
+  | Ast0.Strdots(dots) ->
+      let arity = all_same false tgt (mcode2line dots) [mcode2arity dots] in
+      let dots = mcode dots in
+      make_fragment e tgt arity (Ast0.Strdots(dots))
+  | Ast0.MetaFormatList(pct,name,lenname) ->
+      let arity =
+	all_same false tgt (mcode2line pct)
+	  [mcode2arity pct; mcode2arity name] in
+      let pct = mcode pct in
+      let name = mcode name in
+      make_fragment e tgt arity (Ast0.MetaFormatList(pct,name,lenname))
+
+and make_format =
+  make_opt_unique
+    (function x -> failwith "opt not allowed for string format")
+    (function x -> failwith "unique not allowed for string format")
+
+and string_format tgt e =
+  match Ast0.unwrap e with
+    Ast0.ConstantFormat(str) ->
+      let arity = all_same false tgt (mcode2line str) [mcode2arity str] in
+      let str = mcode str in
+      make_format e tgt arity (Ast0.ConstantFormat(str))
+  | Ast0.MetaFormat(name,constraints) ->
+      let arity = all_same false tgt (mcode2line name) [mcode2arity name] in
+      let name = mcode name in
+      make_format e tgt arity (Ast0.MetaFormat(name,constraints))
+
 (* --------------------------------------------------------------------- *)
 (* Types *)
 
@@ -399,13 +488,6 @@ and top_typeC tgt opt_allowed typ =
       let params = parameter_list tgt params in
       make_typeC typ tgt arity
 	(Ast0.FunctionPointer(ty,lp1,star,rp1,lp2,params,rp2))
-  | Ast0.FunctionType(ty,lp1,params,rp1) ->
-      let arity =
-	all_same opt_allowed tgt (mcode2line lp1)
-	  (List.map mcode2arity [lp1;rp1]) in
-      let ty = get_option (typeC arity) ty in
-      let params = parameter_list tgt params in
-      make_typeC typ tgt arity (Ast0.FunctionType(ty,lp1,params,rp1))
   | Ast0.Array(ty,lb,size,rb) ->
       let arity =
 	all_same opt_allowed tgt (mcode2line lb)
@@ -415,6 +497,18 @@ and top_typeC tgt opt_allowed typ =
       let size = get_option (expression arity) size in
       let rb = mcode rb in
       make_typeC typ tgt arity (Ast0.Array(ty,lb,size,rb))
+  | Ast0.Decimal(dec,lp,length,comma,precision_opt,rp) ->
+      let arity =
+	all_same opt_allowed tgt (mcode2line dec)
+	  [mcode2arity dec;mcode2arity lp;mcode2arity rp] in
+      let dec = mcode dec in
+      let lp = mcode lp in
+      let length = expression arity length in
+      let comma = get_option mcode comma in
+      let precision_opt = get_option (expression arity) precision_opt in
+      let rp = mcode rp in
+      make_typeC typ tgt arity
+	(Ast0.Decimal(dec,lp,length,comma,precision_opt,rp))
   | Ast0.EnumName(kind,name) ->
       let arity =
 	all_same opt_allowed tgt (mcode2line kind) [mcode2arity kind] in
@@ -465,7 +559,7 @@ and top_typeC tgt opt_allowed typ =
       |	_ -> ());
       let res = Ast0.DisjType(starter,types,mids,ender) in
       Ast0.rewrap typ res
-  | Ast0.OptType(_) | Ast0.UniqueType(_) ->
+  | Ast0.OptType(_) | Ast0.UniqueType(_) | Ast0.AsType _ ->
       failwith "unexpected code"
 
 and typeC tgt ty = top_typeC tgt false ty
@@ -516,15 +610,45 @@ and declaration tgt decl =
       let id = ident false arity id in
       let sem = mcode sem in
       make_decl decl tgt arity (Ast0.UnInit(stg,ty,id,sem))
+  | Ast0.FunProto(fi,name,lp1,params,va,rp1,sem) ->
+    let tokens = match va with
+      | None -> [lp1;rp1;sem]
+      | Some (c1,e1) -> [lp1;c1;e1;rp1;sem] in
+      let arity =
+	all_same true tgt (mcode2line lp1)
+	  (List.map mcode2arity tokens) in
+      let fi = List.map (fninfo arity) fi in
+      let name = ident false arity name in
+      let lp1 = mcode lp1 in
+      let params = parameter_list tgt params in
+      let va = match va with
+        | None -> None
+        | Some (c1, e1) -> Some (mcode c1, mcode e1) in 
+      let rp1 = mcode rp1 in
+      let sem = mcode sem in
+      make_decl decl tgt arity (Ast0.FunProto(fi,name,lp1,params,va,rp1,sem))
   | Ast0.MacroDecl(name,lp,args,rp,sem) ->
       let arity =
-	all_same true tgt (mcode2line lp) (List.map mcode2arity [lp;rp;sem]) in
+	all_same true tgt (mcode2line lp)
+	  (List.map mcode2arity [lp;rp;sem]) in
       let name = ident false arity name in
       let lp = mcode lp in
       let args = dots (expression arity) args in
       let rp = mcode rp in
       let sem = mcode sem in
       make_decl decl tgt arity (Ast0.MacroDecl(name,lp,args,rp,sem))
+  | Ast0.MacroDeclInit(name,lp,args,rp,eq,ini,sem) ->
+      let arity =
+	all_same true tgt (mcode2line lp)
+	  (List.map mcode2arity [lp;rp;eq;sem]) in
+      let name = ident false arity name in
+      let lp = mcode lp in
+      let args = dots (expression arity) args in
+      let rp = mcode rp in
+      let ini = initialiser arity ini in
+      let sem = mcode sem in
+      make_decl decl tgt arity
+	(Ast0.MacroDeclInit(name,lp,args,rp,eq,ini,sem))
   | Ast0.TyDecl(ty,sem) ->
       let arity =
 	all_same true tgt (mcode2line sem) [mcode2arity sem] in
@@ -552,9 +676,10 @@ and declaration tgt decl =
   | Ast0.Ddots(dots,whencode) ->
       let arity = all_same true tgt (mcode2line dots) [mcode2arity dots] in
       let dots = mcode dots in
-      let whencode = get_option (declaration Ast0.NONE) whencode in
+      let whencode =
+        get_option (fun (a,e,b) -> (a,e,declaration Ast0.NONE b)) whencode in
       make_decl decl tgt arity (Ast0.Ddots(dots,whencode))
-  | Ast0.OptDecl(_) | Ast0.UniqueDecl(_) ->
+  | Ast0.OptDecl(_) | Ast0.UniqueDecl(_) | Ast0.AsDecl _ ->
       failwith "unexpected code"
 
 (* --------------------------------------------------------------------- *)
@@ -572,6 +697,10 @@ and initialiser tgt i =
       let arity = init_same (mcode2line name) [mcode2arity name] in
       let name = mcode name in
       make_init i tgt arity (Ast0.MetaInit(name,pure))
+  | Ast0.MetaInitList(name,lenname,pure) ->
+      let arity = init_same (mcode2line name) [mcode2arity name] in
+      let name = mcode name in
+      make_init i tgt arity (Ast0.MetaInitList(name,lenname,pure))
   | Ast0.InitExpr(exp) ->
       Ast0.rewrap i (Ast0.InitExpr(expression tgt exp))
   | Ast0.InitList(lb,initlist,rb,ordered) ->
@@ -599,9 +728,10 @@ and initialiser tgt i =
   | Ast0.Idots(dots,whencode) ->
       let arity = init_same (mcode2line dots) [mcode2arity dots] in
       let dots = mcode dots in
-      let whencode = get_option (initialiser Ast0.NONE) whencode in
+      let whencode =
+        get_option (fun (a,e,b) -> (a,e,initialiser Ast0.NONE b)) whencode in
       make_init i tgt arity (Ast0.Idots(dots,whencode))
-  | Ast0.OptIni(_) | Ast0.UniqueIni(_) ->
+  | Ast0.OptIni(_) | Ast0.UniqueIni(_) | Ast0.AsInit _ ->
       failwith "unexpected code"
 
 and designator tgt d =
@@ -684,7 +814,7 @@ and parameterTypeDef tgt param =
       let arity = param_same (mcode2line dots) [mcode2arity dots] in
       let dots = mcode dots in
       make_param param tgt arity (Ast0.Pcircles(dots))
-  | Ast0.OptParam(_) | Ast0.UniqueParam(_) ->
+  | Ast0.OptParam(_) | Ast0.UniqueParam(_) | Ast0.AsParam _ ->
       failwith "unexpected code"
 
 and parameter_list tgt = dots (parameterTypeDef tgt)
@@ -720,7 +850,7 @@ and statement tgt stm =
       make_rule_elem stm tgt arity (Ast0.Seq(lbrace,body,rbrace))
   | Ast0.ExprStatement(exp,sem) ->
       let arity = stm_same (mcode2line sem) [mcode2arity sem] in
-      let exp = expression arity exp in
+      let exp = get_option (expression arity) exp in
       let sem = mcode sem in
       make_rule_elem stm tgt arity (Ast0.ExprStatement(exp,sem))
   | Ast0.IfThen(iff,lp,exp,rp,branch,aft) ->
@@ -765,20 +895,32 @@ and statement tgt stm =
       let rp = mcode rp in
       let sem = mcode sem in
       make_rule_elem stm tgt arity (Ast0.Do(d,body,wh,lp,exp,rp,sem))
-  | Ast0.For(fr,lp,exp1,sem1,exp2,sem2,exp3,rp,body,aft) ->
+  | Ast0.For(fr,lp,first,exp2,sem2,exp3,rp,body,aft) ->
       let arity =
-	stm_same (mcode2line fr) (List.map mcode2arity [fr;lp;sem1;sem2;rp]) in
+	let mcodes = [fr;lp;sem2;rp] in
+	let mcodes =
+	  match Ast0.unwrap first with
+	    Ast0.ForExp(exp1,sem1) -> sem1::mcodes
+	  | Ast0.ForDecl _ -> mcodes in
+	stm_same (mcode2line fr) (List.map mcode2arity mcodes) in
       let fr = mcode fr in
       let lp = mcode lp in
-      let exp1 = get_option (expression arity) exp1 in
-      let sem1 = mcode sem1 in
+      let first =
+	match Ast0.unwrap first with
+	  Ast0.ForExp(exp1,sem1) ->
+	    let exp1 = get_option (expression arity) exp1 in
+	    let sem1 = mcode sem1 in
+	    Ast0.rewrap first (Ast0.ForExp(exp1,sem1))
+	| Ast0.ForDecl (bef,decl) ->
+	    Ast0.rewrap first
+	      (Ast0.ForDecl(bef,declaration arity decl)) in
       let exp2 = get_option (expression arity) exp2 in
-      let sem2= mcode sem2 in
+      let sem2 = mcode sem2 in
       let exp3 = get_option (expression arity) exp3 in
       let rp = mcode rp in
       let body = statement arity body in
       make_rule_elem stm tgt arity
-	(Ast0.For(fr,lp,exp1,sem1,exp2,sem2,exp3,rp,body,aft))
+	(Ast0.For(fr,lp,first,exp2,sem2,exp3,rp,body,aft))
   | Ast0.Iterator(nm,lp,args,rp,body,aft) ->
       let arity = stm_same (mcode2line lp) (List.map mcode2arity [lp;rp]) in
       let nm = ident false arity nm in
@@ -835,6 +977,14 @@ and statement tgt stm =
       let exp = expression arity exp in
       let sem = mcode sem in
       make_rule_elem stm tgt arity (Ast0.ReturnExpr(ret,exp,sem))
+  | Ast0.Exec(exec,lang,code,sem) ->
+      let arity =
+	stm_same (mcode2line exec) (List.map mcode2arity [exec;lang;sem]) in
+      let exec = mcode exec in
+      let lang = mcode lang in
+      let code = dots (exec_code arity) code in
+      let sem = mcode sem in
+      make_rule_elem stm tgt arity (Ast0.Exec(exec,lang,code,sem))
   | Ast0.MetaStmt(name,pure) ->
       let arity = stm_same (mcode2line name) [mcode2arity name] in
       let name = mcode name in
@@ -955,7 +1105,7 @@ and statement tgt stm =
 	     (expression Ast0.NONE))
 	  whn in
       make_rule_elem stm tgt arity (Ast0.Stars(dots,whn))
-  | Ast0.FunDecl(bef,fi,name,lp,params,rp,lbrace,body,rbrace) ->
+  | Ast0.FunDecl(bef,fi,name,lp,params,va,rp,lbrace,body,rbrace,aft) ->
       let arity =
 	all_same true tgt (mcode2line lp)
 	  ((List.map mcode2arity [lp;rp;lbrace;rbrace]) @ (fninfo2arity fi)) in
@@ -963,12 +1113,15 @@ and statement tgt stm =
       let name = ident false arity name in
       let lp = mcode lp in
       let params = parameter_list arity params in
+      let newva = match va with
+        | None -> None
+        | Some (comma, ellipsis) -> Some (mcode comma, mcode ellipsis) in
       let rp = mcode rp in
       let lbrace = mcode lbrace in
       let body = dots (statement arity) body in
       let rbrace = mcode rbrace in
       make_rule_elem stm tgt arity
-	(Ast0.FunDecl(bef,fi,name,lp,params,rp,lbrace,body,rbrace))
+	(Ast0.FunDecl(bef,fi,name,lp,params,newva,rp,lbrace,body,rbrace,aft))
   | Ast0.Include(inc,s) ->
       let arity =
 	all_same true tgt (mcode2line inc) [mcode2arity inc; mcode2arity s] in
@@ -987,9 +1140,37 @@ and statement tgt stm =
       let params = define_parameters arity params in
       let body = dots (statement arity) body in
       make_rule_elem stm tgt arity (Ast0.Define(def,id,params,body))
-  | Ast0.OptStm(_) | Ast0.UniqueStm(_) ->
+  | Ast0.Pragma(prg,id,body) ->
+      let arity = all_same true tgt (mcode2line prg) [mcode2arity prg] in
+      let prg = mcode prg in
+      let id = ident false arity id in
+      let body = pragmainfo arity body in
+      make_rule_elem stm tgt arity (Ast0.Pragma(prg,id,body))
+  | Ast0.OptStm(_) | Ast0.UniqueStm(_) | Ast0.AsStmt _ ->
       failwith "unexpected code"
 
+and make_pragma =
+  make_opt_unique
+    (function x -> failwith "opt not allowed for pragma")
+    (function x -> failwith "unique not allowed for pragma")
+
+and pragmainfo tgt pi =
+  match Ast0.unwrap pi with
+    Ast0.PragmaTuple(lp,args,rp) ->
+      let arity =
+	all_same false tgt (mcode2line lp) [mcode2arity lp;mcode2arity rp] in
+      let lp = mcode lp in
+      let args = dots (expression arity) args in
+      let rp = mcode rp in
+      make_pragma pi tgt arity (Ast0.PragmaTuple(lp,args,rp))
+  | Ast0.PragmaIdList(ids) ->
+      let ids = dots (ident false tgt) ids in
+      make_pragma pi tgt tgt (Ast0.PragmaIdList(ids))
+  | Ast0.PragmaDots (dots) ->
+      let arity = all_same false tgt (mcode2line dots) [mcode2arity dots] in
+      let dots = mcode dots in
+      make_pragma pi tgt arity (Ast0.PragmaDots (dots))
+
 and define_parameters tgt params =
   match Ast0.unwrap params with
     Ast0.NoParams -> params
@@ -1053,11 +1234,11 @@ and fninfo2arity fninfo =
        fninfo)
 
 and whencode notfn alwaysfn expression = function
-    Ast0.WhenNot a -> Ast0.WhenNot (notfn a)
-  | Ast0.WhenAlways a -> Ast0.WhenAlways (alwaysfn a)
-  | Ast0.WhenModifier(x) -> Ast0.WhenModifier(x)
-  | Ast0.WhenNotTrue a -> Ast0.WhenNotTrue (expression a)
-  | Ast0.WhenNotFalse a -> Ast0.WhenNotFalse (expression a)
+    Ast0.WhenNot (w,e,a) -> Ast0.WhenNot (w,e,notfn a)
+  | Ast0.WhenAlways (w,e,a) -> Ast0.WhenAlways (w,e,alwaysfn a)
+  | Ast0.WhenModifier(w,x) -> Ast0.WhenModifier(w,x)
+  | Ast0.WhenNotTrue (w,e,a) -> Ast0.WhenNotTrue (w,e,expression a)
+  | Ast0.WhenNotFalse (w,e,a) -> Ast0.WhenNotFalse (w,e,expression a)
 
 and make_case_line =
   make_opt_unique
@@ -1093,6 +1274,27 @@ and case_line tgt c =
       Ast0.rewrap c (Ast0.DisjCase(starter,case_lines,mids,ender))
   | Ast0.OptCase(_) -> failwith "unexpected OptCase"
 
+and make_exec_code =
+  make_opt_unique
+    (function x -> failwith "opt not allowed for exec code")
+    (function x -> failwith "unique not allowed for exec code")
+
+and exec_code tgt e =
+  match Ast0.unwrap e with
+    Ast0.ExecEval(colon,id) ->
+      let arity = all_same false tgt (mcode2line colon) [mcode2arity colon] in
+      let colon = mcode colon in
+      let id = expression tgt id in
+      make_exec_code e tgt arity (Ast0.ExecEval(colon,id))
+  | Ast0.ExecToken(tok) ->
+      let arity = all_same false tgt (mcode2line tok) [mcode2arity tok] in
+      let tok = mcode tok in
+      make_exec_code e tgt arity (Ast0.ExecToken(tok))
+  | Ast0.ExecDots(dots) ->
+      let arity = all_same false tgt (mcode2line dots) [mcode2arity dots] in
+      let dots = mcode dots in
+      make_exec_code e tgt arity (Ast0.ExecDots(dots))
+
 (* --------------------------------------------------------------------- *)
 (* Function declaration *)
 (* Haven't thought much about arity here... *)
@@ -1104,10 +1306,11 @@ let top_level tgt t =
 	if mcode2arity old_file = Ast0.NONE && mcode2arity new_file = Ast0.NONE
 	then Ast0.FILEINFO(mcode old_file,mcode new_file)
 	else fail t "unexpected arity for file info"
-    | Ast0.DECL(stmt) ->
-	Ast0.DECL(statement tgt stmt)
+    | Ast0.NONDECL(stmt) ->
+	Ast0.NONDECL(statement tgt stmt)
     | Ast0.CODE(rule_elem_dots) ->
 	Ast0.CODE(concat_dots (statement tgt) rule_elem_dots)
+    | Ast0.TOPCODE(rule_elem_dots) ->  fail t "eliminated by top_level"
     | Ast0.ERRORWORDS(exps) ->
 	Ast0.ERRORWORDS(List.map (top_expression false Ast0.NONE) exps)
     | Ast0.OTHER(_) -> fail t "eliminated by top_level")
