@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./ast0_cocci.ml"
 module Ast = Ast_cocci
 module TC = Type_cocci
 
@@ -32,16 +35,15 @@ type arity = OPT | UNIQUE | NONE
 
 type token_info =
     { tline_start : int; tline_end : int;
-      left_offset : int; right_offset : int }
+      left_offset : int; right_offset : int; }
 let default_token_info =
   { tline_start = -1; tline_end = -1; left_offset = -1; right_offset = -1 }
 
 (* MIXED is like CONTEXT, since sometimes MIXED things have to revert to
 CONTEXT - see insert_plus.ml *)
-type count = ONE (* + *) | MANY (* ++ *)
 
 type mcodekind =
-    MINUS       of (Ast.anything list list * token_info) ref
+    MINUS       of (Ast.anything Ast.replacement * token_info) ref
   | PLUS        of Ast.count
   | CONTEXT     of (Ast.anything Ast.befaft * token_info * token_info) ref
   | MIXED       of (Ast.anything Ast.befaft * token_info * token_info) ref
@@ -51,18 +53,24 @@ type position_info = { line_start : int; line_end : int;
 		       column : int; offset : int; }
 
 type info = { pos_info : position_info;
+              (* preceding whitespace*) whitespace : string;
 	      attachable_start : bool; attachable_end : bool;
 	      mcode_start : mcodekind list; mcode_end : mcodekind list;
 	      (* the following are only for + code *)
 	      strings_before : (Ast.added_string * position_info) list;
-	      strings_after : (Ast.added_string * position_info) list }
+	      strings_after : (Ast.added_string * position_info) list;
+	      isSymbolIdent : bool; (* is the token a symbol identifier or not *) }
 
 (* adjacency index is incremented when we skip over dots or nest delimiters
 it is used in deciding how much to remove, when two adjacent code tokens are
 removed. *)
+type adjacency = int
+
+type fake_mcode = info * mcodekind * adjacency
+
 type 'a mcode =
-    'a * arity * info * mcodekind * meta_pos ref (* pos, - only *) *
-      int (* adjacency_index *)
+    'a * arity * info * mcodekind * anything list ref (* pos, - only *) *
+      adjacency (* adjacency_index *)
 (* int ref is an index *)
 and 'a wrap =
     { node : 'a;
@@ -104,9 +112,10 @@ and 'a dots = 'a base_dots wrap
 
 and base_ident =
     Id            of string mcode
-  | MetaId        of Ast.meta_name mcode * Ast.idconstraint * pure
+  | MetaId        of Ast.meta_name mcode * Ast.idconstraint * Ast.seed * pure
   | MetaFunc      of Ast.meta_name mcode * Ast.idconstraint * pure
   | MetaLocalFunc of Ast.meta_name mcode * Ast.idconstraint * pure
+  | AsIdent       of ident * ident (* as ident, always metavar *)
   | DisjId        of string mcode * ident list *
                      string mcode list (* the |s *) * string mcode
   | OptIdent      of ident
@@ -120,17 +129,20 @@ and ident = base_ident wrap
 and base_expression =
     Ident          of ident
   | Constant       of Ast.constant mcode
+  | StringConstant of string mcode (* quote *) * string_fragment dots *
+		      string mcode (* quote *)
   | FunCall        of expression * string mcode (* ( *) *
                       expression dots * string mcode (* ) *)
-  | Assignment     of expression * Ast.assignOp mcode * expression *
+  | Assignment     of expression * assignOp * expression *
 	              bool (* true if it can match an initialization *)
+  | Sequence       of expression * string mcode (* , *) * expression
   | CondExpr       of expression * string mcode (* ? *) * expression option *
 	              string mcode (* : *) * expression
   | Postfix        of expression * Ast.fixOp mcode
   | Infix          of expression * Ast.fixOp mcode
   | Unary          of expression * Ast.unaryOp mcode
-  | Binary         of expression * Ast.binaryOp mcode * expression
-  | Nested         of expression * Ast.binaryOp mcode * expression
+  | Binary         of expression * binaryOp * expression
+  | Nested         of expression * binaryOp * expression
   | Paren          of string mcode (* ( *) * expression *
                       string mcode (* ) *)
   | ArrayAccess    of expression * string mcode (* [ *) * expression *
@@ -143,19 +155,26 @@ and base_expression =
   | SizeOfType     of string mcode (* sizeof *) * string mcode (* ( *) *
                       typeC * string mcode (* ) *)
   | TypeExp        of typeC (* type name used as an expression, only in args *)
+  | Constructor    of string mcode (* ( *) * typeC * string mcode (* ) *) *
+	              initialiser
   | MetaErr        of Ast.meta_name mcode * constraints * pure
   | MetaExpr       of Ast.meta_name mcode * constraints *
 	              TC.typeC list option * Ast.form * pure
   | MetaExprList   of Ast.meta_name mcode (* only in arg lists *) *
 	              listlen * pure
+  | AsExpr         of expression * expression (* as expr, always metavar *)
   | EComma         of string mcode (* only in arg lists *)
   | DisjExpr       of string mcode * expression list *
 	              string mcode list (* the |s *) * string mcode
   | NestExpr       of string mcode * expression dots * string mcode *
-	              expression option * Ast.multi
-  | Edots          of string mcode (* ... *) * expression option
-  | Ecircles       of string mcode (* ooo *) * expression option
-  | Estars         of string mcode (* *** *) * expression option
+	              (string mcode * string mcode * expression) option
+	              (* whencode *) * Ast.multi
+  | Edots          of string mcode (* ... *) * (string mcode * string mcode *
+                      expression) option (* whencode *)
+  | Ecircles       of string mcode (* ooo *) * (string mcode * string mcode *
+	              expression) option (* whencode *)
+  | Estars         of string mcode (* *** *) * (string mcode * string mcode *
+	              expression) option (* whencode *)
   | OptExp         of expression
   | UniqueExp      of expression
 
@@ -172,6 +191,42 @@ and listlen =
   | CstListLen of int
   | AnyListLen
 
+and base_string_fragment =
+    ConstantFragment of string mcode
+  | FormatFragment of string mcode (*%*) * string_format (* format *)
+  | Strdots of string mcode
+  | MetaFormatList of string mcode (*%*) * Ast.meta_name mcode * listlen
+
+and string_fragment = base_string_fragment wrap
+
+and base_string_format =
+    ConstantFormat of string mcode
+  | MetaFormat of Ast.meta_name mcode * Ast.idconstraint
+
+and string_format = base_string_format wrap
+
+(* --------------------------------------------------------------------- *)
+(* First class operators *)
+and base_assignOp = 
+    SimpleAssign of simpleAssignOp mcode
+  | OpAssign of Ast_cocci.arithOp mcode
+  | MetaAssign of Ast_cocci.meta_name mcode * assignOpconstraint * pure
+and simpleAssignOp = string
+and assignOp = base_assignOp wrap
+
+and base_binaryOp =
+    Arith of Ast_cocci.arithOp mcode
+  | Logical of Ast_cocci.logicalOp mcode
+  | MetaBinary of Ast_cocci.meta_name mcode * binaryOpconstraint * pure
+and binaryOp = base_binaryOp wrap
+and assignOpconstraint =
+    AssignOpNoConstraint
+  | AssignOpInSet of assignOp list
+
+and binaryOpconstraint =
+    BinaryOpNoConstraint
+  | BinaryOpInSet of binaryOp list
+
 (* --------------------------------------------------------------------- *)
 (* Types *)
 
@@ -183,11 +238,12 @@ and base_typeC =
   | FunctionPointer of typeC *
 	          string mcode(* ( *)*string mcode(* * *)*string mcode(* ) *)*
                   string mcode (* ( *)*parameter_list*string mcode(* ) *)
-  | FunctionType    of typeC option *
-	               string mcode (* ( *) * parameter_list *
-                       string mcode (* ) *)
   | Array           of typeC * string mcode (* [ *) *
 	               expression option * string mcode (* ] *)
+  | Decimal         of string mcode (* decimal *) * string mcode (* ( *) *
+	               expression *
+	               string mcode option (* , *) * expression option *
+	               string mcode (* ) *) (* IBM C only *)
   | EnumName        of string mcode (*enum*) * ident option (* name *)
   | EnumDef  of typeC (* either StructUnionName or metavar *) *
 	string mcode (* { *) * expression dots * string mcode (* } *)
@@ -196,6 +252,7 @@ and base_typeC =
 	string mcode (* { *) * declaration dots * string mcode (* } *)
   | TypeName        of string mcode
   | MetaType        of Ast.meta_name mcode * pure
+  | AsType          of typeC * typeC (* as type, always metavar *)
   | DisjType        of string mcode * typeC list * (* only after iso *)
                        string mcode list (* the |s *)  * string mcode
   | OptType         of typeC
@@ -215,17 +272,28 @@ and base_declaration =
        should be a separate type for fields, as in the C AST *)
   | MetaField of Ast.meta_name mcode * pure (* structure fields *)
   | MetaFieldList of Ast.meta_name mcode * listlen * pure (* structure fields *)
+  | AsDecl        of declaration * declaration
   | Init of Ast.storage mcode option * typeC * ident * string mcode (*=*) *
 	initialiser * string mcode (*;*)
   | UnInit of Ast.storage mcode option * typeC * ident * string mcode (* ; *)
+  | FunProto of
+	fninfo list * ident (* name *) *
+	string mcode (* ( *) * parameter_list *
+        (string mcode (* , *) * string mcode (* ...... *) ) option *
+	string mcode (* ) *) *
+	string mcode (* ; *)
   | TyDecl of typeC * string mcode (* ; *)
   | MacroDecl of ident (* name *) * string mcode (* ( *) *
         expression dots * string mcode (* ) *) * string mcode (* ; *)
+  | MacroDeclInit of ident (* name *) * string mcode (* ( *) *
+        expression dots * string mcode (* ) *) * string mcode (*=*) *
+	initialiser * string mcode (* ; *)
   | Typedef of string mcode (* typedef *) * typeC * typeC * string mcode (*;*)
   | DisjDecl   of string mcode * declaration list *
-                  string mcode list (* the |s *)  * string mcode
+	          string mcode list (* the |s *)  * string mcode
   (* Ddots is for a structure declaration *)
-  | Ddots      of string mcode (* ... *) * declaration option (* whencode *)
+  | Ddots      of string mcode (* ... *) * (string mcode * string mcode *
+	          declaration) option (* whencode *)
   | OptDecl    of declaration
   | UniqueDecl of declaration
 
@@ -236,6 +304,8 @@ and declaration = base_declaration wrap
 
 and base_initialiser =
     MetaInit of Ast.meta_name mcode * pure
+  | MetaInitList of Ast.meta_name mcode * listlen * pure
+  | AsInit of initialiser * initialiser (* as init, always metavar *)
   | InitExpr of expression
   | InitList of string mcode (*{*) * initialiser_list * string mcode (*}*) *
 	(* true if ordered, as for array, false if unordered, as for struct *)
@@ -246,7 +316,8 @@ and base_initialiser =
   | InitGccName of ident (* name *) * string mcode (*:*) *
 	initialiser
   | IComma of string mcode (* , *)
-  | Idots  of string mcode (* ... *) * initialiser option (* whencode *)
+  | Idots  of string mcode (* ... *) *
+              (string mcode * string mcode * initialiser) option (* whencode *)
   | OptIni    of initialiser
   | UniqueIni of initialiser
 
@@ -269,6 +340,7 @@ and base_parameterTypeDef =
   | Param         of typeC * ident option
   | MetaParam     of Ast.meta_name mcode * pure
   | MetaParamList of Ast.meta_name mcode * listlen * pure
+  | AsParam       of parameterTypeDef * expression (* expr, always metavar *)
   | PComma        of string mcode
   | Pdots         of string mcode (* ... *)
   | Pcircles      of string mcode (* ooo *)
@@ -302,32 +374,32 @@ and define_parameters = base_define_parameters wrap
 (* Statement*)
 
 and base_statement =
+    (*Decl and FunDecl don't need adjacency.  Delete all comments in any case*)
     Decl          of (info * mcodekind) (* before the decl *) * declaration
   | Seq           of string mcode (* { *) * statement dots *
  	             string mcode (* } *)
-  | ExprStatement of expression * string mcode (*;*)
+  | ExprStatement of expression option * string mcode (*;*)
   | IfThen        of string mcode (* if *) * string mcode (* ( *) *
 	             expression * string mcode (* ) *) *
-	             statement * (info * mcodekind) (* after info *)
+	             statement * fake_mcode (* after info *)
   | IfThenElse    of string mcode (* if *) * string mcode (* ( *) *
 	             expression * string mcode (* ) *) *
 	             statement * string mcode (* else *) * statement *
-	             (info * mcodekind)
+	             fake_mcode (* after info *)
   | While         of string mcode (* while *) * string mcode (* ( *) *
 	             expression * string mcode (* ) *) *
-	             statement * (info * mcodekind) (* after info *)
+	             statement * fake_mcode (* after info *)
   | Do            of string mcode (* do *) * statement *
                      string mcode (* while *) * string mcode (* ( *) *
 	             expression * string mcode (* ) *) *
                      string mcode (* ; *)
-  | For           of string mcode (* for *) * string mcode (* ( *) *
-                     expression option * string mcode (*;*) *
+  | For           of string mcode (* for *) * string mcode (* ( *) * forinfo *
 	             expression option * string mcode (*;*) *
                      expression option * string mcode (* ) *) * statement *
-	             (info * mcodekind) (* after info *)
+	             fake_mcode (* after info *)
   | Iterator      of ident (* name *) * string mcode (* ( *) *
 	             expression dots * string mcode (* ) *) *
-	             statement * (info * mcodekind) (* after info *)
+	             statement * fake_mcode (* after info *)
   | Switch        of string mcode (* switch *) * string mcode (* ( *) *
 	             expression * string mcode (* ) *) * string mcode (* { *) *
 	             statement (*decl*) dots *
@@ -339,8 +411,11 @@ and base_statement =
   | Return        of string mcode (* return *) * string mcode (* ; *)
   | ReturnExpr    of string mcode (* return *) * expression *
 	             string mcode (* ; *)
+  | Exec          of string mcode (* EXEC *) * string mcode (* language *) *
+	             exec_code dots * string mcode (* ; *)
   | MetaStmt      of Ast.meta_name mcode * pure
   | MetaStmtList  of Ast.meta_name mcode(*only in statement lists*) * pure
+  | AsStmt        of statement * statement (* as statement, always metavar *)
   | Exp           of expression  (* only in dotted statement lists *)
   | TopExp        of expression (* for macros body *)
   | Ty            of typeC (* only at top level *)
@@ -357,16 +432,33 @@ and base_statement =
 	             (statement dots,statement) whencode list
   | FunDecl of (info * mcodekind) (* before the function decl *) *
 	fninfo list * ident (* name *) *
-	string mcode (* ( *) * parameter_list * string mcode (* ) *) *
+	string mcode (* ( *) * parameter_list *
+	(string mcode (* , *) * string mcode (* ...... *) ) option *
+	string mcode (* ) *) *
 	string mcode (* { *) * statement dots *
-	string mcode (* } *)
+	string mcode (* } *) *
+	(info * mcodekind) (* after the function decl *)
   | Include of string mcode (* #include *) * Ast.inc_file mcode (* file *)
   | Undef of string mcode (* #define *) * ident (* name *)
   | Define of string mcode (* #define *) * ident (* name *) *
 	define_parameters (*params*) * statement dots
+  | Pragma of string mcode (* #pragma *) * ident * pragmainfo
   | OptStm   of statement
   | UniqueStm of statement
 
+and base_pragmainfo =
+    PragmaTuple of string mcode(* ( *) * expression dots * string mcode(* ) *)
+  | PragmaIdList of ident dots
+  | PragmaDots of string mcode
+
+and pragmainfo = base_pragmainfo wrap
+
+and base_forinfo =
+    ForExp of expression option * string mcode (*;*)
+  | ForDecl of (info * mcodekind) (* before the decl *) * declaration
+
+and forinfo = base_forinfo wrap
+
 and fninfo =
     FStorage of Ast.storage mcode
   | FType of typeC
@@ -374,11 +466,12 @@ and fninfo =
   | FAttr of string mcode
 
 and ('a,'b) whencode =
-    WhenNot of 'a
-  | WhenAlways of 'b
-  | WhenModifier of Ast.when_modifier
-  | WhenNotTrue of expression
-  | WhenNotFalse of expression
+    WhenNot of string mcode (* when *) * string mcode (* != *) * 'a
+  | WhenAlways of string mcode (* when *) * string mcode (* = *) * 'b
+  | WhenModifier of string mcode (* when *) * Ast.when_modifier
+  | WhenNotTrue of string mcode (* when *) * string mcode (* != *) * expression
+  | WhenNotFalse of string mcode (* when *) * string mcode (* != *) *
+    expression
 
 and statement = base_statement wrap
 
@@ -392,18 +485,25 @@ and base_case_line =
 
 and case_line = base_case_line wrap
 
+and base_exec_code =
+    ExecEval of string mcode (* : *) * expression
+  | ExecToken of string mcode
+  | ExecDots of string mcode (* ... *)
+
+and exec_code = base_exec_code wrap
+
 (* --------------------------------------------------------------------- *)
 (* Positions *)
 
 and meta_pos =
     MetaPos of Ast.meta_name mcode * Ast.meta_name list * Ast.meta_collect
-  | NoMetaPos
 
 (* --------------------------------------------------------------------- *)
 (* Top-level code *)
 
 and base_top_level =
-    DECL of statement
+    NONDECL of statement
+  | TOPCODE of statement dots
   | CODE of statement dots
   | FILEINFO of string mcode (* old file *) * string mcode (* new file *)
   | ERRORWORDS of expression list
@@ -414,16 +514,31 @@ and rule = top_level list
 
 and parsed_rule =
     CocciRule of
-      (rule * Ast.metavar list *
-	 (string list * string list * Ast.dependency * string * Ast.exists)) *
-	(rule * Ast.metavar list) * Ast.ruletype
-  | ScriptRule of string (* name *) *
-      string * Ast.dependency *
+      (rule (*minus*) * Ast.metavar list (*minus metavars*) *
+	(string list (*isos*) * string list (*drop_isos*) * 
+         Ast.dependency (*dependencies*) * string (*rulename*) * Ast.exists)) *
+      (rule (*plus*) * Ast.metavar list (*plus metavars*)) * Ast.ruletype
+  | ScriptRule of string (* name *) * string * Ast.dependency *
 	(Ast.script_meta_name * Ast.meta_name * Ast.metavar) list *
 	Ast.meta_name list (*script vars*) *
 	string
-  | InitialScriptRule of  string (* name *) *string * Ast.dependency * string
-  | FinalScriptRule of  string (* name *) *string * Ast.dependency * string
+  | InitialScriptRule of string (* name *) * string * Ast.dependency *
+	(Ast.script_meta_name * Ast.meta_name * Ast.metavar) list *
+	string
+  | FinalScriptRule of string (* name *) * string * Ast.dependency *
+	(Ast.script_meta_name * Ast.meta_name * Ast.metavar) list *
+	string (* no script vars possible here *)
+
+(* --------------------------------------------------------------------- *)
+
+and dependency =
+    Dep of string (* rule applies for the current binding *)
+  | AntiDep of dependency (* rule doesn't apply for the current binding *)
+  | EverDep of string (* rule applies for some binding *)
+  | NeverDep of string (* rule never applies for any binding *)
+  | AndDep of dependency * dependency
+  | OrDep of dependency * dependency
+  | NoDep | FailDep
 
 (* --------------------------------------------------------------------- *)
 
@@ -436,6 +551,8 @@ and anything =
   | DotsCaseTag of case_line dots
   | IdentTag of ident
   | ExprTag of expression
+  | AssignOpTag of assignOp
+  | BinaryOpTag of binaryOp
   | ArgExprTag of expression  (* for isos *)
   | TestExprTag of expression (* for isos *)
   | TypeCTag of typeC
@@ -443,12 +560,17 @@ and anything =
   | InitTag of initialiser
   | DeclTag of declaration
   | StmtTag of statement
+  | ForInfoTag of forinfo
   | CaseLineTag of case_line
+  | StringFragmentTag of string_fragment
   | TopTag of top_level
   | IsoWhenTag of Ast.when_modifier
   | IsoWhenTTag of expression
   | IsoWhenFTag of expression
   | MetaPosTag of meta_pos
+  | HiddenVarTag of anything list (* in iso_compile/pattern only *)
+  | WhenTag of string mcode (* when *) * string mcode option
+      (* !=, =, or none if whenmodifier*) * anything (* iso pattern *)
 
 let dotsExpr x = DotsExprTag x
 let dotsParam x = DotsParamTag x
@@ -458,12 +580,16 @@ let dotsDecl x = DotsDeclTag x
 let dotsCase x = DotsCaseTag x
 let ident x = IdentTag x
 let expr x = ExprTag x
+let assignOp x = AssignOpTag x
+let binaryOp x = BinaryOpTag x
 let typeC x = TypeCTag x
 let param x = ParamTag x
 let ini x = InitTag x
 let decl x = DeclTag x
 let stmt x = StmtTag x
+let forinfo x = ForInfoTag x
 let case_line x = CaseLineTag x
+let string_fragment x = StringFragmentTag x
 let top x = TopTag x
 
 (* --------------------------------------------------------------------- *)
@@ -475,16 +601,16 @@ let pos_info =
     column = -1; offset = -1; }
 
 let default_info _ = (* why is this a function? *)
-  { pos_info = pos_info;
+  { pos_info = pos_info; whitespace = "";
     attachable_start = true; attachable_end = true;
     mcode_start = []; mcode_end = [];
-    strings_before = []; strings_after = [] }
+    strings_before = []; strings_after = []; isSymbolIdent = false; }
 
 let default_befaft _ =
   MIXED(ref (Ast.NOTHING,default_token_info,default_token_info))
 let context_befaft _ =
   CONTEXT(ref (Ast.NOTHING,default_token_info,default_token_info))
-let minus_befaft _ = MINUS(ref ([],default_token_info))
+let minus_befaft _ = MINUS(ref (Ast.NOREPLACEMENT,default_token_info))
 
 let wrap x =
   { node = x;
@@ -528,6 +654,7 @@ let get_index x     = !(x.index)
 let set_index x i   = x.index := i
 let get_mcodekind x = !(x.mcodekind)
 let get_mcode_mcodekind (_,_,_,mcodekind,_,_) = mcodekind
+let get_mcode_line (_,_,info,_,_,_) = info.pos_info.line_start
 let get_mcodekind_ref x = x.mcodekind
 let set_mcodekind x mk  = x.mcodekind := mk
 let set_type x t        = x.exp_ty := t
@@ -540,9 +667,47 @@ let get_test_pos x      = x.true_if_test
 let set_test_pos x      = {x with true_if_test = true}
 let get_test_exp x      = x.true_if_test_exp
 let set_test_exp x      = {x with true_if_test_exp = true}
+let clear_test_exp x      = {x with true_if_test_exp = false}
 let get_iso x           = x.iso_info
 let set_iso x i = if !Flag.track_iso_usage then {x with iso_info = i} else x
 let set_mcode_data data (_,ar,info,mc,pos,adj) = (data,ar,info,mc,pos,adj)
+let get_rule_name = function
+  | CocciRule ((_,_,(_,_,_,nm,_)),_,_) | InitialScriptRule (nm,_,_,_,_)
+  | FinalScriptRule (nm,_,_,_,_) | ScriptRule (nm,_,_,_,_,_) -> nm
+
+(* --------------------------------------------------------------------- *)
+
+let rec meta_pos_name = function
+    HiddenVarTag(vars) ->
+	(* totally fake, just drop the rest, only for isos *)
+      meta_pos_name (List.hd vars)
+  | MetaPosTag(MetaPos(name,constraints,_)) -> name
+  | IdentTag(i) ->
+      (match unwrap i with
+	MetaId(name,constraints,seed,pure) -> name
+      | _ -> failwith "bad metavariable")
+  | ExprTag(e) ->
+      (match unwrap e with
+	MetaExpr(name,constraints,ty,form,pure) -> name
+      | MetaExprList(name,len,pure) -> name
+      | _ -> failwith "bad metavariable")
+  | TypeCTag(t) ->
+      (match unwrap t with
+	MetaType(name,pure) -> name
+      | _ -> failwith "bad metavariable")
+  | DeclTag(d) ->
+      (match unwrap d with
+	MetaDecl(name,pure) -> name
+      | _ -> failwith "bad metavariable")
+  | InitTag(i) ->
+      (match unwrap i with
+	MetaInit(name,pure) -> name
+      | _ -> failwith "bad metavariable")
+  | StmtTag(s) ->
+      (match unwrap s with
+	MetaStmt(name,pure) -> name
+      | _ -> failwith "bad metavariable")
+  | _ -> failwith "bad metavariable"
 
 (* --------------------------------------------------------------------- *)
 
@@ -560,71 +725,96 @@ let undots d =
 
 (* --------------------------------------------------------------------- *)
 
-let rec ast0_type_to_type ty =
+let rec ast0_type_to_type inmeta ty =
   match unwrap ty with
-    ConstVol(cv,ty) -> TC.ConstVol(const_vol cv,ast0_type_to_type ty)
+    ConstVol(cv,ty) -> TC.ConstVol(const_vol cv,ast0_type_to_type inmeta ty)
   | BaseType(bty,strings) ->
       TC.BaseType(baseType bty)
   | Signed(sgn,None) ->
       TC.SignedT(sign sgn,None)
   | Signed(sgn,Some ty) ->
-      let bty = ast0_type_to_type ty in
+      let bty = ast0_type_to_type inmeta ty in
       TC.SignedT(sign sgn,Some bty)
-  | Pointer(ty,_) -> TC.Pointer(ast0_type_to_type ty)
+  | Pointer(ty,_) -> TC.Pointer(ast0_type_to_type inmeta ty)
   | FunctionPointer(ty,_,_,_,_,params,_) ->
-      TC.FunctionPointer(ast0_type_to_type ty)
-  | FunctionType _ -> failwith "not supported"
-  | Array(ety,_,_,_) -> TC.Array(ast0_type_to_type ety)
+      TC.FunctionPointer(ast0_type_to_type inmeta ty)
+  | Array(ety,_,_,_) -> TC.Array(ast0_type_to_type inmeta ety)
+  | Decimal(_, _, e1, _, e2, _) ->
+      let e2tc e =
+	match unwrap e with
+	  Constant(c) ->
+	    (match unwrap_mcode c with
+	      Ast.Int n -> TC.Num n
+	    | _ -> failwith "not possible")
+	| Ident(id) ->
+	    (match unwrap id with
+	      Id n -> TC.Name (unwrap_mcode n)
+	    | _ -> failwith "not possible")
+	| MetaExpr(name,NoConstraint,None,Ast.CONST,_) ->
+	    TC.MV(unwrap_mcode name,TC.Unitary,false)
+	| _ -> failwith "unexpected argument to decimal" in
+      let e2 =
+	match e2 with
+	  None -> TC.Num "0"
+	| Some e2 -> e2tc e2 in
+      TC.Decimal(e2tc e1,e2)
   | EnumName(su,Some tag) ->
       (match unwrap tag with
 	Id(tag) ->
 	  TC.EnumName(TC.Name(unwrap_mcode tag))
-      | MetaId(tag,_,_) ->
-	  (Printf.printf
-	     "warning: enum with a metavariable name detected.\n";
-	   Printf.printf
-	     "For type checking assuming the name of the metavariable is the name of the type\n";
-	   TC.EnumName(TC.MV(unwrap_mcode tag,TC.Unitary,false)))
-      | _ -> failwith "unexpected enum type name")
-  | EnumName(su,None) -> failwith "nameless enum - what to do???"
-  | EnumDef(ty,_,_,_) -> ast0_type_to_type ty
+      | MetaId(tag,Ast.IdNoConstraint,_,_) when inmeta ->
+	  TC.EnumName(TC.MV(unwrap_mcode tag,TC.Unitary,false))
+      | MetaId(tag,_,_,_) when inmeta ->
+	  (* would have to duplicate the type in type_cocci.ml?
+	     perhaps polymorphism would help? *)
+	  failwith "constraints not supported on enum type name"
+      | _ ->
+	  (* can't arise for metavariables and doesn't matter for type
+	     checking *)
+	  TC.EnumName(TC.NoName))
+  | EnumName(su,None) -> TC.EnumName TC.NoName
+  | EnumDef(ty,_,_,_) -> ast0_type_to_type inmeta ty
   | StructUnionName(su,Some tag) ->
       (match unwrap tag with
 	Id(tag) ->
 	  TC.StructUnionName(structUnion su,TC.Name(unwrap_mcode tag))
-      | MetaId(tag,Ast.IdNoConstraint,_) ->
-	  (Common.pr2
-	     "warning: struct/union with a metavariable name detected.\n";
-	   Common.pr2
-	     "For type checking assuming the name of the metavariable is the name of the type\n";
-	   TC.StructUnionName(structUnion su,
-			      TC.MV(unwrap_mcode tag,TC.Unitary,false)))
-      | MetaId(tag,_,_) ->
+      | MetaId(tag,Ast.IdNoConstraint,_,_) when inmeta ->
+	  TC.StructUnionName(structUnion su,
+			     TC.MV(unwrap_mcode tag,TC.Unitary,false))
+      | MetaId(tag,_,_,_) when inmeta ->
 	  (* would have to duplicate the type in type_cocci.ml?
 	     perhaps polymorphism would help? *)
 	  failwith "constraints not supported on struct type name"
-      | _ -> failwith "unexpected struct/union type name")
-  | StructUnionName(su,None) -> failwith "nameless structure - what to do???"
-  | StructUnionDef(ty,_,_,_) -> ast0_type_to_type ty
+      | _ ->
+	  (* can't arise for metavariables and doesn't matter for type
+	     checking *)
+	  TC.StructUnionName(structUnion su,TC.NoName))
+  | StructUnionName(su,None) -> TC.StructUnionName(structUnion su,TC.NoName)
+  | StructUnionDef(ty,_,_,_) -> ast0_type_to_type inmeta ty
   | TypeName(name) -> TC.TypeName(unwrap_mcode name)
   | MetaType(name,_) ->
       TC.MetaType(unwrap_mcode name,TC.Unitary,false)
+  | AsType(ty,asty) -> failwith "not created yet"
   | DisjType(_,types,_,_) ->
       Common.pr2_once
 	"disjtype not supported in smpl type inference, assuming unknown";
       TC.Unknown
   | OptType(ty) | UniqueType(ty) ->
-      ast0_type_to_type ty
+      ast0_type_to_type inmeta ty
 
 and baseType = function
     Ast.VoidType -> TC.VoidType
   | Ast.CharType -> TC.CharType
   | Ast.ShortType -> TC.ShortType
+  | Ast.ShortIntType -> TC.ShortIntType
   | Ast.IntType -> TC.IntType
   | Ast.DoubleType -> TC.DoubleType
+  | Ast.LongDoubleType -> TC.LongDoubleType
   | Ast.FloatType -> TC.FloatType
   | Ast.LongType -> TC.LongType
+  | Ast.LongIntType -> TC.LongIntType
   | Ast.LongLongType -> TC.LongLongType
+  | Ast.LongLongIntType -> TC.LongLongIntType
   | Ast.SizeType -> TC.SizeType
   | Ast.SSizeType -> TC.SSizeType
   | Ast.PtrDiffType -> TC.PtrDiffType
@@ -648,10 +838,10 @@ and const_vol t =
 (* this function is a rather minimal attempt.  the problem is that information
 has been lost.  but since it is only used for metavariable types in the isos,
 perhaps it doesn't matter *)
-and make_mcode x = (x,NONE,default_info(),context_befaft(),ref NoMetaPos,-1)
-let make_mcode_info x info = (x,NONE,info,context_befaft(),ref NoMetaPos,-1)
+and make_mcode x = (x,NONE,default_info(),context_befaft(),ref [],-1)
+let make_mcode_info x info = (x,NONE,info,context_befaft(),ref [],-1)
 and make_minus_mcode x =
-  (x,NONE,default_info(),minus_befaft(),ref NoMetaPos,-1)
+  (x,NONE,default_info(),minus_befaft(),ref [],-1)
 
 exception TyConv
 
@@ -669,7 +859,7 @@ let rec reverse_type ty =
   | TC.EnumName(TC.MV(name,_,_)) ->
       EnumName
 	(make_mcode "enum",
-	 Some (context_wrap(MetaId(make_mcode name,Ast.IdNoConstraint,
+	 Some (context_wrap(MetaId(make_mcode name,Ast.IdNoConstraint,Ast.NoVal,
 				   Impure))))
   | TC.EnumName(TC.Name tag) ->
       EnumName(make_mcode "enum",Some(context_wrap(Id(make_mcode tag))))
@@ -677,7 +867,7 @@ let rec reverse_type ty =
       (* not right?... *)
       StructUnionName
 	(reverse_structUnion su,
-	 Some(context_wrap(MetaId(make_mcode name,Ast.IdNoConstraint,
+	 Some(context_wrap(MetaId(make_mcode name,Ast.IdNoConstraint,Ast.NoVal,
 				  Impure(*not really right*)))))
   |  TC.StructUnionName(su,TC.Name tag) ->
       StructUnionName
@@ -693,11 +883,15 @@ and reverse_baseType = function
   | TC.CharType -> Ast.CharType
   | TC.BoolType -> Ast.IntType
   | TC.ShortType -> Ast.ShortType
+  | TC.ShortIntType -> Ast.ShortIntType
   | TC.IntType -> Ast.IntType
   | TC.DoubleType -> Ast.DoubleType
+  | TC.LongDoubleType -> Ast.LongDoubleType
   | TC.FloatType -> Ast.FloatType
   | TC.LongType -> Ast.LongType
+  | TC.LongIntType -> Ast.LongIntType
   | TC.LongLongType -> Ast.LongLongType
+  | TC.LongLongIntType -> Ast.LongLongIntType
   | TC.SizeType -> Ast.SizeType
   | TC.SSizeType -> Ast.SSizeType
   | TC.PtrDiffType -> Ast.PtrDiffType
@@ -734,3 +928,16 @@ let lub_pure x y =
 (* --------------------------------------------------------------------- *)
 
 let rule_name = ref "" (* for the convenience of the parser *)
+
+let string_of_binaryOp op = match (unwrap op) with
+  | Arith arithOp -> Ast.string_of_arithOp (unwrap_mcode arithOp)
+  | Logical logicalOp -> Ast.string_of_logicalOp (unwrap_mcode logicalOp)
+  | MetaBinary _ -> "MetaBinary"
+
+let string_of_assignOp op = match (unwrap op) with
+  | SimpleAssign _ -> "="
+  | OpAssign op' ->
+    let op'' = rewrap op (Arith op') in
+    let s = string_of_binaryOp op'' in
+    s ^ "="
+  | MetaAssign _ -> "MetaAssign"
