@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./ast0_cocci.mli"
 (* --------------------------------------------------------------------- *)
 (* Modified code *)
 
@@ -33,7 +36,7 @@ type token_info =
 val default_token_info : token_info
 
 type mcodekind =
-    MINUS       of (Ast_cocci.anything list list * token_info) ref
+    MINUS       of (Ast_cocci.anything Ast_cocci.replacement * token_info) ref
   | PLUS        of Ast_cocci.count
   | CONTEXT     of (Ast_cocci.anything Ast_cocci.befaft *
 		      token_info * token_info) ref
@@ -44,16 +47,21 @@ type position_info = { line_start : int; line_end : int;
 		       logical_start : int; logical_end : int;
 		       column : int; offset : int; }
 
-type info = { pos_info : position_info;
+type info = { pos_info : position_info;  whitespace : string;
 	      attachable_start : bool; attachable_end : bool;
 	      mcode_start : mcodekind list; mcode_end : mcodekind list;
 	      (* the following are only for + code *)
 	      strings_before : (Ast_cocci.added_string * position_info) list;
-	      strings_after : (Ast_cocci.added_string * position_info) list }
+	      strings_after : (Ast_cocci.added_string * position_info) list;
+	      isSymbolIdent : bool; (* is the token a symbol identifier or not *) }
+
+type adjacency = int
+
+type fake_mcode = info * mcodekind * adjacency
 
 type 'a mcode =
-    'a * arity * info * mcodekind * meta_pos ref (* pos, - only *) *
-      int (* adjacency_index *)
+    'a * arity * info * mcodekind * anything list ref (* pos, - only *) *
+      adjacency (* adjacency_index *)
 
 and 'a wrap =
     { node : 'a;
@@ -93,9 +101,11 @@ and 'a dots = 'a base_dots wrap
 
 and base_ident =
     Id            of string mcode
-  | MetaId        of Ast_cocci.meta_name mcode * Ast_cocci.idconstraint * pure
+  | MetaId        of Ast_cocci.meta_name mcode * Ast_cocci.idconstraint *
+	Ast_cocci.seed * pure
   | MetaFunc      of Ast_cocci.meta_name mcode * Ast_cocci.idconstraint * pure
   | MetaLocalFunc of Ast_cocci.meta_name mcode * Ast_cocci.idconstraint * pure
+  | AsIdent       of ident * ident (* as ident, always metavar *)
   | DisjId        of string mcode * ident list *
                      string mcode list (* the |s *) * string mcode
   | OptIdent      of ident
@@ -109,17 +119,20 @@ and ident = base_ident wrap
 and base_expression =
     Ident          of ident
   | Constant       of Ast_cocci.constant mcode
+  | StringConstant of string mcode (* quote *) * string_fragment dots *
+		      string mcode (* quote *)
   | FunCall        of expression * string mcode (* ( *) *
                       expression dots * string mcode (* ) *)
-  | Assignment     of expression * Ast_cocci.assignOp mcode * expression *
+  | Assignment     of expression * assignOp * expression *
 	              bool (* true if it can match an initialization *)
+  | Sequence       of expression * string mcode (* , *) * expression
   | CondExpr       of expression * string mcode (* ? *) * expression option *
 	              string mcode (* : *) * expression
   | Postfix        of expression * Ast_cocci.fixOp mcode
   | Infix          of expression * Ast_cocci.fixOp mcode
   | Unary          of expression * Ast_cocci.unaryOp mcode
-  | Binary         of expression * Ast_cocci.binaryOp mcode * expression
-  | Nested         of expression * Ast_cocci.binaryOp mcode * expression
+  | Binary         of expression * binaryOp * expression
+  | Nested         of expression * binaryOp * expression
   | Paren          of string mcode (* ( *) * expression *
                       string mcode (* ) *)
   | ArrayAccess    of expression * string mcode (* [ *) * expression *
@@ -132,19 +145,26 @@ and base_expression =
   | SizeOfType     of string mcode (* sizeof *) * string mcode (* ( *) *
                       typeC * string mcode (* ) *)
   | TypeExp        of typeC
+  | Constructor    of string mcode (* ( *) * typeC * string mcode (* ) *) *
+ 	              initialiser
   | MetaErr        of Ast_cocci.meta_name mcode * constraints * pure
   | MetaExpr       of Ast_cocci.meta_name mcode * constraints *
 	              Type_cocci.typeC list option * Ast_cocci.form * pure
   | MetaExprList   of Ast_cocci.meta_name mcode (* only in arglists *) *
 	              listlen * pure
+  | AsExpr         of expression * expression (* as expr, always metavar *)
   | EComma         of string mcode (* only in arglists *)
   | DisjExpr       of string mcode * expression list * string mcode list *
 	              string mcode
   | NestExpr       of string mcode * expression dots * string mcode *
-	              expression option * Ast_cocci.multi
-  | Edots          of string mcode (* ... *) * expression option
-  | Ecircles       of string mcode (* ooo *) * expression option
-  | Estars         of string mcode (* *** *) * expression option
+                      (string mcode * string mcode * expression) option 
+                      (* whencode *) * Ast_cocci.multi
+  | Edots          of string mcode (* ... *) * (string mcode * string mcode *
+                      expression) option (* whencode *)
+  | Ecircles       of string mcode (* ooo *) * (string mcode * string mcode *
+                      expression) option (* whencode *)
+  | Estars         of string mcode (* *** *) * (string mcode * string mcode *
+                      expression) option (* whencode *)
   | OptExp         of expression
   | UniqueExp      of expression
 
@@ -161,6 +181,43 @@ and listlen =
   | CstListLen of int
   | AnyListLen
 
+and base_string_fragment =
+    ConstantFragment of string mcode
+  | FormatFragment of string mcode (*%*) * string_format (* format *)
+  | Strdots of string mcode
+  | MetaFormatList of string mcode (*%*) * Ast_cocci.meta_name mcode * listlen
+
+and string_fragment = base_string_fragment wrap
+
+and base_string_format =
+    ConstantFormat of string mcode
+  | MetaFormat of Ast_cocci.meta_name mcode * Ast_cocci.idconstraint
+
+and string_format = base_string_format wrap
+
+(* --------------------------------------------------------------------- *)
+(* First class operators *)
+and  base_assignOp = 
+    SimpleAssign of simpleAssignOp mcode
+  | OpAssign of Ast_cocci.arithOp mcode
+  | MetaAssign of Ast_cocci.meta_name mcode * assignOpconstraint * pure
+and simpleAssignOp = string
+and assignOp = base_assignOp wrap
+
+and  base_binaryOp =
+    Arith of Ast_cocci.arithOp mcode
+  | Logical of Ast_cocci.logicalOp mcode
+  | MetaBinary of Ast_cocci.meta_name mcode * binaryOpconstraint * pure
+and binaryOp = base_binaryOp wrap
+and assignOpconstraint =
+    AssignOpNoConstraint
+  | AssignOpInSet of assignOp list
+
+and binaryOpconstraint =
+    BinaryOpNoConstraint
+  | BinaryOpInSet of binaryOp list
+
+
 (* --------------------------------------------------------------------- *)
 (* Types *)
 
@@ -172,11 +229,12 @@ and base_typeC =
   | FunctionPointer of typeC *
 	          string mcode(* ( *)*string mcode(* * *)*string mcode(* ) *)*
                   string mcode (* ( *)*parameter_list*string mcode(* ) *)
-  | FunctionType    of typeC option *
-	               string mcode (* ( *) * parameter_list *
-                       string mcode (* ) *)
   | Array           of typeC * string mcode (* [ *) *
 	               expression option * string mcode (* ] *)
+  | Decimal         of string mcode (* decimal *) * string mcode (* ( *) *
+	               expression *
+	               string mcode option (* , *) * expression option *
+	               string mcode (* ) *) (* IBM C only *)
   | EnumName        of string mcode (*enum*) * ident option (* name *)
   | EnumDef  of typeC (* either StructUnionName or metavar *) *
 	string mcode (* { *) * expression dots * string mcode (* } *)
@@ -185,6 +243,7 @@ and base_typeC =
 	string mcode (* { *) * declaration dots * string mcode (* } *)
   | TypeName        of string mcode
   | MetaType        of Ast_cocci.meta_name mcode * pure
+  | AsType          of typeC * typeC (* as type, always metavar *)
   | DisjType        of string mcode * typeC list * (* only after iso *)
                        string mcode list (* the |s *)  * string mcode
   | OptType         of typeC
@@ -201,17 +260,27 @@ and base_declaration =
     MetaDecl   of Ast_cocci.meta_name mcode * pure (* variables *)
   | MetaField  of Ast_cocci.meta_name mcode * pure (* structure fields *)
   | MetaFieldList of Ast_cocci.meta_name mcode * listlen * pure
+  | AsDecl        of declaration * declaration
   | Init       of Ast_cocci.storage mcode option * typeC * ident *
 	string mcode (*=*) * initialiser * string mcode (*;*)
   | UnInit     of Ast_cocci.storage mcode option * typeC * ident *
 	string mcode (* ; *)
+  | FunProto of
+	fninfo list * ident (* name *) *
+	string mcode (* ( *) * parameter_list *
+        (string mcode (* , *) * string mcode (* ...... *) ) option *
+	string mcode (* ) *) * string mcode (* ; *)
   | TyDecl of typeC * string mcode (* ; *)
   | MacroDecl of ident (* name *) * string mcode (* ( *) *
         expression dots * string mcode (* ) *) * string mcode (* ; *)
+  | MacroDeclInit of ident (* name *) * string mcode (* ( *) *
+        expression dots * string mcode (* ) *) * string mcode (*=*) *
+        initialiser * string mcode (* ; *)
   | Typedef of string mcode (* typedef *) * typeC * typeC * string mcode (*;*)
   | DisjDecl   of string mcode * declaration list * string mcode list *
 	          string mcode
-  | Ddots      of string mcode (* ... *) * declaration option (* whencode *)
+  | Ddots      of string mcode (* ... *) * (string mcode * string mcode *
+                  declaration) option (* whencode *)
   | OptDecl    of declaration
   | UniqueDecl of declaration
 
@@ -222,6 +291,8 @@ and declaration = base_declaration wrap
 
 and base_initialiser =
     MetaInit of Ast_cocci.meta_name mcode * pure
+  | MetaInitList of Ast_cocci.meta_name mcode * listlen * pure
+  | AsInit of initialiser * initialiser (* as init, always metavar *)
   | InitExpr of expression
   | InitList of string mcode (*{*) * initialiser_list * string mcode (*}*) *
 	bool (* true if ordered, false if unordered *)
@@ -231,7 +302,8 @@ and base_initialiser =
   | InitGccName of ident (* name *) * string mcode (*:*) *
 	initialiser
   | IComma of string mcode
-  | Idots  of string mcode (* ... *) * initialiser option (* whencode *)
+  | Idots  of string mcode (* ... *) *
+              (string mcode * string mcode * initialiser) option (* whencode *)
   | OptIni    of initialiser
   | UniqueIni of initialiser
 
@@ -254,6 +326,7 @@ and base_parameterTypeDef =
   | Param         of typeC * ident option
   | MetaParam     of Ast_cocci.meta_name mcode * pure
   | MetaParamList of Ast_cocci.meta_name mcode * listlen * pure
+  | AsParam       of parameterTypeDef * expression (* expr, always metavar *)
   | PComma        of string mcode
   | Pdots         of string mcode (* ... *)
   | Pcircles      of string mcode (* ooo *)
@@ -290,29 +363,28 @@ and base_statement =
     Decl          of (info * mcodekind) (* before the decl *) * declaration
   | Seq           of string mcode (* { *) * statement dots *
  	             string mcode (* } *)
-  | ExprStatement of expression * string mcode (*;*)
+  | ExprStatement of expression option * string mcode (*;*)
   | IfThen        of string mcode (* if *) * string mcode (* ( *) *
 	             expression * string mcode (* ) *) *
-	             statement * (info * mcodekind)
+	             statement * fake_mcode (* after info *)
   | IfThenElse    of string mcode (* if *) * string mcode (* ( *) *
 	             expression * string mcode (* ) *) *
 	             statement * string mcode (* else *) * statement *
-	             (info * mcodekind)
+	             fake_mcode (* after info *)
   | While         of string mcode (* while *) * string mcode (* ( *) *
 	             expression * string mcode (* ) *) *
-	             statement * (info * mcodekind) (* after info *)
+	             statement * fake_mcode (* after info *)
   | Do            of string mcode (* do *) * statement *
                      string mcode (* while *) * string mcode (* ( *) *
 	             expression * string mcode (* ) *) *
                      string mcode (* ; *)
-  | For           of string mcode (* for *) * string mcode (* ( *) *
-                     expression option * string mcode (*;*) *
+  | For           of string mcode (* for *) * string mcode (* ( *) * forinfo *
 	             expression option * string mcode (*;*) *
                      expression option * string mcode (* ) *) * statement *
-	             (info * mcodekind) (* after info *)
+	             fake_mcode (* after info *)
   | Iterator      of ident (* name *) * string mcode (* ( *) *
 	             expression dots * string mcode (* ) *) *
-	             statement * (info * mcodekind) (* after info *)
+	             statement * fake_mcode (* after info *)
   | Switch        of string mcode (* switch *) * string mcode (* ( *) *
 	             expression * string mcode (* ) *) * string mcode (* { *) *
 	             statement (*decl*) dots *
@@ -324,9 +396,12 @@ and base_statement =
   | Return        of string mcode (* return *) * string mcode (* ; *)
   | ReturnExpr    of string mcode (* return *) * expression *
 	             string mcode (* ; *)
+  | Exec          of string mcode (* EXEC *) * string mcode (* language *) *
+	             exec_code dots * string mcode (* ; *)
   | MetaStmt      of Ast_cocci.meta_name mcode * pure
   | MetaStmtList  of Ast_cocci.meta_name mcode (*only in statement lists*) *
 	             pure
+  | AsStmt        of statement * statement (* as statement, always metavar *)
   | Exp           of expression  (* only in dotted statement lists *)
   | TopExp        of expression (* for macros body *)
   | Ty            of typeC (* only at top level *)
@@ -343,16 +418,33 @@ and base_statement =
 	             (statement dots,statement) whencode list
   | FunDecl of (info * mcodekind) (* before the function decl *) *
 	fninfo list * ident (* name *) *
-	string mcode (* ( *) * parameter_list * string mcode (* ) *) *
+	string mcode (* ( *) * parameter_list *
+	(string mcode (* , *) * string mcode (* ...... *) ) option *
+	string mcode (* ) *) *
 	string mcode (* { *) * statement dots *
-	string mcode (* } *)
+	string mcode (* } *) *
+	(info * mcodekind) (* after the function decl *)
   | Include of string mcode (* #include *) * Ast_cocci.inc_file mcode(* file *)
   | Undef of string mcode (* #define *) * ident (* name *)
   | Define of string mcode (* #define *) * ident (* name *) *
 	define_parameters (*params*) * statement dots
+  | Pragma of string mcode (* #pragma *) * ident * pragmainfo
   | OptStm   of statement
   | UniqueStm of statement
 
+and base_pragmainfo =
+    PragmaTuple of string mcode(* ( *) * expression dots * string mcode(* ) *)
+  | PragmaIdList of ident dots
+  | PragmaDots of string mcode
+
+and pragmainfo = base_pragmainfo wrap
+
+and base_forinfo =
+    ForExp of expression option * string mcode (*;*)
+  | ForDecl of (info * mcodekind) (* before the decl *) * declaration
+
+and forinfo = base_forinfo wrap
+
 and fninfo =
     FStorage of Ast_cocci.storage mcode
   | FType of typeC
@@ -360,11 +452,12 @@ and fninfo =
   | FAttr of string mcode
 
 and ('a,'b) whencode =
-    WhenNot of 'a
-  | WhenAlways of 'b
-  | WhenModifier of Ast_cocci.when_modifier
-  | WhenNotTrue of expression
-  | WhenNotFalse of expression
+    WhenNot of string mcode (* when *) * string mcode (* != *) * 'a
+  | WhenAlways of string mcode (* when *) * string mcode (* = *) * 'b
+  | WhenModifier of string mcode (* when *) * Ast_cocci.when_modifier
+  | WhenNotTrue of string mcode (* when *) * string mcode (* != *) * expression
+  | WhenNotFalse of string mcode (* when *) * string mcode (* != *) *
+    expression
 
 and statement = base_statement wrap
 
@@ -378,19 +471,26 @@ and base_case_line =
 
 and case_line = base_case_line wrap
 
+and base_exec_code =
+    ExecEval of string mcode (* : *) * expression
+  | ExecToken of string mcode
+  | ExecDots of string mcode (* ... *)
+
+and exec_code = base_exec_code wrap
+
 (* --------------------------------------------------------------------- *)
 (* Positions *)
 
 and meta_pos =
     MetaPos of Ast_cocci.meta_name mcode * Ast_cocci.meta_name list *
 	Ast_cocci.meta_collect
-  | NoMetaPos
 
 (* --------------------------------------------------------------------- *)
 (* Top-level code *)
 
 and base_top_level =
-    DECL of statement
+    NONDECL of statement (* cannot match all of a top-level declaration *)
+  | TOPCODE of statement dots
   | CODE of statement dots
   | FILEINFO of string mcode (* old file *) * string mcode (* new file *)
   | ERRORWORDS of expression list
@@ -411,10 +511,25 @@ and parsed_rule =
 	   Ast_cocci.meta_name * Ast_cocci.metavar) list (*inherited vars*) *
 	Ast_cocci.meta_name list (*script vars*) *
 	string
-  | InitialScriptRule of string (* name *) *
-      string (*language*) * Ast_cocci.dependency * string (*code*)
-  | FinalScriptRule of string (* name *) *
-      string (*language*) * Ast_cocci.dependency * string (*code*)
+  | InitialScriptRule of string (* name *) * string * Ast_cocci.dependency *
+	(Ast_cocci.script_meta_name *
+	   Ast_cocci.meta_name * Ast_cocci.metavar) list (*virtual vars*) *
+	string
+  | FinalScriptRule of string (* name *) * string * Ast_cocci.dependency *
+	(Ast_cocci.script_meta_name *
+	   Ast_cocci.meta_name * Ast_cocci.metavar) list (*virtual vars*) *
+	string
+
+(* --------------------------------------------------------------------- *)
+
+and dependency =
+    Dep of string (* rule applies for the current binding *)
+  | AntiDep of dependency (* rule doesn't apply for the current binding *)
+  | EverDep of string (* rule applies for some binding *)
+  | NeverDep of string (* rule never applies for any binding *)
+  | AndDep of dependency * dependency
+  | OrDep of dependency * dependency
+  | NoDep | FailDep
 
 (* --------------------------------------------------------------------- *)
 
@@ -427,6 +542,8 @@ and anything =
   | DotsCaseTag of case_line dots
   | IdentTag of ident
   | ExprTag of expression
+  | AssignOpTag of assignOp
+  | BinaryOpTag of binaryOp
   | ArgExprTag of expression  (* for isos *)
   | TestExprTag of expression (* for isos *)
   | TypeCTag of typeC
@@ -434,12 +551,17 @@ and anything =
   | InitTag of initialiser
   | DeclTag of declaration
   | StmtTag of statement
+  | ForInfoTag of forinfo
   | CaseLineTag of case_line
+  | StringFragmentTag of string_fragment
   | TopTag of top_level
   | IsoWhenTag of Ast_cocci.when_modifier (*only for when code, in iso phase*)
   | IsoWhenTTag of expression(*only for when code, in iso phase*)
   | IsoWhenFTag of expression(*only for when code, in iso phase*)
-  | MetaPosTag of meta_pos (* only in iso phase *)
+  | MetaPosTag of meta_pos
+  | HiddenVarTag of anything list (* in iso_compile/pattern only *)
+  | WhenTag of string mcode (* when *) * string mcode option *
+    anything (* iso pattern *)
 
 val dotsExpr : expression dots -> anything
 val dotsInit : initialiser dots -> anything
@@ -449,12 +571,16 @@ val dotsDecl : declaration dots -> anything
 val dotsCase : case_line dots -> anything
 val ident : ident -> anything
 val expr : expression -> anything
+val assignOp : assignOp -> anything
+val binaryOp : binaryOp -> anything
 val typeC : typeC -> anything
 val param : parameterTypeDef -> anything
 val ini : initialiser -> anything
 val decl : declaration -> anything
 val stmt : statement -> anything
+val forinfo : forinfo -> anything
 val case_line : case_line -> anything
+val string_fragment : string_fragment -> anything
 val top : top_level -> anything
 
 (* --------------------------------------------------------------------- *)
@@ -474,14 +600,15 @@ val unwrap_mcode : 'a mcode -> 'a
 val rewrap : 'a wrap -> 'b -> 'b wrap
 val rewrap_mcode : 'a mcode -> 'b -> 'b mcode
 val copywrap : 'a wrap -> 'b -> 'b wrap
-val get_pos : 'a mcode -> meta_pos
-val get_pos_ref : 'a mcode -> meta_pos ref
-val set_pos : meta_pos -> 'a mcode -> 'a mcode
+val get_pos : 'a mcode -> anything list
+val get_pos_ref : 'a mcode -> anything list ref
+val set_pos : anything list -> 'a mcode -> 'a mcode
 val get_info : 'a wrap -> info
 val set_info : 'a wrap -> info -> 'a wrap
 val get_index : 'a wrap -> int
 val set_index : 'a wrap -> int -> unit
 val get_line : 'a wrap -> int
+val get_mcode_line : 'a mcode -> int
 val get_line_end : 'a wrap -> int
 val get_mcodekind : 'a wrap -> mcodekind
 val get_mcode_mcodekind : 'a mcode -> mcodekind
@@ -496,6 +623,7 @@ val get_arg_exp : expression -> bool
 val set_test_pos : expression -> expression
 val get_test_pos : 'a wrap -> bool
 val set_test_exp : expression -> expression
+val clear_test_exp : expression -> expression
 val get_test_exp : 'a wrap -> bool
 val set_iso : 'a wrap -> (string*anything) list -> 'a wrap
 val get_iso : 'a wrap -> (string*anything) list
@@ -504,8 +632,11 @@ val set_mcode_data : 'a -> 'a mcode -> 'a mcode
 val make_mcode : 'a -> 'a mcode
 val make_mcode_info : 'a -> info -> 'a mcode
 val make_minus_mcode : 'a -> 'a mcode
+val get_rule_name : parsed_rule -> string
 
-val ast0_type_to_type : typeC -> Type_cocci.typeC
+val meta_pos_name : anything -> Ast_cocci.meta_name mcode
+
+val ast0_type_to_type : bool -> typeC -> Type_cocci.typeC
 val reverse_type : Type_cocci.typeC -> base_typeC
 exception TyConv
 
@@ -514,3 +645,6 @@ val lub_pure : pure -> pure -> pure
 (* --------------------------------------------------------------------- *)
 
 val rule_name : string ref (* for the convenience of the parser *)
+
+val string_of_assignOp : assignOp -> string
+val string_of_binaryOp : binaryOp -> string
