@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./ast0toast.ml"
 (* Arities matter for the minus slice, but not for the plus slice. *)
 
 (* + only allowed on code in a nest (in_nest = true).  ? only allowed on
@@ -54,7 +57,7 @@ let inline_mcodes =
     match (Ast0.get_mcodekind e) with
       Ast0.MINUS(replacements) ->
 	(match !replacements with
-	  ([],_) -> ()
+	  (Ast.NOREPLACEMENT,_) -> ()
 	| replacements ->
 	    let minus_try = function
 		(true,mc) ->
@@ -74,7 +77,7 @@ let inline_mcodes =
 	      | _ -> false in
 	    if not (minus_try(einfo.Ast0.attachable_start,
 			      einfo.Ast0.mcode_start)
-		      or
+		      ||
     		    minus_try(einfo.Ast0.attachable_end,
 			      einfo.Ast0.mcode_end))
 	    then
@@ -100,8 +103,16 @@ let inline_mcodes =
 	      List.iter
 		(function
 		    Ast0.MINUS(mreplacements) ->
-		      let (mrepl,tokeninfo) = !mreplacements in
-		      mreplacements := concat bef beforeinfo mrepl tokeninfo
+		      (match !mreplacements with
+			(Ast.NOREPLACEMENT,tokeninfo) ->
+			  mreplacements :=
+			    (Ast.REPLACEMENT(bef,befit),beforeinfo)
+		      |	(Ast.REPLACEMENT(anythings,it),tokeninfo) ->
+			  let (newbef,newinfo) =
+			    concat bef beforeinfo anythings tokeninfo in
+			  let it = Ast.lub_count befit it in
+			  mreplacements :=
+			    (Ast.REPLACEMENT(newbef,it),newinfo))
 		  | Ast0.CONTEXT(mbefaft) ->
 		      (match !mbefaft with
 			(Ast.BEFORE(mbef,it),mbeforeinfo,a) ->
@@ -133,8 +144,16 @@ let inline_mcodes =
 	      List.iter
 		(function
 		    Ast0.MINUS(mreplacements) ->
-		      let (mrepl,tokeninfo) = !mreplacements in
-		      mreplacements := concat mrepl tokeninfo aft afterinfo
+		      (match !mreplacements with
+			(Ast.NOREPLACEMENT,tokeninfo) ->
+			  mreplacements :=
+			    (Ast.REPLACEMENT(aft,aftit),afterinfo)
+		      |	(Ast.REPLACEMENT(anythings,it),tokeninfo) ->
+			  let (newaft,newinfo) =
+			    concat anythings tokeninfo aft afterinfo in
+			  let it = Ast.lub_count aftit it in
+			  mreplacements :=
+			    (Ast.REPLACEMENT(newaft,it),newinfo))
 		  | Ast0.CONTEXT(mbefaft) ->
 		      (match !mbefaft with
 			(Ast.BEFORE(mbef,it),b,_) ->
@@ -174,11 +193,12 @@ let inline_mcodes =
 	| (Ast.NOTHING,_,_) -> ())
     | Ast0.PLUS _ -> () in
   V0.flat_combiner bind option_default
-    mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
+    mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
     mcode mcode
     do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
     do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
-    do_nothing do_nothing do_nothing
+    do_nothing do_nothing do_nothing do_nothing do_nothing
+    do_nothing do_nothing
 
 (* --------------------------------------------------------------------- *)
 (* For function declarations.  Can't use the mcode at the root, because that
@@ -191,38 +211,48 @@ let check_allminus =
   let option_default = true in
   let mcode (_,_,_,mc,_,_) =
     match mc with
-      Ast0.MINUS(r) -> let (plusses,_) = !r in plusses = []
+      Ast0.MINUS(r) -> let (plusses,_) = !r in plusses = Ast.NOREPLACEMENT
     | _ -> false in
 
-  (* special case for disj *)
+  (* special case for disj and asExpr etc *)
   let ident r k e =
     match Ast0.unwrap e with
       Ast0.DisjId(starter,id_list,mids,ender) ->
 	List.for_all r.VT0.combiner_rec_ident id_list
+    | Ast0.AsIdent(id,asid) -> k id
     | _ -> k e in
 
   let expression r k e =
     match Ast0.unwrap e with
       Ast0.DisjExpr(starter,expr_list,mids,ender) ->
 	List.for_all r.VT0.combiner_rec_expression expr_list
+    | Ast0.AsExpr(exp,asexp) -> k exp
     | _ -> k e in
 
   let declaration r k e =
     match Ast0.unwrap e with
       Ast0.DisjDecl(starter,decls,mids,ender) ->
 	List.for_all r.VT0.combiner_rec_declaration decls
+    | Ast0.AsDecl(decl,asdecl) -> k decl
     | _ -> k e in
 
   let typeC r k e =
     match Ast0.unwrap e with
       Ast0.DisjType(starter,decls,mids,ender) ->
 	List.for_all r.VT0.combiner_rec_typeC decls
+    | Ast0.AsType(ty,asty) -> k ty
+    | _ -> k e in
+
+  let initialiser r k e =
+    match Ast0.unwrap e with
+      Ast0.AsInit(init,asinit) -> k init
     | _ -> k e in
 
   let statement r k e =
     match Ast0.unwrap e with
       Ast0.Disj(starter,statement_dots_list,mids,ender) ->
 	List.for_all r.VT0.combiner_rec_statement_dots statement_dots_list
+    | Ast0.AsStmt(stmt,asstmt) -> k stmt
     | _ -> k e in
 
   let case_line r k e =
@@ -233,10 +263,10 @@ let check_allminus =
 
   V0.flat_combiner bind option_default
     mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
-    mcode mcode
+    mcode mcode mcode mcode
     donothing donothing donothing donothing donothing donothing
-    ident expression typeC donothing donothing declaration
-    statement case_line donothing
+    ident expression donothing donothing typeC initialiser donothing declaration
+    statement donothing case_line donothing donothing
 
 (* --------------------------------------------------------------------- *)
 (* --------------------------------------------------------------------- *)
@@ -257,28 +287,51 @@ let convert_info info =
   { Ast.line = info.Ast0.pos_info.Ast0.line_start;
     Ast.column = info.Ast0.pos_info.Ast0.column;
     Ast.strbef = strings_to_s info.Ast0.strings_before;
-    Ast.straft = strings_to_s info.Ast0.strings_after;}
+    Ast.straft = strings_to_s info.Ast0.strings_after;
+    Ast.whitespace = info.Ast0.whitespace;
+  }
 
 let convert_mcodekind adj = function
     Ast0.MINUS(replacements) ->
       let (replacements,_) = !replacements in
-      Ast.MINUS(Ast.NoPos,[],adj,replacements)
+      Ast.MINUS(Ast.NoPos,[],Ast.ADJ adj,replacements)
   | Ast0.PLUS count -> Ast.PLUS count
   | Ast0.CONTEXT(befaft) ->
-      let (befaft,_,_) = !befaft in Ast.CONTEXT(Ast.NoPos,befaft)
+      let (befaft,_,_) = !befaft in
+      Ast.CONTEXT(Ast.NoPos,befaft)
   | Ast0.MIXED(_) -> failwith "not possible for mcode"
 
+let convert_fake_mcode (_,mc,adj) = convert_mcodekind adj mc
+
+let convert_allminus_mcodekind allminus = function
+    Ast0.CONTEXT(befaft) ->
+      let (befaft,_,_) = !befaft in
+      if allminus
+      then
+	(match befaft with
+	  Ast.NOTHING ->
+	    Ast.MINUS(Ast.NoPos,[],Ast.ALLMINUS,Ast.NOREPLACEMENT)
+	| Ast.BEFORE(a,ct) | Ast.AFTER(a,ct) ->
+	    Ast.MINUS(Ast.NoPos,[],Ast.ALLMINUS,Ast.REPLACEMENT(a,ct))
+	| Ast.BEFOREAFTER(b,a,ct) ->
+	    Ast.MINUS(Ast.NoPos,[],Ast.ALLMINUS,Ast.REPLACEMENT(b@a,ct)))
+      else Ast.CONTEXT(Ast.NoPos,befaft)
+  | _ -> failwith "convert_allminus_mcodekind: unexpected mcodekind"
+
 let pos_mcode(term,_,info,mcodekind,pos,adj) =
   (* avoids a recursion problem *)
-  (term,convert_info info,convert_mcodekind adj mcodekind,Ast.NoMetaPos)
+  (term,convert_info info,convert_mcodekind adj mcodekind,[])
 
 let mcode (term,_,info,mcodekind,pos,adj) =
   let pos =
-    match !pos with
-      Ast0.MetaPos(pos,constraints,per) ->
-	Ast.MetaPos(pos_mcode pos,constraints,per,unitary,false)
-    | _ -> Ast.NoMetaPos in
-  (term,convert_info info,convert_mcodekind adj mcodekind,pos)
+    List.fold_left
+      (function prev ->
+	function
+	    Ast0.MetaPosTag(Ast0.MetaPos(pos,constraints,per)) ->
+	      (Ast.MetaPos(pos_mcode pos,constraints,per,unitary,false))::prev
+	  | _ -> prev)
+      [] !pos in
+  (term,convert_info info,convert_mcodekind adj mcodekind,List.rev pos)
 
 (* --------------------------------------------------------------------- *)
 (* Dots *)
@@ -302,36 +355,6 @@ let dots fn d =
     | Ast0.CIRCLES(x) -> Ast.CIRCLES(List.map fn x)
     | Ast0.STARS(x) -> Ast.STARS(List.map fn x))
 
-(* commas in dotted lists, here due to polymorphism restrictions *)
-
-let add_comma is_comma make_comma itemlist =
-  match Ast0.unwrap itemlist with
-    Ast0.DOTS(x) ->
-      (match List.rev x with
-	[] -> itemlist
-      | e::es ->
-	  if is_comma e
-	  then itemlist
-	  else
-	    let comma =
-	      match Ast0.get_mcodekind e with
-		Ast0.MINUS(_) -> (Ast0.make_minus_mcode ",")
-	      |	_ -> (Ast0.make_mcode ",") in
-	    Ast0.rewrap itemlist
-	      (Ast0.DOTS
-		 (List.rev (Ast0.rewrap e (make_comma comma) :: (e::es)))))
-  | _ -> failwith "not possible"
-
-let add_exp_comma =
-  add_comma
-    (function x -> match Ast0.unwrap x with Ast0.EComma _ -> true | _ -> false)
-    (function x -> Ast0.EComma x)
-
-and add_init_comma =
-  add_comma
-    (function x -> match Ast0.unwrap x with Ast0.IComma _ -> true | _ -> false)
-    (function x -> Ast0.IComma x)
-
 (* --------------------------------------------------------------------- *)
 (* Identifier *)
 
@@ -343,12 +366,14 @@ and ident i =
       Ast0.Id(name) -> Ast.Id(mcode name)
     | Ast0.DisjId(_,id_list,_,_) ->
 	Ast.DisjId(List.map ident id_list)
-    | Ast0.MetaId(name,constraints,_) ->
+    | Ast0.MetaId(name,constraints,_,_) ->
 	Ast.MetaId(mcode name,constraints,unitary,false)
     | Ast0.MetaFunc(name,constraints,_) ->
 	Ast.MetaFunc(mcode name,constraints,unitary,false)
     | Ast0.MetaLocalFunc(name,constraints,_) ->
 	Ast.MetaLocalFunc(mcode name,constraints,unitary,false)
+    | Ast0.AsIdent(id,asid) ->
+	Ast.AsIdent(ident id,ident asid)
     | Ast0.OptIdent(id) -> Ast.OptIdent(ident id)
     | Ast0.UniqueIdent(id) -> Ast.UniqueIdent(ident id))
 
@@ -362,6 +387,8 @@ and expression e =
       Ast0.Ident(id) -> Ast.Ident(ident id)
     | Ast0.Constant(const) ->
 	Ast.Constant(mcode const)
+    | Ast0.StringConstant(lq,str,rq) ->
+	Ast.StringConstant(mcode lq, dots string_fragment str, mcode rq)
     | Ast0.FunCall(fn,lp,args,rp) ->
 	let fn = expression fn in
 	let lp = mcode lp in
@@ -369,7 +396,9 @@ and expression e =
 	let rp = mcode rp in
 	Ast.FunCall(fn,lp,args,rp)
     | Ast0.Assignment(left,op,right,simple) ->
-	Ast.Assignment(expression left,mcode op,expression right,simple)
+	Ast.Assignment(expression left,assignOp op,expression right,simple)
+    | Ast0.Sequence(left,op,right) ->
+	Ast.Sequence(expression left,mcode op,expression right)
     | Ast0.CondExpr(exp1,why,exp2,colon,exp3) ->
 	let exp1 = expression exp1 in
 	let why = mcode why in
@@ -384,9 +413,9 @@ and expression e =
     | Ast0.Unary(exp,op) ->
 	Ast.Unary(expression exp,mcode op)
     | Ast0.Binary(left,op,right) ->
-	Ast.Binary(expression left,mcode op,expression right)
+	Ast.Binary(expression left,binaryOp op,expression right)
     | Ast0.Nested(left,op,right) ->
-	Ast.Nested(expression left,mcode op,expression right)
+	Ast.Nested(expression left,binaryOp op,expression right)
     | Ast0.Paren(lp,exp,rp) ->
 	Ast.Paren(mcode lp,expression exp,mcode rp)
     | Ast0.ArrayAccess(exp1,lb,exp2,rb) ->
@@ -396,44 +425,93 @@ and expression e =
     | Ast0.RecordPtAccess(exp,ar,field) ->
 	Ast.RecordPtAccess(expression exp,mcode ar,ident field)
     | Ast0.Cast(lp,ty,rp,exp) ->
-	Ast.Cast(mcode lp,typeC ty,mcode rp,expression exp)
+	let allminus = check_allminus.VT0.combiner_rec_expression e in
+	Ast.Cast(mcode lp,typeC allminus ty,mcode rp,expression exp)
     | Ast0.SizeOfExpr(szf,exp) ->
 	Ast.SizeOfExpr(mcode szf,expression exp)
     | Ast0.SizeOfType(szf,lp,ty,rp) ->
-	Ast.SizeOfType(mcode szf, mcode lp,typeC ty,mcode rp)
-    | Ast0.TypeExp(ty) -> Ast.TypeExp(typeC ty)
+	let allminus = check_allminus.VT0.combiner_rec_expression e in
+	Ast.SizeOfType(mcode szf, mcode lp,typeC allminus ty,mcode rp)
+    | Ast0.TypeExp(ty) ->
+	let allminus = check_allminus.VT0.combiner_rec_expression e in
+	Ast.TypeExp(typeC allminus ty)
+    | Ast0.Constructor(lp,ty,rp,init) ->
+	let allminus = check_allminus.VT0.combiner_rec_expression e in
+	Ast.Constructor(mcode lp,typeC allminus ty,mcode rp,initialiser init)
     | Ast0.MetaErr(name,cstrts,_)  ->
-	  Ast.MetaErr(mcode name,constraints cstrts,unitary,false)
+	Ast.MetaErr(mcode name,constraints cstrts,unitary,false)
     | Ast0.MetaExpr(name,cstrts,ty,form,_)  ->
-	  Ast.MetaExpr(mcode name,constraints cstrts,unitary,ty,form,false)
+	Ast.MetaExpr(mcode name,constraints cstrts,unitary,ty,form,false)
     | Ast0.MetaExprList(name,lenname,_) ->
 	Ast.MetaExprList(mcode name,do_lenname lenname,unitary,false)
+    | Ast0.AsExpr(expr,asexpr) ->
+	Ast.AsExpr(expression expr,expression asexpr)
     | Ast0.EComma(cm)         -> Ast.EComma(mcode cm)
     | Ast0.DisjExpr(_,exps,_,_)     ->
 	Ast.DisjExpr(List.map expression exps)
     | Ast0.NestExpr(starter,exp_dots,ender,whencode,multi) ->
 	let starter = mcode starter in
-	let whencode = get_option expression whencode in
+	let whencode = get_option (fun (_,_,b) -> expression b) whencode in
 	let ender = mcode ender in
 	Ast.NestExpr(starter,dots expression exp_dots,ender,whencode,multi)
     | Ast0.Edots(dots,whencode) ->
 	let dots = mcode dots in
-	let whencode = get_option expression whencode in
+	let whencode = get_option (fun (_,_,b) -> expression b) whencode in
 	Ast.Edots(dots,whencode)
     | Ast0.Ecircles(dots,whencode) ->
 	let dots = mcode dots in
-	let whencode = get_option expression whencode in
+	let whencode = get_option (fun (_,_,b) -> expression b) whencode in
 	Ast.Ecircles(dots,whencode)
     | Ast0.Estars(dots,whencode) ->
 	let dots = mcode dots in
-	let whencode = get_option expression whencode in
+	let whencode = get_option (fun (_,_,b) -> expression b) whencode in
 	Ast.Estars(dots,whencode)
     | Ast0.OptExp(exp) -> Ast.OptExp(expression exp)
     | Ast0.UniqueExp(exp) -> Ast.UniqueExp(expression exp)) in
   if Ast0.get_test_exp e then Ast.set_test_exp e1 else e1
 
+and assignOp op =
+  rewrap op no_isos
+    (match Ast0.unwrap op with
+      Ast0.SimpleAssign op' -> Ast.SimpleAssign (mcode op')
+    | Ast0.OpAssign op' -> Ast.OpAssign (mcode op')
+    | Ast0.MetaAssign(mv, c, _) ->
+      Ast.MetaAssign(mcode mv, assignOpconstraint c, unitary, false))
+and assignOpconstraint = function
+  | Ast0.AssignOpNoConstraint -> Ast.AssignOpNoConstraint
+  | Ast0.AssignOpInSet ops -> Ast.AssignOpInSet (List.map assignOp ops)
+
+and binaryOp op =
+  rewrap op no_isos
+    (match Ast0.unwrap op with
+      Ast0.Arith op' -> Ast.Arith (mcode op')
+    | Ast0.Logical op' -> Ast.Logical (mcode op')
+    | Ast0.MetaBinary(mv, c, _) ->
+      Ast.MetaBinary(mcode mv, binaryOpconstraint c, unitary, false))    
+and binaryOpconstraint = function
+  | Ast0.BinaryOpNoConstraint -> Ast.BinaryOpNoConstraint
+  | Ast0.BinaryOpInSet ops -> Ast.BinaryOpInSet (List.map binaryOp ops)
+
 and expression_dots ed = dots expression ed
 
+and string_fragment e =
+  rewrap e no_isos
+    (match Ast0.unwrap e with
+      Ast0.ConstantFragment(str) -> Ast.ConstantFragment(mcode str)
+    | Ast0.FormatFragment(pct,fmt) ->
+	Ast.FormatFragment(mcode pct, string_format fmt)
+    | Ast0.Strdots dots -> Ast.Strdots (mcode dots)
+    | Ast0.MetaFormatList(pct,name,lenname) ->
+	Ast.MetaFormatList(mcode pct, mcode name, do_lenname lenname,
+			   unitary,false))
+
+and string_format e =
+  rewrap e no_isos
+    (match Ast0.unwrap e with
+      Ast0.ConstantFormat(str) -> Ast.ConstantFormat(mcode str)
+    | Ast0.MetaFormat(name,constraints) ->
+	Ast.MetaFormat(mcode name,constraints,unitary,false))
+
 and constraints c =
   match c with
       Ast0.NoConstraint        -> Ast.NoConstraint
@@ -451,7 +529,7 @@ and do_lenname = function
 
 and rewrap_iso t t1 = rewrap t (do_isos (Ast0.get_iso t)) t1
 
-and typeC t =
+and typeC allminus t =
   rewrap t (do_isos (Ast0.get_iso t))
     (match Ast0.unwrap t with
       Ast0.ConstVol(cv,ty) ->
@@ -466,7 +544,8 @@ and typeC t =
 	  List.map
 	    (function ty ->
 	      Ast.Type
-		(Some (mcode cv),rewrap_iso ty (base_typeC ty)))
+		(allminus, Some (mcode cv),
+		 rewrap_iso ty (base_typeC allminus ty)))
 	    (collect_disjs ty) in
 	(* one could worry that isos are lost because we flatten the
 	   disjunctions.  but there should not be isos on the disjunctions
@@ -475,44 +554,47 @@ and typeC t =
 	  [ty] -> ty
 	| types -> Ast.DisjType(List.map (rewrap t no_isos) types))
     | Ast0.BaseType(_) | Ast0.Signed(_,_) | Ast0.Pointer(_,_)
-    | Ast0.FunctionPointer(_,_,_,_,_,_,_) | Ast0.FunctionType(_,_,_,_)
-    | Ast0.Array(_,_,_,_) | Ast0.EnumName(_,_) | Ast0.StructUnionName(_,_)
+    | Ast0.FunctionPointer(_,_,_,_,_,_,_)
+    | Ast0.Array(_,_,_,_) | Ast0.Decimal(_,_,_,_,_,_)
+    | Ast0.EnumName(_,_) | Ast0.StructUnionName(_,_)
     | Ast0.StructUnionDef(_,_,_,_) | Ast0.EnumDef(_,_,_,_)
     | Ast0.TypeName(_) | Ast0.MetaType(_,_) ->
-	Ast.Type(None,rewrap t no_isos (base_typeC t))
-    | Ast0.DisjType(_,types,_,_) -> Ast.DisjType(List.map typeC types)
-    | Ast0.OptType(ty) -> Ast.OptType(typeC ty)
-    | Ast0.UniqueType(ty) -> Ast.UniqueType(typeC ty))
+	Ast.Type(allminus,None,rewrap t no_isos (base_typeC allminus t))
+    | Ast0.DisjType(_,types,_,_) ->
+	Ast.DisjType(List.map (typeC allminus) types)
+    | Ast0.AsType(ty,asty) ->
+	Ast.AsType(typeC allminus ty,typeC allminus asty)
+    | Ast0.OptType(ty) -> Ast.OptType(typeC allminus ty)
+    | Ast0.UniqueType(ty) -> Ast.UniqueType(typeC allminus ty))
 
-and base_typeC t =
+and base_typeC allminus t =
   match Ast0.unwrap t with
     Ast0.BaseType(ty,strings) -> Ast.BaseType(ty,List.map mcode strings)
   | Ast0.Signed(sgn,ty) ->
-      Ast.SignedT(mcode sgn,
-		  get_option (function x -> rewrap_iso x (base_typeC x)) ty)
-  | Ast0.Pointer(ty,star) -> Ast.Pointer(typeC ty,mcode star)
+      Ast.SignedT
+	(mcode sgn,
+	 get_option (function x -> rewrap_iso x (base_typeC allminus x)) ty)
+  | Ast0.Pointer(ty,star) -> Ast.Pointer(typeC allminus ty,mcode star)
   | Ast0.FunctionPointer(ty,lp1,star,rp1,lp2,params,rp2) ->
       Ast.FunctionPointer
-	(typeC ty,mcode lp1,mcode star,mcode rp1,
+	(typeC allminus ty,mcode lp1,mcode star,mcode rp1,
 	 mcode lp2,parameter_list params,mcode rp2)
-  | Ast0.FunctionType(ret,lp,params,rp) ->
-      let allminus = check_allminus.VT0.combiner_rec_typeC t in
-      Ast.FunctionType
-	(allminus,get_option typeC ret,mcode lp,
-	 parameter_list params,mcode rp)
   | Ast0.Array(ty,lb,size,rb) ->
-      Ast.Array(typeC ty,mcode lb,get_option expression size,mcode rb)
+      Ast.Array(typeC allminus ty,mcode lb,get_option expression size,
+		mcode rb)
+  | Ast0.Decimal(dec,lp,length,comma,precision_opt,rp) ->
+      Ast.Decimal(mcode dec,mcode lp,expression length,
+		  get_option mcode comma,get_option expression precision_opt,
+		  mcode rp)
   | Ast0.EnumName(kind,name) ->
       Ast.EnumName(mcode kind,get_option ident name)
   | Ast0.EnumDef(ty,lb,ids,rb) ->
-      let ids = add_exp_comma ids in
-      Ast.EnumDef(typeC ty,mcode lb,dots expression ids,mcode rb)
+      Ast.EnumDef(typeC allminus ty,mcode lb,dots expression ids,mcode rb)
   | Ast0.StructUnionName(kind,name) ->
       Ast.StructUnionName(mcode kind,get_option ident name)
   | Ast0.StructUnionDef(ty,lb,decls,rb) ->
-      Ast.StructUnionDef(typeC ty,mcode lb,
-			 dots declaration decls,
-			 mcode rb)
+      Ast.StructUnionDef(typeC allminus ty,mcode lb,
+			 declaration_dots decls, mcode rb)
   | Ast0.TypeName(name) -> Ast.TypeName(mcode name)
   | Ast0.MetaType(name,_) ->
       Ast.MetaType(mcode name,unitary,false)
@@ -530,28 +612,32 @@ and declaration d =
     | Ast0.MetaField(name,_) -> Ast.MetaField(mcode name,unitary,false)
     | Ast0.MetaFieldList(name,lenname,_) ->
 	Ast.MetaFieldList(mcode name,do_lenname lenname,unitary,false)
+    | Ast0.AsDecl(decl,asdecl) ->
+	Ast.AsDecl(declaration decl,declaration asdecl)
     | Ast0.Init(stg,ty,id,eq,ini,sem) ->
+	let allminus = check_allminus.VT0.combiner_rec_declaration d in
 	let stg = get_option mcode stg in
-	let ty = typeC ty in
+	let ty = typeC allminus ty in
 	let id = ident id in
 	let eq = mcode eq in
 	let ini = initialiser ini in
 	let sem = mcode sem in
 	Ast.Init(stg,ty,id,eq,ini,sem)
     | Ast0.UnInit(stg,ty,id,sem) ->
-	(match Ast0.unwrap ty with
-	  Ast0.FunctionType(tyx,lp1,params,rp1) ->
-	    let allminus = check_allminus.VT0.combiner_rec_declaration d in
-	    Ast.UnInit(get_option mcode stg,
-		       rewrap ty (do_isos (Ast0.get_iso ty))
-			 (Ast.Type
-			    (None,
-			     rewrap ty no_isos
-			       (Ast.FunctionType
-				  (allminus,get_option typeC tyx,mcode lp1,
-				   parameter_list params,mcode rp1)))),
-		       ident id,mcode sem)
-	| _ -> Ast.UnInit(get_option mcode stg,typeC ty,ident id,mcode sem))
+	let allminus = check_allminus.VT0.combiner_rec_declaration d in
+	Ast.UnInit(get_option mcode stg,typeC allminus ty,ident id,
+		   mcode sem)
+    | Ast0.FunProto(fi,name,lp,params,va,rp,sem) ->
+	  let fi = List.map fninfo fi in
+	  let name = ident name in
+	  let lp = mcode lp in
+	  let params = parameter_list params in
+          let va = match va with
+            | None -> None
+            | Some (comma,ellipsis) -> Some(mcode comma,mcode ellipsis) in
+	  let rp = mcode rp in
+	  let sem = mcode sem in
+	  Ast.FunProto(fi,name,lp,params,va,rp,sem)
     | Ast0.MacroDecl(name,lp,args,rp,sem) ->
 	let name = ident name in
 	let lp = mcode lp in
@@ -559,22 +645,50 @@ and declaration d =
 	let rp = mcode rp in
 	let sem = mcode sem in
 	Ast.MacroDecl(name,lp,args,rp,sem)
-    | Ast0.TyDecl(ty,sem) -> Ast.TyDecl(typeC ty,mcode sem)
+    | Ast0.MacroDeclInit(name,lp,args,rp,eq,ini,sem) ->
+	let name = ident name in
+	let lp = mcode lp in
+	let args = dots expression args in
+	let rp = mcode rp in
+	let eq = mcode eq in
+	let ini = initialiser ini in
+	let sem = mcode sem in
+	Ast.MacroDeclInit(name,lp,args,rp,eq,ini,sem)
+    | Ast0.TyDecl(ty,sem) ->
+	let allminus = check_allminus.VT0.combiner_rec_declaration d in
+	Ast.TyDecl(typeC allminus ty,mcode sem)
     | Ast0.Typedef(stg,ty,id,sem) ->
-	let id = typeC id in
+	let allminus = check_allminus.VT0.combiner_rec_declaration d in
+	let id = typeC allminus id in
 	(match Ast.unwrap id with
-	  Ast.Type(None,id) -> (* only MetaType or Id *)
-	    Ast.Typedef(mcode stg,typeC ty,id,mcode sem)
+	  Ast.Type(_,None,id) -> (* only MetaType or Id *)
+	    Ast.Typedef(mcode stg,typeC allminus ty,id,mcode sem)
 	| _ -> failwith "bad typedef")
+    | Ast0.Ddots(dots,whencode) -> failwith "should not be possible"
     | Ast0.DisjDecl(_,decls,_,_) -> Ast.DisjDecl(List.map declaration decls)
-    | Ast0.Ddots(dots,whencode) ->
-	let dots = mcode dots in
-	let whencode = get_option declaration whencode in
-	Ast.Ddots(dots,whencode)
     | Ast0.OptDecl(decl) -> Ast.OptDecl(declaration decl)
     | Ast0.UniqueDecl(decl) -> Ast.UniqueDecl(declaration decl))
 
-and declaration_dots l = dots declaration l
+and annotated_decl bef d =
+  rewrap d (do_isos (Ast0.get_iso d))
+    (match Ast0.unwrap d with
+      Ast0.Ddots(dots,whencode) ->
+	(* structure definitions only *)
+	let dots = mcode dots in
+	let whencode = get_option (fun (_,_,b) -> declaration b) whencode in
+	Ast.Ddots(dots,whencode)
+    | _ -> (* for decls where there is no bef information needed *)
+	let bef =
+	  match bef with
+	    None -> (* fake, no change here *)
+	      let bot = Ast0.default_token_info in
+	      Ast0.CONTEXT (ref(Ast.NOTHING,bot,bot))
+	  | Some bef -> bef in
+	let allminus = check_allminus.VT0.combiner_rec_declaration d in
+	Ast.DElem(convert_allminus_mcodekind allminus bef,allminus,
+		  declaration d))
+
+and declaration_dots l = dots (annotated_decl None) l
 
 (* --------------------------------------------------------------------- *)
 (* Initialiser *)
@@ -607,7 +721,7 @@ and strip_idots initlist =
 	      |	Ast0.Idots(dots,None) ->
 		  let (restwhen,restinit,dotinfo) = loop rest in
 		  (restwhen, restinit, (isminus dots)::dotinfo)
-	      |	_ -> 
+	      |	_ ->
 		  let (restwhen,restinit,dotinfo) = loop rest in
 		  (restwhen,x::restinit,dotinfo)) in
 	loop l in
@@ -625,16 +739,18 @@ and initialiser i =
   rewrap i no_isos
     (match Ast0.unwrap i with
       Ast0.MetaInit(name,_) -> Ast.MetaInit(mcode name,unitary,false)
+    | Ast0.MetaInitList(name,lenname,_) ->
+	Ast.MetaInitList(mcode name,do_lenname lenname,unitary,false)
+    | Ast0.AsInit(init,asinit) ->
+	Ast.AsInit(initialiser init,initialiser asinit)
     | Ast0.InitExpr(exp) -> Ast.InitExpr(expression exp)
     | Ast0.InitList(lb,initlist,rb,true) ->
-	let initlist = add_init_comma initlist in
 	Ast.ArInitList(mcode lb,dots initialiser initlist,mcode rb)
     | Ast0.InitList(lb,initlist,rb,false) ->
-	let initlist = add_init_comma initlist in
 	let (whencode,initlist,allminus) = strip_idots initlist in
 	Ast.StrInitList
 	  (allminus,mcode lb,List.map initialiser initlist,mcode rb,
-	   List.map initialiser whencode)
+	   List.map (fun (_,_,b) -> initialiser b) whencode)
     | Ast0.InitGccExt(designators,eq,ini) ->
 	Ast.InitGccExt(List.map designator designators,mcode eq,
 		       initialiser ini)
@@ -643,7 +759,7 @@ and initialiser i =
     | Ast0.IComma(comma) -> Ast.IComma(mcode comma)
     | Ast0.Idots(dots,whencode) ->
 	let dots = mcode dots in
-	let whencode = get_option initialiser whencode in
+	let whencode = get_option (fun (_,_,b) -> initialiser b) whencode in
 	Ast.Idots(dots,whencode)
     | Ast0.OptIni(ini) -> Ast.OptIni(initialiser ini)
     | Ast0.UniqueIni(ini) -> Ast.UniqueIni(initialiser ini))
@@ -662,12 +778,16 @@ and designator = function
 and parameterTypeDef p =
   rewrap p no_isos
     (match Ast0.unwrap p with
-      Ast0.VoidParam(ty) -> Ast.VoidParam(typeC ty)
-    | Ast0.Param(ty,id) -> Ast.Param(typeC ty,get_option ident id)
+      Ast0.VoidParam(ty) -> Ast.VoidParam(typeC false ty)
+    | Ast0.Param(ty,id) ->
+	let allminus = check_allminus.VT0.combiner_rec_parameter p in
+	Ast.Param(typeC allminus ty,get_option ident id)
     | Ast0.MetaParam(name,_) ->
 	Ast.MetaParam(mcode name,unitary,false)
     | Ast0.MetaParamList(name,lenname,_) ->
 	Ast.MetaParamList(mcode name,do_lenname lenname,unitary,false)
+    | Ast0.AsParam(p,asexpr) ->
+	Ast.AsParam(parameterTypeDef p,expression asexpr)
     | Ast0.PComma(cm) -> Ast.PComma(mcode cm)
     | Ast0.Pdots(dots) -> Ast.Pdots(mcode dots)
     | Ast0.Pcircles(dots) -> Ast.Pcircles(mcode dots)
@@ -696,9 +816,7 @@ and statement s =
       (match Ast0.unwrap s with
 	Ast0.Decl((_,bef),decl) ->
 	  Ast.Atomic(rewrap_rule_elem s
-		       (Ast.Decl(convert_mcodekind (-1) bef,
-				 check_allminus.VT0.combiner_rec_statement s,
-				 declaration decl)))
+		       (Ast.Decl(annotated_decl (Some bef) decl)))
       | Ast0.Seq(lbrace,body,rbrace) ->
 	  let lbrace = mcode lbrace in
 	  let body = dots (statement seqible) body in
@@ -709,14 +827,15 @@ and statement s =
 		  tokenwrap rbrace s (Ast.SeqEnd(rbrace)))
       | Ast0.ExprStatement(exp,sem) ->
 	  Ast.Atomic(rewrap_rule_elem s
-		       (Ast.ExprStatement(expression exp,mcode sem)))
-      | Ast0.IfThen(iff,lp,exp,rp,branch,(_,aft)) ->
+		       (Ast.ExprStatement
+			  (get_option expression exp,mcode sem)))
+      | Ast0.IfThen(iff,lp,exp,rp,branch,aft) ->
 	  Ast.IfThen
 	    (rewrap_rule_elem s
 	       (Ast.IfHeader(mcode iff,mcode lp,expression exp,mcode rp)),
 	     statement Ast.NotSequencible branch,
-	     ([],[],[],convert_mcodekind (-1) aft))
-      | Ast0.IfThenElse(iff,lp,exp,rp,branch1,els,branch2,(_,aft)) ->
+	     ([],[],[],convert_fake_mcode aft))
+      | Ast0.IfThenElse(iff,lp,exp,rp,branch1,els,branch2,aft) ->
 	  let els = mcode els in
 	  Ast.IfThenElse
 	    (rewrap_rule_elem s
@@ -724,13 +843,13 @@ and statement s =
 	     statement Ast.NotSequencible branch1,
 	     tokenwrap els s (Ast.Else(els)),
 	     statement Ast.NotSequencible branch2,
-	     ([],[],[],convert_mcodekind (-1) aft))
-      | Ast0.While(wh,lp,exp,rp,body,(_,aft)) ->
+	     ([],[],[],convert_fake_mcode aft))
+      | Ast0.While(wh,lp,exp,rp,body,aft) ->
 	  Ast.While(rewrap_rule_elem s
 		      (Ast.WhileHeader
 			 (mcode wh,mcode lp,expression exp,mcode rp)),
 		    statement Ast.NotSequencible body,
-		    ([],[],[],convert_mcodekind (-1) aft))
+		    ([],[],[],convert_fake_mcode aft))
       | Ast0.Do(d,body,wh,lp,exp,rp,sem) ->
 	  let wh = mcode wh in
 	  Ast.Do(rewrap_rule_elem s (Ast.DoHeader(mcode d)),
@@ -738,27 +857,26 @@ and statement s =
 		 tokenwrap wh s
 		   (Ast.WhileTail(wh,mcode lp,expression exp,mcode rp,
 				  mcode sem)))
-      | Ast0.For(fr,lp,exp1,sem1,exp2,sem2,exp3,rp,body,(_,aft)) ->
+      | Ast0.For(fr,lp,first,exp2,sem2,exp3,rp,body,aft) ->
 	  let fr = mcode fr in
 	  let lp = mcode lp in
-	  let exp1 = get_option expression exp1 in
-	  let sem1 = mcode sem1 in
+	  let first = forinfo first in
 	  let exp2 = get_option expression exp2 in
 	  let sem2= mcode sem2 in
 	  let exp3 = get_option expression exp3 in
 	  let rp = mcode rp in
 	  let body = statement Ast.NotSequencible body in
 	  Ast.For(rewrap_rule_elem s
-		    (Ast.ForHeader(fr,lp,exp1,sem1,exp2,sem2,exp3,rp)),
-		  body,([],[],[],convert_mcodekind (-1) aft))
-      | Ast0.Iterator(nm,lp,args,rp,body,(_,aft)) ->
+		    (Ast.ForHeader(fr,lp,first,exp2,sem2,exp3,rp)),
+		  body,([],[],[],convert_fake_mcode aft))
+      | Ast0.Iterator(nm,lp,args,rp,body,aft) ->
 	  Ast.Iterator(rewrap_rule_elem s
 		      (Ast.IteratorHeader
 			 (ident nm,mcode lp,
 			  dots expression args,
 			  mcode rp)),
 		    statement Ast.NotSequencible body,
-		    ([],[],[],convert_mcodekind (-1) aft))
+		    ([],[],[],convert_fake_mcode aft))
       |	Ast0.Switch(switch,lp,exp,rp,lb,decls,cases,rb) ->
 	  let switch = mcode switch in
 	  let lp = mcode lp in
@@ -787,12 +905,18 @@ and statement s =
 	  Ast.Atomic
 	    (rewrap_rule_elem s
 	       (Ast.ReturnExpr(mcode ret,expression exp,mcode sem)))
+      | Ast0.Exec(exec,lang,code,sem) ->
+	  Ast.Atomic
+	    (rewrap_rule_elem s
+	       (Ast.Exec(mcode exec,mcode lang,dots exec_code code,mcode sem)))
       | Ast0.MetaStmt(name,_) ->
 	  Ast.Atomic(rewrap_rule_elem s
 		       (Ast.MetaStmt(mcode name,unitary,seqible,false)))
       | Ast0.MetaStmtList(name,_) ->
 	  Ast.Atomic(rewrap_rule_elem s
 		       (Ast.MetaStmtList(mcode name,unitary,false)))
+      | Ast0.AsStmt(stmt,asstmt) ->
+	  Ast.AsStmt(statement seqible stmt,statement seqible asstmt)
       | Ast0.TopExp(exp) ->
 	  Ast.Atomic(rewrap_rule_elem s (Ast.TopExp(expression exp)))
       | Ast0.Exp(exp) ->
@@ -800,7 +924,8 @@ and statement s =
       | Ast0.TopInit(init) ->
 	  Ast.Atomic(rewrap_rule_elem s (Ast.TopInit(initialiser init)))
       | Ast0.Ty(ty) ->
-	  Ast.Atomic(rewrap_rule_elem s (Ast.Ty(typeC ty)))
+	  let allminus = check_allminus.VT0.combiner_rec_statement s in
+	  Ast.Atomic(rewrap_rule_elem s (Ast.Ty(typeC allminus ty)))
       | Ast0.Disj(_,rule_elem_dots_list,_,_) ->
 	  Ast.Disj(List.map (function x -> statement_dots seqible x)
 		     rule_elem_dots_list)
@@ -837,22 +962,28 @@ and statement s =
 		 (statement Ast.NotSequencible))
 	      whn in
 	  Ast.Stars(d,whn,[],[])
-      | Ast0.FunDecl((_,bef),fi,name,lp,params,rp,lbrace,body,rbrace) ->
+      | Ast0.FunDecl((_,bef),fi,name,lp,params,va,rp,lbrace,body,rbrace,
+		     (_,aft)) ->
 	  let fi = List.map fninfo fi in
 	  let name = ident name in
 	  let lp = mcode lp in
 	  let params = parameter_list params in
+          let newva = match va with
+            | None -> None
+            | Some (comma, ellipsis) -> Some (mcode comma, mcode ellipsis) in
 	  let rp = mcode rp in
 	  let lbrace = mcode lbrace in
 	  let body = dots (statement seqible) body in
 	  let rbrace = mcode rbrace in
 	  let allminus = check_allminus.VT0.combiner_rec_statement s in
 	  Ast.FunDecl(rewrap_rule_elem s
-			(Ast.FunHeader(convert_mcodekind (-1) bef,
-				       allminus,fi,name,lp,params,rp)),
+			(Ast.FunHeader
+			   (convert_allminus_mcodekind allminus bef,
+			    allminus,fi,name,lp,params,newva,rp)),
 		      tokenwrap lbrace s (Ast.SeqStart(lbrace)),
 		      body,
-		      tokenwrap rbrace s (Ast.SeqEnd(rbrace)))
+		      tokenwrap rbrace s (Ast.SeqEnd(rbrace)),
+		      ([],[],[],convert_allminus_mcodekind allminus aft))
       |	Ast0.Include(inc,str) ->
 	  Ast.Atomic(rewrap_rule_elem s (Ast.Include(mcode inc,mcode str)))
       |	Ast0.Undef(def,id) ->
@@ -863,9 +994,23 @@ and statement s =
 	       (Ast.DefineHeader
 		  (mcode def,ident id, define_parameters params)),
 	     statement_dots Ast.NotSequencible (*not sure*) body)
+      |	Ast0.Pragma(prg,id,body) ->
+	  Ast.Atomic(rewrap_rule_elem s
+		       (Ast.Pragma(mcode prg,ident id,pragmainfo body)))
       | Ast0.OptStm(stm) -> Ast.OptStm(statement seqible stm)
       | Ast0.UniqueStm(stm) -> Ast.UniqueStm(statement seqible stm))
 
+  and pragmainfo pi =
+    rewrap pi no_isos
+      (match Ast0.unwrap pi with
+	Ast0.PragmaTuple(lp,args,rp) ->
+	  let lp = mcode lp in
+	  let args = dots expression args in
+	  let rp = mcode rp in
+	  Ast.PragmaTuple(lp,args,rp)
+      | Ast0.PragmaIdList(ids) -> Ast.PragmaIdList(dots ident ids)
+      | Ast0.PragmaDots (dots) -> Ast.PragmaDots (mcode dots))
+
   and define_parameters p =
     rewrap p no_isos
       (match Ast0.unwrap p with
@@ -886,16 +1031,16 @@ and statement s =
       | Ast0.UniqueDParam(dp) -> Ast.UniqueDParam(define_param dp))
 
   and whencode notfn alwaysfn = function
-      Ast0.WhenNot a -> Ast.WhenNot (notfn a)
-    | Ast0.WhenAlways a -> Ast.WhenAlways (alwaysfn a)
-    | Ast0.WhenModifier(x) -> Ast.WhenModifier(x)
+      Ast0.WhenNot (_,_,a) -> Ast.WhenNot (notfn a)
+    | Ast0.WhenAlways (_,_,a) -> Ast.WhenAlways (alwaysfn a)
+    | Ast0.WhenModifier(_,x) -> Ast.WhenModifier(x)
     | x ->
 	let rewrap_rule_elem ast0 ast =
 	  rewrap ast0 (do_isos (Ast0.get_iso ast0)) ast in
 	match x with
-	  Ast0.WhenNotTrue(e) ->
+	  Ast0.WhenNotTrue(_,_,e) ->
 	    Ast.WhenNotTrue(rewrap_rule_elem e (Ast.Exp(expression e)))
-	| Ast0.WhenNotFalse(e) ->
+	| Ast0.WhenNotFalse(_,_,e) ->
 	    Ast.WhenNotFalse(rewrap_rule_elem e (Ast.Exp(expression e)))
 	| _ -> failwith "not possible"
 
@@ -976,9 +1121,17 @@ and statement s =
 
   statement Ast.Sequencible s
 
+and forinfo fi =
+  match Ast0.unwrap fi with
+    Ast0.ForExp(exp1,sem1) ->
+      let exp1 = get_option expression exp1 in
+      let sem1 = mcode sem1 in
+      Ast.ForExp(exp1,sem1)
+  | Ast0.ForDecl ((_,bef),decl) -> Ast.ForDecl(annotated_decl (Some bef) decl)
+
 and fninfo = function
     Ast0.FStorage(stg) -> Ast.FStorage(mcode stg)
-  | Ast0.FType(ty) -> Ast.FType(typeC ty)
+  | Ast0.FType(ty) -> Ast.FType(typeC false ty)
   | Ast0.FInline(inline) -> Ast.FInline(mcode inline)
   | Ast0.FAttr(attr) -> Ast.FAttr(mcode attr)
 
@@ -1006,6 +1159,13 @@ and case_line c =
 
     | Ast0.OptCase(case) -> Ast.OptCase(case_line case))
 
+and exec_code c =
+  rewrap c no_isos
+    (match Ast0.unwrap c with
+      Ast0.ExecEval(colon,id) -> Ast.ExecEval(mcode colon,expression id)
+    | Ast0.ExecToken(tok) -> Ast.ExecToken(mcode tok)
+    | Ast0.ExecDots(dots) -> Ast.ExecDots(mcode dots))
+
 and statement_dots l = dots statement l
 
 (* --------------------------------------------------------------------- *)
@@ -1016,23 +1176,29 @@ and anything = function
   | Ast0.DotsParamTag(d) -> Ast.ParamDotsTag(parameter_list d)
   | Ast0.DotsInitTag(d) -> failwith "not possible"
   | Ast0.DotsStmtTag(d) -> Ast.StmtDotsTag(statement_dots d)
-  | Ast0.DotsDeclTag(d) -> Ast.DeclDotsTag(declaration_dots d)
+  | Ast0.DotsDeclTag(d) -> Ast.AnnDeclDotsTag(declaration_dots d)
   | Ast0.DotsCaseTag(d) -> failwith "not possible"
   | Ast0.IdentTag(d) -> Ast.IdentTag(ident d)
   | Ast0.ExprTag(d) -> Ast.ExpressionTag(expression d)
+  | Ast0.AssignOpTag d -> Ast.AssignOpTag(assignOp d)
+  | Ast0.BinaryOpTag d -> Ast.BinaryOpTag(binaryOp d)
   | Ast0.ArgExprTag(d) | Ast0.TestExprTag(d) ->
      failwith "only in isos, not converted to ast"
-  | Ast0.TypeCTag(d) -> Ast.FullTypeTag(typeC d)
+  | Ast0.TypeCTag(d) -> Ast.FullTypeTag(typeC false d)
   | Ast0.ParamTag(d) -> Ast.ParamTag(parameterTypeDef d)
   | Ast0.InitTag(d) -> Ast.InitTag(initialiser d)
   | Ast0.DeclTag(d) -> Ast.DeclarationTag(declaration d)
   | Ast0.StmtTag(d) -> Ast.StatementTag(statement d)
+  | Ast0.ForInfoTag(d) -> Ast.ForInfoTag(forinfo d)
   | Ast0.CaseLineTag(d) -> Ast.CaseLineTag(case_line d)
+  | Ast0.StringFragmentTag(d) -> Ast.StringFragmentTag(string_fragment d)
   | Ast0.TopTag(d) -> Ast.Code(top_level d)
   | Ast0.IsoWhenTag(_) -> failwith "not possible"
   | Ast0.IsoWhenTTag(_) -> failwith "not possible"
   | Ast0.IsoWhenFTag(_) -> failwith "not possible"
   | Ast0.MetaPosTag _ -> failwith "not possible"
+  | Ast0.HiddenVarTag _ -> failwith "not possible"
+  | Ast0.WhenTag _ -> failwith "not possible"
 
 (* --------------------------------------------------------------------- *)
 (* Function declaration *)
@@ -1043,11 +1209,10 @@ and top_level t =
     (match Ast0.unwrap t with
       Ast0.FILEINFO(old_file,new_file) ->
 	Ast.FILEINFO(mcode old_file,mcode new_file)
-    | Ast0.DECL(stmt) -> Ast.DECL(statement stmt)
-    | Ast0.CODE(rule_elem_dots) ->
-	Ast.CODE(statement_dots rule_elem_dots)
+    | Ast0.NONDECL(stmt) -> Ast.NONDECL(statement stmt)
+    | Ast0.CODE(rule_elem_dots) -> Ast.CODE(statement_dots rule_elem_dots)
     | Ast0.ERRORWORDS(exps) -> Ast.ERRORWORDS(List.map expression exps)
-    | Ast0.OTHER(_) -> failwith "eliminated by top_level")
+    | Ast0.OTHER(_) | Ast0.TOPCODE(_) -> failwith "eliminated by top_level")
 
 (* --------------------------------------------------------------------- *)
 (* Entry point for minus code *)
