@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./ast_cocci.ml"
 (* --------------------------------------------------------------------- *)
 (* Modified code *)
 
@@ -29,7 +32,9 @@ type added_string = Noindent of string | Indent of string | Space of string
 
 type info = { line : int; column : int;
 	      strbef : (added_string * int (* line *) * int (* col *)) list;
-	      straft : (added_string * int (* line *) * int (* col *)) list }
+	      straft : (added_string * int (* line *) * int (* col *)) list;
+              whitespace : string }
+
 type line = int
 type meta_name = string * string
 (* need to be careful about rewrapping, to avoid duplicating pos info
@@ -41,6 +46,7 @@ type 'a wrap =
       minus_free_vars : meta_name list; (*minus free vars*)
       fresh_vars : (meta_name * seed) list; (*fresh vars*)
       inherited : meta_name list; (*inherited vars*)
+      positive_inherited_positions : meta_name list;
       saved_witness : meta_name list; (*witness vars*)
       bef_aft : dots_bef_aft;
       (* the following is for or expressions *)
@@ -57,12 +63,14 @@ and 'a befaft =
   | BEFOREAFTER of 'a list list * 'a list list * count
   | NOTHING
 
-and 'a mcode = 'a * info * mcodekind * meta_pos (* pos variable *)
+and 'a replacement = REPLACEMENT of 'a list list * count | NOREPLACEMENT
+
+and 'a mcode = 'a * info * mcodekind * meta_pos list (* pos variables *)
     (* pos is an offset indicating where in the C code the mcodekind
        has an effect *)
     (* int list is the match instances, which are only meaningful in annotated
        C code *)
-    (* int is the adjacency index, which is incremented on context dots *)
+    (* adjacency is the adjacency index, which is incremented on context dots *)
 (* iteration is only allowed on context code, the intuition vaguely being
 that there is no way to replace something more than once.  Actually,
 allowing iterated additions on minus code would cause problems with some
@@ -71,8 +79,9 @@ replacements with certainty.  Anyway, iteration doesn't seem to be needed
 on - code for the moment.  Although it may be confusing that there can be
 iterated addition of code before context code where the context code is
 immediately followed by removed code. *)
+and adjacency = ALLMINUS | ADJ of int
 and mcodekind =
-    MINUS       of pos * int list * int * anything list list
+    MINUS       of pos * int list * adjacency * anything replacement
   | CONTEXT     of pos * anything befaft
   | PLUS        of count
 and count = ONE (* + *) | MANY (* ++ *)
@@ -104,8 +113,11 @@ and metavar =
   | MetaFreshIdDecl of meta_name (* name *) * seed (* seed *)
   | MetaTypeDecl of arity * meta_name (* name *)
   | MetaInitDecl of arity * meta_name (* name *)
+  | MetaInitListDecl of arity * meta_name (* name *) * list_len (*len*)
   | MetaListlenDecl of meta_name (* name *)
   | MetaParamDecl of arity * meta_name (* name *)
+  | MetaBinaryOperatorDecl of arity * meta_name
+  | MetaAssignmentOperatorDecl of arity * meta_name
   | MetaParamListDecl of arity * meta_name (*name*) * list_len (*len*)
   | MetaConstDecl of
       arity * meta_name (* name *) * Type_cocci.typeC list option
@@ -116,6 +128,8 @@ and metavar =
       arity * meta_name (* name *) * Type_cocci.typeC list option
   | MetaLocalIdExpDecl of
       arity * meta_name (* name *) * Type_cocci.typeC list option
+  | MetaGlobalIdExpDecl of
+      arity * meta_name (* name *) * Type_cocci.typeC list option
   | MetaExpListDecl of arity * meta_name (*name*) * list_len (*len*)
   | MetaDeclDecl of arity * meta_name (* name *)
   | MetaFieldDecl of arity * meta_name (* name *)
@@ -125,6 +139,9 @@ and metavar =
   | MetaFuncDecl of arity * meta_name (* name *)
   | MetaLocalFuncDecl of arity * meta_name (* name *)
   | MetaPosDecl of arity * meta_name (* name *)
+  | MetaFmtDecl of arity * meta_name (* name *)
+  | MetaFragListDecl of arity * meta_name (* name *) * list_len (*len*)
+  | MetaAnalysisDecl of string * meta_name (* name *)
   | MetaDeclarerDecl of arity * meta_name (* name *)
   | MetaIteratorDecl of arity * meta_name (* name *)
 
@@ -152,6 +169,7 @@ and base_ident =
   | MetaId        of meta_name mcode * idconstraint * keep_binding * inherited
   | MetaFunc      of meta_name mcode * idconstraint * keep_binding * inherited
   | MetaLocalFunc of meta_name mcode * idconstraint * keep_binding * inherited
+  | AsIdent       of ident * ident (* as ident, always metavar *)
 
   | DisjId        of ident list
   | OptIdent      of ident
@@ -165,17 +183,20 @@ and ident = base_ident wrap
 and base_expression =
     Ident          of ident
   | Constant       of constant mcode
+  | StringConstant of string mcode (* quote *) * string_fragment dots *
+		      string mcode (* quote *)
   | FunCall        of expression * string mcode (* ( *) *
                       expression dots * string mcode (* ) *)
-  | Assignment     of expression * assignOp mcode * expression *
+  | Assignment     of expression * assignOp * expression *
 	              bool (* true if it can match an initialization *)
+  | Sequence       of expression * string mcode (* , *) * expression
   | CondExpr       of expression * string mcode (* ? *) * expression option *
 	              string mcode (* : *) * expression
   | Postfix        of expression * fixOp mcode
   | Infix          of expression * fixOp mcode
   | Unary          of expression * unaryOp mcode
-  | Binary         of expression * binaryOp mcode * expression
-  | Nested         of expression * binaryOp mcode * expression
+  | Binary         of expression * binaryOp * expression
+  | Nested         of expression * binaryOp * expression
   | ArrayAccess    of expression * string mcode (* [ *) * expression *
 	              string mcode (* ] *)
   | RecordAccess   of expression * string mcode (* . *) * ident
@@ -191,19 +212,22 @@ and base_expression =
   | Paren          of string mcode (* ( *) * expression *
                       string mcode (* ) *)
 
+  | Constructor    of string mcode (* ( *) * fullType * string mcode (* ) *) *
+	              initialiser
   | MetaErr        of meta_name mcode * constraints * keep_binding *
 	              inherited
   | MetaExpr       of meta_name mcode * constraints * keep_binding *
 	              Type_cocci.typeC list option * form * inherited
   | MetaExprList   of meta_name mcode * listlen * keep_binding *
                       inherited (* only in arg lists *)
+  | AsExpr         of expression * expression (* as expr, always metavar *)
 
   | EComma         of string mcode (* only in arg lists *)
 
   | DisjExpr       of expression list
   | NestExpr       of string mcode (* <.../<+... *) *
 	              expression dots *
-	              string mcode (* ...>/...+> *) * 
+	              string mcode (* ...>/...+> *) *
                       expression option * multi
 
   (* can appear in arg lists, and also inside Nest, as in:
@@ -225,15 +249,24 @@ and constraints =
 (* Constraints on Meta-* Identifiers, Functions *)
 and idconstraint =
     IdNoConstraint
+  | IdPosIdSet         of string list * meta_name list
   | IdNegIdSet         of string list * meta_name list
   | IdRegExpConstraint of reconstraint
 
 and reconstraint =
-  | IdRegExp        of string * Str.regexp
-  | IdNotRegExp     of string * Str.regexp
+  | IdRegExp        of string * Regexp.regexp
+  | IdNotRegExp     of string * Regexp.regexp
+
+and assignOpconstraint =
+    AssignOpNoConstraint
+  | AssignOpInSet of assignOp list
+
+and binaryOpconstraint =
+    BinaryOpNoConstraint
+  | BinaryOpInSet of binaryOp list
 
 (* ANY = int E; ID = idexpression int X; CONST = constant int X; *)
-and form = ANY | ID | LocalID | CONST (* form for MetaExp *)
+and form = ANY | ID | LocalID | GlobalID | CONST (* form for MetaExp *)
 
 and expression = base_expression wrap
 
@@ -242,13 +275,40 @@ and listlen =
   | CstListLen of int
   | AnyListLen
 
-and  unaryOp = GetRef | DeRef | UnPlus |  UnMinus | Tilde | Not
-and  assignOp = SimpleAssign | OpAssign of arithOp
-and  fixOp = Dec | Inc
+and base_string_fragment =
+    ConstantFragment of string mcode
+  | FormatFragment of string mcode (*%*) * string_format (* format *)
+  | Strdots of string mcode
+  | MetaFormatList of string mcode (*%*) * meta_name mcode * listlen *
+	keep_binding * inherited
 
-and  binaryOp = Arith of arithOp | Logical of logicalOp
-and  arithOp =
+and string_fragment = base_string_fragment wrap
+
+and base_string_format =
+    ConstantFormat of string mcode
+  | MetaFormat of meta_name mcode * idconstraint * keep_binding * inherited
+
+and string_format = base_string_format wrap
+
+and unaryOp = GetRef | GetRefLabel | DeRef | UnPlus |  UnMinus | Tilde | Not
+and base_assignOp =
+    SimpleAssign of simpleAssignOp mcode
+  | OpAssign of arithOp mcode
+  | MetaAssign of
+      meta_name mcode * assignOpconstraint * keep_binding * inherited
+and simpleAssignOp = string
+and assignOp = base_assignOp wrap
+and fixOp = Dec | Inc
+
+and base_binaryOp =
+    Arith of arithOp mcode
+  | Logical of logicalOp mcode
+  | MetaBinary of
+      meta_name mcode * binaryOpconstraint * keep_binding * inherited
+and binaryOp = base_binaryOp wrap
+and arithOp =
     Plus | Minus | Mul | Div | Mod | DecLeft | DecRight | And | Or | Xor
+  | Min | Max
 and  logicalOp = Inf | Sup | InfEq | SupEq | Eq | NotEq | AndLog | OrLog
 
 and constant =
@@ -256,12 +316,15 @@ and constant =
   | Char   of string
   | Int    of string
   | Float  of string
+  | DecimalConst of (string * string * string)
 
 (* --------------------------------------------------------------------- *)
 (* Types *)
 
 and base_fullType =
-    Type            of const_vol mcode option * typeC
+    Type            of bool (* true if all minus *) *
+	               const_vol mcode option * typeC
+  | AsType          of fullType * fullType (* as type, always metavar *)
   | DisjType        of fullType list (* only after iso *)
   | OptType         of fullType
   | UniqueType      of fullType
@@ -273,21 +336,18 @@ and base_typeC =
   | FunctionPointer of fullType *
 	          string mcode(* ( *)*string mcode(* * *)*string mcode(* ) *)*
                   string mcode (* ( *)*parameter_list*string mcode(* ) *)
-
-  (* used for the automatic managment of prototypes *)
-  | FunctionType     of bool (* true if all minus for dropping return type *) *
-                   fullType option *
-	           string mcode (* ( *) * parameter_list *
-                   string mcode (* ) *)
-
   | Array           of fullType * string mcode (* [ *) *
 	               expression option * string mcode (* ] *)
+  | Decimal         of string mcode (* decimal *) * string mcode (* ( *) *
+	               expression *
+	               string mcode option (* , *) * expression option *
+	               string mcode (* ) *) (* IBM C only *)
   | EnumName        of string mcode (*enum*) * ident option (* name *)
   | EnumDef  of fullType (* either EnumName or metavar *) *
 	string mcode (* { *) * expression dots * string mcode (* } *)
   | StructUnionName of structUnion mcode * ident option (* name *)
   | StructUnionDef  of fullType (* either StructUnionName or metavar *) *
-	string mcode (* { *) * declaration dots * string mcode (* } *)
+	string mcode (* { *) * annotated_decl dots * string mcode (* } *)
   | TypeName        of string mcode (* pad: should be 'of ident' ? *)
 
   | MetaType        of meta_name mcode * keep_binding * inherited
@@ -295,8 +355,10 @@ and base_typeC =
 and fullType = base_fullType wrap
 and typeC = base_typeC wrap
 
-and baseType = VoidType | CharType | ShortType | IntType | DoubleType
-  | FloatType | LongType | LongLongType | SizeType | SSizeType | PtrDiffType
+and baseType = VoidType | CharType | ShortType | ShortIntType | IntType
+| DoubleType | LongDoubleType | FloatType
+| LongType | LongIntType | LongLongType | LongLongIntType
+| SizeType | SSizeType | PtrDiffType
 
 and structUnion = Struct | Union
 
@@ -313,29 +375,45 @@ and base_declaration =
     Init of storage mcode option * fullType * ident * string mcode (*=*) *
 	initialiser * string mcode (*;*)
   | UnInit of storage mcode option * fullType * ident * string mcode (* ; *)
+  | FunProto of
+	fninfo list * ident (* name *) *
+	string mcode (* ( *) * parameter_list *
+	(string mcode (* , *) * string mcode (* ...... *) ) option *
+	string mcode (* ) *) * string mcode (* ; *)
   | TyDecl of fullType * string mcode (* ; *)
   | MacroDecl of ident (* name *) * string mcode (* ( *) *
         expression dots * string mcode (* ) *) * string mcode (* ; *)
+  | MacroDeclInit of ident (* name *) * string mcode (* ( *) *
+        expression dots * string mcode (* ) *) * string mcode (*=*) *
+        initialiser * string mcode (* ; *)
   | Typedef of string mcode (*typedef*) * fullType *
                typeC (* either TypeName or metavar *) * string mcode (*;*)
   | DisjDecl of declaration list
-  (* Ddots is for a structure declaration *)
-  | Ddots    of string mcode (* ... *) * declaration option (* whencode *)
-
   | MetaDecl of meta_name mcode * keep_binding * inherited
   | MetaField of meta_name mcode * keep_binding * inherited
   | MetaFieldList of meta_name mcode * listlen * keep_binding * inherited
+  | AsDecl        of declaration * declaration
 
   | OptDecl    of declaration
   | UniqueDecl of declaration
 
 and declaration = base_declaration wrap
 
+and base_annotated_decl =
+    DElem of mcodekind (* before the decl *) * bool (* true if all minus *) *
+      declaration
+  (* Ddots is for a structure declaration *)
+  | Ddots    of string mcode (* ... *) * declaration option (* whencode *)
+
+and annotated_decl = base_annotated_decl wrap
+
 (* --------------------------------------------------------------------- *)
 (* Initializers *)
 
 and base_initialiser =
     MetaInit of meta_name mcode * keep_binding * inherited
+  | MetaInitList of meta_name mcode * listlen * keep_binding * inherited
+  | AsInit of initialiser * initialiser (* as init, always metavar *)
   | InitExpr of expression
   | ArInitList of string mcode (*{*) * initialiser dots * string mcode (*}*)
   | StrInitList of bool (* true if all are - *) *
@@ -371,6 +449,8 @@ and base_parameterTypeDef =
   | MetaParam     of meta_name mcode * keep_binding * inherited
   | MetaParamList of meta_name mcode * listlen * keep_binding * inherited
 
+  | AsParam       of parameterTypeDef * expression (* expr, always metavar *)
+
   | PComma        of string mcode
 
   | Pdots         of string mcode (* ... *)
@@ -410,8 +490,7 @@ and meta_collect = PER | ALL
 
 and meta_pos =
     MetaPos of meta_name mcode * meta_name list *
-	meta_collect * keep_binding * inherited
-  | NoMetaPos
+      meta_collect * keep_binding * inherited
 
 (* --------------------------------------------------------------------- *)
 (* Function declaration *)
@@ -426,14 +505,14 @@ and base_rule_elem =
 	             bool (* true if all minus, for dropping static, etc *) *
 	             fninfo list * ident (* name *) *
 	             string mcode (* ( *) * parameter_list *
+                     (string mcode (* , *) * string mcode (* ...... *) ) option *
                      string mcode (* ) *)
-  | Decl          of mcodekind (* before the decl *) *
-                     bool (* true if all minus *) * declaration
+  | Decl          of annotated_decl
 
   | SeqStart      of string mcode (* { *)
   | SeqEnd        of string mcode (* } *)
 
-  | ExprStatement of expression * string mcode (*;*)
+  | ExprStatement of expression option * string mcode (*;*)
   | IfHeader      of string mcode (* if *) * string mcode (* ( *) *
 	             expression * string mcode (* ) *)
   | Else          of string mcode (* else *)
@@ -444,7 +523,7 @@ and base_rule_elem =
 	             expression * string mcode (* ) *) *
                      string mcode (* ; *)
   | ForHeader     of string mcode (* for *) * string mcode (* ( *) *
-                     expression option * string mcode (*;*) *
+                     forinfo *
 	             expression option * string mcode (*;*) *
                      expression option * string mcode (* ) *)
   | IteratorHeader of ident (* name *) * string mcode (* ( *) *
@@ -458,6 +537,8 @@ and base_rule_elem =
   | Return        of string mcode (* return *) * string mcode (* ; *)
   | ReturnExpr    of string mcode (* return *) * expression *
 	             string mcode (* ; *)
+  | Exec          of string mcode (* EXEC *) * string mcode (* language *) *
+	             exec_code dots * string mcode (* ; *)
 
   | MetaRuleElem  of meta_name mcode * keep_binding * inherited
   | MetaStmt      of meta_name mcode * keep_binding * metaStmtInfo *
@@ -473,10 +554,22 @@ and base_rule_elem =
   | Undef         of string mcode (* #define *) * ident (* name *)
   | DefineHeader  of string mcode (* #define *) * ident (* name *) *
 	             define_parameters (*params*)
+  | Pragma        of string mcode (* #pragma *) * ident * pragmainfo
   | Case          of string mcode (* case *) * expression * string mcode (*:*)
   | Default       of string mcode (* default *) * string mcode (*:*)
   | DisjRuleElem  of rule_elem list
 
+and base_pragmainfo =
+    PragmaTuple of string mcode(* ( *) * expression dots * string mcode(* ) *)
+  | PragmaIdList of ident dots
+  | PragmaDots of string mcode
+
+and pragmainfo = base_pragmainfo wrap
+
+and forinfo =
+    ForExp of expression option * string mcode (*;*)
+  | ForDecl of annotated_decl
+
 and fninfo =
     FStorage of storage mcode
   | FType of fullType
@@ -494,7 +587,7 @@ and base_statement =
   | IfThen        of rule_elem (* header *) * statement * end_info (* endif *)
   | IfThenElse    of rule_elem (* header *) * statement *
 	             rule_elem (* else *) * statement * end_info (* endif *)
-  | While         of rule_elem (* header *) * statement * end_info (*endwhile*)
+  | While         of rule_elem (* header *) * statement * end_info(*endwhile*)
   | Do            of rule_elem (* do *) * statement * rule_elem (* tail *)
   | For           of rule_elem (* header *) * statement * end_info (*endfor*)
   | Iterator      of rule_elem (* header *) * statement * end_info (*enditer*)
@@ -503,12 +596,13 @@ and base_statement =
   | Atomic        of rule_elem
   | Disj          of statement dots list
   | Nest          of string mcode (* <.../<+... *) * statement dots *
-	             string mcode (* ...>/...+> *) * 
+	             string mcode (* ...>/...+> *) *
 	             (statement dots,statement) whencode list * multi *
 	             dots_whencode list * dots_whencode list
   | FunDecl       of rule_elem (* header *) * rule_elem (* { *) *
-     	             statement dots * rule_elem (* } *)
+     	             statement dots * rule_elem (* } *) * end_info (*exit*)
   | Define        of rule_elem (* header *) * statement dots
+  | AsStmt        of statement * statement (* as statement, always metavar *)
   | Dots          of string mcode (* ... *) *
 	             (statement dots,statement) whencode list *
 	             dots_whencode list * dots_whencode list
@@ -552,6 +646,13 @@ and base_case_line =
 
 and case_line = base_case_line wrap
 
+and base_exec_code =
+    ExecEval of string mcode (* : *) * expression
+  | ExecToken of string mcode
+  | ExecDots of string mcode (* ... *)
+
+and exec_code = base_exec_code wrap
+
 and inc_file =
     Local of inc_elem list
   | NonLocal of inc_elem list
@@ -561,18 +662,20 @@ and inc_elem =
   | IncDots
 
 and base_top_level =
-    DECL of statement
+    NONDECL of statement
   | CODE of statement dots
   | FILEINFO of string mcode (* old file *) * string mcode (* new file *)
   | ERRORWORDS of expression list
 
 and top_level = base_top_level wrap
 
+and parser_kind = ExpP | TyP | AnyP
+
 and rulename =
     CocciRulename of string option * dependency *
-	string list * string list * exists * bool
+	string list * string list * exists * parser_kind
   | GeneratedRulename of string option * dependency *
-	string list * string list * exists * bool
+	string list * string list * exists * parser_kind
   | ScriptRulename of string option (* name *) * string (* language *) *
 	dependency
   | InitialScriptRulename of string option (* name *) * string (* language *) *
@@ -592,9 +695,13 @@ and rule =
 	(script_meta_name * meta_name * metavar) list (*inherited vars*) *
 	meta_name list (*script vars*) * string
   | InitialScriptRule of  string (* name *) *
-	string (*language*) * dependency * string (*code*)
+	string (*language*) * dependency *
+	(script_meta_name * meta_name * metavar) list (*virtual vars*) *
+	string (*code*)
   | FinalScriptRule of  string (* name *) *
-	string (*language*) * dependency * string (*code*)
+	string (*language*) * dependency *
+	(script_meta_name * meta_name * metavar) list (*virtual vars*) *
+	string (*code*)
 
 and script_meta_name = string option (*string*) * string option (*ast*)
 
@@ -619,6 +726,8 @@ and anything =
   | ConstantTag         of constant
   | UnaryOpTag          of unaryOp
   | AssignOpTag         of assignOp
+  | SimpleAssignOpTag   of simpleAssignOp
+  | OpAssignOpTag       of arithOp
   | FixOpTag            of fixOp
   | BinaryOpTag         of binaryOp
   | ArithOpTag          of arithOp
@@ -629,15 +738,17 @@ and anything =
   | IncFileTag          of inc_file
   | Rule_elemTag        of rule_elem
   | StatementTag        of statement
+  | ForInfoTag          of forinfo
   | CaseLineTag         of case_line
+  | StringFragmentTag   of string_fragment
   | ConstVolTag         of const_vol
   | Token               of string * info option
-  | Pragma              of added_string list
+  | Directive           of added_string list
   | Code                of top_level
   | ExprDotsTag         of expression dots
   | ParamDotsTag        of parameterTypeDef dots
   | StmtDotsTag         of statement dots
-  | DeclDotsTag         of declaration dots
+  | AnnDeclDotsTag      of annotated_decl dots
   | TypeCTag            of typeC
   | ParamTag            of parameterTypeDef
   | SgrepStartTag       of string
@@ -675,6 +786,7 @@ let get_mfvs x             = x.minus_free_vars
 let set_mfvs mfvs x        = {x with minus_free_vars = mfvs}
 let get_fresh x            = x.fresh_vars
 let get_inherited x        = x.inherited
+let get_inherited_pos x    = x.positive_inherited_positions
 let get_saved x            = x.saved_witness
 let get_dots_bef_aft x     = x.bef_aft
 let set_dots_bef_aft d x   = {x with bef_aft = d}
@@ -687,7 +799,7 @@ let get_isos x             = x.iso_info
 let set_isos x isos        = {x with iso_info = isos}
 let get_pos_var (_,_,_,p)  = p
 let set_pos_var vr (a,b,c,_) = (a,b,c,vr)
-let drop_pos (a,b,c,_)     = (a,b,c,NoMetaPos)
+let drop_pos (a,b,c,_)     = (a,b,c,[])
 
 let get_wcfvs (whencode : ('a wrap, 'b wrap) whencode list) =
   Common.union_all
@@ -708,14 +820,18 @@ let get_meta_name = function
   | MetaFreshIdDecl(nm,seed) -> nm
   | MetaTypeDecl(ar,nm) -> nm
   | MetaInitDecl(ar,nm) -> nm
+  | MetaInitListDecl(ar,nm,nm1) -> nm
   | MetaListlenDecl(nm) -> nm
   | MetaParamDecl(ar,nm) -> nm
   | MetaParamListDecl(ar,nm,nm1) -> nm
+  | MetaBinaryOperatorDecl(_,name) -> name
+  | MetaAssignmentOperatorDecl(_,name) -> name
   | MetaConstDecl(ar,nm,ty) -> nm
   | MetaErrDecl(ar,nm) -> nm
   | MetaExpDecl(ar,nm,ty) -> nm
   | MetaIdExpDecl(ar,nm,ty) -> nm
   | MetaLocalIdExpDecl(ar,nm,ty) -> nm
+  | MetaGlobalIdExpDecl(ar,nm,ty) -> nm
   | MetaExpListDecl(ar,nm,nm1) -> nm
   | MetaDeclDecl(ar,nm) -> nm
   | MetaFieldDecl(ar,nm) -> nm
@@ -725,6 +841,9 @@ let get_meta_name = function
   | MetaFuncDecl(ar,nm) -> nm
   | MetaLocalFuncDecl(ar,nm) -> nm
   | MetaPosDecl(ar,nm) -> nm
+  | MetaFmtDecl(ar,nm) -> nm
+  | MetaFragListDecl(ar,nm,nm1) -> nm
+  | MetaAnalysisDecl(code,nm) -> nm
   | MetaDeclarerDecl(ar,nm) -> nm
   | MetaIteratorDecl(ar,nm) -> nm
 
@@ -737,28 +856,32 @@ and tag2c = function
   | SignTag _ -> "SignTag"
   | IdentTag _ -> "IdentTag"
   | ExpressionTag _ -> "ExpressionTag"
-  | ConstantTag _ -> "ConstantTag"
-  | UnaryOpTag _ -> "UnaryOpTag"
-  | AssignOpTag _ -> "AssignOpTag"
-  | FixOpTag _ -> "FixOpTag"
-  | BinaryOpTag _ -> "BinaryOpTag"
-  | ArithOpTag _ -> "ArithOpTag"
+  | ConstantTag _  -> "ConstantTag"
+  | UnaryOpTag _   -> "UnaryOpTag"
+  | AssignOpTag _  -> "AssignOpTag"
+  | SimpleAssignOpTag _  -> "SimpleAssignOpTag"
+  | OpAssignOpTag _  -> "OpAssignOpTag"
+  | FixOpTag _     -> "FixOpTag"
+  | BinaryOpTag _  -> "BinaryOpTag"
+  | ArithOpTag _   -> "ArithOpTag"
   | LogicalOpTag _ -> "LogicalOpTag"
   | DeclarationTag _ -> "DeclarationTag"
-  | InitTag _ -> "InitTag"
-  | StorageTag _ -> "StorageTag"
-  | IncFileTag _ -> "IncFileTag"
+  | InitTag _      -> "InitTag"
+  | StorageTag _   -> "StorageTag"
+  | IncFileTag _   -> "IncFileTag"
   | Rule_elemTag _ -> "Rule_elemTag"
   | StatementTag _ -> "StatementTag"
-  | CaseLineTag _ -> "CaseLineTag"
-  | ConstVolTag _ -> "ConstVolTag"
+  | ForInfoTag _   -> "ForInfoTag"
+  | CaseLineTag _  -> "CaseLineTag"
+  | StringFragmentTag _ -> "StringFragmentTag"
+  | ConstVolTag _  -> "ConstVolTag"
   | Token _ -> "Token"
-  | Pragma _ -> "Pragma"
+  | Directive _ -> "Directive"
   | Code _ -> "Code"
   | ExprDotsTag _ -> "ExprDotsTag"
   | ParamDotsTag _ -> "ParamDotsTag"
   | StmtDotsTag _ -> "StmtDotsTag"
-  | DeclDotsTag _ -> "DeclDotsTag"
+  | AnnDeclDotsTag _ -> "AnnDeclDotsTag"
   | TypeCTag _ -> "TypeCTag"
   | ParamTag _ -> "ParamTag"
   | SgrepStartTag _ -> "SgrepStartTag"
@@ -766,7 +889,8 @@ and tag2c = function
 
 (* --------------------------------------------------------------------- *)
 
-let no_info = { line = 0; column = -1; strbef = []; straft = [] }
+let no_info = { line = 0; column = -1; strbef = []; straft = [];
+                whitespace = "" }
 
 let make_term x =
   {node = x;
@@ -775,6 +899,22 @@ let make_term x =
     minus_free_vars = [];
     fresh_vars = [];
     inherited = [];
+    positive_inherited_positions = [];
+    saved_witness = [];
+    bef_aft = NoDots;
+    pos_info = None;
+    true_if_test_exp = false;
+    safe_for_multi_decls = false;
+    iso_info = [] }
+
+let make_inherited_term x inherited inh_pos =
+  {node = x;
+    node_line = 0;
+    free_vars = [];
+    minus_free_vars = [];
+    fresh_vars = [];
+    inherited = inherited;
+    positive_inherited_positions = inh_pos;
     saved_witness = [];
     bef_aft = NoDots;
     pos_info = None;
@@ -785,16 +925,16 @@ let make_term x =
 let make_meta_rule_elem s d (fvs,fresh,inh) =
   let rule = "" in
   {(make_term
-      (MetaRuleElem(((rule,s),no_info,d,NoMetaPos),Type_cocci.Unitary,false)))
+      (MetaRuleElem(((rule,s),no_info,d,[]),Type_cocci.Unitary,false)))
   with free_vars = fvs; fresh_vars = fresh; inherited = inh}
 
 let make_meta_decl s d (fvs,fresh,inh) =
   let rule = "" in
   {(make_term
-      (MetaDecl(((rule,s),no_info,d,NoMetaPos),Type_cocci.Unitary,false))) with
+      (MetaDecl(((rule,s),no_info,d,[]),Type_cocci.Unitary,false))) with
     free_vars = fvs; fresh_vars = fresh; inherited = inh}
 
-let make_mcode x = (x,no_info,CONTEXT(NoPos,NOTHING),NoMetaPos)
+let make_mcode x = (x,no_info,CONTEXT(NoPos,NOTHING),[])
 
 (* --------------------------------------------------------------------- *)
 
@@ -807,3 +947,39 @@ let undots x =
     DOTS    e -> e
   | CIRCLES e -> e
   | STARS   e -> e
+
+let string_of_arithOp = function
+  | Plus -> "+"
+  | Minus -> "-"
+  | Mul -> "*"
+  | Div -> "/"
+  | Mod -> "%"
+  | DecLeft -> "<<"
+  | DecRight -> ">>"
+  | And -> "&"
+  | Or -> "|"
+  | Xor -> "^"
+  | Min -> "min"
+  | Max -> "max"
+
+let string_of_logicalOp = function
+  | Eq -> "=="
+  | NotEq -> "!="
+  | InfEq -> "<="
+  | SupEq -> ">="
+  | Sup -> ">"
+  | Inf -> "<"
+  | AndLog -> "&&"
+  | OrLog -> "||"
+
+let string_of_binaryOp op = match (unwrap op) with
+  | Arith arithOp -> string_of_arithOp (unwrap_mcode arithOp)
+  | Logical logicalOp -> string_of_logicalOp (unwrap_mcode logicalOp)
+  | MetaBinary _ -> "MetaBinary"
+
+let string_of_assignOp op = match (unwrap op) with
+  | SimpleAssign _ -> "="
+  | OpAssign op' ->
+    let s = string_of_arithOp (unwrap_mcode op') in
+    s ^ "="
+  | MetaAssign _ -> "MetaAssign"
