@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./ast_cocci.mli"
 (* --------------------------------------------------------------------- *)
 (* Modified code *)
 
@@ -29,7 +32,8 @@ type added_string = Noindent of string | Indent of string | Space of string
 
 type info = { line : int; column : int;
 	      strbef : (added_string * int (* line *) * int (* col *)) list;
-	      straft : (added_string * int (* line *) * int (* col *)) list }
+	      straft : (added_string * int (* line *) * int (* col *)) list;
+              whitespace : string }
 type line = int
 type meta_name = string * string
 type 'a wrap =
@@ -39,6 +43,7 @@ type 'a wrap =
       minus_free_vars : meta_name list; (*minus free vars*)
       fresh_vars : (meta_name * seed) list; (*fresh vars*)
       inherited : meta_name list; (*inherited vars*)
+      positive_inherited_positions : meta_name list;
       saved_witness : meta_name list; (*witness vars*)
       bef_aft : dots_bef_aft;
       pos_info : meta_name mcode option; (* pos info, try not to duplicate *)
@@ -54,11 +59,14 @@ and 'a befaft =
   | BEFOREAFTER of 'a list list * 'a list list * count
   | NOTHING
 
-and 'a mcode = 'a * info * mcodekind * meta_pos (* pos variable *)
+and 'a replacement = REPLACEMENT of 'a list list * count | NOREPLACEMENT
+
+and 'a mcode = 'a * info * mcodekind * meta_pos list (* pos variables *)
  (* pos is an offset indicating where in the C code the mcodekind has an
  effect *)
+and adjacency = ALLMINUS | ADJ of int
  and mcodekind =
-    MINUS       of pos * int list * int * anything list list
+    MINUS       of pos * int list * adjacency * anything replacement
   | CONTEXT     of pos * anything befaft
   | PLUS        of count
  and count = ONE (* + *) | MANY (* ++ *)
@@ -90,8 +98,11 @@ and metavar =
   | MetaFreshIdDecl of meta_name (* name *) * seed (* seed *)
   | MetaTypeDecl of arity * meta_name (* name *)
   | MetaInitDecl of arity * meta_name (* name *)
+  | MetaInitListDecl of arity * meta_name (* name *) * list_len (*len*)
   | MetaListlenDecl of meta_name (* name *)
   | MetaParamDecl of arity * meta_name (* name *)
+  | MetaBinaryOperatorDecl of arity * meta_name
+  | MetaAssignmentOperatorDecl of arity * meta_name
   | MetaParamListDecl of arity * meta_name (*name*) * list_len (*len*)
   | MetaConstDecl of
       arity * meta_name (* name *) * Type_cocci.typeC list option
@@ -102,6 +113,8 @@ and metavar =
       arity * meta_name (* name *) * Type_cocci.typeC list option
   | MetaLocalIdExpDecl of
       arity * meta_name (* name *) * Type_cocci.typeC list option
+  | MetaGlobalIdExpDecl of
+      arity * meta_name (* name *) * Type_cocci.typeC list option
   | MetaExpListDecl of arity * meta_name (*name*) * list_len (*len*)
   | MetaDeclDecl of arity * meta_name (* name *)
   | MetaFieldDecl of arity * meta_name (* name *)
@@ -111,6 +124,9 @@ and metavar =
   | MetaFuncDecl of arity * meta_name (* name *)
   | MetaLocalFuncDecl of arity * meta_name (* name *)
   | MetaPosDecl of arity * meta_name (* name *)
+  | MetaFmtDecl of arity * meta_name (* name *)
+  | MetaFragListDecl of arity * meta_name (* name *) * list_len (*len*)
+  | MetaAnalysisDecl of string * meta_name (* name *)
   | MetaDeclarerDecl of arity * meta_name (* name *)
   | MetaIteratorDecl of arity * meta_name (* name *)
 
@@ -138,6 +154,7 @@ and base_ident =
   | MetaId        of meta_name mcode * idconstraint * keep_binding * inherited
   | MetaFunc      of meta_name mcode * idconstraint * keep_binding * inherited
   | MetaLocalFunc of meta_name mcode * idconstraint * keep_binding * inherited
+  | AsIdent       of ident * ident (* as ident, always metavar *)
 
   | DisjId        of ident list
   | OptIdent      of ident
@@ -151,16 +168,19 @@ and ident = base_ident wrap
 and base_expression =
     Ident          of ident
   | Constant       of constant mcode
+  | StringConstant of string mcode (* quote *) * string_fragment dots *
+		      string mcode (* quote *)
   | FunCall        of expression * string mcode (* ( *) *
                       expression dots * string mcode (* ) *)
-  | Assignment     of expression * assignOp mcode * expression * bool
+  | Assignment     of expression * assignOp * expression * bool
+  | Sequence       of expression * string mcode (* , *) * expression
   | CondExpr       of expression * string mcode (* ? *) * expression option *
 	              string mcode (* : *) * expression
   | Postfix        of expression * fixOp mcode
   | Infix          of expression * fixOp mcode
   | Unary          of expression * unaryOp mcode
-  | Binary         of expression * binaryOp mcode * expression
-  | Nested         of expression * binaryOp mcode * expression
+  | Binary         of expression * binaryOp * expression
+  | Nested         of expression * binaryOp * expression
   | ArrayAccess    of expression * string mcode (* [ *) * expression *
 	              string mcode (* ] *)
   | RecordAccess   of expression * string mcode (* . *) * ident
@@ -176,19 +196,22 @@ and base_expression =
   | Paren          of string mcode (* ( *) * expression *
                       string mcode (* ) *)
 
+  | Constructor    of string mcode (* ( *) * fullType * string mcode (* ) *) *
+	              initialiser
   | MetaErr        of meta_name mcode * constraints * keep_binding *
 	              inherited
   | MetaExpr       of meta_name mcode * constraints * keep_binding *
 	              Type_cocci.typeC list option * form * inherited
   | MetaExprList   of meta_name mcode * listlen *
 	              keep_binding * inherited (* only in arg lists *)
+  | AsExpr         of expression * expression (* as expr, always metavar *)
 
   | EComma         of string mcode (* only in arg lists *)
 
   | DisjExpr       of expression list
   | NestExpr       of string mcode (* <.../<+... *) *
 	              expression dots *
-	              string mcode (* ...>/...+> *) * 
+	              string mcode (* ...>/...+> *) *
                       expression option * multi
 
   (* can appear in arg lists, and also inside Nest, as in:
@@ -210,14 +233,23 @@ and constraints =
 (* Constraints on Meta-* Identifiers, Functions *)
 and idconstraint =
     IdNoConstraint
+  | IdPosIdSet         of string list * meta_name list
   | IdNegIdSet         of string list * meta_name list
   | IdRegExpConstraint of reconstraint
 
 and reconstraint =
-  | IdRegExp        of string * Str.regexp
-  | IdNotRegExp     of string * Str.regexp
+  | IdRegExp        of string * Regexp.regexp
+  | IdNotRegExp     of string * Regexp.regexp
 
-and form = ANY | ID | LocalID | CONST (* form for MetaExp *)
+and assignOpconstraint =
+    AssignOpNoConstraint
+  | AssignOpInSet of assignOp list
+
+and binaryOpconstraint =
+    BinaryOpNoConstraint
+  | BinaryOpInSet of binaryOp list
+
+and form = ANY | ID | LocalID| GlobalID | CONST (* form for MetaExp *)
 
 and expression = base_expression wrap
 
@@ -226,13 +258,37 @@ and listlen =
   | CstListLen of int
   | AnyListLen
 
-and  unaryOp = GetRef | DeRef | UnPlus |  UnMinus | Tilde | Not
-and  assignOp = SimpleAssign | OpAssign of arithOp
+and base_string_fragment =
+    ConstantFragment of string mcode
+  | FormatFragment of string mcode (*%*) * string_format (* format *)
+  | Strdots of string mcode
+  | MetaFormatList of string mcode (*%*) * meta_name mcode * listlen *
+	keep_binding * inherited
+
+and string_fragment = base_string_fragment wrap
+
+and base_string_format =
+    ConstantFormat of string mcode
+  | MetaFormat of meta_name mcode * idconstraint * keep_binding * inherited
+
+and string_format = base_string_format wrap
+
+and  unaryOp = GetRef | GetRefLabel | DeRef | UnPlus |  UnMinus | Tilde | Not
+and  base_assignOp = 
+    SimpleAssign of simpleAssignOp mcode
+  | OpAssign of arithOp mcode
+  | MetaAssign of meta_name mcode * assignOpconstraint * keep_binding * inherited
+and simpleAssignOp = string
+and assignOp = base_assignOp wrap
 and  fixOp = Dec | Inc
 
-and  binaryOp = Arith of arithOp | Logical of logicalOp
+and  base_binaryOp =
+    Arith of arithOp mcode
+  | Logical of logicalOp mcode
+  | MetaBinary of meta_name mcode * binaryOpconstraint * keep_binding * inherited
+and binaryOp = base_binaryOp wrap
 and  arithOp =
-    Plus | Minus | Mul | Div | Mod | DecLeft | DecRight | And | Or | Xor
+    Plus | Minus | Mul | Div | Mod | DecLeft | DecRight | And | Or | Xor | Min | Max
 and  logicalOp = Inf | Sup | InfEq | SupEq | Eq | NotEq | AndLog | OrLog
 
 and constant =
@@ -240,12 +296,15 @@ and constant =
   | Char   of string
   | Int    of string
   | Float  of string
+  | DecimalConst of (string * string * string)
 
 (* --------------------------------------------------------------------- *)
 (* Types *)
 
 and base_fullType =
-    Type            of const_vol mcode option * typeC
+    Type            of bool (* true if all minus *) *
+                       const_vol mcode option * typeC
+  | AsType          of fullType * fullType (* as type, always metavar *)
   | DisjType        of fullType list (* only after iso *)
   | OptType         of fullType
   | UniqueType      of fullType
@@ -257,18 +316,18 @@ and base_typeC =
   | FunctionPointer of fullType *
 	          string mcode(* ( *)*string mcode(* * *)*string mcode(* ) *)*
                   string mcode (* ( *)*parameter_list*string mcode(* ) *)
-  | FunctionType     of bool (* true if all minus for dropping return type *) *
-                   fullType option *
-	           string mcode (* ( *) * parameter_list *
-                   string mcode (* ) *)
   | Array           of fullType * string mcode (* [ *) *
 	               expression option * string mcode (* ] *)
+  | Decimal         of string mcode (* decimal *) * string mcode (* ( *) *
+	               expression *
+	               string mcode option (* , *) * expression option *
+	               string mcode (* ) *) (* IBM C only *)
   | EnumName        of string mcode (*enum*) * ident option (* name *)
   | EnumDef  of fullType (* either EnumName or metavar *) *
 	string mcode (* { *) * expression dots * string mcode (* } *)
   | StructUnionName of structUnion mcode * ident option (* name *)
   | StructUnionDef  of fullType (* either StructUnionName or metavar *) *
-	string mcode (* { *) * declaration dots * string mcode (* } *)
+	string mcode (* { *) * annotated_decl dots * string mcode (* } *)
   | TypeName        of string mcode
 
   | MetaType        of meta_name mcode * keep_binding * inherited
@@ -276,8 +335,10 @@ and base_typeC =
 and fullType = base_fullType wrap
 and typeC = base_typeC wrap
 
-and baseType = VoidType | CharType | ShortType | IntType | DoubleType
-  | FloatType | LongType | LongLongType | SizeType | SSizeType | PtrDiffType
+and baseType = VoidType | CharType | ShortType | ShortIntType | IntType
+| DoubleType | LongDoubleType | FloatType
+| LongType | LongIntType | LongLongType | LongLongIntType
+| SizeType | SSizeType | PtrDiffType
 
 and structUnion = Struct | Union
 
@@ -294,27 +355,45 @@ and base_declaration =
     Init of storage mcode option * fullType * ident * string mcode (*=*) *
 	initialiser * string mcode (*;*)
   | UnInit of storage mcode option * fullType * ident * string mcode (* ; *)
+  | FunProto of
+	fninfo list * ident (* name *) *
+	string mcode (* ( *) * parameter_list *
+	(string mcode (* , *) * string mcode (* ...... *) ) option *
+	string mcode (* ) *) * string mcode (* ; *)
   | TyDecl of fullType * string mcode (* ; *)
   | MacroDecl of ident (* name *) * string mcode (* ( *) *
         expression dots * string mcode (* ) *) * string mcode (* ; *)
+  | MacroDeclInit of ident (* name *) * string mcode (* ( *) *
+        expression dots * string mcode (* ) *) * string mcode (*=*) *
+        initialiser * string mcode (* ; *)
   | Typedef of string mcode (*typedef*) * fullType * typeC * string mcode (*;*)
   | DisjDecl   of declaration list
-  | Ddots    of string mcode (* ... *) * declaration option (* whencode *)
 
   | MetaDecl of meta_name mcode * keep_binding * inherited
   | MetaField of meta_name mcode * keep_binding * inherited
   | MetaFieldList of meta_name mcode * listlen * keep_binding * inherited
+  | AsDecl        of declaration * declaration
 
   | OptDecl    of declaration
   | UniqueDecl of declaration
 
 and declaration = base_declaration wrap
 
+and base_annotated_decl =
+    DElem of mcodekind (* before the decl *) * bool (* true if all minus *) *
+      declaration
+  (* Ddots is for a structure declaration *)
+  | Ddots    of string mcode (* ... *) * declaration option (* whencode *)
+
+and annotated_decl = base_annotated_decl wrap
+
 (* --------------------------------------------------------------------- *)
 (* Initializers *)
 
 and base_initialiser =
     MetaInit of meta_name mcode * keep_binding * inherited
+  | MetaInitList of meta_name mcode * listlen * keep_binding * inherited
+  | AsInit of initialiser * initialiser (* as init, always metavar *)
   | InitExpr of expression
   | ArInitList of string mcode (*{*) * initialiser dots * string mcode (*}*)
   | StrInitList of bool (* true if all are - *) *
@@ -349,6 +428,8 @@ and base_parameterTypeDef =
   | MetaParam     of meta_name mcode * keep_binding * inherited
   | MetaParamList of meta_name mcode * listlen * keep_binding * inherited
 
+  | AsParam       of parameterTypeDef * expression (* expr, always metavar *)
+
   | PComma        of string mcode
 
   | Pdots         of string mcode (* ... *)
@@ -389,7 +470,6 @@ and meta_collect = PER | ALL
 and meta_pos =
     MetaPos of meta_name mcode * meta_name list *
 	meta_collect * keep_binding * inherited
-  | NoMetaPos
 
 (* --------------------------------------------------------------------- *)
 (* Function declaration *)
@@ -404,14 +484,14 @@ and base_rule_elem =
 	             bool (* true if all minus, for dropping static, etc *) *
 	             fninfo list * ident (* name *) *
 	             string mcode (* ( *) * parameter_list *
+                     (string mcode (* , *) * string mcode (* ...... *) ) option *
                      string mcode (* ) *)
-  | Decl          of mcodekind (* before the decl *) *
-                     bool (* true if all minus *) * declaration
+  | Decl          of annotated_decl
 
   | SeqStart      of string mcode (* { *)
   | SeqEnd        of string mcode (* } *)
 
-  | ExprStatement of expression * string mcode (*;*)
+  | ExprStatement of expression option * string mcode (*;*)
   | IfHeader      of string mcode (* if *) * string mcode (* ( *) *
 	             expression * string mcode (* ) *)
   | Else          of string mcode (* else *)
@@ -422,7 +502,7 @@ and base_rule_elem =
 	             expression * string mcode (* ) *) *
                      string mcode (* ; *)
   | ForHeader     of string mcode (* for *) * string mcode (* ( *) *
-                     expression option * string mcode (*;*) *
+                     forinfo *
 	             expression option * string mcode (*;*) *
                      expression option * string mcode (* ) *)
   | IteratorHeader of ident (* name *) * string mcode (* ( *) *
@@ -436,6 +516,8 @@ and base_rule_elem =
   | Return        of string mcode (* return *) * string mcode (* ; *)
   | ReturnExpr    of string mcode (* return *) * expression *
 	             string mcode (* ; *)
+  | Exec          of string mcode (* EXEC *) * string mcode (* language *) *
+	             exec_code dots * string mcode (* ; *)
 
   | MetaRuleElem  of meta_name mcode * keep_binding * inherited
   | MetaStmt      of meta_name mcode * keep_binding * metaStmtInfo *
@@ -450,10 +532,22 @@ and base_rule_elem =
   | Undef         of string mcode (* #define *) * ident (* name *)
   | DefineHeader  of string mcode (* #define *) * ident (* name *) *
 	             define_parameters (*params*)
+  | Pragma        of string mcode (* #pragma *) * ident * pragmainfo
   | Case          of string mcode (* case *) * expression * string mcode (*:*)
   | Default       of string mcode (* default *) * string mcode (*:*)
   | DisjRuleElem  of rule_elem list
 
+and base_pragmainfo =
+    PragmaTuple of string mcode(* ( *) * expression dots * string mcode(* ) *)
+  | PragmaIdList of ident dots
+  | PragmaDots of string mcode
+
+and pragmainfo = base_pragmainfo wrap
+
+and forinfo =
+    ForExp of expression option * string mcode (*;*)
+  | ForDecl of annotated_decl
+
 and fninfo =
     FStorage of storage mcode
   | FType of fullType
@@ -480,12 +574,13 @@ and base_statement =
   | Atomic        of rule_elem
   | Disj          of statement dots list
   | Nest          of string mcode (* <.../<+... *) * statement dots *
-	             string mcode (* ...>/...+> *) * 
+	             string mcode (* ...>/...+> *) *
 	             (statement dots,statement) whencode list * multi *
 	             dots_whencode list * dots_whencode list
   | FunDecl       of rule_elem (* header *) * rule_elem (* { *) *
-     	             statement dots * rule_elem (* } *)
+     	             statement dots * rule_elem (* } *) * end_info (*exit*)
   | Define        of rule_elem (* header *) * statement dots
+  | AsStmt        of statement * statement (* as statement, always metavar *)
   | Dots          of string mcode (* ... *) *
 	             (statement dots,statement) whencode list *
 	             dots_whencode list * dots_whencode list
@@ -524,6 +619,13 @@ and base_case_line =
 
 and case_line = base_case_line wrap
 
+and base_exec_code =
+    ExecEval of string mcode (* : *) * expression
+  | ExecToken of string mcode
+  | ExecDots of string mcode (* ... *)
+
+and exec_code = base_exec_code wrap
+
 and inc_file =
     Local of inc_elem list
   | NonLocal of inc_elem list
@@ -533,20 +635,20 @@ and inc_elem =
   | IncDots
 
 and base_top_level =
-    DECL of statement
+    NONDECL of statement (* cannot match all of a top-level declaration *)
   | CODE of statement dots
   | FILEINFO of string mcode (* old file *) * string mcode (* new file *)
   | ERRORWORDS of expression list
 
 and top_level = base_top_level wrap
 
+and parser_kind = ExpP | TyP | AnyP
+
 and rulename =
     CocciRulename of string option * dependency * string list * string list *
-	exists * bool
-      (* true if the whole thing is an expression *)
+	exists * parser_kind
   | GeneratedRulename of string option * dependency *
-	string list * string list * exists * bool
-      (* true if the whole thing is an expression *)
+	string list * string list * exists * parser_kind
   | ScriptRulename of string option (* name *) * string (* language *) *
 	dependency
   | InitialScriptRulename of string option (* name *) * string (* language *) *
@@ -564,10 +666,12 @@ and rule =
       string * dependency *
 	(script_meta_name * meta_name * metavar) list *
 	meta_name list (*script vars*) * string
-  | InitialScriptRule of  string (* name *) *
-	string * dependency * string
-  | FinalScriptRule of  string (* name *) *
-	string * dependency * string
+  | InitialScriptRule of  string (* name *) * string * dependency *
+	(script_meta_name * meta_name * metavar) list (*virtual vars*) *
+	string
+  | FinalScriptRule of  string (* name *) * string * dependency *
+	(script_meta_name * meta_name * metavar) list (*virtual vars*) *
+	string
 
 and script_meta_name = string option (*string*) * string option (*ast*)
 
@@ -592,6 +696,8 @@ and anything =
   | ConstantTag         of constant
   | UnaryOpTag          of unaryOp
   | AssignOpTag         of assignOp
+  | SimpleAssignOpTag   of simpleAssignOp
+  | OpAssignOpTag       of arithOp
   | FixOpTag            of fixOp
   | BinaryOpTag         of binaryOp
   | ArithOpTag          of arithOp
@@ -602,15 +708,17 @@ and anything =
   | IncFileTag          of inc_file
   | Rule_elemTag        of rule_elem
   | StatementTag        of statement
+  | ForInfoTag          of forinfo
   | CaseLineTag         of case_line
+  | StringFragmentTag   of string_fragment
   | ConstVolTag         of const_vol
   | Token               of string * info option
-  | Pragma              of added_string list
+  | Directive           of added_string list
   | Code                of top_level
   | ExprDotsTag         of expression dots
   | ParamDotsTag        of parameterTypeDef dots
   | StmtDotsTag         of statement dots
-  | DeclDotsTag         of declaration dots
+  | AnnDeclDotsTag      of annotated_decl dots
   | TypeCTag            of typeC
   | ParamTag            of parameterTypeDef
   | SgrepStartTag       of string
@@ -631,7 +739,7 @@ val lub_count : count -> count -> count
 (* --------------------------------------------------------------------- *)
 
 val rewrap : 'a wrap -> 'b -> 'b wrap
-val rewrap_mcode : 'a mcode -> 'a -> 'a mcode
+val rewrap_mcode : 'a mcode -> 'b -> 'b mcode
 val unwrap : 'a wrap -> 'a
 val unwrap_mcode : 'a mcode -> 'a
 val get_mcodekind : 'a mcode -> mcodekind
@@ -645,6 +753,7 @@ val get_mfvs : 'a wrap -> meta_name list
 val set_mfvs : meta_name list -> 'a wrap -> 'a wrap
 val get_fresh : 'a wrap -> (meta_name * seed) list
 val get_inherited : 'a wrap -> meta_name list
+val get_inherited_pos : 'a wrap -> meta_name list
 val get_saved : 'a wrap -> meta_name list
 val get_dots_bef_aft : statement -> dots_bef_aft
 val set_dots_bef_aft : dots_bef_aft -> statement -> statement
@@ -655,8 +764,8 @@ val set_test_exp : expression -> expression
 val get_safe_decl : 'a wrap -> bool
 val get_isos : 'a wrap -> (string*anything) list
 val set_isos : 'a wrap -> (string*anything) list -> 'a wrap
-val get_pos_var : 'a mcode -> meta_pos
-val set_pos_var : meta_pos -> 'a mcode -> 'a mcode
+val get_pos_var : 'a mcode -> meta_pos list
+val set_pos_var : meta_pos list -> 'a mcode -> 'a mcode
 val drop_pos : 'a mcode -> 'a mcode
 
 val get_meta_name : metavar -> meta_name
@@ -676,6 +785,13 @@ val make_meta_decl :
       declaration
 
 val make_term : 'a -> 'a wrap
+val make_inherited_term : 'a -> meta_name list (* inherited vars *) ->
+  meta_name list (* definitely inherited positions *) -> 'a wrap
 val make_mcode : 'a -> 'a mcode
 
 val equal_pos : fixpos -> fixpos -> bool
+
+val string_of_arithOp : arithOp -> string
+val string_of_logicalOp : logicalOp -> string
+val string_of_assignOp : assignOp -> string
+val string_of_binaryOp : binaryOp -> string
