@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./check_meta.ml"
 (* For minus fragment, checks that all of the identifier metavariables that
 are used are not declared as fresh, and check that all declared variables
 are used.  For plus fragment, just check that the variables declared as
@@ -102,19 +105,46 @@ let rec ident context old_metas table minus i =
 	else false in
       (match context with
 	ID ->
-	  if not (is_ifdef name) && minus && not err(* warn only once per id *)
+	  if not (is_ifdef name) && minus &&
+	    not err(* warn only once per id *) && not info.Ast0.isSymbolIdent
 	  then
 	    warning
 	      (Printf.sprintf "line %d: should %s be a metavariable?" rl name)
       | _ -> ())
-  | Ast0.MetaId(name,_,_) -> check_table table minus name
+  | Ast0.MetaId(name,_,seedval,_) ->
+      check_table table minus name;
+      seed table minus seedval
   | Ast0.MetaFunc(name,_,_) -> check_table table minus name
   | Ast0.MetaLocalFunc(name,_,_) -> check_table table minus name
+  | Ast0.AsIdent(id,asid) -> failwith "not generated yet"
   | Ast0.DisjId(_,id_list,_,_) ->
       List.iter (ident context old_metas table minus) id_list
   | Ast0.OptIdent(_) | Ast0.UniqueIdent(_) ->
       failwith "unexpected code"
-	
+
+and seed table minus = function
+    Ast.NoVal -> ()
+  | Ast.StringSeed _ -> ()
+  | Ast.ListSeed elems ->
+      List.iter
+	(function
+	    Ast.SeedString _ -> ()
+	  | Ast.SeedId name -> check_table table minus (promote name))
+	elems
+
+(* --------------------------------------------------------------------- *)
+(* Operators *)
+
+let assignOp context old_metas table minus op = match Ast0.unwrap op with
+  | Ast0.SimpleAssign _ -> ()
+  | Ast0.OpAssign _ -> ()
+  | Ast0.MetaAssign (name,_,_) -> check_table table minus name
+
+let binaryOp context old_metas table minus op = match Ast0.unwrap op with
+  | Ast0.Arith _ -> ()
+  | Ast0.Logical _ -> ()
+  | Ast0.MetaBinary (name,_,_) -> check_table table minus name
+
 (* --------------------------------------------------------------------- *)
 (* Expression *)
 
@@ -122,10 +152,16 @@ let rec expression context old_metas table minus e =
   match Ast0.unwrap e with
     Ast0.Ident(id) ->
       ident context old_metas table minus id
+  | Ast0.StringConstant(lq,str,rq) ->
+      dots (string_fragment old_metas table minus) str
   | Ast0.FunCall(fn,lp,args,rp) ->
       expression FN old_metas table minus fn;
       dots (expression ID old_metas table minus) args
   | Ast0.Assignment(left,op,right,_) ->
+      expression context old_metas table minus left;
+      assignOp context old_metas table minus op;
+      expression ID old_metas table minus right
+  | Ast0.Sequence(left,op,right) ->
       expression context old_metas table minus left;
       expression ID old_metas table minus right
   | Ast0.CondExpr(exp1,why,exp2,colon,exp3) ->
@@ -140,6 +176,11 @@ let rec expression context old_metas table minus e =
       expression ID old_metas table minus exp
   | Ast0.Binary(left,op,right) ->
       expression ID old_metas table minus left;
+      binaryOp context old_metas table minus op;
+      expression ID old_metas table minus right
+  | Ast0.Nested(left,op,right) ->
+      expression ID old_metas table minus left;
+      binaryOp context old_metas table minus op;
       expression ID old_metas table minus right
   | Ast0.Paren(lp,exp,rp) ->
       expression ID old_metas table minus exp
@@ -157,12 +198,14 @@ let rec expression context old_metas table minus e =
   | Ast0.SizeOfExpr(szf,exp) -> expression ID old_metas table minus exp
   | Ast0.SizeOfType(szf,lp,ty,rp) -> typeC old_metas table minus ty
   | Ast0.TypeExp(ty) -> typeC old_metas table minus ty
+  | Ast0.Constructor(lp,ty,rp,init) ->
+      typeC old_metas table minus ty; initialiser old_metas table minus init
   | Ast0.MetaExpr(name,_,Some tys,_,_) ->
       List.iter
 	(function x ->
-	  match get_type_name x with
-	    Some(ty) -> check_table table minus (promote ty)
-	  | None -> ())
+	  List.iter
+	    (function ty -> check_table table minus (promote ty))
+	    (get_type_name x))
 	tys;
       check_table table minus name
   | Ast0.MetaExpr(name,_,_,_,_) | Ast0.MetaErr(name,_,_) ->
@@ -172,12 +215,16 @@ let rec expression context old_metas table minus e =
       check_table table minus lenname
   | Ast0.MetaExprList(name,_,_) ->
       check_table table minus name
+  | Ast0.AsExpr(exp,asexp) -> failwith "not generated yet"
   | Ast0.DisjExpr(_,exps,_,_) ->
       List.iter (expression context old_metas table minus) exps
   | Ast0.NestExpr(_,exp_dots,_,w,_) ->
       dots (expression ID old_metas table minus) exp_dots;
-      get_opt (expression ID old_metas table minus) w
-  | Ast0.Edots(_,Some x) | Ast0.Ecircles(_,Some x) | Ast0.Estars(_,Some x) ->
+      get_opt (function (_,_,x) -> expression ID old_metas table minus x) w
+  | Ast0.Edots(_,Some (_,_,x)) | Ast0.Ecircles(_,Some (_,_,x))
+  | Ast0.Estars(_,Some (_,_,x)) ->
+      expression ID old_metas table minus x
+  | Ast0.OptExp(x) | Ast0.UniqueExp(x) ->
       expression ID old_metas table minus x
   | _ -> () (* no metavariable subterms *)
 
@@ -185,10 +232,15 @@ and get_type_name = function
     Type_cocci.ConstVol(_,ty) | Type_cocci.SignedT(_,Some ty)
   | Type_cocci.Pointer(ty)
   | Type_cocci.FunctionPointer(ty) | Type_cocci.Array(ty) -> get_type_name ty
-  | Type_cocci.EnumName(Type_cocci.MV(nm,_,_)) -> Some nm
-  | Type_cocci.StructUnionName(_,Type_cocci.MV(nm,_,_)) -> Some nm
-  | Type_cocci.MetaType(nm,_,_) -> Some nm
-  | _ -> None
+  | Type_cocci.EnumName(Type_cocci.MV(nm,_,_)) -> [nm]
+  | Type_cocci.StructUnionName(_,Type_cocci.MV(nm,_,_)) -> [nm]
+  | Type_cocci.MetaType(nm,_,_) -> [nm]
+  | Type_cocci.Decimal(nm1,nm2) ->
+      let get_name = function
+	  Type_cocci.MV(nm,_,_) -> [nm]
+	| _ -> [] in
+      (get_name nm1) @ (get_name nm2)
+  | _ -> []
 
 (* --------------------------------------------------------------------- *)
 (* Types *)
@@ -202,14 +254,15 @@ and typeC old_metas table minus t =
   | Ast0.FunctionPointer(ty,lp1,star,rp1,lp2,params,rp2) ->
       typeC old_metas table minus ty;
       parameter_list old_metas table minus params
-  | Ast0.FunctionType(ty,lp1,params,rp1) ->
-      get_opt (typeC old_metas table minus) ty;
-      parameter_list old_metas table minus params
   | Ast0.Array(ty,lb,size,rb) ->
       typeC old_metas table minus ty;
       get_opt (expression ID old_metas table minus) size
+  | Ast0.Decimal(dec,lp,length,comma,precision_opt,rp) ->
+      expression ID old_metas table minus length;
+      get_opt (expression ID old_metas table minus) precision_opt
   | Ast0.MetaType(name,_) ->
       check_table table minus name
+  | Ast0.AsType(ty,asty) -> failwith "not generated yet"
   | Ast0.DisjType(_,types,_,_) ->
       List.iter (typeC old_metas table minus) types
   | Ast0.EnumName(en,Some id) -> ident GLOBAL old_metas table minus id
@@ -238,11 +291,12 @@ and declaration context old_metas table minus d =
       check_table table minus lenname
   | Ast0.MetaFieldList(name,_,_) ->
       check_table table minus name
+  | Ast0.AsDecl(decl,asdecl) -> failwith "not generated yet"
   | Ast0.Init(stg,ty,id,eq,ini,sem) ->
+      typeC old_metas table minus ty;
+      ident context old_metas table minus id;
       (match Ast0.unwrap ini with
 	Ast0.InitExpr exp ->
-	  typeC old_metas table minus ty;
-	  ident context old_metas table minus id;
 	  expression ID old_metas table minus exp
       |	_ ->
 	  (*
@@ -250,21 +304,29 @@ and declaration context old_metas table minus d =
 	  then
 	    failwith "complex initializer specification not allowed in - code"
 	  else*)
-	    (typeC old_metas table minus ty;
-	     ident context old_metas table minus id;
-	     initialiser old_metas table minus ini))
+	    initialiser old_metas table minus ini)
   | Ast0.UnInit(stg,ty,id,sem) ->
       typeC old_metas table minus ty; ident context old_metas table minus id
+  | Ast0.FunProto(fi,name,lp1,params,va,rp1,sem) ->
+      ident FN old_metas table minus name;
+      List.iter (fninfo old_metas table minus) fi;
+      parameter_list old_metas table minus params
   | Ast0.MacroDecl(name,lp,args,rp,sem) ->
       ident GLOBAL old_metas table minus name;
       dots (expression ID old_metas table minus) args
+  | Ast0.MacroDeclInit(name,lp,args,rp,eq,ini,sem) ->
+      ident GLOBAL old_metas table minus name;
+      dots (expression ID old_metas table minus) args;
+      (match Ast0.unwrap ini with
+	Ast0.InitExpr exp -> expression ID old_metas table minus exp
+      |	_ -> initialiser old_metas table minus ini)
   | Ast0.TyDecl(ty,sem) -> typeC old_metas table minus ty
   | Ast0.Typedef(stg,ty,id,sem) ->
       typeC old_metas table minus ty;
       typeC old_metas table minus id
   | Ast0.DisjDecl(_,decls,_,_) ->
       List.iter (declaration ID old_metas table minus) decls
-  | Ast0.Ddots(_,Some x) -> declaration ID old_metas table minus x
+  | Ast0.Ddots(_,Some (_,_,x)) -> declaration ID old_metas table minus x
   | Ast0.Ddots(_,None) -> ()
   | Ast0.OptDecl(_) | Ast0.UniqueDecl(_) ->
       failwith "unexpected code"
@@ -276,6 +338,12 @@ and initialiser old_metas table minus ini =
   match Ast0.unwrap ini with
     Ast0.MetaInit(name,_) ->
       check_table table minus name
+  | Ast0.MetaInitList(name,Ast0.MetaListLen lenname,_) ->
+      check_table table minus name;
+      check_table table minus lenname
+  | Ast0.MetaInitList(name,_,_) ->
+      check_table table minus name
+  | Ast0.AsInit(ini,asini) -> failwith "not generated yet"
   | Ast0.InitExpr(exp) -> expression ID old_metas table minus exp
   | Ast0.InitList(lb,initlist,rb,ordered) ->
       dots (initialiser old_metas table minus) initlist
@@ -285,7 +353,7 @@ and initialiser old_metas table minus ini =
   | Ast0.InitGccName(name,eq,ini) ->
       ident FIELD old_metas table minus name;
       initialiser old_metas table minus ini
-  | Ast0.Idots(_,Some x) -> initialiser old_metas table minus x
+  | Ast0.Idots(_,Some (_,_,x)) -> initialiser old_metas table minus x
   | Ast0.OptIni(_) | Ast0.UniqueIni(_) ->
       failwith "unexpected code"
   | _ -> () (* no metavariable subterms *)
@@ -322,6 +390,27 @@ and parameterTypeDef old_metas table minus param =
 and parameter_list old_metas table minus =
   dots (parameterTypeDef old_metas table minus)
 
+(* --------------------------------------------------------------------- *)
+(* String fragment *)
+
+and string_fragment old_metas table minus e =
+  match Ast0.unwrap e with
+    Ast0.ConstantFragment(str) -> ()
+  | Ast0.FormatFragment(pct,fmt) ->
+      string_format old_metas table minus fmt
+  | Ast0.Strdots dots -> ()
+  | Ast0.MetaFormatList(pct,name,Ast0.MetaListLen lenname) ->
+      check_table table minus name;
+      check_table table minus lenname
+  | Ast0.MetaFormatList(pct,name,lenname) ->
+      check_table table minus name
+
+and string_format old_metas table minus e =
+  match Ast0.unwrap e with
+    Ast0.ConstantFormat(str) -> ()
+  | Ast0.MetaFormat(name,_) ->
+      check_table table minus name
+
 (* --------------------------------------------------------------------- *)
 (* Top-level code *)
 
@@ -329,7 +418,8 @@ and statement old_metas table minus s =
   match Ast0.unwrap s with
     Ast0.Decl(_,decl) -> declaration ID old_metas table minus decl
   | Ast0.Seq(lbrace,body,rbrace) -> dots (statement old_metas table minus) body
-  | Ast0.ExprStatement(exp,sem) -> expression ID old_metas table minus exp
+  | Ast0.ExprStatement(exp,sem) ->
+      get_opt (expression ID old_metas table minus) exp
   | Ast0.IfThen(iff,lp,exp,rp,branch,_) ->
       expression ID old_metas table minus exp;
       statement old_metas table minus branch
@@ -343,8 +433,12 @@ and statement old_metas table minus s =
   | Ast0.Do(d,body,wh,lp,exp,rp,sem) ->
       statement old_metas table minus body;
       expression ID old_metas table minus exp
-  | Ast0.For(fr,lp,exp1,sem1,exp2,sem2,exp3,rp,body,_) ->
-      get_opt (expression ID old_metas table minus) exp1;
+  | Ast0.For(fr,lp,first,exp2,sem2,exp3,rp,body,_) ->
+      (match Ast0.unwrap first with
+	Ast0.ForExp(exp1,sem1) ->
+	  get_opt (expression ID old_metas table minus) exp1
+      |	Ast0.ForDecl (_,decl) ->
+	  declaration ID old_metas table minus decl);
       get_opt (expression ID old_metas table minus) exp2;
       get_opt (expression ID old_metas table minus) exp3;
       statement old_metas table minus body
@@ -357,8 +451,11 @@ and statement old_metas table minus s =
       dots (statement old_metas table minus) decls;
       dots (case_line old_metas table minus) cases
   | Ast0.ReturnExpr(ret,exp,sem) -> expression ID old_metas table minus exp
+  | Ast0.Exec(exec,lang,code,sem) ->
+      dots (exec_code ID old_metas table minus) code
   | Ast0.MetaStmt(name,_) ->     check_table table minus name
   | Ast0.MetaStmtList(name,_) -> check_table table minus name
+  | Ast0.AsStmt(stm,asstm) -> failwith "not generated yet"
   | Ast0.Exp(exp) -> expression ID old_metas table minus exp
   | Ast0.TopExp(exp) -> expression ID old_metas table minus exp
   | Ast0.Ty(ty) -> typeC old_metas table minus ty
@@ -376,7 +473,7 @@ and statement old_metas table minus s =
 	(whencode (dots (statement old_metas table minus))
 	   (statement old_metas table minus)
 	   (expression ID old_metas table minus)) x
-  | Ast0.FunDecl(_,fi,name,lp,params,rp,lbrace,body,rbrace) ->
+  | Ast0.FunDecl(_,fi,name,lp,params,va,rp,lbrace,body,rbrace,_) ->
       ident FN old_metas table minus name;
       List.iter (fninfo old_metas table minus) fi;
       parameter_list old_metas table minus params;
@@ -388,10 +485,20 @@ and statement old_metas table minus s =
       ident GLOBAL old_metas table minus id;
       define_parameters old_metas table minus params;
       dots (statement old_metas table minus) body
+  | Ast0.Pragma(prg,id,body) ->
+      ident GLOBAL old_metas table minus id;
+      pragmainfo old_metas table minus body
   | Ast0.Label(i,_) -> ident ID old_metas table minus i
   | Ast0.Goto(_,i,_) -> ident ID old_metas table minus i
   | _ -> () (* no metavariable subterms *)
 
+and pragmainfo old_metas table minus pi =
+  match Ast0.unwrap pi with
+      Ast0.PragmaTuple(lp,args,rp) ->
+	dots (expression ID old_metas table minus) args
+    | Ast0.PragmaIdList(ids) -> dots (ident GLOBAL old_metas table minus) ids
+    | Ast0.PragmaDots (dots) -> ()
+
 and define_param old_metas table minus p =
   match Ast0.unwrap p with
     Ast0.DParam(id) -> ident GLOBAL old_metas table minus id
@@ -412,11 +519,11 @@ and fninfo old_metas table minus = function
   | Ast0.FAttr(attr) -> ()
 
 and whencode notfn alwaysfn expression = function
-    Ast0.WhenNot a -> notfn a
-  | Ast0.WhenAlways a -> alwaysfn a
-  | Ast0.WhenModifier(_) -> ()
-  | Ast0.WhenNotTrue a -> expression a
-  | Ast0.WhenNotFalse a -> expression a
+    Ast0.WhenNot (_,_,a) -> notfn a
+  | Ast0.WhenAlways (_,_,a) -> alwaysfn a
+  | Ast0.WhenModifier _ -> ()
+  | Ast0.WhenNotTrue (_,_,a) -> expression a
+  | Ast0.WhenNotFalse (_,_,a) -> expression a
 
 and case_line old_metas table minus c =
   match Ast0.unwrap c with
@@ -429,13 +536,20 @@ and case_line old_metas table minus c =
       List.iter (case_line old_metas table minus) case_lines
   | Ast0.OptCase(case) -> failwith "unexpected code"
 
+and exec_code context old_metas table minus e =
+  match Ast0.unwrap e with
+    Ast0.ExecEval(colon,id) -> expression context old_metas table minus id
+  | Ast0.ExecToken(tok) -> ()
+  | Ast0.ExecDots(dots) -> ()
+
 (* --------------------------------------------------------------------- *)
 (* Rules *)
 
 let top_level old_metas table minus t =
   match Ast0.unwrap t with
-    Ast0.DECL(stmt) -> statement old_metas table minus stmt
-  | Ast0.CODE(stmt_dots) -> dots (statement old_metas table minus) stmt_dots
+    Ast0.NONDECL(stmt) -> statement old_metas table minus stmt
+  | Ast0.CODE(stmt_dots) | Ast0.TOPCODE(stmt_dots) ->
+      dots (statement old_metas table minus) stmt_dots
   | Ast0.ERRORWORDS(exps) ->
       List.iter (expression FN old_metas table minus) exps
   | _ -> () (* no metavariables possible *)
@@ -446,30 +560,42 @@ let rule old_metas table minus rules =
 (* --------------------------------------------------------------------- *)
 
 let positions table rules =
-  let mcode x =
-    match Ast0.get_pos x with
-      Ast0.MetaPos(name,constraints,_) ->
-	let pos = Ast0.unwrap_mcode name in
-	(find_loop table pos) := true
-    | _ -> () in
+  let rec rmcode x = (* needed for type inference, nonpolymorphic *)
+    List.iter
+      (function var ->
+	let name = Ast0.meta_pos_name var in
+	(find_loop table (Ast0.unwrap_mcode name)) := true;
+	rmcode name)
+      (Ast0.get_pos x) in
+  let rec mcode x =
+    List.iter
+      (function var ->
+	let name = Ast0.meta_pos_name var in
+	(find_loop table (Ast0.unwrap_mcode name)) := true;
+	rmcode name)
+      (Ast0.get_pos x) in
   let option_default = () in
   let bind x y = () in
   let donothing r k e = k e in
   let fn =
     V0.flat_combiner bind option_default
-      mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
+      mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
+      mcode mcode mcode mcode
       donothing donothing donothing donothing donothing donothing
       donothing donothing donothing donothing donothing donothing donothing
-      donothing donothing in
+      donothing donothing donothing donothing donothing donothing in
 
   List.iter fn.VT0.combiner_rec_top_level rules
 
 let dup_positions rules =
   let mcode x =
-    match Ast0.get_pos x with
-      Ast0.MetaPos(name,constraints,_) ->
-	let pos = Ast0.unwrap_mcode name in [pos]
-    | _ -> [] in
+    List.concat
+      (List.map
+	 (function
+	     Ast0.MetaPosTag(Ast0.MetaPos(name,constraints,_)) ->
+	       [Ast0.unwrap_mcode name]
+	   | _ -> [])
+	 (Ast0.get_pos x)) in
   let option_default = [] in
   let bind x y = x@y in
 
@@ -484,7 +610,7 @@ let dup_positions rules =
 	  (List.map r.VT0.combiner_rec_expression explist)
     | _ -> k e in
 
-  let typeC r k e = (* not sure relevent because "only after iso" *)
+  let typeC r k e = (* not sure relevant because "only after iso" *)
     match Ast0.unwrap e with
       Ast0.DisjType(_,types,_,_) ->
 	List.fold_left Common.union_set option_default
@@ -508,10 +634,12 @@ let dup_positions rules =
   let donothing r k e = k e in
   let fn =
     V0.flat_combiner bind option_default
-      mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
+      mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
+      mcode mcode mcode mcode
       donothing donothing donothing donothing donothing donothing
-      donothing expression typeC donothing donothing declaration statement
-      donothing donothing in
+      donothing expression donothing donothing typeC donothing
+      donothing declaration statement
+      donothing donothing donothing donothing in
 
   let res =
     List.sort compare
@@ -568,6 +696,7 @@ let check_meta rname old_metas inherited_metavars metavars minus plus =
   let err_table = make_table (err@ierr) in
   let other_table = make_table other in
   let iother_table = make_table iother in
+
   add_to_fresh_table fresh;
   rule old_metas [iother_table;other_table;err_table] true minus;
   positions [iother_table;other_table] minus;
