@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./comm_assoc.ml"
 (* searches for E op ..., for any commutative and associative binary
 operator.  When this satisfies the isomorphism conditions (ie all minus, or
 context for the op and ...), then this is converted to Nested(E,op).
@@ -32,32 +35,43 @@ module Ast0 = Ast0_cocci
 module V0 = Visitor_ast0
 module VT0 = Visitor_ast0_types
 
-let comm_assoc =
-  [Ast.Arith(Ast.Plus);Ast.Arith(Ast.Mul);Ast.Arith(Ast.And);Ast.Arith(Ast.Or);
-    Ast.Logical(Ast.AndLog);Ast.Logical(Ast.OrLog)]
+let comm_assoc_arith =
+  [Ast.Plus; Ast.Mul; Ast.And; Ast.Or]
+
+let comm_assoc_logical =
+  [Ast.AndLog; Ast.OrLog]
+
+let is_comm_assoc op0 = match Ast0.unwrap op0 with
+  | Ast0.Arith op -> List.mem (Ast0.unwrap_mcode op) comm_assoc_arith
+  | Ast0.Logical op -> List.mem (Ast0.unwrap_mcode op) comm_assoc_logical
+  | Ast0.MetaBinary _ -> false
 
 let is_minus e =
   match Ast0.get_mcodekind e with Ast0.MINUS(cell) -> true | _ -> false
 
 let is_context e =
-  !Flag.sgrep_mode2 or (* everything is context for sgrep *)
+  !Flag.sgrep_mode2 || (* everything is context for sgrep *)
   (match Ast0.get_mcodekind e with
     Ast0.CONTEXT(cell) -> true
   | _ -> false)
 
-let nopos mc =
-  match Ast0.get_pos mc with Ast0.MetaPos _ -> false | Ast0.NoMetaPos -> true
+let nopos mc = (Ast0.get_pos mc) = []
+
+let nopos_op op0 = match Ast0.unwrap op0 with
+  | Ast0.Arith op -> nopos op
+  | Ast0.Logical op -> nopos op
+  | Ast0.MetaBinary(mv,_,_) -> nopos mv
 
 let process_binops rule_name =
   let expr r k e1 =
     let e = k e1 in
     match Ast0.unwrap e with
       Ast0.Binary(left,op,right)
-      when List.mem (Ast0.unwrap_mcode op) comm_assoc ->
+      when is_comm_assoc op ->
 	(match Ast0.unwrap right with
 	  Ast0.Edots(d,None) ->
 	    if (is_minus e || (is_context e && is_context right))
-		&& nopos op && nopos d
+		&& nopos_op op && nopos d
 	    (* keep dots to record required modif *)
 	    then Ast0.rewrap e (Ast0.Nested(left,op,right))
 	    else
