@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./context_neg.ml"
 (* Detects subtrees that are all minus/plus and nodes that are "binding
 context nodes".  The latter is a node whose structure and immediate tokens
 are the same in the minus and plus trees, and such that for every child,
@@ -47,6 +50,8 @@ let set_mcodekind x mcodekind =
   | Ast0.DotsCaseTag(d) -> Ast0.set_mcodekind d mcodekind
   | Ast0.IdentTag(d) -> Ast0.set_mcodekind d mcodekind
   | Ast0.ExprTag(d) -> Ast0.set_mcodekind d mcodekind
+  | Ast0.AssignOpTag(d) -> Ast0.set_mcodekind d mcodekind
+  | Ast0.BinaryOpTag(d) -> Ast0.set_mcodekind d mcodekind
   | Ast0.ArgExprTag(d) | Ast0.TestExprTag(d) ->
       failwith "not possible - iso only"
   | Ast0.TypeCTag(d) -> Ast0.set_mcodekind d mcodekind
@@ -54,12 +59,16 @@ let set_mcodekind x mcodekind =
   | Ast0.DeclTag(d) -> Ast0.set_mcodekind d mcodekind
   | Ast0.InitTag(d) -> Ast0.set_mcodekind d mcodekind
   | Ast0.StmtTag(d) -> Ast0.set_mcodekind d mcodekind
+  | Ast0.ForInfoTag(d) -> Ast0.set_mcodekind d mcodekind
   | Ast0.CaseLineTag(d) -> Ast0.set_mcodekind d mcodekind
+  | Ast0.StringFragmentTag(d) -> Ast0.set_mcodekind d mcodekind
   | Ast0.TopTag(d) -> Ast0.set_mcodekind d mcodekind
   | Ast0.IsoWhenTag(_) -> failwith "only within iso phase"
   | Ast0.IsoWhenTTag(_) -> failwith "only within iso phase"
   | Ast0.IsoWhenFTag(_) -> failwith "only within iso phase"
-  | Ast0.MetaPosTag(p) -> failwith "metapostag only within iso phase"
+  | Ast0.MetaPosTag(p) -> failwith "invisible at this stage"
+  | Ast0.HiddenVarTag(p) -> failwith "hiddenvar only within iso phase"
+  | Ast0.WhenTag _ -> failwith "whentag only within iso phase"
 
 let set_index x index =
   match x with
@@ -71,6 +80,8 @@ let set_index x index =
   | Ast0.DotsCaseTag(d) -> Ast0.set_index d index
   | Ast0.IdentTag(d) -> Ast0.set_index d index
   | Ast0.ExprTag(d) -> Ast0.set_index d index
+  | Ast0.AssignOpTag(d) -> Ast0.set_index d index
+  | Ast0.BinaryOpTag(d) -> Ast0.set_index d index
   | Ast0.ArgExprTag(d) | Ast0.TestExprTag(d) ->
       failwith "not possible - iso only"
   | Ast0.TypeCTag(d) -> Ast0.set_index d index
@@ -78,12 +89,16 @@ let set_index x index =
   | Ast0.InitTag(d) -> Ast0.set_index d index
   | Ast0.DeclTag(d) -> Ast0.set_index d index
   | Ast0.StmtTag(d) -> Ast0.set_index d index
+  | Ast0.ForInfoTag(d) -> Ast0.set_index d index
   | Ast0.CaseLineTag(d) -> Ast0.set_index d index
+  | Ast0.StringFragmentTag(d) -> Ast0.set_index d index
   | Ast0.TopTag(d) -> Ast0.set_index d index
   | Ast0.IsoWhenTag(_) -> failwith "only within iso phase"
   | Ast0.IsoWhenTTag(_) -> failwith "only within iso phase"
   | Ast0.IsoWhenFTag(_) -> failwith "only within iso phase"
-  | Ast0.MetaPosTag(p) -> failwith "metapostag only within iso phase"
+  | Ast0.MetaPosTag(p) -> failwith "invisible at this stage"
+  | Ast0.HiddenVarTag(p) -> failwith "hiddenvar only within iso phase"
+  | Ast0.WhenTag _ -> failwith "whentag only within iso phase"
 
 let get_index = function
     Ast0.DotsExprTag(d) -> Index.expression_dots d
@@ -94,6 +109,8 @@ let get_index = function
   | Ast0.DotsCaseTag(d) -> Index.case_line_dots d
   | Ast0.IdentTag(d) -> Index.ident d
   | Ast0.ExprTag(d) -> Index.expression d
+  | Ast0.AssignOpTag(d) -> Index.assignOp d
+  | Ast0.BinaryOpTag(d) -> Index.binaryOp d
   | Ast0.ArgExprTag(d) | Ast0.TestExprTag(d) ->
       failwith "not possible - iso only"
   | Ast0.TypeCTag(d) -> Index.typeC d
@@ -101,12 +118,16 @@ let get_index = function
   | Ast0.InitTag(d) -> Index.initialiser d
   | Ast0.DeclTag(d) -> Index.declaration d
   | Ast0.StmtTag(d) -> Index.statement d
+  | Ast0.ForInfoTag(d) -> Index.forinfo d
   | Ast0.CaseLineTag(d) -> Index.case_line d
+  | Ast0.StringFragmentTag(d) -> Index.string_fragment d
   | Ast0.TopTag(d) -> Index.top_level d
   | Ast0.IsoWhenTag(_) -> failwith "only within iso phase"
   | Ast0.IsoWhenTTag(_) -> failwith "only within iso phase"
   | Ast0.IsoWhenFTag(_) -> failwith "only within iso phase"
-  | Ast0.MetaPosTag(p) -> failwith "metapostag only within iso phase"
+  | Ast0.MetaPosTag(p) -> failwith "invisible at this stage"
+  | Ast0.HiddenVarTag(p) -> failwith "hiddenvar only within iso phase"
+  | Ast0.WhenTag _ -> failwith "whentag only within iso phase"
 
 (* --------------------------------------------------------------------- *)
 (* Collect the line numbers of the plus code.  This is used for disjunctions.
@@ -145,12 +166,28 @@ let collect_plus_lines top =
     match mcodekind with
       Ast0.PLUS _ -> insert info.Ast0.pos_info.Ast0.line_start
     | _ -> () in
+  let statement r k s =
+    let mcode info bef = mcode ((),(),info,bef,(),-1) in
+    match Ast0.unwrap s with
+	(* cases for everything with extra mcode *)
+      | Ast0.Decl((info,bef),_) ->
+	  bind (mcode info bef) (k s)
+      |	Ast0.FunDecl((info,bef),_,_,_,_,_,_,_,_,_,(ainfo,aft)) ->
+	  bind (mcode info bef) (bind (k s) (mcode ainfo aft))
+      | Ast0.IfThen(_,_,_,_,_,(info,aft,adj))
+      | Ast0.IfThenElse(_,_,_,_,_,_,_,(info,aft,adj))
+      | Ast0.Iterator(_,_,_,_,_,(info,aft,adj))
+      | Ast0.While(_,_,_,_,_,(info,aft,adj))
+      | Ast0.For(_,_,_,_,_,_,_,_,(info,aft,adj)) ->
+	  bind (k s) (mcode info aft)
+      |	_ -> k s in
   let fn =
     V0.flat_combiner bind option_default
       mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
-      donothing donothing donothing donothing donothing donothing
+      mcode mcode
       donothing donothing donothing donothing donothing donothing donothing
-      donothing donothing in
+      donothing donothing donothing donothing donothing donothing donothing
+      statement donothing donothing donothing donothing in
   fn.VT0.combiner_rec_top_level top
 
 (* --------------------------------------------------------------------- *)
@@ -231,6 +268,9 @@ let bind c1 c2 =
 let option_default = (*Bind(Neutral,[],[],[],[],[])*)
   Recursor(Neutral,[],[],[])
 
+let contains_added_strings info =
+  not (info.Ast0.strings_before = []) || not (info.Ast0.strings_after = [])
+
 let mcode (_,_,info,mcodekind,pos,_) =
   let offset = info.Ast0.pos_info.Ast0.offset in
   match mcodekind with
@@ -261,7 +301,11 @@ let nc_mcode (_,_,info,mcodekind,pos,_) =
 	 associated with is - or context.  In any case, the context offsets are
 	 used for identification, and this invisible node should not be needed
 	 for this purpose. *)
-      Token(Neutral,offset,mcodekind,[])
+      if contains_added_strings info
+      then
+	(* can we have ++ for strings? *)
+	Token(NotAllMarked,offset,mcodekind,[])
+      else Token(Neutral,offset,mcodekind,[])
   | _ -> failwith "not possible"
 
 let is_context = function Ast0.CONTEXT(_) -> true | _ -> false
@@ -354,6 +398,7 @@ let classify is_minus all_marked table code =
 
   (* not clear why we have the next two cases, since DisjDecl and
   DisjType shouldn't have been constructed yet, as they only come from isos *)
+  (* actually, DisjDecl now allowed in source struct decls *)
   let declaration r k e =
     compute_result Ast0.decl e
       (match Ast0.unwrap e with
@@ -423,32 +468,50 @@ let classify is_minus all_marked table code =
       | Ast0.Stars(dots,whencode) ->
 	  k (Ast0.rewrap s (Ast0.Stars(dots,[])))
       | Ast0.Disj(starter,statement_dots_list,_,ender) ->
-	  disj_cases s starter statement_dots_list r.VT0.combiner_rec_statement_dots
+	  disj_cases s starter statement_dots_list
+	    r.VT0.combiner_rec_statement_dots
 	    ender
 	(* cases for everything with extra mcode *)
-      |	Ast0.FunDecl((info,bef),_,_,_,_,_,_,_,_)
       | Ast0.Decl((info,bef),_) ->
 	  bind (nc_mcode ((),(),info,bef,(),-1)) (k s)
-      | Ast0.IfThen(_,_,_,_,_,(info,aft))
-      | Ast0.IfThenElse(_,_,_,_,_,_,_,(info,aft))
-      | Ast0.Iterator(_,_,_,_,_,(info,aft))
-      | Ast0.While(_,_,_,_,_,(info,aft))
-      | Ast0.For(_,_,_,_,_,_,_,_,_,(info,aft)) ->
-	  bind (k s) (nc_mcode ((),(),info,aft,(),-1))
-      |	_ -> k s
+      | Ast0.FunDecl((info,bef),_,_,_,_,_,_,_,_,_,(ainfo,aft)) ->
+	  (* not sure that the use of start is relevant here *)
+	  let a1 = nc_mcode ((),(),info,bef,(),-1) in
+	  let a2 = nc_mcode ((),(),ainfo,aft,(),-1) in
+	  let b = k s in
+	  bind a1 (bind b a2)
+      (* For these, the info of the aft mcode is derived from the else
+	 branch.  These might not correspond for a context if, eg if
+	 only the else branch is replaced.  Thus we take instead the
+	 info of the starting keyword.  In a context case, these will be
+	 the same on the - and + sides.  All that is used as an offset,
+	 and it is only used as a key, so this is safe to do. For an
+         iterator, we take the left parenthesis, which should have the
+	 same property. *)
+      | Ast0.IfThen(start,_,_,_,_,(info,aft,adj))
+      | Ast0.IfThenElse(start,_,_,_,_,_,_,(info,aft,adj))
+      | Ast0.Iterator(_,start,_,_,_,(info,aft,adj))
+      | Ast0.While(start,_,_,_,_,(info,aft,adj))
+      | Ast0.For(start,_,_,_,_,_,_,_,(info,aft,adj)) ->
+	  let mcode_info (_,_,info,_,_,_) = info in
+	  bind (k s) (nc_mcode ((),(),mcode_info start,aft,(),adj))
+      |	_ -> k s) in
 
-) in
+  let string_fragment r k s =
+    compute_result Ast0.string_fragment s (k s) in
 
   let do_top builder r k e = compute_result builder e (k e) in
 
   let combiner =
     V0.flat_combiner bind option_default
-      mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
+      mcode mcode mcode mcode mcode mcode mcode mcode mcode
+      mcode mcode mcode mcode mcode
       (do_nothing Ast0.dotsExpr) (do_nothing Ast0.dotsInit)
       (do_nothing Ast0.dotsParam) (do_nothing Ast0.dotsStmt)
       (do_nothing Ast0.dotsDecl) (do_nothing Ast0.dotsCase)
-      ident expression typeC initialiser param declaration
-      statement case_line (do_top Ast0.top) in
+      ident expression (do_nothing Ast0.assignOp) (do_nothing Ast0.binaryOp) typeC initialiser param declaration
+      statement (do_nothing Ast0.forinfo) case_line string_fragment
+      (do_top Ast0.top) in
   combiner.VT0.combiner_rec_top_level code
 
 (* --------------------------------------------------------------------- *)
@@ -460,6 +523,20 @@ the same context children *)
 let equal_mcode (_,_,info1,_,_,_) (_,_,info2,_,_,_) =
   info1.Ast0.pos_info.Ast0.offset = info2.Ast0.pos_info.Ast0.offset
 
+let assignOp_equal_mcode op1 op2 =
+  match (Ast0.unwrap op1, Ast0.unwrap op2) with
+    Ast0.SimpleAssign op1', Ast0.SimpleAssign op2' -> equal_mcode op1' op2'
+  | Ast0.OpAssign op1', Ast0.OpAssign op2' -> equal_mcode op1' op2'
+  | Ast0.MetaAssign(mv1,_,_), Ast0.MetaAssign(mv2,_,_) -> equal_mcode mv1 mv2
+  | _ -> false
+
+let binaryOp_equal_mcode op1 op2 =
+  match (Ast0.unwrap op1, Ast0.unwrap op2) with
+    Ast0.Arith op1', Ast0.Arith op2' -> equal_mcode op1' op2'
+  | Ast0.Logical op1', Ast0.Logical op2' -> equal_mcode op1' op2'
+  | Ast0.MetaBinary(mv1,_,_), Ast0.MetaBinary(mv2,_,_) -> equal_mcode mv1 mv2
+  | _ -> false
+
 let equal_option e1 e2 =
   match (e1,e2) with
     (Some x, Some y) -> equal_mcode x y
@@ -476,7 +553,7 @@ let dots fn d1 d2 =
 let rec equal_ident i1 i2 =
   match (Ast0.unwrap i1,Ast0.unwrap i2) with
     (Ast0.Id(name1),Ast0.Id(name2)) -> equal_mcode name1 name2
-  | (Ast0.MetaId(name1,_,_),Ast0.MetaId(name2,_,_)) ->
+  | (Ast0.MetaId(name1,_,_,_),Ast0.MetaId(name2,_,_,_)) ->
       equal_mcode name1 name2
   | (Ast0.MetaFunc(name1,_,_),Ast0.MetaFunc(name2,_,_)) ->
       equal_mcode name1 name2
@@ -495,16 +572,20 @@ let rec equal_expression e1 e2 =
   match (Ast0.unwrap e1,Ast0.unwrap e2) with
     (Ast0.Ident(_),Ast0.Ident(_)) -> true
   | (Ast0.Constant(const1),Ast0.Constant(const2)) -> equal_mcode const1 const2
+  | (Ast0.StringConstant(lq1,const1,rq1),Ast0.StringConstant(lq2,const2,rq2))->
+      equal_mcode lq1 lq2 && equal_mcode rq1 rq2
   | (Ast0.FunCall(_,lp1,_,rp1),Ast0.FunCall(_,lp2,_,rp2)) ->
       equal_mcode lp1 lp2 && equal_mcode rp1 rp2
   | (Ast0.Assignment(_,op1,_,_),Ast0.Assignment(_,op2,_,_)) ->
+      assignOp_equal op1 op2
+  | (Ast0.Sequence(_,op1,_),Ast0.Sequence(_,op2,_)) ->
       equal_mcode op1 op2
   | (Ast0.CondExpr(_,why1,_,colon1,_),Ast0.CondExpr(_,why2,_,colon2,_)) ->
       equal_mcode why1 why2 && equal_mcode colon1 colon2
   | (Ast0.Postfix(_,op1),Ast0.Postfix(_,op2)) -> equal_mcode op1 op2
   | (Ast0.Infix(_,op1),Ast0.Infix(_,op2)) -> equal_mcode op1 op2
   | (Ast0.Unary(_,op1),Ast0.Unary(_,op2)) -> equal_mcode op1 op2
-  | (Ast0.Binary(_,op1,_),Ast0.Binary(_,op2,_)) -> equal_mcode op1 op2
+  | (Ast0.Binary(_,op1,_),Ast0.Binary(_,op2,_)) -> binaryOp_equal op1 op2
   | (Ast0.Paren(lp1,_,rp1),Ast0.Paren(lp2,_,rp2)) ->
       equal_mcode lp1 lp2 && equal_mcode rp1 rp2
   | (Ast0.ArrayAccess(_,lb1,_,rb1),Ast0.ArrayAccess(_,lb2,_,rb2)) ->
@@ -520,6 +601,8 @@ let rec equal_expression e1 e2 =
   | (Ast0.SizeOfType(szf1,lp1,_,rp1),Ast0.SizeOfType(szf2,lp2,_,rp2)) ->
       equal_mcode szf1 szf2 && equal_mcode lp1 lp2 && equal_mcode rp1 rp2
   | (Ast0.TypeExp(_),Ast0.TypeExp(_)) -> true
+  | (Ast0.Constructor(lp1,_,rp1,_),Ast0.Constructor(lp2,_,rp2,_)) ->
+      equal_mcode lp1 lp2 && equal_mcode rp1 rp2
   | (Ast0.MetaErr(name1,_,_),Ast0.MetaErr(name2,_,_))
   | (Ast0.MetaExpr(name1,_,_,_,_),Ast0.MetaExpr(name2,_,_,_,_))
   | (Ast0.MetaExprList(name1,_,_),Ast0.MetaExprList(name2,_,_)) ->
@@ -540,6 +623,21 @@ let rec equal_expression e1 e2 =
   | (Ast0.UniqueExp(_),Ast0.UniqueExp(_)) -> true
   | _ -> false
 
+and assignOp_equal op1 op2 =
+  match (Ast0.unwrap op1, Ast0.unwrap op2) with
+    | Ast0.SimpleAssign o1, Ast0.SimpleAssign o2 -> equal_mcode o1 o2
+    | Ast0.OpAssign o1, Ast0.OpAssign o2 -> equal_mcode o1 o2
+    | Ast0.MetaAssign(mv1,_,_), Ast0.MetaAssign(mv2,_,_) ->
+      equal_mcode mv1 mv2
+    | _ -> false
+and binaryOp_equal op1 op2 =
+  match (Ast0.unwrap op1, Ast0.unwrap op2) with
+    | Ast0.Arith o1, Ast0.Arith o2 -> equal_mcode o1 o2
+    | Ast0.Logical o1, Ast0.Logical o2 -> equal_mcode o1 o2
+    | Ast0.MetaBinary(mv1,_,_), Ast0.MetaBinary(mv2,_,_) ->
+      equal_mcode mv1 mv2
+    | _ -> false
+
 let rec equal_typeC t1 t2 =
   match (Ast0.unwrap t1,Ast0.unwrap t2) with
     (Ast0.ConstVol(cv1,_),Ast0.ConstVol(cv2,_)) -> equal_mcode cv1 cv2
@@ -551,14 +649,16 @@ let rec equal_typeC t1 t2 =
       equal_mcode star1 star2
   | (Ast0.Array(_,lb1,_,rb1),Ast0.Array(_,lb2,_,rb2)) ->
       equal_mcode lb1 lb2 && equal_mcode rb1 rb2
+  | (Ast0.Decimal(dec1,lp1,_,comma1,_,rp1),
+     Ast0.Decimal(dec2,lp2,_,comma2,_,rp2)) ->
+       equal_mcode dec1 dec2 && equal_mcode lp1 lp2 &&
+       equal_option comma1 comma2 && equal_mcode rp1 rp2
   | (Ast0.EnumName(kind1,_),Ast0.EnumName(kind2,_)) ->
       equal_mcode kind1 kind2
   | (Ast0.EnumDef(_,lb1,_,rb1),Ast0.EnumDef(_,lb2,_,rb2)) ->
        equal_mcode lb1 lb2 && equal_mcode rb1 rb2
   | (Ast0.StructUnionName(kind1,_),Ast0.StructUnionName(kind2,_)) ->
       equal_mcode kind1 kind2
-  | (Ast0.FunctionType(ty1,lp1,p1,rp1),Ast0.FunctionType(ty2,lp2,p2,rp2)) ->
-      equal_mcode lp1 lp2 && equal_mcode rp1 rp2
   | (Ast0.StructUnionDef(_,lb1,_,rb1),
      Ast0.StructUnionDef(_,lb2,_,rb2)) ->
        equal_mcode lb1 lb2 && equal_mcode rb1 rb2
@@ -574,6 +674,14 @@ let rec equal_typeC t1 t2 =
   | (Ast0.UniqueType(_),Ast0.UniqueType(_)) -> true
   | _ -> false
 
+let equal_fninfo x y =
+  match (x,y) with
+    (Ast0.FStorage(s1),Ast0.FStorage(s2)) -> equal_mcode s1 s2
+  | (Ast0.FType(_),Ast0.FType(_)) -> true
+  | (Ast0.FInline(i1),Ast0.FInline(i2)) -> equal_mcode i1 i2
+  | (Ast0.FAttr(i1),Ast0.FAttr(i2)) -> equal_mcode i1 i2
+  | _ -> false
+
 let equal_declaration d1 d2 =
   match (Ast0.unwrap d1,Ast0.unwrap d2) with
     (Ast0.MetaDecl(name1,_),Ast0.MetaDecl(name2,_))
@@ -584,14 +692,32 @@ let equal_declaration d1 d2 =
       equal_option stg1 stg2 && equal_mcode eq1 eq2 && equal_mcode sem1 sem2
   | (Ast0.UnInit(stg1,_,_,sem1),Ast0.UnInit(stg2,_,_,sem2)) ->
       equal_option stg1 stg2 && equal_mcode sem1 sem2
-  | (Ast0.MacroDecl(nm1,lp1,_,rp1,sem1),Ast0.MacroDecl(nm2,lp2,_,rp2,sem2)) ->
+  | (Ast0.FunProto(fninfo1,name1,lp1,p1,va1,rp1,sem1),
+     Ast0.FunProto(fninfo2,name2,lp2,p2,va2,rp2,sem2)) ->
+       let equal_varargs va1 va2 = match (va1,va2) with
+         | None, None -> true
+         | Some (c1, e1), Some (c2, e2) ->
+           equal_mcode c1 c2 && equal_mcode e1 e2
+         | _ -> false in
+       (List.length fninfo1) = (List.length fninfo2) &&
+       List.for_all2 equal_fninfo fninfo1 fninfo2 &&
+       equal_mcode lp1 lp2 && equal_varargs va1 va2 &&
+       equal_mcode rp1 rp2 && equal_mcode sem1 sem2
+  | (Ast0.MacroDecl(nm1,lp1,_,rp1,sem1),Ast0.MacroDecl(nm2,lp2,_,rp2,sem2))->
       equal_mcode lp1 lp2 && equal_mcode rp1 rp2 && equal_mcode sem1 sem2
+  | (Ast0.MacroDeclInit(nm1,lp1,_,rp1,eq1,_,sem1),
+     Ast0.MacroDeclInit(nm2,lp2,_,rp2,eq2,_,sem2))->
+      equal_mcode lp1 lp2 && equal_mcode rp1 rp2 && equal_mcode eq1 eq2
+	 && equal_mcode sem1 sem2
   | (Ast0.TyDecl(_,sem1),Ast0.TyDecl(_,sem2)) -> equal_mcode sem1 sem2
   | (Ast0.Ddots(dots1,_),Ast0.Ddots(dots2,_)) -> equal_mcode dots1 dots2
   | (Ast0.OptDecl(_),Ast0.OptDecl(_)) -> true
   | (Ast0.UniqueDecl(_),Ast0.UniqueDecl(_)) -> true
-  | (Ast0.DisjDecl _,_) | (_,Ast0.DisjDecl _) ->
-      failwith "DisjDecl not expected here"
+  | (Ast0.DisjDecl(starter1,_,mids1,ender1),
+     Ast0.DisjDecl(starter2,_,mids2,ender2)) ->
+       equal_mcode starter1 starter2 &&
+       List.for_all2 equal_mcode mids1 mids2 &&
+       equal_mcode ender1 ender2
   | _ -> false
 
 let equal_designator d1 d2 =
@@ -610,6 +736,8 @@ let equal_initialiser i1 i2 =
   match (Ast0.unwrap i1,Ast0.unwrap i2) with
     (Ast0.MetaInit(name1,_),Ast0.MetaInit(name2,_)) ->
       equal_mcode name1 name2
+  | (Ast0.MetaInitList(name1,_,_),Ast0.MetaInitList(name2,_,_)) ->
+      equal_mcode name1 name2
   | (Ast0.InitExpr(_),Ast0.InitExpr(_)) -> true
   | (Ast0.InitList(lb1,_,rb1,o1),Ast0.InitList(lb2,_,rb2,o2)) ->
       (* can't compare orderedness, because this can differ between -
@@ -643,8 +771,8 @@ let equal_parameterTypeDef p1 p2 =
 
 let rec equal_statement s1 s2 =
   match (Ast0.unwrap s1,Ast0.unwrap s2) with
-    (Ast0.FunDecl(_,fninfo1,_,lp1,_,rp1,lbrace1,_,rbrace1),
-     Ast0.FunDecl(_,fninfo2,_,lp2,_,rp2,lbrace2,_,rbrace2)) ->
+    (Ast0.FunDecl(_,fninfo1,_,lp1,_,_,rp1,lbrace1,_,rbrace1,_),
+     Ast0.FunDecl(_,fninfo2,_,lp2,_,_,rp2,lbrace2,_,rbrace2,_)) ->
        (List.length fninfo1) = (List.length fninfo2) &&
        List.for_all2 equal_fninfo fninfo1 fninfo2 &&
        equal_mcode lp1 lp2 && equal_mcode rp1 rp2 &&
@@ -665,10 +793,16 @@ let rec equal_statement s1 s2 =
   | (Ast0.Do(d1,_,whl1,lp1,_,rp1,sem1),Ast0.Do(d2,_,whl2,lp2,_,rp2,sem2)) ->
       equal_mcode whl1 whl2 && equal_mcode d1 d2 &&
       equal_mcode lp1 lp2 && equal_mcode rp1 rp2 && equal_mcode sem1 sem2
-  | (Ast0.For(fr1,lp1,_,sem11,_,sem21,_,rp1,_,_),
-     Ast0.For(fr2,lp2,_,sem12,_,sem22,_,rp2,_,_)) ->
+  | (Ast0.For(fr1,lp1,first1,_,sem21,_,rp1,_,_),
+     Ast0.For(fr2,lp2,first2,_,sem22,_,rp2,_,_)) ->
+       let first =
+	 match (Ast0.unwrap first1,Ast0.unwrap first2) with
+	   (Ast0.ForExp(_,sem1),Ast0.ForExp(_,sem2)) -> 
+	     equal_mcode sem1 sem2
+	 | (Ast0.ForDecl _,Ast0.ForDecl _) -> true
+	 | _ -> false in
        equal_mcode fr1 fr2 && equal_mcode lp1 lp2 &&
-       equal_mcode sem11 sem12 && equal_mcode sem21 sem22 &&
+       first && equal_mcode sem21 sem22 &&
        equal_mcode rp1 rp2
   | (Ast0.Iterator(nm1,lp1,_,rp1,_,_),Ast0.Iterator(nm2,lp2,_,rp2,_,_)) ->
       equal_mcode lp1 lp2 && equal_mcode rp1 rp2
@@ -689,6 +823,9 @@ let rec equal_statement s1 s2 =
       equal_mcode ret1 ret2 && equal_mcode sem1 sem2
   | (Ast0.ReturnExpr(ret1,_,sem1),Ast0.ReturnExpr(ret2,_,sem2)) ->
       equal_mcode ret1 ret2 && equal_mcode sem1 sem2
+  | (Ast0.Exec(exec1,lang1,_,sem1),Ast0.Exec(exec2,lang2,_,sem2)) ->
+      equal_mcode exec1 exec2 && equal_mcode lang1 lang2 &&
+      equal_mcode sem1 sem2
   | (Ast0.MetaStmt(name1,_),Ast0.MetaStmt(name2,_))
   | (Ast0.MetaStmtList(name1,_),Ast0.MetaStmtList(name2,_)) ->
       equal_mcode name1 name2
@@ -711,18 +848,12 @@ let rec equal_statement s1 s2 =
       equal_mcode def1 def2
   | (Ast0.Define(def1,_,_,_),Ast0.Define(def2,_,_,_)) ->
       equal_mcode def1 def2
+  | (Ast0.Pragma(prg1,_,_),Ast0.Pragma(prg2,_,_)) ->
+      equal_mcode prg1 prg2
   | (Ast0.OptStm(_),Ast0.OptStm(_)) -> true
   | (Ast0.UniqueStm(_),Ast0.UniqueStm(_)) -> true
   | _ -> false
 
-and equal_fninfo x y =
-  match (x,y) with
-    (Ast0.FStorage(s1),Ast0.FStorage(s2)) -> equal_mcode s1 s2
-  | (Ast0.FType(_),Ast0.FType(_)) -> true
-  | (Ast0.FInline(i1),Ast0.FInline(i2)) -> equal_mcode i1 i2
-  | (Ast0.FAttr(i1),Ast0.FAttr(i2)) -> equal_mcode i1 i2
-  | _ -> false
-
 let equal_case_line c1 c2 =
   match (Ast0.unwrap c1,Ast0.unwrap c2) with
     (Ast0.Default(def1,colon1,_),Ast0.Default(def2,colon2,_)) ->
@@ -739,7 +870,7 @@ let equal_case_line c1 c2 =
 
 let rec equal_top_level t1 t2 =
   match (Ast0.unwrap t1,Ast0.unwrap t2) with
-    (Ast0.DECL(_),Ast0.DECL(_)) -> true
+    (Ast0.NONDECL(_),Ast0.NONDECL(_)) -> true
   | (Ast0.FILEINFO(old_file1,new_file1),Ast0.FILEINFO(old_file2,new_file2)) ->
       equal_mcode old_file1 old_file2 && equal_mcode new_file1 new_file2
   | (Ast0.CODE(_),Ast0.CODE(_)) -> true
@@ -801,9 +932,10 @@ let contextify_all =
 
   V0.flat_combiner bind option_default
     mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
-    do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
-    do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
-    do_nothing do_nothing do_nothing
+    mcode mcode
+    do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
+    do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
+    do_nothing do_nothing do_nothing do_nothing do_nothing
 
 let contextify_whencode =
   let bind x y = () in
@@ -812,25 +944,27 @@ let contextify_whencode =
   let expression r k e =
     k e;
     match Ast0.unwrap e with
-      Ast0.NestExpr(_,_,_,Some whencode,_)
-    | Ast0.Edots(_,Some whencode)
-    | Ast0.Ecircles(_,Some whencode)
-    | Ast0.Estars(_,Some whencode) ->
+      Ast0.NestExpr(_,_,_,Some (_,_,whencode),_)
+    | Ast0.Edots(_,Some (_,_,whencode))
+    | Ast0.Ecircles(_,Some (_,_,whencode))
+    | Ast0.Estars(_,Some (_,_,whencode)) ->
 	contextify_all.VT0.combiner_rec_expression whencode
     | _ -> () in
 
   let initialiser r k i =
     match Ast0.unwrap i with
-      Ast0.Idots(dots,Some whencode) ->
+      Ast0.Idots(dots,Some (_,_,whencode)) ->
 	contextify_all.VT0.combiner_rec_initialiser whencode
     | _ -> k i in
 
   let whencode = function
-      Ast0.WhenNot sd -> contextify_all.VT0.combiner_rec_statement_dots sd
-    | Ast0.WhenAlways s -> contextify_all.VT0.combiner_rec_statement s
-    | Ast0.WhenModifier(_) -> ()
-    | Ast0.WhenNotTrue(e) -> contextify_all.VT0.combiner_rec_expression e
-    | Ast0.WhenNotFalse(e) -> contextify_all.VT0.combiner_rec_expression e in
+      Ast0.WhenNot (_,_,sd) ->
+	contextify_all.VT0.combiner_rec_statement_dots sd
+    | Ast0.WhenAlways (_,_,s) -> contextify_all.VT0.combiner_rec_statement s
+    | Ast0.WhenModifier _ -> ()
+    | Ast0.WhenNotTrue(_,_,e) -> contextify_all.VT0.combiner_rec_expression e
+    | Ast0.WhenNotFalse(_,_,e) -> contextify_all.VT0.combiner_rec_expression e
+  in
 
   let statement r k (s : Ast0.statement) =
     k s;
@@ -851,7 +985,7 @@ let contextify_whencode =
 (* --------------------------------------------------------------------- *)
 
 (* the first int list is the tokens in the node, the second is the tokens
-in the descendents *)
+in the descendants *)
 let minus_table =
   (Hashtbl.create(50) : (int list, Ast0.anything * int list list) Hashtbl.t)
 let plus_table =
@@ -859,10 +993,11 @@ let plus_table =
 
 let iscode t =
   match Ast0.unwrap t with
-    Ast0.DECL(_) -> true
+    Ast0.NONDECL(_) -> true
   | Ast0.FILEINFO(_) -> true
   | Ast0.ERRORWORDS(_) -> false
   | Ast0.CODE(_) -> true
+  | Ast0.TOPCODE(_)
   | Ast0.OTHER(_) -> failwith "unexpected top level code"
 
 (* ------------------------------------------------------------------- *)
@@ -876,7 +1011,7 @@ let concat = function
 	  [] -> []
 	| x::rest ->
 	    (match Ast0.unwrap x with
-	      Ast0.DECL(s) -> let stms = loop rest in s::stms
+	      Ast0.NONDECL(s) -> let stms = loop rest in s::stms
 	    | Ast0.CODE(ss) ->
 		let stms = loop rest in
 		(match Ast0.unwrap ss with
@@ -942,30 +1077,33 @@ let rec is_init s =
 let rec is_decl s =
   match Ast0.unwrap s with
     Ast0.Decl(_,e) -> true
-  | Ast0.FunDecl(_,_,_,_,_,_,_,_,_) -> true
+  | Ast0.FunDecl(_,_,_,_,_,_,_,_,_,_,_) -> true
   | Ast0.Disj(_,stmts,_,_) -> isall is_decl stmts
   | _ -> false
 
 let rec is_fndecl s =
   match Ast0.unwrap s with
-    Ast0.FunDecl(_,_,_,_,_,_,_,_,_) -> true
+    Ast0.FunDecl(_,_,_,_,_,_,_,_,_,_,_) -> true
   | Ast0.Disj(_,stmts,_,_) -> isall is_fndecl stmts
   | _ -> false
 
 let rec is_toplevel s =
   match Ast0.unwrap s with
     Ast0.Decl(_,e) -> true
-  | Ast0.FunDecl(_,_,_,_,_,_,_,_,_) -> true
+  | Ast0.FunDecl(_,_,_,_,_,_,_,_,_,_,_) -> true
   | Ast0.Disj(_,stmts,_,_) -> isall is_toplevel stmts
-  | Ast0.ExprStatement(fc,_) ->
+  | Ast0.ExprStatement(Some fc,_) ->
       (match Ast0.unwrap fc with
 	Ast0.FunCall(_,_,_,_) -> true
       |	_ -> false)
   | Ast0.Include(_,_) -> true
   | Ast0.Undef(_,_) -> true
+  | Ast0.Pragma(_,_,_) -> true
   | Ast0.Define(_,_,_,_) -> true
   | _ -> false
 
+(* consider code and topcode to be the same; difference handled
+in top_level.ml *)
 let check_compatible m p =
   let fail _ =
     failwith
@@ -973,17 +1111,21 @@ let check_compatible m p =
 	 "incompatible minus and plus code starting on lines %d and %d"
 	 (Ast0.get_line m) (Ast0.get_line p)) in
   match (Ast0.unwrap m, Ast0.unwrap p) with
-    (Ast0.DECL(decl1),Ast0.DECL(decl2)) ->
+    (Ast0.NONDECL(decl1),Ast0.NONDECL(decl2)) ->
       if not (is_decl decl1 && is_decl decl2)
       then fail()
-  | (Ast0.DECL(decl1),Ast0.CODE(code2)) ->
+  | (Ast0.NONDECL(decl1),Ast0.CODE(code2)) ->
+      (* This is probably the only important case.  We don't want to
+	 replace top-level declarations by arbitrary code. *)
       let v1 = is_decl decl1 in
       let v2 = List.for_all is_toplevel (Ast0.undots code2) in
-      if !Flag.make_hrule = None && v1 && not v2 then fail()
-  | (Ast0.CODE(code1),Ast0.DECL(decl2)) ->
+      if !Flag.make_hrule = None && v1 && not v2
+      then fail()
+  | (Ast0.CODE(code1),Ast0.NONDECL(decl2)) ->
       let v1 = List.for_all is_toplevel (Ast0.undots code1) in
       let v2 = is_decl decl2 in
-      if v1 && not v2 then fail()
+      if v1 && not v2
+      then fail()
   | (Ast0.CODE(code1),Ast0.CODE(code2)) ->
       let v1 = isonly is_init code1 in
       let v2a = isonly is_init code2 in
@@ -996,16 +1138,34 @@ let check_compatible m p =
 	  (function tester ->
 	    let v1 = isonly tester code1 in
 	    let v2 = isonly tester code2 in
-	    if (v1 && not v2) or (!Flag.make_hrule = None && v2 && not v1)
+	    if (v1 && not v2) || (!Flag.make_hrule = None && v2 && not v1)
 	    then fail())
 	  testers;
 	let v1 = isonly is_fndecl code1 in
 	let v2 = List.for_all is_toplevel (Ast0.undots code2) in
-	if !Flag.make_hrule = None && v1 && not v2 then fail()
+	if !Flag.make_hrule = None && v1 && not v2
+	then fail()
   | (Ast0.FILEINFO(_,_),Ast0.FILEINFO(_,_)) -> ()
   | (Ast0.OTHER(_),Ast0.OTHER(_)) -> ()
   | _ -> fail()
 
+(* can't just remove expressions or types, not sure if all cases are needed. *)
+let check_complete m =
+   match Ast0.unwrap m with
+     Ast0.NONDECL(code) ->
+       if is_exp code || is_ty code
+       then
+	 failwith
+	   (Printf.sprintf "invalid minus starting on line %d"
+	      (Ast0.get_line m))
+   | Ast0.CODE(code) ->
+       if isonly is_exp code || isonly is_ty code
+       then
+	 failwith
+           (Printf.sprintf "invalid minus starting on line %d"
+              (Ast0.get_line m))
+   | _ -> ()
+
 (* ------------------------------------------------------------------- *)
 
 (* returns a list of corresponding minus and plus trees *)
@@ -1019,12 +1179,14 @@ let context_neg minus plus =
     | ([],l) ->
 	failwith (Printf.sprintf "%d plus things remaining" (List.length l))
     | (minus,[]) ->
+	List.iter check_complete minus;
 	plus_lines := [];
 	let _ =
 	  List.map
 	    (function m ->
 	      classify true
-		(function _ -> Ast0.MINUS(ref([],Ast0.default_token_info)))
+		(function _ ->
+		  Ast0.MINUS(ref(Ast.NOREPLACEMENT,Ast0.default_token_info)))
 		minus_table m)
 	    minus in
 	[]
@@ -1035,9 +1197,9 @@ let context_neg minus plus =
 	let mend = minfo.Ast0.pos_info.Ast0.logical_end in
 	let pstart = pinfo.Ast0.pos_info.Ast0.logical_start in
 	let pend = pinfo.Ast0.pos_info.Ast0.logical_end in
-	if (iscode m or iscode p) &&
-	  (mend + 1 = pstart or pend + 1 = mstart or (* adjacent *)
-	   (mstart <= pstart && mend >= pstart) or
+	if (iscode m || iscode p) &&
+	  (mend + 1 = pstart || pend + 1 = mstart || (* adjacent *)
+	   (mstart <= pstart && mend >= pstart) ||
 	   (pstart <= mstart && pend >= mstart)) (* overlapping or nested *)
 	then
 	  begin
@@ -1050,14 +1212,15 @@ let context_neg minus plus =
 	    collect_plus_lines p;
 	    let _ =
 	      classify true
-		(function _ -> Ast0.MINUS(ref([],Ast0.default_token_info)))
+		(function _ ->
+		  Ast0.MINUS(ref(Ast.NOREPLACEMENT,Ast0.default_token_info)))
 		minus_table m in
 	    let _ = classify false (function c -> Ast0.PLUS c) plus_table p in
 	    traverse minus_table plus_table;
 	    (m,p)::loop(minus,plus)
 	  end
 	else
-	  if not(iscode m or iscode p)
+	  if not(iscode m || iscode p)
 	  then loop(minus,plus)
 	  else
 	    if mstart < pstart
@@ -1066,7 +1229,9 @@ let context_neg minus plus =
 		plus_lines := [];
 		let _ =
 		  classify true
-		    (function _ -> Ast0.MINUS(ref([],Ast0.default_token_info)))
+		    (function _ ->
+		      Ast0.MINUS(ref(Ast.NOREPLACEMENT,
+				     Ast0.default_token_info)))
 		    minus_table m in
 		loop(minus,pall)
 	      end
