@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./data.ml"
 module Ast0 = Ast0_cocci
 module Ast = Ast_cocci
 
@@ -35,10 +38,11 @@ type incl_iso =
   | Virt of string list (* virtual rules *)
 
 type clt =
-    line_type * int * int * int * int (* starting spaces *) *
+    line_type * int * int * int * int * int (* starting spaces *) *
       (Ast_cocci.added_string * Ast0.position_info) list (* code before *) *
       (Ast_cocci.added_string * Ast0.position_info) list (* code after *) *
-      Ast0.meta_pos (* position variable, minus only *)
+      Ast0.anything list (* position variable, minus only *) *
+      string (* whitespace before *)
 
 (* ---------------------------------------------------------------------- *)
 
@@ -73,94 +77,109 @@ let call_in_meta f =
 let all_metadecls =
   (Hashtbl.create(100) : (string, Ast.metavar list) Hashtbl.t)
 
-let clear_meta: (unit -> unit) ref =
-  ref (fun _ -> failwith "uninitialized add_meta")
+let uninitialized_add_meta = fun _ -> failwith "uninitialized add_meta"
+
+let clear_meta: (unit -> unit) ref = 
+  ref uninitialized_add_meta
 
 let add_meta_meta: (Ast.meta_name -> Ast0.pure -> unit) ref =
-  ref (fun _ -> failwith "uninitialized add_meta")
+  ref uninitialized_add_meta
 
 let add_id_meta:
     (Ast.meta_name -> iconstraints -> Ast0.pure -> unit) ref =
-  ref (fun _ -> failwith "uninitialized add_meta")
+  ref uninitialized_add_meta
 
 let add_virt_id_meta_found: (string -> string -> unit) ref =
-  ref (fun _ -> failwith "uninitialized add_meta")
+  ref uninitialized_add_meta
 
 let add_virt_id_meta_not_found:
     (Ast_cocci.meta_name -> Ast0_cocci.pure -> unit) ref =
-  ref (fun _ -> failwith "uninitialized add_meta")
+  ref uninitialized_add_meta
 
-let add_fresh_id_meta: (Ast.meta_name -> unit) ref =
-  ref (fun _ -> failwith "uninitialized add_meta")
+let add_fresh_id_meta: (Ast.meta_name -> Ast.seed -> unit) ref =
+  ref uninitialized_add_meta
 
 let add_type_meta: (Ast.meta_name -> Ast0.pure -> unit) ref =
-  ref (fun _ -> failwith "uninitialized add_meta")
+  ref uninitialized_add_meta
 
 let add_init_meta: (Ast.meta_name -> Ast0.pure -> unit) ref =
-  ref (fun _ -> failwith "uninitialized add_meta")
+  ref uninitialized_add_meta
+
+let add_initlist_meta:
+    (Ast.meta_name -> Ast.list_len -> Ast0.pure -> unit) ref =
+  ref uninitialized_add_meta
 
 let add_param_meta: (Ast.meta_name -> Ast0.pure -> unit) ref =
-  ref (fun _ -> failwith "uninitialized add_meta")
+  ref uninitialized_add_meta
 
 let add_paramlist_meta:
     (Ast.meta_name -> Ast.list_len -> Ast0.pure -> unit) ref =
-  ref (fun _ -> failwith "uninitialized add_meta")
+  ref uninitialized_add_meta
 
 let add_const_meta:
     (Type_cocci.typeC list option -> Ast.meta_name -> econstraints ->
       Ast0.pure -> unit)
     ref =
-  ref (fun _ -> failwith "uninitialized add_meta")
+  ref uninitialized_add_meta
 
 let add_err_meta:
     (Ast.meta_name -> econstraints -> Ast0.pure -> unit) ref =
-  ref (fun _ -> failwith "uninitialized add_meta")
+  ref uninitialized_add_meta
 
 let add_exp_meta:
     (Type_cocci.typeC list option -> Ast.meta_name -> econstraints ->
       Ast0.pure -> unit)
     ref =
-  ref (fun _ -> failwith "uninitialized add_meta")
+  ref uninitialized_add_meta
 
 let add_idexp_meta:
     (Type_cocci.typeC list option -> Ast.meta_name -> econstraints ->
       Ast0.pure -> unit)
     ref =
-  ref (fun _ -> failwith "uninitialized add_meta")
+  ref uninitialized_add_meta
 
 let add_local_idexp_meta:
     (Type_cocci.typeC list option -> Ast.meta_name -> econstraints ->
       Ast0.pure -> unit)
     ref =
+  ref uninitialized_add_meta
+
+let add_global_idexp_meta:
+    (Type_cocci.typeC list option -> Ast.meta_name -> econstraints ->
+      Ast0.pure -> unit)
+    ref =
   ref (fun _ -> failwith "uninitialized add_meta")
 
 let add_explist_meta:
     (Ast.meta_name -> Ast.list_len -> Ast0.pure -> unit) ref =
-  ref (fun _ -> failwith "uninitialized add_meta")
+  ref uninitialized_add_meta
 
 let add_decl_meta: (Ast.meta_name -> Ast0.pure -> unit) ref =
-  ref (fun _ -> failwith "uninitialized add_meta")
+  ref uninitialized_add_meta
 
 let add_field_meta: (Ast.meta_name -> Ast0.pure -> unit) ref =
-  ref (fun _ -> failwith "uninitialized add_meta")
+  ref uninitialized_add_meta
 
 let add_field_list_meta:
     (Ast.meta_name -> Ast.list_len -> Ast0.pure -> unit) ref =
-  ref (fun _ -> failwith "uninitialized add_meta")
+  ref uninitialized_add_meta
+
+let add_symbol_meta: (string -> unit) ref =
+  ref uninitialized_add_meta
 
 let add_stm_meta: (Ast.meta_name -> Ast0.pure -> unit) ref =
-  ref (fun _ -> failwith "uninitialized add_meta")
+  ref uninitialized_add_meta
 
 let add_stmlist_meta: (Ast.meta_name -> Ast0.pure -> unit) ref =
-  ref (fun _ -> failwith "uninitialized add_meta")
+  ref uninitialized_add_meta
 
 let add_func_meta:
     (Ast.meta_name -> iconstraints -> Ast0.pure -> unit) ref =
-  ref (fun _ -> failwith "uninitialized add_meta")
+  ref uninitialized_add_meta
 
 let add_local_func_meta:
     (Ast.meta_name -> iconstraints -> Ast0.pure -> unit) ref =
-  ref (fun _ -> failwith "uninitialized add_meta")
+  ref uninitialized_add_meta
 
 let add_declarer_meta:
     (Ast.meta_name -> iconstraints -> Ast0.pure -> unit) ref =
@@ -172,19 +191,50 @@ let add_iterator_meta:
 
 let add_pos_meta:
     (Ast.meta_name -> pconstraints -> Ast.meta_collect -> unit) ref =
-  ref (fun _ -> failwith "uninitialized add_meta")
+  ref uninitialized_add_meta
+
+let add_fmt_meta: (Ast.meta_name -> iconstraints -> unit) ref =
+  ref uninitialized_add_meta
+
+let add_fmtlist_meta:
+    (Ast.meta_name -> Ast.list_len -> unit) ref =
+  ref uninitialized_add_meta
+
+let add_assignOp_meta:
+    (Ast.meta_name ->
+      Ast0.assignOpconstraint -> Ast0.pure -> unit) ref =
+  ref uninitialized_add_meta
+
+let add_binaryOp_meta:
+    (Ast.meta_name ->
+      Ast0.binaryOpconstraint -> Ast0.pure -> unit) ref =
+  ref uninitialized_add_meta
 
 let add_type_name: (string -> unit) ref =
   ref (fun _ -> failwith "uninitialized add_type")
 
+let add_attribute: (string -> unit) ref =
+  ref (fun _ -> failwith "uninitialized add_attribute")
+
 let add_declarer_name: (string -> unit) ref =
   ref (fun _ -> failwith "uninitialized add_decl")
 
 let add_iterator_name: (string -> unit) ref =
   ref (fun _ -> failwith "uninitialized add_iter")
 
+let uninitialized_install_bindings =
+  fun _ -> failwith "uninitialized install_bindings"
+
 let init_rule: (unit -> unit) ref =
-  ref (fun _ -> failwith "uninitialized install_bindings")
+  ref uninitialized_install_bindings
 
 let install_bindings: (string -> unit) ref =
-  ref (fun _ -> failwith "uninitialized install_bindings")
+  ref uninitialized_install_bindings
+
+(* ---------------------------------------------------------------------- *)
+(* String format things *)
+
+let format_metavariables      =
+  ref ([] : (string * (Ast.meta_name * iconstraints)) list)
+let format_list_metavariables =
+  ref ([] : (string * (Ast.meta_name * Ast.list_len)) list)
