@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./data.mli"
 (* types that clutter the .mly file *)
 (* for iso metavariables, true if they can only match nonmodified, unitary
    metavariables *)
@@ -32,10 +35,11 @@ type incl_iso =
   | Virt of string list (* virtual rules *)
 
 type clt =
-    line_type * int * int * int * int (* starting spaces *) *
-      (Ast_cocci.added_string * Ast0_cocci.position_info) list (*code before*) *
-      (Ast_cocci.added_string * Ast0_cocci.position_info) list (*code after *) *
-      Ast0_cocci.meta_pos (* position variable, minus only *)
+    line_type * int * int * int * int * int (* starting spaces *) *
+     (Ast_cocci.added_string * Ast0_cocci.position_info) list (*code before*) *
+     (Ast_cocci.added_string * Ast0_cocci.position_info) list (*code after *) *
+     Ast0_cocci.anything list (* position variable, minus only *) *
+     string (* whitespace before *)
 
 (* ---------------------------------------------------------------------- *)
 
@@ -73,12 +77,16 @@ val add_virt_id_meta_found: (string -> string -> unit) ref
 val add_virt_id_meta_not_found:
     (Ast_cocci.meta_name -> Ast0_cocci.pure -> unit) ref
 
-val add_fresh_id_meta: (Ast_cocci.meta_name -> unit) ref
+val add_fresh_id_meta: (Ast_cocci.meta_name -> Ast_cocci.seed -> unit) ref
 
 val add_type_meta: (Ast_cocci.meta_name -> Ast0_cocci.pure -> unit) ref
 
 val add_init_meta: (Ast_cocci.meta_name -> Ast0_cocci.pure -> unit) ref
 
+val add_initlist_meta:
+    (Ast_cocci.meta_name -> Ast_cocci.list_len -> Ast0_cocci.pure ->
+      unit) ref
+
 val add_param_meta: (Ast_cocci.meta_name -> Ast0_cocci.pure -> unit) ref
 
 val add_paramlist_meta:
@@ -104,6 +112,10 @@ val add_local_idexp_meta:
     (Type_cocci.typeC list option -> Ast_cocci.meta_name ->
       econstraints -> Ast0_cocci.pure -> unit) ref
 
+val add_global_idexp_meta:
+    (Type_cocci.typeC list option -> Ast_cocci.meta_name ->
+      econstraints -> Ast0_cocci.pure -> unit) ref
+
 val add_explist_meta:
     (Ast_cocci.meta_name -> Ast_cocci.list_len -> Ast0_cocci.pure ->
       unit) ref
@@ -112,6 +124,8 @@ val add_decl_meta: (Ast_cocci.meta_name -> Ast0_cocci.pure -> unit) ref
 
 val add_field_meta: (Ast_cocci.meta_name -> Ast0_cocci.pure -> unit) ref
 
+val add_symbol_meta: (string -> unit) ref
+
 val add_field_list_meta:
     (Ast_cocci.meta_name -> Ast_cocci.list_len -> Ast0_cocci.pure -> unit) ref
 
@@ -134,8 +148,22 @@ val add_iterator_meta:
 val add_pos_meta:
     (Ast_cocci.meta_name -> pconstraints -> Ast_cocci.meta_collect -> unit) ref
 
+val add_fmt_meta: (Ast_cocci.meta_name -> iconstraints -> unit) ref
+
+val add_fmtlist_meta: (Ast_cocci.meta_name -> Ast_cocci.list_len -> unit) ref
+
+val add_assignOp_meta: 
+    (Ast_cocci.meta_name ->
+      Ast0_cocci.assignOpconstraint -> Ast0_cocci.pure -> unit) ref
+
+val add_binaryOp_meta: 
+    (Ast_cocci.meta_name ->
+      Ast0_cocci.binaryOpconstraint -> Ast0_cocci.pure -> unit) ref
+
 val add_type_name: (string -> unit) ref
 
+val add_attribute: (string -> unit) ref
+
 val add_declarer_name: (string -> unit) ref
 
 val add_iterator_name: (string -> unit) ref
@@ -143,3 +171,11 @@ val add_iterator_name: (string -> unit) ref
 val init_rule: (unit -> unit) ref
 
 val install_bindings: (string -> unit) ref
+
+(* ---------------------------------------------------------------------- *)
+(* String format things *)
+
+val format_metavariables :
+    (string * (Ast_cocci.meta_name * iconstraints)) list ref
+val format_list_metavariables :
+    (string * (Ast_cocci.meta_name * Ast_cocci.list_len)) list ref
