@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./disjdistr.ml"
 module Ast = Ast_cocci
 module V = Visitor_ast
 
@@ -46,6 +49,16 @@ let rec disjmult f = function
       let rest = disjmult f xs in
       disjmult2 cur rest (function cur -> function rest -> cur :: rest)
 
+let rec disjmult_two fstart frest (start,rest) =
+  let cur = fstart start in
+  let rest = disjmult frest rest in
+  disjmult2 cur rest (function cur -> function rest -> (cur,rest))
+
+let rec disjtwoelems fstart frest (start,rest) =
+  let cur = fstart start in
+  let rest = frest rest in
+  disjmult2 cur rest (function cur -> function rest -> (cur,rest))
+
 let disjoption f = function
     None -> [None]
   | Some x -> List.map (function x -> Some x) (f x)
@@ -61,9 +74,12 @@ let disjdots f d =
 
 let rec disjty ft =
   match Ast.unwrap ft with
-    Ast.Type(cv,ty) ->
+    Ast.Type(allminus,cv,ty) ->
       let ty = disjtypeC ty in
-      List.map (function ty -> Ast.rewrap ft (Ast.Type(cv,ty))) ty
+      List.map (function ty -> Ast.rewrap ft (Ast.Type(allminus,cv,ty))) ty
+  | Ast.AsType(ty,asty) -> (* as ty doesn't contain disj *)
+      let ty = disjty ty in
+      List.map (function ty -> Ast.rewrap ft (Ast.AsType(ty,asty))) ty
   | Ast.DisjType(types) -> List.concat (List.map disjty types)
   | Ast.OptType(ty) ->
       let ty = disjty ty in
@@ -84,27 +100,40 @@ and disjtypeC bty =
 	(function ty ->
 	  Ast.rewrap bty (Ast.FunctionPointer(ty,lp1,star,rp1,lp2,params,rp2)))
 	ty
-  | Ast.FunctionType (s,ty,lp1,params,rp1) ->
-      let ty = disjoption disjty ty in
-      List.map
-	(function ty ->
-	  Ast.rewrap bty (Ast.FunctionType (s,ty,lp1,params,rp1)))
-	ty
   | Ast.Array(ty,lb,size,rb) ->
       disjmult2 (disjty ty) (disjoption disjexp size)
 	(function ty -> function size ->
 	  Ast.rewrap bty (Ast.Array(ty,lb,size,rb)))
-  | Ast.EnumName(_,_) | Ast.StructUnionName(_,_) -> [bty]
+  | Ast.Decimal(dec,lp,length,comma,precision_opt,rp) ->
+      disjmult2 (disjexp length) (disjoption disjexp precision_opt)
+	(function length -> function precision_opt ->
+	  Ast.rewrap bty (Ast.Decimal(dec,lp,length,comma,precision_opt,rp)))
+  | Ast.EnumName(enum,name) ->
+      let name = disjoption disjident name in
+      List.map (function name -> Ast.rewrap bty (Ast.EnumName(enum,name))) name
+  | Ast.StructUnionName(su,name) ->
+      let name = disjoption disjident name in
+      List.map
+	(function name -> Ast.rewrap bty (Ast.StructUnionName(su,name)))
+	name
   | Ast.EnumDef(ty,lb,ids,rb) ->
       disjmult2 (disjty ty) (disjdots disjexp ids)
 	(function ty -> function ids ->
 	  Ast.rewrap bty (Ast.EnumDef(ty,lb,ids,rb)))
   | Ast.StructUnionDef(ty,lb,decls,rb) ->
-      disjmult2 (disjty ty) (disjdots disjdecl decls)
+      disjmult2 (disjty ty) (disjdots anndisjdecl decls)
 	(function ty -> function decls ->
 	  Ast.rewrap bty (Ast.StructUnionDef(ty,lb,decls,rb)))
   | Ast.TypeName(_) | Ast.MetaType(_,_,_) -> [bty]
 
+and anndisjdecl d =
+  match Ast.unwrap d with
+    Ast.DElem(bef,allminus,decls) ->
+      List.map
+	(function decl -> Ast.rewrap d (Ast.DElem(bef,allminus,decl)))
+	(disjdecl decls)
+  | Ast.Ddots(_,_) -> [d]
+
 and disjident e =
   match Ast.unwrap e with
     Ast.DisjId(id_list) -> List.concat (List.map disjident id_list)
@@ -118,7 +147,8 @@ and disjident e =
 
 and disjexp e =
   match Ast.unwrap e with
-    Ast.Ident(_) | Ast.Constant(_) -> [e] (* even Ident can't contain disj *)
+    Ast.Ident(_) | Ast.Constant(_) | Ast.StringConstant(_) ->
+      [e] (* even Ident can't contain disj, nor StringConstant *)
   | Ast.FunCall(fn,lp,args,rp) ->
       disjmult2 (disjexp fn) (disjdots disjexp args)
 	(function fn -> function args ->
@@ -127,6 +157,10 @@ and disjexp e =
       disjmult2 (disjexp left) (disjexp right)
 	(function left -> function right ->
 	  Ast.rewrap e (Ast.Assignment(left,op,right,simple)))
+  | Ast.Sequence(left,op,right) ->
+      disjmult2 (disjexp left) (disjexp right)
+	(function left -> function right ->
+	  Ast.rewrap e (Ast.Sequence(left,op,right)))
   | Ast.CondExpr(exp1,why,Some exp2,colon,exp3) ->
       let res = disjmult disjexp [exp1;exp2;exp3] in
       List.map
@@ -184,8 +218,15 @@ and disjexp e =
   | Ast.TypeExp(ty) ->
       let ty = disjty ty in
       List.map (function ty -> Ast.rewrap e (Ast.TypeExp(ty))) ty
+  | Ast.Constructor(lp,ty,rp,init) ->
+      disjmult2 (disjty ty) (disjini init)
+	(function ty ->
+	  function exp -> Ast.rewrap e (Ast.Constructor(lp,ty,rp,init)))
   | Ast.MetaErr(_,_,_,_) | Ast.MetaExpr(_,_,_,_,_,_)
   | Ast.MetaExprList(_,_,_,_) | Ast.EComma(_) -> [e]
+  | Ast.AsExpr(exp,asexp) -> (* as exp doesn't contain disj *)
+      let exp = disjexp exp in
+      List.map (function exp -> Ast.rewrap e (Ast.AsExpr(exp,asexp))) exp
   | Ast.DisjExpr(exp_list) -> List.concat (List.map disjexp exp_list)
   | Ast.NestExpr(starter,expr_dots,ender,whencode,multi) ->
       (* not sure what to do here, so ambiguities still possible *)
@@ -202,8 +243,11 @@ and disjparam p =
   match Ast.unwrap p with
     Ast.VoidParam(ty) -> [p] (* void is the only possible value *)
   | Ast.Param(ty,id) ->
-      let ty = disjty ty in
-      List.map (function ty -> Ast.rewrap p (Ast.Param(ty,id))) ty
+      disjmult2 (disjty ty) (disjoption disjident id)
+	(fun ty id -> Ast.rewrap p (Ast.Param(ty,id)))
+  | Ast.AsParam(pm,asexp) -> (* as exp doesn't contain disj *)
+      let pm = disjparam pm in
+      List.map (function pm -> Ast.rewrap p (Ast.AsParam(pm,asexp))) pm
   | Ast.MetaParam(_,_,_) | Ast.MetaParamList(_,_,_,_) | Ast.PComma(_) -> [p]
   | Ast.Pdots(dots) | Ast.Pcircles(dots) -> [p]
   | Ast.OptParam(param) ->
@@ -215,7 +259,10 @@ and disjparam p =
 
 and disjini i =
   match Ast.unwrap i with
-    Ast.MetaInit(_,_,_) -> [i]
+    Ast.MetaInit(_,_,_) | Ast.MetaInitList(_,_,_,_) -> [i]
+  | Ast.AsInit(ini,asini) ->
+      let ini = disjini ini in
+      List.map (function ini -> Ast.rewrap i (Ast.AsInit(ini,asini))) ini
   | Ast.InitExpr(exp) ->
       let exp = disjexp exp in
       List.map (function exp -> Ast.rewrap i (Ast.InitExpr(exp))) exp
@@ -259,10 +306,17 @@ and designator = function
 	(function min -> function max ->
 	  Ast.DesignatorRange(lb,min,dots,max,rb))
 
+and disjfninfo = function
+  | Ast.FType(ty) -> List.map (function ty -> Ast.FType(ty)) (disjty ty)
+  | fi -> [fi]
+
 and disjdecl d =
   match Ast.unwrap d with
     Ast.MetaDecl(_,_,_) | Ast.MetaField(_,_,_)
   | Ast.MetaFieldList(_,_,_,_) -> [d]
+  | Ast.AsDecl(decl,asdecl) ->
+      let decl = disjdecl decl in
+      List.map (function decl -> Ast.rewrap d (Ast.AsDecl(decl,asdecl))) decl
   | Ast.Init(stg,ty,id,eq,ini,sem) ->
       disjmult2 (disjty ty) (disjini ini)
 	(function ty -> function ini ->
@@ -270,10 +324,20 @@ and disjdecl d =
   | Ast.UnInit(stg,ty,id,sem) ->
       let ty = disjty ty in
       List.map (function ty -> Ast.rewrap d (Ast.UnInit(stg,ty,id,sem))) ty
+  | Ast.FunProto(fninfo,name,lp1,params,va,rp1,sem) ->
+      let fninfo = disjmult disjfninfo fninfo in
+      List.map
+	(function fninfo ->
+	  Ast.rewrap d (Ast.FunProto(fninfo,name,lp1,params,va,rp1,sem)))
+	fninfo
   | Ast.MacroDecl(name,lp,args,rp,sem) ->
       List.map
 	(function args -> Ast.rewrap d (Ast.MacroDecl(name,lp,args,rp,sem)))
 	(disjdots disjexp args)
+  | Ast.MacroDeclInit(name,lp,args,rp,eq,ini,sem) ->
+      disjmult2 (disjdots disjexp args) (disjini ini)
+	(function args -> function ini ->
+	  Ast.rewrap d (Ast.MacroDeclInit(name,lp,args,rp,eq,ini,sem)))
   | Ast.TyDecl(ty,sem) ->
       let ty = disjty ty in
       List.map (function ty -> Ast.rewrap d (Ast.TyDecl(ty,sem))) ty
@@ -281,7 +345,6 @@ and disjdecl d =
       let ty = disjty ty in (* disj not allowed in id *)
       List.map (function ty -> Ast.rewrap d (Ast.Typedef(stg,ty,id,sem))) ty
   | Ast.DisjDecl(decls) -> List.concat (List.map disjdecl decls)
-  | Ast.Ddots(_,_) -> [d]
   | Ast.OptDecl(decl) ->
       let decl = disjdecl decl in
       List.map (function decl -> Ast.rewrap d (Ast.OptDecl(decl))) decl
@@ -300,23 +363,27 @@ let orify_rule_elem re exp rebuild =
 let orify_rule_elem_ty = generic_orify_rule_elem disjty
 let orify_rule_elem_param = generic_orify_rule_elem disjparam
 let orify_rule_elem_decl = generic_orify_rule_elem disjdecl
+let orify_rule_elem_anndecl = generic_orify_rule_elem anndisjdecl
 let orify_rule_elem_ini = generic_orify_rule_elem disjini
 
 let rec disj_rule_elem r k re =
   match Ast.unwrap re with
-    Ast.FunHeader(bef,allminus,fninfo,name,lp,params,rp) ->
-      generic_orify_rule_elem (disjdots disjparam) re params
-	(function params ->
+    Ast.FunHeader(bef,allminus,fninfo,name,lp,params,va,rp) ->
+      generic_orify_rule_elem
+	(disjtwoelems disjident (disjdots disjparam)) re
+	(name,params)
+	(fun (name,params) ->
 	  Ast.rewrap re
-	    (Ast.FunHeader(bef,allminus,fninfo,name,lp,params,rp)))
-  | Ast.Decl(bef,allminus,decl) ->
-      orify_rule_elem_decl re decl
-	(function decl -> Ast.rewrap re (Ast.Decl(bef,allminus,decl)))
+	    (Ast.FunHeader(bef,allminus,fninfo,name,lp,params,va,rp)))
+  | Ast.Decl decl ->
+      orify_rule_elem_anndecl re decl
+	(function decl -> Ast.rewrap re (Ast.Decl decl))
   | Ast.SeqStart(brace) -> re
   | Ast.SeqEnd(brace) -> re
-  | Ast.ExprStatement(exp,sem) ->
+  | Ast.ExprStatement(Some exp,sem) ->
       orify_rule_elem re exp
-	(function exp -> Ast.rewrap re (Ast.ExprStatement(exp,sem)))
+	(function exp -> Ast.rewrap re (Ast.ExprStatement(Some exp,sem)))
+  | Ast.ExprStatement(None,sem) -> re
   | Ast.IfHeader(iff,lp,exp,rp) ->
       orify_rule_elem re exp
 	(function exp -> Ast.rewrap re (Ast.IfHeader(iff,lp,exp,rp)))
@@ -328,12 +395,19 @@ let rec disj_rule_elem r k re =
   | Ast.WhileTail(whl,lp,exp,rp,sem) ->
       orify_rule_elem re exp
 	(function exp -> Ast.rewrap re (Ast.WhileTail(whl,lp,exp,rp,sem)))
-  | Ast.ForHeader(fr,lp,e1,sem1,e2,sem2,e3,rp) ->
-      generic_orify_rule_elem (disjmult (disjoption disjexp)) re [e1;e2;e3]
-	(function
-	    [exp1;exp2;exp3] ->
-	      Ast.rewrap re (Ast.ForHeader(fr,lp,exp1,sem1,exp2,sem2,exp3,rp))
-	  | _ -> failwith "not possible")
+  | Ast.ForHeader(fr,lp,first,e2,sem2,e3,rp) ->
+      let disjfirst = function
+	  Ast.ForExp(e1,sem1) ->
+	    List.map (function e1 -> Ast.ForExp(e1,sem1))
+	      (disjoption disjexp e1)
+	| Ast.ForDecl decl ->
+	    List.map (function decl -> Ast.ForDecl decl) (anndisjdecl decl) in
+      generic_orify_rule_elem
+	(disjmult_two disjfirst (disjoption disjexp)) re (first,[e2;e3])
+        (function
+            (first,[exp2;exp3]) ->
+              Ast.rewrap re (Ast.ForHeader(fr,lp,first,exp2,sem2,exp3,rp))
+          |  _ -> failwith "not possible")
   | Ast.IteratorHeader(whl,lp,args,rp) ->
       generic_orify_rule_elem (disjdots disjexp) re args
 	(function args -> Ast.rewrap re (Ast.IteratorHeader(whl,lp,args,rp)))
@@ -345,6 +419,7 @@ let rec disj_rule_elem r k re =
   | Ast.ReturnExpr(ret,exp,sem) ->
       orify_rule_elem re exp
 	(function exp -> Ast.rewrap re (Ast.ReturnExpr(ret,exp,sem)))
+  | Ast.Exec _ -> re (* no ors possible *)
   | Ast.MetaRuleElem(_,_,_) | Ast.MetaStmt(_,_,_,_)
   | Ast.MetaStmtList(_,_,_) -> re
   | Ast.Exp(exp) ->
@@ -359,6 +434,18 @@ let rec disj_rule_elem r k re =
   | Ast.Include(inc,s) -> re
   | Ast.Undef(def,id) -> re
   | Ast.DefineHeader(def,id,params) -> re
+  | Ast.Pragma(prg,id,body) ->
+      let pragmabody body =
+	match Ast.unwrap body with
+	  Ast.PragmaTuple(lp,args,rp) ->
+	    let args = disjdots disjexp args in
+	    List.map
+	      (function args -> Ast.rewrap body (Ast.PragmaTuple(lp,args,rp)))
+	      args
+	| Ast.PragmaIdList(ids) -> [body]
+	| Ast.PragmaDots(dots)  -> [body] in
+      generic_orify_rule_elem pragmabody re body
+	(function body -> Ast.rewrap re (Ast.Pragma(prg,id,body)))
   | Ast.Default(def,colon) -> re
   | Ast.Case(case,exp,colon) ->
       orify_rule_elem re exp
@@ -371,10 +458,12 @@ let disj_all =
   let mcode x = x in
   let donothing r k e = k e in
   V.rebuilder
-    mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
-    donothing donothing donothing donothing donothing
+    mcode mcode mcode mcode mcode mcode mcode mcode mcode
+    mcode mcode mcode mcode mcode
     donothing donothing donothing donothing donothing donothing donothing
-    disj_rule_elem donothing donothing donothing donothing
+    donothing donothing donothing donothing donothing donothing donothing
+    donothing donothing
+    donothing disj_rule_elem donothing donothing donothing donothing
 
 (* ----------------------------------------------------------------------- *)
 (* collect iso information at the rule_elem level *)
@@ -386,10 +475,12 @@ let collect_all_isos =
   let donothing r k e = Common.union_set (Ast.get_isos e) (k e) in
   let doanything r k e = k e in
   V.combiner bind option_default
-    mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
+    mcode mcode mcode mcode mcode mcode mcode mcode mcode
+    mcode mcode mcode mcode mcode
     donothing donothing donothing donothing donothing donothing donothing
     donothing donothing donothing donothing donothing donothing donothing
-    donothing donothing doanything
+    donothing donothing
+    doanything donothing donothing donothing donothing doanything
 
 let collect_iso_info =
   let mcode x = x in
@@ -401,9 +492,11 @@ let collect_iso_info =
 	let isos = collect_all_isos.V.combiner_rule_elem e in
 	Ast.set_isos e isos in
   V.rebuilder
-    mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
+    mcode mcode mcode mcode mcode mcode mcode mcode mcode
+    mcode mcode mcode mcode mcode
     donothing donothing donothing donothing donothing donothing donothing
-     donothing
+    donothing donothing
+    donothing donothing donothing donothing
     donothing donothing donothing donothing rule_elem donothing donothing
     donothing donothing
 
