@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./flag_parsing_cocci.ml"
 (* uses E rather than A and adds comments indicating the start and end of
 each matched term *)
 
@@ -30,5 +33,20 @@ let sgrep_mode = ref false (* no longer supported, subsumed by sgrep2 *)
 let show_SP = ref false
 let show_iso_failures = ref true
 
+(* interpret the patch in an inverted way *)
+let interpret_inverted = ref false
+
 let iso_limit = ref (None : int option) (*(Some 3)*)
 let disabled_isos = ref ([] : string list)
+
+(* Used to debug embedded ML scripts *)
+let keep_ml_script = ref false
+
+let id_utils_index = ref ".id-utils.index" (* default id-utils value *)
+
+let in_minus = ref false
+
+let debug_parse_cocci = ref false
+
+(* When using the parser to generate hardened semantic patches *)
+let generating_mode = ref false
