@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./free_vars.ml"
 (* For each rule return the list of variables that are used after it.
 Also augment various parts of each rule with unitary, inherited, and freshness
 informations *)
@@ -71,7 +74,7 @@ let collect_refs include_constraints =
   (* the following considers that anything that occurs non-unitarily in one
      branch occurs nonunitarily in all branches.  This is not optimal, but
      doing better seems to require a breadth-first traversal, which is
-     perhaps better to avoid.  Also, unitarily is represented as occuring once,
+     perhaps better to avoid.  Also, unitarily is represented as occurring once,
      while nonunitarily is represented as twice - more is irrelevant *)
   (* cases for disjs and metavars *)
   let bind_disj refs_branches =
@@ -110,6 +113,9 @@ let collect_refs include_constraints =
 	bind [tyname] res
     | TC.MetaType(tyname,_,_) ->
 	bind [tyname] res
+    | TC.Decimal(e1,e2) ->
+	let e2mv = function TC.MV(mv,_,_) -> [mv] | _ -> [] in
+	bind (e2mv e1) (e2mv e2)
     | TC.SignedT(_,Some ty) -> type_collect res ty
     | ty -> res in
 
@@ -142,6 +148,23 @@ let collect_refs include_constraints =
       | Ast.DisjExpr(exps) -> bind_disj (List.map k exps)
       | _ -> option_default) in
 
+  let astfvfrag recursor k ft =
+    bind (k ft)
+      (match Ast.unwrap ft with
+	Ast.MetaFormatList(pct,name,Ast.MetaListLen (lenname,_,_),_,_) ->
+	  [metaid name;metaid lenname]
+      |	Ast.MetaFormatList(pct,name,_,_,_) -> [metaid name]
+      | _ -> option_default) in
+
+  let astfvfmt recursor k ft =
+    bind (k ft)
+      (match Ast.unwrap ft with
+	Ast.MetaFormat(name,_,_,_) ->
+	  (* constraint can only be a regexp, so no need to check
+	     include_constraints or check for Ast.IdNegIdSet *)
+	  [metaid name]
+      | _ -> option_default) in
+
   let astfvdecls recursor k d =
     bind (k d)
       (match Ast.unwrap d with
@@ -169,6 +192,9 @@ let collect_refs include_constraints =
     bind (k ty)
       (match Ast.unwrap ty with
 	Ast.MetaInit(name,_,_) -> [metaid name]
+      | Ast.MetaInitList(name,Ast.MetaListLen(lenname,_,_),_,_) ->
+	  [metaid name;metaid lenname]
+      | Ast.MetaInitList(name,_,_,_) -> [metaid name]
       | _ -> option_default) in
 
   let astfvparam recursor k p =
@@ -196,19 +222,23 @@ let collect_refs include_constraints =
 	  bind_disj (List.map recursor.V.combiner_statement_dots stms)
       | _ -> option_default) in
 
-  let mcode r mc =
+  let mcode r mc = (*
     if include_constraints
-    then
-      match Ast.get_pos_var mc with
-	Ast.MetaPos(name,constraints,_,_,_) -> (metaid name)::constraints
-      | _ -> option_default
-    else option_default in
+    then *)
+      List.concat
+	(List.map
+	   (function Ast.MetaPos(name,constraints,_,_,_) ->
+	     (metaid name)::(if include_constraints then constraints else []))
+	   (Ast.get_pos_var mc))
+    (* else option_default *) in
 
   V.combiner bind option_default
-    mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
+    mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
+    mcode mcode mcode mcode
     donothing donothing donothing donothing donothing
-    astfvident astfvexpr astfvfullType astfvtypeC astfvinit astfvparam
-    astfvdecls astfvrule_elem astfvstatement donothing donothing donothing_a
+    astfvident astfvexpr astfvfrag astfvfmt donothing donothing
+    astfvfullType astfvtypeC astfvinit astfvparam astfvdecls donothing
+    astfvrule_elem astfvstatement donothing donothing donothing_a
 
 let collect_all_refs = collect_refs true
 let collect_non_constraint_refs = collect_refs false
@@ -219,6 +249,44 @@ let collect_all_rule_refs minirules =
 
 let collect_all_minirule_refs = collect_all_refs.V.combiner_top_level
 
+let collect_pos_positions =
+  let bind x y = x @ y in
+  let option_default = [] in
+  let donothing recursor k e = k e in (* just combine in the normal way *)
+
+  let metaid (x,_,_,_) = x in
+
+  let mcode r mc =
+      List.concat
+	(List.map
+	   (function Ast.MetaPos(name,constraints,_,_,_) -> [metaid name])
+	   (Ast.get_pos_var mc)) in
+
+  let cprule_elem recursor k re =
+    match Ast.unwrap re with
+      Ast.DisjRuleElem relist ->
+	(*take the intersection of the results*)
+	let subres = List.map k relist in
+	List.fold_left Common.inter_set (List.hd subres) (List.tl subres)
+    | _ -> k re in
+
+  let cpstmt recursor k s =
+    match Ast.unwrap s with
+      Ast.Disj stmlist ->
+	(*take the intersection of the results*)
+	let subres =
+	  List.map recursor.V.combiner_statement_dots stmlist in
+	List.fold_left Common.inter_set (List.hd subres) (List.tl subres)
+    | _ -> k s in
+
+  V.combiner bind option_default
+    mcode mcode mcode mcode mcode mcode mcode mcode mcode
+    mcode mcode mcode mcode mcode
+    donothing donothing donothing donothing donothing donothing donothing
+    donothing donothing donothing donothing donothing donothing
+    donothing donothing donothing donothing cprule_elem cpstmt
+    donothing donothing donothing
+
 (* ---------------------------------------------------------------- *)
 
 let collect_saved =
@@ -247,6 +315,9 @@ let collect_saved =
 	bind [tyname] res
     | TC.MetaType(tyname,TC.Saved,_) ->
 	bind [tyname] res
+    | TC.Decimal(e1,e2) ->
+	let e2mv = function TC.MV(mv,TC.Saved,_) -> [mv] | _ -> [] in
+	bind (e2mv e1) (e2mv e2)
     | TC.SignedT(_,Some ty) -> type_collect res ty
     | ty -> res in
 
@@ -271,6 +342,21 @@ let collect_saved =
 	| _ -> option_default) in
     bind tymetas vars in
 
+  let astfvfrag recursor k ft =
+    bind (k ft)
+      (match Ast.unwrap ft with
+	Ast.MetaFormatList(pct,name,Ast.MetaListLen (lenname,_,_),
+			   TC.Saved,_) ->
+	  [metaid name;metaid lenname]
+      |	Ast.MetaFormatList(pct,name,_,TC.Saved,_) -> [metaid name]
+      | _ -> option_default) in
+
+  let astfvfmt recursor k ft =
+    bind (k ft)
+      (match Ast.unwrap ft with
+	Ast.MetaFormat(name,_,TC.Saved,_) -> [metaid name]
+      | _ -> option_default) in
+
   let astfvtypeC recursor k ty =
     bind (k ty)
       (match Ast.unwrap ty with
@@ -281,6 +367,12 @@ let collect_saved =
     bind (k ty)
       (match Ast.unwrap ty with
 	Ast.MetaInit(name,TC.Saved,_) -> [metaid name]
+      |	Ast.MetaInitList(name,Ast.MetaListLen (lenname,ls,_),ns,_) ->
+	  let namesaved =
+	    match ns with TC.Saved -> [metaid name] | _ -> [] in
+	  let lensaved =
+	    match ls with TC.Saved -> [metaid lenname] | _ -> [] in
+	  lensaved @ namesaved
       | _ -> option_default) in
 
   let astfvparam recursor k p =
@@ -320,15 +412,20 @@ let collect_saved =
 	 | _ -> option_default)) in
 
   let mcode r e =
-    match Ast.get_pos_var e with
-      Ast.MetaPos(name,_,_,TC.Saved,_) -> [metaid name]
-    | _ -> option_default in
+    List.fold_left
+      (function acc ->
+	function
+	    Ast.MetaPos(name,_,_,TC.Saved,_) -> (metaid name) :: acc
+	  | _ -> acc)
+      option_default (Ast.get_pos_var e) in
 
   V.combiner bind option_default
     mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
+    mcode mcode
     donothing donothing donothing donothing donothing
-    astfvident astfvexpr donothing astfvtypeC astfvinit astfvparam
-    astfvdecls astfvrule_elem donothing donothing donothing donothing
+    astfvident astfvexpr astfvfrag astfvfmt donothing donothing donothing
+    astfvtypeC astfvinit astfvparam astfvdecls donothing astfvrule_elem 
+    donothing donothing donothing donothing
 
 (* ---------------------------------------------------------------- *)
 
@@ -342,7 +439,10 @@ let cip_mcodekind r mck =
       (List.map (function l -> List.fold_left (@) [] (List.map astfvs l))
 	 anythings) in
   match mck with
-    Ast.MINUS(_,_,_,anythings) -> process_anything_list_list anythings
+    Ast.MINUS(_,_,_,replacement) ->
+      (match replacement with
+	Ast.REPLACEMENT(anythings,_) -> process_anything_list_list anythings
+      |	Ast.NOREPLACEMENT -> [])
   | Ast.CONTEXT(_,befaft) ->
       (match befaft with
 	Ast.BEFORE(ll,_) -> process_anything_list_list ll
@@ -398,9 +498,16 @@ let collect_in_plus_term =
 
   (* case for things with bef/aft mcode *)
 
+  let annotated_decl decl =
+    match Ast.unwrap decl with
+      Ast.DElem(bef,_,_) -> bef
+    | _ -> failwith "not possible" in
+
   let astfvrule_elem recursor k re =
     match Ast.unwrap re with
-      Ast.FunHeader(bef,_,fi,nm,_,params,_) ->
+      Ast.FunHeader(bef,_,fi,nm,_,params,_,_) ->
+	bind (cip_mcodekind recursor bef) (k re)
+	  (* no clue why this code is here *) (*
 	let fi_metas =
 	  List.concat
 	    (List.map
@@ -424,23 +531,26 @@ let collect_in_plus_term =
 	bind fi_metas
 	  (bind nm_metas
 	     (bind param_metas
-		(bind (cip_mcodekind recursor bef) (k re))))
-    | Ast.Decl(bef,_,_) ->
-	bind (cip_mcodekind recursor bef) (k re)
+		(bind (cip_mcodekind recursor bef) (k re)))) *)
+    | Ast.Decl decl ->
+	bind (cip_mcodekind recursor (annotated_decl decl)) (k re)
+    | Ast.ForHeader(fr,lp,Ast.ForDecl(decl),e2,sem2,e3,rp) ->
+	bind (cip_mcodekind recursor (annotated_decl decl)) (k re)
     | _ -> k re in
 
   let astfvstatement recursor k s =
     match Ast.unwrap s with
       Ast.IfThen(_,_,(_,_,_,aft)) | Ast.IfThenElse(_,_,_,_,(_,_,_,aft))
     | Ast.While(_,_,(_,_,_,aft)) | Ast.For(_,_,(_,_,_,aft))
-    | Ast.Iterator(_,_,(_,_,_,aft)) ->
+    | Ast.Iterator(_,_,(_,_,_,aft)) | Ast.FunDecl(_,_,_,_,(_,_,_,aft)) ->
 	bind (k s) (cip_mcodekind recursor aft)
     | _ -> k s in
 
   V.combiner bind option_default
     mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
-    donothing donothing donothing donothing donothing
-    donothing donothing donothing donothing donothing donothing
+    mcode mcode donothing donothing
+    donothing donothing donothing donothing donothing donothing donothing
+    donothing donothing donothing donothing donothing donothing donothing
     donothing astfvrule_elem astfvstatement donothing donothing donothing
 
 let collect_in_plus metavars minirules =
@@ -470,7 +580,7 @@ let classify_variables metavar_decls minirules used_after =
 
   let donothing r k e = k e in
   let check_unitary name inherited =
-    if List.mem name inplus or List.mem name used_after
+    if List.mem name inplus || List.mem name used_after
     then TC.Saved
     else if not inherited && List.mem name unitary
     then TC.Unitary
@@ -483,12 +593,13 @@ let classify_variables metavar_decls minirules used_after =
     (check_unitary name inherited,inherited) in
 
   let mcode mc =
-    match Ast.get_pos_var mc with
-      Ast.MetaPos(name,constraints,per,unitary,inherited) ->
-	let (unitary,inherited) = classify name in
-	Ast.set_pos_var (Ast.MetaPos(name,constraints,per,unitary,inherited))
-	  mc
-    | _ -> mc in
+    let p =
+      List.map
+	(function Ast.MetaPos(name,constraints,per,unitary,inherited) ->
+	  let (unitary,inherited) = classify name in
+	  Ast.MetaPos(name,constraints,per,unitary,inherited))
+	(Ast.get_pos_var mc) in
+    Ast.set_pos_var p mc in
 
   let ident r k e =
     let e = k e in
@@ -511,14 +622,21 @@ let classify_variables metavar_decls minirules used_after =
     | TC.FunctionPointer(ty) -> TC.FunctionPointer(type_infos ty)
     | TC.Array(ty) -> TC.Array(type_infos ty)
     | TC.EnumName(TC.MV(name,_,_)) ->
-	let (unitary,inherited) = classify (name,(),(),Ast.NoMetaPos) in
+	let (unitary,inherited) = classify (name,(),(),[]) in
 	TC.EnumName(TC.MV(name,unitary,inherited))
     | TC.StructUnionName(su,TC.MV(name,_,_)) ->
-	let (unitary,inherited) = classify (name,(),(),Ast.NoMetaPos) in
+	let (unitary,inherited) = classify (name,(),(),[]) in
 	TC.StructUnionName(su,TC.MV(name,unitary,inherited))
     | TC.MetaType(name,_,_) ->
-	let (unitary,inherited) = classify (name,(),(),Ast.NoMetaPos) in
+	let (unitary,inherited) = classify (name,(),(),[]) in
 	Type_cocci.MetaType(name,unitary,inherited)
+    | TC.Decimal(e1,e2) ->
+	let e2mv = function
+	    TC.MV(mv,_,_) ->
+	      let (unitary,inherited) = classify (mv,(),(),[]) in
+	      TC.MV(mv,unitary,inherited)
+	  | e -> e in
+	TC.Decimal(e2mv e1,e2mv e2)
     | TC.SignedT(sgn,Some ty) -> TC.SignedT(sgn,Some (type_infos ty))
     | ty -> ty in
 
@@ -533,8 +651,6 @@ let classify_variables metavar_decls minirules used_after =
 	let ty = get_option (List.map type_infos) ty in
 	Ast.rewrap e (Ast.MetaExpr(name,constraints,unitary,ty,form,inherited))
     | Ast.MetaExprList(name,Ast.MetaListLen(lenname,_,_),_,_) ->
-	(* lenname should have the same properties of being unitary or
-	   inherited as name *)
 	let (unitary,inherited) = classify name in
 	let (lenunitary,leninherited) = classify lenname in
 	Ast.rewrap e
@@ -543,12 +659,34 @@ let classify_variables metavar_decls minirules used_after =
 	      Ast.MetaListLen(lenname,lenunitary,leninherited),
 	      unitary,inherited))
     | Ast.MetaExprList(name,lenname,_,_) ->
-	(* lenname should have the same properties of being unitary or
-	   inherited as name *)
 	let (unitary,inherited) = classify name in
 	Ast.rewrap e (Ast.MetaExprList(name,lenname,unitary,inherited))
     | _ -> e in
 
+  let string_fragment r k ft =
+    let ft = k ft in
+    match Ast.unwrap ft with
+      Ast.MetaFormatList(pct,name,Ast.MetaListLen (lenname,_,_),_,_) ->
+	let (unitary,inherited) = classify name in
+	let (lenunitary,leninherited) = classify lenname in
+	Ast.rewrap ft
+	  (Ast.MetaFormatList
+	     (pct,name,
+	      Ast.MetaListLen(lenname,lenunitary,leninherited),
+	      unitary,inherited))
+    | Ast.MetaFormatList(pct,name,lenname,_,_) ->
+	let (unitary,inherited) = classify name in
+	Ast.rewrap ft (Ast.MetaFormatList(pct,name,lenname,unitary,inherited))
+    | _ -> ft in
+
+  let string_format r k ft =
+    let ft = k ft in
+    match Ast.unwrap ft with
+      Ast.MetaFormat(name,constraints,_,_) ->
+	let (unitary,inherited) = classify name in
+	Ast.rewrap ft (Ast.MetaFormat(name,constraints,unitary,inherited))
+    | _ -> ft in
+
   let typeC r k e =
     let e = k e in
     match Ast.unwrap e with
@@ -563,6 +701,16 @@ let classify_variables metavar_decls minirules used_after =
       Ast.MetaInit(name,_,_) ->
 	let (unitary,inherited) = classify name in
 	Ast.rewrap e (Ast.MetaInit(name,unitary,inherited))
+    | Ast.MetaInitList(name,Ast.MetaListLen (lenname,_,_),_,_) ->
+	let (unitary,inherited) = classify name in
+	let (lenunitary,leninherited) = classify lenname in
+	Ast.rewrap e
+	  (Ast.MetaInitList
+	     (name,Ast.MetaListLen(lenname,lenunitary,leninherited),
+	      unitary,inherited))
+    | Ast.MetaInitList(name,lenname,_,_) ->
+	let (unitary,inherited) = classify name in
+	Ast.rewrap e (Ast.MetaInitList(name,lenname,unitary,inherited))
     | _ -> e in
 
   let param r k e =
@@ -617,8 +765,10 @@ let classify_variables metavar_decls minirules used_after =
 
   let fn = V.rebuilder
       mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
+      mcode mcode
       donothing donothing donothing donothing donothing
-      ident expression donothing typeC init param decl rule_elem
+      ident expression string_fragment string_format donothing donothing donothing typeC
+      init param decl donothing rule_elem
       donothing donothing donothing donothing in
 
   List.map fn.V.rebuilder_top_level minirules
@@ -634,8 +784,8 @@ are referenced.  Store them in a hash table. *)
 multiple times.  But we get the advantage of not having too many variants
 of the same functions. *)
 
-(* Inherited doesn't include position constraints.  If they are not bound
-then there is no constraint. *)
+(* Inherited doesn't include negative position constraints.  If they are
+not bound then there is no constraint. *)
 
 let astfvs metavars bound =
   let fresh =
@@ -657,11 +807,50 @@ let astfvs metavars bound =
 	([],[]) l in
     (List.rev matched, List.rev freshvars) in
 
+  let refront re =
+    match Ast.unwrap re with
+      Ast.FunHeader _ -> "FunHeader"
+    | Ast.Decl _ -> "Decl"
+    | Ast.SeqStart _ -> "SeqStart"
+    | Ast.SeqEnd _ -> "SeqEnd"
+    | Ast.ExprStatement _ -> "ExprStatement"
+    | Ast.IfHeader _ -> "IfHeader"
+    | Ast.Else _ -> "Else"
+    | Ast.WhileHeader _ -> "WhileHeader"
+    | Ast.DoHeader _ -> "DoHeader"
+    | Ast.WhileTail _ -> "WhileTail"
+    | Ast.ForHeader _ -> "ForHeader"
+    | Ast.IteratorHeader _ -> "IteratorHeader"
+    | Ast.SwitchHeader _ -> "SwitchHeader"
+    | Ast.Break _ -> "Break"
+    | Ast.Continue _ -> "Continue"
+    | Ast.Label _ -> "Label"
+    | Ast.Goto _ -> "Goto"
+    | Ast.Return _ -> "Return"
+    | Ast.ReturnExpr _ -> "ReturnExpr"
+    | Ast.Exec _ -> "Exec"
+    | Ast.MetaRuleElem _ -> "MetaRuleElem"
+    | Ast.MetaStmt _ -> "MetaStmt"
+    | Ast.MetaStmtList _ -> "MetaStmtList"
+    | Ast.Exp _ -> "Exp"
+    | Ast.TopExp _ -> "TopExp"
+    | Ast.Ty _ -> "Ty"
+    | Ast.TopInit _ -> "TopInit"
+    | Ast.Include _ -> "Include"
+    | Ast.Undef _ -> "Undef"
+    | Ast.DefineHeader _ -> "DefineHeader"
+    | Ast.Pragma _ -> "Pragma"
+    | Ast.Case _ -> "Case"
+    | Ast.Default _ -> "Default"
+    | Ast.DisjRuleElem _ -> "DisjRuleElem" in
+
   (* cases for the elements of anything *)
-  let simple_setup getter k re =
+  let simple_setup refront getter k re =
     let minus_free = nub (getter collect_all_refs re) in
     let minus_nc_free =
       nub (getter collect_non_constraint_refs re) in
+    let minus_pos_free =
+      nub (getter collect_pos_positions re) in
     let plus_free =
       collect_fresh_seed metavars (getter collect_in_plus_term re) in
     let free = Common.union_set minus_free plus_free in
@@ -670,6 +859,8 @@ let astfvs metavars bound =
       List.filter (function x -> not(List.mem x bound)) free in
     let inherited =
       List.filter (function x -> List.mem x bound) nc_free in
+    let inherited_pos =
+      List.filter (function x -> List.mem x bound) minus_pos_free in
     let munbound =
       List.filter (function x -> not(List.mem x bound)) minus_free in
     let (matched,fresh) = collect_fresh unbound in
@@ -678,10 +869,11 @@ let astfvs metavars bound =
       Ast.minus_free_vars = munbound;
       Ast.fresh_vars = fresh;
       Ast.inherited = inherited;
+      Ast.positive_inherited_positions = inherited_pos;
       Ast.saved_witness = []} in
 
   let astfvrule_elem recursor k re =
-    simple_setup (function x -> x.V.combiner_rule_elem) k re in
+    simple_setup refront (function x -> x.V.combiner_rule_elem) k re in
 
   let astfvstatement recursor k s =
     let minus_free = nub (collect_all_refs.V.combiner_statement s) in
@@ -721,6 +913,9 @@ let astfvs metavars bound =
       | Ast.Iterator(header,body,(_,_,_,aft)) ->
 	  let (unbound,_,fresh,inherited) = classify (cip_plus aft) [] in
 	  Ast.Iterator(header,body,(unbound,fresh,inherited,aft))
+      |	Ast.FunDecl(header,lbrace,body,rbrace,(_,_,_,aft)) ->
+	  let (unbound,_,fresh,inherited) = classify (cip_plus aft) [] in
+	  Ast.FunDecl(header,lbrace,body,rbrace,(unbound,fresh,inherited,aft))
       |	s -> s in
 
     let (matched,munbound,fresh,_) = classify free minus_free in
@@ -735,10 +930,12 @@ let astfvs metavars bound =
       Ast.saved_witness = []} in
 
   let astfvstatement_dots recursor k sd =
-    simple_setup (function x -> x.V.combiner_statement_dots) k sd in
+    simple_setup (fun _ -> "statement")
+      (function x -> x.V.combiner_statement_dots) k sd in
 
   let astfvcase_line recursor k cl =
-    simple_setup (function x -> x.V.combiner_case_line) k cl in
+    simple_setup (fun _ -> "case")
+      (function x -> x.V.combiner_case_line) k cl in
 
   let astfvtoplevel recursor k tl =
     let saved = collect_saved.V.combiner_top_level tl in
@@ -749,8 +946,11 @@ let astfvs metavars bound =
 
   V.rebuilder
     mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
+    mcode mcode
     donothing donothing astfvstatement_dots donothing donothing
+    donothing donothing donothing 
     donothing donothing donothing donothing donothing donothing donothing
+    donothing donothing
     astfvrule_elem astfvstatement astfvcase_line astfvtoplevel donothing
 
 (*
@@ -775,9 +975,10 @@ let collect_astfvs rules =
 	    (* why are metavars in rule, but outside for cocci rule??? *)
             let bound = script_vars @ bound in
 	    rule::(loop bound rules)
-	| Ast.InitialScriptRule (_,_,_,_) | Ast.FinalScriptRule (_,_,_,_) ->
-	    (* bound stays as is because script rules have no names, so no
-	       inheritance is possible *)
+        | Ast.InitialScriptRule (_,_,_,_,_)
+	| Ast.FinalScriptRule (_,_,_,_,_) ->
+	    (* bound stays as is because init/finalize provides no names, so
+	       inheritance by others is not possible *)
 	    rule::(loop bound rules)
         | Ast.CocciRule (nm, rule_info, minirules, isexp, ruletype) ->
           let bound =
@@ -787,7 +988,7 @@ let collect_astfvs rules =
 	      (List.map (astfvs metavars bound).V.rebuilder_top_level
 		 minirules),
 	      isexp, ruletype))::
-            (loop ((List.map Ast.get_meta_name metavars)@bound) rules) in
+	  (loop ((List.map Ast.get_meta_name metavars)@bound) rules) in
   loop [] rules
 
 (* ---------------------------------------------------------------- *)
@@ -802,18 +1003,22 @@ let get_neg_pos_list (_,rule) used_after_list =
   let option_default = ([],[]) in
   let metaid (x,_,_,_) = x in
   let mcode r mc =
-    match Ast.get_pos_var mc with
-      Ast.MetaPos(name,constraints,Ast.PER,_,_) ->
-	([metaid name],constraints)
-    | Ast.MetaPos(name,constraints,Ast.ALL,_,_) ->
-	([],(metaid name)::constraints)
-    | _ -> option_default in
+    List.fold_left
+      (function (a,b) ->
+	(function
+	    Ast.MetaPos(name,constraints,Ast.PER,_,_) ->
+	      ((metaid name)::a,constraints@b)
+	  | Ast.MetaPos(name,constraints,Ast.ALL,_,_) ->
+	      (a,(metaid name)::constraints@b)))
+      option_default (Ast.get_pos_var mc) in
   let v =
     V.combiner bind option_default
     mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
-    donothing donothing donothing donothing donothing
+    mcode mcode
+    donothing donothing donothing donothing donothing donothing donothing
+    donothing donothing donothing donothing donothing donothing donothing
     donothing donothing donothing donothing donothing donothing
-    donothing donothing donothing donothing donothing donothing in
+    donothing donothing in
   match rule with
     Ast.CocciRule(_,_,minirules,_,_) ->
       List.map
@@ -857,8 +1062,10 @@ let collect_top_level_used_after metavar_rule_list =
             match r with
               Ast.ScriptRule (_,_,_,mv,_,_) ->
                 drop_virt(List.map (function (_,(r,v),_) -> (r,v)) mv)
-            | Ast.InitialScriptRule (_,_,_,_)
-	    | Ast.FinalScriptRule (_,_,_,_) -> []
+            | Ast.InitialScriptRule (_,_,_,mv,_)
+	    | Ast.FinalScriptRule (_,_,_,mv,_) ->
+		(* only virtual identifiers *)
+		[]
             | Ast.CocciRule (_,_,rule,_,_) ->
 		drop_virt
 	          (Common.union_set (nub (collect_all_rule_refs rule))
@@ -966,7 +1173,8 @@ let collect_used_after metavar_rule_list =
       function used_after ->
         match r with
           Ast.ScriptRule (_,_,_,_,_,_) (* no minirules, so nothing to do? *)
-	| Ast.InitialScriptRule (_,_,_,_) | Ast.FinalScriptRule (_,_,_,_) ->
+	| Ast.InitialScriptRule (_,_,_,_,_)
+	| Ast.FinalScriptRule (_,_,_,_,_) ->
 	    ([], [used_after], [[]], [])
         | Ast.CocciRule (name, rule_info, minirules, _,_) ->
           collect_local_used_after metavars minirules used_after
