@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,8 +24,17 @@
  *)
 
 
+# 0 "./get_constants2.mli"
+(* Not is never used in the result.  A bit ugly, but avoids reimplementing
+some operators in get_constants2 *)
+type combine =
+    And of combine list | Or of combine list | Not of combine
+  | Elem of string | False | True
+
 val get_constants :
     Ast_cocci.rule list ->
       (((Ast_cocci.meta_name list) list) list) (*negated pos vars*) ->
 	  (string list option (* grep result *) *
-	     string list option (* non-grep result, if any *))
+	     string list option (* non-grep result, if any *) *
+	     (Str.regexp * Str.regexp list) option (* cocci-grep result *) *
+	     combine option (* raw non-grep result, if any *))
