@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,8 +24,9 @@
  *)
 
 
+# 0 "./index.ml"
 (* create an index for each constructor *)
-(* current max is 152 *)
+(* current max is 185 *)
 
 (* doesn't really work - requires that identical terms with no token
 subterms (eg dots) not appear on the same line *)
@@ -79,19 +82,22 @@ let case_line_dots d =
 let ident i =
   match Ast0.unwrap i with
       Ast0.Id(name) -> [10]
-    | Ast0.MetaId(name,_,_) -> [11]
+    | Ast0.MetaId(name,_,_,_) -> [11]
     | Ast0.MetaFunc(name,_,_) -> [12]
     | Ast0.MetaLocalFunc(name,_,_) -> [13]
     | Ast0.DisjId(_,id_list,_,_) -> [152]
     | Ast0.OptIdent(id) -> [14]
     | Ast0.UniqueIdent(id) -> [15]
+    | Ast0.AsIdent _ -> failwith "not possible"
 
 let expression e =
   match Ast0.unwrap e with
     Ast0.Ident(id) -> [17]
   | Ast0.Constant(const) -> [18]
+  | Ast0.StringConstant(lq,str,rq) -> [165]
   | Ast0.FunCall(fn,lp,args,rp) -> [19]
   | Ast0.Assignment(left,op,right,simple) -> [20]
+  | Ast0.Sequence(left,op,right) -> [156]
   | Ast0.CondExpr(exp1,why,exp2,colon,exp3) -> [21]
   | Ast0.Postfix(exp,op) -> [22]
   | Ast0.Infix(exp,op) -> [23]
@@ -106,6 +112,7 @@ let expression e =
   | Ast0.SizeOfExpr(szf,exp) -> [98] (* added after *)
   | Ast0.SizeOfType(szf,lp,ty,rp) -> [99] (* added after *)
   | Ast0.TypeExp(ty) -> [123] (* added after *)
+  | Ast0.Constructor(lp,ty,rp,init) -> [155]
   | Ast0.MetaErr(name,_,_) -> [32]
   | Ast0.MetaExpr(name,_,ty,_,_) -> [33]
   | Ast0.MetaExprList(name,_,_) -> [34]
@@ -117,6 +124,17 @@ let expression e =
   | Ast0.Estars(dots,whencode) -> [40]
   | Ast0.OptExp(exp) -> [41]
   | Ast0.UniqueExp(exp) -> [42]
+  | Ast0.AsExpr _ -> failwith "not possible"
+
+let assignOp op = match Ast0.unwrap op with
+  | Ast0.SimpleAssign _ -> [180]
+  | Ast0.OpAssign _ -> [181]
+  | Ast0.MetaAssign(_,_,_) -> [182]
+
+let binaryOp op = match Ast0.unwrap op with
+  | Ast0.Arith _ -> [183]
+  | Ast0.Logical _ -> [184]
+  | Ast0.MetaBinary(_,_,_) -> [185]
 
 let typeC t =
   match Ast0.unwrap t with
@@ -125,8 +143,8 @@ let typeC t =
   | Ast0.Signed(sign,ty) -> [129]
   | Ast0.Pointer(ty,star) -> [49]
   | Ast0.FunctionPointer(ty,lp1,star,rp1,lp2,params,rp2) -> [131]
-  | Ast0.FunctionType(ty,lp1,params,rp1) -> [132]
   | Ast0.Array(ty,lb,size,rb) -> [50]
+  | Ast0.Decimal(dec,lp,length,comma,precision_opt,rp) -> [160]
   | Ast0.EnumName(kind,name) -> [146]
   | Ast0.EnumDef(ty,lb,decls,rb) -> [150]
   | Ast0.StructUnionName(kind,name) -> [51]
@@ -136,6 +154,7 @@ let typeC t =
   | Ast0.DisjType(_,type_list,_,_) -> [130]
   | Ast0.OptType(ty) -> [45]
   | Ast0.UniqueType(ty) -> [46]
+  | Ast0.AsType _ -> failwith "not possible"
 
 let declaration d =
   match Ast0.unwrap d with
@@ -144,17 +163,21 @@ let declaration d =
   | Ast0.MetaFieldList(name,_,_) -> [152]
   | Ast0.Init(stg,ty,id,eq,exp,sem) -> [54]
   | Ast0.UnInit(stg,ty,id,sem) -> [55]
+  | Ast0.FunProto(fi,name,lp1,params,va,rp1,sem) -> [132]
   | Ast0.MacroDecl(name,lp,args,rp,sem) -> [137]
+  | Ast0.MacroDeclInit(name,lp,args,rp,eq,ini,sem) -> [157]
   | Ast0.TyDecl(ty,sem) -> [116]
   | Ast0.Typedef(stg,ty,id,sem) -> [143]
   | Ast0.DisjDecl(_,decls,_,_) -> [97] (* added after *)
   | Ast0.Ddots(dots,whencode) -> [133]
   | Ast0.OptDecl(decl) -> [56]
   | Ast0.UniqueDecl(decl) -> [57]
+  | Ast0.AsDecl _ -> failwith "not possible"
 
 let initialiser i =
   match Ast0.unwrap i with
     Ast0.MetaInit(nm,_) -> [106] (* added after *)
+  | Ast0.MetaInitList(nm,_,_) -> [153] (* added after *)
   | Ast0.InitExpr(exp) -> [102]
   | Ast0.InitList(lb,initlist,rb,ordered) -> [103]
   | Ast0.InitGccExt(designators,eq,ini) -> [104]
@@ -163,6 +186,7 @@ let initialiser i =
   | Ast0.Idots(d,whencode) -> [109]
   | Ast0.OptIni(id) -> [110]
   | Ast0.UniqueIni(id) -> [111]
+  | Ast0.AsInit _ -> failwith "not possible"
 
 let parameterTypeDef p =
   match Ast0.unwrap p with
@@ -175,10 +199,11 @@ let parameterTypeDef p =
   | Ast0.Pcircles(dots) -> [65]
   | Ast0.OptParam(param) -> [66]
   | Ast0.UniqueParam(param) -> [67]
+  | Ast0.AsParam _ -> failwith "not possible"
 
 let statement s =
   match Ast0.unwrap s with
-    Ast0.FunDecl(bef,fninfo,name,lp,params,rp,lbrace,body,rbrace) -> [68]
+    Ast0.FunDecl(bef,fninfo,name,lp,params,va,rp,lbrace,body,rbrace,aft) -> [68]
   | Ast0.Decl(bef,decl) -> [69]
   | Ast0.Seq(lbrace,body,rbrace) -> [70]
   | Ast0.ExprStatement(exp,sem) -> [71]
@@ -186,7 +211,7 @@ let statement s =
   | Ast0.IfThenElse(iff,lp,exp,rp,branch1,els,branch2,aft) -> [73]
   | Ast0.While(whl,lp,exp,rp,body,_) -> [74]
   | Ast0.Do(d,body,whl,lp,exp,rp,sem) -> [75]
-  | Ast0.For(fr,lp,e1,sem1,e2,sem2,e3,rp,body,_) -> [76]
+  | Ast0.For(fr,lp,first,e2,sem2,e3,rp,body,_) -> [76]
   | Ast0.Iterator(nm,lp,args,rp,body,_) -> [142]
   | Ast0.Switch(switch,lp,exp,rp,lb,decls,cases,rb) -> [125]
   | Ast0.Break(br,sem) -> [100]
@@ -195,6 +220,7 @@ let statement s =
   | Ast0.Goto(goto,l,sem) -> [145]
   | Ast0.Return(ret,sem) -> [77]
   | Ast0.ReturnExpr(ret,exp,sem) -> [78]
+  | Ast0.Exec(exec,lang,code,sem) -> [170]
   | Ast0.MetaStmt(name,_) -> [79]
   | Ast0.MetaStmtList(name,_) -> [80]
   | Ast0.Disj(_,statement_dots_list,_,_) -> [81]
@@ -209,8 +235,21 @@ let statement s =
   | Ast0.Include(inc,name) -> [118]
   | Ast0.Undef(def,id) -> [151]
   | Ast0.Define(def,id,params,body) -> [119]
+  | Ast0.Pragma(prg,id,body) -> [161]
   | Ast0.OptStm(re) -> [87]
   | Ast0.UniqueStm(re) -> [88]
+  | Ast0.AsStmt _ -> failwith "not possible"
+
+let forinfo fi =
+  match Ast0.unwrap fi with
+    Ast0.ForExp(exp,sem) -> [158]
+  | Ast0.ForDecl (bef,decl) -> [159]
+
+and pragmainfo pi =
+  match Ast0.unwrap pi with
+    Ast0.PragmaTuple(lp,args,rp) -> [162]
+  | Ast0.PragmaIdList(ids) -> [163]
+  | Ast0.PragmaDots (dots) -> [164]
 
 let case_line c =
   match Ast0.unwrap c with
@@ -219,12 +258,20 @@ let case_line c =
   | Ast0.DisjCase(_,case_lines,_,_) -> [107]
   | Ast0.OptCase(case) -> [128]
 
+let string_fragment f =
+  match Ast0.unwrap f with
+    Ast0.ConstantFragment(str) -> [166]
+  | Ast0.FormatFragment(pct,fmt) -> [167]
+  | Ast0.Strdots(dots) -> [168]
+  | Ast0.MetaFormatList(pct,name,lenname) -> [169]
+
 let top_level t =
   match Ast0.unwrap t with
-    Ast0.DECL(stmt) -> [90]
+    Ast0.NONDECL(stmt) -> [90]
   | Ast0.FILEINFO(old_file,new_file) -> [92]
   | Ast0.CODE(stmt_dots) -> [94]
   | Ast0.ERRORWORDS(exps) -> [95]
   | Ast0.OTHER(_) -> [96]
+  | Ast0.TOPCODE(_) -> [154]
 
 (* 99-101 already used *)
