@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./insert_plus.ml"
 (* The error message "no available token to attach to" often comes in an
 argument list of unbounded length.  In this case, one should move a comma so
 that there is a comma after the + code. *)
@@ -93,12 +96,16 @@ it *)
   let res =
     V0.flat_combiner bind option_default
       mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
+      mcode mcode
       (donothing Ast0.dotsExpr) (donothing Ast0.dotsInit)
       (donothing Ast0.dotsParam) (donothing Ast0.dotsStmt)
       (donothing Ast0.dotsDecl) (donothing Ast0.dotsCase)
-      (donothing Ast0.ident) expression (donothing Ast0.typeC) initialiser
+      (donothing Ast0.ident) expression  (donothing Ast0.assignOp)
+      (donothing Ast0.binaryOp)
+      (donothing Ast0.typeC) initialiser
       (donothing Ast0.param) (donothing Ast0.decl) statement
-      (donothing Ast0.case_line) topfn in
+      (donothing Ast0.forinfo) (donothing Ast0.case_line)
+      (donothing Ast0.string_fragment) topfn in
   res.VT0.combiner_rec_top_level e
 
 (* --------------------------------------------------------------------- *)
@@ -121,8 +128,8 @@ let create_root_token_table minus =
   Hashtbl.iter
     (function tokens ->
       function (node,_) ->
-	let key =
-	  match node with
+	let rec key n =
+	  match n with
 	    Ast0.DotsExprTag(d) -> Ast0.get_index d
 	  | Ast0.DotsInitTag(d) -> Ast0.get_index d
 	  | Ast0.DotsParamTag(d) -> Ast0.get_index d
@@ -131,6 +138,8 @@ let create_root_token_table minus =
 	  | Ast0.DotsCaseTag(d) -> Ast0.get_index d
 	  | Ast0.IdentTag(d) -> Ast0.get_index d
 	  | Ast0.ExprTag(d) -> Ast0.get_index d
+	  | Ast0.AssignOpTag d -> Ast0.get_index d
+	  | Ast0.BinaryOpTag d -> Ast0.get_index d
 	  | Ast0.ArgExprTag(d) | Ast0.TestExprTag(d) ->
 	      failwith "not possible - iso only"
 	  | Ast0.TypeCTag(d) -> Ast0.get_index d
@@ -138,14 +147,18 @@ let create_root_token_table minus =
 	  | Ast0.InitTag(d) -> Ast0.get_index d
 	  | Ast0.DeclTag(d) -> Ast0.get_index d
 	  | Ast0.StmtTag(d) -> Ast0.get_index d
+	  | Ast0.ForInfoTag(d) -> Ast0.get_index d
 	  | Ast0.CaseLineTag(d) -> Ast0.get_index d
+	  | Ast0.StringFragmentTag(d) -> Ast0.get_index d
 	  | Ast0.TopTag(d) -> Ast0.get_index d
 	  | Ast0.IsoWhenTag(_) -> failwith "only within iso phase"
 	  | Ast0.IsoWhenTTag(_) -> failwith "only within iso phase"
 	  | Ast0.IsoWhenFTag(_) -> failwith "only within iso phase"
-	  | Ast0.MetaPosTag(p) -> failwith "metapostag only within iso phase"
+	  | Ast0.MetaPosTag(p) -> failwith "not in plus code"
+	  | Ast0.HiddenVarTag(p) -> failwith "only within iso phase"
+	  | Ast0.WhenTag(_,_,w) -> key w
 	in
-	Hashtbl.add root_token_table key tokens)
+	Hashtbl.add root_token_table (key node) tokens)
     CN.minus_table;
   List.iter
     (function r ->
@@ -165,20 +178,23 @@ let collect_minus_join_points root =
     then [(Unfavored,info,mcodekind)]
     else [(Favored,info,mcodekind)] in
 
+  let unfavored_mcode (x,_,info,mcodekind,_,_) =
+    [(Unfavored,info,mcodekind)] in
+
   let do_nothing r k e =
     let info = Ast0.get_info e in
     let index = Ast0.get_index e in
     match Ast0.get_mcodekind e with
       (Ast0.MINUS(_)) as mc -> [(Favored,info,mc)]
     | (Ast0.CONTEXT(_)) as mc when not(index = root_index) ->
-	(* This was unfavored at one point, but I don't remember why *)
+	(* was unfavored, not sure why *)
       [(Favored,info,mc)]
     | _ -> k e in
 
 (* don't want to attach to the outside of DOTS, because metavariables can't
 bind to that; not good for isomorphisms *)
 
-  let dots f k d =
+  let dots f k d checklast =
     let multibind l =
       let rec loop = function
 	  [] -> option_default
@@ -186,17 +202,40 @@ bind to that; not good for isomorphisms *)
 	| x::xs -> bind x (loop xs) in
       loop l in
 
-    match Ast0.unwrap d with
-      Ast0.DOTS(l) -> multibind (List.map f l)
-    | Ast0.CIRCLES(l) -> multibind (List.map f l)
-    | Ast0.STARS(l) -> multibind (List.map f l) in
+    let l = Ast0.undots d in
+    match checklast with
+      None -> multibind (List.map f l)
+    | Some checklast ->
+	match List.rev l with
+	  last::others ->
+	    multibind ((List.map f (List.rev others)) @ [checklast last])
+	| [] -> multibind [] in
 
-  let edots r k d = dots r.VT0.combiner_rec_expression k d in
-  let idots r k d = dots r.VT0.combiner_rec_initialiser k d in
-  let pdots r k d = dots r.VT0.combiner_rec_parameter k d in
-  let sdots r k d = dots r.VT0.combiner_rec_statement k d in
-  let ddots r k d = dots r.VT0.combiner_rec_declaration k d in
-  let cdots r k d = dots r.VT0.combiner_rec_case_line k d in
+  (* have to unfavor a trailing comme because it might not match anything *)
+  let edots r k d = dots r.VT0.combiner_rec_expression k d
+      (Some
+	 (function e ->
+	   match Ast0.unwrap e with
+	     Ast0.EComma(comma) -> unfavored_mcode comma
+	   | _ -> r.VT0.combiner_rec_expression e))
+  in
+  let idots r k d = dots r.VT0.combiner_rec_initialiser k d
+      (Some
+	 (function i ->
+	   match Ast0.unwrap i with
+	     Ast0.IComma(comma) -> unfavored_mcode comma
+	   | _ -> r.VT0.combiner_rec_initialiser i)) in
+
+  let pdots r k d = dots r.VT0.combiner_rec_parameter k d
+      (Some
+	 (function p ->
+	   match Ast0.unwrap p with
+	     Ast0.PComma(comma) -> unfavored_mcode comma
+	   | _ -> r.VT0.combiner_rec_parameter p)) in
+
+  let sdots r k d = dots r.VT0.combiner_rec_statement k d None in
+  let ddots r k d = dots r.VT0.combiner_rec_declaration k d None in
+  let cdots r k d = dots r.VT0.combiner_rec_case_line k d None in
 
   (* a case for everything that has a Opt *)
 
@@ -233,12 +272,14 @@ bind to that; not good for isomorphisms *)
  (*     Ast0.IfThen(_,_,_,_,_,aft)
     | Ast0.IfThenElse(_,_,_,_,_,_,_,aft)
     | Ast0.While(_,_,_,_,_,aft)
-    | Ast0.For(_,_,_,_,_,_,_,_,_,aft)
+    | Ast0.For(_,_,_,_,_,_,_,_,aft)
     | Ast0.Iterator(_,_,_,_,_,aft) ->
 	redo_branched (do_nothing r k s) aft*)
-    | Ast0.FunDecl((info,bef),fninfo,name,lp,params,rp,lbrace,body,rbrace) ->
-	(Toplevel,info,bef)::(k s)
-    | Ast0.Decl((info,bef),decl) -> (Decl,info,bef)::(k s)
+    | Ast0.FunDecl((info,bef),fninfo,name,lp,params,va,rp,lbrace,body,rbrace,
+		   (aftinfo,aft)) ->
+	(Toplevel,info,bef)::(k s)@[(Toplevel,aftinfo,aft)]
+    | Ast0.Decl((info,bef),decl) ->
+	(Decl,info,bef)::(k s)
     | Ast0.Nest(starter,stmt_dots,ender,whencode,multi) ->
 	mcode starter @ r.VT0.combiner_rec_statement_dots stmt_dots @
 	mcode ender
@@ -249,6 +290,12 @@ bind to that; not good for isomorphisms *)
 	r.VT0.combiner_rec_statement s
     | _ -> do_nothing r k s in
 
+  let forinfo r k s =
+    match Ast0.unwrap s with
+      Ast0.ForDecl((info,bef),decl) ->
+	(Decl,info,bef)::(k s)
+    | _ -> k s in
+
   let expression r k e =
     match Ast0.unwrap e with
       Ast0.NestExpr(starter,expr_dots,ender,whencode,multi) ->
@@ -308,15 +355,18 @@ bind to that; not good for isomorphisms *)
 
   V0.flat_combiner bind option_default
     mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
+    mcode mcode
     edots idots pdots sdots ddots cdots
-    ident expression typeC initialiser param decl statement case_line do_top
+    ident expression do_nothing do_nothing
+    typeC initialiser param decl statement forinfo
+    case_line do_nothing do_top
 
 
 let call_collect_minus context_nodes :
     (int * (minus_join_point * Ast0.info * Ast0.mcodekind) list) list =
   List.map
     (function e ->
-      match e with
+      let rec f' = (function
 	Ast0.DotsExprTag(e) ->
 	  (Ast0.get_index e,
 	   (collect_minus_join_points e).VT0.combiner_rec_expression_dots e)
@@ -341,6 +391,12 @@ let call_collect_minus context_nodes :
       | Ast0.ExprTag(e) ->
 	  (Ast0.get_index e,
 	   (collect_minus_join_points e).VT0.combiner_rec_expression e)
+      | Ast0.AssignOpTag op ->
+	  (Ast0.get_index op,
+	   (collect_minus_join_points op).VT0.combiner_rec_assignOp op)
+      | Ast0.BinaryOpTag op ->
+	  (Ast0.get_index op,
+	   (collect_minus_join_points op).VT0.combiner_rec_binaryOp op)
       | Ast0.ArgExprTag(e) | Ast0.TestExprTag(e) ->
 	  failwith "not possible - iso only"
       | Ast0.TypeCTag(e) ->
@@ -358,6 +414,12 @@ let call_collect_minus context_nodes :
       | Ast0.StmtTag(e) ->
 	  (Ast0.get_index e,
 	   (collect_minus_join_points e).VT0.combiner_rec_statement e)
+      | Ast0.ForInfoTag(e) ->
+	  (Ast0.get_index e,
+	   (collect_minus_join_points e).VT0.combiner_rec_forinfo e)
+      | Ast0.StringFragmentTag(e) ->
+	  (Ast0.get_index e,
+	   (collect_minus_join_points e).VT0.combiner_rec_string_fragment e)
       | Ast0.CaseLineTag(e) ->
 	  (Ast0.get_index e,
 	   (collect_minus_join_points e).VT0.combiner_rec_case_line e)
@@ -367,7 +429,9 @@ let call_collect_minus context_nodes :
       | Ast0.IsoWhenTag(_) -> failwith "only within iso phase"
       | Ast0.IsoWhenTTag(_) -> failwith "only within iso phase"
       | Ast0.IsoWhenFTag(_) -> failwith "only within iso phase"
-      | Ast0.MetaPosTag(p) -> failwith "metapostag only within iso phase")
+      | Ast0.MetaPosTag(p) -> failwith "not in plus code"
+      | Ast0.HiddenVarTag(p) -> failwith "only within iso phase"
+      | Ast0.WhenTag(_,_,w) -> f' w) in f' e)
     context_nodes
 
 (* result of collecting the join points should be sorted in nondecreasing
@@ -411,7 +475,6 @@ let process_minus minus =
 	 verify res;
 	 res)
        minus)
-
 (* --------------------------------------------------------------------- *)
 (* --------------------------------------------------------------------- *)
 (* collect the plus tokens *)
@@ -422,9 +485,13 @@ let mk_ident x            = Ast.IdentTag (Ast0toast.ident x)
 let mk_expression x       = Ast.ExpressionTag (Ast0toast.expression x)
 let mk_constant x         = Ast.ConstantTag x
 let mk_unaryOp x          = Ast.UnaryOpTag x
-let mk_assignOp x         = Ast.AssignOpTag x
+let mk_assignOp x         = Ast.AssignOpTag (Ast0toast.assignOp x)
+let mk_simpleAssignOp x   = Ast.SimpleAssignOpTag x
+let mk_opAssignOp x       = Ast.OpAssignOpTag x
 let mk_fixOp x            = Ast.FixOpTag x
-let mk_binaryOp x         = Ast.BinaryOpTag x
+let mk_binaryOp x         = Ast.BinaryOpTag (Ast0toast.binaryOp x)
+let mk_arithOp x          = Ast.ArithOpTag x
+let mk_logicalOp x        = Ast.LogicalOpTag x
 let mk_arithOp x          = Ast.ArithOpTag x
 let mk_logicalOp x        = Ast.LogicalOpTag x
 let mk_declaration x      = Ast.DeclarationTag (Ast0toast.declaration x)
@@ -432,7 +499,9 @@ let mk_topdeclaration x   = Ast.DeclarationTag (Ast0toast.declaration x)
 let mk_storage x          = Ast.StorageTag x
 let mk_inc_file x         = Ast.IncFileTag x
 let mk_statement x        = Ast.StatementTag (Ast0toast.statement x)
+let mk_forinfo x          = Ast.ForInfoTag (Ast0toast.forinfo x)
 let mk_case_line x        = Ast.CaseLineTag (Ast0toast.case_line x)
+let mk_string_fragment x  = Ast.StringFragmentTag (Ast0toast.string_fragment x)
 let mk_const_vol x        = Ast.ConstVolTag x
 let mk_token x info       = Ast.Token (x,Some info)
 let mk_meta (_,x) info    = Ast.Token (x,Some info)
@@ -441,9 +510,9 @@ let mk_code x             = Ast.Code (Ast0toast.top_level x)
 let mk_exprdots x  = Ast.ExprDotsTag (Ast0toast.expression_dots x)
 let mk_paramdots x = Ast.ParamDotsTag (Ast0toast.parameter_list x)
 let mk_stmtdots x  = Ast.StmtDotsTag (Ast0toast.statement_dots x)
-let mk_decldots x  = Ast.DeclDotsTag (Ast0toast.declaration_dots x)
+let mk_decldots x  = Ast.AnnDeclDotsTag (Ast0toast.declaration_dots x)
 let mk_casedots x  = failwith "+ case lines not supported"
-let mk_typeC x     = Ast.FullTypeTag (Ast0toast.typeC x)
+let mk_typeC x     = Ast.FullTypeTag (Ast0toast.typeC false x)
 let mk_init x      = Ast.InitTag (Ast0toast.initialiser x)
 let mk_param x     = Ast.ParamTag (Ast0toast.parameterTypeDef x)
 
@@ -470,7 +539,7 @@ let collect_plus_nodes root =
 	      Ast0.offset = first.Ast0.offset} in
 	  let new_info = {adjust_info with Ast0.pos_info = new_pos_info} in
 	  let string = List.map (function (s,_) -> s) strings_before in
-	  [(new_info,Ast.ONE(*?*),Ast.Pragma (string))] in
+	  [(new_info,Ast.ONE(*?*),Ast.Directive (string))] in
     let bef = extract info.Ast0.strings_before in
     let aft = extract info.Ast0.strings_after in
     (bef,aft) in
@@ -487,13 +556,15 @@ let collect_plus_nodes root =
     | Ast0.CONTEXT _ -> let (bef,aft) = extract_strings info in bef@aft
     | _ -> [] in
 
-  let info (i,_) = let (bef,aft) = extract_strings i in bef@aft in
+  let info (i,_,_) = let (bef,aft) = extract_strings i in bef@aft in
+  let pre_info (i,_) = let (bef,aft) = extract_strings i in bef@aft in
 
-  let do_nothing fn r k e =
+  let do_nothing_extra bef aft fn r k e =
     match Ast0.get_mcodekind e with
       (Ast0.CONTEXT(_)) when not(Ast0.get_index e = root_index) -> []
     | Ast0.PLUS c -> [(Ast0.get_info e,c,fn e)]
-    | _ -> k e in
+    | _ -> bef@(k e)@aft in
+  let do_nothing fn r k e = do_nothing_extra [] [] fn r k e in
 
   (* case for everything that is just a wrapper for a simpler thing *)
   (* case for things with bef aft *)
@@ -504,19 +575,19 @@ let collect_plus_nodes root =
     | Ast0.Ty(ty) -> r.VT0.combiner_rec_typeC ty
     | Ast0.TopInit(init) -> r.VT0.combiner_rec_initialiser init
     | Ast0.Decl(bef,decl) ->
-	(info bef) @ (do_nothing mk_statement r k e)
-    | Ast0.FunDecl(bef,fi,name,lp,params,rp,lbrace,body,rbrace) ->
-	(info bef) @ (do_nothing mk_statement r k e)
+	 do_nothing_extra (pre_info bef) [] mk_statement r k e
+    | Ast0.FunDecl(bef,fi,name,lp,params,va,rp,lbrace,body,rbrace,aft) ->
+	do_nothing_extra (pre_info bef) (pre_info aft) mk_statement r k e
     | Ast0.IfThen(iff,lp,exp,rp,branch1,aft) ->
-	(do_nothing mk_statement r k e) @ (info aft)
+	do_nothing_extra [] (info aft) mk_statement r k e
     | Ast0.IfThenElse(iff,lp,exp,rp,branch1,els,branch2,aft) ->
-	(do_nothing mk_statement r k e) @ (info aft)
+	do_nothing_extra [] (info aft) mk_statement r k e
     | Ast0.While(whl,lp,exp,rp,body,aft) ->
-	(do_nothing mk_statement r k e) @ (info aft)
-    | Ast0.For(fr,lp,e1,sem1,e2,sem2,e3,rp,body,aft) ->
-	(do_nothing mk_statement r k e) @ (info aft)
+	do_nothing_extra [] (info aft) mk_statement r k e
+    | Ast0.For(fr,lp,first,e2,sem2,e3,rp,body,aft) ->
+	do_nothing_extra [] (info aft)mk_statement r k e
     | Ast0.Iterator(nm,lp,args,rp,body,aft) ->
-	(do_nothing mk_statement r k e) @ (info aft)
+	do_nothing_extra [] (info aft) mk_statement r k e
     | _ -> do_nothing mk_statement r k e in
 
   (* statementTag is preferred, because it indicates that one statement is
@@ -529,31 +600,35 @@ let collect_plus_nodes root =
 
   let toplevel r k e =
     match Ast0.unwrap e with
-      Ast0.DECL(s) -> r.VT0.combiner_rec_statement s
+      Ast0.NONDECL(s) -> r.VT0.combiner_rec_statement s
     | Ast0.CODE(sdots) -> r.VT0.combiner_rec_statement_dots sdots
     | _ -> do_nothing mk_code r k e in
 
   let initdots r k e = k e in
 
   V0.flat_combiner bind option_default
-    (imcode mk_meta) (imcode mk_token) (mcode mk_constant) (mcode mk_assignOp)
+    (imcode mk_meta) (imcode mk_token) (mcode mk_constant)
+    (mcode mk_simpleAssignOp) (mcode mk_opAssignOp)
     (mcode mk_fixOp)
-    (mcode mk_unaryOp) (mcode mk_binaryOp) (mcode mk_const_vol)
+    (mcode mk_unaryOp) (mcode mk_arithOp)
+    (mcode mk_logicalOp) (mcode mk_const_vol)
     (mcode mk_sign) (mcode mk_structUnion)
     (mcode mk_storage) (mcode mk_inc_file)
     (do_nothing mk_exprdots) initdots
     (do_nothing mk_paramdots) stmt_dots (do_nothing mk_decldots)
     (do_nothing mk_casedots)
     (do_nothing mk_ident) (do_nothing mk_expression)
+    (do_nothing mk_assignOp) (do_nothing mk_binaryOp)
     (do_nothing mk_typeC) (do_nothing mk_init) (do_nothing mk_param)
     (do_nothing mk_declaration)
-    stmt (do_nothing mk_case_line) toplevel
+    stmt (do_nothing mk_forinfo) (do_nothing mk_case_line)
+    (do_nothing mk_string_fragment) toplevel
 
 let call_collect_plus context_nodes :
     (int * (Ast0.info * Ast.count * Ast.anything) list) list =
   List.map
     (function e ->
-      match e with
+      let rec f' = (function
 	Ast0.DotsExprTag(e) ->
 	  (Ast0.get_index e,
 	   (collect_plus_nodes e).VT0.combiner_rec_expression_dots e)
@@ -578,6 +653,12 @@ let call_collect_plus context_nodes :
       | Ast0.ExprTag(e) ->
 	  (Ast0.get_index e,
 	   (collect_plus_nodes e).VT0.combiner_rec_expression e)
+      | Ast0.AssignOpTag(op) ->
+	  (Ast0.get_index op,
+	   (collect_plus_nodes op).VT0.combiner_rec_assignOp op)
+      | Ast0.BinaryOpTag(op) ->
+	  (Ast0.get_index op,
+	   (collect_plus_nodes op).VT0.combiner_rec_binaryOp op)
       | Ast0.ArgExprTag(_) | Ast0.TestExprTag(_) ->
 	  failwith "not possible - iso only"
       | Ast0.TypeCTag(e) ->
@@ -595,16 +676,24 @@ let call_collect_plus context_nodes :
       | Ast0.StmtTag(e) ->
 	  (Ast0.get_index e,
 	   (collect_plus_nodes e).VT0.combiner_rec_statement e)
+      | Ast0.ForInfoTag(e) ->
+	  (Ast0.get_index e,
+	   (collect_plus_nodes e).VT0.combiner_rec_forinfo e)
       | Ast0.CaseLineTag(e) ->
 	  (Ast0.get_index e,
 	   (collect_plus_nodes e).VT0.combiner_rec_case_line e)
+      | Ast0.StringFragmentTag(e) ->
+	  (Ast0.get_index e,
+	   (collect_plus_nodes e).VT0.combiner_rec_string_fragment e)
       | Ast0.TopTag(e) ->
 	  (Ast0.get_index e,
 	   (collect_plus_nodes e).VT0.combiner_rec_top_level e)
       | Ast0.IsoWhenTag(_) -> failwith "only within iso phase"
       | Ast0.IsoWhenTTag(_) -> failwith "only within iso phase"
       | Ast0.IsoWhenFTag(_) -> failwith "only within iso phase"
-      | Ast0.MetaPosTag(p) -> failwith "metapostag only within iso phase")
+      | Ast0.MetaPosTag(p) -> failwith "not visible here"
+      | Ast0.HiddenVarTag(_) -> failwith "only within iso phase"
+      | Ast0.WhenTag(_,_,w) -> f' w) in f' e)
     context_nodes
 
 (* The plus fragments are converted to a list of lists of lists.
@@ -701,7 +790,7 @@ let favored = function Favored -> true | Unfavored | Toplevel | Decl -> false
 
 let top_code =
   List.for_all
-    (List.for_all (function Ast.Code _ | Ast.Pragma _ -> true | _ -> false))
+    (List.for_all (function Ast.Code _ | Ast.Directive _ -> true | _ -> false))
 
 let storage_code =
   List.for_all
@@ -720,7 +809,7 @@ let predecl_code =
     | Ast.Rule_elemTag _
     | Ast.StmtDotsTag _
     | Ast.Code _
-    | Ast.Pragma _ -> true
+    | Ast.Directive _ -> true
       (* the following should definitely be false *)
     | Ast.FullTypeTag _ | Ast.BaseTypeTag _ | Ast.StructUnionTag _
     | Ast.SignTag _
@@ -781,14 +870,19 @@ let init thing info =
      Ast0.left_offset = info.Ast0.pos_info.Ast0.offset;
      Ast0.right_offset = info.Ast0.pos_info.Ast0.offset})
 
+let it2c = function Ast.ONE -> "one" | Ast.MANY -> "many"
+
 let attachbefore (infop,c,p) = function
     Ast0.MINUS(replacements) ->
       let (repl,ti) = !replacements in
-      let (bef,ti) =
- 	match repl with
- 	  [] -> init p infop
- 	| repl -> insert p infop repl ti in
-      replacements := (bef,ti)
+      (match repl with
+ 	Ast.NOREPLACEMENT ->
+	  let (bef,ti) = init p infop in
+	  replacements := (Ast.REPLACEMENT(bef,c),ti)
+      | Ast.REPLACEMENT(repl,it) ->
+	  let it = Ast.lub_count it c in
+	  let (bef,ti) = insert p infop repl ti in
+	  replacements := (Ast.REPLACEMENT(bef,it),ti))
   | Ast0.CONTEXT(neighbors) ->
       let (repl,ti1,ti2) = !neighbors in
       (match repl with
@@ -812,11 +906,14 @@ let attachbefore (infop,c,p) = function
 let attachafter (infop,c,p) = function
     Ast0.MINUS(replacements) ->
        let (repl,ti) = !replacements in
-       let (aft,ti) =
- 	match repl with
- 	  [] -> init p infop
- 	| repl -> insert p infop repl ti in
-       replacements := (aft,ti)
+       (match repl with
+ 	Ast.NOREPLACEMENT ->
+	  let (aft,ti) = init p infop in
+	  replacements := (Ast.REPLACEMENT(aft,c),ti)
+      | Ast.REPLACEMENT(repl,it) ->
+	  let it = Ast.lub_count it c in
+	  let (aft,ti) = insert p infop repl ti in
+	  replacements := (Ast.REPLACEMENT(aft,it),ti))
   | Ast0.CONTEXT(neighbors) ->
       let (repl,ti1,ti2) = !neighbors in
       (match repl with
@@ -856,7 +953,7 @@ let allminus = function
 let rec before_m1 ((f1,infom1,m1) as x1) ((f2,infom2,m2) as x2) rest = function
     [] -> ()
   | (((infop,_,pcode) as p) :: ps) as all ->
-      if less_than_start infop infom1 or
+      if less_than_start infop infom1 ||
 	(allminus m1 && less_than_end infop infom1) (* account for trees *)
       then
 	if toplevel f1
@@ -884,7 +981,7 @@ and after_m1 ((f1,infom1,m1) as x1) ((f2,infom2,m2) as x2) rest = function
 	 what it can infer from something being CONTEXT with no top-level
 	 modifications.  for the moment, we thus give an error, asking the
 	 user to rewrite the semantic patch. *)
-      if greater_than_end infop infom1 or is_minus m1 or !empty_isos
+      if greater_than_end infop infom1 || is_minus m1 || !empty_isos
       then
 	if less_than_start infop infom2
 	then
@@ -963,11 +1060,14 @@ let merge_one : (minus_join_point * Ast0.info * 'a) list *
   Printf.printf "minus code\n";
   List.iter
     (function (_,info,_) ->
-      Printf.printf "start %d end %d real_start %d real_end %d\n"
+      Printf.printf
+	"start %d end %d real_start %d real_end %d attachable start %b attachable end %b\n"
 	info.Ast0.pos_info.Ast0.logical_start
 	info.Ast0.pos_info.Ast0.logical_end
 	info.Ast0.pos_info.Ast0.line_start
-	info.Ast0.pos_info.Ast0.line_end)
+	info.Ast0.pos_info.Ast0.line_end
+	info.Ast0.attachable_start
+	info.Ast0.attachable_end)
     m;
   Printf.printf "plus code\n";
   List.iter
@@ -980,6 +1080,7 @@ let merge_one : (minus_join_point * Ast0.info * 'a) list *
       Pretty_print_cocci.print_anything "" p;
       Format.print_newline())
     p;
+  Printf.printf "end of plus code\n";
   *)
   match (m,p) with
     (_,[]) -> ()
@@ -1016,44 +1117,53 @@ let reevaluate_contextness =
        Ast0.CONTEXT(mc) -> let (ba,_,_) = !mc in [ba]
      | _ -> [] in
 
-   let info (_,mc) =
+   let pre_info (_,mc) =
      match mc with
        Ast0.CONTEXT(mc) -> let (ba,_,_) = !mc in [ba]
      | _ -> [] in
 
-   let donothing r k e =
+   let info (_,mc,_) =
+     match mc with
+       Ast0.CONTEXT(mc) -> let (ba,_,_) = !mc in [ba]
+     | _ -> [] in
+
+   let donothing_extra extra r k e =
      match Ast0.get_mcodekind e with
        Ast0.CONTEXT(mc) ->
-	 if List.exists (function Ast.NOTHING -> false | _ -> true) (k e)
+	 let updates = extra @ (k e) in
+	 if List.exists (function Ast.NOTHING -> false | _ -> true) updates
 	 then Ast0.set_mcodekind e (Ast0.MIXED(mc));
 	 []
      | _ -> let _ = k e in [] in
+   let donothing r k e = donothing_extra [] r k e in
 
    (* a case for everything with bef or aft *)
    let stmt r k e =
      match Ast0.unwrap e with
        Ast0.Decl(bef,decl) ->
-	 (info bef) @ (donothing r k e)
-     | Ast0.FunDecl(bef,fi,name,lp,params,rp,lbrace,body,rbrace) ->
-	 (info bef) @ (donothing r k e)
+	 donothing_extra (pre_info bef) r k e
+     | Ast0.FunDecl(bef,fi,name,lp,params,va,rp,lbrace,body,rbrace,aft) ->
+	 donothing_extra ((pre_info bef) @ (pre_info aft)) r k e
      | Ast0.IfThen(iff,lp,exp,rp,branch1,aft) ->
-	 (donothing r k e) @ (info aft)
+	 donothing_extra (info aft) r k e
      | Ast0.IfThenElse(iff,lp,exp,rp,branch1,els,branch2,aft) ->
-	 (donothing r k e) @ (info aft)
+	 donothing_extra (info aft) r k e
      | Ast0.While(whl,lp,exp,rp,body,aft) ->
-	 (donothing r k e) @ (info aft)
-     | Ast0.For(fr,lp,e1,sem1,e2,sem2,e3,rp,body,aft) ->
-	 (donothing r k e) @ (info aft)
+	 donothing_extra (info aft) r k e
+     | Ast0.For(fr,lp,first,e2,sem2,e3,rp,body,aft) ->
+	 donothing_extra (info aft) r k e
      | Ast0.Iterator(nm,lp,args,rp,body,aft) ->
-	 (donothing r k e) @ (info aft)
+	 donothing_extra (info aft) r k e
      | _ -> donothing r k e in
 
   let res =
     V0.flat_combiner bind option_default
       mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
+      mcode mcode
       donothing donothing donothing donothing donothing donothing donothing
-      donothing
-      donothing donothing donothing donothing stmt donothing donothing in
+      donothing donothing donothing donothing
+      donothing donothing donothing stmt donothing donothing donothing
+      donothing in
   res.VT0.combiner_rec_top_level
 
 (* --------------------------------------------------------------------- *)
