@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./pretty_print_cocci.ml"
 open Format
 module Ast = Ast_cocci
 
@@ -66,7 +69,7 @@ and print_anything_list = function
 	(match bef with
 	  Ast.Rule_elemTag(_) | Ast.AssignOpTag(_) | Ast.BinaryOpTag(_)
 	| Ast.ArithOpTag(_) | Ast.LogicalOpTag(_)
-	| Ast.Token("if",_) | Ast.Token("while",_) -> true | _ -> false) or
+	| Ast.Token("if",_) | Ast.Token("while",_) -> true | _ -> false) ||
 	(match aft with
 	  Ast.Rule_elemTag(_) | Ast.AssignOpTag(_) | Ast.BinaryOpTag(_)
 	| Ast.ArithOpTag(_) | Ast.LogicalOpTag(_) | Ast.Token("{",_) -> true
@@ -90,11 +93,13 @@ let print_string_befaft fn x info =
 
 let print_meta (r,x) = print_string r; print_string ":"; print_string x
 
-let print_pos = function
-    Ast.MetaPos(name,_,_,_,_) ->
-      let name = Ast.unwrap_mcode name in
-      print_string "@"; print_meta name
-  | _ -> ()
+let print_pos l =
+  List.iter
+    (function
+	Ast.MetaPos(name,_,_,_,_) ->
+	  let name = Ast.unwrap_mcode name in
+	  print_string "@"; print_meta name)
+    l
 
 let mcode fn = function
     (x, _, Ast.MINUS(_,_,adj,plus_stream), pos) ->
@@ -102,7 +107,10 @@ let mcode fn = function
       then print_string (if !Flag.sgrep_mode2 then "*" else "-");
       fn x; print_pos pos;
       if !print_plus_flag
-      then print_anything ">>> " plus_stream
+      then
+	(match plus_stream with
+	  Ast.NOREPLACEMENT -> ()
+	| Ast.REPLACEMENT(plus_stream,_) -> print_anything ">>> " plus_stream)
   | (x, _, Ast.CONTEXT(_,plus_streams), pos) ->
       if !print_plus_flag
       then
@@ -116,7 +124,9 @@ let mcode fn = function
 let print_mcodekind = function
     Ast.MINUS(_,_,_,plus_stream) ->
       print_string "MINUS";
-      print_anything ">>> " plus_stream
+      (match plus_stream with
+	Ast.NOREPLACEMENT -> ()
+      | Ast.REPLACEMENT(plus_stream,_) -> print_anything ">>> " plus_stream)
   | Ast.CONTEXT(_,plus_streams) ->
       print_around (function _ -> print_string "CONTEXT") () plus_streams
   | Ast.PLUS _ -> print_string "PLUS"
@@ -161,26 +171,31 @@ let print_disj_list fn l =
 (* --------------------------------------------------------------------- *)
 
 let print_type keep info = function
-    None -> ()
-	(* print_string "/* ";
+    None -> () (*
+	;print_string "/* notype ";(*
            print_string "keep:"; print_unitary keep;
-           print_string " inherited:"; print_bool inherited;
-           print_string " */"*)
-  | Some ty -> ()
-      (*;
+           print_string " inherited:"; print_bool inherited;*)
+           print_string " */" *)
+  | Some ty -> () (*
+     ;
       print_string "/* ";
       print_between (function _ -> print_string ", ") Type_cocci.typeC ty;(*
       print_string "keep:"; print_unitary keep;
       print_string " inherited:"; print_bool inherited;*)
-      print_string " */"*)
+      print_string " */" *)
 
 (* --------------------------------------------------------------------- *)
-(* Contraint on Identifier and Function *)
+(* Constraint on Identifier and Function *)
 (* FIXME: Not called at the moment *)
 
 let rec idconstraint = function
     Ast.IdNoConstraint  -> print_string "/* No constraint */"
+  | Ast.IdPosIdSet (str,meta)     ->
+      print_string " =";
+      List.iter (function s -> print_string (" "^s)) str;
+      List.iter (function (r,n) -> print_string " "; print_meta(r,n)) meta
   | Ast.IdNegIdSet (str,meta)     ->
+      print_string " !=";
       List.iter (function s -> print_string (" "^s)) str;
       List.iter (function (r,n) -> print_string " "; print_meta(r,n)) meta
   | Ast.IdRegExpConstraint re -> regconstraint re
@@ -200,6 +215,7 @@ let rec ident i =
   | Ast.MetaId(name,_,keep,inherited) -> mcode print_meta name
   | Ast.MetaFunc(name,_,_,_) -> mcode print_meta name
   | Ast.MetaLocalFunc(name,_,_,_) -> mcode print_meta name
+  | Ast.AsIdent(id,asid) -> ident id; print_string "@"; ident asid
   | Ast.DisjId(id_list) -> print_disj_list ident id_list
   | Ast.OptIdent(id) -> print_string "?"; ident id
   | Ast.UniqueIdent(id) -> print_string "!"; ident id
@@ -216,12 +232,19 @@ let rec expression e =
   match Ast.unwrap e with
     Ast.Ident(id) -> ident id
   | Ast.Constant(const) -> mcode constant const
+  | Ast.StringConstant(lq,str,rq) ->
+      mcode print_string lq;
+      dots (function _ -> ()) string_fragment str;
+      mcode print_string rq
   | Ast.FunCall(fn,lp,args,rp) ->
       expression fn; mcode print_string_box lp;
       dots (function _ -> ()) expression args;
       close_box(); mcode print_string rp
   | Ast.Assignment(left,op,right,simple) ->
-      expression left; print_string " "; mcode assignOp op;
+      expression left; print_string " "; assignOp op;
+      print_string " "; expression right
+  | Ast.Sequence(left,op,right) ->
+      expression left; mcode print_string op;
       print_string " "; expression right
   | Ast.CondExpr(exp1,why,exp2,colon,exp3) ->
       expression exp1; print_string " "; mcode print_string why;
@@ -231,10 +254,10 @@ let rec expression e =
   | Ast.Infix(exp,op) -> mcode fixOp op; expression exp
   | Ast.Unary(exp,op) -> mcode unaryOp op; expression exp
   | Ast.Binary(left,op,right) ->
-      expression left; print_string " "; mcode binaryOp op; print_string " ";
+      expression left; print_string " "; binaryOp op; print_string " ";
       expression right
   | Ast.Nested(left,op,right) ->
-      expression left; print_string " "; mcode binaryOp op; print_string " ";
+      expression left; print_string " "; binaryOp op; print_string " ";
       expression right
   | Ast.Paren(lp,exp,rp) ->
       mcode print_string_box lp; expression exp; close_box();
@@ -256,11 +279,15 @@ let rec expression e =
       mcode print_string_box lp; fullType ty; close_box();
       mcode print_string rp
   | Ast.TypeExp(ty) -> fullType ty
+  | Ast.Constructor(lp,ty,rp,init) ->
+      mcode print_string_box lp; fullType ty; close_box();
+      mcode print_string rp; initialiser init
 
   | Ast.MetaErr(name,_,_,_) -> mcode print_meta name
   | Ast.MetaExpr(name,_,keep,ty,form,inherited) ->
       mcode print_meta name; print_type keep inherited ty
   | Ast.MetaExprList(name,_,_,_) -> mcode print_meta name
+  | Ast.AsExpr(exp,asexp) -> expression exp; print_string "@"; expression asexp
   | Ast.EComma(cm) -> mcode print_string cm; print_space()
   | Ast.DisjExpr(exp_list) -> print_disj_list expression exp_list
   | Ast.NestExpr(starter,expr_dots,ender,Some whencode,multi) ->
@@ -279,31 +306,58 @@ let rec expression e =
   | Ast.OptExp(exp) -> print_string "?"; expression exp
   | Ast.UniqueExp(exp) -> print_string "!"; expression exp
 
-and  unaryOp = function
+and string_fragment e =
+  match Ast.unwrap e with
+    Ast.ConstantFragment(str) -> mcode print_string str
+  | Ast.FormatFragment(pct,fmt) ->
+      mcode print_string pct;
+      string_format fmt
+  | Ast.Strdots dots -> mcode print_string dots
+  | Ast.MetaFormatList(pct,name,lenname,_,_) ->
+      mcode print_string pct;
+      mcode print_meta name
+
+and string_format e =
+  match Ast.unwrap e with
+    Ast.ConstantFormat(str) -> mcode print_string str
+  | Ast.MetaFormat(name,_,_,_) -> mcode print_meta name
+
+and unaryOp = function
     Ast.GetRef -> print_string "&"
+  | Ast.GetRefLabel -> print_string "&&"
   | Ast.DeRef -> print_string "*"
   | Ast.UnPlus -> print_string "+"
   | Ast.UnMinus -> print_string "-"
   | Ast.Tilde -> print_string "~"
   | Ast.Not -> print_string "!"
 
-and  assignOp = function
-    Ast.SimpleAssign -> print_string "="
-  | Ast.OpAssign(aop) -> arithOp aop; print_string "="
+and assignOp op =
+  match Ast.unwrap op with
+    Ast.SimpleAssign _ -> print_string "="
+  | Ast.OpAssign(aop) -> arithOp (Ast.unwrap_mcode aop); print_string "="
+  | Ast.MetaAssign(metavar,_,_,_) -> mcode print_meta metavar
+                                                   
+and simpleAssignOp op = print_string "="
 
-and  fixOp = function
+and opAssignOp aop = arithOp aop; print_string "=" 
+
+and fixOp = function
     Ast.Dec -> print_string "--"
   | Ast.Inc -> print_string "++"
 
-and  binaryOp = function
-    Ast.Arith(aop) -> arithOp aop
-  | Ast.Logical(lop) -> logicalOp lop
+and binaryOp op =
+  match Ast.unwrap op with
+    Ast.Arith(aop) -> arithOp (Ast.unwrap_mcode aop)
+  | Ast.Logical(lop) -> logicalOp (Ast.unwrap_mcode lop)
+  | Ast.MetaBinary(metavar,_,_,_) -> mcode print_meta metavar
 
 and  arithOp = function
     Ast.Plus -> print_string "+"
   | Ast.Minus -> print_string "-"
   | Ast.Mul -> print_string "*"
   | Ast.Div -> print_string "/"
+  | Ast.Min -> print_string "<?"
+  | Ast.Max -> print_string ">?"
   | Ast.Mod -> print_string "%"
   | Ast.DecLeft -> print_string "<<"
   | Ast.DecRight -> print_string ">>"
@@ -326,6 +380,7 @@ and constant = function
   | Ast.Char(s) -> print_string "'"; print_string s; print_string "'"
   | Ast.Int(s) -> print_string s
   | Ast.Float(s) -> print_string s
+  | Ast.DecimalConst(s,_,_) -> print_string s
 
 (* --------------------------------------------------------------------- *)
 (* Declarations *)
@@ -341,9 +396,15 @@ and storage = function
 
 and fullType ft =
   match Ast.unwrap ft with
-    Ast.Type(cv,ty) ->
-      print_option (function x -> mcode const_vol x; print_string " ") cv;
-      typeC ty
+    Ast.Type(_,cv,ty) ->
+      (match Ast.unwrap ty with
+	Ast.Pointer(_,_) ->
+	  typeC ty;
+	  print_option (function x -> print_string " "; mcode const_vol x) cv
+      |	_ ->
+	  print_option (function x -> mcode const_vol x; print_string " ") cv;
+	  typeC ty)
+  | Ast.AsType(ty,asty) -> fullType ty; print_string "@"; fullType asty
   | Ast.DisjType(decls) -> print_disj_list fullType decls
   | Ast.OptType(ty) -> print_string "?"; fullType ty
   | Ast.UniqueType(ty) -> print_string "!"; fullType ty
@@ -353,9 +414,11 @@ and print_function_pointer (ty,lp1,star,rp1,lp2,params,rp2) fn =
   mcode print_string rp1; mcode print_string lp1;
   parameter_list params; mcode print_string rp2
 
-and print_function_type (ty,lp1,params,rp1) fn =
-  print_option fullType ty; fn(); mcode print_string lp1;
-  parameter_list params; mcode print_string rp1
+and varargs = function
+  | None -> ()
+  | Some (comma, ellipsis) ->
+    mcode print_string comma;
+    mcode print_string ellipsis
 
 and print_fninfo = function
     Ast.FStorage(stg) -> mcode storage stg
@@ -372,11 +435,16 @@ and typeC ty =
   | Ast.FunctionPointer(ty,lp1,star,rp1,lp2,params,rp2) ->
       print_function_pointer (ty,lp1,star,rp1,lp2,params,rp2)
 	(function _ -> ())
-  | Ast.FunctionType (_,ty,lp1,params,rp1) ->
-      print_function_type (ty,lp1,params,rp1) (function _ -> ())
   | Ast.Array(ty,lb,size,rb) ->
       fullType ty; mcode print_string lb; print_option expression size;
       mcode print_string rb
+  | Ast.Decimal(dec,lp,length,comma,precision_opt,rp) ->
+      mcode print_string dec;
+      mcode print_string lp;
+      expression length;
+      print_option (mcode print_string) comma;
+      print_option expression precision_opt;
+      mcode print_string rp
   | Ast.EnumName(kind,name) ->
       mcode print_string kind;
       print_option (function x -> ident x; print_string " ") name
@@ -389,7 +457,7 @@ and typeC ty =
       print_option (function x -> ident x; print_string " ") name
   | Ast.StructUnionDef(ty,lb,decls,rb) ->
       fullType ty; mcode print_string lb;
-      dots force_newline declaration decls;
+      dots force_newline (annotated_decl "") decls;
       mcode print_string rb
   | Ast.TypeName(name) -> mcode print_string name; print_string " "
   | Ast.MetaType(name,_,_) ->
@@ -399,11 +467,15 @@ and baseType = function
     Ast.VoidType -> print_string "void "
   | Ast.CharType -> print_string "char "
   | Ast.ShortType -> print_string "short "
+  | Ast.ShortIntType -> print_string "short int "
   | Ast.IntType -> print_string "int "
   | Ast.DoubleType -> print_string "double "
+  | Ast.LongDoubleType -> print_string "long double "
   | Ast.FloatType -> print_string "float "
   | Ast.LongType -> print_string "long "
+  | Ast.LongIntType -> print_string "long int "
   | Ast.LongLongType -> print_string "long long "
+  | Ast.LongLongIntType -> print_string "long long int "
   | Ast.SizeType -> print_string "size_t "
   | Ast.SSizeType -> print_string "ssize_t "
   | Ast.PtrDiffType -> print_string "ptrdiff_t "
@@ -427,20 +499,20 @@ and const_vol = function
 
 and print_named_type ty id =
   match Ast.unwrap ty with
-    Ast.Type(None,ty1) ->
+    Ast.Type(_,None,ty1) ->
       (match Ast.unwrap ty1 with
 	Ast.FunctionPointer(ty,lp1,star,rp1,lp2,params,rp2) ->
 	  print_function_pointer (ty,lp1,star,rp1,lp2,params,rp2)
 	    (function _ -> print_string " "; ident id)
-      | Ast.FunctionType(_,ty,lp1,params,rp1) ->
-	  print_function_type (ty,lp1,params,rp1)
-	    (function _ -> print_string " "; ident id)
       | Ast.Array(ty,lb,size,rb) ->
 	  let rec loop ty k =
 	    match Ast.unwrap ty with
 	      Ast.Array(ty,lb,size,rb) ->
 		(match Ast.unwrap ty with
-		  Ast.Type(None,ty) ->
+		  Ast.Type(_,cv,ty) ->
+		    print_option
+		      (function x -> mcode const_vol x; print_string " ")
+		      cv;
 		    loop ty
 		      (function _ ->
 			k ();
@@ -458,6 +530,8 @@ and declaration d =
     Ast.MetaDecl(name,_,_) | Ast.MetaField(name,_,_)
   | Ast.MetaFieldList(name,_,_,_) ->
       mcode print_meta name
+  | Ast.AsDecl(decl,asdecl) -> declaration decl; print_string "@";
+      declaration asdecl
   | Ast.Init(stg,ty,id,eq,ini,sem) ->
       print_option (mcode storage) stg; print_named_type ty id;
       print_string " "; mcode print_string eq;
@@ -465,20 +539,39 @@ and declaration d =
   | Ast.UnInit(stg,ty,id,sem) ->
       print_option (mcode storage) stg; print_named_type ty id;
       mcode print_string sem
+  | Ast.FunProto (fninfo,name,lp1,params,va,rp1,sem) ->
+      List.iter print_fninfo fninfo;
+      ident name; mcode print_string_box lp1;
+      parameter_list params; varargs va;
+      close_box(); mcode print_string rp1;
+      mcode print_string sem
   | Ast.MacroDecl(name,lp,args,rp,sem) ->
       ident name; mcode print_string_box lp;
       dots (function _ -> ()) expression args;
       close_box(); mcode print_string rp; mcode print_string sem
+  | Ast.MacroDeclInit(name,lp,args,rp,eq,ini,sem) ->
+      ident name; mcode print_string_box lp;
+      dots (function _ -> ()) expression args;
+      close_box(); mcode print_string rp;
+      print_string " "; mcode print_string eq;
+      print_string " "; initialiser ini; mcode print_string sem
   | Ast.TyDecl(ty,sem) -> fullType ty; mcode print_string sem
   | Ast.Typedef(stg,ty,id,sem) ->
       mcode print_string stg; print_string " "; fullType ty; typeC id;
       mcode print_string sem
   | Ast.DisjDecl(decls) -> print_disj_list declaration decls
+  | Ast.OptDecl(decl) -> print_string "?"; declaration decl
+  | Ast.UniqueDecl(decl) -> print_string "!"; declaration decl
+
+and annotated_decl arity d =
+  match Ast.unwrap d with
+    Ast.DElem(bef,allminus,decl) ->
+      mcode (function _ -> ()) ((),Ast.no_info,bef,[]);
+      print_string arity;
+      declaration decl
   | Ast.Ddots(dots,Some whencode) ->
       mcode print_string dots; print_string "   when != "; declaration whencode
   | Ast.Ddots(dots,None) -> mcode print_string dots
-  | Ast.OptDecl(decl) -> print_string "?"; declaration decl
-  | Ast.UniqueDecl(decl) -> print_string "!"; declaration decl
 
 (* --------------------------------------------------------------------- *)
 (* Initialiser *)
@@ -487,6 +580,10 @@ and initialiser i =
   match Ast.unwrap i with
     Ast.MetaInit(name,_,_) ->
       mcode print_meta name; print_string " "
+  | Ast.MetaInitList(name,_,_,_) ->
+      mcode print_meta name; print_string " "
+  | Ast.AsInit(ini,asini) -> initialiser ini; print_string "@";
+      initialiser asini
   | Ast.InitExpr(exp) -> expression exp
   | Ast.ArInitList(lb,initlist,rb) ->
       mcode print_string lb; open_box 0;
@@ -537,6 +634,8 @@ and parameterTypeDef p =
   | Ast.Pcircles(dots) -> mcode print_string dots
   | Ast.OptParam(param) -> print_string "?"; parameterTypeDef param
   | Ast.UniqueParam(param) -> print_string "!"; parameterTypeDef param
+  | Ast.AsParam(p,asexp) ->
+      parameterTypeDef p; print_string "@"; expression asexp
 
 and parameter_list l = dots (function _ -> ()) parameterTypeDef l
 
@@ -545,16 +644,19 @@ and parameter_list l = dots (function _ -> ()) parameterTypeDef l
 
 let rec rule_elem arity re =
   match Ast.unwrap re with
-    Ast.FunHeader(bef,allminus,fninfo,name,lp,params,rp) ->
-      mcode (function _ -> ()) ((),Ast.no_info,bef,Ast.NoMetaPos);
+    Ast.FunHeader(bef,allminus,fninfo,name,lp,params,va,rp) ->
+      mcode (function _ -> ()) ((),Ast.no_info,bef,[]);
       print_string arity; List.iter print_fninfo fninfo;
       ident name; mcode print_string_box lp;
-      parameter_list params; close_box(); mcode print_string rp;
+      parameter_list params;
+      begin match va with
+            | None -> ()
+            | Some (comma,ellipsis) ->
+               mcode print_string comma;
+              mcode print_string ellipsis
+      end; close_box(); mcode print_string rp;
       print_string " "
-  | Ast.Decl(bef,allminus,decl) ->
-      mcode (function _ -> ()) ((),Ast.no_info,bef,Ast.NoMetaPos);
-      print_string arity;
-      declaration decl
+  | Ast.Decl(ann_decl) -> annotated_decl arity ann_decl
   | Ast.SeqStart(brace) ->
       print_string arity; mcode print_string brace;
       if !print_newlines_disj then start_block()
@@ -562,7 +664,7 @@ let rec rule_elem arity re =
       if !print_newlines_disj then end_block();
       print_string arity; mcode print_string brace
   | Ast.ExprStatement(exp,sem) ->
-      print_string arity; expression exp; mcode print_string sem
+      print_string arity; print_option expression exp; mcode print_string sem
   | Ast.IfHeader(iff,lp,exp,rp) ->
       print_string arity;
       mcode print_string iff; print_string " "; mcode print_string_box lp;
@@ -580,10 +682,9 @@ let rec rule_elem arity re =
       mcode print_string whl; print_string " "; mcode print_string_box lp;
       expression exp; close_box(); mcode print_string rp;
       mcode print_string sem
-  | Ast.ForHeader(fr,lp,e1,sem1,e2,sem2,e3,rp) ->
+  | Ast.ForHeader(fr,lp,first,e2,sem2,e3,rp) ->
       print_string arity;
-      mcode print_string fr; mcode print_string_box lp;
-      print_option expression e1; mcode print_string sem1;
+      mcode print_string fr; mcode print_string_box lp; forinfo first;
       print_option expression e2; mcode print_string sem2;
       print_option expression e3; close_box();
       mcode print_string rp; print_string " "
@@ -608,6 +709,11 @@ let rec rule_elem arity re =
   | Ast.ReturnExpr(ret,exp,sem) ->
       print_string arity; mcode print_string ret; print_string " ";
       expression exp; mcode print_string sem
+  | Ast.Exec(exec,lang,code,sem) ->
+      print_string arity; mcode print_string exec; print_string " ";
+      mcode print_string lang; print_string " ";
+      dots (function _ -> print_string " ") exec_code code;
+      mcode print_string sem
   | Ast.MetaRuleElem(name,_,_) ->
       print_string arity; mcode print_meta name
   | Ast.MetaStmt(name,_,_,_) ->
@@ -625,6 +731,9 @@ let rec rule_elem arity re =
   | Ast.DefineHeader(def,id,params) ->
       mcode print_string def; print_string " "; ident id;
       print_define_parameters params
+  | Ast.Pragma(prg,id,body) ->
+      mcode print_string prg; print_string " "; ident id; print_string " ";
+      pragmainfo body
   | Ast.Default(def,colon) ->
       mcode print_string def; mcode print_string colon; print_string " "
   | Ast.Case(case,exp,colon) ->
@@ -639,6 +748,19 @@ let rec rule_elem arity re =
 	res;
       force_newline(); print_string ")"
 
+and forinfo = function
+    Ast.ForExp(e1,sem1) ->
+      print_option expression e1; mcode print_string sem1
+  | Ast.ForDecl(ann_decl) -> annotated_decl "" ann_decl
+
+and pragmainfo pi =
+  match Ast.unwrap pi with
+      Ast.PragmaTuple(lp,args,rp) ->
+	mcode print_string_box lp;
+	dots (function _ -> ()) expression args;
+	close_box(); mcode print_string rp
+    | Ast.PragmaIdList(ids) -> dots (function _ -> ()) ident ids
+    | Ast.PragmaDots (dots) -> mcode print_string dots
 
 and print_define_parameters params =
   match Ast.unwrap params with
@@ -664,33 +786,34 @@ and statement arity s =
       rule_elem arity rbrace
   | Ast.IfThen(header,branch,(_,_,_,aft)) ->
       rule_elem arity header; statement arity branch;
-      mcode (function _ -> ()) ((),Ast.no_info,aft,Ast.NoMetaPos)
+      mcode (function _ -> ()) ((),Ast.no_info,aft,[])
   | Ast.IfThenElse(header,branch1,els,branch2,(_,_,_,aft)) ->
       rule_elem arity header; statement arity branch1; print_string " ";
       rule_elem arity els; statement arity branch2;
-      mcode (function _ -> ()) ((),Ast.no_info,aft,Ast.NoMetaPos)
+      mcode (function _ -> ()) ((),Ast.no_info,aft,[])
   | Ast.While(header,body,(_,_,_,aft)) ->
       rule_elem arity header; statement arity body;
-      mcode (function _ -> ()) ((),Ast.no_info,aft,Ast.NoMetaPos)
+      mcode (function _ -> ()) ((),Ast.no_info,aft,[])
   | Ast.Do(header,body,tail) ->
       rule_elem arity header; statement arity body;
       rule_elem arity tail
   | Ast.For(header,body,(_,_,_,aft)) ->
       rule_elem arity header; statement arity body;
-      mcode (function _ -> ()) ((),Ast.no_info,aft,Ast.NoMetaPos)
+      mcode (function _ -> ()) ((),Ast.no_info,aft,[])
   | Ast.Iterator(header,body,(_,_,_,aft)) ->
       rule_elem arity header; statement arity body;
-      mcode (function _ -> ()) ((),Ast.no_info,aft,Ast.NoMetaPos)
+      mcode (function _ -> ()) ((),Ast.no_info,aft,[])
   | Ast.Switch(header,lb,decls,cases,rb) ->
       rule_elem arity header; rule_elem arity lb;
       dots force_newline (statement arity) decls;
       List.iter (function x -> case_line arity x; force_newline()) cases;
       rule_elem arity rb
   | Ast.Atomic(re) -> rule_elem arity re
-  | Ast.FunDecl(header,lbrace,body,rbrace) ->
+  | Ast.FunDecl(header,lbrace,body,rbrace,(_,_,_,aft)) ->
       rule_elem arity header; rule_elem arity lbrace;
       dots force_newline (statement arity) body;
-      rule_elem arity rbrace
+      rule_elem arity rbrace;
+      mcode (function _ -> ()) ((),Ast.no_info,aft,[])
   | Ast.Disj([stmt_dots]) ->
       print_string arity;
       dots (function _ -> if !print_newlines_disj then force_newline())
@@ -706,6 +829,8 @@ and statement arity s =
   | Ast.Define(header,body) ->
       rule_elem arity header; print_string " ";
       dots force_newline (statement arity) body
+  | Ast.AsStmt(stm,asstm) ->
+      statement arity stm; print_string "@"; statement arity asstm
   | Ast.Nest(starter,stmt_dots,ender,whn,multi,_,_) ->
       print_string arity;
       nest_dots starter ender (statement arity)
@@ -756,6 +881,12 @@ and case_line arity c =
       dots force_newline (statement arity) code
   | Ast.OptCase(case) -> case_line "?" case
 
+and exec_code e =
+  match Ast.unwrap e with
+    Ast.ExecEval(colon,id) -> mcode print_string colon; expression id
+  | Ast.ExecToken(tok) -> mcode print_string tok
+  | Ast.ExecDots(dots) -> mcode print_string dots
+
 (* --------------------------------------------------------------------- *)
 (* CPP code *)
 
@@ -781,7 +912,7 @@ let top_level t =
     Ast.FILEINFO(old_file,new_file) ->
       print_string "--- "; mcode print_string old_file; force_newline();
       print_string "+++ "; mcode print_string new_file
-  | Ast.DECL(stmt) -> statement "" stmt
+  | Ast.NONDECL(stmt) -> statement "" stmt
   | Ast.CODE(stmt_dots) ->
       dots force_newline (statement "") stmt_dots
   | Ast.ERRORWORDS(exps) ->
@@ -805,6 +936,8 @@ let _ =
     | Ast.ConstantTag(x) -> constant x
     | Ast.UnaryOpTag(x) -> unaryOp x
     | Ast.AssignOpTag(x) -> assignOp x
+    | Ast.SimpleAssignOpTag(x) -> simpleAssignOp (Ast.make_mcode x)
+    | Ast.OpAssignOpTag(x) -> opAssignOp x
     | Ast.FixOpTag(x) -> fixOp x
     | Ast.BinaryOpTag(x) -> binaryOp x
     | Ast.ArithOpTag(x) -> arithOp x
@@ -815,11 +948,13 @@ let _ =
     | Ast.IncFileTag(x) -> inc_file x
     | Ast.Rule_elemTag(x) -> rule_elem "" x
     | Ast.StatementTag(x) -> statement "" x
+    | Ast.ForInfoTag(x) -> forinfo x
     | Ast.CaseLineTag(x) -> case_line "" x
+    | Ast.StringFragmentTag(x) -> string_fragment x
     | Ast.ConstVolTag(x) -> const_vol x
     | Ast.Token(x,Some info) -> print_string_befaft print_string x info
     | Ast.Token(x,None) -> print_string x
-    | Ast.Pragma(xs) ->
+    | Ast.Directive(xs) ->
 	let print = function
 	    Ast.Noindent s | Ast.Indent s | Ast.Space s -> print_string s in
 	print_between force_newline print xs
@@ -827,7 +962,7 @@ let _ =
     | Ast.ExprDotsTag(x) -> dots (function _ -> ()) expression x
     | Ast.ParamDotsTag(x) -> parameter_list x
     | Ast.StmtDotsTag(x) -> dots (function _ -> ()) (statement "") x
-    | Ast.DeclDotsTag(x) -> dots (function _ -> ()) declaration x
+    | Ast.AnnDeclDotsTag(x) -> dots (function _ -> ()) (annotated_decl "") x
     | Ast.TypeCTag(x) -> typeC x
     | Ast.ParamTag(x) -> parameterTypeDef x
     | Ast.SgrepStartTag(x) -> print_string x
@@ -861,14 +996,17 @@ let script_header str lang deps code =
   force_newline();
   print_string "@@";
   force_newline();
+  let code =
+    String.concat "\n"
+      (Str.split (Str.regexp "[\n\r\011\012]#.*[\n\r\011\012]") code) in
   print_string code;
   force_newline()
 
 let unparse z =
   match z with
-    Ast.InitialScriptRule (name,lang,deps,code) ->
+    Ast.InitialScriptRule (name,lang,deps,mv,code) ->
       script_header "initialize" lang deps code
-  | Ast.FinalScriptRule (name,lang,deps,code) ->
+  | Ast.FinalScriptRule (name,lang,deps,mv,code) ->
       script_header "finalize" lang deps code
   | Ast.ScriptRule (name,lang,deps,bindings,script_vars,code) ->
       script_header "script" lang deps code
