@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./pretty_print_cocci.mli"
 val unparse : Ast_cocci.rule -> unit
 val unparse_to_string : Ast_cocci.rule -> string
 val expression : Ast_cocci.expression -> unit
@@ -37,15 +40,20 @@ val print_mcodekind : Ast_cocci.mcodekind -> unit
 
 val constant : Ast_cocci.constant -> unit
 val assignOp : Ast_cocci.assignOp -> unit
+val simpleAssignOp : Ast_cocci.simpleAssignOp -> unit
+val opAssignOp : Ast_cocci.arithOp -> unit
 val fixOp : Ast_cocci.fixOp -> unit
 val unaryOp : Ast_cocci.unaryOp -> unit
 val binaryOp : Ast_cocci.binaryOp -> unit
+val arithOp : Ast_cocci.arithOp -> unit
+val logicalOp : Ast_cocci.logicalOp -> unit
 val const_vol : Ast_cocci.const_vol -> unit
 val sign : Ast_cocci.sign -> unit
 val structUnion : Ast_cocci.structUnion -> unit
 val storage : Ast_cocci.storage -> unit
 val baseType : Ast_cocci.baseType -> unit
 val fullType : Ast_cocci.fullType -> unit
+val typeC : Ast_cocci.typeC -> unit
 val inc_file : Ast_cocci.inc_file -> unit
 
 val print_around :
@@ -58,3 +66,5 @@ val print_minus_flag : bool ref
 
 val print_rule_elem : Ast_cocci.rule_elem -> unit
 val print_when_modif : Ast_cocci.when_modifier -> unit
+
+val dep : bool -> Ast_cocci.dependency -> unit
