@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./single_statement.ml"
 (* detect statements that are between dots in the minus code, because they
 may need a special treatment if they are if branches *)
 
@@ -72,7 +75,7 @@ let any_statements =
     (List.exists
        (function
 	   Ast.StatementTag(_) | Ast.StmtDotsTag(_)
-	 | Ast.DeclarationTag(_) | Ast.DeclDotsTag(_) -> true | _ -> false))
+	 | Ast.DeclarationTag(_) | Ast.AnnDeclDotsTag(_) -> true | _ -> false))
 
 let modif_before x =
   match Ast0.get_mcodekind x with
@@ -80,10 +83,10 @@ let modif_before x =
   | Ast0.MINUS mc ->
       (match !mc with
 	(* do better for the common case of replacing a stmt by another one *)
-	([[Ast.StatementTag(s)]],ti) ->
+	((Ast.REPLACEMENT([[Ast.StatementTag(s)]],c)) as old,ti) ->
 	  (match Ast.unwrap s with
 	    Ast.IfThen(_,_,_) -> true (* potentially dangerous *)
-	  | _ -> mc := ([[Ast.StatementTag(s)]],ti); false)
+	  | _ -> mc := (old,ti); false)
       |	(_,_) -> true)
   | Ast0.CONTEXT mc | Ast0.MIXED mc ->
       (match !mc with
@@ -97,11 +100,11 @@ let modif_after x =
   | Ast0.MINUS mc ->
       (match !mc with
 	(* do better for the common case of replacing a stmt by another one *)
-	([[Ast.StatementTag(s)]],ti) ->
+	((Ast.REPLACEMENT([[Ast.StatementTag(s)]],count)) as old,ti) ->
 	  (match Ast.unwrap s with
 	    Ast.IfThen(_,_,_) -> true (* potentially dangerous *)
-	  | _ -> mc := ([[Ast.StatementTag(s)]],ti); false)
-      |	(l,_) when any_statements l -> true
+	  | _ -> mc := (old,ti); false)
+      |	(Ast.REPLACEMENT(l,_),_) when any_statements l -> true
       |	(l,ti) -> mc := (l,ti); false)
   | Ast0.CONTEXT mc | Ast0.MIXED mc ->
       (match !mc with
@@ -111,37 +114,41 @@ let modif_after x =
 
 (* Identifier *)
 let rec left_ident i =
-  modif_before i or
+  modif_before i ||
   match Ast0.unwrap i with
     Ast0.Id(name) -> modif_before_mcode name
-  | Ast0.MetaId(name,_,_) -> modif_before_mcode name
+  | Ast0.MetaId(name,_,_,_) -> modif_before_mcode name
   | Ast0.MetaFunc(name,_,_) -> modif_before_mcode name
   | Ast0.MetaLocalFunc(name,_,_) -> modif_before_mcode name
   | Ast0.DisjId(_,id_list,_,_) -> List.exists left_ident id_list
   | Ast0.OptIdent(id) -> left_ident id
   | Ast0.UniqueIdent(id) -> left_ident id
+  | Ast0.AsIdent _ -> failwith "not possible"
 
 let rec right_ident i =
-  modif_after i or
+  modif_after i ||
   match Ast0.unwrap i with
     Ast0.Id(name) -> modif_after_mcode name
-  | Ast0.MetaId(name,_,_) -> modif_after_mcode name
+  | Ast0.MetaId(name,_,_,_) -> modif_after_mcode name
   | Ast0.MetaFunc(name,_,_) -> modif_after_mcode name
   | Ast0.MetaLocalFunc(name,_,_) -> modif_after_mcode name
   | Ast0.DisjId(_,id_list,_,_) -> List.exists right_ident id_list
   | Ast0.OptIdent(id) -> right_ident id
   | Ast0.UniqueIdent(id) -> right_ident id
+  | Ast0.AsIdent _ -> failwith "not possible"
 
 (* --------------------------------------------------------------------- *)
 (* Expression *)
 
 let rec left_expression e =
-  modif_before e or
+  modif_before e ||
   match Ast0.unwrap e with
     Ast0.Ident(id) -> left_ident id
   | Ast0.Constant(const) -> modif_before_mcode const
+  | Ast0.StringConstant(lq,str,rq) -> modif_before_mcode lq
   | Ast0.FunCall(fn,lp,args,rp) -> left_expression fn
   | Ast0.Assignment(left,op,right,_) -> left_expression left
+  | Ast0.Sequence(left,op,right) -> left_expression left
   | Ast0.CondExpr(exp1,why,exp2,colon,exp3) -> left_expression exp1
   | Ast0.Postfix(exp,op) -> left_expression exp
   | Ast0.Infix(exp,op) -> modif_before_mcode op
@@ -156,6 +163,7 @@ let rec left_expression e =
   | Ast0.SizeOfExpr(szf,exp) -> modif_before_mcode szf
   | Ast0.SizeOfType(szf,lp,ty,rp) -> modif_before_mcode szf
   | Ast0.TypeExp(ty) -> left_typeC ty
+  | Ast0.Constructor(lp,ty,rp,init) -> modif_before_mcode lp
   | Ast0.MetaErr(name,_,_) -> modif_before_mcode name
   | Ast0.MetaExpr(name,_,ty,_,_) -> modif_before_mcode name
   | Ast0.MetaExprList(name,_,_) -> modif_before_mcode name
@@ -166,21 +174,22 @@ let rec left_expression e =
   | Ast0.Edots(dots,_) | Ast0.Ecircles(dots,_) | Ast0.Estars(dots,_) -> false
   | Ast0.OptExp(exp) -> left_expression exp
   | Ast0.UniqueExp(exp) -> left_expression exp
+  | Ast0.AsExpr _ -> failwith "not possible"
 
 (* --------------------------------------------------------------------- *)
 (* Types *)
 
 and left_typeC t =
-  modif_before t or
+  modif_before t ||
   match Ast0.unwrap t with
     Ast0.ConstVol(cv,ty) -> modif_before_mcode cv
   | Ast0.BaseType(ty,strings) -> modif_before_mcode (List.hd strings)
   | Ast0.Signed(sgn,ty) -> modif_before_mcode sgn
   | Ast0.Pointer(ty,star) -> left_typeC ty
   | Ast0.FunctionPointer(ty,lp1,star,rp1,lp2,params,rp2) -> left_typeC ty
-  | Ast0.FunctionType(Some ty,lp1,params,rp1) -> left_typeC ty
-  | Ast0.FunctionType(None,lp1,params,rp1) -> modif_before_mcode lp1
   | Ast0.Array(ty,lb,size,rb) -> left_typeC ty
+  | Ast0.Decimal(dec,lp,length,comma,precision_opt,rp) ->
+      modif_before_mcode dec
   | Ast0.EnumName(kind,name) -> modif_before_mcode kind
   | Ast0.EnumDef(ty,lb,ids,rb) -> left_typeC ty
   | Ast0.StructUnionName(kind,name) -> modif_before_mcode kind
@@ -190,6 +199,7 @@ and left_typeC t =
   | Ast0.DisjType(lp,types,mids,rp) -> List.exists left_typeC types
   | Ast0.OptType(ty) -> left_typeC ty
   | Ast0.UniqueType(ty) -> left_typeC ty
+  | Ast0.AsType _ -> failwith "not possible"
 
 (* --------------------------------------------------------------------- *)
 (* Variable declaration *)
@@ -197,7 +207,7 @@ and left_typeC t =
    split out into multiple declarations of a single variable each. *)
 
 and left_declaration d =
-  modif_before d or
+  modif_before d ||
   match Ast0.unwrap d with
     Ast0.MetaDecl(name,_) | Ast0.MetaField(name,_)
   | Ast0.MetaFieldList(name,_,_) ->
@@ -206,33 +216,41 @@ and left_declaration d =
   | Ast0.Init(None,ty,id,eq,ini,sem) -> left_typeC ty
   | Ast0.UnInit(Some stg,ty,id,sem) -> modif_before_mcode stg
   | Ast0.UnInit(None,ty,id,sem) -> left_typeC ty
+  | Ast0.FunProto(fninfo,name,lp1,params,va,rp1,sem) ->
+      (* should not be nested in anything anyway *)
+      false
   | Ast0.MacroDecl(name,lp,args,rp,sem) -> left_ident name
+  | Ast0.MacroDeclInit(name,lp,args,rp,eq,ini,sem) -> left_ident name
   | Ast0.TyDecl(ty,sem) -> left_typeC ty
   | Ast0.Typedef(stg,ty,id,sem) -> modif_before_mcode stg
   | Ast0.DisjDecl(_,decls,_,_) -> List.exists left_declaration decls
   | Ast0.Ddots(dots,_) -> false
   | Ast0.OptDecl(decl) -> left_declaration decl
   | Ast0.UniqueDecl(decl) -> left_declaration decl
+  | Ast0.AsDecl _ -> failwith "not possible"
 
 and right_declaration d =
-  modif_before d or
+  modif_before d ||
   match Ast0.unwrap d with
     Ast0.MetaDecl(name,_) | Ast0.MetaField(name,_)
   | Ast0.MetaFieldList(name,_,_) ->
       modif_before_mcode name
   | Ast0.Init(_,ty,id,eq,ini,sem) -> modif_after_mcode sem
   | Ast0.UnInit(_,ty,id,sem) -> modif_after_mcode sem
+  | Ast0.FunProto(fninfo,name,lp1,params,va,rp1,sem) -> modif_after_mcode sem
   | Ast0.MacroDecl(name,lp,args,rp,sem) -> modif_after_mcode sem
+  | Ast0.MacroDeclInit(name,lp,args,rp,eq,ini,sem) -> modif_after_mcode sem
   | Ast0.TyDecl(ty,sem) -> modif_after_mcode sem
   | Ast0.Typedef(stg,ty,id,sem) -> modif_after_mcode sem
   | Ast0.DisjDecl(_,decls,_,_) -> List.exists right_declaration decls
   | Ast0.Ddots(dots,_) -> false
   | Ast0.OptDecl(decl) -> right_declaration decl
   | Ast0.UniqueDecl(decl) -> right_declaration decl
+  | Ast0.AsDecl _ -> failwith "not possible"
 
 (* --------------------------------------------------------------------- *)
 (* Top-level code *)
-
+(* These functions seem to be never used
 and left_statement s =
   modif_before s or
   match Ast0.unwrap s with
@@ -240,15 +258,16 @@ and left_statement s =
       (* irrelevant *) false
   | Ast0.Decl(_,decl) -> left_declaration decl
   | Ast0.Seq(lbrace,body,rbrace) -> modif_before_mcode lbrace
-  | Ast0.ExprStatement(exp,sem) -> left_expression exp
-  | Ast0.IfThen(iff,lp,exp,rp,branch1,(info,aft)) -> modif_before_mcode iff
-  | Ast0.IfThenElse(iff,lp,exp,rp,branch1,els,branch2,(info,aft)) ->
+  | Ast0.ExprStatement(Some exp,sem) -> left_expression exp
+  | Ast0.ExprStatement(None,sem) -> modif_before_mcode sem
+  | Ast0.IfThen(iff,lp,exp,rp,branch1,aft) -> modif_before_mcode iff
+  | Ast0.IfThenElse(iff,lp,exp,rp,branch1,els,branch2,aft) ->
       modif_before_mcode iff
-  | Ast0.While(whl,lp,exp,rp,body,(info,aft)) -> modif_before_mcode whl
+  | Ast0.While(whl,lp,exp,rp,body,aft) -> modif_before_mcode whl
   | Ast0.Do(d,body,whl,lp,exp,rp,sem) -> modif_before_mcode d
-  | Ast0.For(fr,lp,e1,sem1,e2,sem2,e3,rp,body,(info,aft)) ->
+  | Ast0.For(fr,lp,first,e2,sem2,e3,rp,body,aft) ->
       modif_before_mcode fr
-  | Ast0.Iterator(nm,lp,args,rp,body,(info,aft)) -> left_ident nm
+  | Ast0.Iterator(nm,lp,args,rp,body,aft) -> left_ident nm
   | Ast0.Switch(switch,lp,exp,rp,lb,decls,cases,rb) ->
       modif_before_mcode switch
   | Ast0.Break(br,sem) -> modif_before_mcode br
@@ -257,6 +276,7 @@ and left_statement s =
   | Ast0.Goto(goto,l,sem) -> modif_before_mcode goto
   | Ast0.Return(ret,sem) -> modif_before_mcode ret
   | Ast0.ReturnExpr(ret,exp,sem) -> modif_before_mcode ret
+  | Ast0.Exec(exec,lang,code,sem) -> modif_before_mcode exec
   | Ast0.MetaStmt(name,pure) -> modif_before_mcode name
   | Ast0.MetaStmtList(name,_) -> modif_before_mcode name
   | Ast0.Disj(_,statement_dots_list,_,_) ->
@@ -271,8 +291,10 @@ and left_statement s =
   | Ast0.Include(inc,s) -> modif_before_mcode inc
   | Ast0.Undef(def,id) -> modif_before_mcode def
   | Ast0.Define(def,id,params,body) -> modif_before_mcode def
+  | Ast0.Pragma(prg,id,body) -> modif_before_mcode prg
   | Ast0.OptStm(re) -> left_statement re
   | Ast0.UniqueStm(re) -> left_statement re
+  | Ast0.AsStmt _ -> failwith "not possible"
 
 and right_statement s =
   modif_after s or
@@ -282,14 +304,14 @@ and right_statement s =
   | Ast0.Decl(_,decl) -> right_declaration decl
   | Ast0.Seq(lbrace,body,rbrace) -> modif_after_mcode rbrace
   | Ast0.ExprStatement(exp,sem) -> modif_after_mcode sem
-  | Ast0.IfThen(iff,lp,exp,rp,branch1,(info,aft)) -> modif_after_mcodekind aft
-  | Ast0.IfThenElse(iff,lp,exp,rp,branch1,els,branch2,(info,aft)) ->
+  | Ast0.IfThen(iff,lp,exp,rp,branch1,(_,aft,_)) -> modif_after_mcodekind aft
+  | Ast0.IfThenElse(iff,lp,exp,rp,branch1,els,branch2,(_,aft,_)) ->
       modif_after_mcodekind aft
-  | Ast0.While(whl,lp,exp,rp,body,(info,aft)) -> modif_after_mcodekind aft
+  | Ast0.While(whl,lp,exp,rp,body,(_,aft,_)) -> modif_after_mcodekind aft
   | Ast0.Do(d,body,whl,lp,exp,rp,sem) -> modif_after_mcode sem
-  | Ast0.For(fr,lp,e1,sem1,e2,sem2,e3,rp,body,(info,aft)) ->
+  | Ast0.For(fr,lp,first,e2,sem2,e3,rp,body,(_,aft,_)) ->
       modif_after_mcodekind aft
-  | Ast0.Iterator(nm,lp,args,rp,body,(info,aft)) ->
+  | Ast0.Iterator(nm,lp,args,rp,body,(_,aft,_)) ->
       modif_after_mcodekind aft
   | Ast0.Switch(switch,lp,exp,rp,lb,decls,cases,rb) -> modif_after_mcode rb
   | Ast0.Break(br,sem) -> modif_after_mcode sem
@@ -298,6 +320,7 @@ and right_statement s =
   | Ast0.Goto(goto,l,sem) -> modif_after_mcode sem
   | Ast0.Return(ret,sem) -> modif_after_mcode sem
   | Ast0.ReturnExpr(ret,exp,sem) -> modif_after_mcode sem
+  | Ast0.Exec(exec,lang,code,sem) -> modif_after_mcode sem
   | Ast0.MetaStmt(name,pure) -> modif_after_mcode name
   | Ast0.MetaStmtList(name,_) -> modif_after_mcode name
   | Ast0.Disj(_,statement_dots_list,_,_) ->
@@ -312,8 +335,11 @@ and right_statement s =
   | Ast0.Include(inc,s) -> modif_after_mcode s
   | Ast0.Undef(def,id) -> right_ident id
   | Ast0.Define(def,id,params,body) -> right_dots right_statement body
+  | Ast0.Pragma(prg,id,body) -> right_pragma body -- not defined, b/c not used
   | Ast0.OptStm(re) -> right_statement re
   | Ast0.UniqueStm(re) -> right_statement re
+  | Ast0.AsStmt _ -> failwith "not possible"
+*)
 
 (* --------------------------------------------------------------------- *)
 
@@ -328,10 +354,10 @@ let rec adding_something s =
     Ast0.MINUS(mc) ->
       (match !mc with
 	(* do better for the common case of replacing a stmt by another one *)
-	([[Ast.StatementTag(s)]],ti) ->
+	((Ast.REPLACEMENT([[Ast.StatementTag(s)]],c)) as old,ti) ->
 	  (match Ast.unwrap s with
 	    Ast.IfThen(_,_,_) -> true (* potentially dangerous *)
-	  | _ -> mc := ([[Ast.StatementTag(s)]],ti); false)
+	  | _ -> mc := (old,ti); false)
       |	(_,_) -> true)
   | Ast0.CONTEXT(mc) ->
       let (text,tinfo1,tinfo2) = !mc in
@@ -348,7 +374,7 @@ and contains_only_minus =
   let mcodekind = function
       Ast0.MINUS(mc) ->
 	(match !mc with
-	  ([],_) -> true
+	  (Ast.NOREPLACEMENT,_) -> true
 	| _ -> false)
     | Ast0.CONTEXT(mc) -> false
     | _ -> false in
@@ -404,10 +430,10 @@ and contains_only_minus =
     | _ -> k e in
 
   V0.flat_combiner bind option_default
-    mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
+    mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
     dots dots dots dots dots dots
-    identifier expression typeC donothing donothing declaration
-    statement case_line donothing
+    identifier expression donothing donothing typeC donothing donothing declaration
+    statement donothing case_line donothing donothing
 
 
 (* needs a special case when there is a Disj or an empty DOTS *)
@@ -430,10 +456,18 @@ let add_braces orig_s =
   let s =
     (Iso_pattern.rebuild_mcode None).VT0.rebuilder_rec_statement orig_s in
   let new_mcodekind =
+    let add_times = Ast.ONE in
     match Ast0.get_mcodekind s with
       Ast0.MINUS(mc) ->
 	let (text,tinfo) = !mc in
-	Ast0.MINUS(ref([Ast.mkToken "{"]::text@[[Ast.mkToken "}"]],tinfo))
+	let inner_text =
+	  match text with
+	    Ast.NOREPLACEMENT -> [[Ast.mkToken "{}"]]
+	  | Ast.REPLACEMENT(anythings,Ast.ONE) ->
+	      [Ast.mkToken "{"]::anythings@[[Ast.mkToken "}"]]
+	  | Ast.REPLACEMENT(anythings,Ast.MANY) ->
+	      failwith "++ not supported when braces must be added" in
+	Ast0.MINUS(ref(Ast.REPLACEMENT(inner_text,Ast.ONE),tinfo))
     | Ast0.CONTEXT(mc) ->
 	let (text,tinfo1,tinfo2) = !mc in
 	let new_text =
@@ -444,16 +478,16 @@ let add_braces orig_s =
 	  match text with
 	    Ast.BEFORE(bef,_) ->
 	      Ast.BEFOREAFTER([Ast.mkToken "{"]::bef,[[Ast.mkToken "}"]],
-			      Ast.ONE)
+			      add_times)
 	  | Ast.AFTER(aft,_) ->
 	      Ast.BEFOREAFTER([[Ast.mkToken "{"]],aft@[[Ast.mkToken "}"]],
-			      Ast.ONE)
+			      add_times)
 	  | Ast.BEFOREAFTER(bef,aft,_) ->
 	      Ast.BEFOREAFTER([Ast.mkToken "{"]::bef,aft@[[Ast.mkToken "}"]],
-			      Ast.ONE)
+			      add_times)
 	  | Ast.NOTHING ->
 	      Ast.BEFOREAFTER([[Ast.mkToken "{"]],[[Ast.mkToken "}"]],
-			      Ast.ONE) in
+			      add_times) in
 	Ast0.CONTEXT(ref(new_text,tinfo1,tinfo2))
     | Ast0.MIXED(mc) ->
 	let (text,tinfo1,tinfo2) = !mc in
@@ -461,16 +495,16 @@ let add_braces orig_s =
 	  match text with
 	    Ast.BEFORE(bef,_) ->
 	      Ast.BEFOREAFTER([Ast.mkToken "{"]::bef,[[Ast.mkToken "}"]],
-			      Ast.ONE)
+			      add_times)
 	  | Ast.AFTER(aft,_) ->
 	      Ast.BEFOREAFTER([[Ast.mkToken "{"]],aft@[[Ast.mkToken "}"]],
-			      Ast.ONE)
+			      add_times)
 	  | Ast.BEFOREAFTER(bef,aft,_) ->
 	      Ast.BEFOREAFTER([Ast.mkToken "{"]::bef,aft@[[Ast.mkToken "}"]],
-			      Ast.ONE)
+			      add_times)
 	  | Ast.NOTHING ->
 	      Ast.BEFOREAFTER([[Ast.mkToken "{"]],[[Ast.mkToken "}"]],
-			      Ast.ONE) in
+			      add_times) in
 	Ast0.MIXED(ref(new_text,tinfo1,tinfo2))
     | _ -> failwith "unexpected plus code" in
   Ast0.set_mcodekind s new_mcodekind;
@@ -491,7 +525,8 @@ let all_minus s =
 
 let rec unchanged_minus s =
   match Ast0.get_mcodekind s with
-    Ast0.MINUS(mc) -> (match !mc with ([],_) -> true | _ -> false)
+    Ast0.MINUS(mc) ->
+      (match !mc with (Ast.NOREPLACEMENT,_) -> true | _ -> false)
   | _ -> false
 
 let rec do_branch s =
@@ -523,20 +558,20 @@ let rec statement dots_before dots_after s =
 	Ast0.set_dots_bef_aft s (Ast0.DroppingBetweenDots(with_braces)))
       else if adding_something s
       then
-	(let with_braces = add_braces s in
-	Ast0.set_dots_bef_aft s (Ast0.AddingBetweenDots(with_braces)))
+	let with_braces = add_braces s in
+	Ast0.set_dots_bef_aft s (Ast0.AddingBetweenDots(with_braces))
       else s
     else s in
 
   match Ast0.unwrap s with
-    Ast0.FunDecl(x,fninfo,name,lp,params,rp,lbrace,body,rbrace) ->
+    Ast0.FunDecl(x,fninfo,name,lp,params,va,rp,lbrace,body,rbrace,y) ->
       (* true for close brace, because that represents any way we can
 	 exit the function, which is not necessarily followed by an explicit
 	 close brace. *)
       Ast0.rewrap s
-	(Ast0.FunDecl(x,fninfo,name,lp,params,rp,lbrace,
+	(Ast0.FunDecl(x,fninfo,name,lp,params,va,rp,lbrace,
 		      statement_dots false true body,
-		      rbrace))
+		      rbrace,y))
   | Ast0.Decl(_,_) -> s
   | Ast0.Seq(lbrace,body,rbrace) ->
       Ast0.rewrap s
@@ -561,10 +596,10 @@ let rec statement dots_before dots_after s =
       do_one
 	(Ast0.rewrap s
 	   (Ast0.Do(d,statement false false body,whl,lp,exp,rp,sem)))
-  | Ast0.For(fr,lp,e1,sem1,e2,sem2,e3,rp,body,x) ->
+  | Ast0.For(fr,lp,first,e2,sem2,e3,rp,body,x) ->
       do_one
 	(Ast0.rewrap s
-	   (Ast0.For(fr,lp,e1,sem1,e2,sem2,e3,rp,
+	   (Ast0.For(fr,lp,first,e2,sem2,e3,rp,
 		     statement false false body,x)))
   | Ast0.Iterator(nm,lp,args,rp,body,x) ->
       do_one
@@ -584,6 +619,7 @@ let rec statement dots_before dots_after s =
   | Ast0.Goto(goto,l,sem) -> do_one s
   | Ast0.Return(ret,sem) -> do_one s
   | Ast0.ReturnExpr(ret,exp,sem) -> do_one s
+  | Ast0.Exec(exec,lang,code,sem) -> do_one s
   | Ast0.MetaStmt(name,_) -> do_one s
   | Ast0.MetaStmtList(name,_) -> do_one s
   | Ast0.Disj(starter,statement_dots_list,mids,ender) ->
@@ -608,12 +644,14 @@ let rec statement dots_before dots_after s =
   | Ast0.Include(inc,string) -> s (* doesn't affect the need for braces *)
   | Ast0.Undef(def,id) -> s (* same as include *)
   | Ast0.Define(def,id,params,body) -> s (* same as include *)
+  | Ast0.Pragma(prg,id,body) -> s (* same as include *)
   | Ast0.OptStm(re) ->
       Ast0.rewrap s
 	(Ast0.OptStm(statement dots_before dots_after re))
   | Ast0.UniqueStm(re) ->
       Ast0.rewrap s
 	(Ast0.UniqueStm(statement dots_before dots_after re))
+  | Ast0.AsStmt _ -> failwith "not possible"
 
 and case_line c =
   Ast0.rewrap c
@@ -651,7 +689,7 @@ and statement_dots dots_before dots_after d =
 let top_level t =
   Ast0.rewrap t
     (match Ast0.unwrap t with
-      Ast0.DECL(stmt_dots) -> Ast0.DECL(statement true true stmt_dots)
+      Ast0.NONDECL(stmt_dots) -> Ast0.NONDECL(statement true true stmt_dots)
     | Ast0.CODE(stmt_dots) -> Ast0.CODE(statement_dots true true stmt_dots)
     | t -> t)
 
