@@ -1,5 +1,9 @@
 (*
- * Copyright 2005-2010, Ecole des Mines de Nantes, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
+ * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
+ * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
  * This file is part of Coccinelle.
  *
@@ -20,6 +24,7 @@
  *)
 
 
+# 0 "./test_exps.ml"
 module Ast = Ast_cocci
 module Ast0 = Ast0_cocci
 module V0 = Visitor_ast0
@@ -49,8 +54,8 @@ let set_test_poss =
       Ast0.CondExpr(e1,q,e2,c,e3) ->
 	Ast0.rewrap e (Ast0.CondExpr(process_exp e1,q,e2,c,e3))
     | Ast0.Binary(e1,op,e2) ->
-	(match Ast0.unwrap_mcode op with
-	  Ast.Logical(Ast.AndLog) | Ast.Logical(Ast.OrLog) ->
+	(match Ast0.unwrap op with
+	  Ast0.Logical op' when (let op''=Ast0.unwrap_mcode op' in op''=Ast.AndLog || op''=Ast.OrLog) ->
 	    Ast0.rewrap e (Ast0.Binary(process_exp e1,op,process_exp e2))
 	| _ -> e)
     | Ast0.Unary(e1,op) ->
@@ -60,8 +65,8 @@ let set_test_poss =
     | _ -> e in
 
   let process_wc = function
-      Ast0.WhenNotTrue(e) -> Ast0.WhenNotTrue(process_exp e)
-    | Ast0.WhenNotFalse(e) -> Ast0.WhenNotFalse(process_exp e)
+      Ast0.WhenNotTrue(w,ee,e) -> Ast0.WhenNotTrue(w,ee,process_exp e)
+    | Ast0.WhenNotFalse(w,ee,e) -> Ast0.WhenNotFalse(w,ee,process_exp e)
     | wc -> wc in
 
   let statement r k s =
@@ -75,9 +80,9 @@ let set_test_poss =
 	Ast0.rewrap s (Ast0.While(i,lp,process_exp e,rp,s1,aft))
     | Ast0.Do(d,s1,w,lp,e,rp,sc) ->
 	Ast0.rewrap s (Ast0.Do(d,s1,w,lp,process_exp e,rp,sc))
-    | Ast0.For(f,lp,e1,sc1,Some e2,sc2,e3,rp,s1,aft) ->
+    | Ast0.For(f,lp,first,Some e2,sc2,e3,rp,s1,aft) ->
 	Ast0.rewrap s
-	  (Ast0.For(f,lp,e1,sc1,Some (process_exp e2),sc2,e3,rp,s1,aft))
+	  (Ast0.For(f,lp,first,Some (process_exp e2),sc2,e3,rp,s1,aft))
     | Ast0.Dots(d,wc) ->
 	Ast0.rewrap s (Ast0.Dots(d,List.map process_wc wc))
     | Ast0.Nest(l,s1,r,wc,m) ->
