@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,4 +24,8 @@
  *)
 
 
-val top_level : Ast0_cocci.rule -> Ast0_cocci.rule
+# 0 "./top_level.mli"
+val top_level :
+    bool (* true if must be code *) -> Ast0_cocci.rule -> Ast0_cocci.rule
+
+val clean : Ast0_cocci.rule -> Ast0_cocci.rule
