@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./type_cocci.ml"
 (* for metavariables in general, but here because needed for metatypes *)
 type inherited = bool (* true if inherited *)
 type keep_binding = Unitary (* need no info *)
@@ -36,6 +39,7 @@ type typeC =
   | Pointer         of typeC
   | FunctionPointer of typeC (* only return type *)
   | Array           of typeC (* drop size info *)
+  | Decimal         of name * name
   | EnumName        of name
   | StructUnionName of structUnion * name
   | TypeName        of string
@@ -45,13 +49,15 @@ type typeC =
 and name =
     NoName
   | Name of string
+  | Num of string
   | MV of meta_name * keep_binding * inherited
 
 and tagged_string = string
 
-and baseType = VoidType | CharType | ShortType | IntType | DoubleType
-| FloatType | LongType | LongLongType | BoolType
-| SizeType | SSizeType | PtrDiffType
+and baseType = VoidType | CharType | ShortType | ShortIntType | IntType
+| DoubleType | LongDoubleType | FloatType
+| LongType | LongIntType | LongLongType | LongLongIntType
+| SizeType | SSizeType | PtrDiffType | BoolType
 
 and structUnion = Struct | Union
 
@@ -71,6 +77,8 @@ let rec type2c = function
   | Pointer(ty) -> (type2c ty) ^ "*"
   | FunctionPointer(ty) -> (type2c ty) ^ "(*)(...)"
   | Array(ty) -> (type2c ty) ^ "[] "
+  | Decimal(e1,e2) ->
+      Printf.sprintf "decimal(%s,%s) " (print_name e1) (print_name e2)
   | EnumName(name) -> "enum " ^ (print_name name)
   | StructUnionName(kind,name) -> (structUnion kind) ^ (print_name name)
   | TypeName(name) -> name ^ " "
@@ -91,16 +99,21 @@ and print_name = function
     NoName -> ""
   | MV ((_,name),_,_) -> name ^ " "
   | Name name -> name ^ " "
+  | Num n -> n ^ " "
 
 and baseType = function
     VoidType -> "void "
   | CharType -> "char "
   | ShortType -> "short "
+  | ShortIntType -> "short int "
   | IntType -> "int "
   | DoubleType -> "double "
+  | LongDoubleType -> "long double "
   | FloatType -> "float "
   | LongType -> "long "
+  | LongIntType -> "long int "
   | LongLongType -> "long long "
+  | LongLongIntType -> "long long int "
   | BoolType -> "bool "
   | SizeType -> "size_t "
   | SSizeType -> "ssize_t "
