@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./type_infer.ml"
 module T = Type_cocci
 module Ast = Ast_cocci
 module Ast0 = Ast0_cocci
@@ -45,6 +48,7 @@ let err wrapped ty s =
 type id = Id of string | Meta of Ast.meta_name
 
 let int_type = T.BaseType(T.IntType)
+let void_type = T.BaseType(T.VoidType)
 let bool_type = T.BaseType(T.BoolType)
 let char_type = T.BaseType(T.CharType)
 let float_type = T.BaseType(T.FloatType)
@@ -101,10 +105,10 @@ let rec propagate_types env =
 
   let ident r k i =
     match Ast0.unwrap i with
-      Ast0.Id(id) ->
+      Ast0.Id((s, _, _, _, _, _) as id) ->
 	(try Some(List.assoc (Id(Ast0.unwrap_mcode id)) env)
 	with Not_found -> None)
-    | Ast0.MetaId(id,_,_) ->
+    | Ast0.MetaId(id,_,_,_) ->
 	(try Some(List.assoc (Meta(Ast0.unwrap_mcode id)) env)
 	with Not_found -> None)
     | Ast0.DisjId(_,id_list,_,_) ->
@@ -115,17 +119,21 @@ let rec propagate_types env =
 	| Some t ->
 	    List.iter (function i -> Ast0.set_type i (Some t)) id_list;
 	    Some t)
+    | Ast0.AsIdent _ -> failwith "not possible"
     | _ -> k i in
 
   let strip_cv = function
       Some (T.ConstVol(_,t)) -> Some t
     | t -> t in
 
+  let ast0_type_to_type = Ast0.ast0_type_to_type false in
+
   (* types that might be integer types.  should char be allowed? *)
   let rec is_int_type = function
       T.BaseType(T.IntType)
     | T.BaseType(T.LongType)
     | T.BaseType(T.ShortType)
+    | T.BaseType(T.SizeType)
     | T.MetaType(_,_,_)
     | T.TypeName _
     | T.EnumName _
@@ -141,13 +149,16 @@ let rec propagate_types env =
 	  Ast0.Ident(id) -> Ast0.set_type e res; res
 	| Ast0.Constant(const) ->
 	    (match Ast0.unwrap_mcode const with
-		 Ast.String(_) -> Some (T.Pointer(char_type))
+	         Ast.String(_) -> Some (T.Pointer(char_type))
 	       | Ast.Char(_) -> Some (char_type)
 	       | Ast.Int(_) -> Some (int_type)
-	       | Ast.Float(_) ->  Some (float_type))
-              (* pad: note that in C can do either ptr(...) or ( *ptr)(...)
-               * so I am not sure this code is enough.
-               *)
+	       | Ast.Float(_) ->  Some (float_type)
+	       | Ast.DecimalConst(_,l,p) ->
+		   Some (T.Decimal(T.Num l, T.Num p)))
+        (* pad: note that in C can do either ptr(...) or ( *ptr)(...)
+         * so I am not sure this code is enough.
+         *)
+	| Ast0.StringConstant _ -> Some (T.Array(char_type))
 	| Ast0.FunCall(fn,lp,args,rp) ->
 	    (match Ast0.get_type fn with
 		 Some (T.FunctionPointer(ty)) -> Some ty
@@ -164,6 +175,7 @@ let rec propagate_types env =
 	| Ast0.Assignment(exp1,op,exp2,_) ->
 	    let ty = lub_type (Ast0.get_type exp1) (Ast0.get_type exp2) in
 	      Ast0.set_type exp1 ty; Ast0.set_type exp2 ty; ty
+	| Ast0.Sequence(exp1,op,exp2) -> Ast0.get_type exp2
 	| Ast0.CondExpr(exp1,why,Some exp2,colon,exp3) ->
 	    let ty = lub_type (Ast0.get_type exp2) (Ast0.get_type exp3) in
 	      Ast0.set_type exp2 ty; Ast0.set_type exp3 ty; ty
@@ -176,6 +188,7 @@ let rec propagate_types env =
 		   (match Ast0.get_type exp with
 			None -> Some (T.Pointer(T.Unknown))
 		      |	Some t -> Some (T.Pointer(t)))
+	       | Ast.GetRefLabel -> Some (T.Pointer(void_type))
 	       | Ast.DeRef ->
 		   (match Ast0.get_type exp with
 			Some (T.Pointer(t)) -> Some t
@@ -200,11 +213,12 @@ let rec propagate_types env =
 	      | (t1,t2) ->
 		  let ty = lub_type t1 t2 in
 		    Ast0.set_type exp1 ty; Ast0.set_type exp2 ty; ty in
-	      (match Ast0.unwrap_mcode op with
-		   Ast.Arith(op) -> same_type (ty1, ty2)
-		 | Ast.Logical(Ast.AndLog) | Ast.Logical(Ast.OrLog) ->
+	      (match Ast0.unwrap op with
+		 Ast0.Arith(op) -> same_type (ty1, ty2)
+                 | Ast0.MetaBinary _ -> same_type (ty1, ty2)
+		 | Ast0.Logical(op') when (let op''=Ast0.unwrap_mcode op' in op''=Ast.AndLog || op''=Ast.OrLog) ->
 		     Some(bool_type)
-		 | Ast.Logical(op) ->
+		 | Ast0.Logical(op) ->
 		     let ty = lub_type ty1 ty2 in
 		     Ast0.set_type exp1 ty; Ast0.set_type exp2 ty;
 		     Some(bool_type))
@@ -229,7 +243,8 @@ let rec propagate_types env =
 	    (match strip_cv (Ast0.get_type exp) with
 		 None -> None
 	       | Some (T.StructUnionName(_,_)) -> None
-	       | Some (T.TypeName(_)) -> None
+	       | Some (T.TypeName(s)) ->
+			  None
 	       | Some (T.MetaType(_,_,_)) -> None
 	       | Some x -> err exp x "non-structure type in field ref")
 	| Ast0.RecordPtAccess(exp,ar,field) ->
@@ -239,19 +254,23 @@ let rec propagate_types env =
 		   (match strip_cv (Some t) with
 		      | Some (T.Unknown) -> None
 		      | Some (T.MetaType(_,_,_)) -> None
-		      | Some (T.TypeName(_)) -> None
-		      | Some (T.StructUnionName(_,_)) -> None
+		      | Some (T.TypeName(s)) -> 
+			  None
+		      | Some (T.StructUnionName(s,t)) -> 
+			  None
 		      | Some x ->
 			  err exp (T.Pointer(t))
 			    "non-structure pointer type in field ref"
 		      |	_ -> failwith "not possible")
 	       | Some (T.MetaType(_,_,_)) -> None
-	       | Some (T.TypeName(_)) -> None
+	       | Some (T.TypeName(s)) ->
+		   None
 	       | Some x -> err exp x "non-structure pointer type in field ref")
-	| Ast0.Cast(lp,ty,rp,exp) -> Some(Ast0.ast0_type_to_type ty)
+	| Ast0.Cast(lp,ty,rp,exp) -> Some(ast0_type_to_type ty)
 	| Ast0.SizeOfExpr(szf,exp) -> Some(int_type)
 	| Ast0.SizeOfType(szf,lp,ty,rp) -> Some(int_type)
 	| Ast0.TypeExp(ty) -> None
+	| Ast0.Constructor(lp,ty,rp,init) -> Some(ast0_type_to_type ty)
 	| Ast0.MetaErr(name,_,_) -> None
 	| Ast0.MetaExpr(name,_,Some [ty],_,_) -> Some ty
 	| Ast0.MetaExpr(name,_,ty,_,_) -> None
@@ -267,35 +286,37 @@ let rec propagate_types env =
 		     Some t)
 	| Ast0.NestExpr(starter,expr_dots,ender,None,multi) ->
 	    let _ = r.VT0.combiner_rec_expression_dots expr_dots in None
-	| Ast0.NestExpr(starter,expr_dots,ender,Some e,multi) ->
+	| Ast0.NestExpr(starter,expr_dots,ender,Some (_,_,e),multi) ->
 	    let _ = r.VT0.combiner_rec_expression_dots expr_dots in
 	    let _ = r.VT0.combiner_rec_expression e in None
 	| Ast0.Edots(_,None) | Ast0.Ecircles(_,None) | Ast0.Estars(_,None) ->
 	    None
-	| Ast0.Edots(_,Some e) | Ast0.Ecircles(_,Some e)
-	| Ast0.Estars(_,Some e) ->
+	| Ast0.Edots(_,Some (_,_,e)) | Ast0.Ecircles(_,Some (_,_,e))
+	| Ast0.Estars(_,Some (_,_,e)) ->
 	    let _ = r.VT0.combiner_rec_expression e in None
 	| Ast0.OptExp(exp) -> Ast0.get_type exp
-	| Ast0.UniqueExp(exp) -> Ast0.get_type exp in
+	| Ast0.UniqueExp(exp) -> Ast0.get_type exp
+	| Ast0.AsExpr _ -> failwith "not possible" in
       Ast0.set_type e ty;
       ty in
 
   let rec strip id =
     match Ast0.unwrap id with
       Ast0.Id(name)                -> [Id(Ast0.unwrap_mcode name)]
-    | Ast0.MetaId(name,_,_)        -> [Meta(Ast0.unwrap_mcode name)]
+    | Ast0.MetaId(name,_,_,_)      -> [Meta(Ast0.unwrap_mcode name)]
     | Ast0.MetaFunc(name,_,_)      -> [Meta(Ast0.unwrap_mcode name)]
     | Ast0.MetaLocalFunc(name,_,_) -> [Meta(Ast0.unwrap_mcode name)]
     | Ast0.DisjId(_,id_list,_,_)   -> List.concat (List.map strip id_list)
     | Ast0.OptIdent(id)            -> strip id
-    | Ast0.UniqueIdent(id)         -> strip id in
+    | Ast0.UniqueIdent(id)         -> strip id
+    | Ast0.AsIdent _ -> failwith "not possible" in
 
   let process_whencode notfn allfn exp = function
-      Ast0.WhenNot(x) -> let _ = notfn x in ()
-    | Ast0.WhenAlways(x) -> let _ = allfn x in ()
-    | Ast0.WhenModifier(_) -> ()
-    | Ast0.WhenNotTrue(x) -> let _ = exp x in ()
-    | Ast0.WhenNotFalse(x) -> let _ = exp x in () in
+      Ast0.WhenNot(_,_,x) -> let _ = notfn x in ()
+    | Ast0.WhenAlways(_,_,x) -> let _ = allfn x in ()
+    | Ast0.WhenModifier _ -> ()
+    | Ast0.WhenNotTrue(_,_,x) -> let _ = exp x in ()
+    | Ast0.WhenNotFalse(_,_,x) -> let _ = exp x in () in
 
   (* assume that all of the declarations are at the beginning of a statement
      list, which is required by C, but not actually required by the cocci
@@ -330,22 +351,27 @@ let rec propagate_types env =
       Ast0.MetaDecl(_,_) | Ast0.MetaField(_,_)
     | Ast0.MetaFieldList(_,_,_) -> []
     | Ast0.Init(_,ty,id,_,exp,_) ->
-	let _ =
-	  (propagate_types env).VT0.combiner_rec_initialiser exp in
-	let ty = Ast0.ast0_type_to_type ty in
+	let _ = (propagate_types env).VT0.combiner_rec_initialiser exp in
+	let ty = ast0_type_to_type ty in
 	List.map (function i -> (i,ty)) (strip id)
     | Ast0.UnInit(_,ty,id,_) ->
-	let ty = Ast0.ast0_type_to_type ty in
+	let ty = ast0_type_to_type ty in
 	List.map (function i -> (i,ty)) (strip id)
+    | Ast0.FunProto(fi,nm,lp,params,va,rp,sem) -> []
     | Ast0.MacroDecl(_,_,_,_,_) -> []
+    | Ast0.MacroDeclInit(_,_,_,_,_,exp,_) ->
+        let _ = (propagate_types env).VT0.combiner_rec_initialiser exp in
+	[]
     | Ast0.TyDecl(ty,_) -> []
               (* pad: should handle typedef one day and add a binding *)
-    | Ast0.Typedef(_,_,_,_) -> []
+    | Ast0.Typedef((a,_,_,_,_,_),b,c,(d,_,_,_,_,_)) -> 
+	[]
     | Ast0.DisjDecl(_,disjs,_,_) ->
 	List.concat(List.map (process_decl env) disjs)
     | Ast0.Ddots(_,_) -> [] (* not in a statement list anyway *)
     | Ast0.OptDecl(decl) -> process_decl env decl
-    | Ast0.UniqueDecl(decl) -> process_decl env decl in
+    | Ast0.UniqueDecl(decl) -> process_decl env decl
+    | Ast0.AsDecl _ -> failwith "not possible" in
 
   let statement_dots r k d =
     match Ast0.unwrap d with
@@ -372,22 +398,37 @@ let rec propagate_types env =
 
   let statement r k s =
     match Ast0.unwrap s with
-      Ast0.FunDecl(_,fninfo,name,lp,params,rp,lbrace,body,rbrace) ->
+      Ast0.FunDecl(_,fninfo,name,lp,params,va,rp,lbrace,body,rbrace,_) ->
 	let rec get_binding p =
 	  match Ast0.unwrap p with
 	    Ast0.Param(ty,Some id) ->
-	      let ty = Ast0.ast0_type_to_type ty in
+	      let ty = ast0_type_to_type ty in
 	      List.map (function i -> (i,ty)) (strip id)
 	  | Ast0.OptParam(param) -> get_binding param
+	  | Ast0.AsParam(param,e) -> get_binding param
 	  | _ -> [] in
 	let fenv = List.concat (List.map get_binding (Ast0.undots params)) in
 	(propagate_types (fenv@env)).VT0.combiner_rec_statement_dots body
     | Ast0.IfThen(_,_,exp,_,_,_) | Ast0.IfThenElse(_,_,exp,_,_,_,_,_)
-    | Ast0.While(_,_,exp,_,_,_) | Ast0.Do(_,_,_,_,exp,_,_)
-    | Ast0.For(_,_,_,_,Some exp,_,_,_,_,_) ->
+    | Ast0.While(_,_,exp,_,_,_) | Ast0.Do(_,_,_,_,exp,_,_) ->
 	let _ = k s in
-	post_bool exp;
-	None
+        post_bool exp;
+        None
+    | Ast0.For(a,b,first,exp,c,d,e,f,g) ->
+	(match Ast0.unwrap first with
+	  Ast0.ForExp _ ->
+	    (match exp with
+	      Some exp ->
+		let _ = k s in
+		post_bool exp;
+		None
+	    | None -> k s)
+	| Ast0.ForDecl (_,decl) ->
+	(* not super elegant..., reuses a ; (d) *)
+	    let newenv = (process_decl env decl)@env in
+	    let dummy = Ast0.rewrap first (Ast0.ForExp (None,c)) in
+	    (propagate_types newenv).VT0.combiner_rec_statement
+	      (Ast0.rewrap s (Ast0.For(a,b,dummy,exp,c,d,e,f,g))))
     | Ast0.Switch(_,_,exp,_,_,decls,cases,_) ->
 	let senv = process_statement_list r env (Ast0.undots decls) in
 	let res =
