@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./unify_ast.ml"
 (* --------------------------------------------------------------------- *)
 (* Given two patterns, A and B, determine whether B can match any matched
 subterms of A.  For simplicity, this doesn't maintain an environment; it
@@ -38,6 +41,22 @@ let return b = if b then MAYBE else NO
 
 let unify_mcode (x,_,_,_) (y,_,_,_) = x = y
 
+let unify_assignOp_mcode op1 op2 =
+  match (Ast.unwrap op1, Ast.unwrap op2) with
+    | (Ast.SimpleAssign op1', Ast.SimpleAssign op2') -> unify_mcode op1' op2'
+    | (Ast.OpAssign op1', Ast.OpAssign op2') -> unify_mcode op1' op2'
+    | (Ast.MetaAssign(mv1,_,_,_), Ast.MetaAssign(mv2,_,_,_)) ->
+      unify_mcode mv1 mv2
+    | _ -> false
+
+let unify_binaryOp_mcode op1 op2 =
+  match (Ast.unwrap op1, Ast.unwrap op2) with
+    | (Ast.Arith op1', Ast.Arith op2') -> unify_mcode op1' op2'
+    | (Ast.Logical op1', Ast.Logical op2') -> unify_mcode op1' op2'
+    | (Ast.MetaBinary(mv1,_,_,_), Ast.MetaBinary(mv2,_,_,_)) ->
+      unify_mcode mv1 mv2
+    | _ -> false
+
 let ret_unify_mcode a b = return (unify_mcode a b)
 
 let unify_option f t1 t2 =
@@ -77,7 +96,7 @@ let unify_lists fn dfn la lb =
     | (cura::resta,curb::restb) ->
 	(match fn cura curb with
 	  MAYBE -> loop (resta,restb)
-	| NO -> if dfn cura or dfn curb then MAYBE else NO) in
+	| NO -> if dfn cura || dfn curb then MAYBE else NO) in
   loop (la,lb)
 
 let unify_dots fn dfn d1 d2 =
@@ -117,6 +136,16 @@ let idots e =
     Ast.Idots(_,_) -> true
   | _ -> false
 
+let strdots e =
+  match Ast.unwrap e with
+    Ast.Strdots(_) -> true
+  | _ -> false
+
+let ecdots e =
+  match Ast.unwrap e with
+    Ast.ExecDots(_) -> true
+  | _ -> false
+
 (* --------------------------------------------------------------------- *)
 (* Identifier *)
 
@@ -131,6 +160,13 @@ let rec unify_ident i1 i2 =
   | (_,Ast.MetaFunc(_,_,_,_))
   | (_,Ast.MetaLocalFunc(_,_,_,_)) -> return true
 
+  | (Ast.AsIdent(id1,asid1),_) ->
+      disjunct_all_bindings
+	(List.map (function x -> unify_ident x i2) [id1;asid1])
+  | (_,Ast.AsIdent(id2,asid2)) ->
+      disjunct_all_bindings
+	(List.map (function x -> unify_ident x i1) [id2;asid2])
+
   | (Ast.DisjId(i1),_) ->
       disjunct_all_bindings (List.map (function x -> unify_ident x i2) i1)
   | (_,Ast.DisjId(i2)) ->
@@ -148,14 +184,18 @@ and unify_expression e1 e2 =
   match (Ast.unwrap e1,Ast.unwrap e2) with
     (Ast.Ident(i1),Ast.Ident(i2)) -> unify_ident i1 i2
   | (Ast.Constant(c1),Ast.Constant(c2))-> return (unify_mcode c1 c2)
+  | (Ast.StringConstant(lq1,str1,rq1),Ast.StringConstant(lq2,str2,rq2)) ->
+      unify_dots unify_string_fragment strdots str1 str2
   | (Ast.FunCall(f1,lp1,args1,rp1),Ast.FunCall(f2,lp2,args2,rp2)) ->
       conjunct_bindings
 	(unify_expression f1 f2)
 	(unify_dots unify_expression edots args1 args2)
   | (Ast.Assignment(l1,op1,r1,_),Ast.Assignment(l2,op2,r2,_)) ->
-      if unify_mcode op1 op2
+      if unify_assignOp_mcode op1 op2
       then conjunct_bindings (unify_expression l1 l2) (unify_expression r1 r2)
       else return false
+  | (Ast.Sequence(l1,_,r1),Ast.Sequence(l2,_,r2)) ->
+      conjunct_bindings (unify_expression l1 l2) (unify_expression r1 r2)
   | (Ast.CondExpr(tst1,q1,thn1,c1,els1),Ast.CondExpr(tst2,q2,thn2,c2,els2)) ->
       conjunct_bindings (unify_expression tst1 tst2)
 	(conjunct_bindings (unify_option unify_expression thn1 thn2)
@@ -167,7 +207,7 @@ and unify_expression e1 e2 =
   | (Ast.Unary(e1,op1),Ast.Unary(e2,op2)) ->
       if unify_mcode op1 op2 then unify_expression e1 e2 else return false
   | (Ast.Binary(l1,op1,r1),Ast.Binary(l2,op2,r2)) ->
-      if unify_mcode op1 op2
+      if unify_binaryOp_mcode op1 op2
       then conjunct_bindings (unify_expression l1 l2) (unify_expression r1 r2)
       else return false
   | (Ast.ArrayAccess(ar1,lb1,e1,rb1),Ast.ArrayAccess(ar2,lb2,e2,rb2)) ->
@@ -183,6 +223,8 @@ and unify_expression e1 e2 =
   | (Ast.SizeOfType(szf1,lp1,ty1,rp1),Ast.SizeOfType(szf2,lp2,ty2,rp2)) ->
       unify_fullType ty1 ty2
   | (Ast.TypeExp(ty1),Ast.TypeExp(ty2)) -> unify_fullType ty1 ty2
+  | (Ast.Constructor(lp1,ty1,rp1,i1),Ast.Constructor(lp2,ty2,rp2,i2)) ->
+      conjunct_bindings (unify_fullType ty1 ty2) (unify_initialiser i1 i2)
   | (Ast.Paren(lp1,e1,rp1),Ast.Paren(lp2,e2,rp2)) ->
       unify_expression e1 e2
 
@@ -193,6 +235,13 @@ and unify_expression e1 e2 =
   | (_,Ast.MetaExpr(_,_,_,_,_,_))
   | (_,Ast.MetaExprList(_,_,_,_)) -> return true
 
+  | (Ast.AsExpr(exp1,asexp1),_) ->
+      disjunct_all_bindings
+	(List.map (function x -> unify_expression x e2) [exp1;asexp1])
+  | (_,Ast.AsExpr(exp2,asexp2)) ->
+      disjunct_all_bindings
+	(List.map (function x -> unify_expression x e1) [exp2;asexp2])
+
   | (Ast.EComma(cm1),Ast.EComma(cm2)) -> return true
 
   | (Ast.DisjExpr(e1),_) ->
@@ -213,15 +262,42 @@ and unify_expression e1 e2 =
   | (_,Ast.UniqueExp(_)) -> failwith "unsupported expression"
   | _ -> return false
 
+(* --------------------------------------------------------------------- *)
+(* Strings *)
+
+and unify_string_fragment e1 e2 =
+  match (Ast.unwrap e1,Ast.unwrap e2) with
+    (Ast.ConstantFragment(str1),Ast.ConstantFragment(str2)) ->
+      return (unify_mcode str1 str2)
+  | (Ast.FormatFragment(pct1,fmt1),Ast.FormatFragment(pct2,fmt2)) ->
+      unify_string_format fmt1 fmt2
+  | (Ast.Strdots(dots1),Ast.Strdots(dots2)) -> return true
+  | (Ast.MetaFormatList(pct,name,len,_,_),_)
+  | (_,Ast.MetaFormatList(pct,name,len,_,_)) -> return true
+  | _ -> return false
+
+and unify_string_format e1 e2 =
+  match (Ast.unwrap e1,Ast.unwrap e2) with
+    (Ast.ConstantFormat(str1), Ast.ConstantFormat(str2)) ->
+      return (unify_mcode str1 str2)
+  | (Ast.MetaFormat(name,_,_,_),_)
+  | (_,Ast.MetaFormat(name,_,_,_)) -> return true
+
 (* --------------------------------------------------------------------- *)
 (* Types *)
 
 and unify_fullType ft1 ft2 =
   match (Ast.unwrap ft1,Ast.unwrap ft2) with
-    (Ast.Type(cv1,ty1),Ast.Type(cv2,ty2)) ->
+    (Ast.Type(_,cv1,ty1),Ast.Type(_,cv2,ty2)) ->
       if bool_unify_option unify_mcode cv1 cv2
       then unify_typeC ty1 ty2
       else return false
+  | (Ast.AsType(ty1,asty1),_) ->
+      disjunct_all_bindings
+	(List.map (function x -> unify_fullType x ft2) [ty1;asty1])
+  | (_,Ast.AsType(ty2,asty2)) ->
+      disjunct_all_bindings
+	(List.map (function x -> unify_fullType x ft1) [ty2;asty2])
   | (Ast.DisjType(ft1),_) ->
       disjunct_all_bindings (List.map (function x -> unify_fullType x ft2) ft1)
   | (_,Ast.DisjType(ft2)) ->
@@ -253,17 +329,14 @@ and unify_typeC t1 t2 =
 	 conjunct_bindings (unify_fullType tya tyb)
 	   (unify_dots unify_parameterTypeDef pdots paramsa paramsb)
        else return false
-  | (Ast.FunctionType(_,tya,lp1a,paramsa,rp1a),
-     Ast.FunctionType(_,tyb,lp1b,paramsb,rp1b)) ->
-       if List.for_all2 unify_mcode [lp1a;rp1a] [lp1b;rp1b]
-       then
-	 conjunct_bindings (unify_option unify_fullType tya tyb)
-	   (unify_dots unify_parameterTypeDef pdots paramsa paramsb)
-       else return false
-  | (Ast.FunctionType _ , _) -> failwith "not supported"
   | (Ast.Array(ty1,lb1,e1,rb1),Ast.Array(ty2,lb2,e2,rb2)) ->
       conjunct_bindings
 	(unify_fullType ty1 ty2) (unify_option unify_expression e1 e2)
+  | (Ast.Decimal(dec1,lp1,len1,comma1,prec_opt1,rp1),
+     Ast.Decimal(dec2,lp2,len2,comma2,prec_opt2,rp2)) ->
+       conjunct_bindings
+	(unify_expression len1 len2)
+	 (unify_option unify_expression prec_opt1 prec_opt2)
   | (Ast.EnumName(s1,Some ts1),Ast.EnumName(s2,Some ts2)) ->
       if unify_mcode s1 s2 then unify_ident ts1 ts2 else return false
   | (Ast.EnumName(s1,None),Ast.EnumName(s2,None)) ->
@@ -278,7 +351,7 @@ and unify_typeC t1 t2 =
   | (Ast.StructUnionDef(ty1,lb1,decls1,rb1),
      Ast.StructUnionDef(ty2,lb2,decls2,rb2)) ->
        conjunct_bindings (unify_fullType ty1 ty2)
-	 (unify_dots unify_declaration ddots decls1 decls2)
+	 (unify_dots unify_annotated_decl ddots decls1 decls2)
   | (Ast.TypeName(t1),Ast.TypeName(t2)) -> return (unify_mcode t1 t2)
 
   | (Ast.MetaType(_,_,_),_)
@@ -306,10 +379,29 @@ and unify_declaration d1 d2 =
       if bool_unify_option unify_mcode stg1 stg2
       then conjunct_bindings (unify_fullType ft1 ft2) (unify_ident id1 id2)
       else return false
+  | (Ast.FunProto(fi1,nm1,lp1,params1,va1,rp1,sem1),
+     Ast.FunProto(fi2,nm2,lp2,params2,va2,rp2,sem2)) ->
+       let l1 = match va1 with
+         | None -> [lp1;rp1]
+         | Some (c1,e1) -> [lp1;c1;e1;rp1] in
+       let l2 = match va2 with
+         | None -> [lp2;rp2]
+         | Some (c2,e2) -> [lp2;c2;e2;rp2] in
+       if List.for_all2 unify_mcode l1 l2
+       then
+         conjunct_bindings (unify_fninfo fi1 fi2)
+	   (conjunct_bindings (unify_ident nm1 nm2)
+	      (unify_dots unify_parameterTypeDef pdots params1 params2))
+       else return false
   | (Ast.MacroDecl(n1,lp1,args1,rp1,sem1),
      Ast.MacroDecl(n2,lp2,args2,rp2,sem2)) ->
        conjunct_bindings (unify_ident n1 n2)
 	 (unify_dots unify_expression edots args1 args2)
+  | (Ast.MacroDeclInit(n1,lp1,args1,rp1,eq1,ini1,sem1),
+     Ast.MacroDeclInit(n2,lp2,args2,rp2,eq2,ini2,sem2)) ->
+       conjunct_bindings (unify_ident n1 n2)
+	 (conjunct_bindings (unify_dots unify_expression edots args1 args2)
+	    (unify_initialiser ini1 ini2))
   | (Ast.TyDecl(ft1,s1),Ast.TyDecl(ft2,s2)) -> unify_fullType ft1 ft2
   | (Ast.Typedef(stg1,ft1,id1,s1),Ast.Typedef(stg2,ft2,id2,s2)) ->
       conjunct_bindings (unify_fullType ft1 ft2) (unify_typeC id1 id2)
@@ -319,8 +411,6 @@ and unify_declaration d1 d2 =
   | (_,Ast.DisjDecl(d2)) ->
       disjunct_all_bindings
 	(List.map (function x -> unify_declaration d1 x) d2)
-  (* dots can match against anything.  return true to be safe. *)
-  | (Ast.Ddots(_,_),_) | (_,Ast.Ddots(_,_)) -> return true
 
   | (Ast.OptDecl(_),_)
   | (Ast.UniqueDecl(_),_)
@@ -328,12 +418,20 @@ and unify_declaration d1 d2 =
   | (_,Ast.UniqueDecl(_)) -> failwith "unsupported decl"
   | _ -> return false
 
+and unify_annotated_decl d1 d2 =
+  match (Ast.unwrap d1,Ast.unwrap d2) with
+    (Ast.DElem(_,_,d1),Ast.DElem(_,_,d2)) -> unify_declaration d1 d2
+  (* dots can match against anything.  return true to be safe. *)
+  | (Ast.Ddots(_,_),_) | (_,Ast.Ddots(_,_)) -> return true
+
 (* --------------------------------------------------------------------- *)
 (* Initializer *)
 
 and unify_initialiser i1 i2 =
   match (Ast.unwrap i1,Ast.unwrap i2) with
     (Ast.MetaInit(_,_,_),_) | (_,Ast.MetaInit(_,_,_)) -> return true
+  | (Ast.MetaInitList(_,_,_,_),_) | (_,Ast.MetaInitList(_,_,_,_)) ->
+      return true
   | (Ast.InitExpr(expa),Ast.InitExpr(expb)) ->
       unify_expression expa expb
   | (Ast.ArInitList(_,initlista,_),
@@ -392,6 +490,10 @@ and unify_parameterTypeDef p1 p2 =
   | (Ast.Pdots(_),_) | (_,Ast.Pdots(_))
   | (Ast.Pcircles(_),_) | (_,Ast.Pcircles(_)) -> return true
 
+  (* not sure what to do with the asexp.... *)
+  | (Ast.AsParam(param1,asexp1),_) -> unify_parameterTypeDef param1 p2
+  | (_,Ast.AsParam(param2,asexp2)) -> unify_parameterTypeDef p1 param2
+
   | (Ast.OptParam(_),_)
   | (Ast.UniqueParam(_),_)
   | (_,Ast.OptParam(_))
@@ -429,18 +531,18 @@ and unify_define_param p1 p2 =
 
 and unify_rule_elem re1 re2 =
   match (Ast.unwrap re1,Ast.unwrap re2) with
-    (Ast.FunHeader(_,_,fi1,nm1,lp1,params1,rp1),
-     Ast.FunHeader(_,_,fi2,nm2,lp2,params2,rp2)) ->
+    (Ast.FunHeader(_,_,fi1,nm1,lp1,params1,va1,rp1),
+     Ast.FunHeader(_,_,fi2,nm2,lp2,params2,v2,rp2)) ->
        conjunct_bindings (unify_fninfo fi1 fi2)
 	 (conjunct_bindings (unify_ident nm1 nm2)
 	    (unify_dots unify_parameterTypeDef pdots params1 params2))
-  | (Ast.Decl(_,_,d1),Ast.Decl(_,_,d2)) -> unify_declaration d1 d2
+  | (Ast.Decl d1,Ast.Decl d2) -> unify_annotated_decl d1 d2
 
   | (Ast.SeqStart(lb1),Ast.SeqStart(lb2)) -> return true
   | (Ast.SeqEnd(rb1),Ast.SeqEnd(rb2)) -> return true
 
   | (Ast.ExprStatement(e1,s1),Ast.ExprStatement(e2,s2)) ->
-      unify_expression e1 e2
+      unify_option unify_expression e1 e2
   | (Ast.IfHeader(if1,lp1,e1,rp1),Ast.IfHeader(if2,lp2,e2,rp2)) ->
       unify_expression e1 e2
   | (Ast.Else(e1),Ast.Else(e2)) -> return true
@@ -449,10 +551,15 @@ and unify_rule_elem re1 re2 =
   | (Ast.DoHeader(d1),Ast.DoHeader(d2)) -> return true
   | (Ast.WhileTail(wh1,lp1,e1,rp1,s1),Ast.WhileTail(wh2,lp2,e2,rp2,s2)) ->
       unify_expression e1 e2
-  | (Ast.ForHeader(fr1,lp1,e11,s11,e21,s21,e31,rp1),
-     Ast.ForHeader(fr2,lp2,e12,s12,e22,s22,e32,rp2)) ->
-       conjunct_bindings
-	 (unify_option unify_expression e11 e12)
+  | (Ast.ForHeader(fr1,lp1,first1,e21,s21,e31,rp1),
+     Ast.ForHeader(fr2,lp2,first2,e22,s22,e32,rp2)) ->
+       let first =
+	 match (first1,first2) with
+	   (Ast.ForExp(e11,s11),Ast.ForExp(e12,s1)) ->
+	     unify_option unify_expression e11 e12
+	 | (Ast.ForDecl d1,Ast.ForDecl d2) -> unify_annotated_decl d1 d2
+	 | _ -> return false in
+       conjunct_bindings first
 	 (conjunct_bindings
 	    (unify_option unify_expression e21 e22)
 	    (unify_option unify_expression e31 e32))
@@ -464,6 +571,8 @@ and unify_rule_elem re1 re2 =
   | (Ast.DefineHeader(_,n1,p1),Ast.DefineHeader(_,n2,p2)) ->
       conjunct_bindings (unify_ident n1 n2)
 	(unify_define_parameters p1 p2)
+  | (Ast.Pragma(_,i1,n1),Ast.Pragma(_,i2,n2)) ->
+      conjunct_bindings (unify_ident i1 i2) (unify_pragmainfo n1 n2)
   | (Ast.Break(r1,s1),Ast.Break(r2,s2)) -> return true
   | (Ast.Continue(r1,s1),Ast.Continue(r2,s2)) -> return true
   | (Ast.Label(l1,dd1),Ast.Label(l2,dd2)) -> unify_ident l1 l2
@@ -471,6 +580,10 @@ and unify_rule_elem re1 re2 =
   | (Ast.Return(r1,s1),Ast.Return(r2,s2)) -> return true
   | (Ast.ReturnExpr(r1,e1,s1),Ast.ReturnExpr(r2,e2,s2)) ->
       unify_expression e1 e2
+  | (Ast.Exec(exec1,lang1,code1,sem1),Ast.Exec(exec2,lang2,code2,sem2)) ->
+      if unify_mcode lang1 lang2
+      then unify_dots unify_exec_code ecdots code1 code2
+      else return false
 
   | (Ast.DisjRuleElem(res1),_) ->
       disjunct_all_bindings
@@ -500,6 +613,15 @@ and unify_rule_elem re1 re2 =
   | (_,Ast.Ty(t2)) -> subtype (unify_fullType t2) re1
   | _ -> return false
 
+and unify_pragmainfo pi1 pi2 =
+  match (Ast.unwrap pi1,Ast.unwrap pi2) with
+      (Ast.PragmaTuple(lp1,args1,rp1),Ast.PragmaTuple(lp2,args2,rp2)) ->
+	unify_dots unify_expression edots args1 args2
+    | (Ast.PragmaIdList(ids1),Ast.PragmaIdList(ids2)) ->
+	unify_dots unify_ident (function _ -> false) ids1 ids2
+    | (Ast.PragmaDots(_),_) | (_,Ast.PragmaDots(_)) -> return true
+    | _ -> return false
+
 and unify_fninfo patterninfo cinfo =
   let patterninfo = List.sort compare patterninfo in
   let cinfo = List.sort compare cinfo in
@@ -520,6 +642,15 @@ and unify_fninfo patterninfo cinfo =
     | _ -> return false in
   loop (patterninfo,cinfo)
 
+and unify_exec_code ec1 ec2 =
+  match (Ast.unwrap ec1,Ast.unwrap ec2) with
+    (Ast.ExecEval(colon1,id1),Ast.ExecEval(colon2,id2)) ->
+      unify_expression id1 id2
+  | (Ast.ExecToken(tok1),Ast.ExecToken(tok2)) ->
+      return (unify_mcode tok1 tok2)
+  | (Ast.ExecDots(_),_) | (_,Ast.ExecDots(_)) ->  return true
+  | _ -> return false
+
 and subexp f =
   let bind = conjunct_bindings in
   let option_default = return false in
@@ -528,9 +659,11 @@ and subexp f =
   let donothing r k e = k e in
   let recursor = V.combiner bind option_default
       mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
-      donothing donothing donothing donothing donothing
-      donothing expr donothing donothing donothing donothing donothing
-      donothing donothing donothing donothing donothing in
+      mcode mcode
+      donothing donothing donothing donothing donothing donothing expr 
+      donothing donothing donothing donothing donothing donothing
+      donothing donothing donothing donothing donothing donothing donothing
+      donothing donothing in
   recursor.V.combiner_rule_elem
 
 and subtype f =
@@ -541,9 +674,11 @@ and subtype f =
   let donothing r k e = k e in
   let recursor = V.combiner bind option_default
       mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
-      donothing donothing donothing donothing donothing
-      donothing donothing fullType donothing donothing donothing donothing
-      donothing donothing donothing donothing donothing in
+      mcode mcode
+      donothing donothing donothing donothing donothing donothing
+      donothing donothing donothing donothing donothing fullType
+      donothing donothing donothing donothing donothing donothing
+      donothing donothing donothing donothing in
   recursor.V.combiner_rule_elem
 
 let rec unify_statement s1 s2 =
@@ -582,7 +717,7 @@ let rec unify_statement s1 s2 =
 	   s2)
   | (Ast.Nest(_,s1,_,_,_,_,_),Ast.Nest(_,s2,_,_,_,_,_)) ->
       unify_dots unify_statement sdots s1 s2
-  | (Ast.FunDecl(h1,lb1,s1,rb1),Ast.FunDecl(h2,lb2,s2,rb2)) ->
+  | (Ast.FunDecl(h1,lb1,s1,rb1,_),Ast.FunDecl(h2,lb2,s2,rb2,_)) ->
       conjunct_bindings (unify_rule_elem h1 h2)
 	(conjunct_bindings (unify_rule_elem lb1 lb2)
 	   (conjunct_bindings (unify_dots unify_statement sdots s1 s2)
