@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./unitary_ast0.ml"
 (* find unitary metavariables *)
 module Ast0 = Ast0_cocci
 module Ast = Ast_cocci
@@ -79,15 +82,15 @@ let get_free checker t =
     unitary@nonunitary@nonunitary in
 
   let whencode afn bfn expression = function
-      Ast0.WhenNot(a) -> afn a
-    | Ast0.WhenAlways(b) -> bfn b
-    | Ast0.WhenModifier(_) -> option_default
-    | Ast0.WhenNotTrue(a) -> expression a
-    | Ast0.WhenNotFalse(a) -> expression a in
+      Ast0.WhenNot(_,_,a) -> afn a
+    | Ast0.WhenAlways(_,_,b) -> bfn b
+    | Ast0.WhenModifier(_,_) -> option_default
+    | Ast0.WhenNotTrue(_,_,a) -> expression a
+    | Ast0.WhenNotFalse(_,_,a) -> expression a in
 
   let ident r k i =
     match Ast0.unwrap i with
-      Ast0.MetaId(name,_,_) | Ast0.MetaFunc(name,_,_)
+      Ast0.MetaId(name,_,_,_) | Ast0.MetaFunc(name,_,_)
     | Ast0.MetaLocalFunc(name,_,_) -> checker name
     | Ast0.DisjId(starter,id_list,mids,ender) ->
 	detect_unitary_frees(List.map r.VT0.combiner_rec_ident id_list)
@@ -178,8 +181,8 @@ let update_unitary unitary =
 
   let ident r k i =
     match Ast0.unwrap i with
-      Ast0.MetaId(name,constraints,_) ->
-	Ast0.rewrap i (Ast0.MetaId(name,constraints,is_unitary name))
+      Ast0.MetaId(name,constraints,seed,_) ->
+	Ast0.rewrap i (Ast0.MetaId(name,constraints,seed,is_unitary name))
     | Ast0.MetaFunc(name,constraints,_) ->
 	Ast0.rewrap i (Ast0.MetaFunc(name,constraints,is_unitary name))
     | Ast0.MetaLocalFunc(name,constraints,_) ->
@@ -248,7 +251,8 @@ let do_unitary rules =
     | (r::rules) ->
       match r with
         Ast0.ScriptRule (_,_,_,_,_,_)
-      | Ast0.InitialScriptRule (_,_,_,_) | Ast0.FinalScriptRule (_,_,_,_) ->
+      | Ast0.InitialScriptRule (_,_,_,_,_)
+      | Ast0.FinalScriptRule (_,_,_,_,_) ->
           let (x,rules) = loop rules in
           (x, r::rules)
       | Ast0.CocciRule ((minus,metavars,chosen_isos),((plus,_) as plusz),rt) ->
