@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./unparse_ast0.ml"
 open Format
 module Ast = Ast_cocci
 module Ast0 = Ast0_cocci
@@ -43,12 +46,15 @@ let print_between = Common.print_between
 (* --------------------------------------------------------------------- *)
 (* Positions *)
 
-let meta_pos = function
-    Ast0.MetaPos(name,_,_) ->
+let rec meta_pos l =
+  List.iter
+    (function var ->
+      let current_name = Ast0.meta_pos_name var in
+      let (_,name) = Ast0.unwrap_mcode current_name in
       print_string "@";
-      let (_,name) = Ast0.unwrap_mcode name in
-      print_string name
-  | Ast0.NoMetaPos -> ()
+      print_string name;
+      meta_pos (Ast0.get_pos current_name))
+    l
 
 (* --------------------------------------------------------------------- *)
 (* Modified code *)
@@ -71,7 +77,9 @@ let mcodekind brackets fn x info mc =
       then fn x
       else (print_string "-";
 	    print_string lb; fn x; print_string rb);
-      U.print_anything ">>> " plus_stream
+      (match plus_stream with
+	Ast.NOREPLACEMENT -> ()
+      | Ast.REPLACEMENT(plus_stream,_) -> U.print_anything ">>> " plus_stream)
   | Ast0.CONTEXT(plus_streams) ->
       let (lb,rb) =
 	if !quiet
@@ -151,18 +159,33 @@ let rec ident i =
     (function _ ->
       match Ast0.unwrap i with
 	Ast0.Id(name) -> mcode print_string name
-      | Ast0.MetaId(name,_,_) -> mcode print_meta name
+      | Ast0.MetaId(name,_,_,_) -> mcode print_meta name
       | Ast0.MetaFunc(name,_,_) -> mcode print_meta name
       | Ast0.MetaLocalFunc(name,_,_) -> mcode print_meta name
       | Ast0.DisjId(_,id_list,_,_) -> do_disj id_list ident
       | Ast0.OptIdent(id) -> print_string "?"; ident id
-      | Ast0.UniqueIdent(id) -> print_string "!"; ident id)
+      | Ast0.UniqueIdent(id) -> print_string "!"; ident id
+      | Ast0.AsIdent(id,asid) -> ident id; print_string "@"; ident asid)
 
 (* --------------------------------------------------------------------- *)
 (* Expression *)
 
 let print_string_box s = print_string s; open_box 0
 
+let assignOp op =
+  print_context op 
+    (function _ -> match Ast0.unwrap op with
+      | Ast0.SimpleAssign op' -> mcode U.simpleAssignOp op'
+      | Ast0.OpAssign op' -> mcode U.opAssignOp op'
+      | Ast0.MetaAssign(name,_,_) -> mcode print_meta name)
+
+let binaryOp op =
+  print_context op 
+    (function _ -> match Ast0.unwrap op with
+      | Ast0.Arith op' -> mcode U.arithOp op'
+      | Ast0.Logical op' -> mcode U.logicalOp op'
+      | Ast0.MetaBinary(name,_,_) -> mcode print_meta name)
+
 let rec expression e =
   print_option Type_cocci.typeC (Ast0.get_type e);
   print_context e
@@ -170,12 +193,22 @@ let rec expression e =
       match Ast0.unwrap e with
 	Ast0.Ident(id) -> ident id
       | Ast0.Constant(const) -> mcode U.constant const
+      | Ast0.StringConstant(lq,str,rq) ->
+	  mcode print_string lq;
+	  let _ = dots (function _ -> ()) string_fragment str in
+	  mcode print_string rq
       | Ast0.FunCall(fn,lp,args,rp) ->
 	  expression fn; mcode print_string_box lp;
 	  let _ = dots (function _ -> ()) expression args in
 	  close_box(); mcode print_string rp
       | Ast0.Assignment(left,op,right,_) ->
-	  expression left; print_string " "; mcode U.assignOp op;
+	expression left;
+        print_string " ";
+        assignOp op;
+	print_string " ";
+        expression right
+      | Ast0.Sequence(left,op,right) ->
+	  expression left; mcode print_string op;
 	  print_string " "; expression right
       | Ast0.CondExpr(exp1,why,exp2,colon,exp3) ->
 	  expression exp1; print_string " "; mcode print_string why;
@@ -186,13 +219,19 @@ let rec expression e =
       | Ast0.Unary(exp,op) -> mcode U.unaryOp op; expression exp
       | Ast0.Binary(left,op,right) ->
 	  print_string "(";
-	  expression left; print_string " "; mcode U.binaryOp op;
-	  print_string " "; expression right;
+	  expression left;
+          print_string " ";
+          binaryOp op;
+          print_string " ";
+          expression right;
 	  print_string ")"
       | Ast0.Nested(left,op,right) ->
 	  print_string "(";
-	  expression left; print_string " "; mcode U.binaryOp op;
-	  print_string " "; expression right;
+	  expression left;
+          print_string " ";
+          binaryOp op;
+	  print_string " ";
+          expression right;
 	  print_string ")"
       | Ast0.Paren(lp,exp,rp) ->
 	  mcode print_string_box lp; expression exp; close_box();
@@ -214,6 +253,9 @@ let rec expression e =
 	  mcode print_string_box lp; typeC ty; close_box();
 	  mcode print_string rp
       | Ast0.TypeExp(ty) -> typeC ty
+      | Ast0.Constructor(lp,ty,rp,init) ->
+	  mcode print_string_box lp; typeC ty; close_box();
+	  mcode print_string rp; initialiser init
       | Ast0.MetaErr(name,_,_) -> mcode print_meta name
       | Ast0.MetaExpr(name,_,ty,_,pure) ->
 	  mcode print_meta name; print_types ty(*;
@@ -230,24 +272,42 @@ let rec expression e =
 	  mcode print_string starter;
 	  start_block(); dots force_newline expression expr_dots; end_block();
 	  mcode print_string ender
-      | Ast0.NestExpr(starter,expr_dots,ender,Some whencode,multi) ->
+      | Ast0.NestExpr(starter,expr_dots,ender,Some (_,_,whencode),multi) ->
 	  mcode print_string starter; print_string "   WHEN != ";
 	  expression whencode;
 	  start_block(); dots force_newline expression expr_dots; end_block();
 	  mcode print_string ender
-      | Ast0.Edots(dots,Some whencode)
-      | Ast0.Ecircles(dots,Some whencode)
-      | Ast0.Estars(dots,Some whencode) ->
+      | Ast0.Edots(dots,Some (_,_,whencode))
+      | Ast0.Ecircles(dots,Some (_,_,whencode))
+      | Ast0.Estars(dots,Some (_,_,whencode)) ->
 	  mcode print_string dots; print_string "   WHEN != ";
 	  expression whencode
       | Ast0.Edots(dots,None)
       | Ast0.Ecircles(dots,None)
       | Ast0.Estars(dots,None) -> mcode print_string dots
       | Ast0.OptExp(exp) -> print_string "?"; expression exp
-      | Ast0.UniqueExp(exp) -> print_string "!"; expression exp)
+      | Ast0.UniqueExp(exp) -> print_string "!"; expression exp
+      |	Ast0.AsExpr(exp,asexp) -> expression exp; print_string "@";
+	  expression asexp)
 
 and expression_dots x = dots (function _ -> ()) expression x
 
+and string_fragment e =
+  match Ast0.unwrap e with
+    Ast0.ConstantFragment(str) -> mcode print_string str
+  | Ast0.FormatFragment(pct,fmt) ->
+      mcode print_string pct;
+      string_format fmt
+  | Ast0.Strdots dots -> mcode print_string dots
+  | Ast0.MetaFormatList(pct,name,lenname) ->
+      mcode print_string pct;
+      mcode print_meta name
+
+and string_format e =
+  match Ast0.unwrap e with
+    Ast0.ConstantFormat(str) -> mcode print_string str
+  | Ast0.MetaFormat(name,_) -> mcode print_meta name
+
 (* --------------------------------------------------------------------- *)
 (* Types *)
 
@@ -256,10 +316,6 @@ and print_function_pointer (ty,lp1,star,rp1,lp2,params,rp2) fn =
   mcode print_string rp1; mcode print_string lp2;
   parameter_list params; mcode print_string rp2
 
-and print_function_type (ty,lp1,params,rp1) fn =
-  print_option typeC ty; fn(); mcode print_string lp1;
-  parameter_list params; mcode print_string rp1
-
 and typeC t =
   print_context t
     (function _ ->
@@ -274,11 +330,16 @@ and typeC t =
       | Ast0.FunctionPointer(ty,lp1,star,rp1,lp2,params,rp2) ->
 	  print_function_pointer (ty,lp1,star,rp1,lp2,params,rp2)
 	    (function _ -> ())
-      | Ast0.FunctionType(ty,lp1,params,rp1) ->
-	  print_function_type (ty,lp1,params,rp1) (function _ -> ())
       | Ast0.Array(ty,lb,size,rb) ->
 	  typeC ty; mcode print_string lb; print_option expression size;
 	  mcode print_string rb
+      | Ast0.Decimal(dec,lp,length,comma,precision_opt,rp) ->
+	  mcode print_string dec;
+	  mcode print_string lp;
+	  expression length;
+	  print_option (mcode print_string) comma;
+	  print_option expression precision_opt;
+	  mcode print_string rp
       | Ast0.EnumName(kind,name) ->
 	  mcode print_string kind;
 	  print_option (function x -> ident x; print_string " ") name
@@ -297,7 +358,8 @@ and typeC t =
       | Ast0.MetaType(name,_)-> mcode print_meta name; print_string " "
       | Ast0.DisjType(_,types,_,_) -> do_disj types typeC
       | Ast0.OptType(ty) -> print_string "?"; typeC ty
-      | Ast0.UniqueType(ty) -> print_string "!"; typeC ty)
+      | Ast0.UniqueType(ty) -> print_string "!"; typeC ty
+      | Ast0.AsType(ty,asty) -> typeC ty; print_string "@"; typeC asty)
 
 (* --------------------------------------------------------------------- *)
 (* Variable declaration *)
@@ -309,9 +371,6 @@ and print_named_type ty id =
     Ast0.FunctionPointer(ty,lp1,star,rp1,lp2,params,rp2) ->
       print_function_pointer (ty,lp1,star,rp1,lp2,params,rp2)
 	(function _ -> print_string " "; ident id)
-  | Ast0.FunctionType(ty,lp1,params,rp1) ->
-      print_function_type (ty,lp1,params,rp1)
-	(function _ -> print_string " "; ident id)
   | Ast0.Array(ty,lb,size,rb) ->
       let rec loop ty k =
 	match Ast0.unwrap ty with
@@ -342,10 +401,23 @@ and declaration d =
       | Ast0.UnInit(stg,ty,id,sem) ->
 	  print_option (mcode U.storage) stg; print_named_type ty id;
 	  mcode print_string sem
+      | Ast0.FunProto(fninfo,name,lp1,params,va,rp1,sem) ->
+	  List.iter print_fninfo fninfo;
+	  ident name; mcode print_string_box lp1;
+	  parameter_list params; varargs va;
+	  close_box(); mcode print_string rp1;
+	  mcode print_string sem
       | Ast0.MacroDecl(name,lp,args,rp,sem) ->
 	  ident name; mcode print_string_box lp;
 	  let _ = dots (function _ -> ()) expression args in
 	  close_box(); mcode print_string rp; mcode print_string sem
+      | Ast0.MacroDeclInit(name,lp,args,rp,eq,ini,sem) ->
+	  ident name; mcode print_string_box lp;
+	  let _ = dots (function _ -> ()) expression args in
+	  close_box(); mcode print_string rp;
+          print_string " ";
+          mcode print_string eq; print_string " "; initialiser ini;
+	  mcode print_string sem
       | Ast0.TyDecl(ty,sem) -> typeC ty; mcode print_string sem
       | Ast0.Typedef(stg,ty,id,sem) ->
 	  mcode print_string stg; typeC ty; typeC id;
@@ -356,12 +428,14 @@ and declaration d =
 	    (function _ -> print_string "\n|"; force_newline())
 	    declaration decls;
 	  print_string "\n)"
-      | Ast0.Ddots(dots,Some whencode) ->
+      | Ast0.Ddots(dots,Some (_,_,whencode)) ->
 	  mcode print_string dots; print_string "   when != ";
 	  declaration whencode
       | Ast0.Ddots(dots,None) -> mcode print_string dots
       | Ast0.OptDecl(decl) -> print_string "?"; declaration decl
-      | Ast0.UniqueDecl(decl) -> print_string "!"; declaration decl)
+      | Ast0.UniqueDecl(decl) -> print_string "!"; declaration decl
+      | Ast0.AsDecl(decl,asdecl) ->
+	  declaration decl; print_string "@"; declaration asdecl)
 
 and declaration_dots l = dots (function _ -> ()) declaration l
 
@@ -373,6 +447,7 @@ and initialiser i =
     (function _ ->
       match Ast0.unwrap i with
 	Ast0.MetaInit(name,_)-> mcode print_meta name; print_string " "
+      |	Ast0.MetaInitList(name,_,_)-> mcode print_meta name; print_string " "
       |	Ast0.InitExpr(exp) -> expression exp
       | Ast0.InitList(lb,initlist,rb,ordered) ->
           (*doesn't show commas dropped in unordered case*)
@@ -385,12 +460,14 @@ and initialiser i =
       | Ast0.InitGccName(name,eq,ini) ->
 	  ident name; mcode print_string eq; initialiser ini
       | Ast0.IComma(cm) -> mcode print_string cm; force_newline()
-      | Ast0.Idots(d,Some whencode) ->
+      | Ast0.Idots(d,Some (_,_,whencode)) ->
 	  mcode print_string d; print_string "   WHEN != ";
 	  initialiser whencode
       | Ast0.Idots(d,None) -> mcode print_string d
       | Ast0.OptIni(ini) -> print_string "?"; initialiser ini
-      | Ast0.UniqueIni(ini) -> print_string "!"; initialiser ini)
+      | Ast0.UniqueIni(ini) -> print_string "!"; initialiser ini
+      | Ast0.AsInit(ini,asini) -> initialiser ini; print_string "@";
+	  initialiser asini)
 
 and designator = function
       Ast0.DesignatorField(dot,id) -> mcode print_string dot; ident id
@@ -418,10 +495,15 @@ and parameterTypeDef p =
       | Ast0.Pdots(dots) -> mcode print_string dots
       | Ast0.Pcircles(dots) -> mcode print_string dots
       | Ast0.OptParam(param) -> print_string "?"; parameterTypeDef param
-      | Ast0.UniqueParam(param) -> print_string "!"; parameterTypeDef param)
+      | Ast0.UniqueParam(param) -> print_string "!"; parameterTypeDef param
+      |	Ast0.AsParam(p,asexp) -> parameterTypeDef p; print_string "@";
+	  expression asexp)
 
 and parameter_list l = dots (function _ -> ()) parameterTypeDef l
-
+and varargs va = match va with
+  | None -> ()
+  | Some (comma, ellipsis) ->
+    mcode print_string comma; mcode print_string ellipsis
 (* --------------------------------------------------------------------- *)
 (* Top-level code *)
 
@@ -429,42 +511,44 @@ and statement arity s =
   print_context s
     (function _ ->
       match Ast0.unwrap s with
-	Ast0.FunDecl(_,fninfo,name,lp,params,rp,lbrace,body,rbrace) ->
+	Ast0.FunDecl(_,fninfo,name,lp,params,va,rp,lbrace,body,rbrace,_) ->
 	  print_string arity;
 	  List.iter print_fninfo fninfo;
 	  ident name; mcode print_string_box lp;
-	  parameter_list params; close_box(); mcode print_string rp;
+	  parameter_list params; varargs va; close_box(); mcode print_string rp;
 	  print_string " ";
 	  print_string arity; mcode print_string lbrace; start_block();
 	  dots force_newline (statement arity) body;
 	  end_block(); print_string arity; mcode print_string rbrace
-      | Ast0.Decl(_,decl) -> print_string arity; declaration decl
+      | Ast0.Decl(_,decl) ->
+	  print_string arity; declaration decl
       | Ast0.Seq(lbrace,body,rbrace) ->
 	  print_string arity; mcode print_string lbrace; start_block();
 	  dots force_newline (statement arity) body;
 	  end_block(); print_string arity; mcode print_string rbrace
       | Ast0.ExprStatement(exp,sem) ->
-	  print_string arity; expression exp; mcode print_string sem
-      | Ast0.IfThen(iff,lp,exp,rp,branch1,(info,aft)) ->
+	  print_string arity; print_option expression exp;
+	  mcode print_string sem
+      | Ast0.IfThen(iff,lp,exp,rp,branch1,(info,aft,adj)) ->
 	  print_string arity;
 	  mcode print_string iff; print_string " "; mcode print_string_box lp;
 	  expression exp; close_box(); mcode print_string rp; print_string " ";
 	  statement arity branch1;
-	  mcode (function _ -> ()) ((),(),info,aft,ref Ast0.NoMetaPos,-1)
-      | Ast0.IfThenElse(iff,lp,exp,rp,branch1,els,branch2,(info,aft)) ->
+	  mcode (function _ -> ()) ((),(),info,aft,ref [],adj)
+      | Ast0.IfThenElse(iff,lp,exp,rp,branch1,els,branch2,(info,aft,adj)) ->
 	  print_string arity;
 	  mcode print_string iff; print_string " "; mcode print_string_box lp;
 	  expression exp; close_box(); mcode print_string rp; print_string " ";
 	  statement arity branch1;
 	  print_string arity; mcode print_string els; print_string " ";
 	  statement arity branch2;
-	  mcode (function _ -> ()) ((),(),info,aft,ref Ast0.NoMetaPos,-1)
-      | Ast0.While(whl,lp,exp,rp,body,(info,aft)) ->
+	  mcode (function _ -> ()) ((),(),info,aft,ref [],adj)
+      | Ast0.While(whl,lp,exp,rp,body,(info,aft,adj)) ->
 	  print_string arity;
 	  mcode print_string whl; print_string " "; mcode print_string_box lp;
 	  expression exp; close_box(); mcode print_string rp; print_string " ";
 	  statement arity body;
-	  mcode (function _ -> ()) ((),(),info,aft,ref Ast0.NoMetaPos,-1)
+	  mcode (function _ -> ()) ((),(),info,aft,ref [],adj)
       | Ast0.Do(d,body,whl,lp,exp,rp,sem) ->
 	  print_string arity; mcode print_string d; print_string " ";
 	  statement arity body;
@@ -472,21 +556,24 @@ and statement arity s =
 	  mcode print_string whl; print_string " "; mcode print_string_box lp;
 	  expression exp; close_box(); mcode print_string rp;
 	  mcode print_string sem
-      | Ast0.For(fr,lp,e1,sem1,e2,sem2,e3,rp,body,(info,aft)) ->
+      | Ast0.For(fr,lp,first,e2,sem2,e3,rp,body,(info,aft,adj)) ->
 	  print_string arity;
 	  mcode print_string fr; mcode print_string_box lp;
-	  print_option expression e1; mcode print_string sem1;
+	  (match Ast0.unwrap first with
+	    Ast0.ForExp(e1,sem1) ->
+	      print_option expression e1; mcode print_string sem1
+	  | Ast0.ForDecl (_,decl) -> declaration decl);
 	  print_option expression e2; mcode print_string sem2;
 	  print_option expression e3; close_box();
 	  mcode print_string rp; print_string " "; statement arity body;
-	  mcode (function _ -> ()) ((),(),info,aft,ref Ast0.NoMetaPos,-1)
-      | Ast0.Iterator(nm,lp,args,rp,body,(info,aft)) ->
+	  mcode (function _ -> ()) ((),(),info,aft,ref [],adj)
+      | Ast0.Iterator(nm,lp,args,rp,body,(info,aft,adj)) ->
 	  print_string arity;
 	  ident nm; print_string " "; mcode print_string_box lp;
 	  let _ = dots (function _ -> ()) expression args in
 	  close_box(); mcode print_string rp; print_string " ";
 	  statement arity body;
-	  mcode (function _ -> ()) ((),(),info,aft,ref Ast0.NoMetaPos,-1)
+	  mcode (function _ -> ()) ((),(),info,aft,ref [],adj)
       |	Ast0.Switch(switch,lp,exp,rp,lb,decls,cases,rb) ->
 	  print_string arity;
 	  mcode print_string switch; print_string " ";
@@ -507,6 +594,11 @@ and statement arity s =
       | Ast0.ReturnExpr(ret,exp,sem) ->
 	  print_string arity; mcode print_string ret; print_string " ";
 	  expression exp; mcode print_string sem
+      | Ast0.Exec(exec,lang,code,sem) ->
+	  print_string arity; mcode print_string exec; print_string " ";
+	  mcode print_string lang; print_string " ";
+	  dots (function _ -> print_string " ") exec_code code;
+	  mcode print_string sem
       | Ast0.MetaStmt(name,pure) ->
 	  print_string arity; mcode print_meta name;(*
 	  print_string "^";
@@ -555,8 +647,22 @@ and statement arity s =
 	  print_define_parameters params;
 	  print_string " ";
 	  dots force_newline (statement arity) body
+      | Ast0.Pragma(prg,id,body) ->
+	  mcode print_string prg; print_string " "; ident id;
+	  print_string " "; pragmainfo body
       | Ast0.OptStm(re) -> statement "?" re
-      | Ast0.UniqueStm(re) -> statement "!" re)
+      | Ast0.UniqueStm(re) -> statement "!" re
+      | Ast0.AsStmt(stm,asstm) -> statement arity stm; print_string "@";
+	  statement arity asstm)
+
+and pragmainfo pi =
+  match Ast0.unwrap pi with
+      Ast0.PragmaTuple(lp,args,rp) ->
+	mcode print_string_box lp;
+	dots (function _ -> ()) expression args;
+	close_box(); mcode print_string rp
+    | Ast0.PragmaIdList(ids) -> dots (function _ -> ()) ident ids
+    | Ast0.PragmaDots(dots) -> mcode print_string dots
 
 and print_define_parameters params =
   match Ast0.unwrap params with
@@ -581,14 +687,14 @@ and print_fninfo = function
   | Ast0.FAttr(attr) -> mcode print_string attr
 
 and whencode notfn alwaysfn = function
-    Ast0.WhenNot a ->
+    Ast0.WhenNot (_,_,a) ->
       print_string "   WHEN != "; open_box 0; notfn a; close_box()
-  | Ast0.WhenAlways a ->
+  | Ast0.WhenAlways (_,_,a) ->
       print_string "   WHEN = "; open_box 0; alwaysfn a; close_box()
-  | Ast0.WhenModifier x -> print_string "   WHEN "; U.print_when_modif x
-  | Ast0.WhenNotTrue a ->
+  | Ast0.WhenModifier (_,x) -> print_string "   WHEN "; U.print_when_modif x
+  | Ast0.WhenNotTrue (_,_,a) ->
       print_string "   WHEN != TRUE "; open_box 0; expression a; close_box()
-  | Ast0.WhenNotFalse a ->
+  | Ast0.WhenNotFalse (_,_,a) ->
       print_string "   WHEN != FALSE "; open_box 0; expression a; close_box()
 
 and case_line arity c =
@@ -615,6 +721,12 @@ and case_line arity c =
 and statement_dots l = dots (function _ -> ()) (statement "") l
 and case_dots l = dots (function _ -> ()) (case_line "") l
 
+and exec_code e =
+  match Ast0.unwrap e with
+    Ast0.ExecEval(colon,id) -> mcode print_string colon; expression id
+  | Ast0.ExecToken(tok) -> mcode print_string tok
+  | Ast0.ExecDots(dots) -> mcode print_string dots
+
 (* --------------------------------------------------------------------- *)
 (* Top level code *)
 
@@ -625,8 +737,8 @@ let top_level t =
 	Ast0.FILEINFO(old_file,new_file) ->
 	  print_string "--- "; mcode print_string old_file; force_newline();
 	  print_string "+++ "; mcode print_string new_file
-      | Ast0.DECL(stmt) -> statement "" stmt
-      | Ast0.CODE(stmt_dots) ->
+      | Ast0.NONDECL(stmt) -> statement "" stmt
+      | Ast0.CODE(stmt_dots) | Ast0.TOPCODE(stmt_dots) ->
 	  dots force_newline (statement "") stmt_dots
       | Ast0.ERRORWORDS(exps) ->
 	  print_string "error words = [";
@@ -638,7 +750,7 @@ let top_level t =
 let rule =
   print_between (function _ -> force_newline(); force_newline()) top_level
 
-let unparse_anything x =
+let rec unparse_anything x =
   let q = !quiet in
   quiet := true;
   (match x with
@@ -658,6 +770,10 @@ let unparse_anything x =
   | Ast0.ExprTag(d) | Ast0.ArgExprTag(d) | Ast0.TestExprTag(d) ->
       print_string "Exp:"; force_newline();
       expression d
+  | Ast0.AssignOpTag(d) ->
+      print_string ("AssignOp: " ^ (Ast0.string_of_assignOp d)); force_newline();
+  | Ast0.BinaryOpTag(d) ->
+      print_string ("BinaryOp: " ^ (Ast0.string_of_binaryOp d)); force_newline();
   | Ast0.TypeCTag(d) -> typeC d
   | Ast0.ParamTag(d) -> parameterTypeDef d
   | Ast0.InitTag(d)  -> initialiser d
@@ -665,12 +781,21 @@ let unparse_anything x =
   | Ast0.StmtTag(d)  ->
       print_string "Stm:"; force_newline();
       statement "" d
+  | Ast0.ForInfoTag(fi)  ->
+      print_string "ForInfo:"; force_newline();
+      (match Ast0.unwrap fi with
+	Ast0.ForExp(e1,sem1) ->
+	  print_option expression e1; mcode print_string sem1
+      | Ast0.ForDecl (_,decl) -> declaration decl)
   | Ast0.CaseLineTag(d)  -> case_line "" d
+  | Ast0.StringFragmentTag(d)  -> string_fragment d
   | Ast0.TopTag(d)       -> top_level d
   | Ast0.IsoWhenTag(x)   -> U.print_when_modif x
   | Ast0.IsoWhenTTag(e)  -> expression e
   | Ast0.IsoWhenFTag(e)  -> expression e
-  | Ast0.MetaPosTag(var) -> meta_pos var);
+  | Ast0.MetaPosTag(var) -> meta_pos [x]
+  | Ast0.HiddenVarTag(var) -> failwith "should not need to be printed"
+  | Ast0.WhenTag(w,e,a) -> unparse_anything a);
   quiet := q;
   print_newline()
 
@@ -682,3 +807,7 @@ let unparse x =
   print_newline()
 
 let unparse_to_string x = Common.format_to_string (function _ -> unparse x)
+
+let show_cocci_parse_tree comment parse_tree =
+  Printf.printf "%s\n" comment;
+  top_level parse_tree; Format.print_newline()
