@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./unparse_ast0.mli"
 val expression_dots : Ast0_cocci.expression Ast0_cocci.dots -> unit
 val parameter_list : Ast0_cocci.parameterTypeDef Ast0_cocci.dots -> unit
 val statement_dots : Ast0_cocci.statement Ast0_cocci.dots -> unit
@@ -35,3 +38,5 @@ val top_level : Ast0_cocci.top_level -> unit
 
 val unparse : Ast0_cocci.rule -> unit
 val unparse_anything : Ast0_cocci.anything -> unit
+
+val show_cocci_parse_tree : string -> Ast0_cocci.top_level -> unit
