@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,9 +24,14 @@
  *)
 
 
+# 0 "./visitor_ast.mli"
 type 'a combiner =
     {combiner_ident : Ast_cocci.ident -> 'a;
      combiner_expression : Ast_cocci.expression -> 'a;
+     combiner_fragment : Ast_cocci.string_fragment -> 'a;
+     combiner_format : Ast_cocci.string_format -> 'a;
+     combiner_assignOp : Ast_cocci.assignOp -> 'a;
+     combiner_binaryOp : Ast_cocci.binaryOp -> 'a;
      combiner_fullType : Ast_cocci.fullType -> 'a;
      combiner_typeC : Ast_cocci.typeC -> 'a;
      combiner_declaration : Ast_cocci.declaration -> 'a;
@@ -36,12 +43,10 @@ type 'a combiner =
      combiner_case_line : Ast_cocci.case_line -> 'a;
      combiner_top_level : Ast_cocci.top_level -> 'a;
      combiner_anything : Ast_cocci.anything  -> 'a;
-     combiner_expression_dots :
-	 Ast_cocci.expression Ast_cocci.dots -> 'a;
-     combiner_statement_dots :
-	     Ast_cocci.statement Ast_cocci.dots -> 'a;
-     combiner_declaration_dots :
-		 Ast_cocci.declaration Ast_cocci.dots -> 'a}
+     combiner_expression_dots : Ast_cocci.expression Ast_cocci.dots -> 'a;
+     combiner_statement_dots : Ast_cocci.statement Ast_cocci.dots -> 'a;
+     combiner_anndecl_dots : Ast_cocci.annotated_decl Ast_cocci.dots -> 'a;
+     combiner_initialiser_dots : Ast_cocci.initialiser Ast_cocci.dots -> 'a}
 
 type ('mc,'a) cmcode = 'a combiner -> 'mc Ast_cocci.mcode -> 'a
 type ('cd,'a) ccode = 'a combiner -> ('cd -> 'a) -> 'cd -> 'a
@@ -51,10 +56,12 @@ val combiner :
       ((Ast_cocci.meta_name,'a) cmcode) ->
       ((string,'a) cmcode) ->
       ((Ast_cocci.constant,'a) cmcode) ->
-      ((Ast_cocci.assignOp,'a) cmcode) ->
+      ((Ast_cocci.simpleAssignOp,'a) cmcode) ->
+      ((Ast_cocci.arithOp,'a) cmcode) ->
       ((Ast_cocci.fixOp,'a) cmcode) ->
       ((Ast_cocci.unaryOp,'a) cmcode) ->
-      ((Ast_cocci.binaryOp,'a) cmcode) ->
+      ((Ast_cocci.arithOp,'a) cmcode) ->
+      ((Ast_cocci.logicalOp,'a) cmcode) ->
       ((Ast_cocci.const_vol,'a) cmcode) ->
       ((Ast_cocci.sign,'a) cmcode) ->
       ((Ast_cocci.structUnion,'a) cmcode) ->
@@ -63,14 +70,20 @@ val combiner :
       ((Ast_cocci.expression Ast_cocci.dots,'a) ccode) ->
       ((Ast_cocci.parameterTypeDef Ast_cocci.dots,'a) ccode) ->
       ((Ast_cocci.statement Ast_cocci.dots,'a) ccode) ->
-      ((Ast_cocci.declaration Ast_cocci.dots,'a) ccode) ->
+      ((Ast_cocci.annotated_decl Ast_cocci.dots,'a) ccode) ->
+      ((Ast_cocci.initialiser Ast_cocci.dots,'a) ccode) ->
       ((Ast_cocci.ident,'a) ccode) ->
       ((Ast_cocci.expression,'a) ccode) ->
+      ((Ast_cocci.string_fragment,'a) ccode) ->
+      ((Ast_cocci.string_format,'a) ccode) ->
+      ((Ast_cocci.assignOp,'a) ccode) ->
+      ((Ast_cocci.binaryOp,'a) ccode) ->
       ((Ast_cocci.fullType,'a) ccode) ->
       ((Ast_cocci.typeC,'a) ccode) ->
       ((Ast_cocci.initialiser,'a) ccode) ->
       ((Ast_cocci.parameterTypeDef,'a) ccode) ->
       ((Ast_cocci.declaration,'a) ccode) ->
+      ((Ast_cocci.annotated_decl,'a) ccode) ->
       ((Ast_cocci.rule_elem,'a) ccode) ->
       ((Ast_cocci.statement,'a) ccode) ->
       ((Ast_cocci.case_line,'a) ccode) ->
@@ -83,6 +96,10 @@ type 'a inout = 'a -> 'a (* for specifying the type of rebuilder *)
 type rebuilder =
     {rebuilder_ident : Ast_cocci.ident inout;
       rebuilder_expression : Ast_cocci.expression inout;
+      rebuilder_fragment : Ast_cocci.string_fragment inout;
+      rebuilder_format : Ast_cocci.string_format inout;
+      rebuilder_assignOp : Ast_cocci.assignOp inout;
+      rebuilder_binaryOp : Ast_cocci.binaryOp inout;
       rebuilder_fullType : Ast_cocci.fullType inout;
       rebuilder_typeC : Ast_cocci.typeC inout;
       rebuilder_declaration : Ast_cocci.declaration inout;
@@ -95,7 +112,8 @@ type rebuilder =
       rebuilder_top_level : Ast_cocci.top_level inout;
       rebuilder_expression_dots : Ast_cocci.expression Ast_cocci.dots inout;
       rebuilder_statement_dots : Ast_cocci.statement Ast_cocci.dots inout;
-      rebuilder_declaration_dots : Ast_cocci.declaration Ast_cocci.dots inout;
+      rebuilder_anndecl_dots : Ast_cocci.annotated_decl Ast_cocci.dots inout;
+      rebuilder_initialiser_dots : Ast_cocci.initialiser Ast_cocci.dots inout;
       rebuilder_define_param_dots: Ast_cocci.define_param Ast_cocci.dots inout;
       rebuilder_define_param : Ast_cocci.define_param inout;
       rebuilder_define_parameters : Ast_cocci.define_parameters inout;
@@ -108,10 +126,12 @@ val rebuilder :
     (Ast_cocci.meta_name rmcode) ->
     (string rmcode) ->
     (Ast_cocci.constant rmcode) ->
-    (Ast_cocci.assignOp rmcode) ->
+    (Ast_cocci.simpleAssignOp rmcode) ->
+    (Ast_cocci.arithOp rmcode) ->
     (Ast_cocci.fixOp rmcode) ->
     (Ast_cocci.unaryOp rmcode) ->
-    (Ast_cocci.binaryOp rmcode) ->
+    (Ast_cocci.arithOp rmcode) ->
+    (Ast_cocci.logicalOp rmcode) ->
     (Ast_cocci.const_vol rmcode) ->
     (Ast_cocci.sign rmcode) ->
     (Ast_cocci.structUnion rmcode) ->
@@ -120,14 +140,20 @@ val rebuilder :
     (Ast_cocci.expression Ast_cocci.dots rcode) ->
     (Ast_cocci.parameterTypeDef Ast_cocci.dots rcode) ->
     (Ast_cocci.statement Ast_cocci.dots rcode) ->
-    (Ast_cocci.declaration Ast_cocci.dots rcode) ->
+    (Ast_cocci.annotated_decl Ast_cocci.dots rcode) ->
+    (Ast_cocci.initialiser Ast_cocci.dots rcode) ->
     (Ast_cocci.ident rcode) ->
     (Ast_cocci.expression rcode) ->
+    (Ast_cocci.string_fragment rcode) ->
+    (Ast_cocci.string_format rcode) ->
+    (Ast_cocci.assignOp rcode) ->
+    (Ast_cocci.binaryOp rcode) ->
     (Ast_cocci.fullType rcode) ->
     (Ast_cocci.typeC rcode) ->
     (Ast_cocci.initialiser rcode) ->
     (Ast_cocci.parameterTypeDef rcode) ->
     (Ast_cocci.declaration rcode) ->
+    (Ast_cocci.annotated_decl rcode) ->
     (Ast_cocci.rule_elem rcode) ->
     (Ast_cocci.statement rcode) ->
     (Ast_cocci.case_line rcode) ->
