@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./visitor_ast0.mli"
 val combiner_functions : 'a Visitor_ast0_types.combiner_functions
 val combiner :
   ('a -> 'a -> 'a) ->
@@ -33,10 +36,12 @@ val flat_combiner :
     ((Ast_cocci.meta_name,'a) Visitor_ast0_types.flat_cmcode) ->
     ((string,'a) Visitor_ast0_types.flat_cmcode) ->
     ((Ast_cocci.constant,'a) Visitor_ast0_types.flat_cmcode) ->
-    ((Ast_cocci.assignOp,'a) Visitor_ast0_types.flat_cmcode) ->
+    ((Ast0_cocci.simpleAssignOp,'a) Visitor_ast0_types.flat_cmcode) ->
+    ((Ast_cocci.arithOp,'a) Visitor_ast0_types.flat_cmcode) ->
     ((Ast_cocci.fixOp,'a) Visitor_ast0_types.flat_cmcode) ->
     ((Ast_cocci.unaryOp,'a) Visitor_ast0_types.flat_cmcode) ->
-    ((Ast_cocci.binaryOp,'a) Visitor_ast0_types.flat_cmcode) ->
+    ((Ast_cocci.arithOp,'a) Visitor_ast0_types.flat_cmcode) ->
+    ((Ast_cocci.logicalOp,'a) Visitor_ast0_types.flat_cmcode) ->
     ((Ast_cocci.const_vol,'a) Visitor_ast0_types.flat_cmcode) ->
     ((Ast_cocci.sign,'a) Visitor_ast0_types.flat_cmcode) ->
     ((Ast_cocci.structUnion,'a) Visitor_ast0_types.flat_cmcode) ->
@@ -51,12 +56,16 @@ val flat_combiner :
     ((Ast0_cocci.case_line Ast0_cocci.dots,'a) Visitor_ast0_types.ccode) ->
     ((Ast0_cocci.ident,'a) Visitor_ast0_types.ccode) ->
     ((Ast0_cocci.expression,'a) Visitor_ast0_types.ccode) ->
+    ((Ast0_cocci.assignOp,'a) Visitor_ast0_types.ccode) ->
+    ((Ast0_cocci.binaryOp,'a) Visitor_ast0_types.ccode) ->
     ((Ast0_cocci.typeC,'a) Visitor_ast0_types.ccode) ->
     ((Ast0_cocci.initialiser,'a) Visitor_ast0_types.ccode) ->
     ((Ast0_cocci.parameterTypeDef,'a) Visitor_ast0_types.ccode) ->
     ((Ast0_cocci.declaration,'a) Visitor_ast0_types.ccode) ->
     ((Ast0_cocci.statement,'a) Visitor_ast0_types.ccode) ->
+    ((Ast0_cocci.forinfo,'a) Visitor_ast0_types.ccode) ->
     ((Ast0_cocci.case_line,'a) Visitor_ast0_types.ccode) ->
+    ((Ast0_cocci.string_fragment,'a) Visitor_ast0_types.ccode) ->
     ((Ast0_cocci.top_level,'a) Visitor_ast0_types.ccode) ->
     'a Visitor_ast0_types.combiner_rec_functions
 
@@ -68,10 +77,12 @@ val flat_rebuilder :
     (Ast_cocci.meta_name Visitor_ast0_types.rmcode) ->
     (string Visitor_ast0_types.rmcode) ->
     (Ast_cocci.constant Visitor_ast0_types.rmcode) ->
-    (Ast_cocci.assignOp Visitor_ast0_types.rmcode) ->
+    (Ast0_cocci.simpleAssignOp Visitor_ast0_types.rmcode) ->
+    (Ast_cocci.arithOp Visitor_ast0_types.rmcode) ->
     (Ast_cocci.fixOp Visitor_ast0_types.rmcode) ->
     (Ast_cocci.unaryOp Visitor_ast0_types.rmcode) ->
-    (Ast_cocci.binaryOp Visitor_ast0_types.rmcode) ->
+    (Ast_cocci.arithOp Visitor_ast0_types.rmcode) ->
+    (Ast_cocci.logicalOp Visitor_ast0_types.rmcode) ->
     (Ast_cocci.const_vol Visitor_ast0_types.rmcode) ->
     (Ast_cocci.sign Visitor_ast0_types.rmcode) ->
     (Ast_cocci.structUnion Visitor_ast0_types.rmcode) ->
@@ -85,12 +96,16 @@ val flat_rebuilder :
     (Ast0_cocci.case_line Ast0_cocci.dots Visitor_ast0_types.rcode) ->
     (Ast0_cocci.ident Visitor_ast0_types.rcode) ->
     (Ast0_cocci.expression Visitor_ast0_types.rcode) ->
+    (Ast0_cocci.assignOp Visitor_ast0_types.rcode) ->
+    (Ast0_cocci.binaryOp Visitor_ast0_types.rcode) ->
     (Ast0_cocci.typeC Visitor_ast0_types.rcode) ->
     (Ast0_cocci.initialiser Visitor_ast0_types.rcode) ->
     (Ast0_cocci.parameterTypeDef Visitor_ast0_types.rcode) ->
     (Ast0_cocci.declaration Visitor_ast0_types.rcode) ->
     (Ast0_cocci.statement Visitor_ast0_types.rcode) ->
+    (Ast0_cocci.forinfo Visitor_ast0_types.rcode) ->
     (Ast0_cocci.case_line Visitor_ast0_types.rcode) ->
+    (Ast0_cocci.string_fragment Visitor_ast0_types.rcode) ->
     (Ast0_cocci.top_level Visitor_ast0_types.rcode) ->
       Visitor_ast0_types.rebuilder_rec_functions
 
