@@ -1,5 +1,9 @@
 (*
- * Copyright 2005-2010, Ecole des Mines de Nantes, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
+ * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
+ * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
  * This file is part of Coccinelle.
  *
@@ -20,6 +24,7 @@
  *)
 
 
+# 0 "./visitor_ast0_types.ml"
 module Ast0 = Ast0_cocci
 module Ast = Ast_cocci
 
@@ -28,6 +33,8 @@ type ('a,'n) inout = 'a -> ('n * 'a)
 type 'n all_functions =
     {ident : (Ast0.ident,'n) inout;
       expression : (Ast0.expression,'n) inout;
+      assignOp : (Ast0.assignOp,'n) inout;
+      binaryOp : (Ast0.binaryOp,'n) inout;
       typeC : (Ast0.typeC,'n) inout;
       declaration : (Ast0.declaration,'n) inout;
       initialiser : (Ast0.initialiser,'n) inout;
@@ -35,7 +42,9 @@ type 'n all_functions =
       parameter : (Ast0.parameterTypeDef,'n) inout;
       parameter_list : (Ast0.parameter_list,'n) inout;
       statement : (Ast0.statement,'n) inout;
+      forinfo : (Ast0.forinfo,'n) inout;
       case_line : (Ast0.case_line,'n) inout;
+      string_fragment : (Ast0.string_fragment,'n) inout;
       top_level : (Ast0.top_level,'n) inout;
       expression_dots : (Ast0.expression Ast0.dots,'n) inout;
       statement_dots : (Ast0.statement Ast0.dots,'n) inout;
@@ -51,6 +60,8 @@ type ('a,'n) combiner_inout = 'a -> 'n
 type 'n combiner_rec_functions =
     {combiner_rec_ident : (Ast0.ident,'n) combiner_inout;
       combiner_rec_expression : (Ast0.expression,'n) combiner_inout;
+      combiner_rec_assignOp : (Ast0.assignOp,'n) combiner_inout;
+      combiner_rec_binaryOp : (Ast0.binaryOp,'n) combiner_inout;
       combiner_rec_typeC : (Ast0.typeC,'n) combiner_inout;
       combiner_rec_declaration : (Ast0.declaration,'n) combiner_inout;
       combiner_rec_initialiser : (Ast0.initialiser,'n) combiner_inout;
@@ -59,7 +70,9 @@ type 'n combiner_rec_functions =
       combiner_rec_parameter : (Ast0.parameterTypeDef,'n) combiner_inout;
       combiner_rec_parameter_list : (Ast0.parameter_list,'n) combiner_inout;
       combiner_rec_statement : (Ast0.statement,'n) combiner_inout;
+      combiner_rec_forinfo   : (Ast0.forinfo,'n) combiner_inout;
       combiner_rec_case_line : (Ast0.case_line,'n) combiner_inout;
+      combiner_rec_string_fragment : (Ast0.string_fragment,'n) combiner_inout;
       combiner_rec_top_level : (Ast0.top_level,'n) combiner_inout;
       combiner_rec_expression_dots :
 	(Ast0.expression Ast0.dots,'n) combiner_inout;
@@ -79,10 +92,12 @@ type 'n combiner_functions =
   {combiner_meta_mcode : (Ast.meta_name,'n) cmcode;
    combiner_string_mcode : (string,'n) cmcode;
    combiner_const_mcode : (Ast.constant,'n) cmcode;
-   combiner_assign_mcode : (Ast.assignOp,'n) cmcode;
+   combiner_simpleAssign_mcode : (Ast0.simpleAssignOp,'n) cmcode;
+   combiner_opAssign_mcode : (Ast.arithOp,'n) cmcode;
    combiner_fix_mcode : (Ast.fixOp,'n) cmcode;
    combiner_unary_mcode : (Ast.unaryOp,'n) cmcode;
-   combiner_binary_mcode : (Ast.binaryOp,'n) cmcode;
+   combiner_arithOp_mcode : (Ast.arithOp,'n) cmcode;
+   combiner_logicalOp_mcode : (Ast.logicalOp,'n) cmcode;
    combiner_cv_mcode : (Ast.const_vol,'n) cmcode;
    combiner_sign_mcode : (Ast.sign,'n) cmcode;
    combiner_struct_mcode : (Ast.structUnion,'n) cmcode;
@@ -96,12 +111,16 @@ type 'n combiner_functions =
    combiner_dotscasefn : (Ast0.case_line Ast0.dots,'n) ccode;
    combiner_identfn : (Ast0.ident,'n) ccode;
    combiner_exprfn : (Ast0.expression,'n) ccode;
+   combiner_assignOpfn : (Ast0.assignOp,'n) ccode;
+   combiner_binaryOpfn : (Ast0.binaryOp,'n) ccode;
    combiner_tyfn : (Ast0.typeC,'n) ccode;
    combiner_initfn : (Ast0.initialiser,'n) ccode;
    combiner_paramfn : (Ast0.parameterTypeDef,'n) ccode;
    combiner_declfn : (Ast0.declaration,'n) ccode;
    combiner_stmtfn : (Ast0.statement,'n) ccode;
+   combiner_forinfofn : (Ast0.forinfo,'n) ccode;
    combiner_casefn : (Ast0.case_line,'n) ccode;
+   combiner_string_fragmentfn : (Ast0.string_fragment,'n) ccode;
    combiner_topfn : (Ast0.top_level,'n) ccode}
 
 (* ----------------------------------------------------------------------- *)
@@ -112,6 +131,8 @@ type 'a rebuilder_inout = 'a -> 'a
 type rebuilder_rec_functions =
     {rebuilder_rec_ident : Ast0.ident rebuilder_inout;
       rebuilder_rec_expression : Ast0.expression rebuilder_inout;
+      rebuilder_rec_assignOp : Ast0.assignOp rebuilder_inout;
+      rebuilder_rec_binaryOp : Ast0.binaryOp rebuilder_inout;
       rebuilder_rec_typeC : Ast0.typeC rebuilder_inout;
       rebuilder_rec_declaration : Ast0.declaration rebuilder_inout;
       rebuilder_rec_initialiser : Ast0.initialiser rebuilder_inout;
@@ -120,7 +141,9 @@ type rebuilder_rec_functions =
       rebuilder_rec_parameter : Ast0.parameterTypeDef rebuilder_inout;
       rebuilder_rec_parameter_list : Ast0.parameter_list rebuilder_inout;
       rebuilder_rec_statement : Ast0.statement rebuilder_inout;
+      rebuilder_rec_forinfo : Ast0.forinfo rebuilder_inout;
       rebuilder_rec_case_line : Ast0.case_line rebuilder_inout;
+      rebuilder_rec_string_fragment : Ast0.string_fragment rebuilder_inout;
       rebuilder_rec_top_level : Ast0.top_level rebuilder_inout;
       rebuilder_rec_expression_dots :
 	Ast0.expression Ast0.dots rebuilder_inout;
@@ -140,10 +163,12 @@ type rebuilder_functions =
   {rebuilder_meta_mcode : Ast_cocci.meta_name rmcode;
    rebuilder_string_mcode : string rmcode;
    rebuilder_const_mcode : Ast.constant rmcode;
-   rebuilder_assign_mcode : Ast.assignOp rmcode;
+   rebuilder_simpleAssign_mcode : Ast0.simpleAssignOp rmcode;
+   rebuilder_opAssign_mcode : Ast.arithOp rmcode;
    rebuilder_fix_mcode : Ast.fixOp rmcode;
    rebuilder_unary_mcode : Ast.unaryOp rmcode;
-   rebuilder_binary_mcode : Ast.binaryOp rmcode;
+   rebuilder_arithOp_mcode : Ast.arithOp rmcode;
+   rebuilder_logicalOp_mcode : Ast.logicalOp rmcode;
    rebuilder_cv_mcode : Ast.const_vol rmcode;
    rebuilder_sign_mcode : Ast.sign rmcode;
    rebuilder_struct_mcode : Ast.structUnion rmcode;
@@ -157,12 +182,17 @@ type rebuilder_functions =
    rebuilder_dotscasefn : Ast0.case_line Ast0.dots rcode;
    rebuilder_identfn : Ast0.ident rcode;
    rebuilder_exprfn : Ast0.expression rcode;
+   rebuilder_assignOpfn : Ast0.assignOp rcode;
+   rebuilder_binaryOpfn : Ast0.binaryOp rcode;
+  
    rebuilder_tyfn : Ast0.typeC rcode;
    rebuilder_initfn : Ast0.initialiser rcode;
    rebuilder_paramfn : Ast0.parameterTypeDef rcode;
    rebuilder_declfn : Ast0.declaration rcode;
    rebuilder_stmtfn : Ast0.statement rcode;
+   rebuilder_forinfofn : Ast0.forinfo rcode;
    rebuilder_casefn : Ast0.case_line rcode;
+   rebuilder_string_fragmentfn : Ast0.string_fragment rcode;
    rebuilder_topfn : Ast0.top_level rcode}
 
 (* ----------------------------------------------------------------------- *)
@@ -176,10 +206,12 @@ type 'n combiner_rebuilder_functions =
   {combiner_rebuilder_meta_mcode : (Ast_cocci.meta_name,'n) rcmcode;
    combiner_rebuilder_string_mcode : (string,'n) rcmcode;
    combiner_rebuilder_const_mcode : (Ast.constant,'n) rcmcode;
-   combiner_rebuilder_assign_mcode : (Ast.assignOp,'n) rcmcode;
+   combiner_rebuilder_simpleAssign_mcode : (Ast0.simpleAssignOp,'n) rcmcode;
+   combiner_rebuilder_opAssign_mcode : (Ast.arithOp,'n) rcmcode;
    combiner_rebuilder_fix_mcode : (Ast.fixOp,'n) rcmcode;
    combiner_rebuilder_unary_mcode : (Ast.unaryOp,'n) rcmcode;
-   combiner_rebuilder_binary_mcode : (Ast.binaryOp,'n) rcmcode;
+   combiner_rebuilder_arithOp_mcode : (Ast.arithOp,'n) rcmcode;
+   combiner_rebuilder_logicalOp_mcode : (Ast.logicalOp,'n) rcmcode;
    combiner_rebuilder_cv_mcode : (Ast.const_vol,'n) rcmcode;
    combiner_rebuilder_sign_mcode : (Ast.sign,'n) rcmcode;
    combiner_rebuilder_struct_mcode : (Ast.structUnion,'n) rcmcode;
@@ -194,10 +226,14 @@ type 'n combiner_rebuilder_functions =
    combiner_rebuilder_dotscasefn : (Ast0.case_line Ast0.dots,'n) rccode;
    combiner_rebuilder_identfn : (Ast0.ident,'n) rccode;
    combiner_rebuilder_exprfn : (Ast0.expression,'n) rccode;
+   combiner_rebuilder_assignOpfn : (Ast0.assignOp,'n) rccode;
+   combiner_rebuilder_binaryOpfn : (Ast0.binaryOp,'n) rccode;
    combiner_rebuilder_tyfn : (Ast0.typeC,'n) rccode;
    combiner_rebuilder_initfn : (Ast0.initialiser,'n) rccode;
    combiner_rebuilder_paramfn : (Ast0.parameterTypeDef,'n) rccode;
    combiner_rebuilder_declfn : (Ast0.declaration,'n) rccode;
    combiner_rebuilder_stmtfn : (Ast0.statement,'n) rccode;
+   combiner_rebuilder_forinfofn : (Ast0.forinfo,'n) rccode;
    combiner_rebuilder_casefn : (Ast0.case_line,'n) rccode;
+   combiner_rebuilder_string_fragmentfn : (Ast0.string_fragment,'n) rccode;
    combiner_rebuilder_topfn : (Ast0.top_level,'n) rccode}
