@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./asttopopl.ml"
 module Ast = Ast_cocci
 module Past = Ast_popl
 
@@ -37,7 +40,7 @@ let term s inif =
       (match Ast.unwrap ast with
 	Ast.ExprStatement(_,_) -> Past.Atomic ast
       | Ast.Exp(_) -> Past.Atomic ast
-      | Ast.Decl(_,_,_) -> Past.Atomic ast
+      | Ast.Decl(_) -> Past.Atomic ast
       | Ast.ReturnExpr(_,_,_) -> Past.Atomic ast
       | Ast.MetaStmt(_,_,_,_) when inif -> Past.Atomic ast
       | Ast.DisjRuleElem(_) -> Past.Atomic ast
