@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./popltoctl.ml"
 module Past = Ast_popl
 module Ast = Ast_cocci
 module V = Visitor_ast
@@ -37,7 +40,7 @@ type formula =
 (* --------------------------------------------------------------------- *)
 
 let contains_modif =
-  let bind x y = x or y in
+  let bind x y = x || y in
   let option_default = false in
   let mcode r (_,_,kind,_) =
     match kind with
@@ -45,27 +48,35 @@ let contains_modif =
     | Ast.PLUS _ -> failwith "not possible"
     | Ast.CONTEXT(_,info) -> not (info = Ast.NOTHING) in
   let do_nothing r k e = k e in
+  let annotated_decl decl =
+    match Ast.unwrap decl with
+      Ast.DElem(bef,_,_) -> bef
+    | _ -> failwith "not possible" in
   let rule_elem r k re =
     let res = k re in
     match Ast.unwrap re with
-      Ast.FunHeader(bef,_,fninfo,name,lp,params,rp) ->
-      bind (mcode r ((),(),bef,Ast.NoMetaPos)) res
-    | Ast.Decl(bef,_,decl) -> bind (mcode r ((),(),bef,Ast.NoMetaPos)) res
+      Ast.FunHeader(bef,_,fninfo,name,lp,params,va,rp) ->
+      bind (mcode r ((),(),bef,[])) res
+    | Ast.Decl decl -> bind (mcode r ((),(),annotated_decl decl,[])) res
+    | Ast.ForHeader(fr,lp,Ast.ForDecl(decl),e2,sem2,e3,rp) ->
+	bind (mcode r ((),(),annotated_decl decl,[])) res
     | _ -> res in
   let recursor =
     V.combiner bind option_default
-      mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
-      do_nothing do_nothing do_nothing do_nothing do_nothing
+      mcode mcode mcode mcode mcode mcode mcode mcode mcode
+      mcode mcode mcode mcode mcode
+      do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
+      do_nothing do_nothing do_nothing do_nothing
       do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
       do_nothing rule_elem do_nothing do_nothing do_nothing do_nothing in
   recursor.V.combiner_rule_elem
 
 let ctl_exists keep_wit v x =
-  CTL.Exists(!Flag_popl.keep_all_wits or keep_wit,v,x)
+  CTL.Exists(!Flag_popl.keep_all_wits || keep_wit,v,x)
 
 let predmaker keep_wit term =
-  if (!Flag_popl.keep_all_wits or keep_wit) &&
-     (!Flag_popl.mark_all or contains_modif term)
+  if (!Flag_popl.keep_all_wits || keep_wit) &&
+     (!Flag_popl.mark_all || contains_modif term)
   then
     let v = ("","_v") in
     ctl_exists true v
@@ -236,4 +247,4 @@ and do_between_dots keep_wit ba term after =
 
 (* --------------------------------------------------------------------- *)
 
-let toctl sl = ctl_seq true ctl_true sl
+let toctl sl = Asttoctl2.CODE (ctl_seq true ctl_true sl)
