@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./pretty_print_popl.ml"
 open Format
 module Past = Ast_popl
 module Ast = Ast_cocci
@@ -47,7 +50,10 @@ let print_around printer term = function
 let mcode fn = function
     (x, _, Ast.MINUS(_,_,_,plus_stream), pos) ->
       print_string "-"; fn x;
-      Pretty_print_cocci.print_anything ">>> " plus_stream
+      (match plus_stream with
+	Ast.NOREPLACEMENT -> ()
+      | Ast.REPLACEMENT(plus_stream,_) ->
+	  Pretty_print_cocci.print_anything ">>> " plus_stream)
   | (x, _, Ast.CONTEXT(_,plus_streams), pos) ->
 	print_around fn x plus_streams
   | (x, info, Ast.PLUS _, pos) -> fn x
@@ -64,7 +70,7 @@ and print_term = function
     Past.Atomic(term) -> Pretty_print_cocci.rule_elem "" term
   | Past.IfThen(test,thn,(_,_,_,aft)) ->
       print_term test; print_term thn;
-      mcode (function _ -> ()) ((),Ast.no_info,aft,Ast.NoMetaPos)
+      mcode (function _ -> ()) ((),Ast.no_info,aft,[])
   | Past.TExists((_,v),term) -> print_string "exists "; print_string v;
       print_string " ."; force_newline(); print_term term
 
