@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./no_pycocci.ml"
 open Ast_c
 open Common
 open Pycocci_aux
@@ -29,21 +32,23 @@ module StringMap = Map.Make (String)
 
 exception Pycocciexception
 
+let errmsg = "Semantic patch uses python, but Coccinelle has been compiled without Python support"
+
 let python_support = false
 
-let check_return_value v = failwith "no python"
-let check_int_return_value v = failwith "no python"
+let check_return_value v = failwith errmsg
+let check_int_return_value v = failwith errmsg
 
 let initialised = ref false
 
 let cocci_file_name = ref ""
 
 (* dealing with python modules loaded *)
-let get_module module_name = failwith "no python"
+let get_module module_name = failwith errmsg
 
-let is_module_loaded module_name = failwith "no python"
+let is_module_loaded module_name = failwith errmsg
 
-let load_module module_name = failwith "no python"
+let load_module module_name = failwith errmsg
 (* end python module handling part *)
 
 (* initialisation routines *)
@@ -53,37 +58,40 @@ let pycocci_init () = initialised := true
 (* end initialisation routines *)
 
 (* python interaction *)
-let split_fqn fqn = failwith "no python"
+let split_fqn fqn = failwith errmsg
 
-let pycocci_get_class_type fqn = failwith "no python"
+let pycocci_get_class_type fqn = failwith errmsg
 
-let pycocci_instantiate_class fqn args = failwith "no python"
+let pycocci_instantiate_class fqn args = failwith errmsg
 
 (* end python interaction *)
 
 let inc_match = ref false
+let exited = ref false
 
-let include_match v = failwith "no python"
+let include_match v = failwith errmsg
+
+let sp_exit _ = failwith errmsg
 
 let build_method (mname, camlfunc, args) pymodule classx classdict =
-  failwith "no python"
+  failwith errmsg
 
-let build_class cname parent methods pymodule = failwith "no python"
+let build_class cname parent methods pymodule = failwith errmsg
 
-let has_environment_binding env name = failwith "no python"
+let has_environment_binding env name = failwith errmsg
 
-let get_cocci_file args = failwith "no python"
+let get_cocci_file args = failwith errmsg
 
-let build_classes env = failwith "no python"
+let build_classes env = failwith errmsg
 
-let build_variable name value = failwith "no python"
+let build_variable name value = failwith errmsg
 
-let contains_binding e (_,(r,m),_) = failwith "no python"
+let contains_binding e (_,(r,m),_) = failwith errmsg
 
-let construct_variables mv e = failwith "no python"
+let construct_variables mv e = failwith errmsg
 
-let construct_script_variables mv = failwith "no python"
-let retrieve_script_variables mv = failwith "no python"
+let construct_script_variables mv = failwith errmsg
+let retrieve_script_variables mv = failwith errmsg
 
 let set_coccifile cocci_file =
 	cocci_file_name := cocci_file;
@@ -91,10 +99,10 @@ let set_coccifile cocci_file =
 
 
 let pyrun_simplestring s =
-  failwith "no python"
+  failwith errmsg
 
 let py_isinitialized () =
-  failwith "no python"
+  failwith errmsg
 
 let py_finalize () =
-  failwith "no python"
+  failwith errmsg
