@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./pycocci.mli"
 val build_classes : Ast_cocci.meta_name list -> unit
 val construct_variables :
     (string * Ast_cocci.meta_name * Ast_cocci.metavar) list
@@ -29,6 +32,7 @@ val construct_variables :
 val construct_script_variables : Ast_cocci.meta_name list -> unit
 val pyrun_simplestring : string -> int
 val inc_match : bool ref
+val exited : bool ref
 val retrieve_script_variables : Ast_cocci.meta_name list -> string list
 exception Pycocciexception 
 val set_coccifile : string -> unit
