@@ -1,4 +1,6 @@
-# Copyright 2010, INRIA, University of Copenhagen
+# Copyright 2012-2015, Inria
+# Julia Lawall, Gilles Muller
+# Copyright 2010-2011, INRIA, University of Copenhagen
 # Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
 # Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
 # Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -119,11 +121,11 @@ foreach my $f (@files) {
   }                         
   die "PB: one of the file already exist: $base" if (-e "$target_dir/$base");
 
-  `git-cat-file blob $commit_old:$f > $target_dir/$base`;
-  `git-cat-file blob $commit_new:$f > $target_dir/$res`;
+  `git cat-file blob $commit_old:$f > $target_dir/$base`;
+  `git cat-file blob $commit_new:$f > $target_dir/$res`;
 
-  `git-cat-file blob $commit_old:$f > $old_dir/$base`;
-  `git-cat-file blob $commit_new:$f > $new_dir/$base`;
+  `git cat-file blob $commit_old:$f > $old_dir/$base`;
+  `git cat-file blob $commit_new:$f > $new_dir/$base`;
 
   $kerneldir_of_file->{$base} = `dirname $f`;
   chomp $kerneldir_of_file->{$base};
@@ -174,13 +176,13 @@ foreach my $f (@driverheaders_in_include) {
     my $dir = `dirname $f`;
     chomp $dir;
     `mkdir -p $target_dir/$dir`;
-    `git-cat-file blob $commit_old:$f > $target_dir/$f`;
-    `git-cat-file blob $commit_new:$f > $target_dir/$f.res`;
+    `git cat-file blob $commit_old:$f > $target_dir/$f`;
+    `git cat-file blob $commit_new:$f > $target_dir/$f.res`;
 
     `mkdir -p $old_dir/$dir`;
     `mkdir -p $new_dir/$dir`;
-    `git-cat-file blob $commit_old:$f > $old_dir/$f`;
-    `git-cat-file blob $commit_new:$f > $new_dir/$f`;
+    `git cat-file blob $commit_old:$f > $old_dir/$f`;
+    `git cat-file blob $commit_new:$f > $new_dir/$f`;
     
   }
 }
@@ -205,7 +207,7 @@ foreach my $line (@linuxheaders) {
       my $dir = `dirname $f`;
       chomp $dir;
       `mkdir -p $target_dir/include/$dir`;
-      `git-cat-file blob $commit_old:include/$f > $target_dir/include/$f`;
+      `git cat-file blob $commit_old:include/$f > $target_dir/include/$f`;
     }
   } else { pr2 "pb regexp: $line"; }
 }
@@ -248,6 +250,6 @@ foreach my $h (keys %{$hfiles}) {
     pr2 "BUT I CONTINUE, but may have more .failed in the end";
     pr2 "-------------------------------------";
   } else {
-    `git-cat-file blob $commit_old:$h > $target_dir/$base`;
+    `git cat-file blob $commit_old:$h > $target_dir/$base`;
   }
 }
