@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,8 +24,8 @@
  *)
 
 
+# 0 "./testing.ml"
 open Common
-open Sexplib
 
 (*****************************************************************************)
 (* Test framework *)
@@ -44,7 +46,7 @@ let testone prefix x compare_with_expected_flag =
 
   let expected_res   = prefix ^ x ^ ".res" in
   begin
-    let cocci_infos = Cocci.pre_engine (cocci_file, !Config.std_iso) in
+    let (cocci_infos,_) = Cocci.pre_engine (cocci_file, !Config.std_iso) in
     let res = Cocci.full_engine cocci_infos [cfile] in
     Cocci.post_engine cocci_infos;
     let generated =
@@ -52,15 +54,15 @@ let testone prefix x compare_with_expected_flag =
       | Some (Some outfile) ->
           if List.length res > 1
           then pr2 ("note that not just " ^ cfile ^ " was involved");
-
-          let tmpfile = "/tmp/"^Common.basename cfile in
+          let tmpfile =
+	    sprintf "%s/%s" Config.get_temp_dir_name (Common.basename cfile) in
           pr2 (sprintf "One file modified. Result is here: %s" tmpfile);
           Common.command2 ("mv "^outfile^" "^tmpfile);
           tmpfile
       | Some None ->
           pr2 "no modification on the input file";
           cfile
-      | None -> raise Impossible
+      | None -> raise (Impossible 163)
     in
     if compare_with_expected_flag
     then
@@ -75,13 +77,16 @@ let testone prefix x compare_with_expected_flag =
  * it is possible that a test file work in -test but may not
  * work while used inside a -testall. If we have some bugs in our
  * parser that modify some global state and that those states
- * are not reseted between each test file, then having run previous
+ * are not reset between each test file, then having run previous
  * test files may have an influence on another test file which mean
  * than a test may work in isolation (via -test) but not otherwise
  * (via -testall). Fortunately such bugs are rare.
  *
  *)
-let testall ?(expected_score_file="tests/SCORE_expected.sexp") () =
+(* If extra test is provided, then all failing tests with the standard
+   comparison are considered ok, and only the correct result are subjected to
+   the extra test *)
+let testall_bis extra_test expected_score_file update_score_file =
 
   let score  = empty_score () in
 
@@ -95,20 +100,23 @@ let testall ?(expected_score_file="tests/SCORE_expected.sexp") () =
   begin
     expected_result_files +> List.iter (fun res ->
       let x =
-        if res =~ "\\(.*\\).res" then matched1 res else raise Impossible in
+        if res =~ "\\(.*\\).res"
+	then matched1 res
+	else raise (Impossible 164) in
       let base = if x =~ "\\(.*\\)_ver[0-9]+" then matched1 x else x in
       let cfile      = "tests/" ^ x ^ ".c" in
       let cocci_file = "tests/" ^ base ^ ".cocci" in
       let expected = "tests/" ^ res in
 
-      let timeout_testall = 30 in
+      let timeout_testall = 60 in
 
       try (
         Common.timeout_function timeout_testall  (fun () ->
 
 	  pr2 res;
 
-	  let cocci_infos = Cocci.pre_engine (cocci_file, !Config.std_iso) in
+	  let (cocci_infos,_) =
+	    Cocci.pre_engine (cocci_file, !Config.std_iso) in
           let xs = Cocci.full_engine cocci_infos [cfile] in
 	  Cocci.post_engine cocci_infos;
 
@@ -118,8 +126,19 @@ let testall ?(expected_score_file="tests/SCORE_expected.sexp") () =
             | None -> cfile
           in
 
-          let (correct, diffxs) = Compare_c.compare_default generated expected
-          in
+          let (correct, diffxs) =
+	    Compare_c.compare_default generated expected in
+
+	  let (correct, diffxs) =
+	    match extra_test with
+	      None -> (correct, diffxs)
+	    | Some extra_test ->
+		(match correct with
+		  Compare_c.Correct -> extra_test generated expected
+		| _ ->
+		    (* if there is an extra test, we don't care about the
+		       things that fail on the first test *)
+		    (Compare_c.Correct,[])) in
 
           (* I don't use Compare_c.compare_result_to_string because
            * I want to indent a little more the messages.
@@ -131,8 +150,10 @@ let testall ?(expected_score_file="tests/SCORE_expected.sexp") () =
                 (Str.regexp "\"/tmp/cocci-output.*\"") "<COCCIOUTPUTFILE>" s
               in
               (* on macos the temporary files are stored elsewhere *)
-              let s = Str.global_replace
-                (Str.regexp "\"/var/folders/.*/cocci-output.*\"") "<COCCIOUTPUTFILE>" s
+              let s =
+		Str.global_replace
+                  (Str.regexp "\"/var/folders/.*/cocci-output.*\"")
+		  "<COCCIOUTPUTFILE>" s
               in
               let s =
                 "INCORRECT:" ^ s ^ "\n" ^
@@ -174,9 +195,6 @@ let testall ?(expected_score_file="tests/SCORE_expected.sexp") () =
     pr2 "regression testing  information";
     pr2 "--------------------------------";
 
-    (* now default argument of testall:
-    let expected_score_file = "tests/SCORE_expected.sexp" in
-    *)
     let expected_score_file_orig = "tests/SCORE_expected_orig.sexp" in
     let best_of_both_file = "tests/SCORE_best_of_both.sexp" in
     let actual_score_file = "tests/SCORE_actual.sexp" in
@@ -185,16 +203,22 @@ let testall ?(expected_score_file="tests/SCORE_expected.sexp") () =
     let (expected_score : score) =
       if Sys.file_exists expected_score_file
       then
+        Common.load_score expected_score_file ()
+        (*
         let sexp = Sexp.load_sexp expected_score_file in
         Sexp_common.score_of_sexp sexp
+        *)
       else
         if Sys.file_exists expected_score_file_orig
         then begin
           pr2 (spf "use expected orig file (%s)" expected_score_file_orig);
           Common.command2 (spf "cp %s %s" expected_score_file_orig
                                           expected_score_file);
-          let sexp = Sexp.load_sexp expected_score_file in
+          (*
+	  let sexp = Sexp.load_sexp expected_score_file in
           Sexp_common.score_of_sexp sexp
+	  *)
+	  Common.load_score expected_score_file ()
         end
        else
           empty_score()
@@ -202,16 +226,21 @@ let testall ?(expected_score_file="tests/SCORE_expected.sexp") () =
 
     let new_bestscore = Common.regression_testing_vs score expected_score in
 
-
+    (*
     let xs = Common.hash_to_list score in
     let sexp = Sexp_common.sexp_of_score_list xs in
     let s_score = Sexp.to_string_hum sexp in
     Common.write_file ~file:(actual_score_file) s_score;
+    *)
+    Common.save_score score actual_score_file;
 
+    (*
     let xs2 = Common.hash_to_list new_bestscore in
     let sexp2 = Sexp_common.sexp_of_score_list xs2 in
     let s_score2 = Sexp.to_string_hum sexp2 in
     Common.write_file ~file:(best_of_both_file) s_score2;
+    *)
+    Common.save_score new_bestscore best_of_both_file;
 
     Common.print_total_score score;
 
@@ -237,16 +266,32 @@ let testall ?(expected_score_file="tests/SCORE_expected.sexp") () =
       else begin
         pr2 "Current score is greater than expected :)";
         pr2 (spf "(was expecting %d but got %d)" expected_good good);
-        pr2 "Generating new expected score file and saving old one";
-        Common.command2_y_or_no_exit_if_no
-          (spf "mv %s %s" expected_score_file (expected_score_file ^ ".save"));
-        Common.command2_y_or_no_exit_if_no
-          (spf "mv %s %s" best_of_both_file expected_score_file);
+        if update_score_file then
+        begin
+          pr2 "Generating new expected score file and saving old one";
+          Common.command2_y_or_no_exit_if_no
+            (spf "mv %s %s" expected_score_file (expected_score_file ^ ".save"));
+          Common.command2_y_or_no_exit_if_no
+            (spf "mv %s %s" best_of_both_file expected_score_file);
+        end;
+
+        (* when there are sufficient number of tests, abort if a substantial
+         * amount of tests fail, which would indicate a broken build.
+         *)
+        if total > 40 && good < (total * 3) / 4
+        then begin
+	  pr2 "Still, less 75% the tests passed. Returning a nonzero exist status.";
+          raise (UnixExit 1);
+        end;
+
         raise (UnixExit 0);
       end
 
   end
 
+let testall = testall_bis None
+let test_spacing = testall_bis (Some Compare_c.exact_compare)
+
 (* ------------------------------------------------------------------------ *)
 
 type okfailed = Ok | SpatchOK | Failed
@@ -259,7 +304,7 @@ let t_to_s = function
 
 let delete_previous_result_files infile =
   [Ok;SpatchOK;Failed] +> List.iter (fun kind ->
-    Common.command2 ("rm -f " ^ infile ^ t_to_s kind)
+    Common.remove_file (infile ^ t_to_s kind)
   )
 
 (* quite similar to compare_with_expected  below *)
@@ -267,7 +312,7 @@ let test_okfailed cocci_file cfiles =
   cfiles +> List.iter delete_previous_result_files;
 
   (* final_files contain the name of an output file (a .ok or .failed
-   * or .spatch_ok), and also some additionnal strings to be printed in
+   * or .spatch_ok), and also some additional strings to be printed in
    * this output file in addition to the general error message of
    * full_engine. *)
   let final_files = ref [] in
@@ -289,7 +334,7 @@ let test_okfailed cocci_file cfiles =
     try (
       Common.timeout_function_opt !Flag_cocci.timeout (fun () ->
 
-	let cocci_infos = Cocci.pre_engine (cocci_file, !Config.std_iso) in
+	let (cocci_infos,_) = Cocci.pre_engine (cocci_file, !Config.std_iso) in
         let outfiles = Cocci.full_engine cocci_infos cfiles in
 	Cocci.post_engine cocci_infos;
 
@@ -382,11 +427,11 @@ let test_regression_okfailed () =
 
   let newscore  = Common.empty_score () in
   let oks =
-    Common.cmd_to_list ("find -name \"*.ok\"")
+    Common.cmd_to_list ("find . -name \"*.ok\"")
     ++
-    Common.cmd_to_list ("find -name \"*.spatch_ok\"")
+    Common.cmd_to_list ("find . -name \"*.spatch_ok\"")
   in
-  let failed = Common.cmd_to_list ("find -name \"*.failed\"") in
+  let failed = Common.cmd_to_list ("find . -name \"*.failed\"") in
 
   if null (oks ++ failed)
   then failwith "no ok/failed file, you certainly did a make clean"
@@ -454,11 +499,10 @@ let compare_with_expected outfiles =
 (*****************************************************************************)
 
 let test_parse_cocci file =
-  Flag_parsing_cocci.show_SP := true;
   if not (file =~ ".*\\.cocci")
   then pr2 "warning: seems not a .cocci file";
 
-  let (_,xs,_,_,_,_,(grep_tokens,query,_)) =
+  let (_,xs,_,_,_,_,(grep_tokens,query,_,_),_) =
     Parse_cocci.process file (Some !Config.std_iso) false in
   xs +> List.iter Pretty_print_cocci.unparse;
   Format.print_newline();
@@ -477,16 +521,62 @@ let test_parse_cocci file =
   (match grep_tokens with
     None -> pr "No query"
   | Some x -> pr (String.concat " || " x));
-  match !Flag.scanner with
-    Flag.NoScanner | Flag.Grep -> ()
-  | Flag.Glimpse | Flag.IdUtils | Flag.Google _ ->
-      Printf.printf "%s tokens\n"
-	(if !Flag.scanner = Flag.Glimpse then "glimpse" else "google");
-      (match query with
-	None -> pr "No query"
-      | Some x -> pr (String.concat "\nor on glimpse failure\n" x))
+  (* could update to accomodate WTCocciGrep *)
+  (match query with
+    None -> pr "No query"
+  | Some x ->
+      Printf.printf "glimpse tokens\n";
+      pr (String.concat "\nor on glimpse failure\n" x))
 
+let print_link t a b =
+  if not (a = b)
+  then
+    (try Hashtbl.find t (a,b)
+    with Not_found ->
+      (Hashtbl.add t (a,b) ();
+       Printf.printf "  \"%s\" -> \"%s\";\n" b a))
 
+let print_dotted_link dst = function
+    "" -> ()
+  | src -> Printf.printf "  \"%s\" -> \"%s\" [style = dotted];\n" src dst
+
+let rec depto t from = function
+    Ast_cocci.Dep x | Ast_cocci.EverDep x | Ast_cocci.NeverDep x ->
+      print_link t from x
+  | Ast_cocci.AndDep(x,y) | Ast_cocci.OrDep(x,y) ->
+      depto t from x; depto t from y
+  | _ -> ()
+
+let test_rule_dependencies file =
+  let t = Hashtbl.create 101 in
+  if not (file =~ ".*\\.cocci")
+  then pr2 "warning: seems not a .cocci file";
+  Iso_pattern.verbose_iso := false;
+  let (_,xs,fvs,_,_,_,_,_) =
+    Parse_cocci.process file (Some !Config.std_iso) false in
+  Printf.printf "digraph {\n";
+  let prevrule = ref "" in
+  List.iter2
+    (fun def fvs ->
+      match def with
+	Ast_cocci.ScriptRule (nm,_,dep,script_vars,_,_) ->
+	  print_dotted_link nm !prevrule;
+	  prevrule := nm;
+	  depto t nm dep;
+	  List.iter (function (_,(parent,_),_) -> print_link t nm parent)
+	    script_vars
+      | Ast_cocci.InitialScriptRule (_,_,_,_,_)
+      | Ast_cocci.FinalScriptRule (_,_,_,_,_) -> ()
+      | Ast_cocci.CocciRule (nm,(dep,_,_),_,_,_) ->
+	  print_dotted_link nm !prevrule;
+	  prevrule := nm;
+	  depto t nm dep;
+	  List.iter (function (parent,_) -> print_link t nm parent)
+	    (List.concat fvs))
+    xs fvs;
+  Printf.printf "}\n";
+  Printf.printf
+    "// pipe to: ccomps -Cx | dot | gvpack -array_1 | neato -n2 -T pdf\n"
 
 (*****************************************************************************)
 (* to be called by ocaml toplevel, to test. *)
