@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,13 +24,15 @@
  *)
 
 
+# 0 "./testing.mli"
 open Common
 
 (*****************************************************************************)
 (* work with tests/ *)
 (*****************************************************************************)
 val testone : string (*prefix*) -> string (*test*) -> bool (*compare_expected*) -> unit
-val testall : ?expected_score_file:string -> unit -> unit
+val testall : string -> bool -> unit
+val test_spacing : string -> bool -> unit
 
 (*****************************************************************************)
 (* works with tests-big/. The .res, .ok, .spatch_ok, .failed, .var *)
@@ -54,6 +58,7 @@ val compare_with_expected : (filename * filename option) list -> unit
  *)
 
 val test_parse_cocci : filename -> unit
+val test_rule_dependencies : filename -> unit
 
 (*****************************************************************************)
 (* to be called by ocaml toplevel, to test. *)
@@ -70,8 +75,11 @@ val sp_of_file :
 	 (*fresh used after list seeds*)
 	 Ast_cocci.meta_name list list list) *
       Ast_cocci.meta_name list list list *
-      string list option *
-      string list option
+      (string list option *
+	 string list option *
+	 (Str.regexp * Str.regexp list) option *
+	 Get_constants2.combine option) *
+      bool (* format info needed for strings *)
 
 (* TODO: Remove
 val rule_elem_of_string : string -> filename option -> Ast_cocci.rule_elem
