@@ -1,5 +1,9 @@
 (*
- * Copyright 2005-2010, Ecole des Mines de Nantes, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
+ * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
+ * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
  * This file is part of Coccinelle.
  *
@@ -20,6 +24,7 @@
  *)
 
 
+# 0 "./alloc_free.ml"
 (* The following finds out for each file, how it does deallocation for each
 allocator *)
 
@@ -189,7 +194,7 @@ let _ =
   then
     begin
       let o = open_out (Printf.sprintf "%s/files" !dir) in
-      Printf.fprintf o "#!/bin/sh\n\n";
+      Printf.fprintf o "#! /bin/sh\n\n";
       sedify o !gen !dir l;
       sedify_ors o !gen !dir l;
       Printf.fprintf o "\nwait\n/bin/rm tmp*out\n";
