@@ -1,5 +1,7 @@
 (*
- * Copyright 2010, INRIA, University of Copenhagen
+ * Copyright 2012-2015, Inria
+ * Julia Lawall, Gilles Muller
+ * Copyright 2010-2011, INRIA, University of Copenhagen
  * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
  * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
@@ -22,6 +24,7 @@
  *)
 
 
+# 0 "./splitpatch.ml"
 (* split patch per file *)
 
 (* ------------------------------------------------------------------------ *)
@@ -32,6 +35,7 @@ create a .splitpatch file in your home directory. *)
 let from = ref "email@xyz.org"
 let git_tree = ref "/var/linuxes/linux-next"
 let git_options = ref "--cc=kernel-janitors@vger.kernel.org --suppress-cc=self"
+let not_linux = ref "--suppress-cc=self"
 let prefix_before = ref (Some "/var/linuxes/linux-next")
 let prefix_after = ref (Some "/var/julia/linuxcopy")
 
@@ -60,6 +64,17 @@ let safe_get_extension s =
     ext::_::rest -> Some (String.concat "." (List.rev rest))
   | _ -> None
 
+let intersect l1 l2 =
+  List.rev
+    (List.fold_left (fun i cur -> if List.mem cur l2 then cur :: i else i)
+       [] l1)
+
+let union l1 l2 =
+  List.rev
+    (List.fold_left
+       (fun i cur -> if not(List.mem cur l2) then cur :: i else i)
+       l2 l1)
+
 (* ------------------------------------------------------------------------ *)
 (* set configuration variables *)
 
@@ -119,7 +134,7 @@ let read_configs template =
       (match Str.bounded_split (Str.regexp "[ \t]*=[ \t]*") l 2 with
 	["from";s] -> from := s
       | ["git_tree";s] -> temporary_git_tree := Some s
-      | ["git_options";s] -> git_options := s
+      | ["git_options";s] -> git_options := s; not_linux := s
       | ["prefix_before";s] -> prefix_before := Some s
       | ["prefix_after";s] -> prefix_after := Some s
       | _ -> Printf.fprintf stderr "unknown line: %s\n" l);
@@ -133,14 +148,27 @@ let read_configs template =
 
 let maintainer_command file =
   Printf.sprintf
-    "cd %s; scripts/get_maintainer.pl --separator , --nogit --norolestats -f %s"
+    "cd %s; scripts/get_maintainer.pl --nokeywords --separator , --nogit --nogit-fallback --norolestats -f %s"
+    !git_tree file
+
+let maintainer_list_command file =
+  Printf.sprintf
+    "cd %s; scripts/get_maintainer.pl --nokeywords --nogit --nogit-fallback --norolestats --nom -f %s"
     !git_tree file
 
 let subsystem_command file =
   Printf.sprintf
-    "cd %s; scripts/get_maintainer.pl --nogit --subsystem --norolestats -f %s | grep -v @"
+    "cd %s; scripts/get_maintainer.pl --nokeywords --nogit-fallback --subsystem --norolestats -f %s | grep -v @"
     !git_tree file
 
+let subject_command file =
+  Printf.sprintf "cd %s; git log --pretty=oneline --abbrev-commit %s"
+    !git_tree file
+
+let affected_files sha =
+  Printf.sprintf "cd %s; git show --oneline --name-only %s"
+    !git_tree sha
+
 let checkpatch_command file =
   Printf.sprintf "cd %s; scripts/checkpatch.pl %s" !git_tree file
 
@@ -174,14 +202,57 @@ let get_template_information file =
     [subject] ->
       let cover = read_up_to_dashes i in
       let message = read_up_to_dashes i in
+      let nonmessage = read_up_to_dashes i in
       if message = []
-      then (subject,None,cover)
-      else (subject,Some cover,message)
+      then (subject,None,cover,[])
+      else (subject,Some cover,message,nonmessage)
   | _ ->
       failwith
 	("Subject must be exactly one line "^
 	 (string_of_int (List.length subject)))
 
+(* ------------------------------------------------------------------------ *)
+(* ------------------------------------------------------------------------ *)
+(* Info processing *)
+
+let get_info info_file =
+  let tbl = Hashtbl.create 101 in
+  (if Sys.file_exists info_file
+  then
+    let lines = cmd_to_list (Printf.sprintf "grep ^## %s" info_file) in
+    List.iter
+      (function line ->
+	if Str.string_match (Str.regexp "## *") line 0
+	then
+	  let line =
+	    let match_end = Str.match_end () in
+	    String.sub line match_end
+	      (String.length line - match_end) in
+	  try
+	    let line =
+	      match !prefix_before with
+		None -> line
+	      | Some pb ->
+		  let pb = pb^"/" in
+		  String.concat "" (Str.split (Str.regexp pb) line) in
+	    let end_of_file =
+	      Str.search_forward (Str.regexp " *: *") line 0 in
+	    let match_end = Str.match_end () in
+	    let file = String.sub line 0 end_of_file in
+	    let info =
+	      String.sub line match_end (String.length line - match_end) in
+	    let cell =
+	      try Hashtbl.find tbl file
+	      with Not_found ->
+		let cell = ref [] in
+		Hashtbl.add tbl file cell;
+		cell in
+	    cell := info :: !cell
+	  with Not_found ->
+	    Printf.fprintf stderr "no file found in %s\n" line)
+      lines);
+  tbl
+
 (* ------------------------------------------------------------------------ *)
 (* ------------------------------------------------------------------------ *)
 (* Patch processing *)
@@ -233,9 +304,10 @@ let split_patch i =
 	cur := l :: !cur;
 	read_hunk msize psize
     | "\\"::_ -> cur := l :: !cur; read_diff_or_atat()
-    | _ ->
+    | _ -> read_diff_or_atat()
+	(*
 	failwith
-	  "expected diff or @@ (diffstat information should not be present)"
+	  "expected diff or @@ (diffstat information should not be present)" *)
   and read_diff _ =
     let l = input_line i in
     let l = fix_date(fix_before_after l "a" !prefix_before) in
@@ -259,6 +331,32 @@ let split_patch i =
 
 (* ------------------------------------------------------------------------ *)
 
+let uctr = ref 0
+
+let found_a_maintainer = ref false
+
+let common_prefix l1 l2 =
+  let rec loop = function
+      ([],_) | (_,[]) -> []
+    | (x::xs,y::ys) when x = y -> x :: (loop (xs,ys))
+    | _ -> [] in
+  match loop (l1,l2) with
+    [] -> None
+  | res -> Some (String.concat "/" res)
+
+let find_common_path file cell =
+  let fs = Str.split (Str.regexp "/") file in
+  let rec loop = function
+      [] ->
+	let c1 = ref [] in
+	cell := ((ref file),c1)::!cell;
+	c1
+    | (f,c1)::xs ->
+	(match common_prefix fs (Str.split (Str.regexp "/") !f) with
+	  None -> loop xs
+	| Some cp -> f := cp; c1) in
+  loop !cell
+
 let resolve_maintainers patches =
   let maintainer_table = Hashtbl.create (List.length patches) in
   List.iter
@@ -269,7 +367,18 @@ let resolve_maintainers patches =
 	      (match Str.split spaces after with
 		file::_ ->
 		  let maintainers =
-		    List.hd (cmd_to_list (maintainer_command file)) in
+		    match (cmd_to_list (maintainer_command file)) with
+		      m::_ -> found_a_maintainer := true; m
+		    | [] ->
+			(* maybe the file is new? *)
+			(match
+			  (cmd_to_list
+			     (maintainer_command (Filename.dirname file)))
+			with
+			  m::_ -> found_a_maintainer := true; m
+			| [] ->
+			    uctr := !uctr + 1;
+			    "unknown"^(string_of_int !uctr)) in
 		  let subsystems =
 		    cmd_to_list (subsystem_command file) in
 		  let info = (subsystems,maintainers) in
@@ -279,7 +388,8 @@ let resolve_maintainers patches =
 		      let cell = ref [] in
 		      Hashtbl.add maintainer_table info cell;
 		      cell in
-		  cell := (file,(diff_line :: rest)) :: !cell
+		  let cell1 = find_common_path file cell in
+		  cell1 := (file,(diff_line :: rest)) :: !cell1
 	      |	_ -> failwith "filename not found")
 	  | _ ->
 	      failwith (Printf.sprintf "prefix a/ not found in %s" diff_line))
@@ -288,37 +398,112 @@ let resolve_maintainers patches =
   maintainer_table
 
 (* ------------------------------------------------------------------------ *)
+(* most common subject from the git logs *)
 
-let common_prefix l1 l2 =
-  let rec loop = function
-      ([],_) | (_,[]) -> []
-    | (x::xs,y::ys) when x = y -> x :: (loop (xs,ys))
-    | _ -> [] in
-  match loop (l1,l2) with
-    [] ->
-      failwith
-	(Printf.sprintf "found nothing in common for %s and %s"
-	   (String.concat "/" l1) (String.concat "/" l2))
-  | res -> res
+let last_char s = String.get s ((String.length s) - 1)
 
-let merge_files the_rest = function
-    [l] -> l
-  | files ->
-      let butlast l = if the_rest then l else List.rev(List.tl(List.rev l)) in
-      match List.map (function s -> Str.split (Str.regexp "/") s) files with
-	first::rest ->
-	  let rec loop res = function
-	      [] -> String.concat "/" res
-	    | x::rest -> loop (common_prefix res x) rest in
-	  loop (butlast first) rest
-      | _ -> failwith "not possible"
+let get_counts l =
+(*Printf.printf "getting counts for:\n";
+  List.iter
+    (function (bonus,x) ->
+      Printf.printf "  %d: %s\n" bonus (String.concat " " x))
+    l;
+  Printf.printf "\n"; *)
+  let tbl = Hashtbl.create 101 in
+  let ct = ref 0 in
+  let max = ((List.length l) / 10) + 1 in (* 10 ranges *)
+  List.iter
+    (function file ->
+      ct := !ct + 1;
+      let cell =
+	try Hashtbl.find tbl file
+	with Not_found ->
+	  let cell = ref 0.0 in
+	  Hashtbl.add tbl file cell;
+	  cell in
+      (* only use 5 ranges *)
+      let weight = 1. /. (float_of_int ((!ct / (max * 2)) + 1)) in
+      cell := !cell +. weight)
+    l;
+  let weighted =
+    List.rev
+      (List.sort compare
+	 (Hashtbl.fold (fun k v rest -> (!v,k)::rest) tbl [])) in
+(*List.iter
+    (function (wt,(bonus,x)) ->
+      Printf.printf "  %f: %d: %s\n" wt bonus (String.concat " " x))
+    weighted; *)
+  let rec loop n = function
+      [] -> []
+    | (_,k)::rest -> (k,n) :: (loop (n+1) rest) in
+  loop 1 weighted
+
+let superset l1 l2 = List.for_all (function x -> List.mem x l1) l2
+
+let get_most_common_subject files default =
+  let all =
+    List.map (function file -> cmd_to_list (subject_command file)) files in
+  let entries =
+    List.map
+      (function entries ->
+	List.rev
+	  (List.fold_left
+	     (fun prev line ->
+	       match Str.split (Str.regexp " +") line with
+		 [] -> failwith ("bad git log line: " ^ line)
+	       | commit::rest ->
+		   let rec loop = function
+		       [] -> []
+		     | x::xs ->
+			 if last_char x = ':'
+			 then x :: loop xs
+			 else [] in
+		   let tags = loop rest in
+		  (* ignore lines with no : *)
+		   match tags with
+		     [] -> prev
+		   | _ ->
+		       let aff_files =
+			 List.tl (cmd_to_list (affected_files commit)) in
+		       let bonus = (* 0 is better *)
+			 if superset files aff_files then 0 else 1 in
+		       (bonus,tags)::prev)
+	     [] entries))
+      all in
+  (* Does there exist a file for which we have no information? *)
+  if List.exists (function x -> x = []) all
+  then default^":"
+  else
+    let common_entries =
+      List.fold_left intersect (List.hd entries) (List.tl entries) in
+    let entries = List.map get_counts entries in
+    let common_entry_counts =
+      List.sort compare
+	(List.map
+	   (function (bonus,entry) as allentry ->
+	     (bonus,
+	      (List.fold_left (+) 0
+		 (List.map (List.assoc allentry) entries),
+	       entry)))
+	   common_entries) in
+(*  Printf.printf "Files:\n";
+    List.iter (function f -> Printf.printf "  %s\n" f) files;
+    Printf.printf "Subjects:\n";
+    List.iter
+      (function (ct,(bonus,f)) ->
+	Printf.printf "  %d: %d: %s\n" ct bonus (String.concat " " f))
+      common_entry_counts;
+    Printf.printf "\n"; *)
+    match common_entry_counts with
+      [] -> default^":"
+    | (_,(_,x))::_ -> String.concat " " x
 
 (* ------------------------------------------------------------------------ *)
 
 let print_all o l =
   List.iter (function x -> Printf.fprintf o "%s\n" x) l
 
-let make_mail_header o date maintainers ctr number subject =
+let make_mail_header o date maintainers ctr number cover subject =
   Printf.fprintf o "From nobody %s\n" date;
   Printf.fprintf o "From: %s\n" !from;
   (match Str.split (Str.regexp_string ",") maintainers with
@@ -327,44 +512,102 @@ let make_mail_header o date maintainers ctr number subject =
       Printf.fprintf o "To: %s\n" x;
       Printf.fprintf o "Cc: %s\n" (String.concat "," xs)
   | _ -> failwith "no maintainers");
-  if number = 1
+  if number = 1 && not cover
   then Printf.fprintf o "Subject: [PATCH] %s\n\n" subject
   else Printf.fprintf o "Subject: [PATCH %d/%d] %s\n\n" ctr number subject
 
-let make_message_files subject cover message date maintainer_table
-    patch front add_ext =
+let print_info o info_tbl files =
+  let do_one prefix file =
+    try
+      let lines = List.rev (!(Hashtbl.find info_tbl file)) in
+      Printf.fprintf o "%s" prefix;
+      List.iter (function line -> Printf.fprintf o "%s\n" line) lines;
+      Printf.fprintf o "\n"
+    with Not_found -> () in
+  match files with
+    [file] -> do_one "" file
+  | l ->
+      List.iter (function file -> do_one (Printf.sprintf "%s:\n" file) file) l
+
+let cluster_by_dir diffs =
+  let info =
+    List.sort compare
+      (List.map
+	 (function (file,diff) ->
+	   (Filename.dirname file, file, diff))
+	 diffs) in
+  let rec loop = function 
+      [] -> []
+    | (dir,file,diffs)::rest ->
+	match loop rest with
+	  ((dira,filea,diffsa)::r)::rest ->
+	    if dir = dira
+	    then ((dir,file,diffs)::(dira,filea,diffsa)::r)::rest
+	    else [(dir,file,diffs)]::((dira,filea,diffsa)::r)::rest
+	| [] -> [[(dir,file,diffs)]]
+	| []::_ -> failwith "not possible" in
+  List.map (List.map (function (dir,file,diffs) -> (file,diffs)))
+    (loop info)
+
+let make_message_files subject cover message nonmessage date maintainer_table
+    patch front add_ext nomerge dirmerge info_tbl =
   let ctr = ref 0 in
   let elements =
     Hashtbl.fold
       (function (services,maintainers) ->
 	function diffs ->
 	  function rest ->
-	    if services=[default_string]
+	    if services=[default_string] or nomerge
 	    then
 	      (* if no maintainer, then one file per diff *)
+	      let diffs =
+		List.concat
+		  (List.map (function (common,diffs) -> !diffs) !diffs) in
 	      (List.map
 		 (function (file,diff) ->
 		   ctr := !ctr + 1;
-		   (!ctr,true,maintainers,[file],[diff]))
-		 (List.rev !diffs)) @
+		   let subject = get_most_common_subject [file] file in
+		   (subject,(!ctr,true,maintainers,[file],[diff])))
+		 (List.rev diffs)) @
 	      rest
 	    else
-	      begin
-		ctr := !ctr + 1;
-		let (files,diffs) = List.split (List.rev !diffs) in
-		(!ctr,false,maintainers,files,diffs)::rest
-	      end)
+	      if dirmerge
+	      then
+		let diffs =
+		  List.concat
+		    (List.map (function (common,diffs) -> !diffs) !diffs) in
+		let diffs = cluster_by_dir diffs in
+		List.map
+		  (function diffs ->
+		    ctr := !ctr + 1;
+		    let (files,diffs) = List.split (List.rev diffs) in
+		    let subject = get_most_common_subject files "???" in
+		    (subject,(!ctr,false,maintainers,files,diffs)))
+		  diffs @
+		rest
+	      else
+		(List.map
+		   (function (common,diffs) ->
+		     ctr := !ctr + 1;
+		     let (files,diffs) = List.split (List.rev !diffs) in
+		     let subject = get_most_common_subject files !common in
+		     (subject,(!ctr,false,maintainers,files,diffs)))
+		   !diffs) @
+		rest)
       maintainer_table [] in
   let number = List.length elements in
   let generated =
     List.map
-      (function (ctr,the_rest,maintainers,files,diffs) ->
+      (function (common,(ctr,the_rest,maintainers,files,diffs)) ->
 	let output_file = add_ext(Printf.sprintf "%s%d" front ctr) in
 	let o = open_out output_file in
-	make_mail_header o date maintainers ctr number
-	  (Printf.sprintf "%s: %s" (merge_files the_rest files) subject);
+	make_mail_header o date maintainers ctr number (not (cover=None))
+	  (Printf.sprintf "%s %s" common subject);
+	print_info o info_tbl files;
 	print_all o message;
 	Printf.fprintf o "\n---\n";
+	print_all o nonmessage;
+	(if not (nonmessage = []) then Printf.fprintf o "\n");
 	let (nm,o1) = Filename.open_temp_file "patch" "patch" in
 	List.iter (print_all o1) (List.rev diffs);
 	close_out o1;
@@ -393,14 +636,7 @@ let make_cover_file n subject cover front date maintainer_table =
     None -> ()
   | Some cover ->
       let common_maintainers =
-	let intersect l1 l2 =
-	  List.rev
-	    (List.fold_left
-	       (function i -> function cur ->
-		 if List.mem cur l2 then cur :: i else i)
-	       [] l1) in
 	let start = ref true in
-	String.concat ","
 	  (Hashtbl.fold
 	     (function (services,maintainers) ->
 	       function diffs ->
@@ -410,9 +646,22 @@ let make_cover_file n subject cover front date maintainer_table =
 		   then begin start := false; cur end
 		   else intersect cur rest)
 	     maintainer_table []) in
+      let maintainers_and_lists =
+	Hashtbl.fold
+	  (function (services,maintainers) ->
+	    function diffs ->
+	      function rest ->
+		let files = List.map (function (file,_) -> !file) !diffs in
+		List.fold_left
+		  (function prev ->
+		    function file ->
+		      union (cmd_to_list (maintainer_list_command file)) prev)
+		  rest files)
+	  maintainer_table common_maintainers in
+      let maintainers_and_lists = String.concat "," maintainers_and_lists in
       let output_file = Printf.sprintf "%s.cover" front in
       let o = open_out output_file in
-      make_mail_header o date common_maintainers 0 n subject;
+      make_mail_header o date maintainers_and_lists 0 n true subject;
       print_all o cover;
       Printf.fprintf o "\n";
       close_out o
@@ -436,7 +685,8 @@ let generate_command front cover generated =
 	(String.concat " " ((front^".cover") :: generated)));
   close_out o
 
-let make_output_files subject cover message maintainer_table patch =
+let make_output_files subject cover message nonmessage
+    maintainer_table patch nomerge dirmerge info_tbl =
   let date = List.hd (cmd_to_list "date") in
   let front = safe_chop_extension patch in
   let add_ext =
@@ -444,19 +694,28 @@ let make_output_files subject cover message maintainer_table patch =
       Some ext -> (function s -> s ^ "." ^ ext)
     | None -> (function s -> s) in
   let generated =
-    make_message_files subject cover message date maintainer_table
-      patch front add_ext in
+    make_message_files subject cover message nonmessage date maintainer_table
+      patch front add_ext nomerge dirmerge info_tbl in
   make_cover_file (List.length generated) subject cover front date
     maintainer_table;
   generate_command front cover generated
 
 (* ------------------------------------------------------------------------ *)
 
+let nomerge = ref false
+let dirmerge = ref false
+
 let parse_args l =
   let (other_args,files) =
-    List.partition
-      (function a -> String.length a > 1 && String.get a 0 = '-')
+    List.partition (function a -> String.length a > 1 && String.get a 0 = '-')
       l in
+  let (nomergep,other_args) =
+    List.partition (function a -> a = "-nomerge") other_args in
+  (if not(nomergep = []) then nomerge := true);
+  let (dirmergep,other_args) =
+    List.partition (function a -> a = "-dirmerge") other_args in
+  (* lazy solution: one directory up from the file level *)
+  (if not(dirmergep = []) then dirmerge := true);
   match files with
     [file] -> (file,String.concat " " other_args)
   | _ -> failwith "Only one file allowed"
@@ -464,14 +723,20 @@ let parse_args l =
 let _ =
   let (file,git_args) = parse_args (List.tl (Array.to_list Sys.argv)) in
   let message_file = (safe_chop_extension file)^".msg" in
+  let info_file = (safe_chop_extension file)^".info" in
   (* set up environment *)
   read_configs message_file;
-  (if not (git_args = "") then git_options := !git_options^" "^git_args);
   (* get message information *)
-  let (subject,cover,message) = get_template_information message_file in
+  let (subject,cover,message,nonmessage) =
+    get_template_information message_file in
+  (* get file-specific information, if any *)
+  let info_tbl = get_info info_file in
   (* split patch *)
   let i = open_in file in
   let patches = split_patch i in
   close_in i;
   let maintainer_table = resolve_maintainers patches in
-  make_output_files subject cover message maintainer_table file
+  (if !found_a_maintainer = false then git_options := !not_linux);
+  (if not (git_args = "") then git_options := !git_options^" "^git_args);
+  make_output_files subject cover message nonmessage
+    maintainer_table file !nomerge !dirmerge info_tbl
